#include <windows.h>
#include <mmsystem.h>
#include <stdio.h>
#include <string.h>

//
// Display the contents of the chunk
void display(char *p, int chunkSize)
{
  long left, right; // left and right boundary
  int ip;           // index in the chunk

  left = *((long *)p);
  for (ip = 4; ip < chunkSize;) {
     int size;           // size of the annotation
     char *annotation;   // annotation to be read
     annotation = (char *)(p+ip);
     size = strlen(annotation) + 1;
     ip += size;
     right = *((long *)(p+ip));
     ip += 4;
     fprintf(stdout, "%d %d %s \n", left, right, annotation);
     left = right;       // The right boundary is the left boundary of the next annotation
  }
}

int main(int argc, char *argv[])
{
  MMCKINFO ck;
  HMMIO hmmio;
  char *words, *phonemes;

  if ( (hmmio=mmioOpen(argv[1],NULL,MMIO_READ)) ==NULL ) {
    fprintf(stderr, "File not found\n");
    exit(0);
  }
  
  // Reads the chunk of words
  mmioSeek(hmmio,12,SEEK_SET);
  ck.ckid=mmioFOURCC('w','o','r','d');
  if (mmioDescend(hmmio,&ck,NULL,MMIO_FINDCHUNK) == MMSYSERR_NOERROR) {
    words = new char[ck.cksize];
    if (mmioSeek(hmmio,ck.dwDataOffset,SEEK_SET) == -1) exit(0);
    if (mmioRead(hmmio,(HPSTR)(words),ck.cksize) == -1) exit(0);
    fprintf(stdout, "*************** WORDS **********************\n");
    display(words, ck.cksize);
  }


  // Reads the chunk of phonetic annotation
  mmioSeek(hmmio,12,SEEK_SET);
  ck.ckid=mmioFOURCC('p','h','o','n');
  if (mmioDescend(hmmio,&ck,NULL,MMIO_FINDCHUNK) == MMSYSERR_NOERROR) {
    phonemes = new char[ck.cksize];
    if (mmioSeek(hmmio,ck.dwDataOffset,SEEK_SET) == -1) exit(0);
    if (mmioRead(hmmio,(HPSTR)(phonemes),ck.cksize) == -1) exit(0);
    fprintf(stdout, "*************** PHONEMES **********************\n");
    display(phonemes, ck.cksize);
  }
}
