/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define	_PI	3.14159265358979323846
#define SQR(x) ((x) * (x))


// See Annex B of the PhD of Morten Olesen entitled "A Speech Production Model
// including the Nasal Cavity" 
main(int argc, char **argv)
{
   FILE *f;
	double ta, tcmte, eps, Fe, F0, T0, Ee, E0, Ra, Rk, Rg, err, err_, der, tc, Fg, w, tp, te, a, a_, UeRight;
   short x;
   int i;

   if (argc != 8) {
     fprintf(stderr,"genLFGlottal <filename> <SamplingRate> <F0> <Ee> <Ra> <Rk> <Rg>\n");
     exit(0);
   }

   f = fopen(argv[1], "wb");
   Fe = atof(argv[2]);
   F0 = atof(argv[3]);
   Ee = atof(argv[4]);
   Ra = atof(argv[5]);
   Rk = atof(argv[6]);
   Rg = atof(argv[7]);

   T0 = Fe / F0;
   tc = T0;
   Fg = Rg / T0;
   w = 2 * _PI * Fg;
   tp = 1 / (2 * Fg);
   te = tp + Rk * tp;
   ta = Ra * (tc - te);

   // Determination of epsilon
   tcmte = tc - te;

   for (i = 0, eps = 1. / ta; i < 5; i++) {
      err = eps * ta - 1 + exp(-eps * tcmte);
      der = ta - tcmte * exp(-eps * tcmte);
      fprintf(stdout, "err = %9.6f eps = %9.6f\n", err, eps);
      eps = eps - err / der;
   }
   fprintf(stdout, "err = %9.6f eps = %9.6f\n", err, eps);


   // Integral of the return
   UeRight = - Ee / eps * (1 + (eps  * ta - 1) / Ra);

   err= 1e100; i = 0; a = 0.0;
   // for (i = 0, a = 0.0; i < 5; i++) {
   do {
     double den, num, UeLeft;
     // Integral of the first part
     den = SQR(a) + SQR(w);
     num = a - w / tan(w * te) + w * exp(-a * te) / sin (w * te);
     UeLeft = -Ee * num / den;
     err_ = err;
     err = UeLeft + UeRight;
     der = -Ee * ((1. - w * te / sin(w * te) * exp(-a * te)) * den - 2. * a * num) / SQR(den);
     fprintf(stdout, "err = %9.6f a = %9.6f\n", err, a);
     a_ = a;
     a = a - err / der;
   }
   while (fabs(err) < fabs(err_) && a_ < a);
   if (fabs(err) > fabs(err_) || a_ > a) {
     a = a_;
     err = err_;
   }
   fprintf(stdout, "err = %9.6f a = %9.6f\n", err, a);
   E0 = -Ee * exp(-a * te) / sin(w * te);

   // 50 ms silence
   for (i = 0; i < Fe / 20; i++) fwrite(&x, sizeof(short), 1, f);
   for (i = 0; i < 50; i++) {
     int n;
     double xx;
     for(n = 0; n < T0; n++) {
   	 double xx = (n <= te) ? E0 * exp(a * n) * sin(w * n)
                            : -Ee /(eps * ta) * (exp(-eps * (n - te)) - exp(-eps * (tc -te)));
       x = (short) xx;
       fwrite(&x, sizeof(short), 1, f);
     }
   }
   fclose(f);
}

#ifdef SCILAB
T0=128;
Ra= 0.2;
Rk=0.3;
Rg=1.0;
a=0.0001:0.0001:10;
zero=zeros(a);
Fg = Rg / T0;
w = 2 * %pi * Fg;
tp = 1 / (2 * Fg);
te = tp + Rk * tp;
UeRight = -5.0754532;
Ue=-(a -w/tan(w * te) + w * exp(-a*te)/sin(w*te)) ./ (a .* a + w^2);
plot2d(a,UeRight+Ue);
plot2d(a,zeros(a));
#endif

