/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier PolyPoint.h
//

#ifndef _POLYPOINT_H
#define _POLYPOINT_H

#include <stdlib.h> // pour le rand() ,realloc()
#include "signal.h"

#define CACHE 0		/* les PP de ce formant ne sont pas visibles. */
#define LIGNE 1		/* les PP sont visibles mais on ne peut pas les selectionner. */
#define ACTIF 2		/* les PP sont visibles (y compris les points) et selectionnables. */

#define SQUARES 	0   // type of display for polypoints
#define LINES		1

extern void InvertPointSel(HDC hdc, int x,int y);
extern void PointSel(int x,int y);

extern void translatePPselectionne(int incr);

// fichier spline.cpp
extern int bsreg(int nd, 	// nombre de points donnes
	  double T[],	// tableau des abscisses des donnees
	  double Z[],	// tableau des ordonnees des donnees
	  int degre, // degre du polynome d'interpolation
	  double a, double b,	// Intervalle de travail
	  int n,	// nombre de sous intervalles de [a,b]
	  double tau,	// parametre de regularisation
	  int ns,	// nombre de points ou est
	                //     calculee la fct spline
	  double alfa[], // coefficient sur la base de spines
	  double X[],	// coordonnees de la spline
	  double Y[]); //     resultat discretisee

typedef float (*YFCT)(float);

//---------------------------------------------------------------------------
       // **************** dclarations pour testfin ******************

class Point2P {             //class pour les pts a date + 1 parametre
             public:
       int date;   // en millisecondes
       int dateEcht; // en chantillons
       float y[1];
       Point2P() { date = 0; y[0] = 0.; }
       Point2P(int d, int t) { date = d; y[0] = t; }
       Point2P(int d, int t[]) { date = d; y[0] = t[0]; }
       Point2P(int d, float t[]) { date = d; y[0] = t[0]; }
};//end class Point2P

class Point3P {             //class pour les pts a date + 2 parametres
             public:
       int date;  // en millisecondes
       int dateEcht; // en chantillons
       float y[2];
       Point3P() { date = 0; for (int i = 0; i < 2; i++) y[i] = 0.; }
       Point3P(int d, int t[]) { date = d; for (int i = 0; i < 2; i++) y[i] = t[i]; }
       Point3P(int d, float t[]) { date = d; for (int i = 0; i < 2; i++) y[i] = t[i]; }
}; //end class Point3P

class Point4P {             //class pour les pts a date + 3 parametres
             public:
       int date;   // en millisecondes
       int dateEcht; // en chantillons
       float y[3];
       Point4P() { date = 0; for (int i = 0; i < 3; i++) y[i] = 0.; }
       Point4P(int d, int t[]) { date = d; for (int i = 0; i < 3; i++) y[i] = t[i]; }
       Point4P(int d, float t[]) { date = d; for (int i = 0; i < 3; i++) y[i] = t[i]; }
}; //end class Point3P

class Point5P  {            //class pour les pts a date + 4 parametres
             public:
       int date;  // en millisecondes
       int dateEcht; // en chantillons
       float y[4];
       Point5P() { date = 0; for (int i = 0; i < 4; i++) y[i] = 0.; }
       Point5P(int d, int t[]) { date = d; for (int i = 0; i < 4; i++) y[i] = t[i]; }
       Point5P(int d, float t[]) { date = d; for (int i = 0; i < 4; i++) y[i] = t[i]; }
}; //end class Point5P

#define DIMPNP(X)  ((sizeof(X) - 2 * sizeof(int)) /sizeof (float))

template <class T1> class TabPoint {

           public:
       long ni, nid;  // nid is the # of points read from the pointer
       T1 *p;
       T1 *TabDessin;
       int width;  // largeur d'affichage
       int event_x_g,event_x_d; // utiliss pour stocker les limites
              // gauche et droite  ne pas dpasser

       TabPoint(long NbElements=0) {
         p=(T1 *)NULL;
         width = 3;
         nid = 0;
         ni = 0;
         TabDessin = (T1 *) NULL;
         newsize(NbElements);
       }

       TabPoint(const TabPoint &x) {
       	ni = x.ni;
         nid = x.nid;
         if (ni) {
           p = (T1 *) malloc(ni * sizeof(T1));
           for (int i = 0; i < ni; i++) p[i] = x.p[i];
           // memcpy(p, x.p, ni * sizeof(T1));
           /* STRING s;
           sprintf(s, "TabPoint copy constructor ni = %d x.date=%d date=%d", ni, x.p[0].date, p[0].date);
           affiche_erreur(s);  */
         }
         else p = (T1 *) NULL;
         if (nid) {
           TabDessin = (T1 *) malloc(nid * sizeof(T1));
           for (int i = 0; i < ni; i++) TabDessin[i] = x.TabDessin[i];
           // memcpy(TabDessin, x.TabDessin, ni * sizeof(T1));
         }
         else TabDessin = (T1 *) NULL;
         width = x.width;
         event_x_g = x.event_x_g;
         event_x_d = event_x_d;
       }

       TabPoint& operator = (const TabPoint &x) {
         if (p) free((void *) p);
         if (TabDessin) free((void *) TabDessin);
       	ni = x.ni;
         nid = x.nid;
         if (ni) {
           p = (T1 *) malloc(ni * sizeof(T1));
           for (int i = 0; i < ni; i++) p[i] = x.p[i];
           // memcpy(p, x.p, ni * sizeof(T1));
           /* STRING s;
           sprintf(s, "TabPoint operator = ni = %d x.date=%d date=%d", ni, x.p[0].date, p[0].date);
           affiche_erreur(s); */
         }
         else p = (T1 *) NULL;
         if (nid) {
           TabDessin = (T1 *) malloc(nid * sizeof(T1));
           for (int i = 0; i < ni; i++) TabDessin[i] = x.TabDessin[i];
           // memcpy(TabDessin, x.TabDessin, ni * sizeof(T1));
         }
         else TabDessin = (T1 *) NULL;
         width = x.width;
         event_x_g = x.event_x_g;
         event_x_d = event_x_d;
         return(*this);
		 }

       ~TabPoint(){
	 		if (p) free((void *) p);
         p = (T1 *) NULL;
         ni = nid = 0;
         if (TabDessin) free((void *) TabDessin);
         TabDessin = (T1 *) NULL;
       }

       T1& operator[](int i) {
            return ((T1 *)p)[i];
       }//end operator[]

       long size() {
         return(ni);
       }

       void newsize(long l){
            this->ni = max((long)0, l);
            //affiche_erreur("tabpoint newsize");
            if (this->ni) {
            	if (p) p = (T1 *) realloc(p, l * sizeof(T1));
               else p = (T1 *) malloc(l * sizeof(T1));
            }
            else {
               if (p) free((char *) p);
               ni = 0;
               p=(T1 *) NULL;
            }//end if
       }//end newsize

       void enMilli();
       void enEcht();
		 void setDefaultValues(int nfields, int att[], float val[]);
       void echantillonnePP(int depMs);
       void removeLastPointPP();
       void removeFirstPointPP();
       void echantillonnePPEcht(int depEcht);
       void spline (int focus);
       void trouveDate(int event_x,int event_y,int indice);
       void drawDown(int event_x,int event_y, int width);
       void drawDown(int event_x,int event_y );
       void drawMotion(int event_x,int event_y );
       void drawUp(int event_x,int event_y);
       void distance(int date, float y, int focus, float correc, float seuil, float *distPoint,
       					float *distCourbe, int *indice);
       void aff(int focus,			// ordinate to be displayed
  					YFCT yecran,		// function to convert ordinate to screen coord
               int mode,			// inverse video if mode is true
               int typeAff,		// lines or squares
               COLORREF couleur, // color
               int ligne);       // thin or thick line
       void translate(int focus, float incr, float vmin, float vmax, float vmul);
       void push(T1 p);
       int indPP(int date) {
         int j;
         for (j = 0; j < ni && date != p[j].date; j++);
         return((j < ni) ? j : -1);
       }
};

#include "tabpoints.cpp"


class PolyPoints {
         public:
       bool modif;      // un point deplace ? VRAI / FAUX
       long nbPt;

       PolyPoints(void){
         nbPt=0;
         selec = modif = false;
       }

		 PolyPoints(const PolyPoints& X) {
         modif = X.modif;
         nbPt = X.nbPt;
         selec = X.selec;
         affiche_erreur("PolyPoints copy constructor");
       }

       PolyPoints& operator = (const PolyPoints &X) {
         modif = X.modif;
         nbPt = X.nbPt;
         selec = X.selec;
         affiche_erreur("PolyPoints operator =");
         return(*this);
       }

       virtual ~PolyPoints() {
         nbPt = 0;
         // affiche_erreur("virtual ~PolyPoints");
       }

       bool estSelec() { return selec; }
       virtual int debut() { affiche_erreur("virtual debut");} // =0;
       virtual int fin(){ affiche_erreur("virtual fin");} //=0;
       virtual void aff(int focus, YFCT yecran, int mode,
       				      int typeAff, COLORREF couleur, int ligne){ affiche_erreur("virtual aff");} //=0;
       bool avant(PolyPoints *s) { return debut() < s->debut(); }
       void selectionne(mode) { selec = mode;}
       virtual void distance(int date, float y, int focus, float correc, float seuil,
       					float *distPoint, float *distCourbe, int *indice){ affiche_erreur("virtual distanxe");} // = 0;
       virtual void setDefaultValues(int nfields, int att[], float val[]){ affiche_erreur("virtual setDefault");} // = 0;
       virtual int date(int i){ affiche_erreur("virtual date");} // = 0;    // en millisecondes
       virtual float y(int i, int focus){ affiche_erreur("virtual y");} // = 0;
       virtual int *pdate(int i){ affiche_erreur("virtual pdate");} // = 0;
       virtual float *py(int i, int focus){ affiche_erreur("virtual py");} // = 0;
       virtual void echantillonne(int depFor){ affiche_erreur("virtual echantillonne");} // = 0;
       virtual void removeFirstPoint(){ affiche_erreur("virtual removeFirstPoint");} // = 0;
       virtual void removeLastPoint(){ affiche_erreur("virtual removeLastPoint");} // = 0;
       virtual void translate(int focus, float incr, float vmin, float vmax, float vmul) { affiche_erreur("virtual translate");} //= 0;
		 virtual void subpp(int n){ affiche_erreur("virtual subpp");} // = 0;
       virtual void push(int date, float t[]){ affiche_erreur("virtual push");} // = 0;
		 virtual int indPP(int date){ affiche_erreur("virtual indpp");} // = 0;
       virtual void spline(int focus){ affiche_erreur("virtual spline");} // = 0;
       virtual PolyPoints *subPP(int deb, int fin){ affiche_erreur("virtual subPP");} // = 0;
       protected :
        bool selec;  // PolyPoints selectionne ? VRAI / FAUX

};
// end class PolyPoints

// Template pour les PolyPoints de dimension quelconque.
template <class TP, class POINT> class PolyPointsAD : public PolyPoints {
   public:
      TP pts;
      
      PolyPointsAD() {
         nbPt = pts.ni;
         cursor = 0;
      }

      PolyPointsAD(TP &t) {
         cursor = 0;
         pts.newsize(t.ni);
         for (int i = 0; i < t.ni; i++) pts[i] = t[i];
         nbPt = pts.ni;
      }

      PolyPointsAD(TabPoint<Point2P> &t, int focus) {
        int dim = DIMPNP(POINT);
        pts.newsize(t.ni);
        for (int i = 0; i < t.ni; i++) {
          pts[i].date = t[i].date;
          for (int j = 0; j < dim; j++)
            if (j == focus) pts[i].y[focus] = t[i].y[0];
            else pts[i].y[j] = -INFINI_F;
		  }
        nbPt = pts.ni;
        cursor = 0;
       }

      PolyPointsAD& operator = (const PolyPointsAD& X) {
         modif = X.modif;
         nbPt = X.nbPt;
         selec = X.selec;
         pts = TP(X.pts);
         cursor = X.cursor;
         return(*this);
       }

		PolyPointsAD(const PolyPointsAD& X) {
         modif = X.modif;
         nbPt = X.nbPt;
         selec = X.selec;
         pts = TP(X.pts);
         cursor = X.cursor;
         /* STRING s;
         sprintf(s, "PolyPointsAD copy constructor  date=%d y=%.0f", pts[0].date, pts[0].y[0]);
         affiche_erreur(s);  */
      }
      ~PolyPointsAD() {
         nbPt = 0;
       }
       bool estSelec() { return selec; }
       bool avant(PolyPoints *s) { return debut() < s->debut(); }
       void selectionne(mode) { selec = mode;}
       void aff(int focus, YFCT yecran, int mode, int typeAff, COLORREF couleur, int ligne) {
       	pts.aff(focus, yecran, mode, typeAff, couleur, ligne);
       }
       int debut() { return pts[0].date;}    // date du debut du polypoints en millisecondes
       int fin() { return pts[pts.ni -1].date;}
       int debutEcht() { return pts[0].dateEcht;}    // date du debut du polypoints en millisecondes
       int finEcht() { return pts[pts.ni -1].dateEcht;}
       void seek(int i) {
         cursor = i;
       }
       POINT *getPoint() {
         if (cursor < nbPt) return &pts[cursor++];
         else return (POINT *) NULL;
       }
       bool operator < (const PolyPointsAD& rhs)
		    {  //affiche_erreur("operateur <");
             return pts[0].date < rhs.pts[0].date;
          }
       bool operator == (const PolyPointsAD& rhs)
		    { affiche_erreur("operateur ==");if  (nbPt != rhs.nbPt) return false;
            else {
              int i, dim = DIMPNP(POINT);
              bool ok;
              for (i = 0, ok = true; i < nbPt && ok; i++) {
                for (int j = 0; j < dim; j++)
                  ok = ok && (pts[i].y[j] == rhs.pts[i].y[j]);
                ok = ok && (pts[i].date == rhs.pts[i].date);
              }
              return (i == nbPt);
            }
          }
       void setDefaultValues(int nfields, int att[], float val[]) {
         pts.setDefaultValues(nfields, att, val);
       }
       void distance(int date, float y, int focus, float correc, float seuil,
       					float *distPoint, float *distCourbe, int *indice) {
         pts.distance(date, y, focus, correc, seuil, distPoint, distCourbe, indice);
       }
       int date(int i) {return pts[i].date;}
       int dateEcht(int i) {return pts[i].dateEcht;}
       float y(int i, int focus) {return pts[i].y[focus];}
       int *pdate(int i) {return &pts[i].date;}
       int *pdateEcht(int i) {return &pts[i].dateEcht;}
       float *py(int i, int focus) {return &(pts[i].y[focus]);};
		 void echantillonne(int depMs) {pts.echantillonnePP(depMs); nbPt = pts.size();}
		 void echantillonneEcht(int depEcht) {pts.echantillonnePPEcht(depEcht); nbPt = pts.size();}
       void translate(int focus, float incr, float vmin, float vmax, float vmul){
       	pts.translate(focus, incr, vmin, vmax, vmul);
       }
       void push(int date, float t[]) {
         pts.push(POINT(date, t));
         nbPt = pts.size();
       }
		 void subpp(int n) { pts.newsize(n); nbPt = pts.ni; };
       PolyPointsAD *subPP(int deb, int fin) {
         if (deb > fin || deb >= nbPt) return((PolyPointsAD *) NULL);
         PolyPointsAD<TP, POINT >  *npp = new PolyPointsAD<TP, POINT >;
         int ideb = max(0, deb), ifin = min(pts.ni - 1, fin);
         npp->pts.newsize(ifin - ideb + 1);
         for (int i = ideb; i <= ifin; i++) npp->pts[i - ideb] = pts[i];
         npp->nbPt = npp->pts.ni;
         return npp;
       }
      int indPP(int date) { int j;
         for (j = 0; j < pts.ni && date != pts[j].date; j++);
         return((j < pts.ni) ? j : -1);
       }
       void spline(int focus) { pts.spline(focus); }
       private :
        int cursor;
}; //end class PolyPointsAD

#endif
