/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include <math.h>
#include "common.h"
#include "outils.h"

// Returns a rough estimation of the possibility that the spectrum corresponds to a back vowel.

float backVowel(
  float spectrum[],
  int n,	// size of the spectrum up to half the sampling frequency.
  double samplingRate)
{
   int f0Hz = 0,
       f1200Hz = 1200. / (samplingRate / 2.) * n,
       f2000Hz = 2000. / (samplingRate / 2.) * n,
       f4000Hz = 4000. / (samplingRate / 2.) * n;

   float maxF1F2 = max_tf(spectrum, f0Hz, f1200Hz),
         maxF3andAbove = max_tf(spectrum, f2000Hz, f4000Hz),
         diff = maxF1F2 - maxF3andAbove;

   return(diff > 15. ? 1 : (diff < 6. ? 0 :  (cos((diff - 6.) / (15. - 6.) * _PI + _PI) + 1.) / 2.));
}

