/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include <math.h>
#include <stdio.h>
#include <windows.h>
#include "common1.h"
#include "SuiviSnake/snake.h"
#define NRANSI
#include "Numerical/nrutil.h"

extern void affiche_erreur(char *message);

static int colCorrigee(int i),
  estFixe(int i),
  verifEloignementPtFixes(),	/* Snakes a points fixes Y. Laprie */
  lissageBerger(int tab[ DIM_X][DIM_Y], int j, int i, int masquer, int dim_x, int dim_y);

static double deriveex(int tab[DIM_X][DIM_Y], double x_old1[], double y_old1[], int i, int debut, int fin, int dim_x, int dim_y),
  deriveey(int tab[DIM_X][DIM_Y], double x_old1[], double y_old1[], int i, int debut, int fin, int dim_x, int dim_y),
  integr_courbe(type_coord_float *courbe, int debut, int fin, int tab[][DIM_Y]),
  diff(double x_old[], double x_new[], double y_old[], double y_new[], int debut, int fin),
  longueur(double x[], double y[], int debut, int fin),
  fx(type_snake *snake, double x_old[], double y_old[], int i, int debut, int fin, int dim_x, int dim_y),
  fy(type_snake *snake, double x_old[], double y_old[], double y_lowRepuls[], double y_highRepuls[], int i, int debut, int fin, int dim_x, int dim_y);

static void smooth(double tab[], double lisse[], int nb, int debut, int fin),
  deriv_2(type_coord_float *courbe, type_coord_float *deriv, int debut, int fin),
  ouvert(),
  ferme(),
  euler(type_snake *snake, double *x_old, double *x_new, double *y_old, double *y_new,
	     double (*f)(type_snake *, double [], double [], int, int, int, int, int),
        double (*g)(type_snake *, double [], double [], double [], double [], int, int, int, int, int),
        double *y_lower, double *y_higher,
	     double *alpha, double *beta, double gamma, int debut,
	     int fin, double **a, double **a1, double pas, double *membre),
  calcul_exe(type_snake *snake, int debut, int fin),
  const_matrice(type_snake *snake, int methode),
  courbure(type_coord_float *snake_pts, int nb_pts, double *res_courbure),
  courbure_ferme(type_coord_float *snake_pts, int nb_pts, double *res_courbure),
  derive_4_libre(type_coord_float *snake_pts, int nb_pts, type_coord_float *res),
  numer(type_snake *snake);

/* Constantes liees au lissage gaussien */
#define CT_DGAUSS 8.0*sqrt(2.)     /* taille du masque derivee de la gaussenne */
#define CT_GAUSS  8.0*sqrt(2.0)
float SIGMA_GAUSS =1.5;


/*
 * Sauvegarde intermediaire.
 */
void sauvegarde(
  type_snake *snake)
{
  if (snake->cst.discret > 1) { /*.. C'est un snake initialise et donc sauvegardable ..*/
    /* Si snake_sauv n'existe pas encore il faut allouer l'espace. */
    int i;
    snake->snake.nb_sauv = snake->snake.nb_pts;
    if (snake->snake.snake_sauv) free(snake->snake.snake_sauv);
    snake->snake.snake_sauv = (type_coord_float *) calloc(snake->cst.discret,sizeof(type_coord_float));

    /* Sauvegarde des resultats dans snakes_pts et snake_sauv */
    for(i = 0; i < snake->eq.discret; i++) {
      snake->snake.snake_sauv[i].x = snake->snake.snake_pts[i].x;
      snake->snake.snake_sauv[i].y = snake->snake.snake_pts[i].y;
	      
      snake->snake.snake_pts[i].x = snake->eq.x_new[i];
      snake->snake.snake_pts[i].y = snake->eq.y_new[i];
      if (snake->snake.snake_pts[i].x > snake->image.dim_x ||
	       snake->snake.snake_pts[i].x < 0 || snake->snake.snake_pts[i].y < 0 ||
	       snake->snake.snake_pts[i].y > snake->image.dim_y) {
       STRING s;
       sprintf(s, " Hors image \n     numero %d, point (%f,%f) \n bimension image %d- %d \n",
	      	   i, snake->snake.snake_pts[i].x, snake->snake.snake_pts[i].y, snake->image.dim_x, snake->image.dim_y);
	    affiche_erreur(s);
      }
    }
  }
}

void videSnake(
  type_snake *snake)
{ 
 if (snake->snake.snake_pts) free(snake->snake.snake_pts);
 snake->snake.snake_pts = (type_coord_float *) NULL;
 snake->snake.nb_pts = 0;
 if (snake->snake.snake_sauv) free(snake->snake.snake_sauv);
 snake->snake.snake_sauv = (type_coord_float *) NULL;
 snake->snake.nb_sauv = 0;
 if (snake->eq.x_old) free(snake->eq.x_old); snake->eq.x_old = (double *) NULL;
 if (snake->eq.y_old) free(snake->eq.y_old); snake->eq.y_old = (double *) NULL;
 if (snake->eq.x_new) free(snake->eq.x_new); snake->eq.x_new = (double *) NULL;
 if (snake->eq.y_new) free(snake->eq.y_new); snake->eq.y_new = (double *) NULL;

 if (snake->eq.a) free_dmatrix(snake->eq.a, 0, snake->eq.discret-1, 0, snake->eq.discret-1);    snake->eq.a =NULL;
 if (snake->eq.a1) free_dmatrix(snake->eq.a1, 0, snake->eq.discret-1, 0, snake->eq.discret-1);  snake->eq.a1=NULL;
 if (snake->eq.b) free_dvector(snake->eq.b, 0, 5 * (snake->eq.discret + 2) - 1);                snake->eq.b =NULL;
 if (snake->eq.droite) free_dmatrix(snake->eq.droite, 0, 1, 0, snake->eq.discret-1);            snake->eq.droite=NULL;
 if (snake->eq.bas) free_dmatrix(snake->eq.bas, 0, 1, 0, snake->eq.discret-1);                  snake->eq.bas=NULL;
 if (snake->eq.alpha) free_dvector(snake->eq.alpha,-1, snake->eq.discret);                      snake->eq.alpha=NULL;
 if (snake->eq.beta ) free_dvector(snake->eq.beta,-1, snake->eq.discret);                       snake->eq.beta=NULL;
 if (snake->snake_init) free(snake->snake_init); snake->snake_init=NULL;                        snake->snake_init=NULL;
 snake->eq.discret = snake->cst.discret = 0;
}

#define NBPTSsnake	3000

/*
 * Initialisation du calcul.
 * On suppose que snake_init contient la courbe avant tout calcul de snake
 * et que snake->nbPointsInit est le nombre de points de cette courbe.
 */ 
void initialiseCalcul(
   type_snake *snake)
{
  int i;

  /* Discretisation du snake */
  snake->snake.nb_pts = snake->nbPointsInit;
  /* Allocation d'un espace arbitraire pour le snake, on rectifie apres coup. */
  snake->snake.snake_pts = (type_coord_float *) calloc(NBPTSsnake,sizeof(type_coord_float));
#ifndef SNAKENORMAL
  snake->snake.nb_pts = snake->cst.discret = discret_snakeFctX(snake, snake->nbPointsInit, 
							       snake->cst.sousEcht);
#else
  snake->snake.nb_pts = snake->cst.discret = discret_snake(snake, snake->nbPointsInit);
#endif
  printf("\n nombre de points discretises %d\n",snake->snake.nb_pts);
  snake->snake.snake_pts = (type_coord_float *) realloc(snake->snake.snake_pts, snake->cst.discret * sizeof(type_coord_float));

  snake->snake.snake_sauv = (type_coord_float *) calloc(snake->cst.discret,sizeof(type_coord_float));
  snake->snake.nb_sauv = snake->cst.discret;

  snake->eq.x_old = (double *) calloc(snake->snake.nb_pts,sizeof(double));
  snake->eq.y_old = (double *) calloc(snake->snake.nb_pts,sizeof(double));
  snake->eq.x_new = (double *) calloc(snake->snake.nb_pts,sizeof(double));
  snake->eq.y_new = (double *) calloc(snake->snake.nb_pts,sizeof(double));
  /* x_old tableau des abcisses du snake 
   * y_old tableau des ordonnees du snake */
  
  for(i=0; i < snake->snake.nb_pts; i++) { 
    snake->eq.y_new[i] = snake->snake.snake_pts[i].y;
    snake->eq.x_new[i] = snake->snake.snake_pts[i].x;
    snake->eq.x_old[i] = 0.;
    snake->eq.y_old[i] = 0.;
  }

  /* Preparation de l'equation */
  snake->eq.discret = snake->cst.discret;
  snake->eq.a = dmatrix(0, snake->eq.discret-1, 0, snake->eq.discret-1);
  snake->eq.a1 = dmatrix(0,snake->eq.discret-1, 0, snake->eq.discret-1);
  snake->eq.b = dvector(0, 5 * (snake->eq.discret + 2) - 1);
  snake->eq.droite = dmatrix(0, 1, 0, snake->eq.discret-1); 
  snake->eq.bas = dmatrix(0, 1, 0, snake->eq.discret-1);

  /* alpha : tableau des coefficients de rigidite */
  /* beta  : tableau des constantes d'elasticite */
  snake->eq.alpha = dvector(-1, snake->eq.discret);
  snake->eq.beta = dvector(-1, snake->eq.discret);
  /* alpha et beta avec prise en compte du poids interne */
  for (i = 0; i < snake->eq.discret; i++) {
    snake->eq.alpha[i] = snake->cst.e_int.ela * snake->cst.e_int.poi;
    snake->eq.beta[i] = snake->cst.e_int.rig * snake->cst.e_int.poi;
  }
  snake->eq.alpha[-1] = snake->eq.alpha[snake->eq.discret] = 0.;
  snake->eq.beta[-1] = snake->eq.beta[snake->eq.discret] = 0.;
  snake->eq.itera = 1;

  snake->eq.pas =.33;

  /* construction de la matrice bande */
  if (snake->cst.fermeture == 15)
  {
    const_matrice(snake, 0);
    memcpy(snake,snake,4);
  }
  else if (snake->cst.fermeture != 3)
    const_matrice(snake, snake->cst.fermeture);
  else {
    const_matrice(snake, 2);
    printf(" fin de la premiere inversion \n");
    const_matrice(snake, 0);
    printf(" fin de la Deuxieme inversion \n");
  }
  
}    

/*
 * Calcul
 */
int calcul(
  type_snake *snake)
{
  initialiseCalcul(snake);
  numer(snake);
}




/* 
 * Calcul numerique du snake 
 * Une iteration.
 */
void uneIterationSnake(
   type_snake *snake, // Snake to be deformed
   type_snake *lowerSnake, // lower snake
   type_snake *higherSnake) // higher snake
{
  // double fx(),fy();
  int i;

  /* conservation de la vieille valeur du snake */
  for(i = 0; i < snake->eq.discret;i++) {
    snake->eq.x_old[i] = snake->eq.x_new[i];
    snake->eq.y_old[i] = snake->eq.y_new[i]; 
  }

  euler(snake, snake->eq.x_old, snake->eq.x_new, snake->eq.y_old,
	     snake->eq.y_new, fx, fy,
        (lowerSnake) ? lowerSnake->eq.y_new : (double *) NULL,
	     (higherSnake) ? higherSnake->eq.y_new : (double *) NULL,
        snake->eq.alpha, snake->eq.beta,
	     snake->cst.gamma, 0, snake->eq.discret, snake->eq.a,
	     snake->eq.a1, snake->eq.pas, snake->eq.membre);
        
  snake->eq.itera++;
}


static void numer(
  type_snake *snake)
{

  /**********************************
     RESOLUTION ITERATIVE DU SYSTEME 
     les cas dependent:
     - de la fonction image utilisee |Gradient| |itensite|
     - des conditions aux limites differentes sur x et y qui font
     qu'il y a deux matrices construites
     - des changements de repere eventuels pour imposer des contraintes
     (ex: appartenance de l'extremite a une droite)
  ************************************/

  while (snake->eq.itera < snake->cst.nb_itera) {
/*    if (afficheIntermediaires && (snake->eq.itera % 5) == 0) {
      XCopyArea(XtDisplay(donnees_spectro.w), donnees_spectro.pixmap, 
		XtWindow(donnees_spectro.w), donnees_spectro.gc, 
		0, 0, (unsigned int) (donnees_spectro.largeur), 
		(unsigned int) (donnees_spectro.hauteur), 0, 0);  
      XFlush(XtDisplay(racine));
    }
*/
    uneIterationSnake(snake, (type_snake *) NULL, (type_snake *) NULL);
  }
  sauvegarde(snake);
}




     
/*------------------------------------------------------------
  Construction et inversion de la matrice du systeme
  La bande est dans le tableau b;
  L'inverse est dans a
  ------------------------------------------------------------*/
static void const_matrice(
  type_snake *s,
  int methode)
{
  double pas2;
  int i, discret, numero;

  pas2 = s->eq.pas * s->eq.pas;
  init(s->eq.a, s->eq.discret, s->eq.discret);

  /* creation de la matrice bande */
  switch (methode) {
  case 0: {
    /* cas d'un snake ouvert */
    for (i=0;i<s->eq.discret;i++) {
      if (0<=(i-2) && (i-2)<s->eq.discret) {
	numero=position(i,i-2,2,s->eq.discret);
	s->eq.b[numero] = s->eq.beta[i-1];}
      if  (0<=(i-1) && (i-1)<s->eq.discret) {
	numero=position(i,i-1,2,s->eq.discret);
	s->eq.b[numero] = -pas2*s->eq.alpha[i] - 2*s->eq.beta[i-1]
	  -2*s->eq.beta[i];
      }
      numero=position(i,i,2,s->eq.discret);
      s->eq.b[numero] = pas2*(s->eq.alpha[i]+s->eq.alpha[i+1])+s->eq.beta[i-1]
	+ 4*s->eq.beta[i] +s->eq.beta[i+1]+s->cst.gamma;
      if (0<=(i+1) && (i+1)<s->eq.discret) {
	numero=position(i,i+1,2,s->eq.discret);
	s->eq.b[numero] = (-pas2*s->eq.alpha[i+1]) - 2*s->eq.beta[i]
	  -2*s->eq.beta[i+1];
      }
      if (0<=(i+2) && (i+2)<s->eq.discret) {
	numero=position(i,i+2,2,s->eq.discret);
	s->eq.b[numero] = s->eq.beta[i+1];
      }
    }

    /* creation de la ligne 0 */
    numero=position(0,0,2,s->eq.discret);
    s->eq.b[numero] = s->eq.beta[1]+pas2*(s->eq.alpha[1]-s->eq.alpha[0])+s->cst.gamma;
    s->eq.b[numero+2] = s->eq.beta[1];
    s->eq.b[numero+1] = (-2*s->eq.beta[1])-pas2*(s->eq.alpha[1]-s->eq.alpha[0]);
    
    /* ligne 1 */
    s->eq.b[numero+3] =  (-pas2*s->eq.alpha[1]) - 2*s->eq.beta[1];
    s->eq.b[numero+4] = pas2*(s->eq.alpha[1]+s->eq.alpha[2])+s->eq.beta[2]
      + 4*s->eq.beta[1]+s->cst.gamma;
    s->eq.b[numero+5] = (-pas2*s->eq.alpha[2])-2*s->eq.beta[2]-2*s->eq.beta[1];
    s->eq.b[numero+6] = s->eq.beta[2];


    /*ligne n-2 */
    numero=position(s->eq.discret-2,s->eq.discret-4,2,s->eq.discret); /* &&&&&&&&&& avant discret
						       a la place de fin*/
  
    s->eq.b[numero] = s->eq.beta[s->eq.discret-3];
    s->eq.b[numero+1] = (-pas2*s->eq.alpha[s->eq.discret-2])-2*s->eq.beta[s->eq.discret-2]-2*s->eq.beta[s->eq.discret-3];
    s->eq.b[numero+2] = pas2*(s->eq.alpha[s->eq.discret-2]+s->eq.alpha[s->eq.discret-1]) + 4*s->eq.beta[s->eq.discret-2]
      +s->eq.beta[s->eq.discret-3]+s->cst.gamma;
    s->eq.b[numero+3] = (-pas2*s->eq.alpha[s->eq.discret-1])-2*s->eq.beta[s->eq.discret-2];

    /* ligne n-1 */
    s->eq.b[numero+4] = s->eq.beta[s->eq.discret-1];
    s->eq.b[numero+6] = s->eq.beta[s->eq.discret-1]+ pas2*(s->eq.alpha[s->eq.discret-1]-s->eq.alpha[s->eq.discret-2])+s->cst.gamma;
    s->eq.b[numero+5] = (-2*s->eq.beta[s->eq.discret-1])-pas2*(s->eq.alpha[s->eq.discret-1]-s->eq.alpha[s->eq.discret-2]);    

    inverser1(s->eq.b,s->eq.a,2,s->eq.discret);
    printf(" TEST D'INVERION\n");
  }
  break;
 
  case 1: { /* cas d'un snake FERME */
    for (i=0;i<s->eq.discret;i++) {
      if (0<=(i-2) && (i-2)<s->eq.discret) {
	numero=position(i,i-2,2,s->eq.discret);
	s->eq.b[numero] = s->eq.beta[i-1];
      }
      if  (0<=(i-1) && (i-1)<s->eq.discret) {
	numero=position(i,i-1,2,s->eq.discret);
	s->eq.b[numero] = -pas2*s->eq.alpha[i] - 2*s->eq.beta[i-1]
	  -2*s->eq.beta[i];
      }
      numero=position(i,i,2,s->eq.discret);
      s->eq.b[numero] = pas2*(s->eq.alpha[i]+s->eq.alpha[i+1])+s->eq.beta[i-1]
	+ 4*s->eq.beta[i] +s->eq.beta[i+1]+s->cst.gamma;
      if (0<=(i+1) && (i+1)<s->eq.discret) {
	numero=position(i,i+1,2,s->eq.discret);
	s->eq.b[numero] = (-pas2*s->eq.alpha[i+1]) - 2*s->eq.beta[i]
	  -2*s->eq.beta[i+1];}
      if (0<=(i+2) && (i+2)<s->eq.discret) {
	numero=position(i,i+2,2,s->eq.discret);
	s->eq.b[numero] = s->eq.beta[i+1];
      }
    }
    /* creation de la ligne 0 */
    numero=position(0,0,2,s->eq.discret);
    s->eq.b[numero]=(s->eq.alpha[0] + s->eq.alpha[1])*pas2 +s->eq.beta[1]
      +4*s->eq.beta[0] + s->eq.beta[s->eq.discret-1]+s->cst.gamma;
    s->eq.b[numero+1]=(-s->eq.alpha[1]*pas2 -2*s->eq.beta[1]-2*s->eq.beta[0]);
    s->eq.b[numero+2]=s->eq.beta[1];

    /* ligne 1 */
    s->eq.b[numero+3]=(-pas2*s->eq.alpha[1]) - 2*s->eq.beta[1] -2*s->eq.beta[0];
    s->eq.b[numero+4]=pas2*(s->eq.alpha[1]+s->eq.alpha[2])+s->eq.beta[2]
      + 4*s->eq.beta[1]+s->eq.beta[0]+s->cst.gamma;
    s->eq.b[numero+5]=(-pas2*s->eq.alpha[2])-2*s->eq.beta[2]-2*s->eq.beta[1];
    s->eq.b[numero+6]=s->eq.beta[2];


    /*ligne n-2 */
    numero=position(s->eq.discret-2,s->eq.discret-4,2,s->eq.discret);
    
    s->eq.b[numero]=s->eq.beta[s->eq.discret-3];
    s->eq.b[numero+1]=(-pas2*s->eq.alpha[s->eq.discret-2])-2*s->eq.beta[s->eq.discret-2]-2*s->eq.beta[s->eq.discret-3];
    s->eq.b[numero+2]=pas2*(s->eq.alpha[s->eq.discret-2]+s->eq.alpha[s->eq.discret-1]) + s->eq.beta[s->eq.discret-1]
      +4*s->eq.beta[s->eq.discret-2]+s->eq.beta[s->eq.discret-3]+s->cst.gamma;
    s->eq.b[numero+3]=(-pas2*s->eq.alpha[s->eq.discret-1])-2*s->eq.beta[s->eq.discret-1] -2*s->eq.beta[s->eq.discret-2];

    /* ligne n-1 */
    s->eq.b[numero+4]=s->eq.beta[s->eq.discret-2];
    s->eq.b[numero+6]=pas2*(s->eq.alpha[s->eq.discret-1] + s->eq.alpha[0]) + s->eq.beta[0]
      +4*s->eq.beta[s->eq.discret-1]+s->eq.beta[s->eq.discret-2]+s->cst.gamma;
    s->eq.b[numero+5]=(-pas2*s->eq.alpha[s->eq.discret-1] -2*s->eq.beta[s->eq.discret-1]-2*s->eq.beta[s->eq.discret-2]);
  

    /* Determination du tableau complementaire des coins */

    s->eq.tab[0] = s->eq.beta[s->eq.discret-1];
    s->eq.tab[1] = (-pas2*s->eq.alpha[0] -2*s->eq.beta[0] -2*s->eq.beta[s->eq.discret-1]);
    s->eq.tab[2] = s->eq.beta[0];

    s->eq.tab[3] = s->eq.beta[s->eq.discret-1];
    s->eq.tab[4] = (-pas2*s->eq.alpha[0] -2*s->eq.beta[0] -2*s->eq.beta[s->eq.discret-1]);
    s->eq.tab[5] = s->eq.beta[0];


    inverser2(s->eq.b,s->eq.a,2,s->eq.tab,s->eq.droite,s->eq.bas,s->eq.discret);
  }
  break;

  /* @@@@@@@@@@@@@@@@@@ */
  case 2:{ /* snake a extremites fixes
	      On constitue une matrice de taille (s->eq.discret-2)*(s->eq.discret-2) */
    for (i = 0;i < s->eq.discret - 2; i++) {
      if (0 <= (i-2) && (i-2) < s->eq.discret - 2) {
	numero = position(i,i-2,2,s->eq.discret - 2);
	s->eq.b[numero] = s->eq.beta[i-1+1];
      }
      if  (0 <= (i-1) && (i-1) < s->eq.discret - 2) {
	numero = position(i,i-1,2,s->eq.discret-2);
	s->eq.b[numero] = -pas2*s->eq.alpha[i+1] - 2*s->eq.beta[i-1+1]
	  -2*s->eq.beta[i+1];
      }
      numero = position(i,i,2,s->eq.discret-2);
      s->eq.b[numero] = pas2*(s->eq.alpha[i+1]+s->eq.alpha[i+1+1])+
	s->eq.beta[i-1+1] + 4*s->eq.beta[i+1] +s->eq.beta[i+1+1]+s->cst.gamma;
      if (0 <= (i+1) && (i+1) < s->eq.discret - 2) {
	numero=position(i,i+1,2,s->eq.discret-2);
	s->eq.b[numero] = (-pas2*s->eq.alpha[i+1+1]) - 2*s->eq.beta[i+1]
	  -2*s->eq.beta[i+1+1];
      }
      if (0 <= (i+2) && (i+2) < s->eq.discret - 2) {
	numero=position(i,i+2,2,s->eq.discret-2);
	s->eq.b[numero] = s->eq.beta[i+1+1];
      }
    }

    /* constitution de la ligne 0 */
   
    numero=position(0,0,2,s->eq.discret-2);
    s->eq.b[numero]=pas2*(s->eq.alpha[1]+s->eq.alpha[2])+s->eq.beta[2]
      + 4*s->eq.beta[1]+s->cst.gamma;
    s->eq.b[numero+1]=(-pas2*s->eq.alpha[2])-2*s->eq.beta[2]-2*s->eq.beta[1];
    s->eq.b[numero+2]=s->eq.beta[2];

    /* constitution de la ligne 1 */
    s->eq.b[numero+3]=(-pas2*s->eq.alpha[2])-2*s->eq.beta[1]-2*s->eq.beta[2];
    s->eq.b[numero+4]=pas2*(s->eq.alpha[2]+s->eq.alpha[3])+s->eq.beta[1]
      +4*s->eq.beta[2]+s->eq.beta[3]+s->cst.gamma;
    s->eq.b[numero+5]=(-pas2*s->eq.alpha[3])-2*s->eq.beta[2]-2*s->eq.beta[3];
    s->eq.beta[numero+6]=s->eq.beta[3];


    /* constitution de la ligne s->eq.discret-4 */
    numero=position(s->eq.discret-4,s->eq.discret-6,2,s->eq.discret-2);
    s->eq.b[numero]=s->eq.beta[s->eq.discret-4];
    s->eq.b[numero+1]=(-pas2*s->eq.alpha[s->eq.discret-3])-2*s->eq.beta[s->eq.discret-4]-2*s->eq.beta[s->eq.discret-3];
    s->eq.b[numero+2]=pas2*(s->eq.alpha[s->eq.discret-3]+s->eq.alpha[s->eq.discret-2])+s->eq.beta[s->eq.discret-4]+
      4*s->eq.beta[s->eq.discret-3]+s->eq.beta[s->eq.discret-2]+s->cst.gamma;
    s->eq.b[numero+3]=(-pas2*s->eq.alpha[s->eq.discret-2])-2*s->eq.beta[s->eq.discret-3]-2*s->eq.beta[s->eq.discret-2];
		   
		   
    /* constitution de la ligne s->eq.discret-3 */
    numero=position(s->eq.discret-3,s->eq.discret-5,2,s->eq.discret-2);
    s->eq.b[numero]=s->eq.beta[s->eq.discret-3];
    s->eq.b[numero+1]=(-pas2*s->eq.alpha[s->eq.discret-2])-2*s->eq.beta[s->eq.discret-2]-2*s->eq.beta[s->eq.discret-3];
    s->eq.b[numero+2]=pas2*(s->eq.alpha[s->eq.discret-2]+s->eq.alpha[s->eq.discret-1])+4*s->eq.beta[s->eq.discret-2]
      +s->eq.beta[s->eq.discret-3]+s->cst.gamma; /* LAPRIE c'est ici yavait s->eq.alpha[s->eq.discret-3] a la place de s->eq.alpha[s->eq.discret-1] */


    /* constitution du second membre sans tenir compte de v[0] etv[n]  */
    s->eq.membre[0]=(pas2*s->eq.alpha[1]) + 2*s->eq.beta[1];
    s->eq.membre[1]=(-s->eq.beta[1]);
    s->eq.membre[2]=(-s->eq.beta[s->eq.discret-2]);		   
    s->eq.membre[3]=pas2*s->eq.alpha[s->eq.discret-1]+2*s->eq.beta[s->eq.discret-2];

    inverser1(s->eq.b,s->eq.a,2,s->eq.discret-2);
  }
  break;
  }
  // fprintf(stderr," inversion terminee \n");
}



/* determination du second membre de  l'equation */
static double fx(
  type_snake *snake,
  double x_old[],
  double y_old[],
  int i,
  int debut,
  int fin,
  int dim_x,
  int dim_y)
{ 
  double z;

  z = - snake->cst.ener_poi * (  snake->cst.ener_int * deriveex(IMA_INT,x_old,y_old,i,debut,fin,dim_x,dim_y)
//		       + snake->cst.ener_gra * deriveex(IMA_GRAM,x_old,y_old,i,debut,fin,dim_x,dim_y));
		       + snake->cst.ener_gra * deriveex(IMA_GRA,x_old,y_old,i,debut,fin,dim_x,dim_y));
  /* - prend en compte l'attraction vers l'image gradient */

  return z;
}

extern FILE *logrepuls;
static double fy(
  type_snake *snake,
  double x_old[],
  double y_old[],
  double y_lowRepuls[], 
  double y_highRepuls[],
  int i,
  int debut,
  int fin,
  int dim_x,
  int dim_y)
{  
  double z, zrepuls;
   
  z = - snake->cst.ener_poi * (snake->cst.ener_int * deriveey(IMA_INT,x_old,y_old,i,debut,fin,dim_x,dim_y)
		       + snake->cst.ener_gra * deriveey((interactionRepulsive) ? IMA_GRA : IMA_GRAM,x_old,y_old,i,debut,fin,dim_x,dim_y));

  zrepuls = 0.;
  if (interactionRepulsive) {
    if (y_lowRepuls) {
      zrepuls -= snake->cst.ener_repuls * IMA_GRA[IROUND(x_old[i])][IROUND(y_lowRepuls[i])]  * exp(-SQR((y_old[i] - y_lowRepuls[i]) / snake->cst.sigma_repuls));
    } 
    else zrepuls -= snake->cst.ener_repuls * 100.  * exp(-SQR(y_old[i] * 4. / snake->cst.sigma_repuls)); // pour ne pas aller sous 0.

    if (y_highRepuls) {
      zrepuls += snake->cst.ener_repuls * IMA_GRA[IROUND(x_old[i])][IROUND(y_highRepuls[i])] * exp(-SQR((y_old[i] - y_highRepuls[i]) / snake->cst.sigma_repuls));
    } 
    /*  fprintf(logrepuls,"y %4.2f ylow %4.2f yhigh %4.2f IMA %4d rpulsion %5.2f\n ",y_old[i],   (y_lowRepuls) ? y_lowRepuls[i] : -1.,
	(y_highRepuls) ? y_highRepuls[i] : -1., (y_lowRepuls) ? IMA_GRA[ROUND(x_old[i])][ROUND(y_lowRepuls[i])] : IMA_GRA[ROUND(x_old[i])][ROUND(y_highRepuls[i])], zrepuls); caution to test repulsion */
  }
  return z + zrepuls;
}

/* schema d'Euler iteratif de resolution      */
/* x_old : vecteur de discretisation du snake */
/* x_new : nouveau vecteur apres iteration    */
/* f     : fonction des contraintes externes  */
/*         derivee par rapport a x            */
/* g     : derivee par rapport a y            */
/* gamma : parametre du schema                */
/* discret Nombre de points de discretisation */

static void euler(
		  type_snake *snake,
		  double *x_old,
		  double *x_new,
		  double *y_old,
		  double *y_new,
		  double (*f)(type_snake *, double [], double [], int, int, int, int, int),
		  double (*g)(type_snake *, double [], double [], double [], double [], int, int, int, int, int),
		  double *y_lower, // les ordonnees des snakes au-dessous et au-dessus (les formants Fn-1, Fn+1
		  double *y_higher,
		  double *alpha,
		  double *beta,
		  double gamma,
		  int debut,
		  int fin,
		  double **a,
		  double **a1,
		  double pas,
		  double *membre)
{
  int i, discret;
  double   v[SNAX], w[SNAX], z, zz, pas2, pas4;
  
  discret = fin - debut;

  pas2 = pas * pas; pas4 = pas2 * pas2;
  for (i = debut; i < fin; i++) {
    z = (*f)(snake, x_old, y_old, i, debut, fin, snake->image.dim_x, snake->image.dim_y);
    z = gamma * x_old[i] - (pas4) * z;
    v[i] = z;
    zz = (*g)(snake, x_old, y_old, y_lower, y_higher, i, debut, fin, snake->image.dim_x, snake->image.dim_y);
    zz = gamma * y_old[i] - (pas4) * zz;
    w[i]=zz;
  }

  if (snake->cst.fermeture == 2) {
    v[1]+= membre[0]*x_old[0];w[1] += membre[0]*y_old[0];
    v[2]+= membre[1]*x_old[0];w[2] += membre[1]*y_old[0];
    v[fin-3]+= membre[2]*x_old[fin-1];w[fin-3] += membre[2]*y_old[fin-1];
    v[fin-2]+= membre[3]*x_old[fin-1];w[fin-2] += membre[3]*y_old[fin-1];
  
    matvec_bis(a,v,x_new,discret-2,discret-2,debut+1);
    matvec_bis(a,w,y_new,discret-2,discret-2,debut+1);
  }
  else if (snake->cst.fermeture == 3) {
    v[1]+= membre[0]*x_old[0];
    v[2]+= membre[1]*x_old[0];
    v[fin-3]+= membre[2]*x_old[fin-1];
    v[fin-2]+= membre[3]*x_old[fin-1];

    matvec_bis(a,v,x_new,discret-2,discret-2,debut+1);
    matvec_bis(a1,w,y_new,discret,discret,debut);

  } 
  else if (snake->cst.fermeture == 15) {
    matvec_bis(a,w,y_new,discret,discret,debut);
    for(i=debut;i<discret;i++) x_new[i]=x_old[i];
  } 

  else  { 
    matvec_bis(a,v,x_new,discret,discret,debut);
    matvec_bis(a,w,y_new,discret,discret,debut);
  }
}


/* calcul de la longueur d'une ligne polygonale */
static double longueur(
  double x[],
  double y[],            /* x et y tableau des coordonnees */
  int debut,
  int fin)                /*n longueur du vecteur */
{
  int i;
  double mesure;


  mesure=0.;
  for(i=debut;i<fin-1;i++)
    mesure += sqrt( (x[i+1]-x[i])*(x[i+1]-x[i]) + (y[i+1]-y[i])*(y[i+1]-y[i]));
  return mesure;
}



/* Derivee partielle en x d'une fonction de deux variables stockee dans
 * le tableau tab.( Contraintes dans l'image ).
 * n est la dimension du tableau
 * Resultat de la fonction est la derivee en x au point courant du snake
 * de parametre i (x(i),y(i)).
 */

static double deriveex(
  int tab[DIM_X][DIM_Y],
  double x_old1[],
  double y_old1[],
  int i,
  int debut,
  int fin,
  int dim_x,
  int dim_y)
{
  double dx;
  /* determination du pixel le plus proche du snake */
  int x_old = IROUND(x_old1[i]),
      y_old = IROUND(y_old1[i]);


  if (x_old>=0 && x_old<dim_x && y_old>0 && y_old<dim_y ) {
    /* point situe dans l'image */
    if(!x_old)
      dx= lissageBerger(tab,1,y_old,masque,dim_x,dim_y)-lissageBerger(tab,0,y_old,masque,dim_x,dim_y);
    else if(x_old == dim_x- 1)
      dx = lissageBerger(tab,dim_x -1,y_old,masque,dim_x,dim_y) - lissageBerger(tab,dim_x - 2,y_old,masque,dim_x,dim_y);
    else
      dx = (lissageBerger(tab,x_old+1,y_old,masque,dim_x,dim_y) - lissageBerger(tab,x_old-1,y_old,masque,dim_x,dim_y)) / 2.;
  }
  else
    /* cas d'un point situe hors de l'image */
    dx=0.;

  return dx;

}


/* Calcul des derivees partielles en y */


static double deriveey(
  int tab[DIM_X][DIM_Y],
  double x_old1[],
  double y_old1[],
  int i,
  int debut,
  int fin,
  int dim_x,
  int dim_y)
{ 
  double dy;
  short x_old = IROUND(x_old1[i]),
        y_old = IROUND(y_old1[i]);


  if (x_old>=0 && x_old<dim_x && y_old>0 && y_old<dim_y ) {
    if (!y_old)
      dy = lissageBerger(tab,x_old,1,masque,dim_x,dim_y) - lissageBerger(tab,x_old,0,masque,dim_x,dim_y);
  
    else if(y_old == dim_y - 1)
      dy= lissageBerger(tab,x_old,dim_y - 1,masque,dim_x,dim_y) - lissageBerger(tab,x_old,dim_y - 2,masque,dim_x,dim_y);
    else
      dy = (lissageBerger(tab,x_old,y_old+1,masque,dim_x,dim_y) - lissageBerger(tab,x_old,y_old - 1,masque,dim_x,dim_y))/2.;
  }
  else dy = 0;
  return dy;
}




static int lissageBerger(
  int tab[ DIM_X][DIM_Y],
  int j,
  int i,
  int masquer,
  int dim_x,
  int dim_y)
{  
  double res;
   int nres;
   int x1,x2,x3;

	
   switch(masquer) {
   case 3 : if (0<i && i<dim_x-1 && 0<j && j<dim_y-1)
              { res = 4*tab[j][i] + 2*(tab[j-1][i] +tab[j+1][i] +tab[j][i+1] +tab[j][i-1]) + tab[j-1][i-1]+tab[j-1][i+1]+tab[j+1][i-1] + tab[j+1][i+1];
		/*fprintf(stderr," RES %g ab %d ",res,tab[j][i]);*/
            res = (float) res/16.;}break;
   
   case 5 : if (1<i && i<dim_x-2 && 1<j && j<dim_y-2)
     {      x1 = tab[j-1][i] +tab[j+1][i] +tab[j][i+1] +tab[j][i-1];
	    x2 = tab[j-1][i-1]+tab[j-1][i+1]+tab[j+1][i-1] + tab[j+1][i+1] + tab[j+2][i-2] + tab[j-2][i] + tab[j+2][i] + tab[j+2][i+2];
	    x3 = tab[j-2][i-2] + tab[j-1][i-2] + tab[j-2][i-1]  +  tab[j+2][i-2]+tab[j+1][i-2]+tab[j+2][i-1] + tab[j-2][i+2]+tab[j-1][i+2] +tab[j-2][i+1] + tab[j+2][i+2]+tab[j+1][i+2]+tab[j+2][i+1];
            res = 8*tab[j][i] + 4*x1 +2 * x2 + x3;
            res = res /52.;break;
	  }
   case 1 : res = tab[j][i];
/*   default : res = tab[i][j];*/
	  
   }

  nres = res;
  return nres;
}





/* Version 2 de la courbure */

static void courbure(
  type_coord_float *snake_pts,
  int nb_pts,
  double *res_courbure)
{
  int i;
  double l1,l2;
  type_coord_float u,v;
  for(i=1;i<nb_pts-1;i++) {
    /* determination des vecteurs unitaires portes par XiXi+1 et XiXi-1
       La courbure est calculee a partir de ces points pour eviter les pbs
       de non equidistances des points. On prend 3 fois les vecteurs unitaires
       puisque les points dont censes etre equidistants de 3 pixels */
     
    l1= distance(snake_pts[i],snake_pts[i-1]);
    l2= distance(snake_pts[i],snake_pts[i+1]);

    u.x =3*( snake_pts[i-1].x-snake_pts[i].x)/l1 + snake_pts[i].x;
    u.y =3*( snake_pts[i-1].y-snake_pts[i].y)/l1 + snake_pts[i].y;

    v.x =3*( snake_pts[i+1].x-snake_pts[i].x)/l2 + snake_pts[i].x;
    v.y =3*( snake_pts[i+1].y-snake_pts[i].y)/l2 + snake_pts[i].y;

     
     
    res_courbure[i] = sqrt(pow(v.x-2*snake_pts[i].x+u.x,2.)
			   +
			   pow(v.y-2*snake_pts[i].y+u.y,2.));
  }
  res_courbure[0]=res_courbure[nb_pts-1]=0.;
}




static void courbure_ferme(
  type_coord_float *snake_pts,
  int nb_pts,
  double *res_courbure)
{
  int i;
  double l1,l2;
  type_coord_float u,v;
  for(i=0;i<nb_pts;i++) {
    /* determination des vecteurs unitaires portes par XiXi+1 et XiXi-1
       La courbure est calculee a partir de ces points pour eviter les pbs
       de non equidistances des points. On prend 3 fois les vecteurs unitaires
       puisque les points dont censes etre equidistants de 3 pixels */
    if (i==0) {
      l1= distance(snake_pts[0],snake_pts[nb_pts-1]);
      l2= distance(snake_pts[0],snake_pts[1]);

      u.x =3*( snake_pts[nb_pts-1].x-snake_pts[0].x)/l1 + snake_pts[0].x;
      u.y =3*( snake_pts[nb_pts-1].y-snake_pts[0].y)/l1 + snake_pts[0].y;

      v.x =3*( snake_pts[1].x-snake_pts[0].x)/l2 + snake_pts[0].x;
      v.y =3*( snake_pts[1].y-snake_pts[0].y)/l2 + snake_pts[0].y;
      res_courbure[0]= sqrt(pow(v.x-2*snake_pts[i].x+u.x ,2.)
			    +  pow(v.y-2*snake_pts[i].y+u.y,2.));
    }
     else if (i==nb_pts-1) {
       l1= distance(snake_pts[i],snake_pts[i-1]);
       l2= distance(snake_pts[i],snake_pts[0]);

       u.x =3*( snake_pts[i-1].x-snake_pts[i].x)/l1 + snake_pts[i].x;
	u.y =3*( snake_pts[i-1].y-snake_pts[i].y)/l1 + snake_pts[i].y;

	v.x =3*( snake_pts[0].x-snake_pts[i].x)/l2 + snake_pts[i].x;
	v.y =3*( snake_pts[0].y-snake_pts[i].y)/l2 + snake_pts[i].y;
	res_courbure[i]= sqrt(pow(v.x-2*snake_pts[i].x+u.x
				  ,2.)+ pow(v.y-2*snake_pts[i].y+u.y,2.));
     }
     else {
       l1= distance(snake_pts[i],snake_pts[i-1]);
       l2= distance(snake_pts[i],snake_pts[i+1]);
	 
       u.x =3*( snake_pts[i-1].x-snake_pts[i].x)/l1 + snake_pts[i].x;
       u.y =3*( snake_pts[i-1].y-snake_pts[i].y)/l1 + snake_pts[i].y;
	 
       v.x =3*( snake_pts[i+1].x-snake_pts[i].x)/l2 + snake_pts[i].x;
       v.y =3*( snake_pts[i+1].y-snake_pts[i].y)/l2 + snake_pts[i].y;

     
       res_courbure[i]= sqrt(pow(v.x-2*snake_pts[i].x+u.x,2.)
			 +
			 pow(v.y-2*snake_pts[i].y+u.y,2.));
     }
  }
}



/*------------------------------------------------------------
  Calcul de la derivee quatrieme d'une courbe snake_pts
  en utilisant les conditions aux limites des snakes libres.
  (u'' et u''' nuls aux extremites )
------------------------------------------------------------*/
static void derive_4_libre(
  type_coord_float *snake_pts,
  int nb_pts,
  type_coord_float *res)
{
  int i;

  for(i=2;i<nb_pts-2;i++) {
    res[i].x= snake_pts[i+2].x -4*snake_pts[i+1].x+6*snake_pts[i].x
      -4*snake_pts[i-1].x + snake_pts[i-2].x;
    res[i].y= snake_pts[i+2].y -4*snake_pts[i+1].y+6*snake_pts[i].y
      -4*snake_pts[i-1].y + snake_pts[i-2].y;
  }

  /* Traitement des valeurs particulieres */
  res[0].x = snake_pts[2].x -2*snake_pts[1].x+snake_pts[0].x;
  res[0].y = snake_pts[2].y -2*snake_pts[1].y+snake_pts[0].y;

  res[nb_pts-1].x = snake_pts[nb_pts-3].x -2* snake_pts[nb_pts-2].x
                    +snake_pts[nb_pts-1].x;
  res[nb_pts-1].y = snake_pts[nb_pts-3].y -2* snake_pts[nb_pts-2].y
                    +snake_pts[nb_pts-1].y;  

  res[1].x = -2*snake_pts[0].x+5*snake_pts[1].x-4*snake_pts[2].x
             +snake_pts[3].x;
  res[1].y = -2*snake_pts[0].y+5*snake_pts[1].y-4*snake_pts[2].y
             +snake_pts[3].y;  
  
  res[nb_pts-2].x = snake_pts[nb_pts-4].x -4*snake_pts[nb_pts-3].x
                    +5*snake_pts[nb_pts-2].x-2*snake_pts[nb_pts-1].x;
  res[nb_pts-2].y = snake_pts[nb_pts-4].y -4*snake_pts[nb_pts-3].y
                    +5*snake_pts[nb_pts-2].y-2*snake_pts[nb_pts-1].y;  
}



/*------------------------------------------------------------
    Calcul de la derivee quatrieme d'une courbe snake_pts
      en utilisant les conditions aux limites des snakes fermes
------------------------------------------------------------*/

void derive_4_ferme(
  type_coord_float *snake_pts,
  int nb_pts,
  type_coord_float *res)
{
  int i;

  for(i=2;i<nb_pts-2;i++) {
    res[i].x= snake_pts[i+2].x -4*snake_pts[i+1].x+6*snake_pts[i].x
      -4*snake_pts[i-1].x + snake_pts[i-2].x;
    res[i].y= snake_pts[i+2].y -4*snake_pts[i+1].y+6*snake_pts[i].y
      -4*snake_pts[i-1].y + snake_pts[i-2].y;
  }

  /* Traitement des valeurs particulieres */
  res[0].x = snake_pts[2].x -4*snake_pts[1].x+6*snake_pts[0].x
                   -4*snake_pts[nb_pts-1].x + snake_pts[nb_pts-2].x;
  
  res[0].y = snake_pts[2].y -4*snake_pts[1].y+6*snake_pts[0].y
                   -4*snake_pts[nb_pts-1].y + snake_pts[nb_pts-2].y;
  

  res[1].x= snake_pts[3].x -4*snake_pts[2].x+6*snake_pts[1].x
               -4*snake_pts[0].x + snake_pts[nb_pts-1].x;

  res[1].y= snake_pts[3].y -4*snake_pts[2].y+6*snake_pts[1].y
               -4*snake_pts[0].y + snake_pts[nb_pts-1].y;

  res[nb_pts-2].x= snake_pts[0].x -4*snake_pts[nb_pts-1].x+
    6*snake_pts[nb_pts-2].x  -4*snake_pts[nb_pts-3].x + snake_pts[nb_pts-4].x;

  res[nb_pts-2].y= snake_pts[0].y -4*snake_pts[nb_pts-1].y+
    6*snake_pts[nb_pts-2].y  -4*snake_pts[nb_pts-3].y + snake_pts[nb_pts-4].y;


 res[nb_pts-1].x= snake_pts[1].x -4*snake_pts[0].x+6*snake_pts[nb_pts-1].x
                  -4*snake_pts[nb_pts-2].x + snake_pts[nb_pts-3].x;

 res[nb_pts-1].y= snake_pts[1].y -4*snake_pts[0].y+6*snake_pts[nb_pts-1].y
                  -4*snake_pts[nb_pts-2].y + snake_pts[nb_pts-3].y;
  
}



/*------------------------------------------------------------
      Calcul de la derivee seconde d'une courbe snake_pts
      en utilisant les conditions aux limites des snakes fermes
 ------------------------------------------------------------*/

void derive_2_ferme(
  type_coord_float *snake_pts,
  int discret,
  type_coord_float *res)
{
  int i;
  for(i=1;i<discret-1;i++) {
    res[i].x=snake_pts[i+1].x-2*snake_pts[i].x+snake_pts[i-1].x;
    res[i].y=snake_pts[i+1].y-2*snake_pts[i].y+snake_pts[i-1].y;
  }
  res[0].x = snake_pts[1].x-2*snake_pts[0].x+snake_pts[discret-1].x;
  res[0].y = snake_pts[1].y-2*snake_pts[0].y+snake_pts[discret-1].y;  
  
  res[discret-1].x=snake_pts[0].x-2*snake_pts[discret-1].x+
                 snake_pts[discret-2].x;
  res[discret-1].y=snake_pts[0].y-2*snake_pts[discret-1].y+
                 snake_pts[discret-2].y;
}






/*------------------------------------------------------------
  Lissage d'un tableau lineaire sur 2nb+1 points avec des poids
  croissants de 1 en 1.
  ex : pour un lissage sur 3 points, on aura les poids 1 2 1
  Resultat dans lisse.
  ------------------------------------------------------------*/

static void smooth(
  double tab[],
  double lisse[],
  int nb,
  int debut,
  int fin)
{
  int i,j,pds;

  for(i=debut;i<fin;i++) {
    lisse[i]=0;pds =0;
     for (j= -nb;j<=nb;j++)
       if(i+j>=debut && i+j<fin) {
	 lisse[i] += tab[i+j]; 
	 pds +=1;
       }
     lisse[i] /= pds;/* pds est la somme des poids*/
  }
}


/*------------------------------------------------------------
  Calcul de l'intergrale
  ------------------------------------------------------------*/
static double integr_courbe(
  type_coord_float *courbe,
  int debut,
  int fin,
  int tab[][DIM_Y])
{
  int i,j,nombre;
  type_coord_float m;
  double mesure,somme=0;


  /* choix du pas de l'ordre du pixel */
  for(i=debut;i<fin-1;i++) {
    mesure = distance(courbe[i],courbe[i+1]);
    nombre = IROUND(mesure);
    if (nombre)
      for(j=0;j<nombre;j++) {
	     m.x = courbe[i].x+j*((courbe[i+1].x-courbe[i].x)/nombre);
        m.y = courbe[i].y+j*((courbe[i+1].y-courbe[i].y)/nombre);
	     somme += (interpol(m,tab)*mesure/nombre);
      }
    else somme += interpol(courbe[i],tab)*mesure;
  }
 
  printf("\n INTERGRALE %f \n",somme);
  return somme;
}





/*------------------------------------------------------------
  Derivee numerique de la courbe .
  Resultat dans deriv
  ------------------------------------------------------------*/

static void deriv_2(
  type_coord_float *courbe,
  type_coord_float *deriv,
  int debut,
  int fin)
{
  int i;
  for (i=debut+1;i<fin-1;i++) {
    deriv[i].x = ( courbe[i+1].x - courbe[i-1].x)/2.;
    deriv[i].y = ( courbe[i+1].y - courbe[i-1].y)/2.;
  }

  deriv[0].x = courbe[1].x -courbe[0].x;
  deriv[0].y = courbe[1].y -courbe[0].y;

  deriv[fin-1].x = courbe[fin-1].x -courbe[fin-2].x;
  deriv[fin-1].y = courbe[fin-1].y -courbe[fin-2].y;

  deriv[fin].x = courbe[fin-1].x -courbe[fin-2].x;
  deriv[fin].y = courbe[fin-1].y -courbe[fin-2].y;
}
     

