/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include <math.h>
#include <stdio.h>
#include <string.h>

#include "common.h"
#include "fftlpc.h"
#include "spectre.h"
#include "signal.h"
#include "opt_for.h"
#include "outils.h"
#include "fft_mayer.h"
#include "pics.h"
#include "SuiviSnake/snake.h"
#include "SuiviSnake/calculim.h"

/* Pour savoir si l'image est a jour. */
static STRING speechSignalName ="";
static int startImage, endImage, d_eImage;
static TypeSnakeImage typeSnakeImage;

#define NBNIV		256	/* image sur NBNIV niveaux de gris */

static void calculimCeps(SAMPLE *speech, int premEcht, int derEcht, int lifterStart, int lifterEnd, int cepstreOuHalle),
			   calculimLPC(SAMPLE *speech, int premEcht, int derEcht, int order),
            savePgm(int image[DIM_X][DIM_Y], int largeur, int hauteur, char *filename);


/*
 * Retourne true si l'image est a jour.
 */
static bool imageAJour(
  int d_e,
  int start,
  int end,
  char *filename,
  TypeSnakeImage type)
{
  return(!strcmp(speechSignalName, filename) && d_e == d_eImage && startImage == start && endImage == end && type == typeSnakeImage);
}

void saveImSpectroSnakePGM(
  char *filename)
{
  savePgm(IMA_GRA, DIM_X, DIM_Y, filename);
}

void calculImCeps(
  SAMPLE *speech,
  int d_e,
  int start,
  int end,
  char *speechFilename,
  bool withHalle)
{
  if (!imageAJour(d_e, start, end, speechFilename, SnakeImageCepstral)) {
    fenSnake = ORDRECEPSTREIM;
    deplaceSnake = (int)((end - start + 1 - ORDRECEPSTREIM) / (DIM_X - 1)) + 1;
    calculimCeps(speech, start, end, (int) enEchtf(opt_for.nb_cep[0]), (int) enEchtf(opt_for.nb_cep[1]),
                 (withHalle) ? 1 : 0);
    d_eImage = d_e;
    startImage = start;
    endImage = end;
    strcpy(speechSignalName, speechFilename);
    typeSnakeImage = SnakeImageCepstral;
  }
}

void calculImLPC(
  SAMPLE *speech,
  int d_e,
  int start,
  int end,
  char *speechFilename)
{
  if (!imageAJour(d_e, start, end, speechFilename, SnakeImageLPC)) {
    fenSnake = enEcht(16);
    deplaceSnake = (int)((end - start + 1 - fenSnake) / (DIM_X - 1)) + 1;
    calculimLPC(speech, start, end, ordre_lpc_msTOecht(opt_for.or_lpc));

    d_eImage = d_e;
    startImage = start;
    endImage = end;
    strcpy(speechSignalName, speechFilename);
    typeSnakeImage = SnakeImageLPC;
  }
}

/*
 * Calcul de l'image obtenue par lissage cepstral sur le signal.
 */
static void calculimCeps(
  SAMPLE *speech,
  int premEcht,
  int derEcht,		/* premier et dernier echantillon de l'image a calculer */
  int lifterStart,
  int lifterEnd,
  int cepstreOuHalle)	/* 0 si cepstre et 1 si Halle */
{
  int ordre = ORDRECEPSTREIM, i;

  // actualiseIm(CEPSTRE);     ATTENTION
  largeurIm = (derEcht - premEcht + 1 - ordre) / deplaceSnake + 1;
  hauteurIm = ordre / 2;
  
  for (i = 0; i < largeurIm; i++) {
    FLOATbuf spectre(ORDRECEPSTREIM,"cepstre dans calculimCeps"),
             *PX = &spectre;
    float *spectreHalle = new float[ORDRECEPSTREIM];
    int j;
    if (!cepstreOuHalle) cepstre(&PX, speech, premEcht + i * deplaceSnake, 1, 0, ORDRECEPSTREIM, ORDRECEPSTREIM, ORDRECEPSTREIM,
            lifterStart, lifterEnd, 1, true, false);
    else halle(spectreHalle, speech, premEcht + i * deplaceSnake, ORDRECEPSTREIM, ORDRECEPSTREIM,
                   lifterStart, lifterEnd, 0., 6, true);

    for (j = 0; j < ORDRECEPSTREIM / 2; j++) {
      // on enleve 10 dB a l'enveloppe vraie car l'enveloppe vraie est plus
      // haut que le lissage cepstral.
      float e = (!cepstreOuHalle) ? spectre[j] : spectreHalle[j] - 10.;
      IMA[i][j]= e;
      IMA_GRAM[i][j] = IMA_GRA[i][j] =
          (int) ((e <= SILENCE_SPECTRO) ? 0 : ((e >= MAX_SPECTRO + 5.) ? NBNIV - 1 :
		     1 + (int) ((NBNIV - 2.) * (e - SILENCE_SPECTRO) / (MAX_SPECTRO + 5. - SILENCE_SPECTRO))));
    }
    delete [] spectreHalle;
  }
}

#define FENLPCMS 16
/* Racines de toutes les LPC de l'image (permet de connaitre le rayon de chaque racine
   et donc de recalculer le spectre si la racine ne correspond pas a un pic. */
static racines_lpc allLpcRoots;
static void calculimLPC(
  SAMPLE *speech,
  int premEcht,
  int derEcht,		/* premier et dernier echantillon de l'image a calculer */
  int order)   // prediction order of LPC
{
  int ii;
  fenSnake = enEcht(FENLPCMS);
  float *r = new float[order + 1],
        *a = new float[order + 1],
        *sf = new float[order + 1],
        *xi = new float[fenSnake],
        *xr = new float[fenSnake];
  float *fpos = new float[order +1],
        *fbw = new float[order + 1],
        *coef = new float[order + 1];
  double *realfft = new double[hauteurIm * 2],
         *imagfft = new double[hauteurIm * 2];

  //actualiseIm(LPC);         ATTENTION
  hauteurIm = 256;
  largeurIm = (derEcht - premEcht + 1 - fenSnake) / deplaceSnake + 1;

  initham_((float *) NULL, fenSnake);
  allLpcRoots.resize(largeurIm);
  for (ii = 0; ii < largeurIm; ii++) {
    SAMPLE *cnv = &speech[premEcht + ii * deplaceSnake];
    float e;

    preac_f_(cnv, 0, xr, fenSnake);
    ham_ff_(xr, xr, (float *) NULL, fenSnake);

    autolpc (xr, r, a, sf, &e, fenSnake, order);

    realfft[0] = 1.0;
    imagfft[0] = 0.0;
    coef[0] = 1.;

    allLpcRoots[ii].resizeOrder(order);
    allLpcRoots[ii].ak[0] = e;
    for (int j = 1;  j <= order; j++)  {
      realfft[j] = -a[j];
      imagfft[j] = 0;
      coef[j] = allLpcRoots[ii].ak[j] = -a[j];
    }

    for (int j = order + 1; j < hauteurIm * 2; j++) realfft[j] = imagfft[j]  = 0.;
    fft(hauteurIm * 2, realfft, imagfft);

    fband(order, coef, (float) en_cours.frequence, 700., 0.0, (float) (en_cours.frequence / 2.0), fpos, fbw);

    allLpcRoots[ii].resize((int) fpos[0]);
    for (int j = 1 ; j <= allLpcRoots[ii].ni ; j++) {
      allLpcRoots[ii][j-1].fr = fpos[j];				/* frequence en Hz */
      allLpcRoots[ii][j-1].lar = fbw[j];				/* largeur de bande en Hz */
    }


    if (fabs(e) < EPSILON) e = EPSILON;
    e = (float) log10((double)e);
    for(int j = 0; j < hauteurIm; j++) {
      float xr = realfft[j],
            xi = imagfft[j],
      x = 10 * (e - log10(xr * xr + xi * xi + EPSILON));
      IMA[ii][j] = x;
      IMA_GRAM[ii][j] = IMA_GRA[ii][j] = (int) ((x <= SILENCE_SPECTRO) ? 0 :
			                 ((x >= MAX_SPECTRO + 5.) ? NBNIV - 1 :
			    				   1 + (int) ((NBNIV - 2.) * (x - SILENCE_SPECTRO) / (MAX_SPECTRO + 5. - SILENCE_SPECTRO))));
    }
  }
  delete[] r;
  delete[] a;
  delete[] sf;
  delete[] xi;
  delete[] xr;
  delete[] fpos;
  delete[] fbw;
  delete[] coef;
  delete[] realfft;
  delete[] imagfft;
}

void savePgm(
  int image[DIM_X][DIM_Y],
  int largeur,
  int hauteur,
  char *filename)
{
  FILE *ficImage = fopen(filename,"wb");

  int i, j;

  if (!ficImage) {
    affiche_erreur("Unable to create the PGM file");
    return;
  }

    
  fprintf(ficImage, "P5 %d %d 255 ", largeurIm, hauteurIm);
  for (int i = hauteurIm - 1; i >= 0; i--)
    for (int j = 0; j < largeurIm; j++) {
      unsigned char x = image[j][i];
      fwrite(&x, sizeof(char), 1, ficImage);
    }
  fclose(ficImage);  
}


#ifdef UIY

/* Recalcul d'un spectre LPC avec un rayon permettant de faire appaitre un pic pour une
   racine dont la largeur de bande est trop grande. */
void recalculeSpectre(
  double freqFormant,	/* frequence estimee du formant (en Hz) */
  int iSpectre,		/* numero du spectre */
  int ordre_lpc)	/* ordre de la lpc */
{
   int i;
   float dist, rayon;

   /* On cherche la racine la plus proche de la frequence estimee du formant */
   for (i = 0, rayon = 1., dist = INFINI; i < racLpc[iSpectre].effectif; i++)
     if (fabs(freqFormant - racLpc[iSpectre].racines[i].fr) < dist) {
       dist = fabs(freqFormant - racLpc[iSpectre].racines[i].fr);
       /* on prend un rayond proche de celui de la racine a` renforcer */
       rayon =  exp(-M_PI / en_cours.frequence * racLpc[iSpectre].racines[i].lar);
     }

   /* On considere que si la distance entre la racine supposee et la frequence du formant
      est trop grande la racine n'est pas la bonne et on ne recalcule donc pas le spectre. */
   if (dist < 200.) {
     float fft[fenSnake*4]; 
     int j;
     float e = racLpc[iSpectre].poles[0];
     fft[0] = 1.0;
     fft[1] = 0.0; 
     for (j = 1;  j <= ordre_lpc; j++)  {
      fft[j << 1] = racLpc[iSpectre].poles[j] * pow((double) rayon, (double) -j);
      fft[(j << 1) + 1] = 0;
    }
    for (j = ordre_lpc + 1; j < fenSnake * 2; j++)
      fft[j << 1] = fft[(j << 1) + 1]  = 0.;
    fftc512(fft);
    if (fabs(e) < EPSILON) e = EPSILON;
    e = (float) log10((double)e);
    for(j = 0; j < 256; j++) {
      float xr = fft[j << 1], 
	    xi = fft[(j << 1) + 1],
	    x = 10 * (e - log10(xr * xr + xi * xi + EPSILON));
      IMA_GRAM[iSpectre][j] = IMA_GRA[iSpectre][j] =
	(int) ((x <= SILENCE_SPECTRO) ? 0 :
	((x >= MAX_SPECTRO + 5.) ? NBNIV - 1 :
	 1 + (int) ((NBNIV - 2.) * (x - SILENCE_SPECTRO) / (MAX_SPECTRO + 5. - SILENCE_SPECTRO))));
    }
  }
}

/*
 * Sauvegarde d'une partie d'image definie par les colonnes extremes a sauver.
 */
int *sauvePartieIm(
  int iDeb, 
  int iFin)	   /* indices de la premiere et de la derniere colonnes a` sauvegarder. */
{
  int *image =  ALLOUERM(int, DIM_Y * (iFin - iDeb + 1));

  blkcpy((char *) image, (char *) &IMA_GRA[iDeb][0], DIM_Y * (iFin - iDeb + 1) * sizeof(int));
  return(image);
}
    
/*
 * Restauration d'une partie d'image.
 */
void restaurePartieIm(
  int numSpectreDeb, 
  int numSpectreFin, 
  int *partieDim)
{
  blkcpy((char *) &IMA_GRA[numSpectreDeb][0], (char *) partieDim, 
	 DIM_Y * (numSpectreFin - numSpectreDeb + 1) * sizeof(int));
}
#endif



