/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include <math.h>

#include "SuiviSnake/snake.h"



/* Inversion d'une matrice  bande ( a bande reguliere) stockee lineairement
   ayant deux blocs triangulaires de dimension 2 dans les coins.
 * Les bande successives   sont stockees a la queue leu leu.
   Les 2 coins sont stockes dans un tableau tab.
 * La matrice de depart n'est pas conservee.
 * dbande est la demi largeur  de bande
 * la fonction position determine la place de l'element a(i,j)
 * dans le tableau lineaire.

 | a[1]  a[2]   a[3]                            tab[3] tab[4]  |
 | a[4]  a[5]   a[6]  a[7]                             tab[5]  |
 | a[8]  a[9]   a[10] a[11]  a[12]                             |
 |       a[13]  a[14] a[15]  a[16] a[17]                       |
 |                                                             |
 |                                                             |
 |                                                             |
 |                                                             |
 |tab[0]                                                       |
 |tab[1] tab[2]                                                |

 Apres usage de la triangulation, les elements 3 4 et 5 de tab generent
 une matrice verticale [n][2] dans la coin droit de la matrice
 La 1ere colonne va jusqu'a la ligne n-dbande-2;
 La 2ieme colonne va jusqu'a la ligne n-dbande -1

 Il est egalement genere une matrice [2][n] en bas de la matrice
 stockee dans bas[2][n]. */

  
/* a est stockee dans un tableau lineaire */

/*triangulation de la matrice bande a.(bande est la largeur de bande) */
/*Les memes operations etant effectuees sur Id et stockees dans b */


void trianguler2(
  double *a,
  double **b,
  int dbande,
  double tab[], /* tab : tableau des elements de coin */
  double **droite,/* matrice a "droite" issue de la triangulation
			 et resultant des elements tab[3 a 5]
			 numerotation droite[0][0] droite[1][0]
			              droite[0][1] droite[1][1]
				      droite[0][2] .........*/
  double **bas,/* matrice en bas de la matrice initiale
			bas[0][0] bas[0][1] bas[0][2] bas[0][3]
			bas[1][0] bas[1][1] bas[1][2]  bas[1][3]*/
  int n)
{
  double x,*p,*q,kk;
  int k,l,ll,i,j,numero,numero1,num1,num2,bande,i1;
 
  printf(" trianguler \n");

  bande=2*dbande+1;

/* initialisation de b */

  for (i=0;i<n ;i++)
    {p=b[i];
    for(j=0;j<n;j++)
      (*(p+j))=0.;
    (*(p+i))=1.;
    }
  printf(" fin initialisation de b\n");	
/* initialisation de droite et de bas */
  for(i=0;i<2;i++)
    for(j=0;j<n;j++)
      {droite[i][j]=0.;bas[i][j]=0.;}

  printf(" initialisation terminee \n"); 
  droite[0][0]=tab[3];
  droite[1][0] = tab[4];
  droite[1][1] = tab[5];

  bas[0][0] = tab[0];
  bas[1][0] = tab[1];
  bas[1][1] = tab[2];

  printf(" affectation terminee \n");
  /* -->>>> changement de n en n-1 */
  for(j=0;j<n;j++)


    /* travail sur les colonnes de la matrice  */
    /* recherche du coeff max de la colonne j */
    { 
      numero=position(j,j,dbande,n);
      x =a[numero];
      k=j;
      for(l=j;l<n;l++)
	numero=position(l,j,dbande,n);
      if (abs(a[numero])>x) {x=a[numero];
      k=l;}


      if (x==0) printf(" matrice non inversible ");
      else {k=l;
/* if (k!=l) {echanger(a,k,j,n);
 *           echanger(b,k,j,n);}*/

      numero1=position(j,j,dbande,n);

	 
      for(i=j+1;i<=j+dbande;i++)
	/* on met des 0 dans la colonne j jusqu'a la ligne j+dbande
	   En dessous, il y a forcement 0 a cause de la structure de bande */
     
	if (i<n)
	  {
	    numero=position(i,j,dbande,n);
	    kk=a[numero]/a[numero1];


	    /* seules les lignes n-4 a n-1 sont touchees par l'influence de la
	       matrice droite.
	       rq: droite vaut 0 la ou elle n'a pas de sens
	       Les operations ci dessous peuvent etre effectuees partout */
       
	    if (i==n-4 && j<=n-dbande-3)
	      a[position(n-4,n-2,dbande,n)] -= kk*droite[0][j];
       	  

	    if (i>=n-3)
	      {       a[position(i,n-2,dbande,n)] -= kk*droite[0][j];
	      a[position(i,n-1,dbande,n)] -= kk*droite[1][j];}

	    /* colonnes n-1 et n-2  */

	 
	 
	    for(ll=j-dbande;ll<=j+dbande;ll++) if (ll>=0 && ll<n) {
         /* (*(p+ll))=(*(p+ll))-a[j][ll]*kk;*/
		   /* (*(q+ll))=(*(q+ll))-b[j][ll]*kk;*/
		   num1=position(i,ll,dbande,n);
		   num2=position(j,ll,dbande,n);

		   a[num1]=a[num1]-(a[num2]*kk);
       }

	    /* calcul de la matrice generee a droite */
	    if (i<= (n-dbande-3))
	      droite[0][i]   += ( -kk*droite[0][j]);

	    if (i<= (n-dbande-2))
	      droite[1][i]   += ( -kk*droite[1][j]);
       
	    for (ll=0;ll<n;ll++)
	      b[i][ll]=b[i][ll]-(b[j][ll]*kk); 
	  }




/* Traitement des 2 lignes inferieures de la matrice */

      for(k=0;k<2;k++)
	   
	{ 
	  kk = bas[k][j]/a[numero1];

	  for(ll=j;ll<=n-dbande-3+k;ll++)
	    if (ll<=j+dbande)
	      bas[k][ll] -= (kk*a[position(j,ll,dbande,n)]);
	     
	  /* modification de la matrice a */
	  for(ll=n-dbande-2+k;ll<n-2;ll++)
	    if (ll<=j+dbande)
	      a[position(n-2+k,ll,dbande,n)] -= (kk*a[position(j,ll,dbande,n)]);
	    
	  if (n-2>j+dbande)
	    a[position(n-2+k,n-2,dbande,n)] -= (kk*droite[0][j]);
	  else
	    a[position(n-2+k,n-2,dbande,n)] -= (kk*a[position(j,n-2,dbande,n)]);
	  if(n-1>j+dbande)
	    a[position(n-2+k,n-1,dbande,n)] -= (kk*droite[1][j]);
		
	  else
	    a[position(n-2+k,n-1,dbande,n)] -= (kk*a[position(j,n-1,dbande,n)]);
	       
     

	  /*mise a jour de la matrice b */
	  for(ll=0;ll<n;ll++)
	    b[n-2+k][ll] -= (kk*b[j][ll]);
	}
      }

    }
}



void resolution2(
  double *a,
  double **b,
  double dbande, 
  double tab[],
  double **droite,
  double **bas,
  int n)
{
  double x;
 int i,j ,k,numero,num;


 for(k=0;k<n;k++)
   {numero=position(n-1,n-1,dbande,n);
    b[n-1][k]=b[n-1][k]/a[numero];}
 
 for (i=n-2;i>=0;i--)
   {numero=position(i,i,dbande,n);
    for (k=0;k<n;k++)
      { x=0.0;
	for (j=i+1;j<=i+dbande;j++)
	 
	     if (j<n)
	       {num=position(i,j,dbande,n);
		x=x+a[num]*b[j][k];}
	x += droite[0][i]*b[n-2][k]+droite[1][i]*b[n-1][k];
	 
      
	b[i][k]=(b[i][k]-x)/a[numero];}
      
   }
 }




/* inversion de la matrice a .resultat dans b */ 
void  inverser2(
  double *a,
  double **b,
  int dbande,
  double tab[],
  double **droite,
  double **bas,
  int n)
{
  int i,j,longueur;
 
  /* longueur du tableau a */
  longueur=(n)*(2*dbande+1) - dbande*(dbande+1);

  trianguler2(a,b,dbande,tab,droite,bas,n);
  resolution2(a,b,dbande,tab,droite,bas,n);
}


