/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include "SuiviSnake/snake.h"

/*
 * Discretisation simplifiee dans le cas ou seules les ordonnees peuvent varier.
 */
int  discret_snakeFctX(
  type_snake *snake,
  int nb_pts,
  int sousEcht)
{ 
  int i, j;
  for (i = 0, j = 0; i < nb_pts; i = i + sousEcht,j++) 
    snake->snake.snake_pts[j] = snake->snake_init[i];
  return (j);
}


/*--------------------------------------------------------------------------*/
/* Procedure de Discretisation du SNAKE :                                   */
/* on discretise le contenu de SNAKE_PTS                                    */
  /* Le Snake cree  discretise en une suite de points equidistants.       */
/* Le pas de discretisation est donne par :  longueur_SNAKE / discret       */
/* discret est la constante de discretisation.                              */
/*--------------------------------------------------------------------------*/

int  discret_snake(
  type_snake *snake,
  int nb_pts)
{ 
  float             len_snake = 0.;
  register          i, j;
  float             pas,
                    lambda;
  int               fin, discret;
  type_coord_float  A, 
                    B,
                    C; /* A, B et C sont 3 points consecutifs sur le SNAKE */
  type_coord_float     *snake_init,
                       *snake_pts;


  snake_init = (type_coord_float *)snake->snake_init;
  snake_pts = (type_coord_float *)snake->snake.snake_pts;
  
/*..... Calcul de la longueur du SNAKE .....................................*/

  for (i = 1, A = snake_init[0]; i < nb_pts; A = snake_init[i], i++)
  len_snake += (float)sqrt(puissance(A.x - snake_init[i].x, 2) + 
                    puissance(A.y - snake_init[i].y, 2)); 
 if (snake->cst.autom ==1)
   discret = (int)len_snake/1;
 //   discret = (int)len_snake/3;
  
/*..... Cas ou le calcul du SNAKE n'est d'aucun interet.....................*/
  if (discret == 0)    return -1;
  if (len_snake == 0.) return 0;

/*..... pas de discretisation ..............................................*/
  pas = (float) len_snake / (float) (discret - 1);

/*..... Initialisation de la procedure de discretisation ...................*/
  A = snake_init[0]; B = snake_init[1];
  snake_pts[0] = A;

  for (i = 1, j = 1; i < nb_pts;)
  { 
/*..... Calcul de lambda, parametre de l'equation barycentrique ............*/

    lambda = 1.-( pas / distance(A, B));
/* printf("%f %f\n",pas / distance(A, B),lambda); */

/*..... Cas ou le point P est en dehors du segment [A,B] ...................*/
/* -0.2 a la place de 0 pour eviter les raccourcissements brutaux */
    if (lambda < 0.0 || lambda > 1.)

      

      { 
/*..... on recherche le point P tel que : d(A,P) = pas et P appartient .....*/
/*..... au segment [B,C] (1) ...............................................*/
        C = snake_init[++i]; 
        fin = 0;
        while ( i < nb_pts && (! ( fin) ) )
	  { type_coord_float   CB, AC; 
            float              sca, disc;
            float              dCB = distance(C, B),
                               dAC = distance(A, C);

/*..... Resolution de l'equation du 2d degre correspondant au systeme (1)..*/
            CB.x = B.x - C.x; CB.y = B.y - C.y;
            AC.x = C.x - A.x; AC.y = C.y - A.y;
            sca = scalaire(CB, AC);
            disc =  sca * sca - dCB*dCB*(dAC * dAC - pas * pas);

            if (disc >= 0.) 
	      { disc = sqrt(disc);
/*..... calcul de la 1ere Solution .........................................*/
                lambda = (- sca + disc) / (dCB * dCB);

/*..... calcul de la 2de solution ..........................................*/
                 if ( lambda <= 0. || lambda > 1.)
                    lambda = (- sca - disc) / (dCB * dCB);

/*..... S'il y a une solution alors maj de A, B et C .......................*/
                 if (fin = ( lambda > 0. &&  lambda <= 1.))
                    { A.x = lambda * B. x + (1 - lambda) * C.x;
                      A.y = lambda * B. y + (1 - lambda) * C.y;
                      B = C;
                      snake_pts[j++] = A;
                    }
                 else  { B = C; C = snake_init[++i];}
	      }
            else { B = C ; C = snake_init[++i];}
	  } /* fin while */
      } /* fin partie alors */

    else
/*..... Cas ou le point appartient au segment [A,B].........................*/
/*..... A prend la valeur du point P .......................................*/
      { A.x = lambda * A. x + (1 - lambda) * B.x;
        A.y = lambda * A. y + (1 - lambda) * B.y;
        snake_pts[j++] = A;
      }
  }



/*..... on retourne le nombre de points de discretisation generes ..........*/
  printf(" nombre de pts souhaites %d  nombre effectif %d\n",discret,j);
  if (j > MAX_PTS) { 
    affiche_erreur(" Le nombre maximum de points est atteint \n Je tronque a MAX_PTS");
    j = MAX_PTS-10;
  }

  else if (j == -1)  /*.. la constante de discretisation est trop faible ..*/
    affiche_erreur("WARNING : Discretisation impossible, changez la constante de discretisation ....");

  else if (j == 0 || j == 1) /*.. le Snake n'a pas ete cree ..*/
    affiche_erreur("WARNING : Discretisation impossible ... Le SNAKE n'existe pas ...");
  
  return j;
}
#ifdef UIY
/*------------------------------------------------------------
Affichage des courbes (toutes, une ou modulo 5)
------------------------------------------------------------*/
void aff_tout(
  Widget w,
  type_snake *snake,
  XmAnyCallbackStruct *cbs)
{
  indic_aff_courbe=1;
}

void aff_une(
  Widget w,
  type_snake *snake,
  XmAnyCallbackStruct *cbs)
{
  indic_aff_courbe=0;
}

void aff_mod_5(
  Widget w,
  type_snake *snake,
  XmAnyCallbackStruct *cbs)
{
  indic_aff_courbe=5;
}
#endif
