/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include <math.h>
#include "SuiviSnake/snake.h"
#include "Numerical/nrutil.h"
#include "Numerical/nr.h"

extern int VAL_PROPRE;


static void matvec_bande(double a[], int dbande, double v[], double w[], int debut, int fin);

void aff_mat(
  double  a[SNAX][SNAX],
  int n)
{
  int i,j;
  for(i=0;i<n;i++)
    {for(j=0;j<n;j++)
      printf(" %g ",a[i][j]);
    printf("\n");
    }
}

/* initialisation d'une matrice(n,m)  a zero */

void init(
  double **A,
  int n,
  int m)
{
  double    *p;
  int i,j;
  for(i=0;i<n;i++) {
    p=A[i];
    for (j=0;j<m;j++)
      (*(p++))=0.;
  }
}


void affvec(
  double A[],
  int N)
{
  int i;

  for(i=0;i<N-1;i++)
    printf("(%d ,%lf)\n",i,A[i]);
  printf("(%d,%lf)\n",N-1,A[N-1]);
  printf("\n");
  return;
}



void affvec_float(
  float A[],
  int N)
{
  int i;

  for(i=0;i<N-1;i++)
    printf("(%d ,%f)\n",i,A[i]);
  printf("(%d,%f)\n",N-1,A[N-1]);
  printf("\n");
  return;
}




void affvec_(
  type_coord_float u[],
  double A[],
  int N)
{
  int i;

  for(i=0;i<N-1;i++)
    printf("%d ,[%f %f],  %lf)\n",i,u[i].x,u[i].y,A[i]);
  printf("%d, [%f %f],%lf \n",N-1,u[N-1].x,u[N-1].y,A[N-1]);
  printf("\n");
  return;
}




/*produit de la matrice A par le vecteur u . Au=v */

void matvec(
  double **A,
  double u[],
  double v[],
  int n,
  int m)
{ 
  double     *p;
  int i,j;
  for(i=0;i<n;i++)
    {p=A[i];
    (*(v+i))=0.;
    for(j=0;j<m;j++)
      (*(v+i))=(*(v+i)) + ((*(p+j))*(*(u+j)));
    }
}



/*produit de la matrice A par le vecteur u . Au=v
 A est  de taille n*m
 u est de longueur queconque.
 ON prend seulement n coordonnes a partir del' indice i
 Les autres coordonnees restent inchangees */

void matvec_bis(
  double **A,
  double u[],
  double v[],
  int n,
  int m,
  int debut)
{
  double     *p;
  int i,j;
  for(i=0;i<n;i++) {
    p=A[i];
    (*(v+i+debut))=0.;
    for(j=0;j<m;j++)
      (*(v+i+debut))=(*(v+i+debut)) + ((*(p+j))*(*(u+j+debut)));
    }
}


void matvec_s(
  double A[SNAX][SNAX],
  double u[SNAX],
  double v[SNAX],
  int n,
  int m)
{
  double   *p;
  int i,j;
  for(i=0;i<n;i++)
    {p=A[i];
    (*(v+i))=0.;
    for(j=0;j<m;j++)
      (*(v+i))=(*(v+i)) + ((*(p+j))*(*(u+j)));
    }
}


/* produit d'une matrice a (n,m) par un scalaire */
/* resultat dans b */

void scal_mat(
  double scal,
  double a[SNAX][SNAX],
  double b[SNAX][SNAX],
  int n,
  int m)
{
  int i,j;
  double    *p,*q;
  for (i=0;i<n;i++)
    {p=a[i];q=b[i];
    for(j=0;j<m;j++)
      (*(q++))=scal*(*(p++));
    }
}



/* produit matriciel.  resultat dans c */
/* A(n,m) B(m,p)                       */

void produit(
  double A[SNAX][SNAX],
  double B[SNAX][SNAX],
  double C[SNAX][SNAX],
  int n,
  int m,
  int p)
{
  double    *pp,*q,x;
  int i,j,k;

  for(i=0;i<n;i++)
    {pp=A[i];q=C[i];
    for(j=0;j<p;j++)
      {x=0.;
      for(k=0;k<m;k++)
        x=x+(*(pp+k))*B[k][j];
      (*(q+j))=x;
      }
    }
} 



/*  somme de matrices (n,m) */
void somme(
  double A[SNAX][SNAX],
  double B[SNAX][SNAX],
  double C[SNAX][SNAX],
  int n,
  int m)
{
  int  i,j;
  double    *p,*q,*r;
 
  for (i=0;i<n;i++)
    { p=A[i] ; q=B[i] ; r=C[i] ;
    for(j=0;j<m;j++)
      (*(r+j))=(*(p+j)) + (*(q+j));
    }
}



/*  somme alpha A + beta B de matrices (n,m) */
void comblin(
  double A[SNAX][SNAX],
  double alpha,
  double B[SNAX][SNAX],
  double beta,
  double C[SNAX][SNAX],
  int n,
  int m)
{
  int  i,j;
  double    *p,*q,*r;
 
  for (i=0;i<n;i++)
    { p=A[i] ; q=B[i] ; r=C[i] ;
    for(j=0;j<m;j++)
      (*(r+j))=alpha*(*(p+j)) + beta*(*(q+j));
    }
}




/* transposition de la matrice A (n,m) .Resultat dans B  */

void transposer(
  double A[SNAX][SNAX],
  double B[SNAX][SNAX],
  int n,
  int m)
{
  int i,j;
  double    *p;

  for(i=0;i<m;i++)
    for(j=0;j<n;j++)
      B[i][j]=A[j][i];
}


/* recherche d'un vecteur w normal au vecteur v */
void normal(
  double v[SNAX],
  double w[SNAX])
{
  double    x;
  int i,j;

  if (v[0]!=0) {w[0]=(-v[1]-v[2])/v[0];
  w[1]=1.;
  w[2]=1.;}
  else {if (v[1]!=0) {w[0]=0.;
  w[1]=(-v[2]/v[1]);
  w[2]=1;}
  else    {w[0]=1.;
  w[1]=1.;
  w[2]=0.;}
  };
 
  x=w[0]*w[0]+w[1]*w[1]+w[2]*w[2];
  x=sqrt(x);
  for(i=0;i<3;i++)
    w[i]=w[i]/x;
} 
        

/*produit scalaire des vecteurs u et v */
double scalaire1(
  double u[SNAX],
  double v[SNAX])
{
  double    x;
  x=u[0]*v[0] + u[1]*v[1] + u[2]*v[2];
  return x;
}

/*------------------------------------------------------------
  Produit scalaire dans Rn
  ------------------------------------------------------------*/
double scalaire_(
  double u[],
  double v[],
  int debut,
  int fin)
{
  double    x=0;
  int i;
  for(i=debut;i<fin;i++)
    x +=u[i]*v[i];
  return x;
}


/*------------------------------------------------------------
  calcul de tx A x ou A matrice creuse
  ------------------------------------------------------------*/

double norme_A(
  double a[],
  int dbande,
  double x[],
  int debut,
  int fin)
{
  double v[MAX_PTS];
  matvec_bande(a,dbande,x,v,debut,fin);
  return  scalaire_(v,x,debut,fin);
}
	           
	
	       

     

/*produit vectoriel des vecteurs u et v .Resultat dans w */
void produit_vect(
  double u[SNAX],
  double v[SNAX],
  double w[SNAX])
{
  w[0]=u[1]*v[2] - u[2]*v[1];
  w[1]=v[0]*u[2] - u[0]*v[2];
  w[2]=u[0]*v[1] - u[1]*v[0];
}



/* Produit de la matrice carree bande a stockee lineairement avec des coins
   par le vecteur v. Resultat dans w.
   a: stockage lineaire de la matrice bande
   tab : tableau des coins
   */
void matvec_circ(
  double a[],
  double tab[],
  int dbande,
  double v[],
  double w[],
  int n)
{
  int i,j,numero;
  for(i=0;i<n;i++)
    {w[i] = 0.;
    for(j=i-dbande;j<=i+dbande;j++)
      {if (j<n && j>=0)
	w[i] += a[position(i,j,dbande,n)];
      }
    /* rectifications dues aux coins */
    switch(i){
    case 0 : w[0] += tab[3]*v[n-2] + tab[4]*v[n-1];break;
    case 1 : w[1] += tab[5]*v[n-1];break;

    }
    if(i== n-2)
      w[n-2] += tab[0]*v[0];
    else
      if(i==n-1)
	w[n-1] += tab[1]*v[0] + tab[2]*v[1];
    }
}


static void matvec_bande(
  double a[],
  int dbande,
  double v[],
  double w[],
  int debut,
  int fin)
{
  int i,j,numero;
  for(i=0;i<fin-debut;i++)
    {w[i+debut] = 0.;
    for(j=i-dbande;j<=i+dbande;j++)
      {if (j<fin-debut && j>=0)
	w[i+debut] += a[position(i,j,dbande,fin-debut)]*v[j+debut];
      }
    }
}
 
/* distance entre deux points dans R2 */
double dist_euclid(
  double x1,
  double y1,
  double x2,
  double y2)
{
  return sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
}





#ifdef w777
/*------------------------------------------------------------
  Procedure donnant les valeurs propres d'une matrice
  (Voir Numerical recipes &11.5 et 11.6
  a matrice dont on cherche les valeurs propres
  n taille de la matrice
  w_r partie reelle des valeurs propres
  w_r partie imaginaire des valeurs propres
------------------------------------------------------------*/

void val_hqr(a,n,wr,wi)
     double a[][SNAX];
     int n;
     float *wr,*wi;

{


	int i,j;
	float **b;
	float aa[SNAX][SNAX];

	/* Conevrsion double en float */
	for(i=0;i<n;i++)
	  for(j=0;j<n;j++)
	    aa[i][j]=(float) a[i][j];

	b=convert_matrix(&aa[0][0],1,SNAX,1,SNAX);
	printf("matrix:\n");
	for (i=1;i<=n;i++) {
	  for (j=1;j<=n;j++) printf("%12.2f",b[i][j]);
	  printf("\n");
	}
	balanc(b,n);
	elmhes(b,n);
	hqr(b,n,wr,wi);
	printf("eigenvalues:\n");
	printf("%11s %16s \n","real","imag.");
	for (i=1;i<=n;i++) printf("%15f %14f\n",wr[i],wi[i]);
	free_convert_matrix(b,1,n,1,n);

}
#endif
#ifdef q777

/*------------------------------------------------------------
  Positionnement de l'indicateur VAL_PROPRE indiquant si on veut le calcul des
  valeurs propres de la matrice
  VAL_PROPRE = 1 on affiche 

------------------------------------------------------------*/

void t_val_propre(m,mi)
     Menu         m;
     Menu_item    mi;

{
  type_client    *client;
  Textsw         fen_text;
  
  client        = (type_client *) menu_get(m , MENU_CLIENT_DATA);
  fen_text      = client->interface.texte;

     

  VAL_PROPRE = 1 - VAL_PROPRE;

  if (VAL_PROPRE)
    voir_message(fen_text," Calcul et Affichage des valeurs propres ",1);
  else
    voir_message(fen_text," Pas d' Affichage des valeurs propres ",1);
}

#endif



/*------------------------------------------------------------
  Resolution de systemes lineaires par la methode SVD
  Voir NUmerical recipes
  ------------------------------------------------------------*/
void solve_svd(
  float **a,
  float *c,
  float *x,
  int n,
  int m)
{
  int k,l,j;
  float **u,**v,*w;
  float wmax,wmin;
  u =matrix(1,SNAX,1,SNAX);
  v=matrix(1,SNAX,1,SNAX);
  w=vector(1,SNAX);
  /* copy a into u */
  for (k=1;k<=n;k++)
    for (l=1;l<=n;l++) u[k][l]=(float) a[k][l]; 
  /* decompose matrix a */
  svdcmp(u,n,n,w,v);
  /* find maximum singular value */

  wmax=0.0;
  for (k=1;k<=n;k++)
    if (w[k] > wmax) wmax=w[k];
  /* define "small" */
  wmin=wmax*(1.0e-6);
  /* zero the "small" singular values */
  for (k=1;k<=n;k++)
    if (w[k] < wmin) w[k]=0.0;

  svbksb(u,w,v,n,n,c,x);
/*	printf(" solution vector is:\n");
	for (k=1;k<=n;k++) printf("%12.6f",x[k]);
	printf("\n original right-hand side vector:\n");
	for (k=1;k<=n;k++) printf("%12.6f",c[k]);
	printf("\n (matrix)*(sol'n vector):\n");
	for (k=1;k<=n;k++) {
	c[k]=0.0;
	for (j=1;j<=n;j++)
	c[k] += a[k][j]*x[j];
	}
	for (k=1;k<=n;k++) printf("%12.6f",c[k]);
	printf("\n");

	printf ("***********************************\n");
	printf("press RETURN for next problem\n");
	
	*/      
  free_matrix(v,1,SNAX,1,SNAX);
  free_vector(w,1,SNAX);
}



#ifdef hh777
void t_solve(m,mi)
     Menu         m;
     Menu_item    mi;

{
  type_client    *client;
  Textsw         fen_text;
  float *x,*y,*cx,*cy,**aaa1;
  int i,j,discret,k,l;
  FILE *fff;

  client        = (type_client *) menu_get(m , MENU_CLIENT_DATA);
  fen_text      = client->interface.texte;


      x=vector(1,SNAX);
      y=vector(1,SNAX); 
      cx=vector(1,SNAX);
      cy=vector(1,SNAX);
      aaa1=matrix(1,SNAX,1,SNAX);
      

      fff=fopen("Num/mat","r");
      while (!feof(fff)) {
	fscanf(fff,"%d %d ",&discret,&discret);
	for (k=1;k<=discret;k++)
	  for (l=1;l<=discret;l++) fscanf(fff,"%f ",&aaa1[k][l]);
	for (l=1;l<=discret;l++) fscanf(fff,"%f ",&cx[l]);
      }

      solve_svd(aaa1,cx,x,discret,discret);
}
	





#endif
