/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef _SNAKE_H
#define _SNAKE_H

#include "common.h"
enum TypeInitialSnakes{InitialCurvesLPC, InitialCurvesTrueenv};

#define DIM_X 512
#define DIM_Y 256
#define ORDRECEPSTREIM	512		/* ordre du lissage cepstral pour le calcul de l'image de suivi */
#define MAX_PTS 1000
#define SNAX 1000						/* taille des matrices pour le calcul des snakes */
#define INICST {2., 0, 1, 15, 0, 0.,SNAKE_IMA_ENER_GRA,0.,SNAKE_E_REPULS,SNAKE_SIGMA_REPULS,{SNAKE_E_INT_ELA, 3.,0.},60}


/*----- DEFINITION ET INITIALISATION DES AUTRES PARAMETRES ------------------*/
#define                 SNAKE_GAMMA         1.
#define                 SNAKE_DISC          30
#define                 SNAKE_IMA_ENER_INT  1.
#define                 SNAKE_IMA_ENER_GRA  20.
#define                 SNAKE_IMA_ENER_POI  1.
#define                 SNAKE_E_INT_ELA     3.
#define                 SNAKE_E_INT_RIG     1.
#define                 SNAKE_E_INT_POI     1.
#define                 SNAKE_E_EXT_POI     1.
#define			SNAKE_E_REPULS      10.
#define			SNAKE_SIGMA_REPULS  7.


typedef struct {
  int   x, y;
} type_coord;  

typedef struct {
  float   x, y;
} type_coord_float;  

struct snakeCstes { 
  double gamma;
  int    discret;
  int    sousEcht;		   /* Sous-chantillonnage de la courbe initiale. */
  int    fermeture;
  int    autom;     		   /* autom =1 si discretisation automatique */
  double ener_int;
  double ener_gra;
  double ener_poi;
  double ener_repuls; 
  double sigma_repuls;                  
  struct { double ela,
	     rig,
	     poi;}    e_int;
  int nb_itera;
};

typedef struct donEquation {
  int discret;		/* ordre du systeme et donc taille des matrices */
  double **a,		/* matrice pentadiagonale */
         **a1,		/* matrice pentadiagonale pour le type 3 */
         *b,		/* bande de la matrice */
         **droite,      /* partie droite de l'equation. (matrice 2 x N) */
         **bas,		/* matrice 2 x N */
         *alpha, *beta, 
         tab[6],	/* variables necessaires a l'inversion de matrices 
			   afin de representer les termes hors diagonale */
         membre[4];

  double pas;
  int itera;
  double *x_old, *y_old, *x_new, *y_new; /* Coordonnees du snake en cours
					    de calcul. */
} snaEq;

typedef  struct  {
  int nbPointsInit;
  type_coord_float *snake_init;
  struct { 
    STRING nom_int;
    STRING nom_gra;
    int dim_x, dim_y,
        num; /* ancien interface.num */
   } image;

  struct { 
    int nb_pts, nb_sauv;
    type_coord_float *snake_sauv,*snake_pts;
    STRING  nom_sna;
  } snake;
  struct snakeCstes cst;
  snaEq eq;
} type_snake;

extern void uneIterationSnake(type_snake *snake, type_snake *lowerSnake, type_snake *higherSnake);
extern void sauvegarde(type_snake *snake);
extern void videSnake(type_snake *snake);
extern unsigned int largeurIm;		/* Dimensions de l'image de gradient. */
extern unsigned int hauteurIm; // ATTENTION ESSAI
extern int deplaceSnake;		/* nombre d'echantillons entre 2 colonnes de l'image gradient */
extern int fenSnake;		/* nombre d'echantillons d'un spectre de l'image gradient */
extern bool interactionRepulsive;
extern bool afficheIntermediaires;
extern int masque;

extern bool afficheIntermediaires; 		/* affichage des courbes intermediaires. */
extern int iterationShiftDisplay;
extern float IMA[DIM_X][DIM_Y];
extern int IMA_INT[DIM_X][DIM_Y];
extern int IMA_GRA[DIM_X][DIM_Y];
extern int IMA_GRAM[DIM_X][DIM_Y];
extern struct snakeCstes cst;  // Constants controlling the evolution of snakes

extern void init(double **A, int n, int m);
extern void  inverser1(double *a, double **b, int dbande, int n);
extern void  inverser2(double *a, double **b, int dbande, double tab[], double **droite, double **bas, int n);
extern int position(int i, int j, int dbande, int n);
extern void initialiseCalcul(type_snake *snake);
extern double interpol(type_coord_float m, int tab[][DIM_Y]);
extern int  discret_snakeFctX(type_snake *snake, int nb_pts, int sousEcht);
extern float scalaire(type_coord_float U, type_coord_float V);
extern double distance(type_coord_float A, type_coord_float B);
extern float puissance(float t, int n);
extern void matvec_bis(double **A, double u[], double v[], int n, int m, int debut);
extern void savePgm(int image[DIM_X][DIM_Y], int largeur, int hauteur, char *filename);
#endif
