/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include "common.h"
#include "SuiviSnake/snake.h"
#include "guiklat.h"
#include "SuiviSnake/snakeParameters.h"

struct snakeCstes cst = INICST;	/* Constantes controlant l'evolution des
				   snakes et l'influence du champ image. */
bool afficheIntermediaires = false;
int iterationShiftDisplay = 5;
bool yasnakeParametersControl = false;
TWsnakeParametersControl *snakeParametersControl;

DEFINE_RESPONSE_TABLE1(TWsnakeParametersControl,TDialog)
  EV_CHILD_NOTIFY_ALL_CODES(IDOK,IDOk),
  EV_WM_CLOSE,
END_RESPONSE_TABLE;

TWsnakeParametersControl::TWsnakeParametersControl(PTWindowsObject AParent, LPSTR name)
	:TDialog(AParent,name)
{
  PT[0] =  new TStatic(this, IDC_STATICTEXT1);
  PT[1] =  new TStatic(this, IDC_STATICTEXT32);
  PT[2] =  new TStatic(this, IDC_STATICTEXT24);
  PT[3] =  new TStatic(this, IDC_STATICTEXT30);
  PT[4] =  new TStatic(this, IDC_STATICTEXT2);
  PT[5] =  new TStatic(this, IDC_STATICTEXT3);
  PT[6] =  new TStatic(this, IDC_STATICTEXT4);
  PTnbIterations = new TEdit(this, IDC_EDIT3);
  PTDiscretization = new TEdit(this, IDC_EDIT4);
  PTSpectroWeight = new TEdit(this, IDC_EDIT5);
  PTalpha = new TEdit(this, IDC_EDIT6);
  PTbeta = new TEdit(this, IDC_EDIT7);
  PTgamma = new TEdit(this, IDC_EDIT8);
  PTrepulsion = new TEdit(this, IDC_EDIT9);
  PTCheckIntermediate = new TCheckBox(this, IDC_CHECKBOX1);
  PTshiftDisplay = new TEdit(this, IDC_EDIT10);
  PTOk = new TButton(this,IDOK);
}

void TWsnakeParametersControl::EvClose()
{
  TDialog::EvClose();
  yasnakeParametersControl = false;
}

TWsnakeParametersControl::~TWsnakeParametersControl()
{
  delete PTnbIterations;
  delete PTDiscretization;
  delete PTSpectroWeight;
  delete PTalpha;
  delete PTbeta;
  delete PTgamma;
  delete PTrepulsion;
  delete PTCheckIntermediate;
  delete PTshiftDisplay;

  delete PTOk;
  for (int i = 0; i < 7; i++) delete PT[i];

  yasnakeParametersControl = false;
}


bool TWsnakeParametersControl::CanClose()
{
  STRING s;
  
  PTnbIterations->GetText(s,10);
  cst.nb_itera = atoi(s);
  PTDiscretization->GetText(s,10);
  cst.sousEcht = atoi(s);
  PTSpectroWeight->GetText(s,10);
  cst.ener_gra = atoi(s);
  PTalpha->GetText(s,10);
  cst.e_int.ela = atof(s);
  PTbeta->GetText(s,10);
  cst.e_int.rig = atof(s);
  PTgamma->GetText(s,10);
  cst.gamma = atof(s);
  PTrepulsion->GetText(s,10);
  cst.ener_repuls = atof(s);
  PTshiftDisplay->GetText(s, 10);
  iterationShiftDisplay = atof(s);
  afficheIntermediaires = (PTCheckIntermediate->GetCheck() == BF_CHECKED);
  yasnakeParametersControl = false;
  return true;
}

void TWsnakeParametersControl::SetupWindow()
{
	STRING s;
   TDialog::SetupWindow();
   sprintf(s,"%d", cst.nb_itera);
   PTnbIterations->SetText(s);
   sprintf(s,"%d", cst.sousEcht);
   PTDiscretization->SetText(s);
   sprintf(s,"%.0f", cst.ener_gra);
   PTSpectroWeight->SetText(s);
   sprintf(s,"%.0f", cst.e_int.ela);
   PTalpha->SetText(s);
   sprintf(s,"%.0f", cst.e_int.rig );
   PTbeta->SetText(s);
   sprintf(s,"%.0f", cst.gamma);
   PTgamma->SetText(s);
   sprintf(s,"%.0f", cst.ener_repuls);
   PTrepulsion->SetText(s);
   sprintf(s,"%d", iterationShiftDisplay);
   PTshiftDisplay->SetText(s);
   PTCheckIntermediate->SetCheck((afficheIntermediaires) ? BF_CHECKED : BF_UNCHECKED);
}

void TWsnakeParametersControl::IDOk(WPARAM)
{
  TDialog::CmOk();
  yasnakeParametersControl = false;
}

void Wpiquefor::CMSnakeParametersControl()
// fonction d'activation de la fentre dialogue
{
  if (!yasnakeParametersControl) {
    snakeParametersControl = new TWsnakeParametersControl(PTWtop, "Wsnake");
    snakeParametersControl->Create();
    snakeParametersControl->ShowWindow(SW_SHOW);
  }
}

// These functions are unsed unless snakes can be launched from this control panel
int TWsnakeParametersControl::GetNumberOfIterations()
{
  STRING s;
  int nb;

  PTnbIterations->GetText(s,10);
  nb = max(atoi(s), 0);
  sprintf(s,"%d", nb);
  PTnbIterations->SetText(s);
  return(nb);
}


int TWsnakeParametersControl::GetDiscretization()
{
  STRING s;
  int step;

  PTDiscretization->GetText(s,10);
  step = min(max(atoi(s), 1), 4);
  sprintf(s,"%d", step);
  PTDiscretization->SetText(s);
  return(step);
}


float TWsnakeParametersControl::GetAlpha()
{
  STRING s;
  float alpha;

  PTalpha->GetText(s,10);
  alpha = atof(s);
  sprintf(s,"%.0f", alpha);
  PTalpha->SetText(s);
  return(alpha);
}

float TWsnakeParametersControl::GetBeta()
{
  STRING s;
  float Beta;

  PTbeta->GetText(s,10);
  Beta = atof(s);
  sprintf(s,"%.0f", Beta);
  PTbeta->SetText(s);
  return(Beta);
}

float TWsnakeParametersControl::GetGamma()
{
  STRING s;
  float Gamma;

  PTgamma->GetText(s,10);
  Gamma = atof(s);
  sprintf(s,"%.0f", Gamma);
  PTgamma->SetText(s);
  return(Gamma);
}

float TWsnakeParametersControl::GetRepulsion()
{
  STRING s;
  float repulsion;

  PTrepulsion->GetText(s,10);
  repulsion = atof(s);
  sprintf(s,"%.0f", repulsion);
  PTrepulsion->SetText(s);
  return(repulsion);
}

float TWsnakeParametersControl::GetSpectroWeigth()
{
  STRING s;
  float weigth;

  PTSpectroWeight->GetText(s,10);
  weigth = atof(s);
  sprintf(s,"%.0f", weigth);
  PTSpectroWeight->SetText(s);
  return(weigth);
}
