/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include "common.h"
#include "pics.h"
#include "spectro.h"
#include "pitchm.h"
#include "opt_for.h"
#include "SuiviSnake/snake.h"
#include "SuiviSnake/calculim.h"
#include "guiklat.h"
#include <time.h>

#define M 6			/* Nombre de formants a suivre */
#define DIM_X 512
#define DIM_Y 256

extern int ordre_lpc_msTOecht(float ordre_ms);

extern Listyl fpattern[NBMAJEURS];
extern Wpiquefor *wpiquefor;  // formant editor window
extern int paramActif;   // active parameter (which is has been read)
extern int depFor;     // time shift of formant trajectory sampling in ms
FILE *logrepuls;

unsigned int largeurIm;		/* Dimensions de l'image de gradient. */
unsigned int hauteurIm = 256; // ATTENTION ESSAI
int deplaceSnake = 0;		/* nombre d'echantillons entre 2 colonnes de l'image gradient */
int fenSnake = 0;		/* nombre d'echantillons d'un spectre de l'image gradient */
float IMA[DIM_X][DIM_Y];		/* same as the original gradient image but in float and without SILENCE threshold. */
int IMA_INT[DIM_X][DIM_Y];
int IMA_GRA[DIM_X][DIM_Y];   /* Gradient image (spectrogram) before probability influence */
int IMA_GRAM[DIM_X][DIM_Y];  /* Gradient image incorporating the probability influence */

int masque = 5;

bool interactionRepulsive = true; /* Interaction entre snakes: mode rpulsif -> les formants se repoussent quand ils s'approchent
							      mode domaineExclusif -> chaque formant a un domaine exclusif qu'il
							      qu'il modifie en se dformant. */

static void initialiserSnake(type_snake *snake),
            PPtoSnakeBetweenDates(Item *liste, type_snake *snak, int dateDeb, int dateFin),
            creer3SnakesLPC(type_snake snake[M], int premEcht, int derEcht),
            creer3SnakesTrueenv(type_snake snake[M], int premEcht, int derEcht),
            creer3Snakes(type_snake snake[M], int premEcht, int derEcht, TypeInitialSnakes typeInitialSnakes),
            snakeToPP(type_snake *snake, int formant),
            calculim(),		/* calcul de l'image par lissage cepstral */
            initImage(int *image[DIM_Y], int largeur, int hauteur),
            displaySnake(type_snake *snake, int couleur);

/*
 * Modify the influence of the spectrogram image according 
 * to the formant to frequency affiliation proability.
 */
static void modifyImage(
  int dest[DIM_X][DIM_Y], 
  int src[DIM_X][DIM_Y], 
  double **paff, 
  type_snake *snake)
{
  int i,	/* column index in the image */
      ip;	/* point index in the snake */ 
  double *x_new = snake->eq.x_new;

  for (i = (int)x_new[0], ip = 0; i < (int)x_new[snake->eq.discret - 1]; i++) {
    if (((float) i) <= x_new[0]) { /* This is the case at the first point. */
      int j;
      if (ip != 0) 
        affiche_erreur("This should be the very first point");
      for (j = 0; j < DIM_Y; j++) 
	     dest[i][j] = src[i][j] * paff[ip][j];
    }
    else if (dans((float)i, x_new[ip], x_new[ip + 1])) {
      int j;
      float r = (((float) i) - x_new[ip]) / (x_new[ip + 1] - x_new[ip]);
      for (j = 0; j < DIM_Y; j++) 
	     dest[i][j] = src[i][j] * (paff[ip][j] + r * (paff[ip+1][j] - paff[ip][j]));
    }
    else if (i > x_new[ip + 1]) {
      for (;i > x_new[ip + 1];ip++);
      if (ip >= snake->eq.discret)
        affiche_erreur("Out of the snake");
      else {
	     int j;
        float r = (((float) i) - x_new[ip]) / (x_new[ip + 1] - x_new[ip]);
        for (j = 0; j < DIM_Y; j++)
          dest[i][j] = src[i][j] * (paff[ip][j] + r * (paff[ip+1][j] - paff[ip][j]));
      }
    }
  }
}


// #define PROBA1
/* #define SANSDONSUN */
#define EXCLUSIONMUTUELLE
/*
 * Suivi simultane des snakes.
 */
void suivreFormantsSimultane(
  long int premEcht,
  long int derEcht,
  bool fromScratch,
  bool onlySelected,  // only for selected polypoints (and thus only when fromScratch is false.
  TypeInitialSnakes typeInitialSnakes)
{
  type_snake snake[M];
  int start, end;
  int m, iter, nbPoints;
  int mi, dim, mj, alreadyDiscretized = 0;
  float var[M], shiftHz = en_cours.frequence / (2. * hauteurIm);
  double **paff[M];			// affiliations of pixels to formants
  bool selected[M];     	// true if the formant is selected

  // hauteur maximale en nombre de pixels jusqu' laquelle on considre l'image.
  // Aller plus loin risque de donner des gaussiennes trop plates pour le ou les derniers formants.
  int hauteurMax = freqToEcr(5000.) * 2.;

  // logrepuls=fopen("logrepuls","w"); Caution to test repulsion
  for (m = 0; m < M; m++)
    snake[m].snake_init = (type_coord_float *) NULL;

  /* Si les formants existent dj on rcupre ces courbes sinon on cre
     les courbes et qu'aucune region n'est selectionnee. */
  if (fromScratch) {
    if (abs(ct.debRegion) > INFINI_I / 2 || ct.debRegion == ct.finRegion)
       creer3Snakes(snake, start = max(ct.d_a, ct.d_e) + enEcht(50) , end = min(ct.f_a, ct.f_e) - enEcht(50), typeInitialSnakes );
    else creer3Snakes(snake, start = premEcht, end = derEcht, typeInitialSnakes);
  }
  else {
    int startPoints[M], endPoints[M], dateDeb, dateFin, debRegion, finRegion;

    if (abs(ct.debRegion) > INFINI_I / 2 || ct.debRegion == ct.finRegion) {
      debRegion = enMilli(max(ct.d_a, ct.d_e));
      finRegion = enMilli(min(ct.f_a, ct.f_e));
      for (int i = 0; i < M; i++) {
        startPoints[i] = enMilli(max(ct.d_a, ct.d_e));
        endPoints[i] = enMilli(min(ct.f_a, ct.f_e));
      }
    }
    else {
      for (int i = 0; i < M; i++) {
        startPoints[i] = enMilli(premEcht);
        endPoints[i] = enMilli(derEcht);
      }
    }

    if (onlySelected) {   // The region where formants selected are within startPoints and endPoints is seached for.
      int dateDeb, dateFin;
      for (int i = 0; i < M; i++) {
        selected[i] = false;
        Item *q;
        for (q = fpattern[i + F1].list; q && q->val->fin() <= startPoints[i] && !q->val->estSelec(); q = q->next);
        if (q) {
	       int deb = q->val->debut();
	       startPoints[i] = (deb > startPoints[i]) ? deb : startPoints[i];
          if (startPoints[i] > enMilli(ct.finRegion)) startPoints[i] = endPoints[i] = INFINI_I;
          else {
            selected[i] = q->val->estSelec();
            endPoints[i] = min(endPoints[i], q->val->fin());
          }
        }
      }
      dateDeb = max_ti(startPoints, 0, M - 1);
      dateFin = min_ti(endPoints, 0, M - 1);
      // The intersection of selected
      for (int i = 0; i < M; i++) {
        startPoints[i] = dateDeb;
        endPoints[i] = dateFin;
      }
    }

    // Then, the time interval on which the M formants are all defined is searched for.
    for (int i = 0; i < M; i++) {
      Item *q;
      for (q = fpattern[i + F1].list; q && q->val->fin() <= startPoints[i]; )
        q = q->next;
      if (q) {
	     int deb = q->val->debut();
	     startPoints[i] = (deb > startPoints[i]) ? deb : startPoints[i];
        if (startPoints[i] > finRegion) startPoints[i] = endPoints[i] = INFINI_I;
      }
      else {  // no valid PP has been found.
        startPoints[i] = endPoints[i] = INFINI_I;
        break;
      }
      if (startPoints[i] < INFINI_I / 2)
        endPoints[i] = min(endPoints[i], q->val->fin());
    }
    dateDeb = max_ti(startPoints, 0, M - 1);
    dateFin = min_ti(endPoints, 0, M - 1);

    if (dateDeb >= dateFin) creer3Snakes(snake, start = max(ct.d_a, ct.d_e) + enEcht(50),
    											     end = min(ct.f_a, ct.f_e) - enEcht(50), typeInitialSnakes);
    else {
      start = dateDeb;
      end = dateFin;
        for (int i = 0; i < M; i++)
        PPtoSnakeBetweenDates(fpattern[i + F1].list, &snake[i], dateDeb, dateFin);
      alreadyDiscretized = 1;
    }
  }

  if (start >= end) return;
  // int IMA_PROB[M][DIM_X][DIM_Y];  Borland compiler fails on this line. Transformed into the following code.

  // Remove the display of the highlighted region
  ct.debRegion = INFINI_L;
  ct.finRegion = -INFINI_L;
  ::InvalidateRect(wspectro->wsona->HWindow, NULL, true);
  wspectro->wsona->UpdateWindow();
  ::InvalidateRect(wsignal->wsig->HWindow, NULL, true);
  wsignal->wsig->UpdateWindow();

  for (m = 0; m < M; m++) {
    initialiserSnake(&snake[m]);
    if (alreadyDiscretized) snake[m].cst.sousEcht = 1;
    snake[m].image.dim_x = (int) largeurIm;
    snake[m].image.dim_y = (int) hauteurIm;
    initialiseCalcul(&snake[m]);
  }

  int ***IMA_PROB;
  IMA_PROB = (int ***) malloc(M * sizeof(int **));
  for (int i = 0; i < M; i++) {
    IMA_PROB[i] = (int **) malloc(DIM_X * sizeof(int *));
    for (int j = 0; j < DIM_X; j++)
      IMA_PROB[i][j] = (int *) malloc(DIM_Y * sizeof(int));
  }

  initImage(IMA_PROB[0], DIM_X, DIM_Y);
  initImage(IMA_PROB[1], DIM_X, DIM_Y);
  initImage(IMA_PROB[2], DIM_X, DIM_Y);

  nbPoints = snake[0].snake.nb_pts;
  for (m = 0; m < M; m++) {
    int i;
    /*  allocations des pointeurs de lignes */
    paff[m] = (double **) calloc((size_t) nbPoints, sizeof(double *));
    /*  allocations  des lignes */
    for (i = 0; i < nbPoints; i++) paff[m][i] = (double *) calloc(DIM_Y, sizeof(double));
    //    var[m] = SQR(200. + m * 50.);
    var[m] = SQR(200.);
  }


  for (iter = 0 ; iter < snake[0].cst.nb_itera; iter++) {
    int i;
    // fprintf(logrepuls,"----Itration %d\n",iter); Caution to test repulsion

    /* First, we compute affiliation of formant to frequency. */
    for (i = 0; i < nbPoints; i++) {
      int m;
      for (m = 0; m < M ; m++) if (!onlySelected || !iter || selected[m]) {
	     int j;
	     float yip1, yim1;
	     if (!m) {
	       yim1 = 200.;
	       yip1 = snake[m + 1].eq.y_new[i] * shiftHz + 200.;
	     }
	     else if (m == M - 1) {
	       yim1 = snake[m - 1].eq.y_new[i] * shiftHz + 200.;
	       yip1 = 2 * (hauteurMax * shiftHz) - (snake[m - 1].eq.y_new[i] * shiftHz + 200.);
	     }
	     else {
	       yim1 = snake[m - 1].eq.y_new[i] * shiftHz + 200.;
	       yip1 = snake[m + 1].eq.y_new[i] * shiftHz - 200.;
	     }
	     for (j = 0; j < hauteurMax; j++) {
	       float jHz = j  * shiftHz;
	       if (jHz <= yim1 || jHz >= yip1)
	         paff[m][i][j] = 0;
	       else {
            double xxx = fabs(4 * (jHz-yim1)*(jHz-yip1)/((yip1  - yim1)*(yip1 - yim1)));
            if (xxx < EPSILON)
            	affiche_erreur("Stupid repulsion calculation");
            paff[m][i][j] = exp(0.2*log(xxx + EPSILON));
          }
	       if (iter == 50) IMA_PROB[m][i][j] =  IROUND(paff[m][i][j] * 255);
        }
      }
    }
    if (iter == 50) {
      int i;
      for (i = 0; i < M; i++) {
        STRING simage;
#ifdef ARETABLIR
        sprintf(simage, "imageProba%1d.pgm", i);
        savePgm(IMA_PROB[i], largeurIm, hauteurIm, simage);
#endif
      }
    }

    if (afficheIntermediaires) {
      wspectro->wsona->spectro_nettoyer(0,INFINI_I);
    }

    for (m = 0; m < M ; m++) if (!onlySelected || selected[m]) {
      /* For each formant  the influence of the probability is incorporated. */
      modifyImage(IMA_GRAM, IMA_GRA, paff[m], &snake[m]);
      /* Then the formant is deformed. */
      uneIterationSnake(&snake[m], (m) ? &snake[m - 1] : (type_snake *) NULL, (m < M - 1) ? &snake[m + 1] : (type_snake *) NULL);
    }
    if (afficheIntermediaires && !(iter % iterationShiftDisplay)) {
	     /* STRING s;
	     sprintf(s,"im_%d_%d.pgm",m,iter);
	     savePgm(IMA_GRAM, largeurIm, hauteurIm, s);  */
        wspectro->wsona->SetPenWidth(2);
        for (int i = 0; i < M; i++) displaySnake(&snake[i], (!i) ? ROUGE : (!(i - 1) ? MAGENTA : (!(i - 2) ? BLEU : VERT)));
        ::InvalidateRect(wspectro->wsona->HWindow, NULL, true);
        wspectro->wsona->UpdateWindow();
    }
  }


  for (m = 0; m < M; m++) {
    int i;
    for (i = 0; i < nbPoints; i++) free((char *)paff[m][i]);
    free((char *)paff[m]);
  }
  // fclose(logrepuls); caution to test repulsion

  for (int i = 0; i < M; i++) {
    for (int j = 0; j < DIM_X; j++) free((char *) IMA_PROB[i][j]);
    free((char *) IMA_PROB[i]);
  }
  free((char *) IMA_PROB);


  for (m = 0; m < M; m++) {
    sauvegarde(&snake[m]);
    snakeToPP(&snake[m], m);
    videSnake(&snake[m]);
  }
}

static void displaySnake(
  type_snake *snake,
  int couleur)
{
  HDC hDC;
  hDC = GetDC(wspectro->wsona->HWindow);
  int xold = wspectro->wsona->LXtoDX((float) snake->snake.snake_pts[0].x * deplaceSnake + ct.d_a + fenSnake / 2),
      yold = wspectro->wsona->LYtoDY(snake->snake.snake_pts[0].y / 2.);

  MgiHue(couleur, wspectro->wsona);
  SetROP2(wspectro->wsona->hDCcomp, R2_COPYPEN);
  MoveToEx(wspectro->wsona->hDCcomp, xold, yold, (POINT *) NULL);
  for (int i = 1; i < snake->eq.discret; i++, i++) {
    int x = wspectro->wsona->LXtoDX((float) snake->eq.x_new[i] * deplaceSnake + ct.d_a + fenSnake / 2),
        y = wspectro->wsona->LYtoDY(snake->eq.y_new[i] / 2.);
//    PointSel(x, y);
    LineTo(wspectro->wsona->hDCcomp, x, y);
    xold = x;
    yold = y;
  }

  // modified by ghazi
  ReleaseDC(wspectro->wsona->HWindow,hDC);
}



static void snakeToPP(
  type_snake *snake,
  int formant)
{
  int i, nbPoints;

  wpiquefor->get_ppp()->newsize(0);

#ifdef ESSAI
  for (int i = 0; i < snake->nbPointsInit; i++, nbPoints++) {
    int date = enMilli(snake->snake_init[i].x * deplaceSnake + ct.d_a + fenSnake / 2);
    float y = canToHz(snake->snake_init[i].y / 2.);
    wpiquefor->get_ppp()->push(Point2P(date, y));
  }
#else
  for (i = 0; i < snake->snake.nb_pts; i++, i++) {
    int date = enMilli(snake->snake.snake_pts[i].x * deplaceSnake + ct.d_a + fenSnake / 2);
    float y = canToHz(snake->snake.snake_pts[i].y / 2.);
    wpiquefor->get_ppp()->push(Point2P(date, y));
  }
#endif

  ::paramActif = formant + F1;
  ::typeScale = SCALEFr;
  wpiquefor->get_ppp()->echantillonnePP(depFor);

  wpiquefor->validatePPP();
  wpiquefor->chScaleKlatt();
}


/* 
 * Transforms points of a polypoints between two instant times into a snake.
 */
static void PPtoSnakeBetweenDates(
  Item *liste,
  type_snake *snak,
  int dateDeb,
  int dateFin)
{
  int i, n;
  Item *q;

  /* Allocation de l'espace des snakes de depart. 
   * en echantillonnant tous les points (pixels)
   */
  n = DIM_X;
  snak->snake_init = (type_coord_float *) calloc(n, sizeof(type_coord_float));

  // search for the first pp that may give points between dateDeb and dateFin.
  for (q = liste; q && (q->val->fin() <= dateDeb); q = q->next);
  for (i = 0; q && (q->val->debut() <= dateFin); q = q->next) {
    int j;
    // jump the first points which are before dateDeb
    for (j = 0; j < q->val->nbPt && q->val->date(j) < dateDeb; j++);
    for (; j < q->val->nbPt && q->val->date(j) < dateFin; j++) {
      float x, y;
      if (i >= n) {
        n+= 100;
        snak->snake_init = (type_coord_float *) realloc(snak->snake_init, n * sizeof(type_coord_float));
      }
      x = (float) (enEcht(q->val->date(j)) - fenSnake / 2 - ct.d_a) / deplaceSnake;
      y = 2 * freqToEcr(q->val->y(j,FREQUENCE));
      if (!i || ((int) x) != ((int) snak->snake_init[i - 1].x)) {
        snak->snake_init[i].x = (float) (enEcht(q->val->date(j)) - fenSnake / 2 - ct.d_a) / deplaceSnake;
        snak->snake_init[i].y = y;
        i++;
      }
    }
  }
  snak->nbPointsInit = i;
  snak->snake_init = (type_coord_float *) realloc(snak->snake_init, snak->nbPointsInit * sizeof(type_coord_float));
}


static float domFormantsDefault[6][2] = {{200., 1000.}, {500., 2500.}, {1500., 3600.}, {2500., 4500.}, {3000., 8000.}, {3000., 8000.}};
static float domFormantsBackVowel[6][2] = {{200., 1000.}, {400., 1500.}, {1500., 3000.}, {2500., 4500.}, {3000., 8000.}, {3000., 8000.}};
static float (*domFormants)[2] = domFormantsDefault;
/*
 * Checks whether roots up to the iroot th root are visible.
 */
static bool lpRootVisible(
  racines1_lpc& r,
  int iroot)
{
  int i;
  bool visible;
  for (i = 0, visible = true; i <= iroot; i++) {
    double bw = r[i].lar,
           energyLevel = r[i].db,
           frequence = r[i].fr;

    visible = (visible && bw < 450. && energyLevel > SILENCE_SPECTRO && frequence < 5000.);
  }
  return(visible);
}

/*
 * Moyenne mobile des racines de LPC.
 */
static float moyenneRacine(
  int premEcht,
  int m,  /* numero de la racine a prendre */
  int l,  /* nombre de points de la fenetre de moyenne */
  int n,  /* indice du point a modifier */
  int nn) /* nombre de valeurs calcules */
{
  int i;
  float s, si;

  for (i = - l / 2, s = 0., si = 0.; i <= l / 2; i++)
    if (dans(n + i, 0, nn -1) && m < (*les_raclpc)[n + i].size()
	     && pm.getReadf0(premEcht + (n + i) * deplaceSnake) > 0
	     && dans((*les_raclpc)[n + i][m].fr, domFormants[m][0], domFormants[m][1])
	     && lpRootVisible((*les_raclpc)[n + i], m))
        {
          float r = (1. + cos(2.* _PI / l)) / 2.;
          s = s + (*les_raclpc)[n + i][m].fr * r;
          si += r;
        }
  return((si > EPSILON) ? s / si : 0.);
}

extern float backVowel(float *, int, double);
/*
 * Creation de 3 snakes initiaux par moyenne mobile.
 */
static void creer3Snakes(
  type_snake snake[M],
  int premEcht,
  int derEcht,
  TypeInitialSnakes typeInitialSnakes)
{
  if (typeInitialSnakes == InitialCurvesLPC) creer3SnakesLPC(snake, premEcht, derEcht);
  else if (typeInitialSnakes == InitialCurvesTrueenv) creer3SnakesTrueenv(snake, premEcht, derEcht);
}

static void creer3SnakesLPC(
  type_snake snake[M],
  int premEcht,
  int derEcht)
{
  int l, n = -INFINI_L;

  if (premEcht >= derEcht) return;
  /* width of the running average window */
  l = max(1, enEcht(250) / deplaceSnake);
  if (!(l % 2)) l = l + 1;	/* Pour avoir une fenetre de taille impaire */

  suivreRacLpc(space, premEcht, derEcht, ordre_lpc_msTOecht(opt_for.or_lpc), 700., 0., deplaceSnake, n);
  /* Allocation de l'espace des snakes de depart.
   * en echantillonnant tous les points (pixels)
   */
  for (int m = 0; m < M; m++) {
    snake[m].nbPointsInit = n;
    snake[m].snake_init = (type_coord_float *) calloc(n, sizeof(type_coord_float));
  }
  for (int i = 0; i < n; i++) {
    float x = backVowel(IMA[IROUND((premEcht + i * deplaceSnake - ct.d_a) / deplaceSnake)], DIM_Y, en_cours.frequence);
    bool isABackVowel = (x > 0.8);
    if (isABackVowel) domFormants = domFormantsBackVowel;
    else domFormants = domFormantsDefault;

    for (int m = 0; m < M; m++) {
      float y;
      snake[m].snake_init[i].x = (float) (premEcht + i * deplaceSnake - ct.d_a) / deplaceSnake;
      y = moyenneRacine(premEcht, m, l, i, n);
//      y = moyenneRacine(premEcht, m, 1, i, n);
      if (y < EPSILON) {
         if (isABackVowel) y = (m < 2) ? freqToEcr(350. + m * 550.)*2. : freqToEcr(2500. + (m - 2) * 1000.)*2.;
         else y = freqToEcr(500. + m * 1000.)*2.;
         if (m && y < snake[m - 1].snake_init[i].y) y = snake[m - 1].snake_init[i].y + 2;
      }
      else y = freqToEcr(y) * 2.;
      snake[m].snake_init[i].y = y;
    }
  }
}


/*
 * Moyenne mobile des pics de cepstre itratif.
 */
static float moyennePicCepstreIteratif(
  int premEcht,
  int m,  /* numero du pic a prendre */
  int l,  /* nombre de points de la fenetre de moyenne */
  int n,  /* indice du point a modifier */
  int nn) /* nombre de valeurs calcules */
{
  int i;
  float s, si;
               
  for (i = - l / 2, s = 0., si = 0.; i <= l / 2; i++) 
    if (dans(n + i, 0, nn -1) && m < (*pics_cepstre)[n + i].size()
	     && pm.getReadf0(premEcht + (n + i) * deplaceSnake) > 0
	     && dans((*pics_cepstre)[n + i][m].frHz, domFormants[m][0], domFormants[m][1])
	     && (*pics_cepstre)[n + i][m].q > 0.5) {
      float r = (1. + cos(2.* _PI / l)) / 2.,
            freq = (*pics_cepstre)[n + i][m].frHz;
      s = s + freq * r;
      si += r;
    }
  return((si > EPSILON) ? s / si : 0.); 
}

/*
 * Creation de 3 snakes initiaux par moyenne mobile en prenant les pics de Halle.
 */
void creer3SnakesTrueenv(
  type_snake snake[M],
  int premEcht,
  int derEcht)
{
  int m, i, l, npics, n = INFINI_L;
  float fmax = min(5000., en_cours.frequence / 2.); // on prend les pics jusqu' 5000 Hz.

  /* largeur de la fenetre de moyenne mobile */
  l = max(1, enEcht(250) / deplaceSnake);
  if (!(l % 2)) l = l + 1;	/* Pour avoir une fenetre de taille impaire  */

  npics = fmax / 5000. * 8;

  suivrePicsCepstre(space, premEcht, derEcht, 7, 216, (int)enEchtf(opt_for.nb_cep[0]), (int)enEchtf(opt_for.nb_cep[1]),
		   				 0., deplaceSnake, n, true);

  /* Allocation de l'espace des snakes de depart.
   * en echantillonnant tous les points (pixels) 
   */
  for (int m = 0; m < M; m++) {
    snake[m].nbPointsInit = n;
    snake[m].snake_init = (type_coord_float *) calloc(n, sizeof(type_coord_float));
  }


  for (i = 0; i < n; i++) {
    float x = backVowel(IMA[IROUND((premEcht + i * deplaceSnake - ct.d_a) / deplaceSnake)], DIM_Y, en_cours.frequence);
    bool isABackVowel = (x > 0.8);
    if (isABackVowel) domFormants = domFormantsBackVowel;
    else domFormants = domFormantsDefault;

    for (m = 0; m < M; m++) {
      float y;
      snake[m].snake_init[i].x = (float) (premEcht + i * deplaceSnake - ct.d_a) / deplaceSnake;
      y = moyennePicCepstreIteratif(premEcht, m, l, i, n);
//      y = moyennePicCepstreIteratif(premEcht, m, 1, i, n);
      if (y < EPSILON) {
         if (isABackVowel) y = (m < 2) ? freqToEcr(350. + m * 550.)*2. : freqToEcr(2500. + (m - 2) * 1000.)*2.;
         else y = freqToEcr(500. + m * 1000.)*2.;
         if (m && y < snake[m - 1].snake_init[i].y) y = snake[m - 1].snake_init[i].y + 2;
      }
      else y = freqToEcr(y) * 2.;
      snake[m].snake_init[i].y = y;
    }
  }
}



static void initEquationSnake(
  snaEq& eq)
{
  eq.discret = eq.itera = 0;
  eq.pas = 0.;
  eq.a = eq.a1 = eq.droite = eq.bas = (double **) NULL;
  eq.b = eq.alpha = eq.beta = eq.x_old = eq.y_old = eq.x_new = eq.y_new = (double *) NULL;
}

static void initialiserSnake(
  type_snake *snake)
{
  int i;
 
  snake->cst = cst; /* Recopie des constantes sur lesquelles
			    l'utilisateur a le controle. */
  snake->image.dim_x      = 256;
  snake->image.dim_y      = 256;
  strcpy(snake->image.nom_int, "");
  strcpy(snake->image.nom_gra, "");
  snake->cst.ener_int   = SNAKE_IMA_ENER_INT;
  snake->cst.ener_poi   = SNAKE_IMA_ENER_POI;
  snake->cst.e_int.poi        = SNAKE_E_INT_POI;
  snake->snake.snake_pts  = (type_coord_float *) NULL;
  snake->snake.nb_pts     = 0;
  snake->snake.snake_sauv = (type_coord_float *) NULL;
  snake->snake.nb_sauv    = 0;
  snake->cst.autom      = 1;
  initEquationSnake(snake->eq);

#ifdef UIY
  col_snake=100;
  col_snake_sau =200;
  col_courbe = 155;
  strcpy(SIMPLIFIE," ");
  strcpy(SUFFIXE, "");
#endif
}

static void initImage(
  int *image[DIM_Y],
  int largeur,
  int hauteur)
{
  int i,j;
  for (i = hauteurIm - 1; i >= 0; i--)
    for (j = 0; j < largeurIm; j++)
      image[j][i] = 0;
}

