/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */
// fichier affichage.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>

#include "signal.h"
#include "outils.h"
#include "axes.h"

void WScale::affichage(
     long premEchtFen,	  /* premier chantillon de la fentre */
     long derEchtFen)	  /* dernier chantillon de la fentre */
{
  clear_fenetre();
  setLXcoor((float) premEchtFen, (float) derEchtFen);
  setLYcoor(0., 1.);
  axe_x(premEchtFen, derEchtFen, enMillif(premEchtFen), enMillif(derEchtFen),
  		  1., NOIR, 0.0,"%-5.0f", 12, 5, this);
  ::InvalidateRect(HWindow, NULL, true);
}
   //
// Affichage du signal temporel 
//	*---------------*---------------*------------------*
//      ^	      	^		^	           ^
// 	!	      	!		!		   !
//	premEchtFen	premEcht        derEcht		   derEchtFen
//
//	[premEchtFen .. derEchtFen] est le domaine d'affichage
//                                   de la fenetre.
//	[premEcht .. derEcht] est le signal temporel a afficher.
//
// note: affichage() initialise les coordonnes de
//       la fenetre spectro et doit pour cela tre
//       appel avant tout affichage dans la fenetre
//       spectro lorsque ces coordonnes changent

float Wdonnees_signal::affichage(
     SAMPLEbuf& cnv,	  /* signal temporel 			    */
     long premEcht,	  /* echantillon de debut d'affichage 	    */
     long derEcht,	  /* echantillon de fin   d'affichage  	    */
     long premEchtFen,	  /* premier chantillon de la fentre */
     long derEchtFen,	  /* dernier chantillon de la fentre */
     int , //valIncrement,	  /* 1 point sur increment	 	    */
     bool normalize)
{
  int premx,derx; // premier et dernier pixel (en x)
  INTbuf pDX(1,"pDX in affichage()"); // point: x en pixel
  INTbuf pLY(1,"pLY in affichage()"); // point: y logique
  int pn; // nombre de points  afficher
  long j,jinf,jsup,jmin,jmax;
  float v, vmin, vmax;
  int i;

  Wact->wr(ml(2101,"draw sound"));
  clear_fenetre();
  setLXcoor(ct.d_a,ct.f_a);
  wspectro->wsona->setLXcoor(ct.d_a,ct.f_a);
  wspectro->wsona->setLYcoor(0.,NB_CANAUX);

  premx = LXtoDX(premEcht);
  derx = LXtoDX(derEcht);

  if (this->largeur > 4 * (derEcht - premEcht)) {  // il vaut mieux afficher les chantillons car il y a la place
    int echt;

    vmax = -INFINI_F;
    if (normalize) {
      vmax = 0;
      for (echt = _i(premEcht); echt < _i(derEcht); echt++)
        vmax = max(vmax, abs(cnv[echt]));
    }
    vmax = max(vmax, 1);
    setLYcoor(-vmax*1.1,vmax*1.1);

    MgiHue(ROUGE, this);

    for (echt = premEcht; echt <= derEcht; echt++) {
      int xi = LXtoDX(echt), yi = LYtoDY(cnv[_i(echt)]);
      snoibox(xi - 1, yi - 1, xi + 1, yi + 1, this);
    }
  }
  else {
    // 1re phase: prparer les points  afficher
    { // plusieurs chantillons possibles par pixel
      // prendre les min et max de ces chantillons
      int pnmax = 2 *(derx - premx);
      pDX.newsize(pnmax);
      pLY.newsize(pnmax);
      for (i = 0,pn = 0; i < pnmax / 2; i++) {
        jinf = _i(max(premEcht, min(derEcht, (long)DXtoLX(premx + i) + 0.5)));
        jsup = _i(max(premEcht, min(derEcht, (long)DXtoLX(premx + i + 1) + 0.5 - 1)));
        if (jinf <= jsup) {
          // cas normal
          // au moins un chantillon pour ce pixel
          for (j = jinf+1,jmin = jinf,jmax = jinf,vmin = vmax=cnv[jinf]; j<=jsup; j++) {
            v=cnv[j];
            if (v>vmax) { vmax=v; jmax=j; }
            if (v<vmin) { vmin=v; jmin=j; }
          }
		    pDX[pn] = premx+i;
		    pLY[pn] = (jmin < jmax) ? vmin : vmax;
		    pn++;
          if (jsup > jinf) {
			   // 2 chantillons ou plus pour ce pixel
	  		   pDX[pn] = premx+i;
	  		   pLY[pn] = (jmin < jmax) ? vmax : vmin;
	  		   pn++;
          }
        }
      }
    }

    // donner les ordonnes  la fentre signal
    vmax = 0x7fff;
    if (normalize) {
      vmax=0;
      for (i=0;i<pn;i++)
        vmax = max(vmax, abs(pLY[i]));
    }
    vmax = max(vmax, 1);
    setLYcoor(-vmax*1.1,vmax*1.1);

    // trac effectif
    MgiHue(ROUGE, this);
    for (i=1;i<pn;i++)
      Dline2(pDX[i-1],LYtoDY(pLY[i-1]),pDX[i],LYtoDY(pLY[i]));
  }

  Wact->wr("");
  ::InvalidateRect(HWindow, NULL, true);
  return(vmax);
}


