/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier affuneco.cpp
//

#include "common.h"
#pragma hdrstop                                                                  
                                                                                
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "signal.h"
#include "fftlpc.h"
#include "fft_mayer.h"
#include "spectre.h"
#include "pics.h"
#include "fmaxspec.h"
#include "criticalbands.h"

#include "wcoupe.h"
#include "outils.h"
#include "axes.h"

static float largband = 1000.; // largeur de bande
                               // maximale pour
                               // afficher un pole

//                                                                                                                      
// Controle l'ordre de la transformee de Fourier par rapport
// a la longueur de la fenetre d'analyse.
void controleOrdreFFT(
   float &fenSignal,
   int &ordreFFT)
{
  // Firstly, we try to set the FFT order (expressed in the form
  // of log2) greater than the length of the window length.
  while (   enEcht(fenSignal) > (1 << ordreFFT)
         && (1 << ordreFFT) < TAILLE_MAX_HAMMING)
    ordreFFT++;
  // If this is not sufficient we set the window length to the
  // FFT order.
  if (enEcht(fenSignal) > (1 << ordreFFT))
    fenSignal = enMilli(1 << ordreFFT);
}

//
// Affichage de spectres de parole obtenus par plusieurs methodes.
//

void spectreFft(
   float spectre[],
	SAMPLEbuf& parole,
   long parole_debut,
	int preaccentuation,
	int ordre,
	int fen_signal)
{
  FLOATbuf xfft_(ordre + 1, "xfft");
  FLOATbuf yfft_(ordre + 1, "yfft");
  double l2;
  int i;
  float *xfft=(float *)&xfft_[0];
  float *yfft=(float *)&yfft_[0];

  for(i = 0; i <ordre+1; i++) {
  	xfft_[i]=0;
   yfft_[i]=0;
  }

  initham(NULL, fen_signal);
  if (preaccentuation)
  { preac_f(parole, parole_debut-1, xfft_, fen_signal);
    ham_ff(xfft_, xfft_, NULL, fen_signal);
  }
  else ham_sf(parole, parole_debut, xfft_, NULL, fen_signal);
  l2 = log2((double) ordre);
  initFftGen(ordre, IROUND(l2), 1, 1.);
  fftGen(xfft_,yfft_
	 , ordre, IROUND(log2((double) ordre)));
  for(i = 1; i <= ordre / 2; i++)
  { float xx;
    xx = (xfft[i])*(xfft[i]) + (yfft[i])*(yfft[i]);
    spectre[i - 1] = 10*log10(max(xx,0.000001));
  }
  return;
}


static float **criticalBands = (float **) NULL;
static int *centerFrequencies;
static int nbBands_ = 0;
static double samplingRate_ = 0.;
static bool completionStrategy_ = false;
static FrequencyScale frequencyScale_ = Mel;
static MelStrategy melWidthStrategy_ = VariableMelWidth;
static int ordrecb_ = 0;
static FLOATbuf X(4096+1,"X near spectreCriticalBands()");

void spectreCriticalBands(
   float spectre[],
	SAMPLEbuf& parole,
   long parole_debut,
	int preaccentuation,
	int ordre,
	int fen_signal,
   int nbBands,
   double samplingRate,
   bool completionStrategy,
   FrequencyScale frequencyScale,         // perceptive frequency scale
   MelStrategy melWidthStrategy,
   bool withCepstralSmoothing,
   bool withDctIdctLiftering,
   int nbLiftering,
   int nbFreqSamplesDisplayed)
{
  FLOATbuf xfft_(ordre + 1, "xfft");
  FLOATbuf yfft_(ordre + 1, "yfft");
  float *spectreTemp = new float [ordre/2];
  double l2;
  int i, cMaxSpectro;
  float *xfft=(float *)&xfft_[0];
  float *yfft=(float *)&yfft_[0];
  double (*hertz2percep)(double);

  for(i = 0; i <ordre+1; i++) {
  	xfft_[i]=0;
   yfft_[i]=0;
  }

  cMaxSpectro = fMaxSpectro / samplingRate * ordre;

  hertz2percep = (frequencyScale == Mel) ? hertz2mel : ((frequencyScale == Bark) ? hertz2bark : hertz2mely);

  initham(NULL, fen_signal);
  if (   nbBands_ != nbBands || samplingRate_ != samplingRate || frequencyScale_ != frequencyScale
      || ordrecb_ != ordre || completionStrategy_ != completionStrategy
      || melWidthStrategy_ != melWidthStrategy) {
     if (criticalBands) {
       freeCriticalBandFilters(criticalBands, nbBands_);
       delete [] centerFrequencies;
     }
     criticalBands = buildCriticalBandFilters(samplingRate, nbBands, ordre, &centerFrequencies, completionStrategy,
     														 frequencyScale, melWidthStrategy, false);
     ordrecb_ = ordre;
     nbBands_ = nbBands;
     samplingRate_ = samplingRate;
     frequencyScale_ = frequencyScale;
     melWidthStrategy_ = melWidthStrategy;
     completionStrategy_ = completionStrategy;
  }

  if (withCepstralSmoothing) {
    FLOATbuf *PX = &X;
    cepstre(&PX, &parole[0], parole_debut, 1, 0, fen_signal,
            ordre, ordre,
            enEchtf(PTWcoupe->pLisCep1->debut_liftrage), enEchtf(PTWcoupe->pLisCep1->fin_liftrage), 1, preaccentuation, false);
    for(i = 0; i < ordre / 2; i++) spectreTemp[i] = X[i];
  }
  else {
    if (preaccentuation) {
      preac_f(parole, parole_debut-1, xfft_, fen_signal);
      ham_ff(xfft_, xfft_, NULL, fen_signal);
    }
    else ham_sf(parole, parole_debut, xfft_, NULL, fen_signal);
    l2 = log2((double) ordre);
    initFftGen(ordre, IROUND(l2), 1, 1.);
    fftGen(xfft_,yfft_, ordre, IROUND(log2((double) ordre)));
    for(i = 1; i <= ordre / 2; i++) {
     float xx;
      xx = (xfft[i])*(xfft[i]) + (yfft[i])*(yfft[i]);
      spectreTemp[i - 1] = 10*log10(max(xx,0.000001));
    }
  }
  applyCriticalBandsff(spectreTemp, spectre, ordre, cMaxSpectro, criticalBands, centerFrequencies, nbBands);
  if (withDctIdctLiftering) {
    dct(spectre, spectreTemp, nbBands);
    idct(spectreTemp, spectre, nbBands, nbLiftering, ordre / 2, completionStrategy, samplingRate, hertz2percep);
  }

//  if ((nbBands < ordre/ 2) || completionStrategy)
   if (   (nbBands < ordre/ 2 && !withDctIdctLiftering)
       || (completionStrategy && !withDctIdctLiftering))
    spectralCompletion(spectre, ordre, cMaxSpectro, centerFrequencies, nbBands, completionStrategy);
  delete [] spectreTemp;
  return;
}


static float **criticalBandsPLP = (float **) NULL;
static int *centerFrequenciesPLP;
static int nbBandsPLP_ = 0;
static double samplingRatePLP_ = 0.;
static FrequencyScale percepFrequencyScale_ = Mel;
static int fftOrderPLP_ = 0;
static FLOATbuf XLPL(4096+1,"X near spectreCriticalBands()");
#define ORDERlastPLP_FFT 512
//#define AVECFFT

void spectreCriticalBandsPLP(
   float PLPspectrum[],
	SAMPLE parole[],
   long parole_debut,
	int preaccentuation,
	int predictionOrder,
   int fftOrder,
	int fen_signal,
   int nbBands,     // nbBands must be a power of 2 because of the last FFT to compute the spectrum.
   FrequencyScale percepFrequencyScale, // perceptive frequency scale
   bool intensityConversion, // if true the intensity conversion is applied.
   double samplingRate,
   double *e,
   double coefp[])
{
  double *real = new double[fftOrder],
         *perceptive = new double[fftOrder],
         *autoc = new double[nbBands + 1],
         averageLevel = 0.,  // is used to compensate for the intensity
                             // conversion which decreases the energy level
                             // due to the .33 power.
         averagePLP, diffLevel;
  float *spectrum = new float[fftOrder/2],
        *criticalSpectrum = new float[nbBands + 1];
  int cMaxSpectro = fMaxSpectro / samplingRate * fftOrder;


  initham_(NULL, fen_signal);
  if (   nbBandsPLP_ != nbBands || samplingRatePLP_ != samplingRate || fftOrderPLP_ != fftOrder
      || percepFrequencyScale_ != percepFrequencyScale) {
     if (criticalBandsPLP) {
       freeCriticalBandFilters(criticalBandsPLP, nbBandsPLP_);
       delete [] centerFrequenciesPLP;
     }
     criticalBandsPLP = buildCriticalBandFilters(samplingRate, nbBands, fftOrder, &centerFrequenciesPLP, true,
     													       percepFrequencyScale, ConstantMelWidth, false);
     fftOrderPLP_ = fftOrder;
     nbBandsPLP_ = nbBands;
     samplingRatePLP_ = samplingRate;
     percepFrequencyScale_ = percepFrequencyScale;
  }

  if (preaccentuation) {
    preac_d_(parole, parole_debut - 1, real, fen_signal);
    ham_dd_(real, real, NULL, fen_signal);
  }
  else ham_sd_(parole, parole_debut, real, NULL, fen_signal);
  for (int j = fen_signal; j < fftOrder; j++) real[j] = 0.;

  // FFT to compute the spectrum
  realfft(fftOrder, real);

  // magnitude spectrum calculation
  spectrum[0] = SQR(real[0]);
  for (int k = 1; k < fftOrder / 2; k++)
    spectrum[k] = SQR(real[k]) + SQR(real[fftOrder - k]);

  // application of the critical bands
  for (int i = 0; i < nbBands; i++) {
    int j;
    for (j = 0, criticalSpectrum[i] = 0.; j < fftOrder / 2; j++)
      criticalSpectrum[i] += spectrum[j] * criticalBandsPLP[i][j];
  }

  // Intensity conversion
  if (intensityConversion) {
    averageLevel = 0.;
    // calculation of the average level (to correct the display)
    for (int i = 0; i < nbBands; i++) averageLevel += criticalSpectrum[i];
    averageLevel = 10. * log10(averageLevel / nbBands + EPSILON);
    for (int i = 0; i < nbBands; i++)
    	criticalSpectrum[i] = pow(criticalSpectrum[i], 0.33);
  }

  // computation of the autocorrelation coefficients through inverse Fourier transform
  // (only cos because the spectrum is real and symmetric).
  for (int m = 0; m <= predictionOrder; m++) {
    double L = 2. * nbBands + 1.,
           s = 1. / L * criticalSpectrum[0];
    for (int k = 1; k < nbBands; k++)
      s += 2. / L * criticalSpectrum[k] * cos (_PI * 2. / L * k * m);
    autoc[m] = s;
  }

  // Durbin method
  *e = autoc[0];
  if (fabs(*e) < EPSILON) {
    for (int i = 1; i <= predictionOrder; i++) coefp[i] = 0.0;
    coefp[0] = 1.0;
  }
  else {
    double x,
           *at = new double[predictionOrder];
    coefp[0] = 1.0;
    x = autoc[1] / *e;
    coefp[1] = x;
    *e = (1 - x * x) * *e;
    for (int i = 2; i <= predictionOrder; i++) {
      int k = i - 1;
      x = autoc[i];
      for (int j = 1; j <= k; j++) x = x - autoc[i - j] * coefp[j];
      x = x / *e;
      coefp[i] = x;
      for (int j = 1; j <= k; j++)
	     at[j] = coefp[j] - x * coefp[i - j];
      for (int j = 1; j <= k; j++)
	     coefp[j] = at[j];
      *e = (1 - x * x) * *e;
    }
    delete [] at;
  }

// There are two solutions:
// - either compute a fast Fourier transform but that requires a power of 2 points and thus an extra rescaling.
//  There is still a bug in this method...
// - or compute a simple Fourier transform for the # of bands and then apply a simple completion.
// The second solution seems slightly better provided than the prediction order is not too large.
#ifdef AVECFFT
  real[0] = 1.0;
  for (int j = 1;  j <= predictionOrder; j++) real[j] = -coefp[j];
  for (int j = predictionOrder + 1; j < fftOrder; j++) real[j] = 0.;

  realfft(fftOrder, real);

  if (fabs(*e) < EPSILON) *e = 0.0000001;
  *e = log10(*e);

  PLPspectrum[0] = 10. * (*e - log10(SQR(real[0]) + 1.e-06));
  for (int k = 1; k < fftOrder / 2; k++)
    PLPspectrum[k] = 10. * (*e - log10(SQR(real[k]) + SQR(real[fftOrder - k]) + 1e-06));

#else
  double PI2N = _PI * 2. / (nbBands * 2.);
  for (int i = 0;  i < fftOrder; i++) real[i] = 0;

  real[0] = 1.;
  for(int i = 1; i <= predictionOrder; i++) real[0] -= coefp[i];

  for (int k = 1; k < nbBands; k++) {
    real[k] = 1.; real[fftOrder - k] = 0.;
    for(int i = 1; i <= predictionOrder; i++) {
      real[k] -= coefp[i] * cos(PI2N * k * i);
      real[fftOrder - k] -= coefp[i] * sin(PI2N * k * i);
    }
  }
  if (fabs(*e) < EPSILON) *e = 0.0000001;
  *e = log10(*e);

  PLPspectrum[0] = 10. * (*e - log10(SQR(real[0]) + 1.e-06)) + averageLevel;
  for (int k = 1; k < nbBands; k++)
    PLPspectrum[k] = 10. * (*e - log10(SQR(real[k]) + SQR(real[fftOrder - k]) + 1e-06)) + averageLevel;
  for (int i = nbBands;  i < fftOrder / 2; i++) PLPspectrum[i] = 0.;
  if (intensityConversion) {   // to compensate for the difference of level for displaying spectra.
    averagePLP = 0.;
    for (int i = 0; i < nbBands; i++) averagePLP += PLPspectrum[i];
    averagePLP /= nbBands;
    diffLevel = averageLevel - averagePLP;
    for (int i = 0; i < nbBands; i++) PLPspectrum[i] += diffLevel;
  }
#endif


#ifdef AVECFFT
 rescalePerceptiveSpectrum(perceptive, fftOrder / 2, melFrequencyScale,
  									 samplingRate, PLPspectrum, fftOrder / 2);
#else
  spectralCompletion(PLPspectrum, fftOrder, cMaxSpectro, centerFrequenciesPLP, nbBands, false);
#endif

  // reste  afficher ce spectreavec la bonne chelle.
  delete [] real;
  delete [] autoc;
  delete [] spectrum;
  delete [] perceptive;
  delete [] criticalSpectrum;
}


void spectreLPC(
	float spectre_[],
   racines1_lpc &res_lpc,
	SAMPLE *parole,
	long parole_debut,
   int fen_signal,
   int fen_fft,
	int preaccentuation,
	int ordre,
	float rayon)
{
  int nbform;
  float *r = new float[ordre + 1],
	     *a = new float[ordre + 1],
	     *sf= new float[ordre + 1],
        *xr = new float[fen_signal],
        gain,
        *coef = new float[ordre + 1], // coefficients du denominateur de la fct de trandfert
        *fpos = new float[ordre + 1], // nombre, puis frequences des formants
        *fbw = new float[ordre + 1];  // nombre, puis largeur de bande des formants

  double *realfft = new double[fen_fft],
         *imagfft = new double[fen_fft];

  initham_((float *) NULL, fen_signal);
  if (preaccentuation) {
    preac_f_(parole, parole_debut, xr, fen_signal);
    ham_ff_(xr, xr, NULL, fen_signal);
  }
  else ham_sf_(parole, parole_debut, xr, NULL, fen_signal);

  autolpc (xr, r, a, sf, &gain, fen_signal , ordre);

  realfft[0] = 1.0;
  imagfft[0] = 0.0;
  coef[0] = 1.;
  for (int j = 1;  j <= ordre; j++) {
    realfft[j] = -a[j] * pow((double) rayon, (double) -j);
    imagfft[j] = 0;
    coef[j] = -a[j];
  }
  for (int j = ordre + 1; j < fen_fft; j++) realfft[j] = imagfft[j] = 0.;

  fband(ordre, coef, (float) en_cours.frequence, largband, 0.0,
        (float) (en_cours.frequence/2.0), fpos, fbw);

  fft(fen_fft, realfft, imagfft);

  if (gain == 0) gain = 0.0000001;
  gain = log10(gain);

  for (int i = 0; i < fen_fft / 2; i++)
    spectre_[i] = 10. * (gain - log10(realfft[i] * realfft[i] + imagfft[i] * imagfft[i] + EPSILON) );

  nbform = (int) fpos[0];
  res_lpc.resize(nbform);
  for (int i = 1; i <= nbform; i++) {
    int canal = IROUND(fpos[i] / en_cours.frequence * 2. * fen_fft / 2);
    res_lpc[i - 1].val = canal;
    res_lpc[i - 1].lar = fbw[i];
    res_lpc[i - 1].fr = fpos[i];
    res_lpc[i - 1].db = spectre_[canal];
  }

  delete [] r;
  delete [] a;
  delete [] sf;
  delete [] xr;
  delete [] coef;
  delete [] fpos;
  delete [] fbw;
  delete [] realfft;
  delete [] imagfft;
}

/*
 * Calcul du spectre et des poles d'une LPC selective.
 */
void spectreLPCselective(
	SAMPLE *data,	   // speech signal
   long data_debut,
	FLOATbuf& spect,	// spectre de la LPC selective
   int fen_signal,
	int ordre_lpc, 	// ordre de la LPC selective
	float freqEcht,	// frequence d'echantillonnage
	float wa,			// frequences extremes de la LPC
	float wb,       	//     selective (donc de wa  wb Hz)
	int preac)			// VRAI s'il faut une preaccentuation
{
  int  i,nbform;
  int fen_fft=256;
  float err,			   				  // erreur de prediction (gain)
        *xr = new float[fen_signal],  // signal apres preaccentuation et fenetrage
        *coef = new float[ordre_lpc + 1], // coefficients du denominateur de la fct de trandfert
        *fpos = new float[ordre_lpc + 1], // nombre, puis frequences des formants
        *fbw = new float[ordre_lpc + 1];  // nombre, puis largeur de bande des formants
  double *realfft = new double[fen_fft],
         *imagfft = new double[fen_fft];

  char ch[1000];

  spect.newsize(fen_fft/2);

  initham_((float *) NULL, fen_signal);
  if (preac) {
    preac_f_(data, data_debut, xr, fen_signal);
    ham_ff_(xr, xr, (float *) NULL, fen_signal);
  }
  else ham_sf_(data, data_debut, xr, (float *) NULL, fen_signal);

  seleclp(fen_signal, ordre_lpc, wa, wb, freqEcht, xr, &err, coef);

  fband(ordre_lpc, coef, 16000., 99999.9, wa, wb, fpos, fbw);

  nbform = (int) fpos[0];
  sprintf(ch,ml(6304,"file : %s"),en_cours.nom);
  logs.wr1(ch);
  logs.wr1(ml(2320,"Formant\tFreq.\tBande"));
  logs.wr1(ml(2321,"Numro\t\tLargeur"));
  logs.wr1("\t[Hz]\t[Hz]");
  for (i = 1; i <= nbform; i++) {
    sprintf(ch,"%d\t%d\t%d",i,(int)fpos[i], (int)fbw[i]);
    logs.wr1(ch);
  }

  err = 10 * log10(fabs(err)+EPSILON);

  for (i = 0; i <= ordre_lpc; i++) {
    realfft[i] = coef[i];
    imagfft[i] = 0.;
  }
  for (i = ordre_lpc + 1; i < fen_fft; i++) realfft[i] = imagfft[i] = 0.;

  fft(fen_fft, realfft, imagfft);

  for (i = 0; i < fen_fft / 2; i++)
    spect[i]= err - 10.0 * log10(realfft[i] * realfft[i] + imagfft[i] * imagfft[i] + EPSILON);

  delete [] xr;
  delete [] coef;
  delete [] fpos;
  delete [] fbw;
  delete [] realfft;
  delete [] imagfft;
}

static int fen_signal;
static bool ind_fen_signal;
static bool controle_fen_signal(
	  long echt,
     struct ctrl_temps *ctloc,
	  int fen_signal_)
{
  int ind;
  fen_signal = fen_signal_;
  ind =   (echt > ctloc->d_e + fen_signal / 2 + 3)
       &&(echt < ctloc->f_e - fen_signal - 3) ;
//  if ( (!ind)&&(!ind_fen_signal) )
//    affiche_warning(ml(2307,"toutes les coupes ne sont pas calcules"));
  ind_fen_signal = ind_fen_signal || (!ind);
  return ind;
}

void log_spectre(char *titre,int fen_fft,FLOATbuf& spectre)
{
  int i;
  char ch[1000];

  logs.wr1(titre);
  sprintf(ch,ml(6304,"file : %s"),en_cours.nom);
  logs.wr1(ch);
  logs.wr1(ml(2330,"Freq.\tAmpl."));
  logs.wr1("[Hz]\t[dB]");
  for (i = 0; i < fen_fft/2; i++) {
    sprintf(ch,"%.1f\t%.1f", i*(en_cours.frequence/(float)fen_fft),
    			spectre[i]);
    logs.wr1(ch);
  }
}

void log_spectre_(char *titre,int fen_fft, float spectre[])
{
  int i;
  char ch[1000];

  logs.wr1(titre);
  sprintf(ch,ml(6304,"file : %s"),en_cours.nom);
  logs.wr1(ch);
  logs.wr1(ml(2330,"Freq.\tAmpl."));
  logs.wr1("[Hz]\t[dB]");
  for (i = 0; i < fen_fft/2; i++) {
    sprintf(ch,"%.1f\t%.1f", i*(en_cours.frequence/(float)fen_fft), spectre[i]);
    logs.wr1(ch);
  }
}

extern void spectreLPCCEP(float spectre_[], SAMPLE *parole, long parole_debut, int fen_signal,
      									int fftOrder, int preaccentuation, int LPCorder, int cepstrumOrder);
extern void unwrappingPhase(
  SAMPLE parole[],
  double phase[],
  int fenSignal,  // must be a power of 2
  int ordreFFT);
#define PHASEUNWRAPPING
void affUneCoupe(d_planche *fenetre,
	     SAMPLEbuf& cnv,
	     long echt,
        struct ctrl_temps *ctloc,
        STYLEAXES styleAxes)
{
  if (!yaUnSignal) return;
  if (!dans(echt, ctloc->d_a, ctloc->f_a)) return;

  // souris(XC_watch);

  fenetre->clear_fenetre();
  ind_fen_signal=FALSE;

  logs.open1();
  logs.date(echt);
  controleOrdreFFT(PTWcoupe->pFFT1->fen_signal, PTWcoupe->pFFT1->fen_fft);
   if (  PTWcoupe->pFFT1->isAffichee
      && controle_fen_signal(echt, ctloc, enEchtf(PTWcoupe->pFFT1->fen_signal)))
  { // affichage de la premiere fft
    double e = en_cours.frequence / (1 << PTWcoupe->pFFT1->fen_fft);
	 int i,
        cMaxSpectro = fMaxSpectro / en_cours.frequence * (1 << PTWcoupe->pFFT1->fen_fft);

    float x_, y_;

    MgiHue(PTWcoupe->pFFT1->color, fenetre);
    if (PTWcoupe->pFFT1->moduleIsDisplayed) {
      float  *spectre = new float[1 << PTWcoupe->pFFT1->fen_fft];
	 	spectreFft(spectre,cnv,_i(echt)-fen_signal/2+1, PTWcoupe->pFFT1->isPreac, (1 << PTWcoupe->pFFT1->fen_fft), fen_signal);

	 	log_spectre_(ml(2301,"// Transforme de Fourier 1 :")
            ,(1 << PTWcoupe->pFFT1->fen_fft),spectre);
    	snoiwidth(2);
    	x_ = 0.; y_ = spectre[0] + PTWcoupe->pFFT1->niv;
    	for (i = 1; i < cMaxSpectro; i++) {
      	float xx, yy;
      	xx = i * e;
      	yy = spectre[i] + PTWcoupe->pFFT1->niv;
      	snortl(x_, y_, xx, yy, fenetre);
      	x_ = xx;
      	y_ = yy;
    	}
      delete [] spectre;
    }
	 else {
      int ordreFFT = (1 << PTWcoupe->pFFT1->fen_fft);
//    double *phase = new double[(1 << PTWcoupe->pFFT1->fen_fft) / 2 + 1];
    	double *phase = new double[ordreFFT / 2 + 1];
    	e = en_cours.frequence / ordreFFT;
    	unwrappingPhase(&cnv[_i(echt) - ordreFFT / 2 + 1], phase, fen_signal, ordreFFT);
      double maxPhase = max_td(phase, 0, ordreFFT / 2),
           	minPhase = min_td(phase, 0, ordreFFT / 2),
           	dbmin = dBMIN, dbmax = dBMAX;
      x_ = 0.; y_ = (phase[0] - minPhase) / (maxPhase - minPhase + EPSILON) * (dbmax - dbmin) + dbmin ;
      for (i = 1; i < ordreFFT / 2; i++) {
        float xx, yy;
      		xx = i * e;
      		yy = (phase[i] - minPhase) / (maxPhase - minPhase + EPSILON) * (dbmax - dbmin) + dbmin ;
      		snortl(x_, y_, xx, yy, fenetre);
      		x_ = xx;
      		y_ = yy;
      }
      delete [] phase;
	 }
    snoiwidth(1);
    logs.rclf1();
  }

  controleOrdreFFT(PTWcoupe->pFFT2->fen_signal, PTWcoupe->pFFT2->fen_fft);
  if (   PTWcoupe->pFFT2->isAffichee
      && controle_fen_signal(echt, ctloc,
            					 enEchtf(PTWcoupe->pFFT2->fen_signal)))
  { // affichage de la seconde fft
    double e = en_cours.frequence / (1 << PTWcoupe->pFFT2->fen_fft);
	 int i,
        cMaxSpectro = fMaxSpectro / en_cours.frequence * (1 << PTWcoupe->pFFT2->fen_fft);
    float *spectre = new float[1 << PTWcoupe->pFFT2->fen_fft], x_, y_;
    e = en_cours.frequence / ( 1 << PTWcoupe->pFFT2->fen_fft);
    MgiHue(PTWcoupe->pFFT2->color, fenetre);
    if (PTWcoupe->pFFT2->moduleIsDisplayed) {
      float  *spectre = new float[1 << PTWcoupe->pFFT2->fen_fft];
	 	spectreFft(spectre,cnv,_i(echt)-fen_signal/2+1, PTWcoupe->pFFT2->isPreac, (1 << PTWcoupe->pFFT2->fen_fft), fen_signal);

	 	log_spectre_(ml(2302,"// Transforme de Fourier 2 :")
            ,(1 << PTWcoupe->pFFT2->fen_fft),spectre);
    	snoiwidth(2);
    	x_ = 0.; y_ = spectre[0] + PTWcoupe->pFFT2->niv;
    	for (i = 1; i < cMaxSpectro; i++) {
      	float xx, yy;
      	xx = i * e;
      	yy = spectre[i] + PTWcoupe->pFFT2->niv;
      	snortl(x_, y_, xx, yy, fenetre);
      	x_ = xx;
      	y_ = yy;
    	}
      delete [] spectre;
    }
	 else {
      int ordreFFT = (1 << PTWcoupe->pFFT2->fen_fft);
//    double *phase = new double[(1 << PTWcoupe->pFFT2->fen_fft) / 2 + 1];
    	double *phase = new double[ordreFFT / 2 + 1];
    	e = en_cours.frequence / ordreFFT;
    	unwrappingPhase(&cnv[_i(echt) - ordreFFT / 2 + 1], phase, fen_signal, ordreFFT);
      double maxPhase = max_td(phase, 0, ordreFFT / 2),
           	minPhase = min_td(phase, 0, ordreFFT / 2),
           	dbmin = dBMIN, dbmax = dBMAX;
      x_ = 0.; y_ = (phase[0] - minPhase) / (maxPhase - minPhase + EPSILON) * (dbmax - dbmin) + dbmin ;
      for (i = 1; i < ordreFFT / 2; i++) {
        float xx, yy;
      		xx = i * e;
      		yy = (phase[i] - minPhase) / (maxPhase - minPhase + EPSILON) * (dbmax - dbmin) + dbmin ;
      		snortl(x_, y_, xx, yy, fenetre);
      		x_ = xx;
      		y_ = yy;
      }
      delete [] phase;
	 }
    snoiwidth(1);
    logs.rclf1();
  }

   controleOrdreFFT(PTWcoupe->pCriticalBands->fen_signal, PTWcoupe->pCriticalBands->fen_fft);
   if (  PTWcoupe->pCriticalBands->isAffichee
      && controle_fen_signal(echt, ctloc, enEchtf(PTWcoupe->pCriticalBands->fen_signal))) {
    double e = en_cours.frequence / (1 << PTWcoupe->pCriticalBands->fen_fft),
           gain, coefp[128 + 1];

	 int i,
        cMaxSpectro = fMaxSpectro / en_cours.frequence * (1 << PTWcoupe->pCriticalBands->fen_fft);

    float *spectre = new float[1 << PTWcoupe->pCriticalBands->fen_fft], x_, y_;


    MgiHue(PTWcoupe->pCriticalBands->color, fenetre);
    // MgiHue(NOIR, fenetre);   // pour le livre
	 if (PTWcoupe->pCriticalBands->analysis == 1) {
      spectreCriticalBandsPLP(spectre, &cnv[0], _i(echt)-fen_signal/2+1, PTWcoupe->pCriticalBands->isPreac,
                              PTWcoupe->pCriticalBands->PLPorder, (1 << PTWcoupe->pCriticalBands->fen_fft), fen_signal,
                              PTWcoupe->pCriticalBands->nbBands, PTWcoupe->pCriticalBands->frequencyScale,
                              PTWcoupe->pCriticalBands->intensityConversion, en_cours.frequence, &gain, coefp);
    }
    else {
      if (!checkMelFilterWidth(PTWcoupe->pCriticalBands->frequencyScale, PTWcoupe->pCriticalBands->nbBands,
      								 1 << PTWcoupe->pCriticalBands->fen_fft, en_cours.frequence))
        PTWcoupe->pCriticalBands->melWidthStrategy = ConstantMelWidth;
      spectreCriticalBands(spectre,cnv,_i(echt)-fen_signal/2+1,
		     PTWcoupe->pCriticalBands->isPreac,
		     (1 << PTWcoupe->pCriticalBands->fen_fft),
		     fen_signal, PTWcoupe->pCriticalBands->nbBands, en_cours.frequence,
           PTWcoupe->pCriticalBands->completionStrategy, PTWcoupe->pCriticalBands->frequencyScale,
           PTWcoupe->pCriticalBands->melWidthStrategy,
           PTWcoupe->pCriticalBands->withCepstralSmoothing, PTWcoupe->pCriticalBands->withDctIdctLiftering,
           PTWcoupe->pCriticalBands->nbMelLiftering,
           PTWcoupe->pCriticalBands->nbFreqSamplesDisplayed);
    }

	 log_spectre_(ml(2331,"// Analyse bandes critiques :")
            ,(1 << PTWcoupe->pCriticalBands->fen_fft),spectre);

    snoiwidth(2);
    // snoiwidth(3);  // 3, c'est pour le livre
    x_ = 0.; y_ = spectre[0] + PTWcoupe->pCriticalBands->niv;
    for (i = 1; i < cMaxSpectro; i++) {
      float xx, yy;
      xx = i * e;
      yy = spectre[i] + PTWcoupe->pCriticalBands->niv;
      snortl(x_, y_, xx, yy, fenetre);
      x_ = xx;
      y_ = yy;
    }
    snoiwidth(1);
    delete [] spectre;
    logs.rclf1();
  }

  if (   PTWcoupe->pLisCep1->isAffichee
      && controle_fen_signal(echt, ctloc,
	    							 min((int)enEchtf(PTWcoupe->pLisCep1->fen_signal) 
			,(1 << PTWcoupe->pLisCep1->fen_fft1))) ) {
    // affichage du premier lissage cepstral.
    FLOATbuf spectre(1 << PTWcoupe->pLisCep1->fen_fft3,
	       				"spectre LisCep1 in affUneCoupe()");
    FLOATbuf *Pspectre=&spectre;
    float e = en_cours.frequence / (1 << PTWcoupe->pLisCep1->fen_fft3);
	 int i,
        cMaxSpectro = fMaxSpectro / en_cours.frequence * (1 << PTWcoupe->pLisCep1->fen_fft3);

    MgiHue(PTWcoupe->pLisCep1->color, fenetre);
    if (cepstrePartiel(&Pspectre
		  , cnv, _i(echt)-fen_signal/2+1
		  , 1, 0
		  , fen_signal
		  , (1 << PTWcoupe->pLisCep1->fen_fft1)
		  , (1 << PTWcoupe->pLisCep1->fen_fft2)
		  , (1 << PTWcoupe->pLisCep1->fen_fft3)
		  , enEchtf(PTWcoupe->pLisCep1->debut_liftrage)
		  , enEchtf(PTWcoupe->pLisCep1->fin_liftrage)
		  , 1, PTWcoupe->pLisCep1->isPreac, PTWcoupe->pLisCep1->beforeLastSpectrum)
        !=0) {
        float x_, y_;
      if (PTWcoupe->pLisCep1->beforeLastSpectrum) {
     	  log_spectre(ml(2303,"// Lissage cepstral 1 :")
            ,(1 << 8),spectre);
        e = (en_cours.frequence / 2.) / 1000.;
        snoiwidth(2);
        x_ = e; y_ = spectre[1] * 3. + (dBMIN + dBMAX) / 2.;
        for (i = 2; i < min(1000, (1 << (PTWcoupe->pLisCep1->fen_fft2 - 1))); i++) {
          float xx, yy;
          xx = i * e * (1 << (PTWcoupe->pLisCep1->fen_fft1 - PTWcoupe->pLisCep1->fen_fft2));
          yy = spectre[i] * 3.+ (dBMIN + 2 * dBMAX) / 3.;
          snortl(x_, y_, xx, yy, fenetre);
          x_ = xx;
          y_ = yy;
        }
        axe_x(0., fMaxSpectro, 0., 1000., (dBMIN + 2 * dBMAX) / 3., NOIR,0.,"%-4.0f", wcoupe->charHeight, wcoupe->charWidth, fenetre);
        snoiwidth(1);
      }
      else {
        log_spectre(ml(2303,"// Lissage cepstral 1 :"),
            (1 << PTWcoupe->pLisCep1->fen_fft3),spectre);
        e = en_cours.frequence / (1 << PTWcoupe->pLisCep1->fen_fft3);
        snoiwidth(2);
        x_ = 0.; y_ = spectre[0] + PTWcoupe->pLisCep1->niv;
        for (i = 1; i < cMaxSpectro; i++) {
          float xx, yy;
          xx = i * e;
          yy = spectre[i] + PTWcoupe->pLisCep1->niv;
          snortl(x_, y_, xx, yy, fenetre);
          x_ = xx;
          y_ = yy;
        }
        snoiwidth(1);
      }
      logs.rclf1();
    }
  }

  if (   PTWcoupe->pLisCep2->isAffichee
      && controle_fen_signal(echt, ctloc,
                            min((int)enEchtf(PTWcoupe->pLisCep2->fen_signal),
								    (1 << PTWcoupe->pLisCep2->fen_fft1))) )
  { // affichage du second lissage cepstral.
    FLOATbuf spectre(1 << PTWcoupe->pLisCep2->fen_fft3,
	       			   "spectre LisCep2 in affUneCoupe()");
    FLOATbuf *Pspectre=&spectre;
    float e = en_cours.frequence / (1 << PTWcoupe->pLisCep2->fen_fft3);
	 int i,
        cMaxSpectro = fMaxSpectro / en_cours.frequence * (1 << PTWcoupe->pLisCep2->fen_fft3);

    MgiHue(PTWcoupe->pLisCep2->color, fenetre);
    if (cepstrePartiel(&Pspectre
		  , cnv, _i(echt)-fen_signal/2+1
		  , 1, 0
		  , fen_signal
		  , (1 << PTWcoupe->pLisCep2->fen_fft1)
		  , (1 << PTWcoupe->pLisCep2->fen_fft2)
		  , (1 << PTWcoupe->pLisCep2->fen_fft3)
		  , enEchtf(PTWcoupe->pLisCep2->debut_liftrage)
		  , enEchtf(PTWcoupe->pLisCep2->fin_liftrage)
		  , 1, PTWcoupe->pLisCep2->isPreac, PTWcoupe->pLisCep2->beforeLastSpectrum)
        !=0) { 
      float x_, y_;
      if (PTWcoupe->pLisCep2->beforeLastSpectrum) {
     	  log_spectre(ml(2303,"// Lissage cepstral 1 :")
            ,(1 << 8),spectre);
        e = (en_cours.frequence / 2.) / 1000.;
        snoiwidth(2);
        x_ = e; y_ = spectre[1] * 3. + (dBMIN + dBMAX) / 2.;
        for (i = 2; i < min(1000, (1 << (PTWcoupe->pLisCep2->fen_fft2 - 1))); i++) {
          float xx, yy;
          xx = i * e * (1 << (PTWcoupe->pLisCep2->fen_fft1 - PTWcoupe->pLisCep2->fen_fft2));
          yy = spectre[i] * 3.+ (2 * dBMIN + dBMAX) / 3.;
          snortl(x_, y_, xx, yy, fenetre);
          x_ = xx;
          y_ = yy;
        }
       snoiwidth(1);
       axe_x(0., fMaxSpectro, 0., 1000., (2 * dBMIN + dBMAX) / 3., NOIR,0.,"%-4.0f", wcoupe->charHeight, wcoupe->charWidth, fenetre);
      }
      else {
        log_spectre(ml(2303,"// Lissage cepstral 1 :"),
            (1 << PTWcoupe->pLisCep2->fen_fft3),spectre);
        e = en_cours.frequence / (1 << PTWcoupe->pLisCep2->fen_fft3);
        snoiwidth(2);
        x_ = 0.; y_ = spectre[0] + PTWcoupe->pLisCep2->niv;
        for (i = 1; i < cMaxSpectro; i++) {
          float xx, yy;
          xx = i * e;
          yy = spectre[i] + PTWcoupe->pLisCep2->niv;
          snortl(x_, y_, xx, yy, fenetre);
          x_ = xx;
          y_ = yy;
        }
       snoiwidth(1);
      }
		logs.rclf1();
	 }
  }

  if (   PTWcoupe->pLPC->isAffichee
		&& controle_fen_signal(echt, ctloc,
                             enEchtf(PTWcoupe->pLPC->fen_signal)) )
  { // affichage de la lpc
	 float spectre[1024];
    racines1_lpc res_lpc;
    char ch[1000];
    PTWcoupe->pLPC->fen_fft = 10;  // pour 1024
    float e = en_cours.frequence / (1 << PTWcoupe->pLPC->fen_fft), x_, y_;
	 int i,
        cMaxSpectro = fMaxSpectro / en_cours.frequence * (1 << PTWcoupe->pLPC->fen_fft);

	 MgiHue(PTWcoupe->pLPC->color, fenetre);
	 logs.wr1(ml(2305,"// LPC :"));
    sprintf(ch,ml(6304,"file : %s"),en_cours.nom);
    logs.wr1(ch);

	 spectreLPC(spectre, res_lpc, &cnv[0], _i(echt)-fen_signal/2+1, fen_signal, 1024,
    				PTWcoupe->pLPC->isPreac, ordre_lpc_msTOecht(PTWcoupe->pLPC->ordre_lpc),
               PTWcoupe->pLPC->rayon);
    double gain, coefp[128];

    snoiwidth(2);
    x_ = 0.; y_ = spectre[0] + PTWcoupe->pLPC->niv;
    for (i = 1; i < cMaxSpectro; i++) {
      float xx, yy;
      xx = i * e;
      yy = spectre[i] + PTWcoupe->pLPC->niv;
      snortl(x_, y_, xx, yy, fenetre);
      x_ = xx;
      y_ = yy;
    }
    snoiwidth(1);
    logs.wr1(ml(2310,"Formant\tFreq.\tBande\tRayon"));
    logs.wr1(ml(2311,"Numro\t\tLargeur"));
    logs.wr1("\t[Hz]\t[Hz]");
    for (i = 0; i < res_lpc.size(); i++) {
      sprintf(ch,"%d\t%d\t%d\t%.2f",i,
              (int)res_lpc[i].fr, (int)res_lpc[i].lar,
              exp(-M_PI / en_cours.frequence * res_lpc[i].lar));
    logs.wr1(ch);
  }

    logs.rclf1();
  }

  if (   PTWcoupe->pLPCCEP->isAffichee
		&& controle_fen_signal(echt, ctloc,
                             enEchtf(PTWcoupe->pLPCCEP->fen_signal)) ) {
		// int cMaxSpectro = fMaxSpectro / en_cours.frequence * (1 << PTWcoupe->pLPCCEP->fen_fft);
		int cMaxSpectro = fMaxSpectro / en_cours.frequence * 1024;  // Arbitrarilly 1024 to test
  		float spectre[1024];
      float x_, y_, e;

      spectreLPCCEP(spectre, &cnv[0], _i(echt)-fen_signal/2+1, fen_signal, 1024,
      				  PTWcoupe->pLPCCEP->isPreac,
                    ordre_lpc_msTOecht(PTWcoupe->pLPCCEP->ordre_lpc), 30); 
      MgiHue(PTWcoupe->pLPCCEP->color, fenetre);
      snoiwidth(2);
      e = en_cours.frequence / 1024.;
      x_ = 0.; y_ = spectre[0] + PTWcoupe->pLPCCEP->niv;  // LPC to get the same level as LPC
      for (int i = 1; i < cMaxSpectro; i++) {
        float xx, yy;
        xx = i * e;
        yy = spectre[i] + PTWcoupe->pLPC->niv;
        snortl(x_, y_, xx, yy, fenetre);
        x_ = xx;
        y_ = yy;
      }
      snoiwidth(1);

      log_spectre_("LPC cepstrally smoothed spectrum", 1024, spectre);
    	logs.rclf1();
   }


    if (   PTWcoupe->pLPCsel->isAffichee
      && controle_fen_signal(echt, ctloc,
	    							  enEchtf(PTWcoupe->pLPCsel->fen_signal)) )
  {  // affichage de la lpc selective
    FLOATbuf spectre(1,"spectre LPCsel in affUneCoupe()");
    float x_, y_;
    int i;

    MgiHue(PTWcoupe->pLPCsel->color, fenetre);
    logs.wr1(ml(2306,"// LPC slective :"));
    spectreLPCselective(&cnv[0],_i(echt)-fen_signal/2+1, spectre, fen_signal,
    							ordre_lpcsel_msTOecht(PTWcoupe->pLPCsel->ordre_lpc,PTWcoupe->pLPCsel->fInf,PTWcoupe->pLPCsel->fHaute),
		 						(float) en_cours.frequence, PTWcoupe->pLPCsel->fInf, PTWcoupe->pLPCsel->fHaute, VRAI);
    snoiwidth(2);
    x_ = PTWcoupe->pLPCsel->fInf;
    y_ = spectre[0] + PTWcoupe->pLPCsel->niv;
    for (i = 1; i < spectre.size(); i++) {
      float xx, yy;
      xx = PTWcoupe->pLPCsel->fInf + i * ((float)PTWcoupe->pLPCsel->fHaute -PTWcoupe->pLPCsel->fInf) / spectre.size();
      yy = spectre[i] + PTWcoupe->pLPCsel->niv;
      snortl(x_, y_, xx, yy, fenetre);
      x_ = xx;
      y_ = yy;
    }
    snoiwidth(1);
    logs.rclf1();
  }

  if (  PTWcoupe->pTrueEnv->isAffichee
      && controle_fen_signal(echt, ctloc,
                             min((int)enEchtf(PTWcoupe->pTrueEnv->fen_signal),
									  (1 << PTWcoupe->pTrueEnv->fen_fft))) )
  { // affichage du premier lissage cepstral.
    float *spectre = new float[1 << PTWcoupe->pTrueEnv->fen_fft];
    float e = en_cours.frequence / (1 << PTWcoupe->pTrueEnv->fen_fft);
	 int i,
        cMaxSpectro = fMaxSpectro / en_cours.frequence * (1 << PTWcoupe->pTrueEnv->fen_fft);
    float x_, y_;

    MgiHue(PTWcoupe->pTrueEnv->color, fenetre);
    halle(spectre, &cnv[0],
          max(_i(echt) - enEcht(PTWcoupe->pTrueEnv->fen_signal)/2+1, 0),
          1 << PTWcoupe->pTrueEnv->fen_fft,
          enEcht(PTWcoupe->pTrueEnv->fen_signal),
          enEchtf(PTWcoupe->pTrueEnv->debut_liftrage),
          enEchtf(PTWcoupe->pTrueEnv->fin_liftrage),
          PTWcoupe->pTrueEnv->alpha,
          PTWcoupe->pTrueEnv->nbIter,
          PTWcoupe->pTrueEnv->isPreac);
    log_spectre_(ml(2309,"// TrueEnvelope :")
            ,(1 << PTWcoupe->pTrueEnv->fen_fft),spectre);

    snoiwidth(2);
    x_ = 0.; y_ = spectre[0] + PTWcoupe->pTrueEnv->niv;
    for (i = 1; i < cMaxSpectro; i++) {
      float xx, yy;
      xx = i * e;
      yy = spectre[i] + PTWcoupe->pTrueEnv->niv;
      snortl(x_, y_, xx, yy, fenetre);
      x_ = xx;
      y_ = yy;
    }
    snoiwidth(1);
    logs.rclf1();
    delete [] spectre;
  }

  logs.wr1(ml(2308,"*** fin ***"));
  logs.close1();

  {  // to clear the bottom of the window
    float yb, yh;
    MgiHue(BLANC, fenetre);
    fenetre->getLYcoor(yb, yh);
    snorbox(0., yb, fMaxSpectro, dBMIN, fenetre);
  }
  MgiHue(NOIR, fenetre);
  if (styleAxes == NORMAL) {
    axe_x(0., fMaxSpectro, 0., fMaxSpectro, dBMIN, NOIR,0.,"%-4.0f", wcoupe->charHeight, wcoupe->charWidth, fenetre);
    axe_y(dBMIN, dBMAX, dBMIN, dBMAX, 0., NOIR,0., false, false, 0, "%2.0f", wcoupe->charHeight, wcoupe->charWidth, fenetre);
  }
  else {
    axe_xUp(0., fMaxSpectro, 0., fMaxSpectro, dBMIN, dBMAX, NOIR, 20, wcoupe->charHeight, wcoupe->charWidth, 2, 1000., fenetre);
    axe_ySimple(dBMIN, dBMAX, dBMIN, dBMAX, 0., fMaxSpectro, wcoupe->charHeight, wcoupe->charWidth, NOIR, fenetre);
  }

  InvalidateRect(fenetre->HWindow,NULL,TRUE);

  // souris(XC_top_left_arrow);
}

