/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier affv.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include "signal.h"
#include "wformant.h"
#include "polypoint.h"
#include "guiklat.h"
#include "axes.h"
#include "menu.h"
#include "guiklat.h"

extern Wpiquefor *wpiquefor;  // formant editor window

d_planche_f1f2 *wf1f2 = (d_planche_f1f2 *) NULL;

DEFINE_RESPONSE_TABLE1(d_planche_f1f2,d_planche)
  EV_WM_LBUTTONDOWN,
  EV_WM_LBUTTONUP,
  EV_WM_MOUSEMOVE,
END_RESPONSE_TABLE;
d_planche_f1f2::d_planche_f1f2(PTWindowsObject AParent)
    : d_planche(AParent)
{
}
d_planche_f1f2::~d_planche_f1f2()
{
  wf1f2 = (d_planche_f1f2 *) NULL;
}

void d_planche_f1f2::EvLButtonDown(uint, TPoint&)
{ LButtonDown=TRUE;
}
void d_planche_f1f2::EvLButtonUp(uint, TPoint&)
{ LButtonDown=FALSE;
}
void d_planche_f1f2::EvMouseMove(uint, TPoint& point)
{ int event_x,event_y;
  float xr, yr;
  char text1[80],text2[80];

  { event_x=point.x;
	 event_y=point.y;

    snoiunscalexy(event_x, event_y, &xr, &yr, this);
    sprintf(text1,"F1 %uHz",(unsigned int)yr);
    sprintf(text2,"F2 %uHz",(unsigned int)xr);
    Wact->setXY(text1,text2);
  }
}

/**********************************************************/
/*							  */
/* cette procedure affiche les valeurs moyennes de F1 F2  */
/* des voyelles d'un locuteur masculin Francais           */
/* F1 croit de haut en bas, F2 de droite a gauche(pasifou)*/
/*							  */
/**********************************************************/


/*
   F2
   <--------------+
		  |
		  |
		  |
		  |
		  |
		  |
		  V F1

*/

void affv(
     d_planche *fenetre)    /* fenetre ou se passe l'affichage 	*/
{
  int i;
  extern LOGFONT logFontPhonetique();

  fenetre->clear_fenetre();
  { // on initialise les coordonnes de la fentre
    // de manire que :
    //   l'axe des y laisse 28 pixels  gauche
    //   l'axe des x laisse 16 pixels en bas
    // si la fentre est trop petite en x ou en y,
    // l'axe correspondant est supprim
    RECT rect;
    GetClientRect(fenetre->HWindow,&rect);
    fenetre->setLXcoorD(F2MAX,F2MIN
	    ,(rect.right>100) ? 36 : 0   
	    ,rect.right-2);
    fenetre->setLYcoorD(F1MAX,F1MIN
	    ,rect.bottom-((rect.bottom>100) ? 16 : 0)
	    ,2);
  }

  /* on place les labels a l'endroit adequat */
  MgiHue(NOIR, fenetre);
  SetTextAlign(fenetre->hDCcomp,TA_CENTER|TA_BASELINE);
  SetBkMode(fenetre->hDCcomp,TRANSPARENT);

  fenetre->SetPlancheFont(queryFont(strlen(PhoneticFont) ? PhoneticFont : "arial", 7, 14, 0));

  for (i = 0; ; i++)
  { int phon,f1,f2;
    get_F1F2vowel(i,phon,f1,f2);
    if (phon==0) break;
    //
    fenetre->LTextOut(f2,f1,TextOfPhon(phon));
  }

  /* on affiche les axes */
  axe_x(F2MAX,F2MIN,F2MAX,F2MIN
	  , F1MAX, NOIR,0.,"%-4.0f", 12, 5, fenetre);
  SetTextAlign(fenetre->hDCcomp,TA_RIGHT|TA_BOTTOM);
  fenetre->LTextOut(F2MIN,F1MAX,"F2");
  axe_y(F1MAX, F1MIN, F1MAX, F1MIN,
	  	  F2MAX, NOIR, 0., false, false, 0, "%3.0f", 12, 5, fenetre);
  SetTextAlign(fenetre->hDCcomp,TA_LEFT|TA_TOP);
  fenetre->DTextOut(fenetre->LXtoDX(F2MAX)+3
                   ,fenetre->LYtoDY(F1MIN),"F1");

  fenetre->paint();
}

void affVoyelles()
{ long debut,fin;

  if (!yaUnSignal) return; //*en_cours.nom == '\0') return;
  if (  (ct.debRegion >= ct.finRegion)
      || !dans(ct.debRegion, ct.d_e, ct.f_e)
      || !dans(ct.finRegion, ct.d_e, ct.f_e))  {
    // pas de rgion, on ne fait rien
    return;
  }
  else {
    debut=ct.debRegion;
    fin=ct.finRegion;
  }
  souris(XC_watch);
  if (yaUneFenetreMccandless && !yaUneFenetrePiquefor)
    mccandless(debut, fin, en_cours.frequence,wf1f2, wspectro->wsona, space, ROUGE);
  if (yaUneFenetrePiquefor)
    wpiquefor->affPPF1F2(debut, fin, wf1f2, ROUGE);
    
  InvalidateRect(wspectro->wsona->HWindow,NULL,TRUE);
  InvalidateRect(wf1f2->HWindow,NULL,TRUE);
  souris(XC_top_left_arrow);
}
