/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier audio.cpp
//

#include "common.h"
#pragma hdrstop

#include <dir.h>
#include <sys\stat.h>
#include <string.h>
#include <errno.h>
#include "audio.h"
#include "filtre.h"
#include "signal.h"
extern void trouveQuoiAfficher();

WAVEOUTDEVICE WaveOutDevice;

WAVEOUTDEVICE::WAVEOUTDEVICE()
{ id=0;
  num=waveOutGetNumDevs();
  if (num!=0)
    waveOutGetDevCaps(id,&caps,sizeof(WAVEOUTCAPS));
}
void WAVEOUTDEVICE::newid(int n)
{ int i;
  char ch[80];

  id=n;
  i=waveOutGetDevCaps(id,&caps,sizeof(WAVEOUTCAPS));
  if (i!=0)
  { sprintf(ch,"waveOutGetDevCaps(id=%d,,)=0x%x",id,i);
    affiche_erreur(ch);
  }
}

void erboxmmio(int num,char *proc,char *file,char *text)
{
    STRING ch;
    strcpy(ch,"");
    if (strlen(text)!=0) wsprintf(ch,"%s\n",text);
    if (num!=0) wsprintf(ch+strlen(ch),ml(401,"error %d\n"),num);
    wsprintf(ch+strlen(ch),ml(402,"file %s\n"),file);
    wsprintf(ch+strlen(ch),ml(403,"directory "));
    getcwd(ch+strlen(ch),40);
    MessageBox(PTWtop->HWindow
      ,ch
      ,proc,MB_OK);
}
int erboxmmioOpen(int num,char *proc,char *file,HMMIO code)
{ if (code!=NULL) return 0;
  // erreur
  erboxmmio(num,proc,file,"");
  return 1;
}
int erboxmmioClose(int num,char *proc,char *file,UINT code)
{ if (code==0) return 0;
  // erreur
  erboxmmio(num,proc,file,"");
  return 1;
}
int erboxmmioRead(int num,char *proc,char *file,LONG code)
{ if (code!=-1) return 0;
  // erreur
  erboxmmio(num,proc,file,"");
  return 1;
}
int erboxmmioWrite(int num,char *proc,char *file,LONG code)
{ if (code!=-1) return 0;
  // erreur
  erboxmmio(num,proc,file,"");
  return 1;
}
int erboxmmioSeek(int num,char *proc,char *file,LONG code)
{ if (code!=-1) return 0;
  // erreur
  erboxmmio(num,proc,file,"");
  return 1;
}
int erboxmmioDescend(int num,char *proc,char *file,UINT code)
{ if (code==0) return 0;
  // erreur
  char ch[80];
  sprintf(ch,ml(409,"code:%x"),code);
  erboxmmio(num,proc,file,ch);
  return 1;
}

//////////////////////////////////////////////////////
// changement de frquence
//
// marche d'une frquence quelconque
//              (de 8000 Hz  24000Hz)
//        vers la frquence de 11025 Hz
//
// non valid pour d'autres frquences destination,
// 
class ChangementFrequence
{ public:
    // buffer initial
//    int huge *pin; AD
    short *pin;

    long nin;
    unsigned int freqin;
    // buffer rsultat
    HANDLE hpout;
//    int huge *pout; AD
    short *pout; 
    long nout;
    unsigned int freqout;
    //
    //long freqm; // frquence intermdiaire
               // multiple de freqin et freqout
    int din; // freqm/freqin
    int dout; // freqm/freqout
    //
    profiltre filt;

    ChangementFrequence();
    ~ChangementFrequence();
//   void bufferin(int huge *p, long np, unsigned int freq); AD
   void bufferin(short *p, long np, unsigned int freq);
    void bufferout(unsigned int freq);
    void exec();
    float pnf(long i); // buffer intermdiaire non filtr (virtuel)
    int pf(long i); // buffer intermdiaire filtr (virtuel)
};
ChangementFrequence::ChangementFrequence()
{ nout=-1;
}
ChangementFrequence::~ChangementFrequence()
{ if (nout>=0)
  { GlobalUnlock(hpout);
    GlobalFree(hpout);
  }
}
void ChangementFrequence::bufferin(short *p, long np
             ,unsigned int freq)
{ pin=p;
  nin=np;
  freqin=freq;
}
void ChangementFrequence::bufferout(unsigned int freq)
{ freqout=freq;
}
int ChangementFrequence::pf(long i)
{
  int j;
  long k;
  float sum;
  float *p;

  // l'appelant garantit i pour tre sur que p[k]
  //   est bien dans le buffer, 
  //      (i>=0)
  //    &&( ((i+filt.ordre-1)/din)<nin ) 

  sum = 0.0;
  k=(i+din-1)/din;
  p=(float *)&(*filt.f)[0];
  for (j = (int)(k*din-i);
       j < filt.ordre;
       j+=din,k++)
       // accs direct  pin[] et *filt.f
       //   pour aller plus vite
       // les contrles adquats ont t
       //   faits auparavant
    sum += pin[k] * p[j]; 
  return (int)max(-32000
                  ,min(32000
		       ,(long)(sum*din)));
}
void ChangementFrequence::exec()
{ float v;

  v=freqin/(float)freqout; 
  if      (v<0.55) { din=2; dout=1; }
  else if (v<0.63) { din=5; dout=3; }
  else if (v<0.7)  { din=3; dout=2; }
  else if (v<0.86) { din=4; dout=3; }
  else if (v<1.2)  { pout=pin; nout=nin; return; }
  else if (v<1.41) { din=3; dout=4; }
  else if (v<1.58) { din=2; dout=3; }
  else if (v<1.82) { din=3; dout=5; }
  else             { din=1; dout=2; }
  { char ch[80];
    sprintf(ch,"chgt freq : din=%d dout=%d",din,dout);
    Wmsg->wr(ch);
  }
  //
  strcpy(filt.type,"lp");
  filt.fc=0.5/dout;
  filt.att=15.;  
  filt.ordre=30;
  //
  nout=(nin*din-filt.ordre)/dout-2;
	// garantit que l'on ne tapera pas
        // en dehors du buffer pin
  hpout=GlobalAlloc(GMEM_MOVEABLE,nout*sizeof(int));
  pout=(short *)GlobalLock(hpout);
  //
  desfir(&filt);

  { long i,n;
    int j,k,m;
    k=(int)min(1000,nout/20);
    //
    Wact->showpourcent("chng freq");
    GetAsyncKeyState(VK_ESCAPE);
    for ( i=0;
	  (i<nout)&&((GetAsyncKeyState(VK_ESCAPE)&1)==0);
	  i+=k)
    { Wact->pourcent(i,nout);
      m=(int)min(k,nout-i);
      for (j=0,n=i;j<m;j++,n++)
	pout[n]=pf(n*dout);
    }
    Wact->hidepourcent();
    nout=max(0,i-k+m);
  }
}

//void play_n(int huge *pin,long npin
//         ,unsigned int freq,int nbrep)
void play_n(short *pin,long npin
	       ,unsigned int freq,int nbrep)
         // jouer un buffer de samples 16 bits
//
// stratgie:
//     choisir la sortie en 16 bits si possible,
//         et sinon choisir 8 bits
//     puis, essayer de sortir le son  la frquence
//             freq donne en paramtre
//         et sinon effectuer un changement de frquence
//             et sortir le son  11025 Hz
//
{ int i,ind=0,BytesPerSample;
  unsigned int freqout;
  char ch[80];
//  int huge *p;    AD
	short *p;
  long np;
  struct wavehdr_tag waveOutHdr;
  HWAVEOUT hWaveOut=0;
  PCMWAVEFORMAT waveFormat;

  WAVEFORMATEX waveFormatyl;

  ChangementFrequence cf;
  char *pbtamp;

  if ((WaveOutDevice.caps.dwFormats&WAVE_FORMAT_1M16)==0)
  { BytesPerSample=1;
    Wmsg->wr("play() 8 bits");
  }
  else
  { BytesPerSample=2;
    Wmsg->wr("play() 16 bits");
  }

  p=pin;
  np=npin;
  freqout=freq;
  if ( abs(freqout-11025)<10 ) freqout=11025;

  waveFormatyl.wFormatTag=WAVE_FORMAT_PCM;
  waveFormatyl.nChannels=1;
  waveFormatyl.nSamplesPerSec=freqout;
  waveFormatyl.nAvgBytesPerSec=freqout*BytesPerSample;
  waveFormatyl.nBlockAlign=BytesPerSample;
  waveFormatyl.wBitsPerSample=8*BytesPerSample;
  waveFormatyl.cbSize=0;
	if ( (waveOutOpen(&hWaveOut,WaveOutDevice.id
				 ,&waveFormatyl
		  ,NULL,0,WAVE_ALLOWSYNC)) !=0)
  { // waveOutOpen() fail, ressayer en
    // changeant de frquence
    freqout=11025;
	 waveFormatyl.nSamplesPerSec=freqout;
	 waveFormatyl.nAvgBytesPerSec=2*freqout;
	 if ( (i=waveOutOpen(&hWaveOut,WaveOutDevice.id
				 ,&waveFormatyl
		  ,NULL,0,WAVE_ALLOWSYNC)) !=0)
	 { // erreur
		wsprintf(ch,ml(410,"error =0x%x"),i);
		MessageBox(PTWtop->HWindow,ch
	,"waveOutOpen()",MB_OK);
		ind=1;
	 }
	 else
	 { cf.bufferin(pin,npin,freq);
		cf.bufferout(freqout);
		cf.exec();
		p=cf.pout;
		np=cf.nout;
	 }
  }

  // conversion de 16 bits en 8 bits si besoin
  if (BytesPerSample==1)
  { long il;

	 pbtamp =(char *) malloc(np);
	 for (il=0;il<np;il++)
		pbtamp[il]=(char)(0x80^(p[il]>>8));
	 waveOutHdr.lpData=(char *)pbtamp;
  }
  else
	 waveOutHdr.lpData=(char *)p;

  // jouer nbrep fois le buffer p
  GetAsyncKeyState(VK_ESCAPE);
  for (nbrep=max(1,nbrep);
		 (ind==0)&&(nbrep>0)&&((GetAsyncKeyState(VK_ESCAPE)&1)==0);
		 nbrep--)
  { waveOutHdr.dwBufferLength=np*BytesPerSample;
	 waveOutHdr.dwUser=0;
	 waveOutHdr.dwFlags=0;
    waveOutHdr.dwLoops=0;
    waveOutPrepareHeader(hWaveOut,&waveOutHdr,sizeof(WAVEHDR));
    waveOutWrite(hWaveOut,&waveOutHdr,sizeof(WAVEHDR));
    while ((waveOutHdr.dwFlags & WHDR_DONE)==0) continue;
    if ((i=waveOutUnprepareHeader(hWaveOut
	      ,&waveOutHdr,sizeof(WAVEHDR)))!=0)
    { wsprintf(ch,ml(410,"erreur 0x%x"),i);
      MessageBox(PTWtop->HWindow,ch
	       ,"waveOutUnprepareHeader()",MB_OK);
    }
  }

  if (BytesPerSample==1)
  { // librer le tampon utilis pour
    //   la conversion en octets
    free(pbtamp);
  }
  if (ind==0) waveOutClose(hWaveOut);
}


WAVEINDEVICE WaveInDevice;

WAVEINDEVICE::WAVEINDEVICE()
{
  id = 0;
  num = waveInGetNumDevs();
  if (num != 0)
    waveInGetDevCaps(id, &caps, sizeof(WAVEINCAPS));
}

void WAVEINDEVICE::newid(int n)
{
  int i;
  char ch[80];

  id = n;
  i=waveInGetDevCaps(id, &caps, sizeof(WAVEINCAPS));
  if (i!=0)  {
    sprintf(ch,"waveInGetDevCaps(id=%d,,)=0x%x", id, i);
    affiche_erreur(ch);
  }
}

extern void RawtoWav(char* fichierraw,char* fichierwav,float frequence);
static HWAVEIN hWaveIn = 0;
static infoFile dstinfo_sortie;
static char recorded[800]; // fichier parole resultat
static WAVEHDR *waveHeader;

void CALLBACK waveInProcedure (
  HWAVEIN hwi,
  UINT uMsg,
  DWORD dwInstance,
  DWORD dwParam1,
  DWORD dwParam2 )
{
  static int nsample;
  short *speech;

  switch (uMsg) {
  case WIM_OPEN:
   nsample = 0;
   en_cours.frequence = 22050;
     // on ouvre le fichier de sortie temporaire
   strcat(strcpy(dstinfo_sortie.nom, tempdir), "record.raw");
   strcat(strcpy(recorded, tempdir), "record.wav");
   dstinfo_sortie.nb_bits = NB_BITS;
   dstinfo_sortie.swap = SWAPINFILE;
   dstinfo_sortie.hmmio = mmioOpen(dstinfo_sortie.nom, NULL, MMIO_WRITE|MMIO_CREATE);
   dstinfo_sortie.isOpenSignal = TRUE;
   break;
  case WIM_CLOSE:
    //affiche_erreur("Fermeture audio");
    break;
  case WIM_DATA:  {
   MMRESULT erreur;
   int taille;
   WAVEHDR *currentWaveHeader = (WAVEHDR *) dwParam1;
   speech = (short *) currentWaveHeader->lpData ;
   taille = currentWaveHeader->dwBufferLength / sizeof(short);
   space.newsize(nsample + taille);
   for (int i = 0; i < currentWaveHeader->dwBufferLength / sizeof(short); i++) space[nsample + i] = speech[i];
   waveInUnprepareHeader(hWaveIn, currentWaveHeader, sizeof(WAVEHDR));
   free((char *)currentWaveHeader->lpData);
   currentWaveHeader->lpData = (char *) NULL;
   if (dstinfo_sortie.isOpenSignal) {
     ct.tf = nsample + taille;
     ::trouveQuoiAfficher();
     wsignal->aff();
     copyBufToHFile(space, nsample, dstinfo_sortie, currentWaveHeader->dwBufferLength);
     nsample += currentWaveHeader->dwBufferLength / sizeof(short);
     if (nsample == 88200) {
       mmioClose(dstinfo_sortie.hmmio,0);
       dstinfo_sortie.isOpenSignal = FALSE;
       en_cours.frequence = 22050.;
       strcpy(en_cours.nom, recorded);
       RawtoWav(dstinfo_sortie.nom, recorded, en_cours.frequence);
       lect_signal();
     }
   }
#ifdef UIY
   waveInReset(hWaveIn);
    waveInClose(hWaveIn);
   if (erreur != MMSYSERR_NOERROR) {
     char message[800];
     waveInGetErrorText(erreur, message,100);
     affiche_erreur(message);
   }
  }
#endif
  }
  break;
  }
}



class TWRecord : public TDialog
{
  TStatic *PTMessage;
  TButton *PTRecord;
  TButton *PTCancel;
  TEdit *PTDuration;
  
 public:
  TWRecord(PTWindowsObject AParent, LPSTR name);
  ~TWRecord();
  virtual void SetupWindow();
  virtual bool CanClose();

  void IDRecord(WPARAM code);
  void IDCancel(WPARAM code);
  void IDStop(WPARAM code);
 private:
  void startRecording(int duration);
  void stopRecording();
  DECLARE_RESPONSE_TABLE(TWRecord);
};
DEFINE_RESPONSE_TABLE1(TWRecord,TDialog)
  EV_CHILD_NOTIFY_ALL_CODES(143,IDRecord),
  EV_CHILD_NOTIFY_ALL_CODES(144,IDCancel),
  EV_WM_CLOSE,
END_RESPONSE_TABLE;


TWRecord::TWRecord(PTWindowsObject AParent, LPSTR name)
	:TDialog(AParent,name)
{
  PTMessage = new TStatic(this,142, 0);
  PTRecord = new TButton(this,143);
  PTCancel = new TButton(this,144);
  PTDuration = new TEdit(this, 145);
  hWaveIn = 0;
}

bool TWRecord::CanClose()
{
  return TRUE;
}


TWRecord::~TWRecord()
{
  delete PTMessage;
  delete PTRecord;
  delete PTCancel;
  delete PTDuration;
}

void TWRecord::SetupWindow()
{
   TDialog::SetupWindow();
//   PTDuration->SetText("0");
}

void TWRecord::IDRecord(WPARAM)
{
	char StrTemp[11];
   int duration;
   PTDuration->GetText(StrTemp,10);
   duration = atoi(StrTemp);
   if (duration < 0) duration = 1;
   if (duration > 60) duration = 60;
  startRecording(duration);
}

void TWRecord::IDCancel(WPARAM)
{
 TDialog::CmCancel();
}

// Enregistrement
void TWRecord::startRecording(
  int duration)
         // enregistrer un buffer de samples 16 bits
{
  int i,ind=0,BytesPerSample = 2;
  unsigned int freqin = 22050;

  long np;
  struct wavehdr_tag waveOutHdr;
  PCMWAVEFORMAT waveFormat;
  WAVEFORMATEX waveFormatyl;
  char *pbtamp;
  MMRESULT erreur;

  if ((WaveInDevice.caps.dwFormats & WAVE_FORMAT_2M16)== 0) {
  	affiche_erreur("Impossible d'enregistrer en mono  22050 Hz");
   return;
  }

  waveFormatyl.wFormatTag = WAVE_FORMAT_PCM;
  waveFormatyl.nChannels = 1;
  waveFormatyl.nSamplesPerSec = freqin;
  waveFormatyl.nAvgBytesPerSec = freqin * BytesPerSample;
  waveFormatyl.nBlockAlign = BytesPerSample;
  waveFormatyl.wBitsPerSample = 8 * BytesPerSample;
  waveFormatyl.cbSize = 0;

   en_cours.frequence = 22050;
     // on ouvre le fichier de sortie temporaire
   strcat(strcpy(dstinfo_sortie.nom, tempdir), "record.raw");
   strcat(strcpy(recorded, tempdir), "record.wav");
   dstinfo_sortie.nb_bits = NB_BITS;
   dstinfo_sortie.swap = SWAPINFILE;
   dstinfo_sortie.hmmio = mmioOpen(dstinfo_sortie.nom, NULL, MMIO_WRITE|MMIO_CREATE);

  erreur = waveInOpen(&hWaveIn, WaveInDevice.id, &waveFormatyl,
		   0, 0, CALLBACK_NULL);
  if (erreur != MMSYSERR_NOERROR) {
    char message[800];
    waveInGetErrorText(erreur, message,100);
    affiche_erreur(message);
    return;
  }

  waveHeader = (WAVEHDR *) malloc(sizeof(WAVEHDR));
  waveHeader->lpData = (char *) malloc(en_cours.frequence * duration * sizeof(short));
  waveHeader->dwBufferLength = en_cours.frequence * duration * sizeof(short);
  waveHeader->dwFlags = 0;
  erreur = waveInPrepareHeader(hWaveIn, waveHeader, sizeof(WAVEHDR));
  if (erreur != MMSYSERR_NOERROR) {
    char message[800];
    waveInGetErrorText(erreur, message,100);
    affiche_erreur(message);
    return;
  }
  erreur = waveInAddBuffer(hWaveIn, waveHeader, sizeof(WAVEHDR));
  if (erreur != MMSYSERR_NOERROR) {
    char message[800];
    waveInGetErrorText(erreur, message,100);
    affiche_erreur(message);
    return;
  }
  waveInStart(hWaveIn);
  Sleep(duration * 1000 + 100);
  stopRecording();
}


void TWRecord::stopRecording()
{
  MMRESULT erreur;

  erreur = waveInStop(hWaveIn);
  Sleep(500);
  if (erreur != MMSYSERR_NOERROR) {
   char message[800];
   waveInGetErrorText(erreur, message,100);
   affiche_erreur(message);
  }
  waveInReset(hWaveIn);
  waveInClose(hWaveIn);
  space.newsize(waveHeader->dwBytesRecorded / sizeof(short));
  for (int i = 0; i < waveHeader->dwBytesRecorded / sizeof(short); i++)
    space[i] = ((short *) waveHeader->lpData)[i];
  copyBufToHFile(space, 0, dstinfo_sortie, waveHeader->dwBytesRecorded);
  free(waveHeader->lpData);
  free(waveHeader);
  waveHeader = (WAVEHDR *) NULL;
  mmioClose(dstinfo_sortie.hmmio,0);
  dstinfo_sortie.isOpenSignal = FALSE;
  en_cours.frequence = 22050.;
  strcpy(en_cours.nom, recorded);
  RawtoWav(dstinfo_sortie.nom, recorded, en_cours.frequence);
  lect_signal();
  return;
}

void record()
{
  if (UpdateCurrentFileBox() == IDABORT) return;
  PTWtop->GetModule()->ExecDialog(new TWRecord(PTWtop,"Wrecord"));
}

