/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier axes.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include <string.h>

#include "axes.h"

/*
 * Cette fonction trace un axe horizontal et la
 * graduation correspondante simplifie vers le haut.
 */
void axe_xUp(
	float x_min,	// extremites de l'axe		
	float x_max,    //     (en coordonnes de la fentre)
	float v_min,	// valeurs aux extremites	
	float v_max,    //     ( afficher)
	float y_axeb,	// ordonnee de l'axe
   float y_axeh,
	COLORREF couleur,
   int hauteurTrait,
   int hauteurChar,
   int largeurChar,
   int lineWidth,
	float increment, // difference entre deux graduations
			 // successives. Si cette valeur est
			 // nulle, elle est fixee par
			 // le programme.				 
	d_planche *d)    // fenetre d'affichage
{
  float v;
  float yb, yh, d_ytext;
  char s[MAX_LIGNE];

  SetBkMode(d->hDCcomp,TRANSPARENT);
  SetTextAlign(d->hDCcomp,TA_LEFT|TA_TOP);
  d->getLYcoor(yb, yh);

  sprintf(s,"%.1f - %.1f kHz", x_min / 1000., x_max / 1000.);
  /* calcul de la hauteur des traits de graduation	*/
  float d_y = min(hauteurTrait, d->hauteur / 6) * d->yecart/ d->hauteur;


  d->SetPlancheFont(queryFont("arial",largeurChar, hauteurChar, 0));
  d_ytext = (0.5 * hauteurChar) * (yh-yb) / d->hauteur;

  d->DTextOut(d->largeur / 3, d->LYtoDY(y_axeb - d_ytext) - 1, s);

  d->Lline2(x_min , y_axeb ,x_max , y_axeb);
  d->Lline2(x_min , y_axeh ,x_max , y_axeh);
  if (fabs(increment) < EPSILON) increment = (v_max - v_min) / 10.;
  d->SetPenWidth(lineWidth);
  v = (v_max > v_min) ? (v_min + increment) : (v_max - increment);
  do {
    float x = (v - v_min) / (v_max - v_min) * (x_max-  x_min) + x_min;
    d->Lline2(x, y_axeb + d_y ,x , y_axeb);
    d->Lline2(x, y_axeh - d_y ,x , y_axeh);
    if (v_max > v_min) v += increment;
    else v -= increment;
  } while (   ((v_max > v_min) && (v <= v_max + EPSILON))
	        || ((v_max < v_min) && (v >= v_max + EPSILON)));

}

/*
 * Cette fonction trace un axe horizontal et la
 * graduation correspondante.
 */
void axe_x(
	float x_min,	// extremites de l'axe		
	float x_max,    //     (en coordonnes de la fentre)
	float v_min,	// valeurs aux extremites	
	float v_max,    //     ( afficher) 
	float y_axe,	// ordonnee de l'axe		
	COLORREF couleur,
	float increment, // difference entre deux graduations
			 // successives. Si cette valeur est
			 // nulle, elle est fixee par
			 // le programme.				 
	char *format,	 // format d'impression des graduations
   int hauteurChar, //Height of characters
   int largeurChar, // Width of characters		 
	d_planche *d)    // fenetre d'affichage  
// attention: cas des grands nombres
//     pas trait
{

  int ich, nb_chiffres, largeur,
      nc, subGrad;
  float d_y;		/* distance entre l'axe et la graduation */
  float yb, yh, x_g, x_d, x;
  char strnb[MAX_LIGNE];
  float v;

  SetBkMode(d->hDCcomp,TRANSPARENT);
  SetTextAlign(d->hDCcomp,TA_LEFT|TA_TOP);
  MgiHue(couleur, d);
  d->SetPlancheFont(queryFont("arial",largeurChar, hauteurChar, 0));
  //snol(x_min , y_axe ,x_max , y_axe, d);
  d->Lline2(x_min , y_axe ,x_max , y_axe);

  /* calcul du nombre de chiffres des nombres affiches	*/
  /* d'abord on recherche le nombre de chiffres s'il est
       donne (il doit l'etre) */
  for (ich = 1; !dans(format[ich], '0', '9'); ich++);
  nb_chiffres = max(fabs(log10(max(fabs(v_min),fabs(v_max)))),
  					     atoi(format + ich) );
  //snogetvcoor(&x_g, &yb, &x_d, &yh, d);
  d->getLXcoor(x_g, x_d);
  d->getLYcoor(yb, yh);

  /* calcul de la largeur effective */
  largeur=((x_max-x_min)/(x_d-x_g))*d->largeur;

  /* calcul de la hauteur des traits de graduation	*/
  d_y = (0.5 //1.5
         * hauteurChar)*(yh-yb)/d->hauteur;

  /* calcul du nombre maximal de chiffres affichables */
  /* un nombre prend largeurChar * nb_chiffres + 15 d'espace   	   */
  /* on va afficher un nombre sur increment.			   */
  if ( (fabs(increment)<EPSILON) && (x_max!=x_min) ) {
    increment = fabs(v_max-v_min) * (largeurChar * nb_chiffres + 25.) / largeur;
    // On s'arrange pour avoir un increment qui soit
    //   de la forme : un chiffre (1, 2 ou 5) suivi
    //   de zros. nc est le type du dernier chiffre significatif
    // 1 on passe de xxxn000 a xxx(n+1)000
    // 2 on passe de xxxn000 a xxx(n+2)000 ...
    int nz; // nombre de zros
    float v;
    v=log10(increment);
    nz=v;
    v-=nz;
    if (v<log10(2)) nc=2;
    else if (v<log10(5)) nc=5;
    else { nc=1; nz++; }
    increment=nc*pow(10,nz);
  }
  v =((int)(v_min / increment)) * increment;
  if ( fabs(v_min-v) > 0.01) {
    // ajustement au premier multiple de increment
    //    afficher
    if (v_max>v_min) while (v<v_min) v+=increment;
    else while (v>v_min) v-=increment;
  }
  //
  subGrad = (nc == 1) ? 10 : ((nc == 5) ? 5 : 2);

  do {
    SIZE size;
    int DXtext;
    sprintf(strnb, format, v);
    x = (v-v_min) / (v_max-v_min) * (x_max-x_min) + x_min;
    DXtext = d->LXtoDX(x)+2;
    GetTextExtentPoint(d->hDCcomp,strnb,strlen(strnb),&size);
    if (DXtext - size.cx / 2 > 0 && DXtext + size.cx / 2 < d->largeur)
      d->DTextOut(DXtext - size.cx / 2, d->LYtoDY(y_axe - d_y) - 1, strnb);
    d->Lline2(x, y_axe - d_y ,x , y_axe);
    for (int i = 1; i < subGrad; i++) {
      float vv = v - (increment / subGrad) * i;
      x = (vv - v_min) / (v_max - v_min) * (x_max - x_min) + x_min;
      if (vv > v_min) d->Lline2(x, y_axe - d_y / 2, x , y_axe);
    }

    if (v_max > v_min) v += increment;
    else v -= increment;
  } while (   ((v_max > v_min) && (v <= v_max + EPSILON))
	        || ((v_max < v_min) && (v >= v_max + EPSILON)));

  // Displays the subgraduations beyong the last figures displayed.
  for (int i = 1; i < subGrad; i++) {
    float vv = v - (increment / subGrad) * i;
    x = (vv - v_min) / (v_max - v_min) * (x_max - x_min) + x_min;
    if (vv > v_min) d->Lline2(x, y_axe - d_y / 2, x , y_axe);
  }
}

//
// Compute a nice incr for a vertical scale, nice in the sense that the space
// between two lines is acceptable.
float niceVertIncr(float vmin, float vmax, int charHeight, int *numberOfDigits, d_planche *d)
{
  float increment = fabs(vmax - vmin) * (charHeight + 25) / d->hauteur;
  // On s'arrange pour avoir un increment qui soit
  //   de la forme : un chiffre (1, 2 ou 5) suivi
  //   de zros
  int nc   // 1er chiffre
      ,nz; // nombre de zros
  float v;
  v = log10(increment);
  nz = v;
  v -= nz;
  if (v < log10(2)) nc = 2;
  else if (v < log10(5)) nc = 5;
  else {
   nc = 1;
   nz++;
  }
  *numberOfDigits = nc;
  return(nc * pow(10, nz));
}

/*
 * Cette fonction trace un axe vertical simple.
 */
void axe_ySimple(
	float y_min,     // extremites de l'axe
	float y_max,
	float v_min,     // valeurs  afficher aux extremites
	float v_max,
	float x_axel,	 // axis abscissa at left
	float x_axer,	 // axis abscissa at right
   int charHeight,
   int charWidth,
	COLORREF couleur,
	d_planche *d)    // fenetre d'affichage
{
  float xb, xh, y, d_x;

  d->getLXcoor(xb, xh);
  SetBkMode(d->hDCcomp,TRANSPARENT);
  SetTextAlign(d->hDCcomp,TA_LEFT|TA_TOP);
  MgiHue(couleur, d);
  d->Lline2(x_axel, y_min, x_axel, y_max);
  d->Lline2(x_axer, y_min, x_axer, y_max);
  d->getLYcoor(y, y);

  d_x = ((float)(charHeight))/d->largeur*(xh-xb);
  d->SetPlancheFont(queryFont("arial", charWidth, charHeight, 900));

  d->DTextOut(d->LXtoDX(x_axel - 1.5 * d_x) , d->hauteur / 2, "[dB]");
}

/*
 * Cette fonction trace un axe vertical et la graduation correspondante.
 */
void axe_y(
	float y_min,     // extremites de l'axe
	float y_max,
	float v_min,     // valeurs  afficher aux extremites
	float v_max,
	float x_axe,	 // abscisse de l'axe
	COLORREF couleur,
	float increment, // difference entre deux graduations
			           // successives. Si cette valeur est
			           // nulle, elle est fixee par le programme.
   bool forceLast,  // Si true on affiche la dernire graduation en la dcalant au besoin.
   bool withoutLine,   // Si true on n'affiche pas la ligne verticale de l'axe
   int yprotect,	  // To disable text output above this limit.
     	char *format,	  // format d'impression des graduations
   int charHeight,   // Height of characters in pixels
   int charWidth,    // Width of characters in pixels
	d_planche *d)    // fenetre d'affichage
{
  int i, nc, subGrad;
  float d_x;		/* distance entre l'axe et la graduation      */
  float xb,xh,y;
  char strnb[MAX_LIGNE];
  float v;
  
  SetBkMode(d->hDCcomp,TRANSPARENT);
  SetTextAlign(d->hDCcomp,TA_RIGHT|TA_BASELINE);
  MgiHue(couleur, d);
  d->SetPlancheFont(queryFont("arial", charWidth, charHeight, 0));
  if (!withoutLine)  d->Lline2(x_axe, y_min, x_axe, y_max);

  /* calcul du nombre de chiffres des nombres affiches	*/
  /* d'abord on recherche le nombre de chiffres s'il est
       donne (il doit l'etre) */
  for (i = 1; !dans(format[i], '0', '9'); i++); 
  
  d->getLXcoor(xb, xh);
  d->getLYcoor(y, y);

  d_x = ((float)(charHeight))/d->largeur*(xh-xb);

  // calcul du nombre maximal de nombres affichables
  //   dans la hauteur
  if (!increment) increment = niceVertIncr(v_min, v_max, charHeight, &nc, d);
  subGrad = (nc == 1) ? 10 : ((nc == 5) ? 5 : 2);

  for (v = (LROUND(v_min/increment))*increment;
	(v_max > v_min)
	? (v <= v_max + EPSILON)
	: (v >= v_max - EPSILON) ;
       v += (v_max>v_min) ? increment : -increment ) {
    float Ly;
    int Dy;
    Ly = (v-v_min)/(v_max-v_min)*(y_max-y_min)+y_min;
    Dy = d->LYtoDY(Ly);
    sprintf(strnb, format, v);

    if (   (Dy > charHeight)
	     && (Dy <= d->hauteur - 1)
	     && (Ly >= min(y_min, y_max) - EPSILON)
	     && (Ly <= max(y_min, y_max) + EPSILON)) {
      if (d->LYtoDY(Ly) - 1 > yprotect) d->DTextOut(d->LXtoDX(x_axe-d_x*0.2) ,d->LYtoDY(Ly)-1, strnb);
      d->Lline2(x_axe-d_x * 0.5, Ly, x_axe, Ly);
    }
    // Cas de la dernire graduation qui ne peut tre affiche et que l'on dplace si ncessaire.
    else if (fabs(v - v_max) < EPSILON && Dy <= charHeight && forceLast) {
    	TEXTMETRIC textMetrics;
      HDC hDC;
      GetTextMetrics(hDC = GetDC(d->HWindow), &textMetrics);
      // SIZE size;
      // GetTextExtentPoint32(, strnb, strlen(strnb), &size);
      d->Lline2(x_axe - d_x * 0.5, Ly - EPSILON, x_axe, Ly);
      // -4 est compltement empirique car Ascent semble etre surestim.
      if (textMetrics.tmAscent - 5 > yprotect)
        d->DTextOut(d->LXtoDX(x_axe - d_x * 0.2) , textMetrics.tmAscent - 5, strnb);
      ReleaseDC(d->HWindow, hDC);
    }
    for (int i = 1; i < subGrad; i++) {
      float vv = v - (increment / subGrad) * i;
      Ly = (vv - v_min) / (v_max - v_min) * (y_max - y_min) + y_min;
      Dy = d->LYtoDY(Ly);
      if (vv > v_min) d->Lline2(x_axe - d_x * 0.2, Ly, x_axe, Ly);
    }
  }

  for (int i = 1; i < subGrad; i++) {
    float vv = v - (increment / subGrad) * i;
    float Ly = (vv - v_min) / (v_max - v_min) * (y_max - y_min) + y_min;
    float Dy = d->LYtoDY(Ly);
    if (vv > v_min) d->Lline2(x_axe - d_x * 0.2, Ly, x_axe, Ly);
  }
}

