/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// blackman.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include "fftlpc.h"

static FLOATbuf blackman(TAILLE_MAX_HAMMING,"blackman");
static long nombre_de_points;

/*
 * Cette fonction initialise une fenetre de Blackman-Harris.
 * Si le pointeur sur le tableau est different de NULL
 * on initialise une fenetre qui n'est pas la fenetre 
 * privee de ce module.
 * Si le pointeur est egal a NULL on initialise la 
 * fenetre blackman (declaree dans ce module).
 */

void initblack(
     FLOATbuf *h,
     long nb_points)
{
  int i;
  float a0 = 0.35875,
        a1 = 0.48829,
        a2 = 0.14128,
        a3 = 0.01168;
  float *pblack;

  if (nb_points>TAILLE_MAX_HAMMING) {
    affiche_bug("initblack(): too many points");
    return;
  }
  
  nombre_de_points = nb_points;	
  pblack= (float *)((h == NULL) ? &blackman[0] : &(*h)[0]);
  for (i = 0; i < nb_points; i++)
    pblack[i] 
        = (float) (a0 - a1 * cos(2.*_PI * i / nb_points) 
		      + a2 * cos(2.*_PI * 2. * i / nb_points) 
		      - a3 * cos(2.*_PI * 3. * i / nb_points));
}

/* donnees short, resultat float */
void black_sf(
     SAMPLEbuf& cnv,
     long cnv_debut,
     FLOATbuf& res,
     FLOATbuf *h,
     long nb_points)
{ int i;
  float *pblack,*pres;
  SAMPLE huge *pcnv;

  pblack= (float *)((h==NULL) ? &blackman[0] : &(*h)[0]);
  pcnv=(SAMPLE huge *)&cnv[cnv_debut];
  pres=(float *)&res[0];

  for (i = 0;
       i< ((h==NULL) ? nombre_de_points : nb_points);
       i++) 
    pres[i] = (float) (pcnv[i] * pblack[i]);
  return;
}

/* donnees float, resultat float */
void black_ff(
     FLOATbuf& cnv,
     FLOATbuf& res,
     FLOATbuf *h,
     long nb_points)
{ int i;
  float *pblack,*pres,*pcnv;

  pblack= (float *)((h==NULL) ? &blackman[0] : &(*h)[0]);
  pcnv=(float *)&cnv[0];
  pres=(float *)&res[0];

  for (i = 0;
       i< ((h==NULL) ? nombre_de_points : nb_points);
       i++)
    pres[i] = pcnv[i] * pblack[i];
  return;
}

