/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier cepstre.cpp
//

#include "common.h"
#pragma hdrstop

#include <math.h>
#include "fftlpc.h"
#include "spectre.h"
#include "fft_mayer.h"

static float *filtre;

/*
 * Initialisation du filtre cepstral.
 */
static void initfiltre(
	int ninf   // dbut du liftrage (en echt)
	, int nsup // fin du liftrage (en echt)
	, int len) // longueur du filtre
{
  float *p=filtre;
  int i,n1,n2;

  n1=max(0,ninf); n2=max(n1,nsup);
  for (i=0; (i<n1)&&(i<len); i++) p[i] = 1.;
  for (; (i<n2)&&(i<len); i++) 
    p[i] = (float) 0.5 * (1. + cos ((double)(_PI * (i-n1+1) / (n2-n1))));
  for (; i<len; i++) p[i] = 0.;
}

//
//  A compact version of cepstral smoothing (will be used in Automatic speech recognition)
//
void cepstralSpectrum(
  short *speech, float *smoothedSpectrum, int nbSamples, int fftOrder, double *hammingWindow, double *lifter, int lifterLength, bool preemphasis)
{
   double *real = new double[fftOrder],
          *spectrum = new double[fftOrder],
          scale;

    if (preemphasis) for (int k = 0; k < nbSamples; k++)
		  real[k] =  (speech[k + 1] - speech[k]) * hammingWindow[k];
    else for (int j = 0; j < nbSamples; j++) real[j] =  speech[j] * hammingWindow[j];

    for (int k = nbSamples; k < fftOrder; k++) real[k] = 0.;

    // First FFT to compute the spectrum
    realfft(fftOrder, real);

    // magnitude spectrum calculation
    spectrum[0] = 10. * log10(SQR(real[0]) + 1.e-06);
    spectrum[fftOrder / 2] = 10. * log10(SQR(real[fftOrder / 2]) + 1.e-06);
    for (int k = 1; k < fftOrder / 2; k++)
      spectrum[k] = 10. * log10(SQR(real[k]) + SQR(real[fftOrder - k]) + 1e-06);
    for (int k =  fftOrder / 2 + 1; k < fftOrder; k++)        // copy the first half onto the second half
      spectrum[k] = spectrum[fftOrder - k];

    //  Second FFT to compute the cepstrum
    realifft(fftOrder, spectrum);
    scale = 1. / fftOrder;
    for (int i = 0; i < fftOrder; i++) spectrum[i] *= scale;

    // Liftering and completion to 0
    for (int i = 1; i < lifterLength; i++) {
      spectrum[i] *= lifter[i];
      spectrum[fftOrder - i] *= lifter[i];
    }
    for (int i = lifterLength; i < fftOrder / 2; i++) spectrum[i] = spectrum[fftOrder - i] = 0.;
    spectrum[fftOrder / 2] = 0.;

    // Last FFT to get the smoothed cepstrally smoothed spectrum
    realfft(fftOrder, spectrum);

    // Copy the cepstrally smoothed spectrum
    for (int i = 0; i < fftOrder / 2; i++) smoothedSpectrum[i] = spectrum[i];
    delete []spectrum;
    delete []real;
}

#ifdef PourTestercepstralSpectrum
/*
 * cette fonction calcule les cepstres sur tout un tableau.
 * Le filtrage est fait avec un cosinus pour se raccorder continuement.
 */
/* Cette fonction peut servir de prototype pour
   un cepstre d'un ordre different. Seuls les noms
   des FFT doivent etre changes. */
   
int cepstre(
	FLOATbuf **res,  // les resultats
	SAMPLE *cnv, // signal d'entree
	long cnv_debut,
	int nb_cepstre,	// nombre de cepstres a calculer
 	long decal, 	// decalage entre deux cepstres
   int fen_signal_, // taille de la fenetre signal
	int fen_fft1, // 1re fenetre fft (avant liftrage)
   int fen_fft2, // 2me fenetre fft (aprs liftrage)
	int ninf,         // points caractristiques
	int nsup,         //     du filtrage
	int sens, 	// sens de remplissage
  	int avecPreacc,	// VRAI si preaccentuation
   int beforeLastSpectrum)
// en retour, nombre de cepstres calculs
//            qui peut tre diffrent de nb_cepstre
//            en cas d'arrt au clavier
{
  int windowLength, lifterLength, ninf, nsup;

  windowLength = fen_signal_;
  lifterLength = fen_fft1 / 2;

  double *hw = new double[windowLength];
  double *lifter = new double[lifterLength];

  // Initialization of the lifter
  for (int i = 0; i < ninf && i < lifterLength; i++) lifter[i] = 1.;
  for (int i = ninf; i < nsup && i < lifterLength; i++)
    lifter[i] = 0.5 * (1. + cos (M_PI * (i - ninf + 1) / (nsup - ninf)));
  for (int i = nsup; i < lifterLength; i++) lifter[i] = 0.;

  // Initialization of the Hamming window
  for (int k = 0; k < windowLength; k++)
    hw[k] =(float)(0.54 - 0.46 * cos(M_PI * 2. * k / windowLength));

  for (int i = 0; i < nb_cepstre; i++) {
    float *resPar;
    long iecht = cnv_debut+i*decal;
    resPar = (float *) &((**res)[0]);
    cepstralSpectrum(&cnv[cnv_debut+i*decal], resPar, windowLength, fen_fft1, hw, lifter, lifterLength, true);
  }
  delete [] hw;
  delete [] lifter;
  return(nb_cepstre);
}

#endif

/*
 * cette fonction calcule les cepstres sur tout un tableau.
 * Le filtrage est fait avec un cosinus pour se raccorder continuement.
 */
/* Cette fonction peut servir de prototype pour
   un cepstre d'un ordre different. Seuls les noms
   des FFT doivent etre changes. */
   
int cepstre(
	FLOATbuf **res,  // les resultats
	SAMPLE *cnv, // signal d'entree
	long cnv_debut,
	int nb_cepstre,	// nombre de cepstres a calculer
 	long decal, 	// decalage entre deux cepstres
        int fen_signal_, // taille de la fenetre signal
	int fen_fft1, // 1re fenetre fft (avant liftrage)
        int fen_fft2, // 2me fenetre fft (aprs liftrage)
	int ninf,         // points caractristiques
	int nsup,         //     du filtrage
	int sens, 	// sens de remplissage
  	int avecPreacc,	// VRAI si preaccentuation
   int beforeLastSpectrum)
// en retour, nombre de cepstres calculs
//            qui peut tre diffrent de nb_cepstre
//            en cas d'arrt au clavier
{
  int i;
  int fen_signal, filtre_len;

  fen_signal=fen_signal_;
  filtre_len=fen_fft2;

  FLOATbuf tabfft_(max(fen_fft1, fen_fft2) * 2,"tabfft in cepstre()");
  float *tabfft=(float *)&tabfft_[0];
  FLOATbuf huge *resPar;
  FLOATbuf h_(fen_fft1,"h in cepstre()");
  float *h=(float *)&h_[0];

  if (sens == -1) decal = decal * -1;
  filtre=(float *)malloc(filtre_len * sizeof(float));
  initfiltre(ninf, nsup, filtre_len);
  initham(&h_, fen_signal);

  for (i = 0; i < nb_cepstre; i++) {
    int j, k;
    long iecht=cnv_debut+i*decal;
    float xx;

    resPar=res[i];
    /* premphase et fenetrage de hamming */
    if (avecPreacc)
      for (j = 0, k = 0; k < fen_signal; j +=2, k++) {
		  tabfft[j] =  (cnv[iecht+k+1]-cnv[iecht+k])*h[k];
        tabfft[j+1] = 0.;
      }
    else for (j = 0, k = 0; k < fen_signal; j +=2, k++) {
      tabfft[j] =  cnv[iecht+k] * h[k];
      tabfft[j+1] = 0.;
    }
    for (;j<2*fen_fft1;j++) tabfft[j]=0.;

    switch (fen_fft1) {
      case 256: fftc256(tabfft_); break;
      case 512: fftc512(tabfft_); break;
      case 1024: fftc1024(tabfft_); break;
      case 2048: fftc2048(tabfft_); break;
      default: affiche_bug("cepstre() : taille fenetre fft non prvue");
    }

    /* calcul de la premiere moitie du spectre decibel */
    for (j = 0; j <= fen_fft1 / 2; j++) {
      tabfft[j << 1]
        =10.*log10(((xx=tabfft[j<<1]*tabfft[j<<1]
			+tabfft[(j<<1)+1]*tabfft[(j<<1)+1])
		    !=0) ? xx : 1.e-6);
      tabfft[(j << 1) + 1] = 0.;
    }

    /* recopie de la premiere moitie sur la seconde */
    for (j=fen_fft1/2+1, k=fen_fft1/2-1;
	 j<fen_fft1;
	 j++, k--)
    { tabfft[j << 1] = tabfft[k << 1];
      tabfft[(j << 1) + 1] = 0.;
    }

    switch (fen_fft1) {
      case 256: fftic256(tabfft_); break;
      case 512: fftic512(tabfft_); break;
      case 1024: fftic1024(tabfft_); break;
      case 2048: fftic2048(tabfft_); break;
      default: affiche_bug("cepstre() : taille fenetre fft non prvue");
    }

    if (!beforeLastSpectrum) {
    	/* filtrage cepstral (le coefficient 0 n'est pas modifie) */
    	tabfft[1] = 0.;
    	for (j = 1; j < min(filtre_len, fen_fft1); j++) {
    	  tabfft[j << 1] = tabfft[j << 1] * 2. * filtre[j];
     	  tabfft[(j << 1) + 1] = 0.;
      }
    	for (; j < fen_fft2; j++)
     	  tabfft[j << 1] = tabfft[(j << 1) + 1] = 0.;

   	 /* derniere fft */
      switch (fen_fft2) {
    	  case 256: fftc256(tabfft_); break;
     	  case 512: fftc512(tabfft_); break;
     	  case 1024: fftc1024(tabfft_); break;
        case 2048: fftc2048(tabfft_); break;
        default: affiche_bug("cepstre() : taille fenetre fft non prvue");
      }
    }

    /* tranfert des resultats */
    for (j = 0; j < fen_fft2 / 2; j++)
      (*resPar)[j] = tabfft[j << 1];

  }
  free(filtre);
  return i;
}


/*
 * cette fonction calcule les cepstres sur tout un tableau.
 * Le filtrage est fait avec un cosinus pour se raccorder continuement.
 */
/* Cette fonction peut servir de prototype pour
   un cepstre d'un ordre different. Seuls les noms
   des FFT doivent etre changes. */
   
int cepstrePartiel(
	FLOATbuf **res,  // les resultats
	SAMPLEbuf& cnv_, // signal d'entree
	long cnv_debut,
	int nb_cepstre,	// nombre de cepstres a calculer
 	long decal, 	// decalage entre deux cepstres
        int fen_signal_, // taille de la fenetre signal
	int fen_fft1, // 1re fenetre fft (avant liftrage)
   int fen_fft2, // 2me fenetre fft (avant liftrage)
   int fen_fft3, // 3me fentre fft (aprs liftrage)
	int ninf,         // points caractristiques
	int nsup,         //     du filtrage
	int sens, 	// sens de remplissage
  	int avecPreacc,	// VRAI si preaccentuation
   int beforeLastSpectrum)
// en retour, nombre de cepstres calculs
//            qui peut tre diffrent de nb_cepstre
//            en cas d'arrt au clavier
{
  int i;
  int fen_signal, filtre_len;

  fen_signal=fen_signal_;
  filtre_len=fen_fft2;

  FLOATbuf tabfft_(max(fen_fft1, max(fen_fft2, fen_fft3)) * 2,"tabfft in cepstre()");
  float *tabfft=(float *)&tabfft_[0];
  FLOATbuf *resPar;
  FLOATbuf h_(fen_fft1,"h in cepstre()");
  float *h=(float *)&h_[0];
  SAMPLE *cnv=(SAMPLE huge *)&cnv_[0];

  if (sens == -1) decal = decal * -1;
  filtre=(float *)malloc(filtre_len * sizeof(float));
  initfiltre(ninf, nsup, filtre_len);
  initham(&h_, fen_signal);

  for (i = 0; i < nb_cepstre; i++) {
    int j, k;
    long iecht=cnv_debut+i*decal;
    float xx;

    resPar=res[i];
    /* premphase et fenetrage de hamming */
    if (avecPreacc)
      for (j = 0, k = 0; k < fen_signal; j +=2, k++) {
		  tabfft[j] =  (cnv[iecht+k+1]-cnv[iecht+k])*h[k];
        tabfft[j+1] = 0.;
      }
    else for (j = 0, k = 0; k < fen_signal; j +=2, k++) {
      tabfft[j] =  cnv[iecht+k] * h[k];
      tabfft[j+1] = 0.;
    }
    for (;j<2*fen_fft1;j++) tabfft[j]=0.;

    switch (fen_fft1) {
      case 256: fftc256(tabfft_); break;
      case 512: fftc512(tabfft_); break;
      case 1024: fftc1024(tabfft_); break;
      case 2048: fftc2048(tabfft_); break;
      default: affiche_bug("specFft() : taille fenetre fft non prvue");
    }

    /* calcul de la premiere moitie du spectre decibel */
    for (j = 0; j <= fen_fft2 / 2; j++) {
      tabfft[j << 1]
        =10.*log10(((xx=tabfft[j<<1]*tabfft[j<<1]
			+tabfft[(j<<1)+1]*tabfft[(j<<1)+1])
		    !=0) ? xx : 1.e-6);
      tabfft[(j << 1) + 1] = 0.;
    }

    /* recopie de la premiere moitie sur la seconde */
    for (j = fen_fft2/2+1, k = fen_fft2/2-1; j <  fen_fft2; j++, k--) {
      tabfft[j << 1] = tabfft[k << 1];
      tabfft[(j << 1) + 1] = 0.;
    }

    switch (fen_fft2) {
      case 256: fftic256(tabfft_); break;
      case 512: fftic512(tabfft_); break;
      case 1024: fftic1024(tabfft_); break;
      case 2048: fftic2048(tabfft_); break;
      default: affiche_bug("cepstre() : taille fenetre fft non prvue");
    }

    if (!beforeLastSpectrum) {
    	/* filtrage cepstral (le coefficient 0 n'est pas modifie) */
    	tabfft[1] = 0.;
    	for (j = 1; j < min(filtre_len, fen_fft3); j++) {
    	  tabfft[j << 1] = tabfft[j << 1] * 2. * filtre[j];
     	  tabfft[(j << 1) + 1] = 0.;
      }
    	for (; j < fen_fft3; j++)
     	  tabfft[j << 1] = tabfft[(j << 1) + 1] = 0.;

   	 /* derniere fft */
      switch (fen_fft3) {
    	  case 256: fftc256(tabfft_); break;
     	  case 512: fftc512(tabfft_); break;
     	  case 1024: fftc1024(tabfft_); break;
        case 2048: fftc2048(tabfft_); break;
        default: affiche_bug("specFft() : taille fenetre fft non prvue");
      }
    /* tranfert des resultats */
    for (j = 0; j < fen_fft3 / 2; j++)
      (*resPar)[j] = tabfft[j << 1];
    }
    else {
    	/* tranfert des resultats */
    	for (j = 0; j < fen_fft2 / 2; j++)
      	(*resPar)[j] = tabfft[j << 1];
    }
  }
  free(filtre);
  return i;
}
////////////////////////////////////////////////
// fonctions prparamtres
//   cepstre256()
//   cepstre512()
//
int cepstre256(
	FLOATbuf **res,   // les resultats
	SAMPLE *cnv,  // signal d'entree
	long cnv_debut,
	int nb_cepstre,	// nombre de cepstres a calculer
 	long decal, 	// decalage entre deux cepstres
	int n1,         // points caractristiques
	int n2,         //     du filtrage
	int sens, 	// sens de remplissage
  	int avecPreacc)	// VRAI si preaccentuation 
{
  return cepstre(res,cnv,cnv_debut,nb_cepstre,decal,256,256,256, n1,n2,sens,avecPreacc, false);
}

int cepstre512(
	FLOATbuf **res,   // les resultats
	SAMPLE *cnv,  // signal d'entree
	long cnv_debut,
	int nb_cepstre,	// nombre de cepstres a calculer
 	long decal, 	// decalage entre deux cepstres
	int n1,         // points caractristiques
	int n2,         //     du filtrage
	int sens, 	// sens de remplissage
  	int avecPreacc)	// VRAI si preaccentuation
{
  return cepstre(res,cnv,cnv_debut,nb_cepstre,decal,512,512,512,n1,n2,sens,avecPreacc, false);
}
