/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier choix_fi
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <io.h>

#include "notetiq.h"
#include "audio.h"
#include "format_f.h"
#include "files.h"

/////////////////////////////////////////////////
// infoFile_set_nom(filename)
//
// donne le nom de filename aux trois fichiers
//   nom, nom_man, nomAnnot
void infoFile_set_noms(infoFile& infoFile,char *filename)
{  char drive[_MAX_DRIVE], dir[_MAX_DIR]
       ,file[_MAX_FNAME], ext[_MAX_EXT];

  _splitpath(filename,NULL,NULL,file,NULL);
  //
  _splitpath(infoFile.nom,drive,dir,NULL,ext);
  _makepath(infoFile.nom,drive,dir,file,ext);
  //
  _splitpath(infoFile.nom_man,drive,dir,NULL,ext);
  _makepath(infoFile.nom_man,drive,dir,file,ext);
  //
  _splitpath(infoFile.nomAnnot,drive,dir,NULL,ext);
  _makepath(infoFile.nomAnnot,drive,dir,file,ext);
}

/////////////////////////////////////////////////
// init_infoFile_read() routine principale
//   init_infoFile_read_non_WAV() sous_routine
//   init_infoFile_read_WAV() sous_routine
//
//   initialise nv, de type infoFile
//     pour init_infoFile_read_non_WAV()
//      ou init_infoFile_read_WAV()
//       infoFile.frequence
//       infoFile.depl
//       infoFile.longueur
//       infoFile.nb_bits
//       infoFile.swap
//     pour init_infoFile_read_WAV(), en plus
//       infoFile.etiqdepl
//       infoFile.etiqlongueur
//       infoFile.notedepl
//       infoFile.notelongueur
//
// init_infoFile_write() routine principale
//   init_infoFile_write_non_WAV() sous_routine
//   init_infoFile_write_WAV() sous_routine
//
//   initialise nv, de type infoFile
//     pour init_infoFile_write_non_WAV()
//       infoFile.frequence (l'appelant devra contrler
//                    que cette frquence est la mme
//                    que celle du signal  crire)
//       infoFile.depl
//       infoFile.nb_bits
//       infoFile.swap
//     par init_infoFile_read_WAV()
//       infoFile.nb_bits
//       infoFile.swap
//         (l'appelant compltera en initialisant
//          lui-mme infoFile.frequence)
//

static bool init_infoFile_read_non_WAV(
   struct infoFile *nv)	// descripteur a completer
// appel:
//   si nv->nom!="" cas normal
//                      renseigne nv selon
//                      le format nv->numfmt
//   si nv->nom=="" ce cas n'est plus utilis
//                      idem sauf nv->longueur
//
// en retour TRUE si le fichier nv->nom existe
{ char *res;
  long valeurs[NBVAR+1];
  int lg_bloc;
  struct entete *fmt;
  int fic;
  long filelength;
  bool ret;

  Wmsg->wr("init_infoFile_read_non_WAV()");
  ret=FALSE;
  if (strlen(nv->nom)!=0)
  { fic = open(nv->nom,O_RDONLY|O_BINARY);
    if (fic < 0) ret=FALSE;
    else
    { filelength=sof(fic);
      close(fic);
      ret=TRUE;
    }
  }

  fmt=Fmt_par[nv->fmt_son].fmt;
	// MBLANC(valeurs) remet  zro le tampon valeurs
  MBLANC(valeurs); valeurs[NBVAR] = filelength;
	// trouve() va chercher l'info dans le format
  lg_bloc = //((res = trouve(fmt->lg_bloc,fic,valeurs))!=0)
	    //? (double)(*(long *)res) :
	    1;
     
  MBLANC(valeurs); valeurs[NBVAR] = filelength;
  nv->frequence = ((res = trouve(fmt->frequence,fic,valeurs))!=0)
                  ? (double)(*(long *)res) : FREQ_STD;
	
  MBLANC(valeurs); valeurs[NBVAR] = filelength;
  nv->nb_bits =(int)(
                ((res = trouve(fmt->nb_bits,fic,valeurs))!=0)
		? *(long *)res : NB_BITS_STD
		    );
     
  if (ret==TRUE)
  { MBLANC(valeurs); valeurs[NBVAR] = filelength;
    nv->longueur = (((res = trouve(fmt->longueur,fic,valeurs))!=0)
                 ? *(long *)res : 0) * lg_bloc;
  }

  MBLANC(valeurs); valeurs[NBVAR] = filelength;
  nv->depl = (((res = trouve(fmt->debut,fic,valeurs))!=0)
             ? *(long *)res : 0) * lg_bloc;
     
  MBLANC(valeurs); valeurs[NBVAR] = filelength;
  nv->swap = ((res = trouve(fmt->signal_swap,fic,valeurs))!=0)
		? ( strcmp(res,"oui")
		    ? SWAPINFILE : !SWAPINFILE )
		: !SWAPINFILE;
	// attention, les octets dans la mmoire
	// sont naturellement swaps.
	//   swap="oui" dans les fichiers format se
	// traduit donc ici par (swap==SWAPINFILE)
  return ret; 
}
static bool init_infoFile_write_non_WAV(
   struct infoFile *nv)	// descripteur a completer
// en retour TRUE si ok
{ char *res;
  long valeurs[NBVAR+1];
  int lg_bloc;
  struct entete *fmt;
  int fic=0; // pas utilis rellement
  long filelength=0;
  bool ret;

  Wmsg->wr("init_infoFile_write_non_WAV()");
  ret=TRUE;

  fmt=Fmt_par[nv->fmt_son].fmt;
	// MBLANC(valeurs) remet  zro le tampon valeurs
  MBLANC(valeurs); valeurs[NBVAR] = filelength;
	// trouve() va chercher l'info dans le format
  lg_bloc = //((res = trouve(fmt->lg_bloc,fic,valeurs))!=0)
	    //? (double)(*(long *)res) :
	    1;
     
  MBLANC(valeurs); valeurs[NBVAR] = filelength;
  nv->frequence = ((res = trouve(fmt->frequence,fic,valeurs))!=0)
                  ? (double)(*(long *)res) : FREQ_STD;
	
  MBLANC(valeurs); valeurs[NBVAR] = filelength;
  nv->nb_bits =(int)(
                ((res = trouve(fmt->nb_bits,fic,valeurs))!=0)
		? *(long *)res : NB_BITS_STD
		    );
     
  MBLANC(valeurs); valeurs[NBVAR] = filelength;
  nv->depl = (((res = trouve(fmt->debut,fic,valeurs))!=0)
             ? *(long *)res : 0) * lg_bloc;
     
  MBLANC(valeurs); valeurs[NBVAR] = filelength;
  nv->swap = ((res = trouve(fmt->signal_swap,fic,valeurs))!=0)
		? ( strcmp(res,"oui")
		    ? SWAPINFILE : !SWAPINFILE )
		: !SWAPINFILE;
	// attention, les octets dans la mmoire
	// sont naturellement swaps.
	//   swap="oui" dans les fichiers format se
	// traduit donc ici par (swap==SWAPINFILE)
  return ret; 
}

//////////////////////////////////////////////
// classe read_WAV_datas
//
// read_WAV_datas est la classe  utiliser
//   pour lire un fichier .WAV
//
// extrait les informations de format du fichier  lire,
// et permet d'aller lire directement plus tard les
// donnes son, l'tiquetage et les annotations.
//
// mode d'emploi pour lire un fichier .WAV
//     crer un objet read_WAV_datas (appelons-le obj)
//         read_WAV_datas obj;
//             // il ne se passe rien de spcial
//     obj.SetFilename(filename);
//             // les donnes de format du fichier 
//             // lire sont mises dans obj
//             // On peut les recopier ailleurs et
//             // dtruire tout de suite obj qui n'est
//             // plus utile.
//     lire le fichier filename par les procdures
//         standard fopen, fread, fwrite, fseek, fclose ...    
//     dtruire obj si ce n'est pas dj fait. 
//
class read_WAV_datas
{ public:
    STRING filename;
    unsigned int frequence;
    int nb_bits;
    int swap;
    long depl,    // 1er octet du signal dans
                  //       le fichier
         longueur,   // nombre d'octets du signal
         etiqdepl,   // depl et longueur de
         etiqlongueur, // l'tiquetage 
         notedepl,   // depl et longueur des
         notelongueur; // annotations 

    read_WAV_datas() {}
    int SetFilename(char *filename); // lit le "format"
    void error(char *text);
};
	// NB: swap est mis ici par homognt avec
	// le reste de WinSnoori. Il est ici toujours
	// gal  SWAPINFILE (dans le cas des
	// fichiers .WAV, octets toujours swaps
	// dans les fichiers, et jamais en mmoire).

int read_WAV_datas::SetFilename(char *filename)
// 0 en retour si ok
{ int i,ret=0;
  PCMWAVEFORMAT waveFormat;
  MMCKINFO ck;
  HMMIO hmmio;
  char proc[]="read_WAV_datas::SetFilename()";
  STRING fichin;

  strcpy(this->filename,filename);
  strcpy(fichin,filename); 
  if ( (hmmio=mmioOpen(filename,NULL,MMIO_READ))
	==NULL ) { error(ml(3201,"fichier introuvable"));
		   return 1;
                 } 

  // chunk "RIFF"
  if (ret==0)
  { ck.fccType=mmioFOURCC('W','A','V','E');
    if ( mmioDescend(hmmio,&ck,NULL,MMIO_FINDRIFF)
	 !=0 ) { error(ml(3202,"Ce n'est pas un fichier .WAV,\n\
car il ne contient pas de chunk RIFF"));
		 ret=1;
               }
  }

  // chunk "fmt "
  if ((ret==0)&&(mmioSeek(hmmio,12,SEEK_SET)==-1)) ret=1;
  if (ret==0)
  { ck.ckid=mmioFOURCC('f','m','t',' ');
    ck.fccType=0;
    if (erboxmmioDescend(3,proc,fichin
	,mmioDescend(hmmio,&ck,NULL,0)) 
	!=0) ret=1;
  }
  if (ret==0)
  { // lire les datas du chunk "fmt "
    if (erboxmmioRead(4,proc,fichin
	,mmioRead(hmmio,(HPSTR)&waveFormat,16))
        !=0) ret=1;
  }
  if (ret==0)
  { // controles sur le chunk "fmt "
    if (   (waveFormat.wf.wFormatTag!=WAVE_FORMAT_PCM)
         ||(waveFormat.wf.nChannels!=1)
	   //waveFormat.wf.nSamplesPerSec
           //waveFormat.wf.nAvgBytesPerSec
           //waveFormat.wf.nBlockAlign
         ||(  (waveFormat.wBitsPerSample!=16) )
	    //&&(waveFormat.wBitsPerSample!=8) )
       )
    { erboxmmio(5,proc,fichin
	    ,ml(3203,"mauvais format de fichier .WAV"));
      ret=1;
    }
    frequence=(unsigned int)waveFormat.wf.nSamplesPerSec;
    nb_bits=waveFormat.wBitsPerSample;
    swap=SWAPINFILE;
  }

  // chunk "data"
  longueur=0;
  if ((ret==0)&&(mmioSeek(hmmio,12,SEEK_SET)==-1)) ret=1;
  if (ret==0)
  { ck.ckid=mmioFOURCC('d','a','t','a');
    if ((i=erboxmmioDescend(6,proc,fichin
	  ,mmioDescend(hmmio,&ck,NULL,MMIO_FINDCHUNK)))
	  !=0) ret=1;
    depl= (i==0) ? ck.dwDataOffset : 0;
    longueur= (i==0) ? ck.cksize : 0;
  }

  // chunk "word"
  notelongueur=0;
  if ((ret==0)&&(mmioSeek(hmmio,12,SEEK_SET)==-1)) ret=1;
  if (ret==0)
  {  ck.ckid=mmioFOURCC('w','o','r','d');
    i=mmioDescend(hmmio,&ck,NULL,MMIO_FINDCHUNK);
    notedepl= (i==0) ? ck.dwDataOffset : 0;
    notelongueur= (i==0) ? ck.cksize : 0;
  }

  // chunk "phon"
  etiqlongueur=0;
  if ((ret==0)&&(mmioSeek(hmmio,12,SEEK_SET)==-1)) ret=1;
  if (ret==0)
  { ck.ckid=mmioFOURCC('p','h','o','n');
    i=mmioDescend(hmmio,&ck,NULL,MMIO_FINDCHUNK);
    etiqdepl= (i==0) ? ck.dwDataOffset : 0;
    etiqlongueur= (i==0) ? ck.cksize : 0;
  }

  //
  if (erboxmmioClose(7,proc,fichin
		    ,mmioClose(hmmio,0))
	!=0) ret=1;

  return ret;
}
void read_WAV_datas::error(char *text)
{ char ch[200];

  strcpy(ch,filename);
  strcat(ch,"\n");
  strcat(ch,text);
  affiche_erreur(ch);
}

bool init_infoFile_read_WAV(struct infoFile *nv)
// complte la structure nv o seul le nom de
// fichier est donn  l'appel
//
// TRUE en retour si ok
{ read_WAV_datas rWd;

  Wmsg->wr("init_infoFile_read_WAV()");
  if (rWd.SetFilename(nv->nom)!=0) return FALSE;

  nv->frequence=rWd.frequence;
  nv->nb_bits=rWd.nb_bits;
  nv->swap=rWd.swap;

  nv->depl= rWd.depl;
  nv->longueur= rWd.longueur;
  nv->notedepl= rWd.notedepl;
  nv->notelongueur= rWd.notelongueur;
  nv->etiqdepl= rWd.etiqdepl;
  nv->etiqlongueur= rWd.etiqlongueur;

  return TRUE;
}
static bool init_infoFile_write_WAV(struct infoFile *nv)
// complte la structure nv o seul le nom de
// fichier est donn  l'appel
//
// TRUE en retour si ok
{ Wmsg->wr("init_infoFile_write_WAV()");
  nv->nb_bits=NB_BITS;
  nv->swap=SWAPINFILE;
  return TRUE;
}

///////////////////////////////////////////////////
// init_infoFile_read()
//
bool init_infoFile_read(
     struct infoFile *nv)
// complte la structure nv avec les indications
//     sur le format du fichier nv->nom
// l'appelant doit toujours fournir nv->nom
//     et aussi nv->fmt_son si le fichier est non WAV
// retour  TRUE ok,  FALSE problme
{
  if (isWAV(nv->nom))
	return init_infoFile_read_WAV(nv);
  else
   return init_infoFile_read_non_WAV(nv);
}

///////////////////////////////////////////////////
// init_infoFile_write()
//
bool init_infoFile_write(
     struct infoFile *nv)
// complte la structure nv avec les indications
//     sur le format du fichier nv->nom
// l'appelant doit toujours fournir nv->nom
//     et aussi nv->fmt_son si le fichier est non WAV
// retour  TRUE ok,  FALSE problme
{
	if (isWAV(nv->nom))
   	return init_infoFile_write_WAV(nv);
  	else
   	return init_infoFile_write_non_WAV(nv);
}

