/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include "class_sig.h"
#include "outils.h"
#include "signal.h"
#include "spectre.h"
#include "fftlpc.h"
#include "files.h"
#include "audio.h"
#include "wsno.h"

DEFINE_RESPONSE_TABLE1(WSignal,TFrameWindow)
  EV_WM_SIZE,
  EV_CHILD_NOTIFY_ALL_CODES(101,IDscrollbar),
  EV_COMMAND(MENUZoomout, CMzoomOut),
  EV_COMMAND(MENUZoomin, CMzoomIn),
  EV_COMMAND(MENUPlay, CMplay),
END_RESPONSE_TABLE;

//  EV_COMMAND(MENUSpectro,CMSpectro),

WSignal::WSignal(PTWindowsObject AParent, char *title) : TFrameWindow(AParent, title)
{
  this->SetIcon(GetApplication(), IDI_ICON1);

  cts.modeMemoire = TOUT;
  cts.d_a = 0;
  cts.tf = 0;
  cts.f_a = MAXECHTECR -1;
  cts.d_e = 0;
  cts.f_e = 0;
  cts.d_es = cts.d_e;
  cts.f_es = 0;
  cts.debRegion = INFINI_L;
  cts.finRegion = -INFINI_L;
  cts.xDebRegion = -INFINI_I;
  cts.xFinRegion = INFINI_I;
  cts.posBarre = -INFINI_I;
  cts.xBarre = -INFINI_I;

  sig     = new SAMPLEbuf();
  wspectr = new Wdonnees_spectro(this, &cts, sig);
  scroll  = new  TScrollBar(this,101, 0, 0, 100, 16, true);
}

void WSignal::SetupWindow()
{
  int y, h, hscroll, yscroll;
  RECT rect;
  TFrameWindow::SetupWindow();

  ::GetClientRect(HWindow,&rect);
  y = rect.bottom;
  yscroll = 16 - 1;
  hscroll = 16 ;
}

void WSignal::EvSize(uint sizeType , TSize &size) //RTMessage)
{
  RECT rect;

  Attr.H=maxi(100,Attr.H);
  Attr.W=maxi(100,Attr.W);
  ::GetClientRect(HWindow,&rect);
  //

  wspectr->MoveWindow(0, 0, rect.right, rect.bottom - 16, false);
  scroll->MoveWindow(0, rect.bottom - (16 - 1), rect.right, 16, true);
  afficherSpectro();
  /* ::MoveWindow(wf1f2->HWindow,0,0
  		 ,rect.right,rect.bottom,FALSE);
  affv(wf1f2);  */
  ::InvalidateRect(HWindow,NULL,TRUE);
}

void WSignal::CMplay()
{
  if (    cts.debRegion >= cts.finRegion
      || !dans(cts.debRegion, cts.d_e, cts.f_e)
      || !dans(cts.finRegion, cts.d_e, cts.f_e))
    play_n( &((*sig)[_ie(cts.d_es)]),
    		  cts.f_es - cts.d_es + 1,
           en_cours.frequence, 1);
  else play_n(& ((*sig)[_ie(cts.debRegion)]),
              cts.finRegion - cts.debRegion + 1,
              en_cours.frequence, 1);
}

void WSignal::IDscrollbar(WPARAM)
{
  int loval,hival,oldpos;
  long echt;

  scroll->GetRange(loval, hival);
  oldpos = scroll->GetPosition();
  echt = (cts.tf - 1) *( (oldpos-(float)loval) / (hival-(float)loval) );
  allerA(echt);   // dans le signal du buffer
}

bool WSignal::CanClose()
{
  return true;
}



WSignal::~WSignal()
{
  delete wspectr;
  delete scroll;
  delete sig;
  active(PPMENUAllStandard);
  ::DrawMenuBar(PTWtop->HWindow);
}

void WSignal::lecture_signal(char * filename, int where)
{
  strcpy(this->info_en_cours.nom, filename);
  if (init_infoFile_read(&info_en_cours)) {
    cts.tf = info_en_cours.longueur / sizeof(SAMPLE);
    trouveQuoiAfficher(where);
    lireSignal(info_en_cours, info_en_cours.depl + cts.d_e*sizeof(SAMPLE),
   			   *sig, 0, cts.f_e - cts.d_e + 1);
    SetModeZoom(TOUT);
    SetScrollSpectroPosition(0);
  }
}

/*
 * En fonction de la taille du fichier de parole,
 * selectionne la partie qui
 * doit etre affichee. ct.tf doit, bien sur, etre connu.
 */
void WSignal::trouveQuoiAfficher(int where)
{
  cts.d_a = where;
  cts.f_a = where + MAXECHTECR -1;
  cts.d_e = where;
  cts.f_e = min(cts.f_a, cts.tf - 1);
  SetModeZoom(TOUT);
  sig->newsize(cts.f_e - cts.d_e + 1);
  cts.d_es = cts.d_e;
  cts.f_es=min(cts.f_e, cts.f_a);
}

void WSignal::afficherSpectro()
{
  spectrogram(cts.d_es, cts.f_es, ON_NETTOIE);
}

void WSignal::spectro_GetParams(int mode)
// prpare dans sps les paramtres de calcul des spectres
//   ceux communs  tous
//   + fen_fft lorsqu'il existe
{
  paramsp *p;

  spsp.mode=mode;
  spsp.shiftPixel=0;
  switch (mode) // pointeur sur paramtres
  { case FFT_L : p=&psp_FFTL;   break;
    case FFT_E : p=&psp_FFTE;   break;
    case CEP :   p=&psp_LisCep; break;
    case LPC :   p=&psp_LPC;    break;
  }
  switch (mode) // fen_fft
  { case FFT_L : spsp.fen_fft=psp_FFTL.fen_fft; break;
    case FFT_E : spsp.fen_fft=psp_FFTE.fen_fft; break;
    default:  spsp.fen_fft=256; break;
  }
  switch (mode) // fen_signal
  { case FFT_L :
    case FFT_E :
      // il faut au moins 1 echantillon pour calculer une transformee de Fourier...
      spsp.fen_signal=max(1,min(spsp.fen_fft
                         ,(int)enEcht(p->fen_signal)));
      break;
    case CEP :
      spsp.fen_signal=min(512,(int)enEcht(p->fen_signal));
      spsp.fen_fft = (spsp.fen_signal > 256) ? 512 : 256;
      break;
    case LPC :
      spsp.fen_signal=(int)enEcht(p->fen_signal);
      break;
  }
  // paramtres communs
  spsp.depl= max(1, enEcht(p->depl));
  // On verifie quelques valeurs pour eviter les erreurs dues
  // a l'utilisateur.
  spsp.fen_signal = max(1, spsp.fen_signal);
  // For all analyses but LPC, the FFT window must be longer than
  // the number of samples (fen_signal).
  if (mode != LPC && spsp.fen_fft < spsp.fen_signal)
  	 affiche_erreur("Error in the determination of fen_fft in spectro_GetParams");
  spsp.shiftPixel = max(1, p->shiftPixel);
  spsp.isPreac=p->isPreac;
  spsp.silence_spectro=p->silence_spectro;
  spsp.max_spectro=p->max_spectro;
}

//
// cette procedure calcule tout ou partie du spectrogramme
//								
void WSignal::spectrogram(
	long premEcht,	// echantillon de debut de calcul fft
	long derEcht,	// echantillon de fin de calcul de fft
	int clear)	// option de nettoyage avant de visualiser les spectros
// parametre en sortie : les spectres dans spsp
// parametres en entree : tous les autres
{
  long premEchtAff = cts.d_a, derEchtAff = cts.f_a;
  int mode;	// type de calcul
  SpectroDisplay display;

  mode=en_cours.mode;

  display.type = WINDOW;
  display.ws = wspectr;
  wspectr->setLXcoor(premEchtAff, derEchtAff);
  wspectr->setLYcoor(0.,NB_CANAUX);

  // Chargement de la palette de niveau de gris
  switch (mode)
  { case FFT_E : grisSpectro(1.7); break;
    default: grisSpectro(1.0); break;
  }

  // extraction des paramtres de calcul
  spectro_GetParams(mode);

  // calcul global des spectres si besoin
  switch (mode) {
    case LPC:
    case FFT_L:
    case FFT_E:
    case CEP:
      spectroFin(*sig, premEcht, derEcht, mode, cts, display, clear);
      break;
  }
  //spectro_axes_(fret, premEchtAff, derEchtAff, fenetre);
}


// Set the mode for displaying the signal (in the form of
// a spectrogram).
void WSignal::SetModeZoom(int mode)
{
  WORD c=MF_CHECKED, u=MF_UNCHECKED;
  cts.modeMemoire = mode;
  CheckMenuItem(::GetMenu(HWindow), MENUZoomin, (mode==ZOOM)?c:u);
  CheckMenuItem(::GetMenu(HWindow), MENUZoomout,(mode!=ZOOM)?c:u);
}

// Get zoom parameters
void WSignal::CMzoomIn()
{
  if (  (cts.debRegion >= cts.finRegion)
      ||!dans(cts.debRegion, cts.d_e, cts.f_e)
      ||!dans(cts.finRegion, cts.d_e, cts.f_e))
  { affiche_erreur(ml(901,"La rgion  zoomer est mal dfinie"));
    return;
  }


  cts.finRegion = cts.finRegion - 1;	// la marque utilisateur marque la
                                //   fin du secteur de fin
  cts.finRegion = max(cts.finRegion, cts.debRegion+10);
  cts.finRegion = min(cts.finRegion, cts.f_e);
  cts.debRegion = min(cts.debRegion, cts.finRegion-10);
  cts.debRegion = max(cts.debRegion, 0);
  execZoom(cts.debRegion, cts.finRegion);
}

// Displays the zoom
void WSignal::execZoom(
  int d,
  int f)
{
  souris(XC_watch);
  cts.d_es = cts.d_a = max(d, cts.d_e);
  cts.f_a = min(f, cts.f_e);
  cts.f_es = min(cts.f_a, cts.f_e);
  SetModeZoom(ZOOM);

  wspectr->clear_fenetre();
  spectrogram(cts.d_es, cts.f_es, ON_NETTOIE);
  cts.debRegion = cts.finRegion = -1;
  SetScrollSpectroPosition(cts.d_a);
  souris(XC_top_left_arrow);
}

//
// Suppression du zoom.
//
void WSignal::CMzoomOut()
{
  long nbecht; // nombre d'echt de l'cran

  souris(XC_watch);

  cts.debRegion = cts.d_a;
  cts.finRegion = cts.f_a;
  nbecht = MAXECHTECR;

  SetModeZoom(TOUT);
  // on prvoit d'abord le nouvel cran avec
  //   la rgion au centre
  //   et de largeur 2 secondes
  cts.d_a = (cts.debRegion + cts.finRegion) / 2 - nbecht / 2;
  cts.f_a = cts.d_a + nbecht -1;
  // puis on dcale  gauche s'il dpasse  droite
  if (cts.f_a > cts.f_e) {
    cts.f_a = cts.f_e;
    cts.d_a = cts.f_a - nbecht + 1;
  }
  // enfin on dcale  droite s'il dpasse  gauche
  if (cts.d_a < cts.d_e) {
    cts.d_a = cts.d_e;
    cts.f_a = cts.d_a + nbecht - 1;
  }
  //
  cts.d_es = cts.d_a;
  cts.f_es = min(cts.f_a, cts.f_e);

  wspectr->clear_fenetre();
  spectrogram(cts.d_es, cts.f_es, ON_NETTOIE);
  SetScrollSpectroPosition(cts.d_a);
  souris(XC_top_left_arrow);
}

void WSignal::SetScrollSpectroPosition(long echt)
{
  int loval,hival,ipos;
  long lpos;

  scroll->GetRange(loval, hival);


  lpos = loval + LROUND( (echt/(float)cts.tf)*(hival-(float)loval) );
  ipos =( int)max(loval,min(hival,lpos));
  scroll->SetPosition(ipos);
  //
  // amplitudes de dfilement
  //   petite : 1/10me d'cran
  //   grande : un demi cran
  scroll->LineMagnitude = max(1,(int)min(6000,(hival-(float)loval)*((cts.f_a-cts.d_a)/10.)/cts.tf ));
  scroll->PageMagnitude = max(1,(int)min(32000,(hival-(float)loval)*((cts.f_a-cts.d_a)/2.)/cts.tf ));
}

void WSignal::allerA(
	long echt) // numero de l'echantillon ou l'on
	           //    veut aller.
{
  struct ctrl_temps ct_; // ancien ct
  bool indaffNvSpectro = false;

  ct_= cts;

  // On ne prend pas en consideration un ordre de
  //   deplacement s'il existe 
  //   deja une demande de confirmation de sauvegarde ou d'abandon des
  //   modifications faites dans l'editeur.
  // pas de deplacement

  cts.d_a = max(0, min(echt, cts.tf - (ct_.f_a - ct_.d_a)));
  cts.f_a = cts.d_a + (ct_.f_a - ct_.d_a);
  SetScrollSpectroPosition(cts.d_a);
  if (ct_.d_a == cts.d_a) return;
  else {
    // affichage
    cts.debRegion = INFINI_L;
    cts.finRegion = -INFINI_L;
    deplaceLecture();
    wspectr->setLXcoor(cts.d_a, cts.f_a);
    wspectr->setLYcoor(0., NB_CANAUX);

     if (  (cts.d_a > ct_.f_a)
	    ||(cts.f_a < ct_.d_a)
	    || indaffNvSpectro) {
     wspectr->clear_fenetre();
     if (autospectro)
       spectrogram(cts.d_es, cts.f_es, ON_NETTOIE);
     else wspectr->spectro_consolider(0,INFINI_I);
    }
    else if (ct_.d_a < cts.d_a)
     	   deplacementGauche(ct_);
    else deplacementDroite(ct_);
  }
}

//
// Lecture d'un morceau de signal de parole
//    lors d'un deplacement,
//    si le morceau n'est pas dj, bien entendu
//
//  l'appel, on garantit que
//            infoEdit.yaDesModifs == FAUX
void WSignal::deplaceLecture()
{
  int ind=0; // !=0 si lireSignal()  faire
  if (cts.d_a < cts.d_e) {
    ind = 1;
    cts.d_e = cts.d_a;
    cts.f_e = min(cts.f_e, cts.d_e + max(MAXECHTECR, cts.f_a - cts.d_a + 1));
  }
  if (cts.f_a > cts.f_e) {
    ind=1;
    cts.f_e = min(cts.f_a, cts.tf - 1);
    cts.d_e = max(cts.d_e, cts.f_e - max(MAXECHTECR, cts.f_a - cts.d_a + 1));
  }
  if (ind != 0) {
    sig->newsize(cts.f_e - cts.d_e + 1);
    lireSignal(info_en_cours, info_en_cours.depl + cts.d_e * sizeof(SAMPLE),
   			   *sig, 0, cts.f_e - cts.d_e + 1);
  }
  cts.d_es = cts.d_a;
  cts.f_es = min(cts.f_e, cts.f_a);
}

void WSignal::deplacementGauche(struct ctrl_temps& temps)
// ancien ct
// dplacement  gauche ... sur l'cran
//   et donc  droite sur le scrollbar
{
  int w,  // largeur de la partie conserve
      h,  // hauteur de la fenetre
      d; // dcalage (vers la gauche)

  d_plancheTmp wspectrTmp(wspectr);

  w = snoscalex(wspectr,temps.f_a);
  h = wspectr->hauteur;
  d = wspectr->largeur - w;

  // copier la zone conserve
  BitBlt(wspectrTmp.hDCcomp,0,0,w,h,wspectr->hDCcomp,d,0,SRCCOPY);
  // ... et la mettre  son emplacement dfinitif
  wspectr->clear_fenetre();
  BitBlt(wspectr->hDCcomp,0,0,w,h,wspectrTmp.hDCcomp,0,0,SRCCOPY);

  // ... idem pour wspectroBis
  BitBlt(wspectrTmp.hDCcomp,0,0,w,h,wspectr->bis->hDCcomp,d,0,SRCCOPY);
  wspectr->bis->clear_fenetre();
  BitBlt(wspectr->bis->hDCcomp,0,0,w,h,wspectrTmp.hDCcomp,0,0,SRCCOPY);

  //
  if (autospectro) {
    int decal;

    // montrer le rsultat provisoire
    CopyArea_Dbw(wspectr,0,0,wspectr->largeur,wspectr->hauteur,0,0,GXcopy);

    decal = sps.fen_signal/2;

    spectrogram(max(cts.d_es, temps.f_a - decal), cts.f_es, ON_LAISSE );
  }
  ::InvalidateRect(wspectr->HWindow, NULL, true);
}

void WSignal::deplacementDroite(
   struct ctrl_temps& temps) // ancien ct
// dplacement  droite ... sur l'cran
//   et donc  gauche sur le scrollbar
{
  int w,  // largeur de la partie conserve
      h,  // hauteur de la fenetre
      d; // dcalage (vers la droite)
  d_plancheTmp wspectrTmp(wspectr);

  d = snoscalex(wspectr, temps.d_a);
  w = wspectr->largeur-d;
  h = wspectr->hauteur;

  // copier la zone conserve
  BitBlt(wspectrTmp.hDCcomp,0,0,w,h,wspectr->hDCcomp,0,0,SRCCOPY);
  // ... et la mettre  son emplacement dfinitif
  wspectr->clear_fenetre();
  BitBlt(wspectr->hDCcomp,d,0,w,h,wspectrTmp.hDCcomp,0,0,SRCCOPY);

  // ... idem pour wspectroBis
  BitBlt(wspectrTmp.hDCcomp,0,0,w,h,wspectr->bis->hDCcomp,0,0,SRCCOPY);
  wspectr->bis->clear_fenetre();
  BitBlt(wspectr->bis->hDCcomp,d,0,w,h,wspectrTmp.hDCcomp,0,0,SRCCOPY);

  if (autospectro) { // montrer le rsultat provisoire
    CopyArea_Dbw(wspectr, 0, 0, wspectr->largeur, wspectr->hauteur, 0, 0, GXcopy);
    spectrogram(cts.d_es, min(cts.f_es, temps.d_a + sps.fen_signal), ON_LAISSE);
  }
  ::InvalidateRect(wspectr->HWindow, NULL, true);
}

