/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier couleur.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <string.h>

HBRUSH hGBrush[NB_COUL_SPEC]; // brosses grises

typedef struct {
  COLORREF c;
  float h,s,v;
  int n;
} ColorRange;

ColorRange essai [] = {
   {0xFFFFFF, 0., 0., 0., 48},       
   {0xCCFFFF, 0., 0., 0., 46},
   {0x40CFFF, 0., 0., 0., 44},      
   {0x467BFA, 0., 0., 0., 40},      
   {0x131FFF, 0., 0., 0., 28},      
   {0x00008C, 0., 0., 0., 22},      
   {0x000040, 0., 0., 0., 16},      
   {0x000030, 0., 0., 0., 12},      
   {0x000000, 0., 0., 0.,1}         
};
#ifdef UIY
ColorRange essai [] = {
   {0xCD00CD, 0., 0., 0., 48},		// Magenta
   {0xFF0000, 0., 0., 0., 46},      // Blue
   {0xD1CE00, 0., 0., 0., 44},   	// bleu turquoise clair    112
   {0x00FF7F, 0., 0., 0., 40},   	// Chartreuse (vert clair brillant)     137
   {0x00FFFF, 0., 0., 0., 28},   	// Yellow                               149
   {0x00D7FF, 0., 0., 0., 22},   	// Gold                                  173
   {0x009AEE, 0., 0., 0., 16},   	// Orange                                223
   {0x0000FF, 0., 0., 0., 12},   	 // Rouge
   {0x00008B, 0., 0., 0.,1}	  		// Rouge brique
};


//   {0xFF00FF, 0., 0., 0.,  1}       // Magenta
ColorRange essai [] = {
   {BLEU, 0., 0., 0., 8},
   {VERT, 0., 0., 0., 8},
   {JAUNE, 0., 0., 0., 8},
//   {ROUGE, 1}
};

ColorRange essai [] = {
   {0x0000FF, 0xFF0000 ,75},   	// Blue
   {0x00CED1, 0xD1CE00 ,37},   	// bleu turquoise clair    112
   {0x7FFF00, 0x00FF7F ,25},   	// Chartreuse (vert clair brillant)     137
   {0xFFFF00, 0x00FFFF ,12},   	// Yellow                               149
   {0xFFD700, 0x00D7FF ,24},   	// Gold                                  173
   {0xEE9A00, 0x009AEE ,50},   	// Orange                                223
   {0xFF0000, 0x0000FF ,33},   		// Rouge
   {0x8B0000, 0x00008B ,1},	  		// Rouge brique
};
#endif
//   {0xB3EE3A, 8},   	// Olive (vert clair)

//added

/////////////////////////////////////////////////
// type du device pour les couleurs
//     statique, dpend du type de device cran
//
static bool DeviceColorsOnly;
static bool DeviceIsPalette;
// il y a 3 types d'cran selon les 2 variables ci-dessus
//   cran 16 couleurs seulement, donc sans griss
//       DeviceColorsOnly=TRUE  & DeviceIsPalette=FALSE
//   cran avec palette, donc avec griss
//       DeviceColorsOnly=FALSE & DeviceIsPalette=TRUE
//   cran sans palette et avec plus que 16 couleurs
//       DeviceColorsOnly=FALSE & DeviceIsPalette=FALSE

/////////////////////////////////////////////////
// validation des griss  l'utilisation
//     paramtre modifiable par l'utilisateur
//     en cours d'excution
bool graymode; // 0  non, utiliser les couleurs
// 1  oui, utiliser les griss
//        (par dgrad de 16 griss)
//        c'est le cas normal

///////////////////////////////
// la palette pour les griss
HPALETTE hpalGris;

static LOGPALETTE *logpalGris;

COLORREF paletteCouleur[NB_COUL_SPEC];
extern void rgb2hsv(int r, int g, int b, float *h_, float *s_, float *v_);
extern void hsv2rgb(float h, float s, float v, int *r_, int *g_, int *b_);

void interpolateColors(
  ColorRange colorData[],
  int nbColorData,
  COLORREF interpolatedColors[],
  int nbInterpolated)
{
  int i, n;

  for (i = 0, n = 0; i < nbColorData; i++) {
    int  b = (0xff0000 & colorData[i].c) >> 16,
         g = (0xff00 & colorData[i].c) >> 8,
         r = 0xff & colorData[i].c;
    rgb2hsv(r, g, b, &colorData[i].h, &colorData[i].s, &colorData[i].v);
  }
  for (i = 0, n = 0; i < nbColorData - 1; i++) {
    int j, dn = colorData[i].n;
    float dh = colorData[i + 1].h - colorData[i].h,
            ds = colorData[i + 1].s - colorData[i].s,
            dv = colorData[i + 1].v - colorData[i].v;
    for (j = 0; j < colorData[i].n; j++, n++) {
      int r, g, b;
      hsv2rgb(colorData[i].h + dh * j / dn, colorData[i].s + ds * j / dn, colorData[i].v + dv * j / dn, &r, &g, &b);
      interpolatedColors[n] =  (b << 16) + (g << 8) + r;
    }
  }
}



//
// affichage d'un rectangle avec un gris
//
void affGris(d_planche *fenetre 
	     ,LPRECT lprect, int ngris){
  FillRect(fenetre->hDCcomp, lprect, hGBrush[ngris]);
}



//
COLORREF CoulFondSpec() { return BLANC; }

// table des couleurs pour cran 16 couleurs
//   mis dans l'ordre de plus en plus sombre
//   pour l'imprimante
COLORREF Pcolorssss[] =
{ BLANC,
  JAUNE,
  GRISclair,
  CYAN,
  VERT,
  GRISsombre,
  JAUNEsombre,
  MAGENTA,
  CYANsombre,
  VERTsombre,
  ROUGE,
  MAGENTAsombre,
  ROUGEsombre,
  BLEU,
  BLEUsombre,
  NOIR
};

COLORREF Pcolors[NB_COUL_SPEC];
// table des griss
COLORREF Pgrises[NB_COUL_SPEC];

////////////////////////////////////////
// Creation des brosses grises
//      l'initialisation
//
void initGrisBrushes(){
  int i;
  
  for (i = 0; i < NB_COUL_SPEC; i++)
    hGBrush[i]=CreateSolidBrush(
				!graymode
				? Pcolors[i]
				: ( DeviceIsPalette
				    ? PALETTEINDEX(i)
				    : ( DeviceColorsOnly
					? Pcolors[i]
					: Pgrises[i] ) ) );
  
}

static void deleteGrisBrushes(){ 
  int i;
  
  for (i = 0; i < NB_COUL_SPEC; i++)
    DeleteObject(hGBrush[i]);
}

static void reinitGrisBrushes(){
  deleteGrisBrushes();
  initGrisBrushes();
}    


static void Pgrises_compute(double gamma){
  int i;
  
  for (i = 0; i < NB_COUL_SPEC; i++){
    BYTE x;
    x //Pgrises[i]
      =max(0,min(0xff,
		 (int)
		 (0xff
		  * (1. - pow((double)i
			      / (NB_COUL_SPEC-1.), gamma)) )));
    Pgrises[i]=x*0x10101L;
  }
}


/////////////////////////////////
// modification des gris
//
void grisSpectro( //modifyGris(
		 double gamma)	/* exposant de x, le niveau de gris
				   etant (x/n) ^ gamma * 255. */
{ 
  int i;
  PALETTEENTRY pe;
  HDC hDC;
  //  static int prevgraymode=-1;
  static float prevgamma=0.;
  
  if (DeviceColorsOnly) return;
  
  if ( fabs(gamma-prevgamma)<0.01 ) return;
  prevgamma=gamma;
  
  Pgrises_compute(gamma);
  
  if (DeviceIsPalette) {
    // cas 256 couleurs : brosses solides grises
    int x;

      //
    for (i = 0; i < NB_COUL_SPEC; i++) {
	   x=(BYTE)(Pgrises[i]);
	   pe.peRed=x;
      pe.peGreen=x;
      pe.peBlue=x;
	   pe.peFlags=0;
	   SetPaletteEntries(hpalGris,i,1,&pe);
	 }
    //
    RealizePalette(wspectro->wsona->hDCcomp);
    hDC=GetDC(wspectro->wsona->HWindow);
    RealizePalette(hDC);
    UpdateColors(hDC);
    ReleaseDC(wspectro->wsona->HWindow,hDC);
  }
  reinitGrisBrushes();
}


//////////////////////////////////////////////////
void reinitgraymode()
{ reinitGrisBrushes();
}    
bool colors16only()
  // en retour
  //   TRUE  : cran 16 couleurs, sans griss
  //   FALSE : cran avec griss possibles 
{ return DeviceColorsOnly;
}
void GetDeviceTypes(){
  HDC hDC;
  HWND hWnd;
  
  hDC=GetDC(hWnd=GetActiveWindow());
 
  DeviceColorsOnly=TRUE;
  DeviceIsPalette=FALSE;
  if ( ( GetDeviceCaps(hDC,RASTERCAPS)
	 &RC_PALETTE)!=0 )
    { uint sp,nr;

    sp= (UINT)GetDeviceCaps(hDC,SIZEPALETTE);
    nr= (UINT)GetDeviceCaps(hDC,NUMRESERVED);
    if ((sp>nr)&&(sp-nr>NB_COUL_SPEC))
      { DeviceColorsOnly=FALSE;
      DeviceIsPalette=TRUE;
      }
    }
  else
    { uint nc;
    
    nc= (UINT)GetDeviceCaps(hDC,NUMCOLORS);
    if (nc>50)
      { DeviceColorsOnly=FALSE;
      DeviceIsPalette=FALSE;
      }
    }
  
  ReleaseDC(hWnd,hDC);
}


/* 
 * Initialisation de la palette de couleurs de Snoori.
 */
void opencoul()
{ 
  int i,j;
  
  GetDeviceTypes();
  //graymode= (colors16only()) ? 0 : 1;
  graymode= (DeviceColorsOnly) ? 0 : 1;  
  Pgrises_compute(1);
  interpolateColors(essai, sizeof(essai) / sizeof(ColorRange), Pcolors, NB_COUL_SPEC);


  /////////////////////////////////////////////
  // initialisation de la palette
  logpalGris = (LPLOGPALETTE)malloc(sizeof(LOGPALETTE)
			  		+ sizeof(PALETTEENTRY) * NB_COUL_SPEC );
  logpalGris->palVersion = 0x300;
  logpalGris->palNumEntries = NB_COUL_SPEC;
  for (i = 0; i < NB_COUL_SPEC; i++)
    { j=(NB_COUL_SPEC-i-1)*(256/NB_COUL_SPEC+1);
    logpalGris->palPalEntry[i].peRed = j;
    logpalGris->palPalEntry[i].peGreen = j;
    logpalGris->palPalEntry[i].peBlue = j;
    logpalGris->palPalEntry[i].peFlags = PC_RESERVED;
    }
  hpalGris=CreatePalette(logpalGris);
  //
  initGrisBrushes();
}

void closecoul()
{ deleteGrisBrushes();
}



