/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier coupe.cpp
//

#include "common.h"       
#pragma hdrstop

#include <stdio.h>
#include <stdlib.h>
#include "snoori.h"
#include "wcoupe.h"
#include "outils.h"
#include "signal.h"
#include "fftlpc.h"
#include "spectre.h"
#include "fmaxspec.h"
#include "polypoint.h"
#include "guiklat.h"

int yaUneFenetreCoupe=0;
long numEchtCoupeSpectrale;
extern Wpiquefor *wpiquefor;  // formant editor window

// paramtres des coupes spectrales
static paramCoupeFFT    p_FFT1,p_FFT2;
static paramCoupeLisCep p_LisCep1,p_LisCep2;
static paramCoupeLPCCEP p_LPCCEP;
static paramCoupeLPC    p_LPC;
static paramCoupeLPCsel p_LPCsel;
static paramCoupeTrueEnv p_TrueEnv;
static paramCoupeCriticalBands p_CriticalBands;
//

class TButtonColor : TFrameWindow
{ public:
    paramCoupe *p;
    TButtonColor(PTWindowsObject AParent,
            int X, int Y, int W, int H, 
	    paramCoupe *p);
	 virtual void Paint(TDC& dc, bool erase, TRect& rect); //HDC,PAINTSTRUCT&);

	 // rponses aux vnements
	 /*
	 virtual void WMLButtonDown(RTMessage)=[WM_FIRST+WM_LBUTTONDOWN];
	 */
	 void EvLButtonDown(uint Modkeys, TPoint& point);
	 DECLARE_RESPONSE_TABLE(TButtonColor);
};
DEFINE_RESPONSE_TABLE1(TButtonColor,TFrameWindow)
  EV_WM_LBUTTONDOWN,
END_RESPONSE_TABLE;

TButtonColor::TButtonColor(PTWindowsObject AParent,
            int X, int Y, int W, int H,
	    paramCoupe *p)
        : TFrameWindow(AParent,"")
{
  this->p=p;
  Attr.X=X;
  Attr.Y=Y;
  Attr.W=W;
  Attr.H=H;
  Attr.Style=WS_VISIBLE | WS_BORDER | WS_CHILD;
}

void TButtonColor::Paint(TDC& dc, bool, TRect&)
{
  TBrush brush(p->color);
  TRect rect;

  ::GetClientRect(HWindow,&rect);
  if (p->isAffichee)
  { rect.right++;
	 rect.bottom++;
	 dc.FillRect(rect,brush);
  }
  else
  { rect.left++;
	 rect.top++;
	 rect.right--;
	 rect.bottom--;
	 dc.FrameRect(rect,brush);
  }
}
void TButtonColor::EvLButtonDown(uint, TPoint&)
{ p->isAffichee=!p->isAffichee;
  ::InvalidateRect(HWindow,NULL,TRUE);
}

static void initCoupe()
// initialise les paramtres des coupes spectrales
{
  p_FFT1.isOpen=FALSE;
  p_FFT1.num=0;
  p_FFT1.color=BLEUsombre;
  p_FFT1.isAffichee=TRUE;
  //
  p_FFT2.isOpen=FALSE;
  p_FFT2.num=1;
  p_FFT2.color=BLEU;
  p_FFT2.isAffichee=TRUE;
  //
  p_LisCep1.isOpen=FALSE;
  p_LisCep1.num=2;
  p_LisCep1.color=VERTsombre;
  p_LisCep1.isAffichee=TRUE;
  //
  p_LisCep2.isOpen=FALSE;
  p_LisCep2.num=3;
  p_LisCep2.color=VERT;
  p_LisCep2.isAffichee=TRUE;
  //
  p_LPC.isOpen=FALSE;
  p_LPC.num=4;
  p_LPC.color=ROUGE;
  p_LPC.isAffichee=TRUE;
  //
  p_LPCCEP.isOpen=FALSE;
  p_LPCCEP.num=5;
  p_LPCCEP.color=ROUGE;
  p_LPCCEP.isAffichee=TRUE;
  //
  p_LPCsel.isOpen=FALSE;
  p_LPCsel.num=6;
  p_LPCsel.color=MAGENTA;
  p_LPCsel.isAffichee=TRUE;

  p_TrueEnv.isOpen=FALSE;
  p_TrueEnv.num=7;
  p_TrueEnv.color=MAGENTAsombre;
  p_TrueEnv.isAffichee=TRUE;
  //
  p_CriticalBands.isOpen=FALSE;
  p_CriticalBands.num=8;
  p_CriticalBands.color=DARKORANGE;
  p_CriticalBands.isAffichee=TRUE;

}

/////////////////////////////////////////
// fentre Wcoupe
//
// la fentre Wcoupe permet d'afficher 6 spectres simultanment:
//     - FFT               2 spectres diffrents
//     - LPC slective     1 spectre
//     - LPC               1 spectre
//		 - lissage cepstral appliqu aux coefficients LPC
//     - lissage spectral  2 spectre diffrents
//
// les commandes offertes  l'utilisateur sont:
//     - donner les paramtres et/ou
//       valider l'affichage pour chacun des 6 spectres
//           (chaque spectre peut tre affich ou non
//            , et ceci indpendemment entre les
//            diffrents spectres)
//         ==> clicker sur le bouton du spectre
//     - demander de l'aide:  1 bouton Help
//     - quitter:  1 bouton Cancel

Wdonnees_coupe *wcoupe;

DEFINE_RESPONSE_TABLE1(Wdonnees_coupe,d_planche)
  EV_WM_SIZE,
  EV_WM_LBUTTONDOWN,
  EV_WM_LBUTTONUP,
  EV_WM_MOUSEMOVE,
END_RESPONSE_TABLE;
Wdonnees_coupe::Wdonnees_coupe(PTWindowsObject AParent)
    : d_planche(AParent)
{
  lastct = (struct ctrl_temps *) NULL;
  lastSig = (SAMPLEbuf *) NULL;
  charHeight = 24;
  charWidth = 10;

}

void Wdonnees_coupe::EvSize(uint sizeType, TSize &size)
{
  d_planche::EvSize(sizeType, size);
  if (size.cx && size.cy) {
    // on initialise les coordonnes de la fentre
    // de manire que :
    //   l'axe des y laisse 28 pixels  gauche
    //   l'axe des x laisse 16 pixels en bas
    // si la fentre est trop petite en x ou en y,
    // l'axe correspondant est supprim
    RECT rect;
	 ::GetClientRect(this->HWindow,&rect);
    this->setLXcoorD(0., fMaxSpectro,
	    				   (rect.right > 100) ? ((int)(charWidth * 5)) : 0,
	    				   rect.right - 2);
    this->setLYcoorD(dBMIN, dBMAX,
	      			   rect.bottom -((rect.bottom > 100) ? ((int) (charHeight * 1.5)) : 0),2);
    if (lastct) affUneCoupe(wcoupe, *lastSig, numEchtCoupeSpectrale, lastct, PTWcoupe->styleAxes);
  }
}

void Wdonnees_coupe::EvLButtonDown(uint, TPoint&)
{
  LButtonDown=true;
}
void Wdonnees_coupe::EvLButtonUp(uint, TPoint&)
{
   LButtonDown=false;
}

void Wdonnees_coupe::EvMouseMove(uint, TPoint& point)
{
  int event_x,event_y;
  float xr, yr;
  char text1[80],text2[80];

  { event_x=point.x;
    event_y=point.y;

    snoiunscalexy(event_x, event_y, &xr, &yr, this);
    sprintf(text1,"%4.1fdB amp",yr);
    sprintf(text2,"%uHz freq",(int)(max(0.,xr)));
    Wact->setXY(text1,text2);
  }
}

Wcoupe *PTWcoupe;

DEFINE_RESPONSE_TABLE1(Wcoupe,TFrameWindow)
  EV_WM_SIZE,
  EV_CHILD_NOTIFY_ALL_CODES(101,IDparaFFT1),
  EV_CHILD_NOTIFY_ALL_CODES(102,IDparaFFT2),
  EV_CHILD_NOTIFY_ALL_CODES(103,IDparaLisCep1),
  EV_CHILD_NOTIFY_ALL_CODES(104,IDparaLisCep2),
  EV_CHILD_NOTIFY_ALL_CODES(113,IDparaLPC),
  EV_CHILD_NOTIFY_ALL_CODES(105,IDparaLPC),
  EV_CHILD_NOTIFY_ALL_CODES(106,IDparaLPCsel),
  EV_CHILD_NOTIFY_ALL_CODES(107,IDparaTrueEnv),
  EV_CHILD_NOTIFY_ALL_CODES(108,IDparaCriticalBands),
  EV_CHILD_NOTIFY_ALL_CODES(110,IDrecalculer),
  EV_CHILD_NOTIFY_ALL_CODES(109,IDdefauts),
  EV_CHILD_NOTIFY_ALL_CODES(111,IDaxes),
  EV_CHILD_NOTIFY_ALL_CODES(112,IDCopyToClipboard),
END_RESPONSE_TABLE;
Wcoupe::Wcoupe() : TFrameWindow(PTWtop,"")
{
  int x,y,w,h,dy;
  int xb,wb;
  char ch[80];
  static ind=0;

  if (ind==0) { initCoupe(); defauts(-1); ind=1; }

  iconified = true;
  PTWcoupe=this;
  ticket=GetTicket();
  wsprintf(ch,"%s %d.",ml(3401,"Coupe spectrale"),ticket);
  Title=strdup(ch);

  Attr.Style=WS_VISIBLE|WS_POPUPWINDOW|WS_CAPTION
		       |WS_THICKFRAME
		       |WS_MINIMIZEBOX
		       |WS_MAXIMIZEBOX;
  Attr.X=0;
  Attr.Y=100;
  Attr.W=540;
  Attr.H=280;
  this->SetIcon(GetApplication(), IDI_ICON1);
  //
  numEchtCoupeSpectrale = -1;
  ct.posBarre=-1;

  pFFT1   =&p_FFT1;
  pFFT2   =&p_FFT2;
  pLisCep1=&p_LisCep1;
  pLisCep2=&p_LisCep2;
  pLPCCEP =&p_LPCCEP;
  pLPC    =&p_LPC;
  pLPCsel =&p_LPCsel;
  pTrueEnv = &p_TrueEnv;
  pCriticalBands = &p_CriticalBands;

  //
  wcoupe=new Wdonnees_coupe(this);

  x=3; y=3; w=139; h=17; dy=20;
  wb=w-h-1; // largeur des boutons spectres
  xb=x+wb+1; // abscisse des boutons couleurs             
  //
  new TButton(this,101,ml(3402,"FFT 1")
		       ,x,y+dy*pFFT1->num,wb,h,FALSE);
  new TButton(this,102,ml(3403,"FFT 2")
		       ,x,y+dy*pFFT2->num,wb,h,FALSE);
  new TButton(this,103,ml(3404,"Lissage Cepstral 1")
		       ,x,y+dy*pLisCep1->num,wb,h,FALSE);
  new TButton(this,104,ml(3405,"Lissage Cepstral 2")
		       ,x,y+dy*pLisCep2->num,wb,h,FALSE);
  new TButton(this,113,"LPC cepstrum"
		       ,x,y+dy*pLPCCEP->num,wb,h,FALSE);
  new TButton(this,105,ml(3406,"LPC")
		       ,x,y+dy*pLPC->num,wb,h,FALSE);
  new TButton(this,106,ml(3407,"LPC Slective")
		       ,x,y+dy*pLPCsel->num,wb,h,FALSE);
  new TButton(this,107,ml(3426,"Enveloppe vraie")
		       ,x,y+dy*pTrueEnv->num,wb,h,FALSE);
  new TButton(this,108,ml(3427,"Bandes critiques")
		       ,x,y+dy*pCriticalBands->num,wb,h,FALSE);
  // attention: les textes des boutons ci-dessus sont rutiliss
  //   dans affuneco.cpp pour Wact->showpourcent()
  //
  new TButtonColor(this,xb,y+dy*pFFT1->num,h,h,pFFT1);
  new TButtonColor(this,xb,y+dy*pFFT2->num,h,h,pFFT2);
  new TButtonColor(this,xb,y+dy*pLisCep1->num,h,h,pLisCep1);
  new TButtonColor(this,xb,y+dy*pLisCep2->num,h,h,pLisCep2);
  new TButtonColor(this,xb,y+dy*pLPCCEP->num,h,h,pLPCCEP);
  new TButtonColor(this,xb,y+dy*pLPC->num,h,h,pLPC);
  new TButtonColor(this,xb,y+dy*pLPCsel->num,h,h,pLPCsel);
  new TButtonColor(this,xb,y+dy*pTrueEnv->num,h,h,pTrueEnv);
  new TButtonColor(this,xb,y+dy*pCriticalBands->num,h,h,pCriticalBands);
  y+=9*dy;
  //
  new TButton(this,110,ml(3408,"Recalculer"),x,y,w,h,FALSE);
  y+=dy;
  new TButton(this,109,ml(3410,"Dfauts"),x,y,w,h,FALSE);
  y+=dy;
  new TButton(this,111,ml(3419,"Axes (normal/simple)"),x,y,w,h,FALSE);
  y+=dy;
  new TButton(this,112,ml(3428,"Copy to clipboard"),x,y,w,h,FALSE);
  //
  styleAxes = NORMAL;

  yaUneFenetreCoupe = 1;
}
void Wcoupe::EvSize(uint, TSize &) //WMSize(RTMessage)
{
  int i;
  RECT rect;

  Attr.H=max(100,Attr.H);
  Attr.W=max(100,Attr.W);
  ::GetClientRect(HWindow,&rect);
  //
  i=145; // position de la fenetre wcoupe
  ::MoveWindow(wcoupe->HWindow,i,0
		 ,rect.right-i,rect.bottom,FALSE);
  ::InvalidateRect(HWindow,NULL,TRUE);
  { // on initialise les coordonnes de la fentre
    // de manire que :
    //   l'axe des y laisse 28 pixels  gauche
    //   l'axe des x laisse 16 pixels en bas
    // si la fentre est trop petite en x ou en y,
    // l'axe correspondant est supprim
    RECT rect;
	 ::GetClientRect(wcoupe->HWindow,&rect);
    iconified = (rect.right == rect.left || rect.bottom == rect.top);
  }
}


bool Wcoupe::CanClose()
{
  yaUneFenetreCoupe=0;
  wspectro->wsona->MoveBarre(-1);
  if (yaUneFenetrePiquefor && wpiquefor) wpiquefor->wspectr->MoveBarre(-1);
  if (!yaUneFenetrePiquefor) active(PPMENUAllStandard);
  else {
    active(MENUAffCoupe);
    active(MENUZoomin);
    active(MENUZoomout);
    active(MENUEcoute1Fois);
  }
  return true;
}

void Wcoupe::IDrecalculer(WPARAM) //RTMessage)
{
  if (iconified) return;
  if (wcoupe->lastct) affUneCoupe(wcoupe, *wcoupe->lastSig, numEchtCoupeSpectrale, wcoupe->lastct, PTWcoupe->styleAxes);
}

void Wcoupe::defauts(int n)
// initialise ou rinitialise les paramtres
//   spcifiques de
//       toutes les coupes possibles, si n <0
//       la coupe n, si n>=0
{
  if ((n<0)||(n==0))
  { p_FFT1.isPreac=TRUE;
    p_FFT1.moduleIsDisplayed=TRUE;
    p_FFT1.niv=0.;
    p_FFT1.fen_signal=4;
    p_FFT1.fen_fft=9;
	 if (p_FFT1.isOpen) ::InvalidateRect(p_FFT1.PTW->HWindow,NULL,TRUE);
  }
  if ((n<0)||(n==1))
  { p_FFT2.isPreac=TRUE;
    p_FFT2.moduleIsDisplayed=TRUE;
    p_FFT2.niv=-10.;
    p_FFT2.fen_signal=32;
    p_FFT2.fen_fft=10;
	 if (p_FFT2.isOpen) ::InvalidateRect(p_FFT2.PTW->HWindow,NULL,TRUE);
  }
  if ((n<0)||(n==2))
  { p_LisCep1.isPreac=true;
    p_LisCep1.beforeLastSpectrum=false;
    p_LisCep1.niv=5.;
    p_LisCep1.fen_signal=16;
    p_LisCep1.fen_fft1=8;
    p_LisCep1.fen_fft2=8;
    p_LisCep1.fen_fft3=8;
    p_LisCep1.debut_liftrage=1000*NB_CEP_STD1_SEC;
    p_LisCep1.fin_liftrage=1000*NB_CEP_STD2_SEC;
	 if (p_LisCep1.isOpen) ::InvalidateRect(p_LisCep1.PTW->HWindow,NULL,TRUE);
  }
  if ((n<0)||(n==3))
  { p_LisCep2.isPreac=TRUE;
    p_LisCep1.beforeLastSpectrum = false;
    p_LisCep2.niv=-15.;
    p_LisCep2.fen_signal=32;
    p_LisCep2.fen_fft1=9;
    p_LisCep2.fen_fft2=9;
    p_LisCep2.fen_fft3=9;
    p_LisCep2.debut_liftrage=1000*NB_CEP_STD1_SEC;
    p_LisCep2.fin_liftrage=1000*NB_CEP_STD2_SEC;
	 if (p_LisCep2.isOpen) ::InvalidateRect(p_LisCep2.PTW->HWindow,NULL,TRUE);
  }
  if ((n<0)||(n==6))
  { p_TrueEnv.isPreac=TRUE;
    p_TrueEnv.niv = -10.;
    p_TrueEnv.fen_signal = 32;
    p_TrueEnv.fen_fft = 9;
    p_TrueEnv.alpha = 0;
    p_TrueEnv.nbIter = 6;
    p_TrueEnv.debut_liftrage = 1000*NB_CEP_STD1_SEC;
    p_TrueEnv.fin_liftrage = 1000*NB_CEP_STD2_SEC;
	 if (p_TrueEnv.isOpen) ::InvalidateRect(p_TrueEnv.PTW->HWindow,NULL,TRUE);
  }
  if ((n<0)||(n==7))
  { p_CriticalBands.isPreac=TRUE;
    p_CriticalBands.niv=-10.;
    p_CriticalBands.analysis = 2;
    p_CriticalBands.withDctIdctLiftering = true;
    p_CriticalBands.intensityConversion = false;
    p_CriticalBands.PLPorder = 6;
    p_CriticalBands.fen_signal=32;
    p_CriticalBands.fen_fft=10;
    p_CriticalBands.nbBands= 1 << (p_CriticalBands.fen_fft - 1);
    p_CriticalBands.samplingRate = en_cours.frequence;
    p_CriticalBands.fen_fft=10;
    p_CriticalBands.withCepstralSmoothing = false;
    p_CriticalBands.nbMelLiftering = p_CriticalBands.nbBands / 2;
    p_CriticalBands.frequencyScale = true;    // Mel
    p_CriticalBands.completionStrategy = true;    // ASR like. The distribution of spectral points is linear in the Mel or Bark scale
    p_CriticalBands.nbFreqSamplesDisplayed = 1 << (p_CriticalBands.fen_fft - 1);

	 if (p_CriticalBands.isOpen) ::InvalidateRect(p_CriticalBands.PTW->HWindow,NULL,TRUE);
  }
   if ((n<0)||(n==4))
  { p_LPC.isPreac=TRUE;
    p_LPC.niv=0.;
    p_LPC.fen_signal=16;
    p_LPC.ordre_lpc=ORDRE_LPC_STD_MS;
    p_LPC.rayon=1.;
	 if (p_LPC.isOpen) ::InvalidateRect(p_LPC.PTW->HWindow,NULL,TRUE);
  }
   if ((n<0)||(n==8))
  { p_LPCCEP.isPreac=TRUE;
    p_LPCCEP.niv=0.;
    p_LPCCEP.fen_signal=16;
    p_LPCCEP.ordre_lpc=ORDRE_LPC_STD_MS;
    p_LPCCEP.rayon=1.;
	 if (p_LPCCEP.isOpen) ::InvalidateRect(p_LPCCEP.PTW->HWindow,NULL,TRUE);
  }

  if ((n<0)||(n==5))
  { p_LPCsel.isPreac=TRUE;
    p_LPCsel.niv=10.;
    p_LPCsel.fen_signal=16;
    p_LPCsel.ordre_lpc=ORDRE_LPC_STD_MS;
                   // ordre_lpc_std() / 2
		   // + ((ordre_lpc_std() / 2) % 2);
    p_LPCsel.fInf=0;
    p_LPCsel.fHaute=4000;
	 if (p_LPCsel.isOpen) ::InvalidateRect(p_LPCsel.PTW->HWindow,NULL,TRUE);
  }
}
void Wcoupe::IDdefauts(WPARAM) //RTMessage)
{ defauts(-1);
}

void Wcoupe::IDaxes(WPARAM) //RTMessage)
{
   styleAxes = (styleAxes == NORMAL) ? SIMPLE : NORMAL;
}

void Wcoupe::IDCopyToClipboard(WPARAM) //RTMessage)
{
	::OpenClipboard(wcoupe->HWindow);
	::EmptyClipboard();
	::SetClipboardData(CF_BITMAP, wcoupe->hbitmap);
	::CloseClipboard();
}


/*
void coupePositionCurseur(long numEcht)
{ char ch[80];

  numEchtCoupeSpectrale = numEcht;
  sprintf(ch, ml(3412,"Coupe spectrale ( %4ld ms)")
	 , enMilli(numEcht)) ;
  PTWcoupe->SetCaption(ch);
}
*/



/*
 * commande menu d'affichage des coupes spectrales
 */
void Wtop::CMaffCoupe()
{
  if (!yaUneFenetreCoupe) {
    desactive(PPMENUAllStandard);
    GetApplication()->MakeWindow(new Wcoupe);
  }
}

/////////////////////////////////////////////////////
// souris
//
//static int xposCoupe = -1;
void Wdonnees_spectro::drawDownCoupe(int event_x)
{
  drawMotionCoupe(event_x);
}

void Wdonnees_spectro::drawMotionCoupe(int event_x)
{
  float fmuet;
  char ch[80];

  numEchtCoupeSpectrale = info_souris(this, event_x, 0, &fmuet);
  //coupePositionCurseur(numEcht);
  sprintf(ch, ml(3412,"Coupe spectrale ( %4ld ms,  l'chantillon %ld)")
	 , enMilli(numEchtCoupeSpectrale), numEchtCoupeSpectrale) ;
  PTWcoupe->SetCaption(strdup(ch));
  MoveBarre(numEchtCoupeSpectrale);
  if (!PTWcoupe->iconified) {
    if (yaUnSignal) {
      wcoupe->lastct = ctp;
      wcoupe->lastSig = sig;
    }
    affUneCoupe(wcoupe, *sig, numEchtCoupeSpectrale, ctp, PTWcoupe->styleAxes);
  }
}

void Wdonnees_spectro::drawUpCoupe(int event_x)
{
  drawMotionCoupe(event_x);
  if (PTWcoupe->iconified) return;
  affUneCoupe(wcoupe, *sig, numEchtCoupeSpectrale, ctp, PTWcoupe->styleAxes);
}

/////////////////////////////////////////////////////
// fentres paramtres
//

DEFINE_RESPONSE_TABLE1(TWparaCoupe,TFrameWindow)
  EV_CHILD_NOTIFY_ALL_CODES(93,IDdefaults),
  EV_CHILD_NOTIFY_ALL_CODES(94,IDdetails),
END_RESPONSE_TABLE;
TWparaCoupe::TWparaCoupe(PTWindowsObject AParent,char *titre
		  ,paramCoupe *p)
	:TFrameWindow(AParent,titre)
{ char ch[80];

  ticket=GetTicket();
  wsprintf(ch,"%s %d.%d",titre
		  ,ticket,PTWcoupe->ticket);
  Title=strdup(ch);
  DisableAutoCreate();

  param_=p;
  param_->isOpen=TRUE;
  param_->PTW=(PTWindowsObject)this;

  Attr.Style =WS_VISIBLE |  WS_POPUPWINDOW | WS_CAPTION;
  Attr.X = 250+20*param_->num;
  Attr.Y = 20+20*param_->num;
  Attr.W = 280;
  Attr.H = 250;
  this->SetIcon(GetApplication(), IDI_ICON1);

  xx=2;yy=2;ww=Attr.W-6;hh=HNUM;
  // praccentuation
  new TWchoix(this,0,xx,yy,ww
	      ,&param_->isPreac
	      ,ml(3415,"Praccentu")
	      ,choix_YESNO 
	      ,ticket);                yy+=HNUM;
}
void TWparaCoupe::FinirCreate()
{ int w2;

  // niveau d'affichage
  myTStatic(ml(3413,"niveau d'affichage (dB)"));
  new TWfloat(this,0,xx,yy,ww
	      ,&param_->niv
	      ,ml(3414,"niv=")
	      ,-50.,50.,-50.,50.
	      ,ticket);
  //
  yy+=2*HNUM;
  w2=Attr.W/2;
  new TButton(this,93,ml(3417,"Dfauts")
              ,   5,yy,w2-10,24,FALSE);
  new TButton(this,94,ml(3418,"Dtails")
	      ,w2+3,yy,w2-10,24,FALSE);
  yy+=HNUM;
  //
  Attr.H=yy+30;
}
bool TWparaCoupe::CanClose()
{ param_->isOpen=FALSE;
  return TRUE;
}
void TWparaCoupe::myTStatic(char *texte)
{ TStatic *PTS;
  PTS=new TStatic(this,-1,texte
		  ,xx,yy+2,ww,HNUM-2,0);
  yy+=HNUM;
  PTS->Attr.Style&=~SS_LEFT;
  PTS->Attr.Style|=SS_CENTER;
}
void TWparaCoupe::IDdefaults(WPARAM)
{ PTWcoupe->defauts(param_->num);
}
void TWparaCoupe::IDdetails(WPARAM)
{ // switch(param_->num)
  CHARbuf buf(1,"");
  char ch[80];
  int n=param_->num;
  BOOL ind;
  float vf;
  int vi;

  if (!yaUnSignal)
    buf.cpy(ml(3420,"pas de signal"));
  else
  { buf.cpy(ml(3421,"frquence (Hz)="));
    sprintf(ch,"%u",(unsigned int)en_cours.frequence);
    buf.cat(ch);
    // fenetre signal
    switch (n)
    { case 0 : vf=p_FFT1.fen_signal; break;
      case 1 : vf=p_FFT2.fen_signal; break;
      case 2 : vf=p_LisCep1.fen_signal; break;
      case 3 : vf=p_LisCep2.fen_signal; break;
      case 4 : vf=p_LPC.fen_signal; break;
      case 5 : vf=p_LPCsel.fen_signal; break;
      case 6 : vf=p_TrueEnv.fen_signal; break;
      case 8 : vf=p_LPCCEP.fen_signal; break;
    }
    sprintf(ch,ml(3422,"\nfenetre_signal (echt)=%.2f")
              ,enEchtf(vf));
    buf.cat(ch);
    // liftrage
    ind=FALSE;
    switch (n)
    { case 2 : vf=p_LisCep1.debut_liftrage; ind=TRUE; break;
      case 3 : vf=p_LisCep2.debut_liftrage; ind=TRUE; break;
      case 6 : vf=p_TrueEnv.debut_liftrage; ind=TRUE; break;
    }
    if (ind)
    { sprintf(ch,ml(3423,"\ndbut du liftrage (echt)=%.2f")
                ,enEchtf(vf));
      buf.cat(ch);
    }
    switch (n)
    { case 2 : vf=p_LisCep1.fin_liftrage; break;
      case 3 : vf=p_LisCep2.fin_liftrage; break;
      case 6 : vf=p_TrueEnv.fin_liftrage; break;
    }
    if (ind)
    { sprintf(ch,ml(3424,"\nfin du liftrage (echt)=%.2f")
                ,enEchtf(vf));
      buf.cat(ch);
    }
    // ordre LPC
    ind=FALSE;
    switch (n)
    { case 4,8 : vi=ordre_lpc_msTOecht(p_LPC.ordre_lpc);
               ind=TRUE; break;
      case 5 : vi=ordre_lpcsel_msTOecht(p_LPCsel.ordre_lpc
			,p_LPCsel.fInf,p_LPCsel.fHaute);
               ind=TRUE; break;
    }
    if (ind)
    { sprintf(ch,ml(3425,"\nordre_lpc (echt)=%d"),vi);
      buf.cat(ch);
    }
  }
  ::MessageBox(HWindow
	     ,(char *)&buf[0],ml(3418,"details")
		  ,MB_OK | MB_TASKMODAL );

}


