/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier delimite.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <string.h>

#include "snoori.h"
#include "spectro.h"
#include "notetiq.h"
#include "editer.h"
#include "filtre.h"
#include "signal.h"
#include "outils.h"
#include "pitchm.h"
#include "piquef0.h"
#include "audio.h"
#include "polypoint.h"
#include "guiklat.h"
#include "fmaxspec.h"
#include "editmngmt.h"
#include "filterOLA.h"

extern Wpiquefor *wpiquefor;  // formant editor window

///////////////////////////////////////////////////////
// couter le signal
//
void ecoute(
     long deb, 	// indices de debut et 
     long fin,   //     de fin d'ecoute
     int n)	// nombre de repetitions 
{ play_n(&space[_i(deb)],fin-deb+1
         ,en_cours.frequence,n);
}

void fecoute(int n)
// on coute ce qui est en surbrillance  l'cran
// ou  dfaut tout ce qui est  l'cran
{ if (!yaUnSignal) return; 

  if (    ct.debRegion >= ct.finRegion
       || !dans(ct.debRegion, ct.d_e, ct.f_e)
       || !dans(ct.finRegion, ct.d_e, ct.f_e))
    ecoute(ct.d_es,ct.f_es,n);  
  else ecoute(ct.debRegion, ct.finRegion, n);
}

void Wtop::CMnbecoute()
{ static int nbrep=1;
  char ch1[80];
  strcpy(ch1,My_itoa(nbrep));
  if ( MyInputDialog(
		ml(5502,"couter plusieurs fois")
	       ,ml(5503,"entrer le nombre de fois:")
	       ,ch1,78 ) )
  nbrep=atoi(ch1);
  nbrep=max(nbrep,1);
  fecoute(nbrep);
}

void Wtop::CMecoute1Fois()
{ fecoute(1);
}

//
// Affiche le nouveau spectrogramme apres un
// deplacement quand il n'est pas possible
// d'utiliser des parties du spectro precedent.
//
static void affNvSpectro()
{
  SpectroDisplay d;
  d.type = WINDOW;
  d.ws = wspectro->wsona;
  wspectro->wsona->clear_fenetre();
  if (autospectro)
    spectrogram(wspectro->wsona->sig, d, ct.d_es, ct.f_es, ct, ON_NETTOIE );
  else
    wspectro->wsona->spectro_consolider(0,INFINI_I);
  if (yaUnSignal) wspectro->wvscale->affichage(0., fMaxSpectro, wspectro->wsona->incr, "Hz");  
}

void Wtop::CMzoom_tempo()
{
  if (!yaUnSignal) return; 
  if (  (ct.debRegion >= ct.finRegion)
      ||!dans(ct.debRegion, ct.d_e, ct.f_e)
      ||!dans(ct.finRegion, ct.d_e, ct.f_e))
  { affiche_erreur(ml(901,"La rgion  zoomer est mal dfinie"));
    return;
  }    

  souris(XC_watch);

  ct.finRegion = ct.finRegion - 1;	// la marque utilisateur marque la
                                //   fin du secteur de fin 
  ct.finRegion = max(ct.finRegion, ct.debRegion+10);
  ct.finRegion = min(ct.finRegion, ct.f_e);
  ct.debRegion = min(ct.debRegion, ct.finRegion-10);
  ct.debRegion = max(ct.debRegion, 0);

  ct.d_es = ct.d_a = ct.debRegion;
  ct.f_a = ct.finRegion;
  ct.f_es = min(ct.f_a, ct.f_e);
  SetModeZoom(ZOOM); 

  wsignal->aff();
  wsignal->paint();
  SetScrollSpectroPosition(ct.d_a);
  aff_etiqs();
  aff_notes();

  affNvSpectro();
  if (wpiquefor) wpiquefor->affPPoints();
  if (yaUnEditManager && editManager->editMode == OLA) affTrajOLA();
  ct.debRegion = ct.finRegion=-1;

  souris(XC_top_left_arrow);
}

//
// Suppression du zoom.
//
void Wtop::CMsupprimeZoom_tempo()
{ long nbecht; // nombre d'echt de l'cran

  if (!yaUnSignal) return; 
  
  souris(XC_watch);

  ct.debRegion=ct.d_a;
  ct.finRegion=ct.f_a;
  nbecht=MAXECHTECR;

  SetModeZoom(TOUT); //ct.modeMemoire = TOUT;
  // on prvoit d'abord le nouvel cran avec
  //   la rgion au centre
  //   et de largeur 2 secondes
  ct.d_a = (ct.debRegion+ct.finRegion)/2 -nbecht/2;
  ct.f_a = ct.d_a + nbecht -1;
  // puis on dcale  gauche s'il dpasse  droite
  if (ct.f_a>ct.f_e)
  { ct.f_a=ct.f_e;
    ct.d_a=ct.f_a -nbecht +1;
  }
  // enfin on dcale  droite s'il dpasse  gauche
  if (ct.d_a<ct.d_e)
  { ct.d_a=ct.d_e;
    ct.f_a = ct.d_a +nbecht -1;
  }
  //
  ct.d_es=ct.d_a;
  ct.f_es=min(ct.f_a,ct.f_e);

  wsignal->aff();
  wsignal->paint();
  SetScrollSpectroPosition(ct.d_a);
  aff_etiqs();
  aff_notes();

  affNvSpectro();
  wspectro->wsona->spectro_consolider(0,INFINI_I);
  if (wpiquefor) wpiquefor->affPPoints();
  if (yaUnEditManager && editManager->editMode == OLA) affTrajOLA();
  // The spectrogram back-up has to be over the whole window length
  // and not only on the recomputed spectrogram.
  souris(XC_top_left_arrow);
}

//
// Lecture d'un morceau de signal de parole
//    lors d'un deplacement,
//    si le morceau n'est pas dj, bien entendu
//
//  l'appel, on garantit que
//            infoEdit.yaDesModifs == FAUX
static void deplaceLecture()
{ int ind=0; // !=0 si lireSignal()  faire
  if (ct.d_a<ct.d_e)
  { ind=1;
    ct.d_e=ct.d_a;
    ct.f_e=min(ct.f_e
	       ,ct.d_e+max(MAXECHTECR
	                   ,ct.f_a-ct.d_a));
  }
  if (ct.f_a>ct.f_e)
  { ind=1;
    ct.f_e=min(ct.f_a,ct.tf-1);
    ct.d_e=max(ct.d_e
	       ,ct.f_e-max(MAXECHTECR
	                   ,ct.f_a-ct.d_a));
  }
  if (ind!=0) 
  { space.newsize(ct.f_e-ct.d_e+1);
    lireSignal(en_cours
           ,en_cours.depl + ct.d_e*sizeof(SAMPLE) 
	   ,space, 0
	   ,ct.f_e - ct.d_e + 1);
  }
  infoEdit.derEchtFic = ct.f_e;
  ct.d_es = ct.d_a;
  ct.f_es = min(ct.f_e,ct.f_a);   
}

static void deplacementGauche(
     long , //valeurDep,
     struct ctrl_temps& temps) // ancien ct
// dplacement  gauche ... sur l'cran
//   et donc  droite sur le scrollbar
{ int w  // largeur de la partie conserve
     ,h  // hauteur de la fenetre
     ,d; // dcalage (vers la gauche)
  d_planche *fenetre;

  fenetre = wspectro->wsona;
  d_plancheTmp fenetreTmp(fenetre);

  w=snoscalex(fenetre,temps.f_a);
  h=fenetre->hauteur;
  d=fenetre->largeur-w;

  // copier la zone conserve 
  BitBlt(fenetreTmp.hDCcomp,0,0,w,h
        ,fenetre->hDCcomp,d,0
	,SRCCOPY);
  // ... et la mettre  son emplacement dfinitif
  fenetre->clear_fenetre();
  BitBlt(fenetre->hDCcomp,0,0,w,h
        ,fenetreTmp.hDCcomp,0,0
	,SRCCOPY);

  // ... idem pour wspectroBis
  BitBlt(fenetreTmp.hDCcomp,0,0,w,h
	,wspectro->wsona->bis->hDCcomp,d,0
	,SRCCOPY);
  wspectro->wsona->bis->clear_fenetre();
  BitBlt(wspectro->wsona->bis->hDCcomp,0,0,w,h
        ,fenetreTmp.hDCcomp,0,0
	,SRCCOPY);

  //
  if (autospectro) {
    int decal;

    SpectroDisplay display;
    display.type = WINDOW;
    display.ws = wspectro->wsona;
    // montrer le rsultat provisoire
    CopyArea_Dbw(fenetre,0,0
	      ,fenetre->largeur,fenetre->hauteur
	      ,0,0,GXcopy);

    decal=sps.fen_signal/2;

    spectrogram(wspectro->wsona->sig, display, max(ct.d_es,temps.f_a - decal), ct.f_es, ct, ON_LAISSE );
  }
  InvalidateRect(wspectro->wsona->HWindow,NULL,TRUE);
}

static void deplacementDroite(
     long , //valeurDep,
     struct ctrl_temps& temps) // ancien ct
// dplacement  droite ... sur l'cran
//   et donc  gauche sur le scrollbar
{ int w  // largeur de la partie conserve
     ,h  // hauteur de la fenetre
     ,d; // dcalage (vers la droite)
  d_planche *fenetre;

  fenetre = wspectro->wsona;
  d_plancheTmp fenetreTmp(fenetre);

  d=snoscalex(fenetre, temps.d_a);
  w=fenetre->largeur-d;
  h=fenetre->hauteur;

  // copier la zone conserve
  BitBlt(fenetreTmp.hDCcomp,0,0,w,h
	  ,fenetre->hDCcomp,0,0
	  ,SRCCOPY);
  // ... et la mettre  son emplacement dfinitif
  fenetre->clear_fenetre();
  BitBlt(fenetre->hDCcomp,d,0,w,h
          ,fenetreTmp.hDCcomp,0,0
          ,SRCCOPY);

  // ... idem pour wspectroBis
  BitBlt(fenetreTmp.hDCcomp,0,0,w,h
	  ,wspectro->wsona->bis->hDCcomp,0,0
	  ,SRCCOPY);
  wspectro->wsona->bis->clear_fenetre();
  BitBlt(wspectro->wsona->bis->hDCcomp,d,0,w,h
          ,fenetreTmp.hDCcomp,0,0
          ,SRCCOPY);

  if (autospectro) {
    SpectroDisplay display;
    display.type = WINDOW;
    display.ws = wspectro->wsona;
     // montrer le rsultat provisoire
    CopyArea_Dbw(fenetre,0,0
	      ,fenetre->largeur,fenetre->hauteur
	      ,0,0,GXcopy);

    spectrogram(wspectro->wsona->sig, display, ct.d_es, min(ct.f_es, temps.d_a + sps.fen_signal), ct, ON_LAISSE );
  }
  InvalidateRect(wspectro->wsona->HWindow,NULL,TRUE);
}

void allerA(
	long echt) // numero de l'echantillon ou l'on
	           //    veut aller. 
{
  struct ctrl_temps ct_; // ancien ct
  bool indaffNvSpectro = false;

  ct_=ct;
  if (!yaUnSignal) return;

  // On ne prend pas en consideration un ordre de
  //   deplacement s'il existe 
  //   deja une demande de confirmation de sauvegarde ou d'abandon des 
  //   modifications faites dans l'editeur. 
  // pas de deplacement

  ct.d_a = max(0, min(echt, ct.tf - (ct_.f_a - ct_.d_a)));
  ct.f_a = ct.d_a + (ct_.f_a - ct_.d_a);
  SetScrollSpectroPosition(ct.d_a);
  if (ct_.d_a == ct.d_a) return;

  if (infoEdit.yaDesModifsSignal) {
    switch (UpdateCurrentFileBox()) {
      case IDOK: break; //sauverModif(); 
      case IDIGNORE:    //oublierModif();
	       indaffNvSpectro = true;
	       break;
      case IDABORT: //annulDeplac(&ct_);
               ct = ct_;
               SetScrollSpectroPosition(ct.d_a);
               Wact->wr(""); //:: sans doute inutile
               return; 
    }
  }


  { // affichage
    ct.debRegion = INFINI_L;
    ct.finRegion = -INFINI_L;
    ct.posBarre = -1;
    deplaceLecture();
    wsignal->aff();
    wsignal->paint();
    aff_etiqs();
    aff_notes();
    if (  (ct.d_a > ct_.f_a)
	     ||(ct.f_a < ct_.d_a)
	     || indaffNvSpectro   ) affNvSpectro();
    else if (ct_.d_a < ct.d_a)
     	   deplacementGauche(ct.d_a, ct_);
    else deplacementDroite(ct.d_a, ct_);
    // Displays the F0 contour if the window for F0 contour modifications
    // does exist.
    if (yaUneFenetrePiqueF0) {
      switch (wpiquef0->lastCommand) {
        case MENUSeeF0Contour:
          wpiquef0->CMSeeF0Contour();
          break;
        case MENUModifF0Contour:
          wpiquef0->CMModifF0Contour();
          break;
      }
    }
  }
  Wact->wr(""); 
}
void Wdonnees_spectro::affichePositionSpectro(int event_x, int event_y, int withPitch)
{ float xr, yr;
  char ch1[80],ch2[80];

  if (!yaUnSignal) return; //*en_cours.nom=='\0') return;

  snoiunscalexy(event_x, event_y, &xr, &yr,
		this);
  sprintf(ch1,"%u Hz",(unsigned int)canToHz(yr));
  sprintf(ch2,"%ld ms [%ld]",enMilli(xr),LROUND(xr));
  Wact->setXY(ch1,ch2);
  if (yaUnSignal == ASIGNAL && withPitch) Wact->f0(max(pm.getf0(xr), pm.getReadf0(xr)));
  else Wact->f0(0);
}

void Wtop::CMclearSpectro()
{ wspectro->wsona->clear_fenetre();
  wspectro->wsona->spectro_consolider(0,INFINI_I);
}
void Wtop::CMdecoreSpectro()
{ wspectro->wsona->spectro_consolider(0,INFINI_I);
}
void Wtop::CMnettoieSpectro()
{ wspectro->wsona->spectro_nettoyer(0,INFINI_I);
}

