/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier delimite.cpp                                            
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <string.h>
#include <io.h>  // pour close()

#include "snoori.h"
#include "spectro.h"
#include "notetiq.h"
#include "editer.h"
#include "filtre.h"
#include "audio.h"
#include "wcoupe.h"
#include "format_f.h"
#include "signal.h"
#include "outils.h"
#include "files.h"
#include "wformant.h"
#include "polypoint.h"
#include "guiklat.h"
#include "pitchm.h"  // pour reset pitch
#include "piquef0.h"
#include "filterOLA.h"

struct editeur infoEdit;

void resetDefaultSaveName();

static LONG quelEchantillon(d_planche *fenetre, int x, int y)
	/* en pixels dans la fenetre du spectrogramme */
{
  long echt;
  float xr,yr;

  snoiunscalexy(x, y, &xr, &yr, fenetre);
	 /* fenetre pixel -> fenetre reel */

  echt = LROUND(xr);
  return(echt);
}

/////////////////////////////////////////////////
// barre verticale d'un pixel
//   dans les fentres wspectro et wsignal
//
static void InvertBarre(d_planche *fenetre, int event_x)
{
  if (event_x>=0)
    InvertArea(fenetre->HWindow
      ,event_x, 0, 1 ,fenetre->hauteur);
}

void Wdonnees_spectro::MoveBarre(long echt)
{
  InvertBarre(this, ctp->xBarre);
  if (this->wsigAssoc) InvertBarre(this->wsigAssoc, ctp->xBarre);
  ctp->posBarre = echt;
  if ((ctp->posBarre >= ctp->d_a) && (ctp->posBarre <= ctp->f_a))
    ctp->xBarre = this->LXtoDX(echt);
  else ctp->xBarre = -1;
  InvertBarre(this, ctp->xBarre);
  if (this->wsigAssoc) InvertBarre(this->wsigAssoc, ctp->xBarre);
}

// A "temporal" window class should be created !
void Wdonnees_signal::MoveBarre(long echt)
{
  InvertBarre(this, ctp->xBarre);
  InvertBarre(this->wspectrAssoc, ctp->xBarre);
  ct.posBarre = echt;
  if ((ctp->posBarre >= ctp->d_a) && (ctp->posBarre <= ctp->f_a))
    ctp->xBarre = this->LXtoDX(echt);
  else ctp->xBarre = -1;
  InvertBarre(this, ctp->xBarre);
  InvertBarre(this->wspectrAssoc, ctp->xBarre);
}

////////////////////////////////////////////////////
// inversion d'une bande verticale dans une fentre
//   en coordonnes pixel
//
// inverse la bande x1 x2 bornes comprises
// utilis seulement par drawMotion()
//
void affxRegion(
  d_planche *fenetre,
  int x1,
  int x2)
// n'est plus utilis qu'en inversion de rgion
{
  InvertArea(fenetre->HWindow,min(x1,x2), 0,abs(x1-x2)+1,fenetre->hauteur);
}

/////////////////////////////////////////////////////
// affichage de la rgion et de la barre
// (sur un spectrogramme sans rgion ni barre)
//
// par inversion des bandes verticales adquates
//
// utilis pour la mise en surbrillance
// ne surbrille pas de rgion
//     si la rgion n'existe pas
//     ou si la rgion est trop petite
// ne surbrille pas la barre
//     si la barre n'existe pas
//
void affRegion(
  d_planche *fenetre,
  struct ctrl_temps *pct)
{
  int xdebut, xfin;

  if (pct->debRegion < pct->finRegion) {
    xdebut = snoscalex(fenetre,pct->debRegion);
    xfin = snoscalex(fenetre,pct->finRegion);
    if (xfin>xdebut) affxRegion(fenetre,xdebut+1,xfin);
  }
  /* Display the line is not necessary here. */
  if ((pct->posBarre >=  pct->d_a) && (pct->posBarre <= pct->f_a))
    pct->xBarre=fenetre->LXtoDX(pct->posBarre);
  else pct->xBarre = -1;
  InvertBarre(fenetre,pct->xBarre);
}

//////////////////////////////////////
// fonctions souris
//
void Wdonnees_spectro::drawDown(int event_x, int transmet)
{
  if (!yaUnSignal) return;
  else { // cas normal
    if (ctp) {
      ctp->xDebRegion = event_x;
      ctp->xFinRegion = event_x;
      ctp->debRegion=ctp->finRegion=0;
      ::InvalidateRect(HWindow, NULL, FALSE);
      UpdateWindow();
    }
    if (wsigAssoc && transmet) wsigAssoc->drawDown(event_x, 0);
  }
}

void Wdonnees_spectro::drawMotion(int event_x, int transmet)
{
  if (!yaUnSignal) return;
  else { // cas normal
    if (ctp) {
      if (event_x != ctp->xFinRegion) {
        affxRegion(this,
		             ctp->xFinRegion + ((event_x > ctp->xFinRegion) ? 1 : 0),
		             event_x + ((event_x > ctp->xFinRegion) ? 0 : 1) );
        if (wsigAssoc)
           affxRegion(wsigAssoc,
        					 ctp->xFinRegion + ((event_x > ctp->xFinRegion) ? 1 : 0),
		                event_x + ((event_x > ctp->xFinRegion) ? 0 : 1) );
        ctp->xFinRegion = event_x;
      }
    }
  }
}

void Wdonnees_spectro::drawUp(int event_x, int transmet)
{
  if (!yaUnSignal) return;
  else { // cas normal
    if (ctp) {
      ctp->xFinRegion = event_x;
      if (ctp->xFinRegion < ctp->xDebRegion) {
        int t = ctp->xFinRegion;
        ctp->xFinRegion = ctp->xDebRegion;
        ctp->xDebRegion = t;
      }
      ctp->debRegion = min(ctp->f_es,quelEchantillon(this, ctp->xDebRegion, 0));
      ctp->finRegion = min(ctp->f_es,quelEchantillon(this, ctp->xFinRegion, 0));
      //
      Wact->duree(enMillif(this->DXtoLX(ctp->xFinRegion)
		         				 - this->DXtoLX(ctp->xDebRegion)));
    }
    ::InvalidateRect(HWindow,NULL,FALSE);
    if (wsigAssoc && transmet) wsigAssoc->drawUp(event_x, 0);
  }
}

//////////////////////////////////////
// fonctions souris
//
void Wdonnees_signal::drawDown(int event_x, int transmet)
{
  if (!yaUnSignal) return;
  else { // cas normal
    if (ctp) {
      ctp->xDebRegion = event_x;
      ctp->xFinRegion = event_x;
      ctp->debRegion=ctp->finRegion=0;
      ::InvalidateRect(HWindow, NULL, FALSE);
      UpdateWindow();
    }
    if (wspectrAssoc && transmet) wspectrAssoc->drawDown(event_x, 0);
  }
}

void Wdonnees_signal::drawMotion(int event_x, int transmet)
{
  if (!yaUnSignal) return;
  else { // cas normal
    if (ctp) {
      if (event_x != ctp->xFinRegion) {
        affxRegion(this,
		             ctp->xFinRegion + ((event_x > ctp->xFinRegion) ? 1 : 0),
		             event_x + ((event_x > ctp->xFinRegion) ? 0 : 1) );
        if (wspectrAssoc) affxRegion(wspectrAssoc,
    		      ctp->xFinRegion + ((event_x > ctp->xFinRegion) ? 1 : 0),
               event_x + ((event_x > ctp->xFinRegion) ? 0 : 1) );
        ctp->xFinRegion = event_x;
      }
    }
  }
}

void Wdonnees_signal::drawUp(int event_x, int transmet)
{
  if (!yaUnSignal) return;
  else { // cas normal
    if (ctp && transmet) {
      ctp->xFinRegion = event_x;
      if (ctp->xFinRegion < ctp->xDebRegion) {
        int t = ctp->xFinRegion;
        ctp->xFinRegion = ctp->xDebRegion;
        ctp->xDebRegion = t;
        event_x = ctp->xFinRegion;
      }
      ctp->debRegion = min(ct.f_es,quelEchantillon(this, ctp->xDebRegion, 0));
      ctp->finRegion = min(ct.f_es,quelEchantillon(this, ctp->xFinRegion, 0));
      //
      Wact->duree(enMillif(this->DXtoLX(ctp->xFinRegion)
		         				 - this->DXtoLX(ctp->xDebRegion)));
    }
    ::InvalidateRect(HWindow,NULL,FALSE);
    if (wspectrAssoc && transmet) wspectrAssoc->drawUp(event_x, 0);
    if (yaUneFenetreMccandless || yaUneFenetrePiquefor && wf1f2)
      affVoyelles();
  }
}

void reinitSignalAndScreen()
{
  yaUnSignal = NOSIGNAL;
  en_cours.nom[0] = '\0';
  etiqs.clear();
  notes.clear();
  pm.clear();
  piqueforClear();
  trajOLAClear();
  PTWtop->SetCaption(ml(1003,"noname"));
  InvalidateRect(PTWtop->HWindow,NULL,true);
  wsignal->wsig->SendMessage(WM_SIZE,SIZENORMAL,MAKELONG(wsignal->wsig->Attr.W,wsignal->wsig->Attr.H));
  wsignal->wscale->SendMessage(WM_SIZE,SIZENORMAL,MAKELONG(wsignal->wscale->Attr.W,wsignal->wscale->Attr.H));
  SendMessage(wspectro->wsona->HWindow
	     ,WM_SIZE,SIZENORMAL
	     ,MAKELONG(wspectro->wsona->Attr.W
		      ,wspectro->wsona->Attr.H));
  SendMessage(wetiquetage->HWindow
	     ,WM_SIZE,SIZENORMAL
	     ,MAKELONG(wetiquetage->Attr.W
		      ,wetiquetage->Attr.H));
  SendMessage(wannotations->HWindow
	     ,WM_SIZE,SIZENORMAL
	     ,MAKELONG(wannotations->Attr.W
		      ,wannotations->Attr.H));

  desactive(PPMENUAllStandard);
  active(MENUNew);
  active(MENUOpen);
  DrawMenuBar(PTWtop->HWindow);
  yaF0Selected = false;
}

void Wtop::CMnouveau()
{
  if (yaUnSignal && UpdateCurrentFileBox() == IDABORT) return;
  souris(XC_watch);
  reinitSignalAndScreen();
  souris(XC_top_left_arrow);
}

void Wtop::CMzeroFile()
{
  if (yaUnSignal && UpdateCurrentFileBox() == IDABORT) return;
  reinitSignalAndScreen();
  initShadow(4000, false, 0, 2000, 22050);
  active(PPMENUAllStandard);
  initEditSignal();
}

void Wtop::CMlecture_fichier()
{
  STRING nom;
  if (UpdateCurrentFileBox() == IDABORT) return;

  splitmakedir(nom);
  strcat(nom,"*.*");
  if (get_nom_fichier_open(PTWtop,nom)!=IDOK) return;
  lecture_fichier(nom);
}


void Wtop::lecture_fichier(
  char *fichier)
{
  infoFile en_cours_;

  en_cours_ = en_cours;
  strcpy(en_cours_.nom, fichier);
  resetDefaultSaveName(); 

  CMnouveau();  //pour reinitialiser les menus.

  if (!isWAV(en_cours_.nom)) {
    // lecture fichier non WAV
    // choisir le format du fichier de parole
    infoFile_set_noms(en_cours_,en_cours_.nom);
    //
    if (get_format(PTWtop,en_cours_)!=IDOK) return;
  }

  en_cours=en_cours_;
  souris(XC_watch);
  lect_signal();
  souris(XC_top_left_arrow);
}

/*
 * Couper le signal
 */
static void couperRegion(long debCoup, long finCoup)
	// partie a couper definie par les
        //     echantillons de debut et de fin
{
  long il,jl;
  long nEchtLus = 0,
       tailleRegion = finCoup - debCoup + 1,
       d_a = ct.d_a,
       f_a = ct.f_a;		/* on sauvegarde les limites du
                                   domaine d'affichage de Snoori */

  if (!droit_ecrire_message(en_cours.nom)) return;

  if (debCoup > finCoup || !dans(debCoup, ct.d_e, ct.f_e)
      || !dans(finCoup, ct.d_e, ct.f_e)) {
    affiche_erreur(ml(1005,"La rgion  couper mal dfinie"));
    return;
  }

  infoEdit.derOp = COUPER;
  infoEdit.debut = debCoup;
  infoEdit.fin = finCoup;
  init_infoFile_write(&infoEdit.cut);
  infoEdit.cut.frequence=en_cours.frequence;

  /* On sauve le signal qui va etre coupe */
  copieDansCutFile(&infoEdit);

  // cut labels and notes in buffers
  etiqs.edit_delete(debCoup,finCoup);
  notes.edit_delete(debCoup,finCoup);
  aff_etiqs();
  aff_notes();

  /* Destruction de la region */
  for (il = _i(debCoup), jl = _i(finCoup) + 1;
       jl <= _i(ct.f_e); il++, jl++)
    space[il] = space[jl];
  ct.f_e-=tailleRegion;

  // Si on peut, on lit la suite du signal temporel
  // pour remplacer ce qu'on a detruit soit apres
  // la zone coupee
  if (  (ct.f_e<ct.f_a)
      &&(infoEdit.derEchtFic<ct.tf-1) )
  { long nbEchtAlire;
    /* il faut verifier qu'on peut effectivement lire
       nbEchtAlire echantillons */
    nbEchtAlire = min(ct.f_a-ct.f_e
                      ,ct.tf - 1 - infoEdit.derEchtFic);
    space.newsize(ct.f_e+nbEchtAlire
                  -ct.d_e+1);
    nEchtLus = lireSignal(en_cours
			,en_cours.depl
			   + (infoEdit.derEchtFic + 1)
			     * sizeof(SAMPLE)
			,space, _i(ct.f_e+1)
			,nbEchtAlire);
    ct.f_e += nEchtLus;
    infoEdit.derEchtFic += nEchtLus;
  }
  ct.f_es = min(ct.f_e, ct.f_a);

  /* Mise a jour de l'affichage */
  wsignal->aff();

  /* ... et affichage du spectro */
  if (ct.d_a == d_a && ct.f_a == f_a)
  { // Les limites d'affichage n'ont pas change.
    // On peut utiliser l'affichage rapide
    // du spectrogramme 

    // dans ce qui suit rgion veut dire "la zone  couper"
    //
    // bout avant la rgion : conserv en place
    ajtBoutSpectro(ct.d_a, infoEdit.debut - 1,
                   ct.d_a, infoEdit.debut - 1); 
    // bout aprs la rgion : conserv et dplac
    ajtBoutSpectro(infoEdit.debut + tailleRegion, ct.f_a,
                   infoEdit.debut, ct.f_a - tailleRegion);
    // petit bout  la jonction des deux bouts prcdents : recalcul
    ajtBoutSpectro(-1, -1, infoEdit.debut - 2, infoEdit.debut + 1);
    // bout final : vide  calculer
    ajtBoutSpectro(-1, -1, ct.f_a - tailleRegion-1, ct.f_a);
    //
    spectroRapide(wspectro->wsona);
  }
  else {
   SpectroDisplay display;
    display.type = WINDOW;
    display.ws = wspectro->wsona;

    if (autospectro)
      spectrogram(wspectro->wsona->sig, display, ct.d_es, ct.f_es, ct, ON_NETTOIE );
  }

  infoEdit.yaDesModifsSignal = VRAI;
  infoEdit.yaDesModifsEtiquetages = VRAI;
}

/*
 * Couper  (detruire la region mise en relief)
 */
void Wtop::CMfcouper()
{  
  if (!yaUnSignal) return; //*en_cours.nom == '\0')
  if (!droit_ecrire_message(en_cours.nom)) return;

  souris(XC_watch);
  strcat(strcpy(infoEdit.cut.nom,tempdir),"cutfile.wa$");
  couperRegion(ct.debRegion, ct.finRegion);
  ct.debRegion = INFINI_L;
  ct.finRegion = -INFINI_L;
  ::InvalidateRect(HWindow,NULL,TRUE);
  souris(XC_top_left_arrow);
}

/*
 * Effacer  (detruire la region mise en relief)
 */
void Wtop::CMfeffacer()
{  
  if (!yaUnSignal) return;
  if (!droit_ecrire_message(en_cours.nom)) return;

  souris(XC_watch);
  strcat(strcpy(infoEdit.cut.nom,tempdir),"cutfile_.wa$");
  couperRegion(ct.debRegion, ct.finRegion);
  ct.debRegion = INFINI_L;
  ct.finRegion = -INFINI_L;
  ::InvalidateRect(HWindow,NULL,TRUE);
  souris(XC_top_left_arrow);
}

/////////////////////////////////////////////
// dialogue pour criture du fichier courant
//
void UpdateCurrentFile()
    // revoir : on peut sans doute affiner suivant
    //     les cas WAV ou non WAV
{
  if (   infoEdit.yaDesModifsSignal
      || infoEdit.yaDesModifsEtiquetages )
  { infoEdit.save=en_cours;
    repercuteModif(&infoEdit); 
    //
    if (!isWAV(en_cours.nom))
    { if (  //:: en_cours.isOpenPhon
            //:: &&
           (en_cours.fmt_phon!=0) )
        sauvman(&etiqs, en_cours);
      if (  //:: en_cours.isOpenWord
	    //:: &&
           (en_cours.fmt_word!=0) )
	sauveAnnotations(&notes, en_cours);
    }
  }
}


class TWUpdateOrNothing : public TDialog
{ public:
    TWUpdateOrNothing(): TDialog(PTWtop,"WUpdateOrNothing") {}
    virtual void SetupWindow();
	 void IDignore(WPARAM code);
	 void IDabort(WPARAM code);
	 DECLARE_RESPONSE_TABLE(TWUpdateOrNothing);
};
DEFINE_RESPONSE_TABLE1(TWUpdateOrNothing, TDialog)
  EV_CHILD_NOTIFY_ALL_CODES(IDIGNORE, IDignore),
  EV_CHILD_NOTIFY_ALL_CODES(IDABORT, IDabort),
END_RESPONSE_TABLE;

void TWUpdateOrNothing::SetupWindow()
{
  TDialog::SetupWindow();
  SetCaption(ml(1020,"Modifications"));
  ::SetWindowText(GetItemHandle(101),ml(1021,"L'opration en cours ... "));
  ::SetWindowText(GetItemHandle(IDABORT),ml(1022,"&Annuler"));
  ::SetWindowText(GetItemHandle(102),ml(1024,"normal, sauver puis excuter l'opration"));
  ::SetWindowText(GetItemHandle(104),ml(1026,"ni sauver, ni excuter"));
}
void TWUpdateOrNothing::IDignore(WPARAM)
{ CloseWindow(IDIGNORE);
}
void TWUpdateOrNothing::IDabort(WPARAM)
{ CloseWindow(IDABORT);
}

class TWUpdateCurrentFile : public TDialog
{ public:
    TWUpdateCurrentFile(): TDialog(PTWtop,"WUpdateCurrentFile") {}
    virtual void SetupWindow();
	 void IDignore(WPARAM code);
	 void IDabort(WPARAM code);
	 DECLARE_RESPONSE_TABLE(TWUpdateCurrentFile);
};
DEFINE_RESPONSE_TABLE1(TWUpdateCurrentFile,TDialog)
  EV_CHILD_NOTIFY_ALL_CODES(IDIGNORE,IDignore),
  EV_CHILD_NOTIFY_ALL_CODES(IDABORT,IDabort),
END_RESPONSE_TABLE;

void TWUpdateCurrentFile::SetupWindow()
{ TDialog::SetupWindow();
  SetCaption(ml(1020,"Modifications"));
  ::SetWindowText(GetItemHandle(101),ml(1021,"L'opration en cours ... "));
  ::SetWindowText(GetItemHandle(IDABORT),ml(1022,"&Annuler"));
  ::SetWindowText(GetItemHandle(IDIGNORE),ml(1023,"&Ignorer"));
  ::SetWindowText(GetItemHandle(102),ml(1024,"normal, sauver puis excuter l'opration"));
  ::SetWindowText(GetItemHandle(103),ml(1025,"ne pas sauver, mais excuter l'opration"));
  ::SetWindowText(GetItemHandle(104),ml(1026,"ni sauver, ni excuter"));
}
void TWUpdateCurrentFile::IDignore(WPARAM)
{ CloseWindow(IDIGNORE);
}
void TWUpdateCurrentFile::IDabort(WPARAM)
{ CloseWindow(IDABORT);
}
int UpdateCurrentFileBox()
// demande  sauver le current file
// et le sauve si IDOK
    // revoir : on peut sans doute affiner suivant
    //     les cas WAV ou non WAV
{
  if (!Licensed) return IDOK;
  int id = -1;
  long nbecht;

  if (   infoEdit.yaDesModifsSignal
      || infoEdit.yaDesModifsEtiquetages )
  { id=PTWtop->GetModule()->ExecDialog(
	   new TWUpdateCurrentFile());
    switch (id)
    { case IDOK: UpdateCurrentFile();
		 break;
      case IDIGNORE:
	      ct.tf=en_cours.longueur / sizeof(SAMPLE);
	      //
	      ct.f_e=(ct.d_a+ct.f_a)/2+MAXECHTECR/2;
              ct.f_e=min(ct.f_e,ct.tf-1);
	      ct.d_e=ct.f_e-MAXECHTECR;
	      ct.d_e=max(ct.d_e,0);
	      //
	      nbecht=ct.f_a-ct.d_a+1;
              if (ct.f_a>ct.f_e)
              { ct.f_a=ct.f_e;
                ct.d_a=ct.f_a -nbecht +1;
              }
              if (ct.d_a<ct.d_e)
              { ct.d_a=ct.d_e;
                ct.f_a = ct.d_a +nbecht -1;
              }
              //
              ct.d_es = ct.d_a;
	      ct.f_es = min(ct.f_e,ct.f_a);
	      //
	      SetScrollSpectroPosition(ct.d_a);
	      space.newsize(ct.f_e-ct.d_e+1);
	      lireSignal(en_cours
                  ,en_cours.depl + ct.d_e*sizeof(SAMPLE) 
	          ,space, 0
	          ,ct.f_e - ct.d_e + 1);
	      infoEdit.derEchtFic = ct.f_e;
	      infoEdit.yaDesModifsSignal = FAUX;
         infoEdit.yaDesModifsEtiquetages = FAUX;
	      break;
    }
  }
  return id;
}


/*
 *  Substitution d'une partie du fichier de WinSnoori par un fichier.
 *  Le signal de ce fichier prend la place du signal original.
 */
long substituerFichier(
	long ou,  	// chantillon ou on commence la substitution
	bool force) // if true the existing file is saved whatever the length of the insertion.
   				// en retour, nombre d'chantillons substitus
{
  int id = -1;
  long nbSample;

  if (!droit_ecrire_message(en_cours.nom)) return 0; 

  if (!dans(ou, ct.d_e, ct.f_e)) {
    affiche_erreur(
	 ml(1032,"substitution en dehors du buffer interdite"));
    return 0;
  }

  init_infoFile_read(&infoEdit.substitute);
  if (infoEdit.substitute.longueur==0) {
    affiche_warning(ml(1033,"rien  insrer"));
    return 0;
  }
    
  infoEdit.derOp = SUBSTITUER;
  infoEdit.debut = ou;
  
  nbSample = infoEdit.substitute.longueur/sizeof(SAMPLE);
  infoEdit.fin = infoEdit.debut + nbSample - 1;

  if (ou + nbSample - ct.d_e + 1 > MAXECH && !force) {
    // on ne peut pas se contenter de modifier le
    // buffer de Snoori. Il faut aussi modifier
    // le fichier courant
    // On repercute donc les modifs qui n'ont pas
    // encore ete prises en compte dans les fichiers
    id = UpdateCurrentFileBox();
    switch (id) {
     case IDABORT: infoEdit.derOp=PASDOP;
                    return 0;
    }
  }

  // ajout annotations et tiquetage
  { notetiqs netemp;
    // d'abord annotations
    if (isWAV(infoEdit.substitute.nom))
    { readnotetiqs(infoEdit.substitute.nom,netemp
		    ,infoEdit.substitute.notedepl
		    ,infoEdit.substitute.notelongueur);
    }
    notes.edit_paste(infoEdit.debut
	   ,infoEdit.substitute.longueur/sizeof(SAMPLE)
	   ,netemp);
    // puis tiquetage
    netemp.clear();
    if (isWAV(infoEdit.substitute.nom))
    { readnotetiqs(infoEdit.substitute.nom,netemp
		    ,infoEdit.substitute.etiqdepl
        	    ,infoEdit.substitute.etiqlongueur);
    }
    etiqs.edit_paste(infoEdit.debut
	   ,infoEdit.substitute.longueur/sizeof(SAMPLE)
	   ,netemp);
    //
    infoEdit.yaDesModifsEtiquetages=VRAI;
  }

  // substitution effective du signal
  if (force || ou + nbSample - ct.d_e + 1 > MAXECH) {
    if (force || id == IDOK || TWUpdateOrNothing().Execute() == IDOK ) {
      // la partie insre fait dborder le
      // buffer "space". On insre le fichier
      // dans le fichier courant, puis on
      // relit le son en entier.
      substituteFileIntoCurrentFile(&infoEdit);
      ct.tf = en_cours.longueur / sizeof(SAMPLE);
      ct.d_e = ct.d_a;
    	ct.f_e = min(ct.d_e + DUREE_FEN*en_cours.frequence - 1
                 ,min(ct.f_a,ct.tf-1) );
    	infoEdit.yaDesModifsSignal = FAUX;
    	space.newsize(ct.f_e-ct.d_e+1);
    	lireSignal(en_cours,en_cours.depl+ ct.d_e * sizeof(SAMPLE),space, 0,ct.f_e - ct.d_e + 1);
    	infoEdit.derEchtFic = ct.f_e;
    }
    else return 0;
  }
  else {
    // la partie a substituer ne fait pas dborder
    // le buffer "space".

    ct.f_e = max(ct.f_e, ou + nbSample);
    space.newsize(ct.f_e - ct.d_e + 1);
     /* on lit le signal qui remplace l'ancien */
    lireSignal(infoEdit.substitute,infoEdit.substitute.depl,space, _i(ou),nbSample);
    //

    infoEdit.yaDesModifsSignal = VRAI;
  }
  ct.d_es = ct.d_a;
  ct.f_es = min(ct.f_a, ct.f_e);

  wsignal->aff();

  aff_etiqs();
  aff_notes();

  // bout avant la substituteion : conserv en place
  ajtBoutSpectro(ct.d_es, infoEdit.debut - 1,
                 ct.d_es, infoEdit.debut - 1);

  // bout subtitu :  calculer
  ajtBoutSpectro(-1 , -1, infoEdit.debut,
  					  min(ct.f_es, infoEdit.debut + nbSample - 1));
  if (infoEdit.debut + nbSample - 1 < ct.f_es) 
    // bout ventuel aprs la substitution : conserv en place
    ajtBoutSpectro(infoEdit.debut + nbSample, ct.f_a,
                   infoEdit.debut + nbSample, ct.f_a);

  spectroRapide(wspectro->wsona);

  return nbSample;
}

/*
 * Insertion d'un fichier de parole dans le fichier
 *   de WinSnoori.
 */
static long insererFichier(
	long ou,     // sample where to insert the file.
   bool force)  // if true the existing file is saved whatever the length of the insertion.
// en retour, nombre d'chantillons insrs
{
  int id = -1;
  long nbSample;

  if (!droit_ecrire_message(en_cours.nom)) return 0; 

  if (!dans(ou, ct.d_e, ct.f_e)) {
    affiche_erreur(
	 ml(1006,"insertion en dehors du buffer interdite"));
    return 0;
  }

  init_infoFile_read(&infoEdit.insert);
  if (infoEdit.insert.longueur==0) {
    affiche_warning(ml(1007,"rien  insrer"));
    return 0;
  }

  infoEdit.derOp = INSERER;
  infoEdit.debut = ou;

  nbSample = infoEdit.insert.longueur/sizeof(SAMPLE);
  infoEdit.fin = infoEdit.debut + nbSample - 1;

  if (nbSample + ct.f_e - ct.d_e + 1 > MAXECH && !force) {
    // on ne peut pas se contenter de modifier le
    // buffer de Snoori. Il faut aussi modifier
    // le fichier courant
    // On repercute donc les modifs qui n'ont pas
    // encore ete prises en compte dans les fichiers
    id = UpdateCurrentFileBox();
    switch (id) {
      case IDABORT: infoEdit.derOp = PASDOP;
      return 0;
    }
  }

  // ajout annotations et tiquetage
  { notetiqs netemp;
    // d'abord annotations
    if (isWAV(infoEdit.insert.nom)) {
      readnotetiqs(infoEdit.insert.nom, netemp,
      			    infoEdit.insert.notedepl, infoEdit.insert.notelongueur);
    }
    notes.edit_paste(infoEdit.debut,
    						infoEdit.insert.longueur/sizeof(SAMPLE), netemp);
    // puis tiquetage
    netemp.clear();
    if (isWAV(infoEdit.insert.nom)) {
      readnotetiqs(infoEdit.insert.nom, netemp,
      				 infoEdit.insert.etiqdepl, infoEdit.insert.etiqlongueur);
    }
    etiqs.edit_paste(infoEdit.debut, infoEdit.insert.longueur/sizeof(SAMPLE), netemp);
    //
    infoEdit.yaDesModifsEtiquetages=VRAI;
  }

  // insertion effective du signal
  if (force || nbSample + ct.f_e - ct.d_e + 1 > MAXECH) {
    if (force || id == IDOK || TWUpdateOrNothing().Execute() == IDOK ) {
       // la partie insre fait dborder le
      // buffer "space". On insre le fichier
      // dans le fichier courant, puis on
      // relit le son en entier.
      insertFileIntoCurrentFile(&infoEdit);
      ct.tf = en_cours.longueur / sizeof(SAMPLE);
      ct.d_e = ct.d_a;
      ct.f_e = min(ct.d_e + DUREE_FEN*en_cours.frequence - 1
                 ,min(ct.f_a,ct.tf-1) );
      infoEdit.yaDesModifsSignal = FAUX;
      space.newsize(ct.f_e-ct.d_e+1);
      lireSignal(en_cours,en_cours.depl + ct.d_e * sizeof(SAMPLE),space, 0,
      			  ct.f_e - ct.d_e + 1);
      infoEdit.derEchtFic = ct.f_e;
    }
    else return 0;
  }
  else {
    // la partie insre ne fait pas dborder
    // le buffer "space".
    long i;
    ct.f_e += nbSample;
    space.newsize(ct.f_e-ct.d_e+1);
    /* on repousse a droite le signal existant */
    for (i = _i(ct.f_e);
	 i >= _i(ou + nbSample);
	 i--)
      space[i] = space[i - nbSample]; 
    /* on lit le signal a inserer */
    lireSignal(infoEdit.insert
             ,infoEdit.insert.depl
	     ,space, _i(ou)
	     ,nbSample);
    //

    infoEdit.yaDesModifsSignal = VRAI;
  }

  ct.d_es = ct.d_a;
  ct.f_es = min(ct.f_a,ct.f_e);

  wsignal->aff();

  aff_etiqs();
  aff_notes();

  // bout avant l'insertion : conserv en place
  ajtBoutSpectro(ct.d_es, infoEdit.debut - 1,
                 ct.d_es, infoEdit.debut - 1);

  // bout insr :  calculer
  ajtBoutSpectro(-1 , -1
		, infoEdit.debut, min(ct.f_es
		, infoEdit.debut + nbSample - 1));
  if (infoEdit.debut + nbSample - 1 < ct.f_es) 
    // bout ventuel aprs l'insertion : dplac
    ajtBoutSpectro(infoEdit.debut, ct.f_a - nbSample,
                   infoEdit.debut + nbSample, ct.f_a);

  spectroRapide(wspectro->wsona);

  return nbSample;
}

/*
 * Insertion d'un fichier de parole
 */
void Wtop::CMfinserer() 
{
  if (!yaUnSignal) return;

  if (!droit_ecrire_message(en_cours.nom)) return;

  if ( (ct.debRegion > ct.finRegion)
      || !dans(ct.debRegion, ct.d_e, ct.f_e))
  { affiche_erreur(ml(1008,"Le lieu d'insertion est mal dfini"));
    return;
  }

  splitmakedir(infoEdit.insert.nom);
  strcat(infoEdit.insert.nom,"*.*");
  if (get_nom_fichier_open(PTWtop,infoEdit.insert.nom)!=IDOK) return;

  if (!isWAV(infoEdit.insert.nom)) {
    // lecture fichier non WAV

    // choisir le format du fichier de parole
    infoFile_set_noms(infoEdit.insert,infoEdit.insert.nom);
    //
    if (get_format(PTWtop,infoEdit.insert)!=IDOK) return;
  }

  souris(XC_watch);
  init_infoFile_read(&infoEdit.insert);
  { long nb;
    if ((nb = insererFichier(ct.debRegion, false))!=0)
      ct.finRegion = min(ct.debRegion + nb - 1, ct.f_es);
  }
  souris(XC_top_left_arrow);
}

//
//  Insert a file. The file must be a WAV file.
//
void insererUnFichierWAV(
  char *filename,  // the file to insert.
  int where)       // sample where the file must be inserted.
{
  long nb;
  strcpy(infoEdit.insert.nom, filename);
  souris(XC_watch);
  init_infoFile_read(&infoEdit.insert);
  if ((nb = insererFichier(where, true))!=0) {
    ct.finRegion = -INFINI_I;
    ct.debRegion = INFINI_I;
  }
  souris(XC_top_left_arrow);
}


//
// Substitute a part of the current signal with a file.
//
void substitueFichier(
  char *filename,  // fichier qui se substitue au signal existant
  int ou)  			 // chantillon ou on commence la substitution
{
  long nb;
  strcpy(infoEdit.substitute.nom, filename);
  init_infoFile_read(&infoEdit.substitute);
  nb = substituerFichier(ou, true);
}

static void sauveRegionDans(char *nomFichier)
{
  if (!droit_ecrire_message(nomFichier)) return;

  strcpy(infoEdit.save.nom, nomFichier);
  infoEdit.debut = ct.debRegion;
  infoEdit.fin = ct.finRegion;

  if (isWAV(infoEdit.save.nom))
  { infoEdit.save.frequence=en_cours.frequence;
  }
  else
  { // lecture fichier non WAV
    // choisir le format du fichier de parole
    infoFile_set_noms(infoEdit.save,infoEdit.save.nom);
    //
    if (get_format(PTWtop,infoEdit.save)!=IDOK) return;
  }
  init_infoFile_write(&infoEdit.save);  
  if (fabs(infoEdit.save.frequence
	   -en_cours.frequence)>EPSILON)
  { affiche_erreur(ml(1018,"frquence de sauvegarde non compatible ..."));
    return;
  }

  sauveRegion(&infoEdit);
}

static STRING defaultSaveName = "";
static STRING defaultExt = "";

#define ISADIGIT(x) ('0' <= (x) && (x) <= '9')
// Split name in root plus number.
void splitName(
  char *name,
  char *root,
  char *number)
{
   int i;
   for (i = strlen(name) - 1; i >= 0 && ISADIGIT(name[i]); i--);
   strncpy(number, &name[i + 1], strlen(name) - i);  
   strncpy(root, name, i + 1);
   root[i + 1] = 0;
}

void resetDefaultSaveName()
{
	strcpy(defaultSaveName, "");
}

/*
 * Sauvegarde d'une region
 */
void Wtop::CMsauverUneRegion()
{ char filename[_MAX_PATH];
  char ext[_MAX_EXT], drive[_MAX_DRIVE], name[_MAX_FNAME], dir[_MAX_DIR];

  if (!yaUnSignal) return;

  if (  (ct.debRegion>ct.finRegion)
      || !dans(ct.debRegion, ct.d_e, ct.f_e) )
  { affiche_erreur(
	ml(1010,"La rgion  sauver mal dfinie"));
    return;
  }

  _splitpath(en_cours.nom,NULL,NULL,NULL,ext);
  if (strcmp(defaultSaveName, ""))
  _makepath(filename,NULL,NULL,defaultSaveName, defaultExt);
  else _makepath(filename,NULL,NULL,"*",ext);

  if (get_nom_fichier_save(PTWtop,filename)!=IDOK) return;
  if (strcmp(en_cours.nom,filename)==0)
  { ::MessageBox(HWindow
        ,ml(1017,"filename must be different from current filename")
	,ml(1012,"sauvegarde non effectue")
		  ,MB_OK);
    return;
  }
  _splitpath(filename, drive, dir, name, ext);
  if (ISADIGIT(name[strlen(name) - 1])) {
    STRING number, root;
    int n;
    splitName(name, root, number);
    n = atoi(number) + 1;        // + 1 to prepare the next filename
    sprintf(defaultSaveName, "%s%d", root, n);
    sprintf(defaultExt, ext);
  }
  else strcpy(defaultSaveName, "");
  
  souris(XC_watch);
  sauveRegionDans(filename);
  souris(XC_top_left_arrow);
}

/*
 * Copier :
 * On sauve la region dans un fichier temporaire.
 */
void Wtop::CMfcopier()
{
  if (!yaUnSignal) return;

  if (  (ct.debRegion>ct.finRegion)
      || !dans(ct.debRegion, ct.d_e, ct.f_e) ) {
    affiche_erreur(
	ml(1010,"La rgion  sauver mal dfinie"));
    return;
  }
  souris(XC_watch);

  infoEdit.debut = ct.debRegion;
  infoEdit.fin = ct.finRegion;
  strcat(strcpy(infoEdit.cut.nom,tempdir),"cutfile.wa$");
  infoEdit.cut.nb_bits = NB_BITS; //en_cours.nb_bits;
  infoEdit.cut.depl = 0;
  infoEdit.cut.swap = SWAPINFILE; 
  infoEdit.cut.frequence=en_cours.frequence;
  copieDansCutFile(&infoEdit);
  souris(XC_top_left_arrow);
}

static void collerCoupe(long ou)
{
  // on insere tout betement le fichier de couper
  //   (ou de copier) 

  strcpy(infoEdit.insert.nom,infoEdit.cut.nom);
  ct.debRegion=ou;
  { long nb;
    if ((nb = insererFichier( //&infoEdit,
                            ct.debRegion, false))!=0)
      ct.finRegion=min(ct.debRegion+nb-1,ct.f_es);
  }
}

/*
 * Coller :
 * On insere le fichier cree par copier ou par couper.
 */
void Wtop::CMfcoller()
{
  if (!yaUnSignal) return;

  if (!droit_ecrire_message(en_cours.nom)) return;

  if (  (ct.debRegion>ct.finRegion)
      || !dans(ct.debRegion, ct.d_e, ct.f_e) )
  { affiche_erreur(
	ml(1010,"La rgion  sauver mal dfinie"));
    return;
  }
  souris(XC_watch);

  strcat(strcpy(infoEdit.cut.nom,tempdir),"cutfile.wa$");
  collerCoupe(ct.debRegion);
  ::InvalidateRect(HWindow,NULL,TRUE);
  souris(XC_top_left_arrow);
}
  
/*
 * Jouer le fichier complet.
 */
void Wtop::CMplayFile()
{
  if (!yaUnSignal) return;
  if (isWAV(en_cours.nom))
  	 ::PlaySound(en_cours.nom, 0, SND_ASYNC | SND_NOWAIT | SND_FILENAME);
  else {
    STRING tmpFile;
    writeSoundFile wS;
    struct infoFile dstinfo = en_cours;

    strcpy(tmpFile,tempdir);
    strcat(tmpFile,"tmp.wav");
    dstinfo.swap=SWAPINFILE;
    dstinfo.nb_bits=NB_BITS;
    strcpy(dstinfo.nom, tmpFile);

    wS.dstinfo = &dstinfo;
    wS.srcfile(&en_cours, en_cours.depl, en_cours.longueur, 0, 0);
    wS.flush();
    ::PlaySound(tmpFile, 0, SND_ASYNC | SND_NOWAIT | SND_FILENAME);
    unlink(tempdir);
  }
}

/*
 * Sauvegarde du fichier courant sous un autre nom.
 */
void Wtop::CMsauverSous() 
{ char filename[_MAX_PATH];
  char ext[_MAX_EXT];

  if (!yaUnSignal) return;

  _splitpath(en_cours.nom,NULL,NULL,NULL,ext);
  _makepath(filename,NULL,NULL,"*","");
//  _makepath(filename,NULL,NULL,"*",ext);

  if (get_nom_fichier_save(PTWtop,filename)!=IDOK) return;
  if (strcmp(en_cours.nom,filename)==0)
  { ::MessageBox(HWindow
        ,ml(1017,"filename must be different from current filename")
	,ml(1012,"sauvegarde non effectue")
        ,MB_OK); 
    return;
  }
  if (!droit_ecrire_message(filename)) return;

  strcpy(infoEdit.save.nom,filename);
  if (isWAV(infoEdit.save.nom))
  { infoEdit.save.frequence=en_cours.frequence;
  }
  else
  { // lecture fichier non WAV
    // choisir le format du fichier de parole
    infoFile_set_noms(infoEdit.save,infoEdit.save.nom);
    //
    if (get_format(PTWtop,infoEdit.save)!=IDOK) return;
  }
  init_infoFile_write(&infoEdit.save);
  if (fabs(infoEdit.save.frequence
	   -en_cours.frequence)>EPSILON)
  { affiche_erreur(ml(1018,"frquence de sauvegarde non compatible ..."));
    return;
  }

  souris(XC_watch);
  repercuteModif(&infoEdit); 

  if (!isWAV(infoEdit.save.nom))
  { if (infoEdit.save.fmt_phon!=0)
    { if ((isEtiqModifiable
	   =droit_ecrire_message(infoEdit.save.nom_man)
          )==TRUE)
        sauvman(&etiqs, infoEdit.save);
    }
    if (infoEdit.save.fmt_word!=0)
    { if ((isNoteModifiable
	   =droit_ecrire_message(infoEdit.save.nomAnnot)
	  )==TRUE)
	sauveAnnotations(&notes, infoEdit.save);
    }
  }
  
  PTWtop->SetNomFichierParole(en_cours.nom); 
  active(PPMENUAllStandard);
  souris(XC_top_left_arrow);
}

void Wtop::CMsauver()
{ 
  if (!yaUnSignal) return;
  if (!droit_ecrire_message(en_cours.nom)) return;

  souris(XC_watch);
  UpdateCurrentFile();
  souris(XC_top_left_arrow);
}

void Wtop::CMrecord()
{ 
   record();
}

//
// Remplacement d'une region par le fichier construit
//   par couper
//
// ne touche pas aux tiquetages
static void AnnulerEnRemplacant()
{ long nbSample;

  init_infoFile_read(&infoEdit.cut);
  nbSample=infoEdit.cut.longueur/sizeof(SAMPLE);

  // Il suffit simplement de recopier le signal qui
  //   a ete sauve lors du dernier amortir 
  lireSignal( infoEdit.cut, infoEdit.cut.depl
	    ,space, _i(infoEdit.debut)
	    ,nbSample);

  // bout avant la zone  remplacer : conserv en place
  ajtBoutSpectro(ct.d_a, infoEdit.debut - 1,
                 ct.d_a, infoEdit.debut - 1);  
  // bout aprs la zone  remplacer : conserv en place
  ajtBoutSpectro(infoEdit.debut + nbSample + 1, ct.f_a,
                 infoEdit.debut + nbSample + 1, ct.f_a);
  // bout de la zone  remplacer  la mme place
  ajtBoutSpectro(-1, -1, infoEdit.debut, infoEdit.debut + nbSample);

  wsignal->aff();
  wsignal->paint();
  spectroRapide(wspectro->wsona);

  infoEdit.yaDesModifsSignal = VRAI;
}  

/*
 * Annulation de la derniere commande d'edition.
 * La derniere commande est annulable a condition de
 * rester dans l'editeur.
 */
void Wtop::CMfannuler()
{
  annulerModif();
}

void annulerModif()
{
  if (infoEdit.derOp == PASDOP) return;
  souris(XC_watch);
  switch (infoEdit.derOp)
  { case COUPER :
    case EFFACER :
      collerCoupe(infoEdit.debut);
      break;
    case INSERER :
      strcat(strcpy(infoEdit.cut.nom,tempdir),"cutfile_.wa$");
		couperRegion(infoEdit.debut, infoEdit.fin);
      break;
    case AMORTIRG :
    case AMORTIRD :
    case AMORTIRC :
    case FILTRER :
    case FILTEROLA :
    case NOISE :
    case ADDINGVOICING :
      AnnulerEnRemplacant();
      break;
  } 
  //
  infoEdit.derOp = PASDOP;
  ::InvalidateRect(PTWtop->HWindow,NULL,TRUE);
  souris(XC_top_left_arrow);
}

/*
 * Filtrage d'une region.
 */
void filtrage(
	struct profiltre *filt,
	long deb,
	long fin)
{
  if (!yaUnSignal) return;
  if (!droit_ecrire_message(en_cours.nom)) return;

  if (deb > fin || !dans(deb, ct.d_e, ct.f_e)
		|| !dans(fin, ct.d_e, ct.f_e))
  { affiche_erreur(ml(1015,"La rgion  filtrer mal dfinie"));
    return;
  }
  souris(XC_watch);

  infoEdit.derOp = FILTRER;
  infoEdit.debut = deb;
  infoEdit.fin = fin;
  strcat(strcpy(infoEdit.cut.nom,tempdir),"cutfile_.wa$");
  infoEdit.cut.nb_bits = NB_BITS; 
  infoEdit.cut.depl = 0;
  infoEdit.cut.swap = SWAPINFILE; 

  /* On sauve le signal qui va etre coupe */
  copieDansCutFile(&infoEdit);

  filtrer(space, space, _i(deb), _i(fin),
	  *filt); 

  // bout avant la zone filtre : conserv
  ajtBoutSpectro(ct.d_a, deb - 1,
		 ct.d_a, deb - 1);
  // bout aprs le signal filtr : conserv  
  ajtBoutSpectro(fin + 2, ct.f_a,
		 fin + 2, ct.f_a);
  // bout filtr :  calculer
  ajtBoutSpectro(-1, -1, deb, fin+1);

  wsignal->aff();
  wsignal->paint();
  spectroRapide(wspectro->wsona);
  
  infoEdit.yaDesModifsSignal = VRAI;
  ct.debRegion = INFINI_L;
  ct.finRegion = -INFINI_L;
  souris(XC_top_left_arrow);
}

/*
 * Amortissement du signal
 */
void amortir(
	long deb, long fin,
	int cote, // AMORTIRG ou AMORTIRD ou AMORTIRC 
        float gain)
{ long tailleRegion = fin - deb + 1;
  long il,jl;

  if (!droit_ecrire_message(en_cours.nom)) return;

  if (deb > fin || !dans(deb, ct.d_e, ct.f_e)
		|| !dans(fin, ct.d_e, ct.f_e))
  { affiche_erreur(ml(1016,"La rgion  amortir mal dfinie"));
    return;
  }

  infoEdit.derOp = cote;
  infoEdit.debut = deb;
  infoEdit.fin = fin;
  strcat(strcpy(infoEdit.cut.nom,tempdir),"cutfile_.wa$");
  infoEdit.cut.nb_bits = NB_BITS; 
  infoEdit.cut.depl = 0;
  infoEdit.cut.swap = SWAPINFILE; 

  /* On sauve le signal qui va etre coupe */
  copieDansCutFile(&infoEdit);

  /* Amortissement de la region */
  if (cote == AMORTIRG)
    for (il = _i(deb), jl = 0; il <= _i(fin); il++, jl++)
      space[il] = (SAMPLE) (((float)space[il]) * (1. + sin(jl /((float)tailleRegion) * _PI - _PI / 2)) / 2.);
  else if (cote == AMORTIRD)
    for (il = _i(deb), jl = tailleRegion; il <= _i(fin); il++, jl--)
      space[il] = (SAMPLE) (((float)space[il]) * (1. + sin(jl /((float)tailleRegion) * _PI - _PI / 2)) / 2.);
  else {
    // Le signal est multiplie par le gain au centre
    //   de la region a amortir. L'amortissement passe
    //   de 1 a la valeur du gain dans les zones
    //   frontieres gauche et droite.
    long frontiere = min(enEcht(20), tailleRegion / 3)
        , debG, finD, jl ,kl;
    if (frontiere) {
      debG = _i(deb) + frontiere;
      finD = _i(fin) - frontiere;
      for (il = 0, jl = _i(deb), kl = _i(fin);
	   il < frontiere;
	   il++, jl++, kl--) {
       	float r;
			r = 1.-( sin(il/((float)frontiere)*_PI - _PI / 2.) + 1.) / 2. * (1. - gain);
			space[jl] = (SAMPLE) (((float) space[jl]) * r);
			space[kl] = (SAMPLE) (((float) space[kl]) * r);
      }
    }
    else {
      debG = _i(deb);
      finD = _i(fin);
    }
    for (il = debG; il <= finD; il++)
      space[il] = (SAMPLE) ( ((float) space[il]) * gain);
  }

  // bout avant la zone amortie : conserv en place
  ajtBoutSpectro(ct.d_a, infoEdit.debut - 1, ct.d_a, infoEdit.debut - 1);  
  // bout aprs la zone amortie : conserv en place
  ajtBoutSpectro(infoEdit.debut + tailleRegion + 1, ct.f_a,
		 infoEdit.debut + tailleRegion + 1, ct.f_a);
  // bout amorti : recalcul
  ajtBoutSpectro(-1, -1, infoEdit.debut, infoEdit.debut + tailleRegion);

  wsignal->aff();
  wsignal->paint();
  spectroRapide(wspectro->wsona);
  
  infoEdit.yaDesModifsSignal = VRAI;
  ct.debRegion = INFINI_L;
  ct.finRegion = -INFINI_L;
}

/*
 * Amortissement a gauche
 */
void Wtop::CMamortirGauche()
{ if (!yaUnSignal) return;
  souris(XC_watch);
  amortir(ct.debRegion, ct.finRegion, AMORTIRG,1.);
  souris(XC_top_left_arrow);
}

/*
 * Amortissement a droite
 */
void Wtop::CMamortirDroite()
{ if (!yaUnSignal) return;
  souris(XC_watch);
  amortir(ct.debRegion, ct.finRegion, AMORTIRD,1);
  souris(XC_top_left_arrow);
}

/*
 * Amortissement au centre
 */
void Wtop::CMamortirCentre()
{ char ch1[80];

  if (!yaUnSignal) return;
  strcpy(ch1,"1.");
  if ( MyInputDialog(
		ml(1030,"amortir au centre")
	       ,ml(1031,"entrer le gain (<1 pour amortir)")
               ,ch1,78 ) )
  { souris(XC_watch);
    amortir(ct.debRegion, ct.finRegion, AMORTIRC, atof(ch1));
    souris(XC_top_left_arrow);
  }
}
