/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier desfil.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "filtre.h"

profiltre::profiltre()
{ f=new FLOATbuf(0,"profiltre");
  strcpy(type,"no");
  fl=fh=fc=att=0.;
  ordre=0;
}
profiltre::~profiltre()
{ delete f;
}
void profiltre::newsize(int size)
{ ordre=size;
  f->newsize(size);
}

static void ripple(
	int nr,
	float rideal,
	float flow,
	float fhi,
	FLOATbuf& resp, 
	float *f1,
	float *f2,
	float *db,
	float *xi)
{
	int i, j, iflow, ifhi;
	float aux, xnr, rmax, rmin, x1, x0, yy1, yy0, ripl;


	xnr = nr;
	/*  band limits */
	iflow = (int)(2 * xnr * flow + 1.5);
	ifhi = (int)(2 * xnr * fhi + 1.5);
	if (iflow == 0)
		iflow = 1;
	if (ifhi >= nr)
		ifhi = nr - 1;
	/*  find  max  and  min  peaks in band  */
	rmin = rideal;
	rmax = rideal;
	resp[0] = resp[1];
	for (i = iflow ;i <= ifhi ;i++) {
		if (!((resp[i] <= rmax) || (resp[i] < resp[i - 1])
		    || (resp[i] < resp[i + 1])))
			rmax = resp[i];
		if (!((resp[i] >= rmin) || (resp[i] > resp[i - 1])
		    || (resp[i] > resp[i + 1])))
			rmin = resp[i];
	}
	/* peak deviation from ideal  */
	if (rmax - rideal > rideal - rmin)
		ripl = rmax - rideal;
	else 
		ripl = rideal - rmin;
	/* search for lower band range */
	*f1 = flow;
	if (fabs(flow) > EPSILON)  {
		i = iflow - 1;
		do {
			i = i + 1;
		} while (!( (fabs(resp[i] - rideal) <= ripl)
		            || (i == ifhi)  ));
		*xi = i - 1;
		/*  interpolation  */
		x1 = 0.5 * *xi / xnr;
		x0 = 0.5 * (*xi - 1) / xnr;
		yy1 = fabs(resp[i] - rideal);
		yy0 = fabs(resp[i - 1] - rideal);
		if (fabs(yy1 - yy0) < EPSILON) *f1 = INFINI_F;
		else *f1 = (x1 - x0) / (yy1 - yy0) * (ripl - yy0) + x0;
	}
	/*  search for  upper band edg  */
	*f2 = fhi;
	if (fabs(fhi-0.5) > EPSILON)  {
		i = iflow - 1;
		do {
			i = i + 1;
			j = ifhi + iflow - i;
		} while (!( (fabs(resp[j] - rideal) <= ripl)
		            || (i == ifhi) ));
		*xi = j - 1;
		/*  interpolation  */
		x1 = 0.5 * *xi / xnr;
		x0 = 0.5 * (*xi + 1) / xnr;
		yy1 = fabs(resp[j] - rideal);
		yy0 = fabs(resp[j + 1] - rideal);
		if (fabs(yy1 - yy0) < EPSILON) *f2 = INFINI_F;
		else *f2 = (x1 - x0) / (yy1 - yy0) * (ripl - yy0) + x0;
	}
	/*  deviation form ideal in db      */
	aux = ripl + rideal;
	if (aux <= 0)
		aux = 1.0e-20;
	*db = (float) (20. * log((double)aux) / log((double)10.));
	printf("\n"); 
}						/*  ripple  */



static int rdd(int ent)
{
	int aii;
	char st[MAX_LIGNE];
	char fmt[MAX_LIGNE];

	if (ent == 0)
		aii = 1;
	else 
		aii = (int) ( (log((double)abs(ent)) / log((double)10.))
		              + 0.5 );
	if (ent < 0) aii = aii + 1;
	sprintf(fmt,"[%%d] ",aii); 
	printf(fmt,ent);
	scanf("%s",st);
	if (strlen(st) == 0)
		return(ent);
	else return(atoi(st));
}



#define T   1.0e-08
static float ino(float x)
/*  bessel  function   */
{
	float y, de, e, sde;
	int xi, i;


	y = x / 2;
	e = 1;
	de = 1;
	i = 1;
	do {
		xi = i;
		de = de * y / xi;
		sde = de * de;
		e = e + sde;
		i = i + 1;
	} while (!( (e * T - sde > 0) || (i > 25) ));
	return(e);
}				/*  ino  */

static void kaiser(
	int nf,
	FLOATbuf& w,
	int n,
	int ieo,
	float beta)
{
	float xind, xi;
	int i;


	xind = (nf - 1) * (nf - 1);
	for (i = 1; i<=  n; i++) {
		xi = i - 1;
		if (ieo == 0)
			xi = xi + 0.5;
		xi = 4 * xi * xi;
		w[i] = ino((float)(beta * sqrt((double)(1. - xi / xind))));
		w[i] = w[i] / ino((float)beta);
	}
}				/*  kaiser   */


static void flchar(
	int nf,
	char jtype[MAX_LIGNE],
	float fc,
	float fl,
	float fh,
	int n,
	int ieo,
	FLOATbuf& g)
{
	FLOATbuf resp(2048+1,"resp in flchar() near desfir()");
	int xnr, nr, i, j;
	float xi, xj, sum, sumi, twn, twni, f1, f2, db;


	/*    d.f.t. 4096 au plus  */
	nr = 8 * nf;
	if (nr > 2048)
		nr = 2048;
	xnr = nr;
	twn = _PI / xnr;
	sumi = -(g[1] / 2);
	if (ieo == 0)
		sumi = 0;
	for (i = 1 ;i <= nr ;i++) {
		xi = i - 1;
		twni = twn * xi;
		sum = sumi;
		for (j = 1 ;j <= n ;j++) {
			xj = j - 1;
			if (ieo == 0)
				xj = xj + 0.5;
			sum = (float)(sum + g[j] * cos((double)xj * twni));
		}
		resp[i] = 2 * sum;
	}
  if (!strcmp(jtype,"lp"))
  {			/*  lowpass  */
    ripple(nr, 1., 0., fc, resp, &f1, &f2, &db, &xi);
    printf("passband cutoff [lp] : %8.4f     ripple : %6.2f db\n",f2,db);
    ripple(nr, 0., fc, 0.5, resp, &f1, &f2, &db, &xi);
    printf("stopband cutoff [lp] : %8.4f     ripple : %6.2f db\n",f1,db);
  }
  else
  if (!strcmp(jtype , "hp"))
  {			/* high pass */
    ripple(nr, 0., 0., fc, resp, &f1, &f2, &db, &xi);
    printf("stopband cutoff [hp] : %8.4f     ripple : %6.2f db\n",f2,db);
    ripple(nr, 1., fc, 0.5, resp, &f1, &f2, &db, &xi);
    printf("passband cutoff [hp] : %8.4f     ripple : %6.2f db\n",f1,db);
  }
  else
  if (!strcmp(jtype , "bp"))
  {			/* band pass */
    ripple(nr, 0., 0., fl, resp, &f1, &f2, &db, &xi);
    printf("stopband cutoff [bp] : %8.4f     ripple : %6.2f db\n",f2,db);
    ripple(nr, 1., fl, fh, resp, &f1, &f2, &db, &xi);
    printf("passband cutoff [bp] : %8.4f  %8.4f    ripple : %6.2f db\n",
	    f1,f2,db);
    ripple(nr, 0., fh, 0.5, resp, &f1, &f2, &db, &xi);
    printf("stopband cutoff [bp] : %8.4f     ripple : %6.2f db\n",f1,db);
  }
  else
  if (!strcmp(jtype , "bs"))
  {			/* band stop */
    ripple(nr, 1., 0., fl, resp, &f1, &f2, &db, &xi);
    printf("passband cutoff [bs] : %8.4f     ripple : %6.2f db\n",f2,db);
    ripple(nr, 0., fl, fh, resp, &f1, &f2, &db, &xi);
    printf("stopband cutoff [bs] : %8.4f  %8.4f    ripple : %6.2f db\n",
		    f1,f2,db);
    ripple(nr, 1., fh, 0.5, resp, &f1, &f2, &db, &xi);
    printf("passband cutoff [bs] : %8.4f     ripple : %6.2f db\n",f1,db);
  }
} /*  flchar  */

static void designfir(
	FLOATbuf& g,
	int *nf,
	int specif,
	struct profiltre *filt)
/*---------------------------------------------------------------------*/
/* les coefficients sont dans g;  nf est  l"ordre du filtre            */
/* si   specifit=vrai    calculer  caracteristiques reelles du filtre  */
/*_____________________________________________________________________*/


{
	int n, ieo, i1, i, k, j, win;
	float fc, fl, fh, xn ;
	char jtype[MAX_LIGNE];
	float c, c1, c3, att, beta ;
	FLOATbuf w(NMAX+1,"w in designfir() near desfir()");
	char lis[MAX_LIGNE];
	int bv = filt == NULL;

	if (bv)
	{ do {
	    *nf = 20;
	    printf("filter length (number of coefficients) ? ");
	    fflush(stdout);
	    *nf = rdd(*nf);
	  } while (!((*nf <= 1024) && (*nf > 3)));
	  printf("filter type : lowpass(lp); highpass(hp); \
	               bandpass(bp); bandstop(bs)\n");
	  do {
		printf("lp; hp; bp or bs  ? ");
		scanf("%s",jtype);
	  } while (!(   !strcmp(jtype, "lp")
		     || !strcmp(jtype , "hp")
		     || !strcmp(jtype , "bp")
		     || !strcmp(jtype , "bs")));
        }
	else
	{
		strcpy(jtype,filt->type);
		*nf = filt->ordre;
		if (!strcmp(jtype , "bp") || !strcmp(jtype , "bs")) {
		   fl = filt->fl;
		   fh = filt->fh;
		}
		else fc = filt->fc;
		att = filt->att;
	}		

	n = (*nf + 1) / 2;
	if (bv)
	{ if (   !strcmp(jtype , "bp")
	      || !strcmp(jtype , "bs"))
	  {
	    printf("specify  the  cutoff  frequencies :");
	    fl = 0.15;
	    fh = 0.35;
	    do {
	      printf("lower  frequency [0..0.5] ? ");
	      scanf("%f",&fl);
	      printf("high frequency [0..0.5] ? ");
	      scanf("%f",&fh);
	    } while (!((fl > 0) && (fl < 0.5) && (fh > 0)
		     && (fh < 0.5) && (fh > fl)));
	  }
	  else
	  { do
	    { fc = 0.25;
	      printf("cutoff frequency [0..0.5] ? ");
	      scanf("%f",&fc);
	    } while (!((fc > 0) && (fc < 0.5)));
	  }
	}

	ieo = *nf % 2;
	if (  (ieo == 0)
	    && strcmp(jtype ,"lp")
	    && strcmp(jtype,"bp"))
	{
		if (bv)
		{
		  printf("\n");
		  printf("filter length should be even \
		             for  hp and  bs\n");
		  printf("so :  1  has automatically added in.\n");
		  printf("\n");
		}
	        *nf = *nf + 1;
		n = (*nf + 1) / 2;
		ieo = 1;
	}

	c1 = fc;
	if (   !strcmp(jtype , "bp")
	    || !strcmp(jtype , "bs"))
	  c1 = fh - fl;
	if (ieo == 1)
	  g[1] = 2 * c1;
	i1 = ieo + 1;
	for (i = i1 ;i <= n ;i++)
	{	xn = i - 1;
		if (ieo == 0)
			xn = xn + 0.5;
		c = _PI * xn;
		c3 = c * c1;
		if (!strcmp(jtype , "lp") || !strcmp(jtype , "hp"))
			c3 = c3 * 2;
		g[i] = (float) sin((double)c3) / c;
		if (!strcmp(jtype , "bp") || !strcmp(jtype , "bs"))
		  g[i] = (float) (g[i] * 2 * cos((double)c * (fl + fh)));
	}

	/*  kaiser  (io_sinh)  fenetre  */
	for (i = 1 ;i <= n ;i++)
	  w[i] = 1;
	if (bv)
	{ att = 40.;
	  printf("attenuation  in  db  ? ");
	  scanf("%f",&att);
	}
	if (att > 50)
	  beta = 0.1102 * (att - 8.7);
	if ((att >= 20.96) && (att <= 50))
	  beta = (float) exp(log( (double)0.58417
	     			* (att - 20.96))
				   * 0.4)
			       + 0.07886 * (att - 20.96);
	if (att < 20.96)
	  beta = 0;
	win = 1;
	if (bv)
	{ printf("window type :[1] kaiser  [2] others  ?  ");
	  win = rdd(win);
	}
	if (win == 1)
	  kaiser(*nf, w, n, ieo, beta);
	else
	{	if (bv) printf("sorry. not implemented yet. \
		                    kaiser used.\n");
		kaiser(*nf, w, n, ieo, beta);
	}

	for (i = 1 ;i <= n ;i++)
		g[i] = g[i] * w[i];
	if (bv)
	{ printf("list window (y/n)  ? ");
	  scanf("%s",lis);
	  if (lis[0] == 'y')
	  {     printf("*  window    :  points= %d\n",*nf);
		printf("\n");
		printf("attenuation= %6.2f db        beta= %8.6f\n"
		             ,att,beta);
		printf("\n");
		for (i = 1 ;i <= n ;i++)
		{       j = n + 1 - i;
			k = *nf + 1 - i;
			printf("w[%3d] = %g = w[%3d]\n",i,w[j],k);
		}
	  }
	  printf("\n");

	  if (!strcmp(jtype , "lp"))
	  { printf("fir low pass filter : \n");
	    printf("cutoff frequency   = %8.4f\n",fc);
	  }
	  if (!strcmp(jtype , "hp"))
	  { printf("fir high pas filter : ");
	    printf("cutoff frequency = %8.4f\n",fc);
	  }
	  if (!strcmp(jtype , "bp"))
	  { printf("fir bandpass filter : \n");
	    printf("cutoffs frequencies = %8.4f  %8.4f\n",fl,fh);
	  }
	  if (!strcmp(jtype , "bs"))
	  { printf("fir bandstop filter : \n");
	    printf("cutoffs frequencies = %8.4f  %8.4f\n",fl,fh);
	  }
        }

	if (   strcmp(jtype,"lp")
	    && strcmp(jtype,"bp"))
	{ for (i = 2 ;i <= n ;i++)
			g[i] = -g[i];
	  g[1] = 1.0 - g[1];
	}
	/*  lister   reponse  impulsionnelle  */
	if (bv)
	{ printf("\n");
	  printf("* impulsion response of the filter :");
	  printf("\n");
	  for (i = 1 ;i <= n ;i++)
	  { j = n + 1 - i;
	    k = *nf + 1 - i;
	    printf("h[%3d] = %g  = h[%3d]\n",i,g[i],k);
	  }
	  if (specif)
		flchar(*nf, jtype, fc, fl, fh, n, ieo, g);
	  printf("\n");
	}
}				/*  designfir   */


/////////////////////////////////////////////////////
//   conception de filtre fir et ecriture de la
//       reponse impulsionnelle en format
//       virgule flottante dans un fichier
//
void desfir(struct profiltre *filt)
	// Auparavant, il tait possible d'appeler
	// desfir() sans donner le profil du filtre
	// (appel par desfir(NULL) ), 
	// et cette fonction tait alors bavarde.
	// (avec des printf et des scanf).
	// Ceci n'est plus possible.
	// On pourrait rcuprer ces fonctionnalits
	// dans de nouvelles fonctions.
	// Vrai aussi pour designfir()
{
	FLOATbuf sd(NMAX+1,"sd in desfir()");
	int i, j, k, n, nf; //, ii;
	FLOATbuf x(1024,"x in desfir()");
	FILE *fit;     /* text */
	int sp;
	char st[MAX_LIGNE];
	float maxf;

	if (!filt)
	{
	  printf( 
	    "name of the ASCII file for the resulting coefficients : ? ");
	  fflush(stdout);
	  scanf("%s",st);
	  fit = fopen(st, "w");
	  printf("compute specification (y/n) ? ");
	  scanf("%s",st);
	  if ( st[0] == 'y')  sp = VRAI; 
	  else sp = FAUX;
        }

	designfir(sd,&nf, sp, filt);
	n = (nf + 1) / 2;
	maxf = sd[1];
	for (i = 1 ;i <= n ;i++) {
		j = n + 1 - i;
		if (sd[j] > maxf)
			maxf = sd[j];
	}
	for (i = nf ;i <= 1023 ;i++)
		x[i] = 0;
	for (i = 1 ;i <= n ;i++) {
		j = n + 1 - i;
		k = nf + 1 - i;
		x[i - 1] = sd[j];
		x[k - 1] = sd[j];
	}
	i = nf;
	if (!filt) fprintf(fit,"%3d\n",i);
	else
	  filt->ordre = nf;
	if (!filt) for (i = 0 ;i <= nf-1 ;i++) 
		       fprintf(fit,"%20.16f\n", x[i]);
	else
	{ filt->newsize(nf);
	  for (i = 0; i < nf; i++)
	    (*filt->f)[i] = x[i];
	}
	if (!filt) fclose(fit);
}		/*   appelfir   */


