/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier div.cpp
//
                                                         
#include "common.h"
#pragma hdrstop

#include <dirent.h>
#include <malloc.h> // pour stackavail()

#include "audio.h"
#include "spectro.h"
#include "format_f.h" // pour formatdir
#include "files.h"
#include "outils.h"

bool indtest = false;
#ifdef LICENCE
bool Licensed = false;
#else
bool Licensed = true;
#endif
int debug=0,subdebug=0;


int MyTextHeight=0;
int MyTextWidth=0;
char curdir[_MAX_DIR];
char tempdir[_MAX_DIR];
STRING helpfile;
STRING logfile;
STRING CountryFile;
STRING LanguageFile;
char Organization[MAX_LIGNE];
char Password[80];
char PhoneticFont[80];
char SndRecorder[100];
char formatdir[_MAX_DIR];
char WinSnooriDirectory[_MAX_DIR];


choix choix_YESNO[] = { {0,(char *)NULL} //NO
		       ,{1,(char *)NULL} //YES
		       ,{0,""} };


/////////////////////////////////////////////////
// arrt d'un traitement au clavier
//
//   les traitements longs sont arrts si
//          (GetInputState()!=0)
//   ce qui correspond  un caractre tap au clavier
//       ou  un bouton souris enfonc
//   on conseille  l'utilisateur d'utiliser la
//       barre d'espace pour arrter un traitement
//

/////////////////////////////////////////////////
void init_choix_YESNO()
{ choix_YESNO[0].text=ml(215,"NO");
  choix_YESNO[1].text=ml(216,"YES");
}

/*
Les objets ditables de WinSnoori

Dfinition; par objet ditable, on veut dire un objet contenant 
des lments temporels classs par date croissantes, et sur lequel 
on dispose des fonctions couper copier coller.

Les objets ditables sont typiquement faits pour tre visualiss 
avec le signal et le spectrogramme. Les dates sont les numros 
d'chantillon.

Modle C++ des objets ditables:
class Object // l'objet ditable
{ long datelength;
  long n; // nombre d'lments dans le tableau TheElements
  HANDLE h; // handle mmoire du tableau TheElements
  ObjectElement huge **TheElements;

  Object();
  ~Object();
  .
  .

  // fonctions de base de l'diteur 
  edit_cut(long firstdate,long lastdate,Object& object);
  edit_copy(long firstdate,long lastdate,Object& object);
  edit_paste(long date,Object& object);
  edit_delete(long firstdate,long lastdate);

  // autres fonctions
  edit_paste_null(long date,long length); // insre un objet vide
};
class ObjectElement // un lment temporel avec sa date
{ long date; 
  Element element; 
};
class Element // un lment temporel sans sa date
{ // spcifique de chaque objet
  //   pointeur, structure ou variables
};

prcisions sur les fonctions edit_xxx:
  - elles sont redondantes, au sens ou edit_cut(debut,fin,objet) 
  est quivalent :
        edit_copy(debut,fin,objet);
        edit_delete(debut,fin);
  - datelength est la taille temporelle de l'objet. Les dates 
  extrmes de l'objet sont 0 et datelength-1. Les dates des 
  lments de l'objet doivent imprativement tre comprises 
  entre 0 et datelength-1.

liste des objets ditables de WinSnoori:
  signal
    nom: space + ct + en_cours + infoEdit
    remarque: non conforme  la dfinition de la classe Object
  spectro
    nom: sps + cran
    remarque: non conforme  la dfinition de la classe Object
  tiquetage phontique
    nom: etiqs
  tiquetage orthographique
    nom: notes
  pitch
    nom: pm + pitchCorrige ( revoir)
    question: 1 objet ou 3 objets
                  (masculin, fminin, n'importe qui)
  pics de cepstre
    nom: pics_cepstre
  racines de LPC
    nom: les_raclpc
  formants McCandless
    nom: f123s
  formants dits manuellement
    nom: f123se ( crer)

fonctions de WinSnoori lies aux objets:
  - affichage des objets surimposables au spectrogramme.
  - slectivit de l'affichage de ces objets.
          ==> nouvelles commandes menu
  - effacement (remise  zro) de ces objets.
	  ==> nouvelles commandes menu
  - bon suivi de ces objets aux couper coller du signal.

questions lies au dveloppement des fonctions ci-dessus:
  - quand on calcule une nouvelle rgion pour un objet,
  efface-t-on l'ancien objet, ou le complte-t-on par
  le nouveau calcul ?
  - que fait-on sur les objets pour les dates qui ne
  sont pas affichables sur l'cran, et sur celles qui
  sont en dehors du buffer de WinSnoori (le buffer
  space) ?
*/

/////////////////////////////////////////////////
// dbut CountryFile
// traitement de la langue (menus et messages)
//   pour la "nationalisation" de WinSnoori
//
// caractres spciaux des textes du fichier langue
//     \n  return
//     \\  backslash
//     \"  double quote
//     \   suite sur ligne suivante
//             le cas normal dans ce cas est d'utiliser
//             le caractre \ en dernier caractre de
//             la ligne. Cependant \ suivi d'autre chose
//             que n \ " (lettre n, backslash, double quote)
//             indique que le texte continu sur
//             la ligne suivante (ds le 1er caractre).
//
// fonctions donnes dans le fichier .h
//   ml()  c'est la fonction normale  utiliser
//   mls()   utiliser pour un texte qui n'est pas 
//          traduire, mais que l'on souhaite retrouver
//          par grep plus tard
//   mln()  procdure d'excution des ml()
//          ne doit pas tre appele directement par l'utilisateur
//   ml_init()  appel une seule fois au lancement de WinSnoori
//
class ml_tabl_
// classe de stockage des textes
{ public:
    char *pc;
    unsigned int npc; // taille de pc utilise
    int *pn;
    unsigned int *pi;
    int npi; // taille de pi et pn utilise
    //   les tailles alloues pour hpn et hpi
    //   sont les mmes

    ml_tabl_();
    ~ml_tabl_();
    void clear();
    bool add_elt(int num,char *text);
    bool find_elt(int num,char **ptext);
};
ml_tabl_::ml_tabl_()
{
  int i;

  npc=0;
  pc = (char *)malloc(1);
  npi=1003; // paramtre sensible
	    // qui doit tre un nombre premier
	    // d'environ 2 fois le nombre de textes ml()
  pn=(int *)malloc(npi*sizeof(int));
  pi=(unsigned int *)malloc(npi*sizeof(int));
  for (i=0;i<npi;i++) pn[i]=0;
}
ml_tabl_::~ml_tabl_()
{
  if (pc) free(pc);
  if (pn) free(pn);
  if (pi) free(pi);

  pc=0;
  pn=0;
  pi=0;
}
void ml_tabl_::clear()
{
	//this->~ml_tabl_();
   npc=1;
   pc = (char*) realloc(pc,1);
   for (int i=0;i<npi;i++) pn[i]=0;
}
bool ml_tabl_::add_elt(int num,char *text)
{ int i,j,k,len;
  char ch[80];

  len=strlen(text);
  if (npc+len+10>sizeof(pc))
    pc=(char *)realloc(pc,npc+len+200);
  strcpy(pc+npc,text);
  //
  i=num%npi;
  for (j=0;j<npi;j++)
  { k=(j+i)%npi;
    if (pn[k]==0) break;
    if (pn[k]==num)
    { sprintf(ch,"%s : number %d defined more than once"
                ,CountryFile,num);
      affiche_erreur(ch);
      break;
    }
  }
  if (j==npi)
  { // on a fait un tour complet
    // toutes les cases sont occupes
    fatal_error("ml_tabl too small");
    return FALSE;
  }
  pi[k]=npc;
  pn[k]=num;
  //
  npc+=len+1;

  return TRUE;
}
bool ml_tabl_::find_elt(int num,char **ptext)
{ int i,j,k;

  i=num%npi;
  for (j=0;j<npi;j++)
  {
    k=(j+i)%npi;
    if (pn[k]==num) break;
  }
  if (j==npi)
  {
    // on a fait un tour complet
    // aucun lment ne correspond
    return FALSE;
  }
  *ptext=pc+pi[k];
  return TRUE;
}


ml_tabl_ * ml_tabl = NULL;


char *mln(int n)
{
  bool is; // TRUE si chaine existe
  char *text;
  char ch[80];

  is=ml_tabl->find_elt(n,&text);
  if (!is)
  {
    sprintf(ch,"missing resource string %d",n);
    affiche_bug(ch);
    return "";
  }
  return text;
}

bool ml_init()
{
  int i,j,len,n,num;
  bool ret=TRUE;
  const int chin_maxlen=1000;
  char *chin;
  STRING ch;
  FILE *fichier;

  if (ml_tabl) delete ml_tabl;

  ml_tabl=new ml_tabl_();
  ml_tabl->clear();


  strcpy(ch,formatdir);
  strcat(ch,CountryFile);
  if ((fichier=fopen(ch,"rt"))==NULL)
  { fatal_error("missing CountryFile");
    return FALSE;
  }

  n=0;
  chin=(char *)malloc(chin_maxlen+2);
  while ((!feof(fichier))&&ret)
  { if (fgets(chin,chin_maxlen,fichier)!=NULL)
    { len=strlen(chin);
      for (i=0;chin[i]==' ';i++);
      if (  (i+4<len)
          &&((chin[i]!='/')||(chin[i+1]!='/')) )
      { for (j=0;j<len;j++)
	  if (chin[j]==',') { chin[j]=' '; break; }
	sscanf(chin+i,"%d",&num);
	for (;(i<len)&&(chin[i]!='"');i++);
	for (j=i+1;chin[j]!='"';j++)
	{ if (chin[j]==0)
	  { // c'est une fin de ligne inattendue
	    sprintf(ch,"%s : unterminated string\n"
		,CountryFile);
	    strncat(ch,chin,30);
            strcat(ch," ...");
		 affiche_warning(ch);
            break;
          }
	  else if (  (chin[j]=='\\')&&(chin[j+1]=='n') )
	  { // c'est un return
            chin[j]=0xd; chin[j+1]=0xa;
	  }
	  else if (  (chin[j]=='\\')&&(chin[j+1]=='t') )
	  { // c'est une tabulation horizontale
		 int k;
		 for (k=j;chin[k]!=0;k++) chin[k]=chin[k+1];
		 chin[j]='\t';
	  }
	  else if (  (chin[j]=='\\')&&(chin[j+1]=='"') )
	  { // c'est un "
	    int k;
	    for (k=j;chin[k]!=0;k++) chin[k]=chin[k+1];
          }
	  else if (  (chin[j]=='\\')&&(chin[j+1]=='\\') )
	  { // c'est un \
//	    int k;
	    for (int k=j;chin[k]!=0;k++) chin[k]=chin[k+1];
	  }
	  else if (  (chin[j]=='\\') )
	  { // c'est une 'suite sur ligne suivante'
	    if (fgets(chin+j,chin_maxlen-j,fichier)!=NULL)
	      len=strlen(chin);
	    else strcpy(chin+j,"");
          }
	}
	chin[j]=0;
        if (!ml_tabl->add_elt(num,chin+i+1)) ret=FALSE;
        n++;
      }
    }
  }
  free(chin);
  { sprintf(ch,"ml_init() there are %d texts (for %d max)",n,ml_tabl->npi);
    Wmsg->wr(ch);
    sprintf(ch,"...       and %u characters (for 64k max)",ml_tabl->npc);
    Wmsg->wr(ch);
  }
  fclose(fichier);
  return ret;
}
// fin CountryFile
//////////////////////////////////////////

//////////////////////////////////////
// affichages d'erreur
//

extern bool show_warnings;

void affiche_erreur(char *message)
{ // affiche un dialogue box d'erreur
  if (show_warnings)
  MessageBox(GetActiveWindow() //PTWtop->HWindow
	     ,message,ml(204,"erreur")
	     ,MB_OK | MB_TASKMODAL );
}
void affiche_warning(char *message)
{ // affiche un dialogue box d'erreur
  if (show_warnings)
  MessageBox(GetActiveWindow() //PTWtop->HWindow
	     ,message,ml(205,"warning")
	     ,MB_OK | MB_TASKMODAL );
}
void affiche_bug(char *message)
{ // affiche un dialogue box d'erreur
  if (show_warnings)
  MessageBox(GetActiveWindow() //PTWtop->HWindow
	     ,message,"bug"
	     ,MB_OK | MB_TASKMODAL );
}
void affiche_erreur_LigneFichier(char *ligne,char *fichier)
{
  if (!show_warnings) return;
  char ch[200];
  strcpy(ch,ml(206,"ligne errone : "));
  strncat(ch,ligne,40);
  strcat(ch,ml(207,"\ndans le fichier "));
  strcat(ch,fichier);
  affiche_erreur(ch);
}
void fatal_error(char *message)
// affiche erreur fatale et stoppe le programme
{ MessageBox(PTWprologue->HWindow
            ,message
	    ,(ml_tabl->npc>10)
	     ? ml(208,"fatale erreur, exit WinSnoori")
	     : "fatal error, exit WinSnoori"
	       // ci-dessus cas particulier o les ml()
               // ne sont pas encore initialiss
	    ,MB_OK | MB_TASKMODAL );
  PostMessage(PTWprologue->HWindow,WM_CLOSE,0,0);
}

////////////////////////////////////////
unsigned long chtoul(char *ch)
{ union { unsigned long ul; char *ch; } x;
  x.ch=ch;
  return x.ul;
}

void NotImplemented(PTWindowsObject AParent)
{ MessageBox(AParent->HWindow
	     ,"not implemented",""
             ,MB_OK);
}


////////////////////////////////////////////////
//
int GetTicket()
{ static int ticket=0;
  return ++ticket;
}

#define MAXFONTS 100
typedef struct {
  LOGFONT lf;
  HFONT hf;
  int orientation;
} KnownFont;

KnownFont fonts[MAXFONTS];
static int nfonts = 0;

HFONT queryFont(
  const char *name,
  int width,
  int height,
  int orientation)
{
  int i;
  for (i = 0;   i < nfonts
  				 && (strcmp(name, fonts[i].lf.lfFaceName)
                 || width != fonts[i].lf.lfWidth
                 || height != fonts[i].lf.lfHeight
                 || orientation != fonts[i].orientation); i++);
  if (i == MAXFONTS) {
    affiche_erreur("QueryFont: No more font available");
    return(fonts[0].hf);
  }
  else if (i < nfonts) return(fonts[i].hf);
  else if (!strncmp("IPA", name, strlen("IPA"))) {
    fonts[nfonts].lf.lfHeight = height;
    fonts[nfonts].lf.lfWidth = width;
    fonts[nfonts].lf.lfEscapement = 0;
    fonts[nfonts].lf.lfOrientation = 0;
    fonts[nfonts].lf.lfWeight = FW_SEMIBOLD;
    fonts[nfonts].lf.lfItalic = false;
    fonts[nfonts].lf.lfUnderline = false;
    fonts[nfonts].lf.lfStrikeOut = false;
    fonts[nfonts].lf.lfCharSet = SYMBOL_CHARSET;
    fonts[nfonts].lf.lfOutPrecision = OUT_DEFAULT_PRECIS;
    fonts[nfonts].lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
    fonts[nfonts].lf.lfQuality = DRAFT_QUALITY;
    fonts[nfonts].lf.lfPitchAndFamily = FF_MODERN;
    strcpy(fonts[nfonts].lf.lfFaceName, name);
    fonts[nfonts].hf = CreateFontIndirect(&fonts[nfonts].lf);
    nfonts++;
    return(fonts[i].hf);
  }
  else {
    fonts[nfonts].lf.lfHeight = height;
    fonts[nfonts].lf.lfWidth = width;
    fonts[nfonts].lf.lfEscapement = orientation;
    fonts[nfonts].lf.lfOrientation = orientation;
    fonts[nfonts].lf.lfWeight = FW_BOLD;
    fonts[nfonts].lf.lfItalic = false;
    fonts[nfonts].lf.lfUnderline = false;
    fonts[nfonts].lf.lfStrikeOut = false;
    fonts[nfonts].lf.lfCharSet = DEFAULT_CHARSET;
    fonts[nfonts].lf.lfOutPrecision = OUT_DEFAULT_PRECIS;
    fonts[nfonts].lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
    fonts[nfonts].lf.lfQuality = DEFAULT_QUALITY;
    fonts[nfonts].lf.lfPitchAndFamily=DEFAULT_PITCH | FF_DONTCARE;
    strcpy(fonts[nfonts].lf.lfFaceName, name);
    fonts[nfonts].hf = CreateFontIndirect(&fonts[nfonts].lf);
    fonts[nfonts].orientation = orientation;
    nfonts++;
    return(fonts[i].hf);
  }
}


// policeProp() demande une police de taille
//   proportionnelle  la taille de la fentre.
// les coefficients multiplicateurs en x et y sont de 1
//   pour une fentre de 640 (en x) sur 480 (en y).
void policeProp(const char * name, int width, int height, d_planche *fenetre)
{
  fenetre->SetPlancheFont(queryFont(name,
                                   (int) (width * (fenetre->largeur / 640.0)),
										     (int) (height * (fenetre->hauteur / 480.0)), 0));

}

/////////////////////////////////////////////////
// journal et rsultats
//
logfiles logs;
long logs_nc; // nombre de caractres dj rentr

logfiles::logfiles()
{
 clear();
}
void logfiles::open1()
{
  STRING ch;
  strcat(strcpy(ch,tempdir),"results.txt");
  fic1=fopen(ch,"wb");
  logs_nc=0;
}
void logfiles::puts1(char *ch)
{
  int l, r;
  if (logs_nc<0) return;
  if (logs_nc+(l=strlen(ch))>LOGFILE_MAXLEN)
  { fputs("\r\n*** truncated ***",fic1);
    logs_nc=-1;
    return;
  }
  r = fputs(ch,fic1);
  if (r < 0) affiche_erreur("Unable to write in the result file");
  logs_nc+=l;
}
void logfiles::rclf1()
{ puts1("\r\n");
}
void logfiles::wr1(char *ch)
{ puts1(ch);
  rclf1();
}

void logfiles::date(long echt)
{
  STRING s;
  sprintf(s, "Date %d ms (%d sample)", enMilli(echt), echt);
  puts1(s);
  rclf1();
}

void logfiles::close1()
{ fclose(fic1);
  Wact->wrlog(TRUE);
}
void logfiles::clear()
{
  FILE *f;
  char ch[80];
  
  if (!tempdir || !strlen(tempdir)) return;
  strcat(strcpy(ch,tempdir),"log.txt");
  if ((f = fopen(ch, "r"))) {
    fclose(f);
    remove(ch);
  }
}
void logfiles::appendLog1toLog()
{ long len0  // longueur log.txt
      ,len1; // longueur results.txt
  STRING ch0,chb,ch1; //,ch[80];
  FILE *fic;
  CHARbuf buf(LOGFILE_MAXLEN+100,"buf in log");
  unsigned int nu;

  len0=nsof(strcat(strcpy(ch0,tempdir),"log.txt"));
  len1=nsof(strcat(strcpy(ch1,tempdir),"results.txt"));
  if (len1==0)
  { affiche_warning(ml(201,"pas de rsultats"));
    return;
  }
  if (len0+len1>LOGFILE_MAXLEN)
  { remove(strcat(strcpy(chb,tempdir),"log.bak"));
    rename(ch0,chb);
    MessageBox(GetActiveWindow()
	      ,ml(209,"fichier log.txt recopi en log.bak\npuis effac")
	      ,ml(210,"Le fichier journal devient trop grand")
	      ,MB_OK);  
  }
  fic=fopen(ch1,"rb");
  nu=fread(&buf[0],1,LOGFILE_MAXLEN,fic);
  fclose(fic);
  fic=fopen(ch0,"ab");
  fputs("\r\n",fic);
  fwrite(&buf[0],1,nu,fic);
  fclose(fic);
  //
  remove(ch1);
}
void logfiles::show1()
{ STRING ch;
  strcat(strcpy(ch,tempdir),"results.txt");
  if (isFile(ch)) ShowFileText(ch,"");
  else affiche_warning(ml(201,"pas de rsultats"));
}
void logfiles::show()
{ STRING ch;
  strcat(strcpy(ch,tempdir),"log.txt");
  if (isFile(ch)) ShowFileText(ch,"");
  else affiche_warning(ml(202,"pas de journal"));
}

/////////////////////////////////
// My_itoa() et My_ftoa()
//
static char ttoa[MAX_LIGNE];

char *My_itoa(int x)
{ sprintf(ttoa,"%d",x);
  return ttoa;
}
char *My_ftoa(float n)
{ sprintf(ttoa,"%f",n);
  return ttoa;
}

/////////////////////////////////////////////
void affstack()
{ STRING ch;

/*  sprintf(ch,"stackavail()=%u _stklen=%u"
		 ,stackavail(),_stklen);  YL*/
  sprintf(ch,"stackavail()=%u "
		 ,stackavail());
  Wmsg->wr(ch);
}

/******
* Function : init_grafix
* Description :
*     Initialise some useful graphics video adapter
*     characteristics, such as text size in pixels
*     (variables MyTextHeight, MyTextWidth),
*     pixel aspect-ratio etc (variable adapter).
******/
void init_grafix(PTWindowsObject AParent)
{
  HWND hWnd;
  TEXTMETRIC tm;
  HDC hDC;

  hDC=GetDC(hWnd=AParent->HWindow);

  GetTextMetrics(hDC,&tm);
  MyTextHeight=tm.tmHeight;
  MyTextWidth=tm.tmAveCharWidth; 

  ReleaseDC(hWnd,hDC);
}

///////////////////////////////////////////////////////
// fentre paramtres
//
//choix choix_autospectro[] = { {0,"NO"},{1,"YES"},{0,""} };
choix choix_graymode16[] = { {0,"colors"}
			    //,{1,"gray"}
                            //,{2,"gray 2*2"}
			    ,{0,""} };
choix choix_graymode256[]= { {0,"colors"}
			    ,{1,"gray"}
                            //,{2,"gray 2*2"}
			    ,{0,""} };
//static int colors16mode_;
class TWParam : public TFrameWindow
{   int ticket;
  public:
    TWParam(PTWindowsObject AParent);
    virtual void SetupWindow();
    virtual bool CanClose();
	 // rponses aux vnements
	 /*
	 virtual void modif_colors(RTMessage MSG)=[WM_USER+1];
	 */
	 LRESULT modif_colors(WPARAM,LPARAM);
	 DECLARE_RESPONSE_TABLE(TWParam);
};
DEFINE_RESPONSE_TABLE1(TWParam,TFrameWindow)
  EV_MESSAGE(WM_USER+1,modif_colors),
END_RESPONSE_TABLE;

TWParam::TWParam(PTWindowsObject AParent) :
	TFrameWindow(AParent, "")
{ int xx,yy,ww;  // coordonnes courantes dans la fentre
  STRING ch;
  
  ticket=GetTicket();
  wsprintf(ch,ml(211,"options gnrales %d."),ticket);
  SetCaption(ch);
  DisableAutoCreate();
  Attr.Style = WS_VISIBLE | WS_POPUPWINDOW | WS_CAPTION;
  Attr.X = 300;
  Attr.Y = 0;
  Attr.W = 240;

  xx=2; yy=2;
  ww=Attr.W-6;
  // device voix out
  new TWWaveOutDevice(this, xx, yy, ww,ticket);
  yy+=HNUM;
  // autospectro
  new TWchoix(this,0,xx,yy,ww
	      ,&autospectro,"autospectro : "
	      ,choix_YESNO 
	      ,ticket); yy+=HNUM;
  // WindowsWAVfiles
  new TWchoix(this,0,xx,yy,ww
	      ,&WindowsWAVfiles,"WindowsWAVfiles : "
	      ,choix_YESNO 
	      ,ticket); yy+=HNUM;
  // graymode
  new TWchoix(this,1,xx,yy,ww
	      ,&graymode,"graymode : "
	      ,(colors16only())
	       ? choix_graymode16
	       : choix_graymode256
	      ,ticket); yy+=HNUM;
  // debug
  if (indtest)
    new TWint(this,0,xx,yy,ww
	      ,&debug,"debug"
	      ,0,100,0,100
	      ,ticket); yy+=HNUM;


  Attr.H = yy+30;
}
void TWParam::SetupWindow()
{ TFrameWindow::SetupWindow();
}
bool TWParam::CanClose()
{ active(MENUOptions);
  return TRUE;
}
LRESULT TWParam::modif_colors(WPARAM,LPARAM)
{ reinitgraymode();
  return 0;
}
//
void Wtop::CMoptions()
{ GetApplication()->MakeWindow(
	  new TWParam(PTWprologue) );
  desactive(MENUOptions);
}

////////////////////////////////////////////
// les classes TWchange et drives
//
#define HNUMSCROLL 90
#define WNUMSCROLL 250

TWchange::TWchange(PTWindowsObject AParent
	      ,int id,int x,int y,int w
	      ,char *afftext     // texte  afficher
	      ,char ButtonChar
              ,int tick
	      )
              : TFrameWindow(AParent,"")
{ STRING ch;

  Attr.Style = WS_VISIBLE | WS_CLIPSIBLINGS;
  this->afftext=afftext;
  wsprintf(ch,"%s .%d",afftext,tick);
  SetCaption(ch);
  istyle=Attr.Style;
  ix=x;
  iy=y;
  iw=w;
  scroll_visible=0;
  this->id=id;
  this->ButtonChar=ButtonChar;
  //
  winit();
}
void TWchange::winit()
{ char bt[]="0";
  POINT point;

  bt[0]=ButtonChar;
  //
  if (scroll_visible==0)
  { Attr.Style=istyle|WS_CHILD|WS_BORDER;
    Attr.X=ix;
    Attr.Y=iy;
    Attr.W=iw;
    Attr.H=HNUM;
  }
  else
  { HDC hDC=GetDC(HWindow); 
    Attr.Style=istyle|WS_POPUP|WS_CAPTION|WS_BORDER;
    point.x=ix;
    point.y=iy;
	 ::ClientToScreen(Parent->HWindow,&point);
    Attr.X=min(point.x-5
	       ,GetDeviceCaps(hDC,HORZRES)
	         -Attr.W); 
    Attr.Y=point.y+5;
    Attr.H=HNUMSCROLL;
    Attr.W=Attr.W * 2; // Attention
    ReleaseDC(HWindow,hDC);
  }
  if (id>=0)
    new TButton(this,101,bt
	    ,Attr.W-HNUM+1,1,HNUM-4,HNUM-4,FALSE);
}
void TWchange::maj()
{ ::InvalidateRect(HWindow,NULL,TRUE);
}
//
DEFINE_RESPONSE_TABLE1(TWfloat,TWchange)
  EV_CHILD_NOTIFY_ALL_CODES(101,IDmore),
  EV_CHILD_NOTIFY_ALL_CODES(102,IDscrollbar),
END_RESPONSE_TABLE;
TWfloat::TWfloat(PTWindowsObject AParent
	      ,int id,int x,int y,int w
	      ,float *var     // variable  diter
	      ,char *afftext     // texte  afficher
	      ,float vmin, float vmax
	      ,float vminscroll, float vmaxscroll
              ,int tick
	      )
              : TWchange(AParent,id,x,y,w,afftext,'f',tick)
{ this->var=var;
  this->vmin=vmin;
  this->vmax=vmax;
  this->vminscroll=vminscroll;
  this->vmaxscroll=vmaxscroll;
}
void TWfloat::SetupWindow()
{ int loval,hival;

  TFrameWindow::SetupWindow();
  if (scroll_visible!=0) {
    Tscrollbar->GetRange(loval,hival);
    Tscrollbar->SetPosition(
           loval+(*var-vminscroll)/
		 ((vmaxscroll-vminscroll)/(hival-loval)) );
  }
}
void TWfloat::Paint(TDC& dc, bool, TRect&) //HDC hDC, PAINTSTRUCT&)
{ int i;
  STRING ch;

  if (scroll_visible==0)
  { strcpy(ch,afftext);
	 sprintf(ch+strlen(ch),"=%.3g",*var);
	 dc.TextOut(2,0,ch,strlen(ch));
  }
  else
  { dc.SetTextAlign(TA_LEFT|TA_TOP);
	 sprintf(ch,"%.3g",vminscroll);
	 dc.TextOut(2,0,ch,strlen(ch));
	 dc.SetTextAlign(TA_RIGHT|TA_TOP);
	 sprintf(ch,"%.3g",vmaxscroll);
	 dc.TextOut(i=Attr.W-HNUM-6,0,ch,strlen(ch));
	 dc.SetTextAlign(TA_CENTER|TA_TOP);
	 sprintf(ch,"<%.3g<",*var);
	 dc.TextOut(i/2,0,ch,strlen(ch));
  }
}
void TWfloat::IDmore(WPARAM) //RTMessage)
{ scroll_visible= (scroll_visible==0) ? 1 : 0 ;
  DestroyWindow(HWindow);

  winit();
  if (scroll_visible!=0)
  { Tscrollbar=new TScrollBar(this,102,3,30,Attr.W-8,30,TRUE);
  }

  Create();
}
void TWfloat::IDscrollbar(WPARAM) //RTMessage)
{ int loval,hival;

  Tscrollbar->GetRange(loval,hival);
  *var=vminscroll+(Tscrollbar->GetPosition()-loval)
		 *((vmaxscroll-vminscroll)/(hival-loval));
  ::InvalidateRect(HWindow,NULL,TRUE);
  if (id>=0) ::SendMessage(Parent->HWindow,WM_USER+id,0,0);
}
void TWfloat::set(float f)
{ *var=f;
  maj();
}
//
DEFINE_RESPONSE_TABLE1(TWint,TWchange)
  EV_CHILD_NOTIFY_ALL_CODES(101,IDmore),
  EV_CHILD_NOTIFY_ALL_CODES(102,IDscrollbar),
END_RESPONSE_TABLE;
TWint::TWint(PTWindowsObject AParent
			,int id,int x,int y,int w
			,int *var     // variable  diter
			,char *afftext     // texte  afficher
			,int vmin, int vmax
			,int vminscroll, int vmaxscroll
				  ,int tick
			)
			: TWchange(AParent,id,x,y,w,afftext,'i',tick)
{ this->var=var;
  this->vmin=vmin;
  this->vmax=vmax;
  this->vminscroll=vminscroll;
  this->vmaxscroll=vmaxscroll;
}
void TWint::SetupWindow()
{ int loval,hival;

  TFrameWindow::SetupWindow();
  if (scroll_visible!=0) {
    Tscrollbar->SetRange(0, vmaxscroll - vminscroll);
    Tscrollbar->GetRange(loval,hival);
	 Tscrollbar->SetPosition(
			  loval+(*var-vminscroll)/
		 ( ((float)(vmaxscroll-vminscroll))
			/(hival-loval) ) );
  }
}
void TWint::Paint(TDC& dc, bool, TRect&) //HDC hDC, PAINTSTRUCT&)
{ int i;
  STRING ch;

  if (scroll_visible==0)
  { strcpy(ch,afftext);
	 sprintf(ch+strlen(ch),"=%d",*var);
	 dc.TextOut(2,0,ch,strlen(ch));
  }
  else
  { dc.SetTextAlign(TA_LEFT|TA_TOP);
	 sprintf(ch,"%d",vminscroll);
	 dc.TextOut(2,0,ch,strlen(ch));
	 dc.SetTextAlign(TA_RIGHT|TA_TOP);
	 sprintf(ch,"%d",vmaxscroll);
	 dc.TextOut(i=Attr.W-HNUM-6,0,ch,strlen(ch));
	 dc.SetTextAlign(TA_CENTER|TA_TOP);
	 sprintf(ch,"<%d<",*var);
	 dc.TextOut(i/2,0,ch,strlen(ch));
  }
}
void TWint::IDmore(WPARAM) //RTMessage)
{ scroll_visible= (scroll_visible==0) ? 1 : 0 ;
  DestroyWindow(HWindow);

  winit();
  if (scroll_visible!=0)
  { Tscrollbar=new TScrollBar(this,102,3,30,Attr.W-8,30,TRUE);
  }

  Create();
}
void TWint::IDscrollbar(WPARAM) //RTMessage)
{ int loval,hival;

  Tscrollbar->GetRange(loval,hival);
  *var=vminscroll+(Tscrollbar->GetPosition()-loval)
		 *( ((float)(vmaxscroll-vminscroll))
			 /(hival-loval) );
  ::InvalidateRect(HWindow,NULL,TRUE);
  if (id>=0) ::SendMessage(Parent->HWindow,WM_USER+id,0,0);
}
void TWint::set(int i)
{ *var=i;
  maj();
}
//
DEFINE_RESPONSE_TABLE1(TWtext,TWchange)
  EV_CHILD_NOTIFY_ALL_CODES(101,IDmore),
END_RESPONSE_TABLE;
TWtext::TWtext(PTWindowsObject AParent
			,int id,int x,int y,int w
			,char *var     // texte  diter
			,char *afftext     // texte  afficher
			,int maxlen
				  ,int tick
			)
			: TWchange(AParent,id,x,y,w,afftext,'t',tick)
{ this->var=var;
  this->maxlen=maxlen;
}
TWtext::~TWtext()
{
}
void TWtext::Paint(TDC& dc, bool, TRect&) //HDC hDC, PAINTSTRUCT&)
{ STRING ch;

  if (scroll_visible==0)
  { wsprintf(ch,"%s=%s",afftext,var);
	 dc.TextOut(2,0,ch,strlen(ch));
  }
}
void TWtext::IDmore(WPARAM) //RTMessage)
{ if (scroll_visible!=0) Tedit->GetText(var,maxlen);

  scroll_visible= (scroll_visible==0) ? 1 : 0 ;
  DestroyWindow(HWindow);

  winit();
  if (scroll_visible!=0)
  { Tedit=new TEdit(this,-1,var
	,3,3,Attr.W-HNUM-10,24,maxlen,FALSE);
  }

  Create();
}
//
DEFINE_RESPONSE_TABLE1(TWchoix,TWchange)
  EV_CHILD_NOTIFY_ALL_CODES(101,IDmore),
END_RESPONSE_TABLE;
TWchoix::TWchoix(PTWindowsObject AParent
			,int id,int x,int y,int w
			,bool *var     // variable  diter
			,char *afftext     // texte  afficher
			,choix *tchoix
				  ,int tick
			)
			: TWchange(AParent,id,x,y,w,afftext,'c',tick)
{ this->var=var;
  this->tchoix=tchoix;
}
void TWchoix::Paint(TDC& dc, bool, TRect&) //HDC hDC, PAINTSTRUCT&)
{ int i;
  STRING ch;

  strcpy(ch,afftext);
  for (i=0;(strlen(tchoix[i].text)!=0); i++)
	 if (tchoix[i].n==*var)
	 { strcat(ch,tchoix[i].text);
		break;
	 }
  dc.TextOut(2,0,ch,strlen(ch));
}
void TWchoix::IDmore(WPARAM) //RTMessage)
{ int i,index;

  index=0;
  for (i=0;(strlen(tchoix[i].text)!=0); i++)
	 if (tchoix[i].n==*var) index=i;
  index++;
  if (strlen(tchoix[index].text)==0) index=0;
  *var=tchoix[index].n;
  ::InvalidateRect(HWindow,NULL,TRUE);
  if (id>=0) ::SendMessage(Parent->HWindow,WM_USER+id,0,0);
}

//
DEFINE_RESPONSE_TABLE1(TWchoixInt,TWchange)
  EV_CHILD_NOTIFY_ALL_CODES(101,IDmore),
END_RESPONSE_TABLE;
TWchoixInt::TWchoixInt(PTWindowsObject AParent
			,int id,int x,int y,int w
			,int *var     // variable  diter
			,char *afftext     // texte  afficher
			,choix *tchoix
				  ,int tick
			)
			: TWchange(AParent,id,x,y,w,afftext,'c',tick)
{
  this->variable=var;
  this->tchoix=tchoix;
}
void TWchoixInt::Paint(TDC& dc, bool, TRect&) //HDC hDC, PAINTSTRUCT&)
{ int i;
  STRING ch;

  strcpy(ch,afftext);
  for (i=0;(strlen(tchoix[i].text)!=0); i++)
	 if (tchoix[i].n==*variable) {
	   strcat(ch,tchoix[i].text);
		break;
	 }
  dc.TextOut(2,0,ch,strlen(ch));
}
void TWchoixInt::IDmore(WPARAM) //RTMessage)
{ int i,index;

  index=0;
  for (i=0;(strlen(tchoix[i].text)!=0); i++)
	 if (tchoix[i].n==*variable) index=i;
  index++;
  if (strlen(tchoix[index].text)==0) index=0;
  *variable = tchoix[index].n;
  ::InvalidateRect(HWindow,NULL,TRUE);
  if (id>=0) ::SendMessage(Parent->HWindow,WM_USER+id,0,0);
}

//
DEFINE_RESPONSE_TABLE1(TWWaveOutDevice,TWchange)
  EV_CHILD_NOTIFY_ALL_CODES(101,IDmore),
  EV_CHILD_NOTIFY_ALL_CODES(102,IDnext),
END_RESPONSE_TABLE;
TWWaveOutDevice::TWWaveOutDevice(PTWindowsObject AParent
			,int x,int y,int w
			,int tick
			)
			: TWchange(AParent,0,x,y,w
			,"WaveOutDevice",'w',tick)
{
}
void TWWaveOutDevice::Paint(TDC& dc, bool, TRect&) //HDC hDC, PAINTSTRUCT&)
{ STRING ch;

  strcpy(ch,afftext);
  sprintf(ch+strlen(ch),"=%d",WaveOutDevice.id);
  dc.TextOut(2,0,ch,strlen(ch));

  if (scroll_visible!=0)
  { int y,dy;

	 y=0; dy=16;

	 if (WaveOutDevice.num==0)
	 { strcpy(ch,ml(212,"no WaveOutDevice"));
		dc.TextOut(10,10,ch,strlen(ch));
	 }
	 else
	 { WAVEOUTCAPS *Caps;

		Caps=&WaveOutDevice.caps;
		{	wsprintf(ch,"  wMid=%x",Caps->wMid);
			y+=dy; dc.TextOut(4,y,ch,strlen(ch));
			wsprintf(ch,"  wPid=%x",Caps->wPid);
			y+=dy; dc.TextOut(4,y,ch,strlen(ch));
			wsprintf(ch,"  vDriverVersion=%x",Caps->vDriverVersion);
			y+=dy; dc.TextOut(4,y,ch,strlen(ch));
			strcpy(ch,"  szPname=");
			strcat(ch,Caps->szPname);
			y+=dy; dc.TextOut(4,y,ch,strlen(ch));
			wsprintf(ch,"  dwFormats=%lx",Caps->dwFormats);
			y+=dy; dc.TextOut(4,y,ch,strlen(ch));
			wsprintf(ch,"  wChannels=%x",Caps->wChannels);
			y+=dy; dc.TextOut(4,y,ch,strlen(ch));
			wsprintf(ch,"  dwSupport=%x",Caps->dwSupport);
			y+=dy; dc.TextOut(4,y,ch,strlen(ch));
		}
	 }
  }
}
void TWWaveOutDevice::IDmore(WPARAM) //RTMessage)
{ scroll_visible= (scroll_visible==0) ? 1 : 0 ;
  DestroyWindow(HWindow);

  Attr.X=200;
  Attr.W=400;
  winit();
  if (scroll_visible!=0)
  { Attr.H=200;
	 new TButton(this,102,"next"
		 ,Attr.W-HNUM-40,2,36,HNUM-6,FALSE);
  }

  Create();
}
void TWWaveOutDevice::IDnext(WPARAM) //RTMessage)
{ int i;

  if (WaveOutDevice.num==0) return;

  i=WaveOutDevice.id+1;
  if (i>=WaveOutDevice.num) i=0;
  WaveOutDevice.newid(i);
  ::InvalidateRect(HWindow,NULL,TRUE);
}

/////////////////////////////////////////
// les activits
//
// lignes:
//   0-3 : Snoori ans license information
//   5-7 : courant
//         avec une importance particulire pour 0
//   9 : rsultats
//   10-12 : freq, temps, f0
//   13-14 : dure, 1/dure
//

// utilisation des fonctions pourcent
//   le plus simple, sans texte
//       1) un ou plusieurs Wact->pourcent(,)
//             qui ouvre la fentre et l'affiche
//       2)  la fin un seul Wact->hidepourcent()
//             qui ferme la fentre
//   plus compliqu, avec texte
//       1.1) Wact->showpourcent(text)
//             pour donner le texte
//       1.2) un ou plusieurs Wact->pourcent(,)
//             qui ouvre la fentre et l'affiche
//       1.3) recommencer autant de fois que ncessaire
//             les tapes 1.1) et 1.2) 
//       2)  la fin un seul Wact->hidepourcent()
//             qui ferme la fentre
class TWactpourcent : public TFrameWindow
{ public:
    bool isVisible;
    char text[80];
    long n,nmax;

	 TWactpourcent(PTWindowsObject Parent);
    virtual void Paint(TDC& dc, bool erase, TRect& rect); //HDC,PAINTSTRUCT&);
};
TWactpourcent::TWactpourcent(PTWindowsObject Parent)
       : TFrameWindow(Parent,"")
{ isVisible=FALSE;
  Attr.Style = WS_POPUP | WS_DLGFRAME;
  Attr.X=200;
  Attr.Y=100; 
  Attr.W=200; 
  Attr.H=60; 
  n=0;
  nmax=0;
  strcpy(text,"");
}
void TWactpourcent::Paint(TDC& dc, bool, TRect&) //HDC hDC,PAINTSTRUCT&)
// paint la fentre pourcent
// si nmax==0 seul le texte est affich
{ char ch[80];

  if (!isVisible) return;
  if (nmax!=0)
  { sprintf(ch,"%d%%",(int)((100.0*n)/(max(nmax,1))));
	 dc.TextOut(2,2,ch,strlen(ch));
  }
  dc.TextOut(2,22,text,strlen(text));
}
TWactpourcent *Wactpourcent;
TWact *Wact;
TWact::TWact()
{
  strcpy(text0,"");
  strcpy(text1,"");
  strcpy(text2,"");
  isRes=FALSE;
}
void TWact::paint()
{
  wr_n(0,"WinSnoori 1.34");
  wr_n(1,"Copyright 2002-2006 Loria");
  wr_n(2,"  ");
#ifdef LICENCE
  wr_n(3,(Licensed) ? "Licensed to:" : "NOT REGISTERED");
#else
  wr_n(3, "Used by:");
#endif

  if (Organization && strlen(Organization)) wr_n(4,Organization);

  wr_n(5,text0);
  wr_n(6,text1);
  wr_n(7,text2);
  wr_n(8, isRes ? ml(214,"rsultats") : "" );
}

void TWact::wr_n(int n,char *ch)
{ HDC hDC;
  RECT rect;

  hDC=GetDC(PTWprologue->HWindow);
  if (!hDC) {
    affiche_erreur("No device context");
    return;
  }
  rect.left=0;
  rect.right=PTWprologue->Attr.W;
  rect.top=MyTextHeight*( indtest ? n+3 : n );
  rect.bottom=rect.top+MyTextHeight;
  FillRect(hDC,&rect,(HBRUSH)GetStockObject(WHITE_BRUSH));
  SetBkMode(hDC,OPAQUE);
  TextOut(hDC,2,rect.top,ch,strlen(ch));
  ReleaseDC(PTWprologue->HWindow,hDC);
}
void TWact::wr(char *text)
{ strcpy(text0,text); wr_n(5,text0);
  strcpy(text1,""); wr_n(6,text1);
  strcpy(text2,""); wr_n(7,text2);
}
void TWact::wr3(char *Text0,char *Text1,char *Text2)
{
  strcpy(text0,Text0); wr_n(5,text0);
  strcpy(text1,Text1); wr_n(6,text1);
  strcpy(text2,Text2); wr_n(7,text2);
}
void TWact::setXY(char *ch1,char *ch2)
{ wr_n(10,ch1);
  wr_n(11,ch2);
}
void TWact::f0(unsigned int f0)
{ char ch[80];
  sprintf(ch,"f0: %u Hz",f0); wr_n(12,ch);
}
void TWact::duree(float duree)
{ char ch[80];
  
  sprintf(ch,"%.1f ms",duree); wr_n(13,ch);
  if (duree>1) sprintf(ch,"1/%.1f sec",1000./duree);
  else strcpy(ch,"");
  wr_n(14,ch);
}
void TWact::showpourcent(char *text)
// emploi par l'utilisateur
//   sert  donner le texte affich par Wact::pourcent
//   lorsque l'utilisateur n'a pas de texte particulier
//      donner, pas ncessaire d'appeler showpourcent()
// emploi par pourcent()
//   appel showpourcent(NULL)
//   sert  ouvrir la fentre si elle ne l'est pas dj
//     et  afficher son contenu tout de suite
{
  if (text!=NULL)
  { Wactpourcent->nmax=0;
    strcpy(Wactpourcent->text,text);
  }
  if (!Wactpourcent->isVisible)
  { Wactpourcent->isVisible=TRUE;
    Wactpourcent->Show(SW_SHOW);
  }
  SetActiveWindow(Wactpourcent->HWindow);
  InvalidateRect(Wactpourcent->HWindow,NULL,TRUE);
  UpdateWindow(Wactpourcent->HWindow);
}
void TWact::pourcent(long n,long nmax)
{ 
  Wactpourcent->n=n;
  Wactpourcent->nmax=nmax;
  showpourcent(NULL);
}
void TWact::hidepourcent()
{
  Wactpourcent->Show(SW_HIDE);
  Wactpourcent->isVisible=FALSE;
  strcpy(Wactpourcent->text,"");
}
void TWact::wrlog(bool b)
{
  isRes=b;
  wr_n(8, isRes ? ml(214,"rsultats") : "" );
}
void initWact(PTWindowsObject AParent)
{
  Wact=new TWact();
  AParent->GetApplication()->MakeWindow(
      Wactpourcent=new TWactpourcent(AParent));

}

///////////////////////////////////////////////////////
// affichage d'un fichier texte
//
void ShowFileText(char *filename,char *)
{
  STRING ch;

  strcpy(ch,"notepad ");
  strcat(ch,filename);
  WinExec(ch,SW_SHOWNORMAL);
  return;
}

////////////////////////////////////////
// utilisation de TinputDialog
//
class TMyInputDialog : public TInputDialog
{ public:
    TMyInputDialog(PTWindowsObject Parent,
                   char *caption,char *prompt,
	                char *text,int textmaxlen)
	  : TInputDialog(Parent,caption,prompt,text,textmaxlen) {}
    virtual void SetupWindow();
};
void TMyInputDialog::SetupWindow()
{
  TInputDialog::SetupWindow();
  ::SetWindowText(GetItemHandle(IDCANCEL),ml(213,"&Annuler"));
}
bool MyInputDialog(char *caption,char *prompt
		   ,char *text,int textmaxlen)
{
  int res = TMyInputDialog(PTWtop,caption,prompt,text,textmaxlen).Execute();
  return(res == IDOK);
}

////////////////////////////////////////////////////////
// message window
//
//     display various informations during initialisation
//     and running of the programm
//
//   use: to write new information
//              char ch[]
//              Wmsg->wr(ch); // write one line
//
//   note: the message window keep the last 100 lines
//
WindowMessage *Wmsg;
#define NWMESSAGEMIN 100
#define NWMESSAGEMAX 150
WindowMessage::WindowMessage(PTWindowsObject AParent)
	 : TEditWindow(AParent,"message")
{
  DisableAutoCreate();
  Attr.Style=WS_VISIBLE | WS_POPUP|WS_CAPTION
				 |WS_THICKFRAME
				 |WS_MAXIMIZEBOX;

  //if (!indtest)
  Attr.Style&=~WS_VISIBLE;
  isVisible=FALSE;

  Attr.X=120;
  Attr.Y=10;
  Attr.W=280;
  Attr.H=100;
  nWmsg=0;
}
void WindowMessage::wr(char *ch)
{ int i;
  char ch1[20];
  static int iWmsg=0; // nombre de lignes prsentes

  if (!isVisible) return;
  if ( iWmsg>=NWMESSAGEMAX )
  { for (i=0;i<NWMESSAGEMAX-NWMESSAGEMIN;i++)
	 { Editor->DeleteLine(0);
		iWmsg--;
	 }
  }
  nWmsg++; iWmsg++;
  Editor->SetSelection(10000,10000);
  sprintf(ch1,"%d  ",nWmsg);
  Editor->Insert(ch1);
  Editor->Insert(ch);
  Editor->Insert("\r\n");
  ::UpdateWindow(HWindow);
}
void WindowMessage::show()
{ isVisible=TRUE;
  Show(SW_SHOW);
}
//
void initWmessage(PTWindowsObject AParent)
{ Wmsg=new WindowMessage(AParent);
  AParent->GetApplication()->MakeWindow(Wmsg);
}

/////////////////////////////////////////
// BUFbuf
//
// classe commune  toutes les classes buffers
// non vue de l'utilisateur
//
BUFbuf::BUFbuf(long ni,int sizeelt,char *name)
{ this->ni=max(0,ni);
  this->sizeelt=sizeelt;
  p= (void huge *) malloc((this->ni+1)*sizeelt);
  //
  this->name = (char * ) malloc(strlen(name)+1);
  strcpy(this->name,name);
}
BUFbuf::~BUFbuf()
{
 // MessageBox(NULL,"~BUFbuf()","~BUFbuf()",MB_OK);
  if (p) free(p);
  p = (void huge *) NULL;
  if (name) free(name);
  name = (char *) NULL;
}
void BUFbuf::newsize(long ni)
{
  this->ni=max(0,ni);
  if (this->ni) p = realloc(p,(ni+1)*sizeelt);
  else {
    free(p);
    p = (void huge *) NULL;
  }
}

/////////////////////////////////////////
// INTarray
//
// classe commune  toutes les classes buffers
// non vue de l'utilisateur
//
INTarray::INTarray(long ni, char *name)
{ this->ni=max(0,ni);
  p= (int *) malloc((this->ni+1)* sizeof(int));
  //
  this->name = (char * ) malloc(strlen(name)+1);
  strcpy(this->name,name);
}
INTarray::~INTarray()
{
  if (p) free(p);
  p = (int *) NULL;
  if (name) free(name);
  name = (char *) NULL;
}
void INTarray::newsize(long ni)
{
  this->ni=max(0,ni);
  if (this->ni) p = (int *) realloc(p, (ni+1)* sizeof(int));
  else {
    free(p);
    p = (int *) NULL;
  }
}

/////////////////////////////////////////
// INTbuf
//
// sortie de message si lecture ou criture en dehors du buffer
// si lecture en dehors du buffer, on fournit 0
// buffer initialis  zro  la cration
//
/*
INTbuf::INTbuf(long n,char *name)
	   : BUFbuf(n,sizeof(int),name)
{ 
}
*/
#ifdef BUFCONTROLS
int huge & INTbuf::operator[](long i)
{ if ((i<0)||(i>=ni))
  { char ch1[80],ch2[80];

    strcpy(ch1,"range error : ");
    strcat(ch1,name);
    sprintf(ch2,"INTbuf[i] : i=%ld out of range",i);
    MessageBox(GetActiveWindow() //PTWtop->HWindow
              ,ch2,ch1,MB_OK);
    i=ni; ((int huge *)p)[ni]=0;
  }
  return ((int huge *)p)[i];
}
#endif

/////////////////////////////////////////
// SHORTbuf
//
// sortie de message si lecture ou criture en dehors du buffer
// si lecture en dehors du buffer, on fournit 0
// buffer initialis  zro  la cration
//
/*
SHORTbuf::SHORTbuf(long n,char *name)
	   : BUFbuf(n,sizeof(short),name)
{
}
*/
#ifdef BUFCONTROLS
short huge & SHORTbuf::operator[](long i)
{ if ((i<0)||(i>=ni))
  { char ch1[80],ch2[80];

    strcpy(ch1,"range error : ");
    strcat(ch1,name);
    sprintf(ch2,"SHORTbuf[i] : i=%ld out of range",i);
    MessageBox(GetActiveWindow() //PTWtop->HWindow
              ,ch2,ch1,MB_OK);
    i=ni; ((short huge *)p)[ni]=0;
  }
  return ((short huge *)p)[i];
}
#endif

/////////////////////////////////////////
// FLOATbuf
//
// sortie de message si lecture ou criture en dehors du buffer
// si lecture en dehors du buffer, on fournit 0
// buffer initialis  zro  la cration
//
/*
FLOATbuf::FLOATbuf(long n,char *name)
           : BUFbuf(n,sizeof(float),name)
{
}
*/
#ifdef BUFCONTROLS
float huge & FLOATbuf::operator[](long i)
{ if ((i<0)||(i>=ni))
  { char ch1[80],ch2[80];

    strcpy(ch1,"range error : ");
    strcat(ch1,name);
    sprintf(ch2,"FLOATbuf[i] : i=%ld out of range",i);
    MessageBox(GetActiveWindow() //PTWtop->HWindow
              ,ch2,ch1,MB_OK);
    i=ni; ((float huge *)p)[ni]=0;
  }
  return ((float huge *)p)[i];
}
#endif

/////////////////////////////////////////
// DOUBLEbuf
//
// sortie de message si lecture ou criture en dehors du buffer
// si lecture en dehors du buffer, on fournit 0
// buffer initialis  zro  la cration
//
/*
DOUBLEbuf::DOUBLEbuf(long n,char *name)
           : BUFbuf(n,sizeof(double),name)
{
}
*/
#ifdef BUFCONTROLS
double huge & DOUBLEbuf::operator[](long i)
{ if ((i<0)||(i>=ni))
  { char ch1[80],ch2[80];

    strcpy(ch1,"range error : ");
    strcat(ch1,name);
    sprintf(ch2,"DOUBLEbuf[i] : i=%ld out of range",i);
    MessageBox(GetActiveWindow() //PTWtop->HWindow
              ,ch2,ch1,MB_OK);
    i=ni; ((double huge *)p)[ni]=0;
  }
  return ((double huge *)p)[i];
}
#endif

/////////////////////////////////////////
// CHARbuf
//
// sortie de message si lecture ou criture en dehors du buffer
// si lecture en dehors du buffer, on fournit 0
// buffer initialis  zro  la cration
//
/*
CHARbuf::CHARbuf(long n,char *name)
	   : BUFbuf(max(1,n),sizeof(char),name)
{ ((char *)p)[0]=0;
}
*/
char *CHARbuf::cpy(char *ch)
{ newsize(strlen(ch)+1);
  return strcpy( (char *)p ,ch);
}
char *CHARbuf::cat(char *ch)
{ newsize(size()+strlen(ch));
  return strcat( (char *)p ,ch);
}
unsigned int CHARbuf::len()
{ return strlen((char *)p);
}
#ifdef BUFCONTROLS
char huge & CHARbuf::operator[](long i)
{ if ((i<0)||(i>=ni))
  { char ch1[80],ch2[80];

    strcpy(ch1,"range error : ");
    strcat(ch1,name);
    sprintf(ch2,"CHARbuf[i] : i=%ld out of range",i);
    MessageBox(GetActiveWindow() //PTWtop->HWindow
               ,ch2,ch1,MB_OK);
    i=ni; ((char huge *)p)[ni]=0;
  }
  return ((char huge *)p)[i];
}
#endif

