/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include <stdio.h>
#include "ed_c0.h"
#include "cfgfile.h"
#include "cepstre_factory.h"
#include <iostream> //MODIF gcc3
using namespace std;


TEDC0::TEDC0() {
	Threshold = 0;
	Threshold2 = 0;
	MinSpeech = 1;
	MaxSpeechSil = 0;
	MaxSilSpeech = 0;
	KeepStart = 0;
	KeepEnd = 0;
	Smoothing = 0;
	MaxSize = -1;
	MinSize = -1;
	MaxSil = -1;
}

TEDC0::~TEDC0() {
}

int TEDC0::Config(const char * /*basedir*/,TCfgFile &cfg,int /*ver*/) {
	double threshold = 0;
	double threshold2=-1;
	int minspeech = 0;
	int maxspeechsil = 0;
	int maxsilspeech = 0;
	int keepstart = 0;
	int keepend = 0;
	double smoothing = 0;
	int maxsize = -1;
	int minsize = -1;
	int maxsil = -1;
    for(;;) {
    	if(!cfg.Next() ){cfg.SetError("bad end");return 0;};
		if(cfg.ItemIs("Threshold")) {
        	if(!cfg.DoubleValue(threshold)) return 0;
        }else if(cfg.ItemIs("Threshold2")) {
        	if(!cfg.DoubleValue(threshold2)) return 0;
        }else if(cfg.ItemIs("MinSpeech")) {
        	if(!cfg.IntValue(minspeech,1,32000)) return 0;
        }else if(cfg.ItemIs("MaxSpeechSil")) {
        	if(!cfg.IntValue(maxspeechsil,0,32000)) return 0;
        }else if(cfg.ItemIs("MaxSilSpeech")) {
        	if(!cfg.IntValue(maxsilspeech,0,32000)) return 0;
        }else if(cfg.ItemIs("KeepStart")) {
        	if(!cfg.IntValue(keepstart,-100,32000)) return 0;
        }else if(cfg.ItemIs("KeepEnd")) {
        	if(!cfg.IntValue(keepend,-100,32000)) return 0;
        }else if(cfg.ItemIs("Smoothing")) {
        	if(!cfg.DoubleValue(smoothing,0,1)) return 0;
        }else if(cfg.ItemIs("MaxSize")) {
        	if(!cfg.IntValue(maxsize,-1,32000)) return 0;
        }else if(cfg.ItemIs("MinSize")) {
        	if(!cfg.IntValue(minsize,-1,32000)) return 0;
        }else if(cfg.ItemIs("MaxSil")) {
        	if(!cfg.IntValue(maxsil,-1,32000)) return 0;
        }else if(cfg.ItemIs("End")) {
        	break;
        }else  {cfg.SetError("Unknown EDC0 Param");return 0;};
    }
	if(!Config(threshold,threshold2,minspeech,maxspeechsil,maxsilspeech,keepstart,keepend,smoothing
		,maxsize,minsize,maxsil)){cfg.SetError("Bad edc0 config");return 0;};
	return 1;
}
int TEDC0::Config(double threshold,double threshold2,int minspeech,int maxspeechsil,int maxsilspeech,int keepstart,int keepend
				,double smoothing,int maxsize,int minsize,int maxsil) {
	Threshold = threshold;
	if(threshold2>=0) Threshold2 = threshold2;
	else Threshold2 = threshold;
	MinSpeech = minspeech;
	MaxSpeechSil = maxspeechsil;
	MaxSilSpeech = maxsilspeech;
	KeepStart = keepstart;
	KeepEnd = keepend;
	Smoothing = smoothing;
	MaxSize = maxsize;
	MinSize = minsize;
	MaxSil = maxsil;
	
	return 1;
}

enum {
	StateStart,
	StateSpeechPos,
	StateSpeech,
	StateOcc,
	StateSpeechCont,
	StateEnd
};

int TEDC0::Init(TCepstreFactory *cf) {
	if(!TEndDetector::Init(cf)) return 0;
	OldValue = Threshold;
	if((Threshold>0) && (MinSpeech>0)) {
		SetStartDate(-1);
		State = StateStart;
	} else { // No Start detection
		SetStartDate(0);
		State = StateSpeech;
	}	
	return 1;
};

int TEDC0::TestACepstre(const double *data,int date) {
	double v = (OldValue *Smoothing) + (data[CepstreFactory()->C0Index()]*(1-Smoothing));
	OldValue = v;

	// State trans with v
	switch(State) {
		case StateStart :
			if(v>=Threshold) {
				d1=0;
				State = StateSpeechPos;
			}
			break;
		case StateSpeechPos :
			if(v<Threshold) 
				State = StateStart;
			break;
		case StateSpeech :
			if((v<Threshold2) && ((date-StartDate()) >= (MinSize-KeepEnd)) ) {
				State = StateOcc;
				d2 = 0;
				DateSil = date;
			};
			break;
		case StateOcc :
			if(v>=Threshold2) {
				d3 =0;
				State = StateSpeechCont;
			} else 
			break;
		case StateSpeechCont :
			if(v<Threshold2) State = StateOcc;
			break;
		case StateEnd :
			return 1;
	};

	// State trans with dur
	switch(State) {
		case StateStart :
			break;
		case StateSpeechPos :
			if(++d1>=MinSpeech)  {
				// Start
				State = StateSpeech;
				int sd = date-KeepStart-MinSpeech;
				SetStartDate((sd<0)?0:sd);
			} 
			break;
		case StateOcc :
			if((++d2>MaxSilSpeech) ) {
				State = StateEnd;
				SetEndDate(DateSil + KeepEnd); 
			};
			break;
		case StateSpeechCont :
			if(++d3>MaxSpeechSil) 
				State = StateSpeech;
			if(v<Threshold) State = StateOcc;
			break;
	};

	// test MaxSize et MaxSil
	switch(State) {
		case StateStart :
		case StateSpeechPos :
			if((MaxSil>0) && (date > MaxSil)) {
				// cancel
				SetStartDate(0);
				SetEndDate(0);
				State = StateEnd;
			};
			break;
		case StateSpeech :
		case StateSpeechCont :
		case StateOcc :
			if((MaxSize>0) && ((date-StartDate())>=MaxSize) )  {
				SetEndDate(date);
				State =StateEnd;
			}
			break;
	};
	//cout << State << " : "<<v<<endl;		
	return 1;	

};
