/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include <stdio.h>
#include "ed_voca.h"
#include "common.h"
/*
 Minimum duration of speech when speech start (start)
 MinSpeech 200 (en ms)
 TransCost 1
 Maximum duration of silence in speech  (end)
 MaxSilSpeech 150 (en ms)
 Maximum duration of speech in silence  (end)
 MaxSpeechSil 100 (en ms)
 nombre de cepstre a ajouter a la fin
 duree maximum de parole (-1=aucune)
 MaxSize -1
 duree min
 MinSize -1
 dure maximum de silence sans detecter de parole (-1=aucune)
 MaxSil 500
*/


TEDVoca::TEDVoca() {
	MinSpeech = 1;
	MaxSpeechSil = 0;
	MaxSilSpeech = 0;
	KeepEnd = 0;
	MaxSize = -1;
	MinSize = -1;
	MaxSil = -1;
	MinNRJ=0;
}

TEDVoca::~TEDVoca() {
}

int TEDVoca::Config(int minspeech, int maxspeechsil, int maxsilspeech,
						  int maxsize, int minsize, int maxsil, double minnrj, int stepMs) {
	MinSpeech = minspeech;
	MaxSpeechSil = maxspeechsil;
	MaxSilSpeech = maxsilspeech;
	MaxSize = maxsize;
	MinSize = minsize;
	MaxSil = maxsil;
	MinNRJ = minnrj;
   StepMs = stepMs;

	return 1;
}
enum {
	StateStart,
	StateSpeechPos,
	StateSpeech,
	StateOcc,
	StateSpeechCont,
	StateEnd
};


int TEDVoca::Init() {
	if((MinSpeech>0)) {
		SetStartDate(-1);
		State = StateStart;
	} else { // No Start detection
		SetStartDate(0);
		State = StateSpeech;
	}
	SpeechProb =0;
	SilProb =0;
	return 1;
};

inline double DMax(double a,double b) {return (a<b)?b:a ;}
int TEDVoca::TestAFrame(float energy, int date) {
	//if(date<MinSize -Keep) return 1;

	bool IsSil = energy <= MinNRJ;

	// State trans with IsSil
	switch(State) {
		case StateStart :
			if(!IsSil) {   // If there is no more silence the state changes to
         					// the possibility of speech.
				d1=0;
				State = StateSpeechPos;
			}
			break;
		case StateSpeechPos :
			if(IsSil)             // If the silence comes back one moves back to the
         							 // "stand by" status
				State = StateStart;
			break;
		case StateSpeech :
			if((IsSil) && ((date - getStartDate()) >= MinSize) ) {
				State = StateOcc;
				d2 = 0;
				DateSil = date;
			};
			break;
		case StateOcc :
			if(!IsSil) {
				d3 =0;
				State = StateSpeechCont;
			}
			break;
		case StateSpeechCont :
			if(IsSil) State = StateOcc;
			break;
		case StateEnd :
			return 1;
	};

	// State trans with dur
	switch(State) {
		case StateStart :
			break;
		case StateSpeechPos :
			if((++d1) * StepMs >= MinSpeech)  {  // if there may be speech and the duration is beyond MinSpeech
				// one moves to the Speech state
				State = StateSpeech;
				int sd = date - MinSpeech;
				SetStartDate((sd<0)?0:sd);
			}
			break;
		case StateOcc :
			if(((++d2) * StepMs > MaxSilSpeech) ) {  // Obviously it cannot be a stop anymore because the silence is too long.
				State = StateEnd;
				SetEndDate(DateSil);
			};
			break;
		case StateSpeechCont :
			if((++d3) * StepMs > MaxSpeechSil)
				State = StateSpeech;
			if(IsSil) State = StateOcc;
			break;
	};

	// test MaxSize et MaxSil
	switch(State) {
		case StateStart :
		case StateSpeechPos :
			if((MaxSil>0) && (date > MaxSil)) {
				// cancel
				SetStartDate(0);
				SetEndDate(0);
				State = StateEnd;
			};
			break;
		case StateSpeech :
		case StateSpeechCont :
		case StateOcc :
			if((MaxSize>0) && ((date - getStartDate()) >= MaxSize) )  {
				SetEndDate(date);
				State =StateEnd;
			}
			break;
	};
	//cout << State << " : "<<IsSil<<endl;		
	return 1;


};
