/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier en_post.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>

#include "format_f.h"

static int prior[] = {0, 0, 0, 1, 1}; /* les priorites des operateurs */

struct ope {
	int niv;	/* niveau de l'operateur */
	int op;
};

static struct cell p_loc[TAIL_LOC];	/* pile resultat */
static int top_res;			/* sommet de la pile resultat */
static struct ope p_op[TAIL_LOC]; /* pile des operateurs non traites */
static int top_op;		  /* sommet de la pile des operateurs */
static BOOL indvar[NBVAR];	          /* indicateur de presence d'une
													variable dans une expression */

static int ctoop(char c)
{ if (debug==5)
  { char ch[80];
	 sprintf(ch,"ctoop(c=%c)",c);
	 Wmsg->wr(ch);
  }
	switch (c) {
		case '+' : return(PLUS);
		case '-' : return(MOINS);
		case '*' : return(FOIS);
		case '/' : return(DIVISE);
		default : affiche_erreur(
				 ml(3701,"oprateur inconnu"));
	}
		  return 0;
}

static void empile_op(
	int niv,   /* niveau de l'operateur */
	char optr) /* le caractere de l'operateur */
{ if (debug==5)
  { char ch[80];
	 sprintf(ch,"empile_op(niv=%d,optr=%c)",niv,optr);
	 Wmsg->wr(ch);
  }
	if (top_op >= TAIL_LOC)
	  affiche_erreur(
		 ml(3702,"plus de place dans la pile des operateurs"));
	p_op[top_op].niv = niv;
	p_op[top_op].op = ctoop(optr);
	top_op++;
}

static void empile_res(char type,int c)
{ if (debug==5)
  { char ch[80];
	 sprintf(ch,"empile_res(type=%c,c=%d)",type,c);
	 Wmsg->wr(ch);
  }
	if (top_res >= TAIL_LOC)
	  affiche_erreur(ml(3703,"plus de place dans la pile resultat"));
	p_loc[top_res].t = type;
	switch (type) {
		case 'o' : p_loc[top_res].x.op = c; break;
		case 'v' : p_loc[top_res].x.var = c; break;
		case 'n' : p_loc[top_res].x.val = c; break;
	}
	top_res++;
}

static int depile_op()
{ if (debug==5) Wmsg->wr("depile_op()");
  if (!top_op)
	 affiche_erreur(ml(3704,"pile des operateurs vide"));
  return(p_op[--top_op].op);
}





/*
 * Cette fonction transforme une chaine de caracteres representant une
 * expression numerique (les quatres operateurs, nombres entiers)
 * en une pile de cases (notation postfixee).
 */

struct post_pile *en_post(char *s)
{
  int niveau = 0;
  int i = 0;		/* indice dans la chaine de caracteres */
  struct post_pile *res;	/* la pile resultat */
  struct cell *nv_pile;   /* copie de la pile de travail */

  if (debug==5)
  { char ch[80];
	 sprintf(ch,"en_post(\"%s\")",s);
	 Wmsg->wr(ch);
  }
  top_op = top_res = 0;
  { int j;
	 for (j = 0; j < NBVAR; j++)
	 indvar[j] = FAUX;
  }
  while (i < strlen(s))
  { if (s[i] == ' ' || s[i] == '\n')
		i++; /* on saute le blanc */
	 else if (CHIFFRE(s[i]))
	 { int v = (s[i] - '0');
		while (++i < strlen(s)  && CHIFFRE(s[i]))
		  v = v*10 + (s[i] - '0');
		empile_res('n',v);
	 }
	 else if (MINUS(s[i]))
	 {	// cas d'une variable
		indvar[s[i] - 'a'] = VRAI;
		empile_res('v',s[i++]-'a');
	 }
	 else if (s[i] == 'S' && s[++i] == 'O' && s[++i] == 'F')
    { /* cas de la constante SOF */
      empile_res('c','S'-'A');
      i++;
    }
    else if (s[i] == '(')  	/* debut de parenthesage */
      niveau++, i++;
    else if (s[i] == ')')
    {	/* fin de parenthesage */
      while (p_op[top_op].niv == niveau) 
        empile_res('o',depile_op());
      niveau--;
      i++;
    }
    else if (OPERATEUR(s[i]))
    {   /* cas d'un operateur */
      int nop = ctoop(s[i]);
      while (top_op && niveau == p_op[top_op-1].niv
	       && prior[nop] <= prior[p_op[top_op-1].op]) 
        empile_res('o',depile_op());
      empile_op(niveau,s[i]);
      i++;
    }
    else affiche_erreur(
	     ml(3705,"erreur de syntaxe dans une expression"));
  }

  while (top_op) empile_res('o',depile_op());
  res = NEW(struct post_pile);
  /* recopie de la pile expression */
  nv_pile = (struct cell *) malloc ((unsigned int) top_res
											* sizeof(struct cell));
  { int j;
	 for (j = 0; j < top_res; j++)
	 nv_pile[j] = p_loc[j];
  }
  // recopie des indicateurs de variables
  { int j;
	 for (j = 0; j < NBVAR; j++)
	 res->indic[j] = indvar[j];
  }
  res->sommet = top_res;
  res->pile = nv_pile;
  return(res);
}

