/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier energies.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include <strstrea.h>
#include <fstream.h>

#include "spectro.h"
#include "param.h"
#include "spectre.h"
#include "fmaxspec.h"
#include "signal.h"
#include "outils.h"
#include "files.h"
#include "energies.h"

#define FENENER 16.

static struct paramsp psp = {false, 0, 0, VRAI, 0., 0., 0., 0, 0.};
struct paramEner psp_ener = {0., 8000., SILENCE_SPECTRO - 40., MAX_SPECTRO + 10., FENENER, false};

//
// Calcul de l'energie
//
static float energieBande(
	FLOATbuf& spec_,
   int cbas,
   int chaut)
{
  int c;
  float s;
  float *spec=(float *)&spec_[0];

  for (c = cbas, s = 0.; c <= chaut; c++) s += spec[c];
  s  = s / max(chaut - cbas + 1, 1);
  return(s);
}

//
// Number of 1  dB between two consecutive graduations.
//
static double nbdBPerGrad()
{
	int nbGradReq = 5;
   // # of graduations available
   int nbGrad = ((int) fMaxSpectro) / 2000;

   if (nbGrad / nbGradReq < 1)
     return((double)(((int) (nbGradReq / nbGrad)) + 1));
   else if (nbGrad / nbGradReq >= 2)
     return((double) 1. / ((int) (nbGrad / nbGradReq)));
   else return(1.);
}

// Maximum difference between lowest and highest energy displayed.
static double ddBMax()
{
  // # of graduations available
  double nbGrad = fMaxSpectro / 2000.;

  return(nbGrad * nbdBPerGrad() * 10.);
}

#define AFFENER(X) ((float) max(0.,  \
         (max(X, psp_ener.dbmin) - psp_ener.dbmin) / \
             ddBMax() * ((float) NB_CANAUX)))

// Initialisation du calcul de l'energie
void initCalculEnergie()
{
  psp_ener.nbPtFenEner = (int) (en_cours.frequence / 1000. * psp_ener.fenEner);
  if (psp_ener.nbPtFenEner > 2048) {
    psp_ener.nbPtFenEner = psp_ener.ordreFFT = 2048;
    psp_ener.fenEner = enMilli(2048);
  }
  else if (psp_ener.nbPtFenEner > 1024) psp_ener.ordreFFT = 2048;
  else if (psp_ener.nbPtFenEner > 512) psp_ener.ordreFFT = 1024;
  else if (psp_ener.nbPtFenEner > 256) psp_ener.ordreFFT = 512;
  else psp_ener.ordreFFT = 256;

  psp_ener.depEner = psp_ener.nbPtFenEner / 5;
  psp_ener.cbas = IROUND(psp_ener.fmin / en_cours.frequence * psp_ener.ordreFFT),
  psp_ener.chaut = min(IROUND(psp_ener.fmax / en_cours.frequence * psp_ener.ordreFFT),
  							  psp_ener.ordreFFT / 2);
  initSpecFft(psp_ener.nbPtFenEner, psp_ener.ordreFFT);
}

static float enerBandeFminFmax(
  int ech,
  SAMPLEbuf& speech)
{
  FLOATbuf X(psp_ener.ordreFFT / 2 + 1,"static X near CMafficheEner()");

  specFft(speech, ech, X, psp.isPreac, true);
  return (energieBande(X, psp_ener.cbas, psp_ener.chaut));
}

// Computes energy for the current signal
float enerBandeFminFmaxCentre(
  int e)
{
   return(enerBandeFminFmax(max(0, e - psp_ener.nbPtFenEner / 2 - ct.d_e), space));
}

/*
 * Affichage de l'energie
 */
void Wtop::CMAfficheEner()
{
  long debut,fin;


  if ( (ct.debRegion >= ct.finRegion)
       || !dans(ct.debRegion, ct.d_e, ct.f_e)
       || !dans(ct.finRegion, ct.d_e, ct.f_e) ) {
    // pas de rgion dlimite, on fait tout l'cran
    debut=ct.d_es;
    fin=ct.f_es;
  }
  else {
    debut=ct.debRegion;
    fin=ct.finRegion;
  }

  {
    int ech;
    float  xcote, ycote, x_, y_, x1, y1, e;
    STRING titre;

    initCalculEnergie();
    xcote = wspectro->wsona->DXtoLX(3) - wspectro->wsona->DXtoLX(0);
    ycote = wspectro->wsona->DYtoLY(0) - wspectro->wsona->DYtoLY(3);
    MgiHue (BLEU, wspectro->wsona);

    logs.open1();
    sprintf(titre,ml(8546,"calcul de l'energie entre %.0f Hz et %.0f Hz"),
    		   psp_ener.fmin, psp_ener.fmax);
    logs.wr1(titre);
    sprintf(titre,ml(8547,"taille de le fenetre %.1f (ms)"), psp_ener.fenEner);
    logs.wr1(titre);
    logs.wr1("[ms]\t[dB]");

    e = enerBandeFminFmax(max(0, debut - psp_ener.nbPtFenEner / 2 - ct.d_e), space);
    x_ = debut;
    y_ = AFFENER(e);
    
    for (ech = debut; ech + psp_ener.nbPtFenEner < fin; ech += psp_ener.depEner) {
      e = enerBandeFminFmax(max(0, ech - psp_ener.nbPtFenEner / 2 - ct.d_e), space);
      x1 = ech;
      y1 = AFFENER(e);
      if (ech != debut) {
        snol(x_, y_, x1, y1, wspectro->wsona);
        x_ = x1;
        y_ = y1;
      }
      sprintf(titre, "%d  %.0f", enMilli(ech+psp_ener.nbPtFenEner/2), e);
      logs.wr1(titre);
    }

    freeSpecFft();
    logs.close1();
    ::InvalidateRect(HWindow,NULL,TRUE);
  }
}


static void calcEnerFile(char *filename, char *fileout)
{
  HANDLE hFile, hFileMapping;
  PBYTE pbFile;
  ofstream outfp;

  outfp.open(fileout,ios::out);
  hFile = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ, NULL,
                 				 OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
  if (hFile == INVALID_HANDLE_VALUE) return;

  hFileMapping = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);
  CloseHandle(hFile);

  if (hFileMapping == NULL) return;
  
  pbFile = (PBYTE) MapViewOfFile(hFileMapping, FILE_MAP_READ, 0, 0, 0);
  
  {
    int ech;
    SAMPLEbuf fenetre(1, "fenetre de signal temporel pour l'energie");
    initCalculEnergie();

    fenetre.newsize(psp_ener.nbPtFenEner+1);
    for (ech = 0; ech < ct.tf; ech += psp_ener.depEner) {
      int n, i;
      float energie;
      char* buffer = (char *) fenetre.p;

      for (i = 0; i < (psp_ener.nbPtFenEner + 1) * sizeof(SAMPLE); i++)
        buffer[i] = pbFile[sizeof(SAMPLE) * ech + en_cours.depl + i];
      if (en_cours.swap != SWAPINFILE)
        swap((char huge *) buffer, (psp_ener.nbPtFenEner + 1) * sizeof(SAMPLE));
      if ((n = NB_BITS - en_cours.nb_bits) > 0) // left shift
        for (i = 0; i < psp_ener.nbPtFenEner + 1; i++)
          fenetre[i] = fenetre[i] << n;
      energie = enerBandeFminFmax(0, fenetre);
      energie = (float) max(0., (max(energie, psp_ener.dbmin) - psp_ener.dbmin));
      outfp << (int)enMilli(ech + psp_ener.nbPtFenEner / 2) << "   " << energie << "\n";
      outfp.flush();
    }
  }

  outfp.close();
  freeSpecFft();
  UnmapViewOfFile(pbFile);
  CloseHandle(hFileMapping);
}

//////////////////////////////////////////////////
// fentre paramtres de calcul de l'energie
//

class TWparasEner :  public TWparasp
{
  struct paramEner *param;

  public:
    TWparasEner(PTWindowsObject AParent,char *titre, struct paramEner *pEner, paramsp *p);

	 void IDdefauts(WPARAM code);
	 void IDdetails(WPARAM code);
	 LRESULT controle_dbmin(WPARAM,LPARAM);
	 LRESULT controle_dbmax(WPARAM,LPARAM);
	 LRESULT controle_fmin(WPARAM,LPARAM);
	 LRESULT controle_fmax(WPARAM,LPARAM);
	 LRESULT controle_fenEner(WPARAM,LPARAM);
	 DECLARE_RESPONSE_TABLE(TWparasEner);
};
DEFINE_RESPONSE_TABLE1(TWparasEner,TWparasp)
  EV_CHILD_NOTIFY_ALL_CODES(93,IDdefauts),
  EV_CHILD_NOTIFY_ALL_CODES(94,IDdetails),
  EV_MESSAGE(WM_USER+2,controle_dbmin),
  EV_MESSAGE(WM_USER+3,controle_dbmax),
  EV_MESSAGE(WM_USER+4,controle_fmin),
  EV_MESSAGE(WM_USER+5,controle_fmax),
  EV_MESSAGE(WM_USER+6,controle_fenEner),
END_RESPONSE_TABLE;

TWparasEner::TWparasEner(PTWindowsObject AParent,char *titre, struct paramEner *pEner,
								 paramsp *p)
	:TWparasp(AParent,titre,p)
{
  param=pEner;
  //
  // initialiser le contenu des contrles
  //   avec les variables correspondantes
  //
  new TWfloat(this,2,xx,yy,ww
	      ,&param->dbmin
	      ,ml(8540,"Energie minimale (dB)")
         ,SILENCE_SPECTRO - 60., MAX_SPECTRO + 40.
         ,SILENCE_SPECTRO - 60., MAX_SPECTRO + 40.
	      ,ticket); yy+=HNUM;
  new TWfloat(this,3,xx,yy,ww
	      ,&param->dbmax
	      ,ml(8541,"Energie maximale(dB)")
         ,SILENCE_SPECTRO - 60., MAX_SPECTRO + 40.
         ,SILENCE_SPECTRO - 60., MAX_SPECTRO + 40.
	      ,ticket); yy+=HNUM;
  new TWfloat(this,4,xx,yy,ww
	      ,&param->fmin
	      ,ml(8542,"Frequence minimale (Hz)")
         ,0., en_cours.frequence / 2.
         ,0., en_cours.frequence / 2.
	      ,ticket); yy+=HNUM;
  new TWfloat(this,5,xx,yy,ww
	      ,&param->fmax
	      ,ml(8543,"Frequence maximale")
         ,0., en_cours.frequence / 2.
         ,0., en_cours.frequence / 2.
	      ,ticket); yy+=HNUM;
  // fenetre signal
  new TWfloat(this,6,xx,yy,ww
	      ,&param->fenEner
	      ,ml(8544,"fenetre_signal (ms)")
	      ,1.,128.
	      ,1.,128.
	      ,ticket); yy+=HNUM;
  FinirCreate(false);
}

LRESULT TWparasEner::controle_dbmin(WPARAM,LPARAM)
{
  if (param->dbmin > param->dbmax) {
    param->dbmin = param->dbmax - 10.;
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  return 0;
}

LRESULT TWparasEner::controle_dbmax(WPARAM,LPARAM)
{
  if (param->dbmin > param->dbmax) {
    param->dbmax = param->dbmin + 10.;
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  return 0;
}

LRESULT TWparasEner::controle_fmin(WPARAM,LPARAM)
{
  if (param->fmin > param->fmax) {
    param->fmin = max(param->fmax - 1000, 0.);
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  else if (param->fmin > en_cours.frequence / 2. - 1000.) {
    param->fmin = max(en_cours.frequence / 2. - 1000, 0.);
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  return 0;
}

LRESULT TWparasEner::controle_fmax(WPARAM,LPARAM)
{
  if (param->fmax > en_cours.frequence / 2.) {
    param->fmax = en_cours.frequence / 2.;
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  else if (param->fmax < param->fmin) {
    param->fmax = min(en_cours.frequence / 2.,  param->fmin + 1000);
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  return 0;
}


LRESULT TWparasEner::controle_fenEner(WPARAM,LPARAM)
{
  if (enEcht((long) param->nbPtFenEner) > 2048) {
    param->fenEner = enMilli(2048);
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  else if (param->fenEner <= 0) {
    param->fenEner = 16.;
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  return 0;
}

void TWparasEner::IDdefauts(WPARAM)
{
  psp_ener.dbmin = SILENCE_SPECTRO - 40.;
  psp_ener.dbmax = MAX_SPECTRO + 10.;
  psp_ener.fmin = 0.;
  psp_ener.fmax = en_cours.frequence;
  psp_ener.fenEner = 16.;
}

void TWparasEner::IDdetails(WPARAM)
{
  CHARbuf buf(1,"");
  STRING ch;

  if (!yaUnSignal)
    buf.cpy(ml(8530,"pas de signal"));
  else {
    sprintf(ch, "Lowest energy in dB %.1f\n", param->dbmin);
    buf.cpy(ch);
    sprintf(ch, "Highest energy in dB %.1f\n", param->dbmax);
    buf.cat(ch);
    sprintf(ch, "Lowest frequency in Hz %.0f\n", param->fmin);
    buf.cpy(ch);
    sprintf(ch, "Highest frequency in Hz %.0f\n", param->fmax);
    buf.cat(ch);
    sprintf(ch, "Length of the window in ms %f\n", param->fenEner);
    buf.cat(ch);
  }

  ::MessageBox(HWindow
	     ,(char *)&buf[0],ml(8512,"details")
		  ,MB_OK | MB_TASKMODAL );
}

//////////////////////////////////////////////////
// commandes menu de la fentre principale
//
void Wtop::CMOptionsEnergie()
{
 if (psp.isOpen) return;
  GetModule()->MakeWindow(
		new TWparasEner(this,ml(8545,"Paramtres de l'nergie"),
                      &psp_ener, &psp));
}

// Calcul et sauvegarde du pitch sur tout le fichier de parole
void Wtop::CMItereEnergie()
{
  STRING filename;
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "*.*",0,"","");

  if (!yaUnSignal) return;

  strcpy(filename,"*.*");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (TFileSaveDialog(Parent,FileData,0).DoExecute() == IDOK) {
   strcpy(filename,FileData.FileName);
   if (droit_ecrire_message(filename))
    calcEnerFile(en_cours.nom, filename);
  }
}


void Wtop::CMEnergieEchelle()
{
  psp_ener.yaUneEchelle = true;
  ::InvalidateRect(wspectro->wsona->HWindow, NULL, TRUE);
}


//
//   Display the frequency scale for F0
//
void dispEnergyScale()
{
  MgiHue (BLEU, wspectro->wsona);

  wspectro->wvscale->affichage(0., ddBMax(), wspectro->wsona->incr / fMaxSpectro * ddBMax(), "Energy dB");

  psp_ener.yaUneEchelle = false;
}


