/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier energyClass.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>

#include "spectro.h"
#include "param.h"
#include "spectre.h"
#include "fmaxspec.h"
#include "signal.h"
#include "outils.h"
#include "files.h"
#include "energyClass.h"


void energy::init(float fmin, float fmax, float fenEner)  {
  nbPtFenEner = (int) (en_cours.frequence / 1000. * fenEner);
  if (nbPtFenEner > 2048) nbPtFenEner = ordreFFT = 2048;
  else if (nbPtFenEner > 1024) ordreFFT = 2048;
  else if (nbPtFenEner > 512) ordreFFT = 1024;
  else if (nbPtFenEner > 256) ordreFFT = 512;
  else ordreFFT = 256;

  cbas = IROUND(fmin / en_cours.frequence * ordreFFT),
  chaut = min(IROUND(fmax / en_cours.frequence * ordreFFT),
  							  ordreFFT / 2);
  X = new FLOATbuf(ordreFFT / 2 + 1,"static X near energyClass");

  initSpecFft(nbPtFenEner, ordreFFT);
}

double energy::compute(
  SAMPLEbuf& speech, int echt)
{

  int c;
  float s;
  float *spec=(float *)&((*X)[0]);

  specFft(speech, echt, *X, isPreac, true);
  for (c = cbas, s = 0.; c <= chaut; c++) s += spec[c];
  s  = s / max(chaut - cbas + 1, 1);
  return(s);
}

// Computes energy for the current signal
double energy::computeCentered(
  SAMPLEbuf& speech,
  int echt)
{
   return(compute(speech, max(0, echt - nbPtFenEner / 2)));
}




