/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier fband.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include "spectre.h"

#define sqr(x) (x * x)
#define  cpai  0.15915    /* 1/(3.14159*2) */

/* modifie le  12 juin 1989 par DF */
/* pour eviter un bug dans le suivi de formants */

// dfinition de la largeur de bande
//   c'est la largeur complte du pic  -3 dB par
//   rapport au sommet du pic.
//     (-3 dB donne la moiti de la hauteur du pic)

/*****************************************************/
/*  Retourner le formant et la largeur de bande        */
/*****************************************************/
void fband(
	int n,        // ordre de LPC */
	float *coef,  // les coefficients de LPC de 0 a` n */
	float fe,     // fe:freq d'echantillonnage
	float bmax,   // bmax:bande max 
	float wa,     // les bornes wa=0, wb=fe/2 en cas normal. */
	float wb,
	float *freq,  // freqs et bandes, freq[0]=band[0]=nb
	float *band)  //                   de formants
{
  int   nb, i, i1, k, immin;
  float mmin, cf,
        cc,    /* une constante */
        bd,    /* bande */
        rr, ri, /* root reel, root imaginaire */
        troot[128];

  freq[0] = 0.0;
  band[0] = 0.0;

  for (i = 0; i < 128; i++) troot[i] = 0.;
  if (n > 0) {
    root(coef, n, troot);
    n = n * 2; 
    cc = cpai * fe; 
    k = 1; 
    nb = 0; 
    cf = 2 * (wb - wa) / fe;
    while (k <= n) { 
      rr = troot[k]; 
      ri = troot[k + 1];
      if ((ri > 0.0) && (rr != 0.0)) { 
        /* z=zr+jzi-->b=2*(2*pai)*ln(sqr(|z|))  */
        bd =  - cc *(float) log((double) (sqr(rr) + sqr(ri)));
        if ((bd > 0.0) && (bd < bmax)) {
          nb++; 
          band[nb] = (bd * cf + wa); 
	       if (rr > 0.0)
	         freq[nb] = (cc * (float) atan((double)(ri / rr)) * cf + wa);
	       else
	         freq[nb] = (cc * (3.14159 + (float) atan((double)(ri / rr))) * cf + wa);
        }
      }
      k = k + 2;
    }

    // tri des formants 
    for (i = 1; i < nb; i++) { 
      mmin = freq[i];
      immin = i;
      for (k = (i + 1); k <= nb; k++) {
        if (freq[k] < mmin) {
          immin = k;
          mmin = freq[k];
        }
      }
      if (immin > i) {
        mmin = freq[immin]; freq[immin] = freq[i]; freq[i] = mmin; 
        mmin = band[immin]; band[immin] = band[i]; band[i] = mmin;
      }
    }

    // si deux formants tombent dans un meme canal 
    // on supprime celui qui a la largeur la plus grande (DF) 
    for(i=2;i<=nb;i++) {
      if (    (int) (freq[i-1]/fe*2.*NB_CANAUX) == (int) (freq[i]/fe*2.*NB_CANAUX)    )
	     // bug corrig ci-dessus
        // en_cours.frequence remplac par fe
	     if (band[i-1]<=band[i]) {
	       // on supprime le ieme
          for(i1 = i; i1 < nb; i1++) {
            freq[i1]=freq[i1+1];
            band[i1]=band[i1+1];
	       }
	       nb--;
	     }
	     else {
	       // on supprime le i-1 ieme
          for(i1=i-1;i1<nb;i1++) {
             freq[i1]=freq[i1+1];
	          band[i1]=band[i1+1];
	       }
	       nb--;
	     }
      }
      freq[0] = (float) nb;
      band[0] = (float) nb;
    }
    else {
      freq[0] = 0.0;
      band[0] = 0.0;
    }

  if (debug==4) {
    char ch[80];
    int i,nb;
    sprintf(ch,"fband() freq[0]=band[0]=%d",nb=freq[0]);
    Wmsg->wr(ch);
    for (i=1;i<=nb;i++) {
      sprintf(ch,"... %d freq=%.2f band=%.2f"
                ,freq[i],band[i]);
      Wmsg->wr(ch);
    }
  }
}   
