/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier fftgen.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>

#include "spectre.h"

#define PIX2    (M_PI * 2)
#define NMAX	 4096
#define LOG2NMAX 12

#define PERMUTE(a,b) tempr=(a);(a)=(b);(b)=tempr

static int bitReverseFft(int x,int nBits);
static void initMask(int n);

static INTbuf tabj1__(NMAX / 2 * LOG2NMAX + 1,"static tabj1_ near fftGen()"),
              tabj2__(NMAX / 2 * LOG2NMAX + 1,"static tabj2_ near ftGen()");
static FLOATbuf tabcos__(NMAX / 2 * LOG2NMAX + 1,"static tabcos_ near fftGen()"),
  	        tabsin__(NMAX / 2 * LOG2NMAX + 1,"static tabsin_ near fftGen()");
static INTbuf tabjn__(NMAX + 1,"static tabjn_ near fftGen()");


/* Ne peut faire les FFT "que" jusqu'a l'ordre 2^32 */
static INTbuf mask_(32,"static mask near fftGen()");

static int *tabj1_,*tabj2_;
static float *tabcos_,*tabsin_;
static int *tabjn_;
static int *mask;

void fftGen(
	FLOATbuf& x_, FLOATbuf& y_,
	int ordre, int log2Ordre)
{
  register i,j1,j2;
  float t1,t2,tempr;
  int nbCalculs = ordre / 2 * log2Ordre;
  float *x,*y;

  x=(float *)&x_[0];
  y=(float *)&y_[0];

  for (i = 1; i <= nbCalculs; i++)
  { j1 = tabj1_[i];
    j2 = tabj2_[i];
    t1 = x[j1] - x[j2];
    t2 = y[j1] - y[j2];
    x[j1] = x[j1] + x[j2];
    y[j1] = y[j1] + y[j2];
    x[j2] = tabcos_[i] * t1 + tabsin_[i] * t2;
    y[j2] = tabcos_[i] * t2 - tabsin_[i] * t1;
  }

  for (i = 1; i <= ordre; i++)
  { int j;
    if ((j = tabjn_[i]) > i)
    { PERMUTE(x[i], x[j]);
      PERMUTE(y[i], y[j]);
    }
  }
}

void initFftGen(
	int ordre,
	int log2Ordre,
	int isign,
	float ) //rayon) paramtre non utilis
{
	INTbuf p2_(LOG2NMAX + 1,"p2 in initFftGen()");
   	int i,lo,k,
   	    lmx,
   	    lix, lm, li;
   	double scl, arg, c, s;

  int *p2=(int *)&p2_[0];

  tabj1_=(int *)&tabj1__[0];
  tabj2_=(int *)&tabj2__[0];
  tabcos_=(float *)&tabcos__[0];
  tabsin_=(float *)&tabsin__[0];
  tabjn_=(int *)&tabjn__[0];
  mask=(int *)&mask_[0];

	if (ordre > NMAX)
	{ affiche_bug(
	    "initFftGen() : ordre de FFT trop grand");
	  return;
	}

	initMask(LOG2NMAX);
	
	for (i = 1, p2[0] = 1; i<= log2Ordre; i++)
	  p2[i] = p2[i-1] << 1;

	for (lo = 1, k = 1; lo <= log2Ordre; lo++) {
       	    lmx = p2[log2Ordre - lo];
            lix = 2 * lmx;
	    scl = (double) PIX2 / lix;
            for (lm = 1; lm <= lmx; lm++) {
                arg = (double) scl * (lm - 1) * isign;
                c = cos(arg);
                s = sin(arg);
                li = lix;
                while (li <= ordre) {
                      tabj1_[k] = li - lix + lm;
                      tabj2_[k] = tabj1_[k] + lmx;
                      tabcos_[k] = c;
                      tabsin_[k] = s;
                      k++;
                      li += lix;
		}
	    }
	}

	for (i = 1; i <= ordre ; i++)
	  tabjn_[i] = bitReverseFft(i, log2Ordre);
}

static void initMask(int n)
{
  int i, m;

  for (i = 0, m = 1; i < n; i++, m <<= 1) mask[i] = m;
}

/*
 * Bit reversal for FFT.
 * bitReverseFft(x) =  bitReverse(x - 1) + 1
 */
static int bitReverseFft(
	int x,		/* to be bit reversed */
	int nBits)	/* maximum number of bits
			       to be reversed */
{
  int i,
      y = 0;	/* result */
  char ch[80];

  if (x < 1 || x > (mask[nBits - 1] << 1))
  { sprintf(ch,"bug in bitReverseFft() near fftGen()\n x=%d out of range", x);
    fatal_error(ch);
    return(-1);
  }

  for (i = 0; i < nBits; i++) 
    y |= ((x - 1) & mask[i]) ? mask[nBits - 1 - i] : 0;

  return(y + 1);
}

#undef NMAX
#undef LOG2NMAX
#undef PERMUTE








