/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fftr.h
//

#ifndef _FFTR_H
#define _FFTR_H

#include <math.h>
#include <malloc.h>
#include <stdio.h>

/*======================================================================*
 * Prototype de calcul de transformee de Fourier reelle		        *
 * IL FAUT definir les constantes suivantes :				*
 * N : l'ordre de la FFT outil, c-a-d pour une transformee sur 512 reels*
 *     N vaut 256 qui est la taille de la transformee complexe qu'il 	*
 *     faut utiliser.							*
 * ENTETE : le nom de la transformee de Fourier reelle construite       *
 * FFTCOMPLEXE : le nom de la transformee de Fourier complexe outil	*
 *               ATTENTION: il faut que cette transformee outil tra-    *
 *		 vaille sur un tableau de float commencant a 0 et qu'el-*
 *		 le ne fasse pas autre chose qu'une transformee de Fou- *
 *		 rier.							*
 * FFTCOMPINV :  le nom de le transformee de Fourier complexe inverse   *
 * REALFTINIT :	 la fonction d'initialisation des tables de sin et cos  *
 * LIBERE :	 le nom de la fonction de liberation des tables trigo   *
 *======================================================================*/
// exemple de fonction gnre
//     void ENTETE(FLOATbuf& data_,int isign)
//         isign==1 transforme de fourier normale
//         isign==-1 transforme de fourier inverse
//

static TabAuxFFTReelles AUX = {(double *) NULL, (double *) NULL, false};

/*
 *
 */

void REALFTINIT ()

{
  int i;

  if (AUX.tabCos) {
    free((char *) AUX.tabCos);
    AUX.tabCos = (double *) NULL;
  }
  if (AUX.tabSin) {
    free((char *) AUX.tabSin);
    AUX.tabCos = (double *) NULL;
  }

  AUX.tabCos = (double *) calloc((unsigned int) N, (unsigned int) sizeof(double));
  AUX.tabSin = (double *) calloc((unsigned int) N, (unsigned int) sizeof(double));

  for (i = 0; i < N; i++) {
    AUX.tabCos[i] = cos(i * 3.14159265358979323846 / N);
    AUX.tabSin[i] = sin(i * 3.14159265358979323846 / N);
  }
}

/*
 * Cette fonction libere l'expace auxiliaire necessaire au calcul de la
 * fft.
 */

void LIBERE ()

{
  if (AUX.tabCos) {
    free((char *) AUX.tabCos);
    AUX.tabCos = (double *) NULL;
  }
  if (AUX.tabSin) {
    free((char *) AUX.tabSin);
    AUX.tabSin = (double *) NULL;
  }
}

void ENTETE(
     FLOATbuf& data_,
     int sens)
{ int nx2, i;
  double hreel, hhreel, him, hhim, c;
  float *data;
  extern void FFTCOMPLEXE (FLOATbuf& data);
  extern void FFTCOMPINV (FLOATbuf& data);

  if (!AUX.init) { AUX.init = true; REALFTINIT(); }

  data=(float *)&data_[0];

  if (sens == 1) {
    c = -0.5;
    FFTCOMPLEXE(data_);
  } else c = 0.5;

  nx2 = 2 * N;
  for (i = 1; i < N / 2; i++) {
    double sin_n, cos_n;
    int inR, inI, iNs2_nR, iNs2_nI;

    sin_n = sens * AUX.tabSin[i];
    cos_n = AUX.tabCos[i];
    inR = i << 1;
    inI = inR + 1;
    iNs2_nR = nx2 - inR;
    iNs2_nI = iNs2_nR + 1;

    hhreel = 0.5 * (data[inR] + data[iNs2_nR]);
    hhim   = 0.5 * (data[inI] - data[iNs2_nI]);
    hreel  = -c  * (data[inI] + data[iNs2_nI]);
    him    = c   * (data[inR] - data[iNs2_nR]);
    data[inR]     =  hhreel + cos_n * hreel + sin_n * him;
    data[inI]     =  hhim   + cos_n * him   - sin_n * hreel;
    data[iNs2_nR] =  hhreel - cos_n * hreel - sin_n * him;
    data[iNs2_nI] = -hhim   + cos_n * him   - sin_n * hreel;
  }
  if (sens == 1) {
    data[0] = (hhreel = data[0]) + data[1];
    data[N + N] = hhreel - data[1];
    data[1] = data[N + N + 1] = 0.;
  } else {
    data[0] = 0.5 * ((hhreel = data[0]) + data[N + N]);
    data[1] = 0.5 * (hhreel - data[N + N]);
    FFTCOMPINV(data_);
  }
}

#undef N
#undef AUX
#undef ENTETE
#undef FFTCOMPLEXE
#undef FFTCOMPINV
#undef LIBERE
#undef REALFTINIT

#endif
