/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include "common.h"
#include <stdlib.h>
#include <math.h>
#include <sys/types.h>
#include <fstream.h>
#include <windows.h>
#include "outils.h"
#include "signal.h"
#include "files.h"
#include "editer.h"
#include "fftlpc.h"
#include "fft_mayer.h"
#include "fmaxspec.h"
#include "editmngmt.h"
#include "spectre.h"
#include "pics.h"
#include "opt_for.h"
#include "filterOLA.h"
#include <list>

using namespace std;

TabPoint<Point2P> ppOLA;       // Polypoints being read.
list<PolyPointsOLA> patternOLA;

static void filtrageOLA(int debEcht, int finEcht, bool force, bool stop);

#define W0 2. // la somme des pondrations de Hamming (2.16) ou Hanning (2.)
static int N = ORDERFFTOLA, L = N / 2;
static FLOATbuf h(1,"static h near filterOLA()");
static double reelle[ORDERFFTOLA], imag[ORDERFFTOLA], queues[2][3 * ORDERFFTOLA / 8];
static double *modif = (double *) NULL;
static float currentGainOLA, currentWidthOLA;
static PolyPointsOLA *trajModif = (PolyPointsOLA *)NULL;
static void captureAnewPoint(int event_x, int event_y);
static int xPtModif, yPtModif;       	      // Coordinates of the point being tracked.
static int indicePtModif;
static int event_x_g, event_x_d;

void drawDownEdit(int event_x, int event_y)
{
  float width;

  width = editManager->GetWidth();
  editManager->GetGain();
  ppOLA.drawDown(event_x, event_y, wspectro->wsona->LYDtoDY(freqToEcr(width)));
}

void drawMotionEdit(int event_x, int event_y)
{
  ppOLA.drawMotion(event_x, event_y);
}

void affichePpOLA(const PolyPointsOLA& x) {
// ATTENTION Il faut vraiment un passage par rfrence car sinon le destructeur
// est appel lors du dpilage et cela est dramatique s'il y a des pointeurs..
  x.aff(0, freqToEcr, x.estSelec(), LINES, ROUGE, ACTIF);
}

// display all the OLA trajectories
void affTrajOLA()
{
  wspectro->wsona->spectro_nettoyer(0,INFINI_I);
  for_each(patternOLA.begin(), patternOLA.end(), affichePpOLA);
  ::InvalidateRect(wspectro->wsona->HWindow,NULL,true);
}

bool isSelected(const PolyPointsOLA& x) {
  return x.estSelec();
}

void removeSelectTrajOLA()
{
   /*list<PolyPointsOLA>::iterator result =
           remove_if(patternOLA.begin(), patternOLA.end(), isSelected);
   patternOLA.erase(result, patternOLA.end()); */
   list<PolyPointsOLA>::iterator i;
   for(i = patternOLA.begin();  i != patternOLA.end();) {
     list<PolyPointsOLA>::iterator j = i;
     i++;
     if ((*j).estSelec()) patternOLA.erase(j);
   }
#ifdef UIY
   int n;
   do {
     n = 0;
     list<PolyPointsOLA>::iterator result = find_if(patternOLA.begin(), patternOLA.end(), isSelected);
     patternOLA.erase(result);
     count_if(patternOLA.begin(), patternOLA.end(), isSelected, n);
   } while (n);
#endif
   // to prevent any ongoing trajectory editing on a trajectory that does not exist anymore.
   trajModif = (PolyPointsOLA *) NULL;
   affTrajOLA();
}

void drawUpEdit(int event_x,int event_y)
{
  PolyPointsOLA *pppOLA;
  ppOLA.drawUp(event_x, event_y);

  int r = TWchooseOLA(PTWtop,"WchooseOLA").Execute();
  if (r == IDOK) {
    for (int j = 0; j < ppOLA.ni; j++) {
      ppOLA[j].y[0] = canToHz(ppOLA[j].y[0]);
      ppOLA[j].date = enMilli(ppOLA[j].date);
    }
    ppOLA.enEcht();
    pppOLA = new PolyPointsOLA(ppOLA);
    pppOLA->echantillonneEcht(L/4);
    pppOLA->width = currentWidthOLA;
    pppOLA->gain = currentGainOLA;
    /* D'abord on efface tous les PolyPoints affiches. */
    wspectro->wsona->spectro_nettoyer(0,INFINI_I);
    if (dans(pppOLA->debut(), enMilli(ct.d_a), enMilli(ct.f_a))
	      || dans(pppOLA->fin(), enMilli(ct.d_a), enMilli(ct.f_a))
	      || dans(enMilli(ct.d_a), pppOLA->debut(), pppOLA->fin())
	      || dans(enMilli(ct.f_a), pppOLA->debut(), pppOLA->fin()))
    patternOLA.push_back(*pppOLA);
    patternOLA.sort();
    /* STRING s;
    sprintf(s,"APrs cration date[0] = %d",truc.pts[0].date);
    affiche_erreur(s); */

    affTrajOLA();
  }
  ::InvalidateRect(wspectro->wsona->HWindow,NULL,true);
}


DEFINE_RESPONSE_TABLE1(TWchooseOLA,TDialog)
  EV_CHILD_NOTIFY_ALL_CODES(IDOK,IDok),
  EV_CHILD_NOTIFY_ALL_CODES(IDCANCEL,IDcancel),
END_RESPONSE_TABLE;
TWchooseOLA::TWchooseOLA(PTWindowsObject AParent, LPSTR name)
	:TDialog(AParent,name)
{
  PTWidth = new TEdit(this, IDC_EDIT5);
  PTGain = new TEdit(this, IDC_EDIT4);
  add = new TButton(this, IDOK);
  forget = new TButton(this, IDCANCEL);
}

TWchooseOLA::~TWchooseOLA()
{
  delete add;
  delete forget;
  delete PTWidth;
  delete PTGain;
}

void TWchooseOLA::SetupWindow()
{
  STRING s;
  TDialog::SetupWindow();
  sprintf(s,"%.0f", widthOLA);
  PTWidth->SetText(s);
  sprintf(s,"%.0f", gainOLA);
  PTGain->SetText(s);
  //  TRect rect = GetWindowRect();
  // MoveWindow(rect.left, rect.top, 160, height, false);
}

bool TWchooseOLA::CanClose()
{
  STRING s;
  PTWidth->GetText(s,10);
  currentWidthOLA = atof(s);
  PTGain->GetText(s,10);
  currentGainOLA = atof(s);
  return(true);
}
void TWchooseOLA::IDok(WPARAM)
{
  CloseWindow(IDOK);
}

void TWchooseOLA::IDcancel(WPARAM)
{
  CloseWindow(IDCANCEL);
}


static int dateMinEcht(bool force)
{
   list<PolyPointsOLA>::iterator i;
   int dateMini = INFINI_I;
   for(i = patternOLA.begin();  i != patternOLA.end(); i++)
     if (   (force || (*i).estSelec())
         && (*i).pts[0].dateEcht > ct.d_e
         && (*i).pts[0].dateEcht < dateMini) dateMini = (*i).pts[0].dateEcht;
   return(dateMini);
}

static int dateMaxEcht(bool force)
{
   list<PolyPointsOLA>::iterator i;
   int dateMaxi = -INFINI_I;
   for(i = patternOLA.begin();  i != patternOLA.end(); i++)
     if ((force || (*i).estSelec())
         && (*i).pts[(*i).nbPt - 1].dateEcht < ct.f_e
         && (*i).pts[(*i).nbPt - 1].dateEcht > dateMaxi) dateMaxi = (*i).pts[(*i).nbPt - 1].dateEcht;
   return(dateMaxi);
}

static void getGainProfile(
  float *profile,
  int dateEcht,
  bool force,
  bool stop)
{
   list<PolyPointsOLA>::iterator i;
   int iwidth;
   float gain, minGain;

   for (int j = 0; j < N; j++) profile[j] = (stop) ? 1. : 0.;

   for(i = patternOLA.begin(), minGain = INFINI_F;  i != patternOLA.end(); i++)
     if ((force ||(*i).estSelec()) && dans(dateEcht,(*i).debutEcht(), (*i).finEcht())) {
       Point2P *pt = (*i).getPoint();
       iwidth = (*i).width / en_cours.frequence * N / 2;
       gain = (*i).gain;
       if (gain < minGain) minGain = gain;
       if (pt) {
         for (int j = maxi(0, pt->y[0] / en_cours.frequence * N - iwidth);
              j < mini(N / 2 - 1, pt->y[0] / en_cours.frequence * N + iwidth); j++)
           profile[j] =  (stop) ? exp(log(10.) * gain / 20.) : 1.;
       }
     }

   if (!stop) {
     if (minGain > INFINI_F / 2.)
       minGain = 0.;
     for (int j = 0; j < N; j++) if (!profile[j]) profile[j] = exp(log(10.) * minGain / 20.);
   }
}

static void filtrageOLA(
  int debutEcht,
  int finEcht,
  bool force,
  bool stop)
{
  long dfen = debutEcht,
       ffen = finEcht,
       deb = max(dfen - L/2,  0),
  	    fin = min(ffen + L/2, ct.tf - 1);
  long tailleRegion = fin - deb + 1;
  int im, im0, jm, lm, i, j, ipp, lim, ljm;
  float gain[ORDERFFTOLA];

  if (!droit_ecrire_message(en_cours.nom)) return;

  if (deb > fin || !dans(deb, ct.d_e, ct.f_e)
		|| !dans(fin, ct.d_e, ct.f_e))
  { affiche_erreur(ml(9102,"Une ou plusieurs des extrmits trajectoires sont en dehors de la fentre\n Il faut modifier la taille de la fentre de signal"));
    return;
  }

  infoEdit.derOp = FILTEROLA;
  infoEdit.debut = deb;
  infoEdit.fin = fin;
  strcat(strcpy(infoEdit.cut.nom,tempdir),"cutfile_.wa$");
  infoEdit.cut.nb_bits = NB_BITS;
  infoEdit.cut.depl = 0;
  infoEdit.cut.swap = SWAPINFILE;

  /* On sauve le signal qui va etre coupe */
  copieDansCutFile(&infoEdit);

  // initialisations du filtrage
  h.newsize(L + 1);
  inithan(&h, L);
  modif = (double *) calloc(ffen - dfen + L, sizeof(double));
  for (int i = 0; i < ffen - dfen + L; i++) modif[i] = 0.;
  for (int i = 0; i < 3 * L / 4; i++) queues[1][i]= queues[0][i] = 0.;

  // multiplication de l'amplitude avec la fenetre de hamming
  // et padding de zero
  for (im0 = im = _i(max(0, dfen - L / 2)), jm = 0; im <= _i(min(ffen - L / 2, ct.tf - L));
  		 im += L/4, jm += L/4){
    for (i = 0; i < L; i++) {
      reelle[i] = space[im + i] * h[i];
      imag[i]=0.;
      if (im + i < _i(dfen) + L / 4)
        queues[0][jm + i] += h[i];
      if (im + i >= _i(ffen) - L / 4)
        queues[1][im + i - _i(ffen) + L / 4] += h[i];
    }
    for(i = L; i < N; i++){
      reelle[i] = imag[i] = 0.;
    }

    // Transformee de Fourier
    fft(N, reelle, imag);

    getGainProfile(gain, im + ct.d_e + L / 2, force, stop);
    for (int k = 1; k < N/2; k++) {
       reelle[k] = reelle[N-k] = reelle[k] * gain[k];
       imag[k] = imag[k] * gain[k];
       imag[N-k] = -imag[k];
    }
    // Inverse de la transformee de Fourier
    ifft(N, reelle, imag);

    ljm = jm;
    lim = im;
    for (i = 0; i < L; i++) {
      modif[lm = jm + i] += reelle[i] / (N * W0);
    }   

  }
  for (int i = 0; i < 3 * L / 4; i++)
    modif[i] = (modif[i] * W0 + space[im0 + i] * (W0 - queues[0][i])) / W0;
  for (int i = ljm + L / 4, j = 0; i <= lm; i++, j++)
    modif[i] = (modif[i] * W0 + space[lim + L / 4 + j] * (W0 - queues[1][j])) / W0;
  for (im = _i(max(0, dfen - L / 2)), jm = 0; jm < lm; im++, jm++)
    space[im] = (SAMPLE) modif[jm];

  free((char *) modif);
  // bout avant la zone amortie : conserv en place
  ajtBoutSpectro(ct.d_a, infoEdit.debut - 1, ct.d_a, infoEdit.debut - 1);  
  // bout aprs la zone amortie : conserv en place
  ajtBoutSpectro(infoEdit.debut + tailleRegion + 1, ct.f_a,
		 infoEdit.debut + tailleRegion + 1, ct.f_a);
  // bout amorti : recalcul
  ajtBoutSpectro(-1, -1, infoEdit.debut, infoEdit.debut + tailleRegion);

  wsignal->aff();
  wsignal->paint();
  spectroRapide(wspectro->wsona);
  
  infoEdit.yaDesModifsSignal = VRAI;
  ct.debRegion = INFINI_L;
  ct.finRegion = -INFINI_L;
}


/*
 * Trouve le PolyPoint auquel appartient un point, et en cas de succes le point
 * pointee par la souris si elle est suffisamment proche d'un point de la courbe.
 */

PolyPointsOLA* trouvePtOLA(
  int date,	/* en ms */
  float y,   // ordonnee exprimee dans la grandeur du polypoints (Hz, dB ...)
  int *indice,
  int onForce)		/* FAUX pour chercher parmi les polypoints selectionnes, VRAI pour tous les polypoints. */
{
  // PolyPointsOLA& bingo;
  list<PolyPointsOLA>::iterator i, bingo;
  bool trouve = false;

  float domyMax = fMaxSpectro;
  /* correction sur l'ordonnee pour calculer le seuil de proximite sur l'ecran. */
  float correc = (((float) enMilli(ct.f_a - ct.d_a + 1)) / ((float)wspectro->wsona->largeur)) /
		  (domyMax / ((float) wspectro->wsona->hauteur));

  float seuil =  5. * ((float) enMilli(ct.f_a - ct.d_a + 1)) / ((float)wspectro->wsona->largeur);

  for(i = patternOLA.begin(); i != patternOLA.end(); ++i) if ((*i).estSelec() || onForce) {
     int imin;
     float distCourbe, distPoint; // On conserve la distance minimale a la courbe et la
      					              // distance minimale aux points de la courbe.
     (*i).distance(date, y, 0, correc, seuil, &distPoint, &distCourbe, &imin);

     if (distCourbe < seuil) {
       bingo = i;
       trouve = true;
     }

     if (distPoint < seuil) *indice = imin;
     else *indice = -1;
  }
  return((trouve) ? &(*bingo) : (PolyPointsOLA *) NULL);
}


/*
 * Retourne le polyPoint auquel un point appartient.
 */
PolyPointsOLA* trouvePtTrajOLA (
  int date,	/* point que l'on cherche.  date (ms) et ordonnee entiere dans la fenetre. */
  float ordonnee, /* coordonnee reelle dans la fenetre du spectro (entre 0 et NBCANAUX). */
  int *indice)
{
  PolyPointsOLA *bingo;

  /* D'abord on cherche le point dans les polypoints selectionnes. */
  bingo = trouvePtOLA (date, ordonnee, indice, FAUX);

  if (bingo) return(bingo);
  /* Et si on n'a pas trouve on le cherche parmi tous les polypoints. */
  bingo = trouvePtOLA (date, ordonnee, indice, VRAI);
  
  return (bingo);
}


/*
 * Selection ou dselection de plusieurs trajectoires.
 */
static void selectTrajOLA(
  PolyPointsOLA *ppoint,
  int mode)
{
  list<PolyPointsOLA>::iterator i;
  if (!mode || !ppoint) {
    for(i = patternOLA.begin(); i != patternOLA.end(); ++i) (*i).selectionne(false);
  }
  if (ppoint) ppoint->selectionne(true);
}

/*
 * Fin du deplacement des points dans un polypoints.
 * On reechantillonne le PolyPoints.
 */
void arretModifTrajOLA()
{
  selectTrajOLA((PolyPointsOLA *) NULL, 0);
  trajModif->echantillonneEcht(L/4);

  affTrajOLA();
  trajModif = (PolyPointsOLA *) NULL;
}

void drawDownEditPiqTraj(int event_x, int event_y)
{
  int indice, date;
  float x, ordonnee;
  PolyPointsOLA *pp;

  snoiunscalexy(event_x, event_y, &x, &ordonnee, wspectro->wsona);
  date = enMilli((long) x);
  pp = trouvePtTrajOLA(date, canToHz(ordonnee), &indice);
  if (pp) {
     pp->selectionne(true);
     editManager->setCurrentPar(pp->gain, pp->width);
  }
  if (!pp) {  // c'est la deselection d'un ou de plusieurs polypoints.
    int nbSelected = 0;
    count_if(patternOLA.begin(), patternOLA.end(), isSelected, nbSelected);
    if (nbSelected == 1) {
       list<PolyPointsOLA>::iterator result =
           find_if(patternOLA.begin(), patternOLA.end(), isSelected);
       editManager->getCurrentPar(&(*result).gain, &(*result).width);
    }
    if (trajModif) arretModifTrajOLA();	// il y avait un PP en cours de modification.
    selectTrajOLA((PolyPointsOLA *) NULL, 0);
    affTrajOLA();
  }
  else if (pp->estSelec() == false) { // c'est la selection d'un PolyPoint.
    pp->selectionne(true);
    affTrajOLA();
  }
  else if (indice > -1) { // c'est le deplacement d'un point d'un PolyPoint selectionne.
    if (trajModif && trajModif != pp)  // on interdit la mise a jour simultanee de plusieurs PP.
      arretModifTrajOLA();
    else {
      trajModif = pp;
      indicePtModif = indice;
      xPtModif = wspectro->wsona->LXtoDX(enEcht(trajModif->date(indice)));
      yPtModif = wspectro->wsona->LYtoDY(freqToEcr(trajModif->y(indice,0)));
      if (indice > 0)
        event_x_g = maxi(0,
        						 wspectro->wsona->LXtoDX(maxi(ct.d_es,(float)trajModif->dateEcht(indice-1))));
      else event_x_g = maxi(0, 2 * wspectro->wsona->LXtoDX(maxi(ct.d_es,(float)trajModif->dateEcht(indice)))
                              - wspectro->wsona->LXtoDX(maxi(ct.d_es,(float)trajModif->dateEcht(indice+1))));
      if (indice < pp->nbPt - 1)
        event_x_d = mini(wspectro->wsona->largeur-1,
                         wspectro->wsona->LXtoDX(mini(ct.f_es,(float)trajModif->dateEcht(indice+1))));
      else event_x_d = mini(wspectro->wsona->largeur-1,
                            2 * wspectro->wsona->LXtoDX(mini(ct.f_es,(float)trajModif->dateEcht(indice)))
                            - wspectro->wsona->LXtoDX(mini(ct.f_es,(float)trajModif->dateEcht(indice - 1))));
    }
  }
}


void drawMotionEditPiqTraj(int event_x, int event_y)
{
  HDC hDC;
  LOGPEN MyPen;
  HPEN hMyPen,holdpen;

  if (!trajModif) return;
  if (  (event_x <= event_x_g)
      ||(event_x >= event_x_d) ) return;

  hDC=GetDC(wspectro->wsona->HWindow);
  GetObject(GetStockObject(BLACK_PEN),sizeof(LOGPEN),&MyPen);
  hMyPen=CreatePenIndirect(&MyPen);
  holdpen=(HPEN)SelectObject(hDC,hMyPen);

  SetROP2(hDC,R2_NOT);

  // First the old point and segments connected to this point are erased
  InvertPointSel(hDC, xPtModif, yPtModif);
  if (indicePtModif) {
    MoveToEx(hDC, wspectro->wsona->LXtoDX(trajModif->dateEcht(indicePtModif - 1)),
		       wspectro->wsona->LYtoDY(freqToEcr(trajModif->y(indicePtModif - 1, 0))),(POINT *) NULL );
    LineTo(hDC, xPtModif, yPtModif);
  }

  if (indicePtModif < trajModif->nbPt - 1) {
    MoveToEx(hDC, xPtModif, yPtModif, (POINT *) NULL);
    LineTo(hDC,wspectro->wsona->LXtoDX(trajModif->dateEcht(indicePtModif + 1)),
           wspectro->wsona->LYtoDY(freqToEcr(trajModif->y(indicePtModif + 1, 0))) );
  }

  // Then the new point is displayed.
  // Modifying date is not allowed if the parameter is not a major parameter.
  InvertPointSel(hDC, event_x, event_y);

  if (indicePtModif) {
    MoveToEx(hDC,wspectro->wsona->LXtoDX(trajModif->dateEcht(indicePtModif - 1)),
			    wspectro->wsona->LYtoDY(freqToEcr(trajModif->y(indicePtModif - 1, 0))),(POINT *) NULL );
	 LineTo(hDC, event_x, event_y);
  }
  if (indicePtModif < trajModif->nbPt - 1) {
    MoveToEx(hDC,  event_x, event_y, (POINT *) NULL);
	 LineTo(hDC,wspectro->wsona->LXtoDX(trajModif->dateEcht(indicePtModif + 1)),
           wspectro->wsona->LYtoDY(freqToEcr(trajModif->y(indicePtModif + 1, 0))));
  }
  SetROP2(hDC,R2_COPYPEN);

  SelectObject(hDC,holdpen);
  DeleteObject(hMyPen);
  ReleaseDC(wspectro->wsona->HWindow,hDC);

  xPtModif = event_x;
  yPtModif = event_y;
}

void drawUpEditPiqTraj(int event_x, int event_y)
{
  if (!trajModif) return;
  else {
    float x, ordonnee;
    int date, dateEcht;
    snoiunscalexy(xPtModif, yPtModif, &x, &ordonnee, wspectro->wsona);
    // It is better not to modify the abscissa
    // dateEcht = (int) x;
    // date = enMilli(dateEcht);
    // *(trajModif->pdate(indicePtModif)) = date;
    // *(trajModif->pdateEcht(indicePtModif)) = dateEcht;
    *(trajModif->py(indicePtModif, 0)) = canToHz(ordonnee);
    affTrajOLA();
  }
}


void applyOLAPattern(bool force, bool stop)
{
   list<PolyPointsOLA>::iterator i;

   if (!force) {
     int nbSelected = 0;
     count_if(patternOLA.begin(), patternOLA.end(), isSelected, nbSelected);
     // if only one traj is selected we get the current width and gain parameters
     if (nbSelected == 1) {
       list<PolyPointsOLA>::iterator result =
           find_if(patternOLA.begin(), patternOLA.end(), isSelected);
       editManager->getCurrentPar(&(*result).gain, &(*result).width);
    }
   }
   // initialisation du parcours des polypoints.
   for(i = patternOLA.begin();  i != patternOLA.end(); i++)
     if (force || (*i).estSelec())  (*i).seek(0);
   filtrageOLA(dateMinEcht(force), dateMaxEcht(force), force, stop);
   affTrajOLA();
}

void forgetOLAPattern()
{
   patternOLA.erase(patternOLA.begin(), patternOLA.end());
   affTrajOLA();
}

// Save the OLA pattern
void saveOLAPattern()
{
  STRING filename;
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "*.*",0,"","");
  if (yaUnSignal == NOSIGNAL) return;

  strcpy(filename,"*.*");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (TFileSaveDialog(PTWtop,FileData,0).DoExecute() == IDOK) {
   strcpy(filename,FileData.FileName);
   if (droit_ecrire_message(filename)) {
	  list<PolyPointsOLA>::iterator i;
     FILE *f = fopen(filename, "wb");

     souris (XC_watch);
     fwrite(".ola", sizeof(char), 4, f); // pour reconnaitre le fichier

     // initialisation du parcours des polypoints.
     for(i = patternOLA.begin();  i != patternOLA.end(); i++) (*i).save(f);
     fflush(f);
     fclose(f);
     souris (XC_top_left_arrow);
   }
  }
}

//
// Read an OLA pattern
//
void readOLAPattern()
{
 STRING filename;
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "*.*",0,"","");

  if (yaUnSignal == NOSIGNAL) return;
  patternOLA.erase(patternOLA.begin(), patternOLA.end());
  // to prevent any ongoing trajectory editing on a trajectory that does not exist anymore.
  trajModif = (PolyPointsOLA *) NULL;

  strcpy(filename,"*.*");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (TFileOpenDialog(PTWtop,FileData,0).DoExecute() == IDOK) {
    strcpy(filename,FileData.FileName);
    if (is_readable(filename) == 1) {
       char ext[5];
       FILE *f = fopen(filename, "rb");
       PolyPointsOLA traj;

       fread(ext, sizeof(char), 4, f);
       if (strncmp(ext, ".ola", 4)) {
         affiche_erreur(ml(9101,"Ce n'est pas un fichier OLA"));
         fclose(f);
         return;
       }
       do {
         traj.read(f);
         if (traj.nbPt) {
         	patternOLA.push_back(traj);
            patternOLA.sort();
            traj.pts.newsize(0);
         }
       }
       while (traj.nbPt);
       fclose(f);
    }
    else affiche_erreur(ml(9100,"Fichier OLA inaccessible"));

    affTrajOLA();
  }
}

// Delete all OLA trajectories before reading a new speech file
void trajOLAClear()
{
  patternOLA.erase(patternOLA.begin(), patternOLA.end());
}

int extractPeaks(
	double x[],		// spectrum
   int fftOrder,		// size of the spectrum
   float minDomination,  	  //  minimal domination in dB
	PicsSpectre& ex)          // the set of peaks
{
  int i, j;	/* indices courbe et extremum */

  ex.resize(fftOrder / 2);
  i = 0;
  ex[0].xi = i;
  ex[0].frHz = en_cours.frequence / fftOrder * i;
  ex[0].v = x[i];

  j = 1;

  while (i < fftOrder / 2 - 1) {
    if (x[i] < x[i+1]) {	/* a local maximum */
      float xm, ym;
      for (;x[i] <= x[i+1] && i< fftOrder / 2 - 1; i++);
      ex[j].xi = i;
      ex[j].v = x[i];
      ex[j].frHz = en_cours.frequence / fftOrder * i;
      j++;
    }
    else {	/* a local minimum */
      float muet;
      for (;x[i] >= x[i+1] && i < fftOrder / 2 - 1;i++);
      ex[j].xi = i;
      ex[j].frHz = en_cours.frequence / fftOrder * i;
      ex[j].v = x[i];
      j++;
    }
  }

  if (j > 2) {
    ex.sens = (ex[1].v > ex[0].v) ? 1 : 0;
    ex.resize(j - 1);
  }
  else {
    ex.sens = 1;
    ex.resize(0);
  }

  /* only peaks that dominate neighbour minima "minDomination" at least are kept. */
  for (i = 0, j = ex.sens; j < ex.size(); j += 2)
    if ( (!j || ex[j].v > ex[j-1].v + minDomination) && (j == ex.size() - 1 || ex[j].v > ex[j+1].v + minDomination))
      ex[i++] = ex[j];

  ex.resize(i);
}

#define FENLPC	16 // en ms
static void lockOnaSpectrum(
  int dateEcht,
  int analysis,
  bool initialize)
{
   static int ordreFFT, fenFFT;
   list<PolyPointsOLA>::iterator i;
   racines1_lpc res_lpc;
   fenFFT = enEcht(32);
   ordreFFT = (fenFFT < 1024) ? 1024 : 2048;
   double *X = (double *) calloc(ordreFFT / 2 + 1, sizeof(double));
   FLOATbuf	XX(ordreFFT / 2 + 1, "Calcul du spectre harmonique");
   float spectre[512];

   if (initialize) initSpecFft(fenFFT, ordreFFT);

   // Computation of the locking spectrum
   switch (analysis) {
     case FORMANTLPC:
      spectreLPC(spectre, res_lpc, &space[0], dateEcht - enEcht(FENLPC) / 2 - ct.d_e,
      			  enEcht(FENLPC), 512,
                 true, ordre_lpc_msTOecht(opt_for.or_lpc), 1.);
      break;
     case HARMONICS: {
      specFft(space, dateEcht - fenFFT / 2 - ct.d_e, XX, true, true);
      for (int j = 0; j < ordreFFT / 2; j++) X[j] = XX[j]; // copie, hlas
      }
      break;
   }
   
   for(i = patternOLA.begin();  i != patternOLA.end(); i++)
     if ((*i).estSelec() && dans(dateEcht,(*i).debutEcht(), (*i).finEcht())) {
       Point2P *pt = (*i).getPoint();

       if (pt) switch(analysis) {
        case FORMANTLPC: {
           int j, jmin;
           for (j = 0, jmin = 0; j < res_lpc.size(); j++)
              jmin = (   fabs(res_lpc[j].fr - pt->y[0]) <
		         fabs(res_lpc[jmin].fr - pt->y[0])
                      && res_lpc[j].lar < 700.) ? j : jmin;
            if (res_lpc.size() && fabs(res_lpc[jmin].fr - pt->y[0]) < 500.)
              pt->y[0] = res_lpc[jmin].fr;
          }
          break;
        case HARMONICS: {
          int j, jmin;
          PicsSpectre pics;
          extractPeaks(X, ordreFFT, 5., pics);
          for (j = 0, jmin = 0; j < pics.size(); j++)
              jmin = (fabs(pics[j].frHz - pt->y[0]) <
		         				fabs(pics[jmin].frHz - pt->y[0])) ? j : jmin;
          if (pics.size() && fabs(pics[jmin].frHz - pt->y[0]) < 500.)
              pt->y[0] = pics[jmin].frHz;
          break;
       }
     }
   }
   free((void *) X);
}

void lockOLAPattern(int analysis)
{
   list<PolyPointsOLA>::iterator i;

   // initialisation du parcours des polypoints.
   for(i = patternOLA.begin();  i != patternOLA.end(); i++) (*i).seek(0);
   for (int date = dateMinEcht(false); date <= dateMaxEcht(false); date += L / 4)
     lockOnaSpectrum(date, analysis, (date == dateMinEcht(false)) ? true : false);
   affTrajOLA();
}


