/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include "polypoint.h"

#define ORDERFFTOLA 1024

class PolyPointsOLA : public PolyPointsAD<TabPoint<Point2P>, Point2P> {
  public:
   PolyPointsOLA(TabPoint<Point2P> &tabPoints)
     : PolyPointsAD<TabPoint<Point2P>, Point2P>(tabPoints) {
       gain = 0.;
       width = 0.;
   };
   PolyPointsOLA()
     : PolyPointsAD<TabPoint<Point2P>, Point2P>() {
     gain = 0.;
     width = 0.;
   };
   PolyPointsOLA(const PolyPointsOLA&  x) : PolyPointsAD<TabPoint<Point2P>, Point2P>(x)	{
      gain = x.gain;
      width = x.width;
   }
   PolyPointsOLA& operator = (const PolyPointsOLA&  x) {
      gain = x.gain;
      width = x.width;
      return(*this);
   }
   
   void save(FILE *f) {
     fwrite(&nbPt, sizeof(long), 1, f);
     fwrite(&gain, sizeof(float), 1, f);
     fwrite(&width, sizeof(float), 1, f);
     for (int i = 0; i < nbPt; i++) {
       int date = pts[i].dateEcht;
       float y = pts[i].y[0];
       fwrite(&date, sizeof(int), 1, f);
       fwrite(&y, sizeof(float), 1, f);
     }
   }
   void read(FILE *f) {
     int x;
     x = fread(&nbPt, sizeof(long), 1, f);
     if (!x) {
       nbPt = 0;
       pts.newsize(nbPt);
       return;
     }
     if (x) x = fread(&gain, sizeof(float), 1, f);
     if (x) x = fread(&width, sizeof(float), 1, f);
     pts.newsize(nbPt);
     if (x) for (int i = 0; i < nbPt && x; i++) {
       x = fread(&pts[i].dateEcht, sizeof(int), 1, f);
       pts[i].date = enMilli(pts[i].dateEcht);
       if (x) x = fread(&pts[i].y[0], sizeof(float), 1, f);
     }
     if (!x) {
       affiche_erreur("read PolyPointsOLA: unconsistent file");
       pts.newsize(0);
     }
   }
  float gain, width;
};

/////////////////////////////////////////////
// fentre WchooseOLA
//
class TWchooseOLA : public TDialog
{
    TEdit *PTWidth;
    TEdit *PTGain;
    TButton *forget, *add;
 public:
    TWchooseOLA(PTWindowsObject AParent, char *title);
    ~TWchooseOLA();
    void IDok(WPARAM code);
	 void IDcancel(WPARAM code);

    virtual void SetupWindow();
    virtual bool CanClose();
    DECLARE_RESPONSE_TABLE(TWchooseOLA);
};


#define HARMONICS 	0
#define FORMANTLPC	1
struct picSpectre { // 1 pic de spectre
  int xi;	    /* abscisse du point (numero du coef FFT */
  float frHz,   /* frequence du point en Hz */
        v,	    /* amplitude de ce point */
        q;
};

class PicsSpectre { // tous les pics d'un spectre
  public:
    int ni;
    int sens;
    struct picSpectre **p;

    PicsSpectre() {
      ni = 0;
      p = (struct picSepstre **) NULL;
    }

    ~PicsSpectre() {
        clear();
        if (p) free(p);
        p = (struct picSpectre **) NULL;
    };

    void clear() {
     int i;
     for (i = 0; i < ni; i++) delete p[i];
     ni = 0;
     if (p) free(p);
     p = (struct picSpectre **)NULL;
    }

    int size() { return ni; }

    void resize(int nb) {
     if (nb == 0) { clear(); return; }
     if (nb > this->ni) {
       p = (picSpectre **) realloc(p, ((long)nb) * sizeof(picSpectre *));
       for (int i = this->ni; i < nb; i++)
       p[i] = new picSpectre();
     }
     if (nb < this->ni) {
       for (int i = this->ni - 1; i >= nb; i--)
       delete p[i];
       p = (picSpectre **)realloc(p,( (long)nb)*sizeof(picSpectre *));
     }
     this->ni = nb;
    }
    picSpectre & operator[](int i) { return *p[i]; }
};

extern int extractPeaks(double x[], int fftOrder, float minDomination, PicsSpectre& ex) ;
extern void drawDownEdit(int event_x, int event_y);
extern void drawMotionEdit(int event_x, int event_y);
extern void drawUpEdit(int event_x, int event_y);
extern void drawDownEditPiqTraj(int event_x, int event_y);
extern void drawMotionEditPiqTraj(int event_x, int event_y);
extern void drawUpEditPiqTraj(int event_x, int event_y);
extern void removeSelectTrajOLA();
extern void affTrajOLA();
extern void trajOLAClear();
extern void applyOLAPattern(bool force, bool stop);
extern void forgetOLAPattern();
extern void readOLAPattern();
extern void saveOLAPattern();
extern void lockOLAPattern(int analysis);

