/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include "common.h"
#include <stdlib.h>
#include <math.h>
#include <sys/types.h>
#include <fstream.h>
#include <windows.h>
#include "outils.h"
#include "signal.h"
#include "files.h"
#include "editer.h"
#include "fftlpc.h"
#include "fft_mayer.h"
#include "fmaxspec.h"
#include "editmngmt.h"
#include "spectre.h"
#include "pics.h"
#include "opt_for.h"
#include "complex.h"
#include "spectro.h"
#include "pitchm.h"
#include <list>

#define ORDERFFTOLA 1024

using namespace std;

void addVoicingOLA(short *speechin, short *speechout, struct ctrl_temps ctloc, int debEcht, int finEcht, double forcedF0,
					    double Fs, double voicingLevel,
                   double pseudoF1, double bwF1, double dbF1, double higherFrequencyAttenuation, bool addFormant);
//									double a, double b, double c, double f1, double f2);

#define W0 2. // la somme des pondrations de Hamming (2.16) ou Hanning (2.)
static int N = ORDERFFTOLA, L = N / 2;
static FLOATbuf h(1,"static h near filterOLA()");
static double reelle[ORDERFFTOLA], imag[ORDERFFTOLA], reelleSp[ORDERFFTOLA], imagSp[ORDERFFTOLA], module[ORDERFFTOLA],
              voicingModule[ORDERFFTOLA], phase[ORDERFFTOLA], phase0[ORDERFFTOLA], queues[2][3 * ORDERFFTOLA / 8];
static double *modif = (double *) NULL;

double preemphasisDB(
  double f,        // frequency
  double Fs)       // sampling frequency
{
	double level = 20. * log10(sqrt((1. - cos(f/Fs* _PI)) * (1. - cos(f/Fs* _PI)) + sin(f/Fs* _PI) * sin(f/Fs* _PI)));
   return(level);
}

/*
 * Amplitude de la fonction de transfert d'un formant (log of the amplitude)
 */
double formantAmplitude(
  double Fi,	/* formant frequency  */
  double bw,	/* formant bandwidth  */
  double f)	   /* frequency where to compute the amplitude */
{
  float b = 2.* exp(- _PI * bw / en_cours.frequence) * cos(2.* _PI * Fi / en_cours.frequence),
        c = -exp(-2. * _PI * bw / en_cours.frequence),
        a = 1 - b - c;
  fcomplex zinv = Cinv(CComplex(cos(2. * _PI * f / en_cours.frequence), sin(2. * _PI * f / en_cours.frequence))),
    h = RCmul(a, Cinv(Csub(Csub(CComplex(1.,0.), RCmul(b, zinv)), RCmul(c, Cmul(zinv,zinv)))));
  return(log(Cabs(h)));      /// not decibel
}

static double attenuate(
 double f )
{
  if (f < 1000) return 0.;
  else if (f < 4000) return -30 * (1 - (cos((f - 1000.) / (4000. - 1000.) * _PI) + 1.) /2.);
  else return -30.;
}

double dBPreac(
  double f,
  double Fs)
{
	fcomplex x = CComplex(1. - cos(2. * _PI * f / en_cours.frequence),sin(2. * _PI * f / en_cours.frequence));
   return(20. * log10(Cabs(x)));
}

void applyAddVoicing(
	int deb,
   int fin,
   double voicingLevel,
   double F0,
   double voicingWidth)
{
  double F1 = 140.;
  double bwF1 = voicingWidth;
  if (!droit_ecrire_message(en_cours.nom)) return;
  long tailleRegion = fin - deb + 1;

  if (deb > fin || !dans(deb, ct.d_e, ct.f_e)
		|| !dans(fin, ct.d_e, ct.f_e)) {
    affiche_erreur(ml(9110,"La rgion  voiser est mal dfinie"));
    return;
  }

  infoEdit.derOp = ADDINGVOICING;
  infoEdit.debut = max(deb - ORDERFFTOLA,  0);
  infoEdit.fin = min(fin + ORDERFFTOLA, (int) (ct.tf - 1));
  strcat(strcpy(infoEdit.cut.nom,tempdir),"cutfile_.wa$");
  infoEdit.cut.nb_bits = NB_BITS;
  infoEdit.cut.depl = 0;
  infoEdit.cut.swap = SWAPINFILE;

  /* On sauve le signal qui va etre coupe */
  copieDansCutFile(&infoEdit);

  // First, compute the spectrum level so as to adjust the voicing level inthe
  // low frequency band.
  int wceps = enEcht(16);  // A 16 ms window */
  int orderFFT = 1024;
  int nbCep = (fin - deb) / (wceps / 2) + 1;
  int step = wceps / 2;
  int nblif0 = enEcht(NB_CEP_STD1_SEC * 1000.);
  int nblif1 = enEcht(NB_CEP_STD2_SEC * 1000.);

  FLOATbuf **spectreCep =(FLOATbuf**)malloc(((long)nbCep)*sizeof(FLOATbuf *) );
  for (int i = 0; i < nbCep; i++)
    spectreCep[i] =(FLOATbuf *) new FLOATbuf(orderFFT / 2  + 1, "SpectreCepAddVoicing");

  int trueNbCep = cepstre(spectreCep, &space[_i(deb)], 0, nbCep, step, wceps, orderFFT, orderFFT, nblif0, nblif1, 1, 0, 0);
  /* Now seach for the maximum of energy above 1000 Hz
     and measure the average level at the amplitude of the pseudo formant used for voicing. */
  double maxEn = -INFINI_D;
  int kmin = 1000. / en_cours.frequence *  orderFFT;
  int F1_i = IROUND(F1 / en_cours.frequence);
  double levelPseudoF1 = 0.;
  for (int i = 0; i < trueNbCep; i++) {
     for (int j = kmin; j < orderFFT / 2; j++)
        maxEn = ((*spectreCep[i])[j] > maxEn) ? (*spectreCep[i])[j] : maxEn;
        levelPseudoF1 += (*spectreCep[i])[F1_i];
  }
  if (trueNbCep) levelPseudoF1 = levelPseudoF1 / trueNbCep;
  else levelPseudoF1 = 0.;

  double F1level;
  double targetdB = 70.;
  F1level = (targetdB - dBPreac(F1, en_cours.frequence)) / 20 * log(10.) - formantAmplitude(F1, bwF1, F1);

  for (int i = 0; i < nbCep; i++) delete spectreCep[i];
  free((char *) spectreCep);

  addVoicingOLA((short *) space.p, (short *) space.p, ct, _i(deb), _i(fin), F0, en_cours.frequence, voicingLevel, F1, bwF1, F1level, -15., false);

  // bout avant la zone amortie : conserv en place
  ajtBoutSpectro(ct.d_a, infoEdit.debut - 1, ct.d_a, infoEdit.debut - 1);
  // bout aprs la zone amortie : conserv en place
  ajtBoutSpectro(infoEdit.debut + tailleRegion + 1, ct.f_a,
		 infoEdit.debut + tailleRegion + 1, ct.f_a);
  // bout amorti : recalcul
  ajtBoutSpectro(-1, -1, infoEdit.debut, infoEdit.debut + tailleRegion);

  wsignal->aff();
  wsignal->paint();
  spectroRapide(wspectro->wsona);

  infoEdit.yaDesModifsSignal = VRAI;
  ct.debRegion = INFINI_L;
  ct.finRegion = -INFINI_L;
}

extern unsigned int getnewf0(long echt);

void addVoicingOLA(
  short *speechin,  // Both signas can be identical
  short *speechout,
  struct ctrl_temps ctloc,
  int debutEcht,
  int finEcht,
  double forcedF0,  // F0 frequency in case it is imposed
  double Fs,  // sampling frequency
  double revoicingLevel,  // this is the target level in dB in the spectral domain
  double F1, // frequency of the pseudoF1
  double bwF1,    // bandwidth of the pseudo formant used for adding voicing
  double F1level,    // energy adjustment of the pseudoFormant. ,
  double higherFrequencyAttenuation,  // in dB
  bool addFormant)   // true is the artificial formant is added
{
  long dfen = debutEcht,
       ffen = finEcht,
       deb = max(dfen - L/2,  0),
  	    fin = min(ffen + L/2, ct.f_e - 1);
  long tailleRegion = fin - deb + 1;
  int im, im0, jm, lm, i, j, ipp, lim, ljm, I;
  float gain[ORDERFFTOLA];
  bool firstTime = true;

  // initialisations du filtrage
  h.newsize(L + 1);
  inithan(&h, L);
  modif = (double *) calloc(ffen - dfen + L, sizeof(double));
  for (int i = 0; i < ffen - dfen + L; i++) modif[i] = 0.;
  for (int i = 0; i < 3 * L / 4; i++) queues[1][i]= queues[0][i] = 0.;

  // multiplication de l'amplitude avec la fenetre de hamming
  // et padding de zero
  for (im0 = im = max(0, dfen - L / 2), jm = 0, I = 0; im <= ffen - L;
  		 im += L/4, jm += L/4, I += 1){
    for (i = 0; i < L; i++) {
      reelle[i] = speechin[im + i] * h[i];
      imag[i]=0.;
      if (im + i < dfen + L / 4)
        queues[0][jm + i] += h[i];
      if (im + i >= ffen - L / 4)
        queues[1][im + i - ffen + L / 4] += h[i];
    }
    for(i = L; i < N; i++){
      reelle[i] = imag[i] = 0.;
    }

    // Transformee de Fourier
    fft(N, reelle, imag);
    for(i = 0; i < N; i++){
      module[i] = 0.5 * log(SQR(reelle[i]) + SQR(imag[i]) + EPSILON);
      phase[i] = atan2(imag[i], reelle[i]);
      if (firstTime) phase0[i] = phase[i];
    }

    // use a pseudo formant for the voicing                    
    for (int i = 0; i < N; i++)
  	   voicingModule[i] =  F1level + formantAmplitude(F1, bwF1, i * en_cours.frequence / N);

    for(i = 0; i < N; i++){
      reelle[i] =  (addFormant) ? voicingModule[i] : module[i];
      imag[i] = 0.;
    }

    // Inverse de la transformee de Fourier   --> cepstre
    ifft(N, reelle, imag);
    // --> becomes real since it the spectrum is real and symmetric

    double voicingLevel;
    for(i = 0; i < N; i++) reelle[i] /=  N;
    double F0 = (forcedF0) ? forcedF0 : getnewf0(ctloc.d_e + im + L / 2);
    if (fabs(F0) < 10.) {
    	F0 = 100.;
      voicingLevel = 0.;
    }
    else voicingLevel = revoicingLevel;
    
    // double F0 = F0onset + (F0offset - F0onset) / ((double) (ffen - dfen + 1)) * (im - dfen);
    int n0 = (int) (Fs / F0);
    if (n0 < N / 2 && fabs(voicingLevel) > 0.001) {
      double moduleF0 = voicingLevel; // set to be adjusted correctly
      reelle[n0] = voicingLevel;
      reelle[2 * n0] = voicingLevel / 2.;
      reelle[N - n0] = voicingLevel;
      reelle[N - 2 * n0] = voicingLevel / 2.;
    }

    for(i = 0; i < N; i++) // force the imaginary part to zero. Should use realfft instead...
    	imag[i] = 0.;

    // Fourier transform to get a spectrum
    fft(N, reelle, imag);
    if (addFormant) for (int i = 0; i <= N/2; i++) {
      reelle[i] += attenuate(i * Fs / N);
      if (i && i != N/2) reelle[N - i] = reelle[i];
    }

    double correction = higherFrequencyAttenuation * log(10.) / 20.;
    for(i = 0; i < N; i++){    // mixing the voiced low frequency spectrum and the original spectrum.
      if (!addFormant)
        module[i] = exp(reelle[i] + correction);
      else {
        if (reelle[i] > module[i] + correction)
         module[i] = exp(reelle[i]);
        else module[i] = exp(module[i] + correction);
      }

      //module[i] =  exp(reelle[i]);
      //module[i] =  exp(reelle[i]);
      reelle[i] = module[i] * cos(phase[i]);
      imag[i] = module[i] * sin(phase[i]);
    }

    // Inverse de la transformee de Fourier
    ifft(N, reelle, imag);

    ljm = jm;
    lim = im;
    for (i = 0; i < L; i++) {
      modif[lm = jm + i] += reelle[i] / (N * W0);
    }

  }
  for (int i = 0; i < 3 * L / 4; i++)
    modif[i] = (modif[i] * W0 + speechin[im0 + i] * (W0 - queues[0][i])) / W0;
  for (int i = ljm + L / 4, j = 0; i <= lm; i++, j++)
    modif[i] = (modif[i] * W0 + speechin[lim + L / 4 + j] * (W0 - queues[1][j])) / W0;

  for (im = max(0, dfen - L / 2), jm = 0; jm < lm; im++, jm++)
    speechout[im] = (SAMPLE) modif[jm];

  free((char *) modif);
}

int getWindowSizeOLAaddVoicing()
{
	return(L);
}

