/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// formants.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include "signal.h"
#include "pics.h"
#include "outils.h"


#define D_DB_PIC	20	/* on interdit de prendre des
                                   pics dont l'energie
				   est inferieure de D_DB_PIC
				   a celle du max de la fft */
#define DB_BAND		6	/* difference d'energie (dB)
				   pour le calcul de la largeur
				   de bande */
#define HAUT_PIC_MIN	5	/* hauteur relative minmale d'un pic */
#define SIL_SPEC	26.	/* seuil au dessous duquel un
                                   pic n'est plus considere */
#define FORMIN		10	/* le 3canal d'un formant doit
                                   au moins etre egal a 200 Hz */

/*
 * Cette fonction calcule la largeur de bande d'un pic en canal(aux).
 */
static float largBande(
	FLOATbuf& y_,	// spectre etudie 
	int ip,	       	// indice du max dans la courbe dont 
                        //     on cherche la largeur de bande 
	int debut, 	// indices delimitant la partie du 
	int fin,        //     spectre a etudier 
	float xm, 	// interpolation du maximum :
	float ym,	//	   xm = 0 pour ip - 1. 
      	float ddb,	// amplitude en DB pour le calcul de la
                        //     largeur de bande 
	float *xgauche, // limites de 
	float *xdroit)  //     la bande
{
  int i, ii;
  float xg, xd,		/* limites de definition de la largeur de
			   bande a gauche et a droite */
  	xxm;
  float *y=(float *)&y_[0];

  /* largeur a gauche */
  if (xm > 1.) {  /* le maximum reel est atteint pour x > ip */
    xxm = xm - 1.;
    ii = ip;
  }
  else {
    ii = max(ip - 1, debut);
    xxm = xm;
  }

  for (i = ii; i > debut && y[i] > y[i-1] && y[i] > ym - ddb; i--);

  if (i == debut || y[i] <= y[i-1]) xg = (float)i;
  else if (i == ii) 
    xg = (float) ii + xxm - ddb / (ym - y[ii]) * (1. - xxm);
  else {
    float rddb = y[i+1] - (ym - ddb);
    xg = (float) i + 1. - rddb / (y[i+1] - y[i]);
  }

  /* largeur a droite */
  if (xm > 1.) { /* le maximum reel est atteint pour x > ip */
    ii = min(ip + 1, fin);
    xxm = xm - 1.;
  }
  else {
    ii = ip;
    xxm = xm;
  }
    
  for (i = ii; i < fin && y[i] > y[i+1] && y[i] > ym - ddb; i++);

  if (i == fin || y[i] <= y[i+1]) xd = (float)i;
  else if (i == ii) 
    xd = (float) ii -1. + xxm + ddb / (ym - y[ii]) * (1. - xxm);
  else {
    float rddb = y[i-1] - (ym - ddb);
    xd = (float) i - 1. + rddb / (y[i-1] - y[i]);
  }

  if (xgauche) *xgauche = xg;
  if (xdroit) *xdroit = xd;

  return((xd - xg) * en_cours.frequence / 2. / NB_CANAUX);
}



/*
 * Cette fonction permet de preciser la valeur d'un extremum
 * a partir de trois points (interpolation parabolique).
 * Les abscisses sont separees d'1. Le resultat est la position
 * de l'extremum; les abscisses sont 0, 1 et 2 et le resultat
 * est dans cet intervalle. La fonction retourne l'abscisse du
 * maximum.
 */

static float interbole(
	float y0, float y1, float y2,
	float *xm, float *ym) /* le point maximum de la
	            parabole d'interpolation */
{
  if (fabs(y0-2*y1+y2) < EPSILON) { /* les trois points sont alignes */
    *ym = max( y0, y2);
    *xm = (y0 < y2) ? 2. : 0.;
    return(*xm);
  }
  else { /* cas general */
    *ym = -0.125 * (3*y0 -4*y1 +y2) * (3*y0 -4*y1 +y2)
           / (y0 -2*y1 + y2) + y0;
    *xm = (3*y0 -4*y1 +y2) / (2*y0 -4*y1 +2*y2);
    return(*xm);
  }
}



/*
 * Cette fonction extrait les extrema d'une courbe.
 */

int ex_extr(
	FLOATbuf& x_,		// courbe a traiter
	Pics1_Cepstre& ex,       // courbe representee par
	                        //      ses extrema	
	int debut,		// debut et
	int fin)                //    fin de la recherche 
{
  int i,j;	/* indices courbe et extremum */
  float *x=(float *)&x_[0];
  float e = (x_.size() - 1 == 256) ? .5 : 1.;

  ex.resize(NB_MAX_EXTRM); 
  i = debut;
  ex[0].xi = 3 * debut;
  ex[0].frHz = can3ToHz(3. * debut) * e;
  ex[0].v = x[debut];
  ex[0].q = 0.;
  ex[0].p=ex[0].v;
  if (x[debut] > x[debut+1]) 
    ex[0].bw = largBande(x_, debut, debut, fin, 1.
		      , x[debut], 6., (float *) NULL, (float *) NULL);
  else ex[0].bw=100;
  j = 1;

  while (i < fin - 1) {
    if (j>=NB_MAX_EXTRM) return false;
    if (x[i] < x[i+1]) {	/* cas d'un maximum */
      float xm, ym;
      for (;x[i]<=x[i+1] && i < fin - 1;i++);
      ex[j].xi = 3 * (i-1)
		    + (int) (  interbole(x[i-1], x[i], x[i+1], &xm, &ym)
		               / 0.33 + 0.5);
      ex[j].v = x[i];
      ex[j].frHz = can3ToHz((float)ex[j].xi) * e;
      ex[j].q = 0.;
      /* calcul de la largeur de bande */
      ex[j].bw = largBande(x_, i, debut, fin, xm, ym
			, 6., (float *) NULL, (float *) NULL);
      j++;
    }
    else {	/* cas d'un minimum */
      float muet;
      for (;x[i]>=x[i+1] && i < fin - 1;i++);
      ex[j].xi = 3*(i-1)
		    + (int)( interbole(x[i-1], x[i], x[i+1], &muet, &muet)
		             /0.33+0.5);
      ex[j].frHz = can3ToHz((float)ex[j].xi) * e;
      ex[j].v = x[i];
      ex[j].q = 0.;
      j++;
    }
  }

  if (j>2) { ex.sens = (ex[1].v > ex[0].v) ? 1 : 0;
    ex.resize(j-1);
  }
  else {
    ex.sens=1;
    ex.resize(0);
  }

  /* calcul de l'indice de perception et de la largeur
     de bande des PICS */
  for (i = ex.sens; i < ex.size(); //effectif;
       i += 2)
  {
    float vg = (!i) ? 0. : ex[i].v - ex[i-1].v /* vallee gauche */
	 ,vd = (i < ex.size()-1) //effectif - 1)
	       ? ex[i].v - ex[i+1].v : 0.;  /* vallee droite */
    /* calcul de la perceptivite : somme de l'energie
       et de la demi-vallee */
    ex[i].p = ex[i].v + (vg + vd) / 2.;
  }
  return true;
}

/*
 * Cette fonction calcule la qualite d'un pic en prenant
 * la plus difference aux minima qui encadrent ce pic.
 */

static float qualitePic(
	Pics1_Cepstre& resume,
	int i,
	float , //min_cep,
	float max_cep)
{ float x_db,
        xg_db,
        domination;
  int ig, id;

  x_db = resume[i].v;
  xg_db = (i + 2 < resume.size()) 
	  ? ( resume[i].v
	      - resume[i + 2].v )
	  : 0.;

  /* On recherche le domaine frequenciel que le pic domine */
  /* a gauche */
  for (ig = i - 2;
       ig >= 0 && resume[ig].v < resume[i].v;
       ig -= 2);
  ig = (ig < 0) ? 0 : ig + 1;
  /* a droite */
  for (id = i + 2;
       (id < resume.size()) 
       && (resume[id].v < resume[i].v);
       id += 2);
  id = (id >= resume.size()) 
       ? resume.size() 
         - 1 : id - 1;
  domination = resume[id].frHz
               - resume[ig].frHz;

  return((  (x_db > SILENCE_SPECTRO)
	  &&(  (x_db > (max_cep - D_DB_PIC) )
	     ||(xg_db > D_DB_PIC / 2.)
	     ||(domination > 1100.) )) ? 1.0 :0.0);
}



/* 
 * Cette fonction estime la qualite des pics d'une courbe de
 * lissage cepstral.
 */

void estime(
	Pics1_Cepstre& resume, // extrema de la courbe
	float min_cep,	        // valeurs extremes de la 
	float max_cep)          //     fft et du cepstre 
{
	int i;

	for (i = resume.sens;
	     i < resume.size(); 
	     i += 2) 
	  resume[i].q = qualitePic(resume, i, min_cep, max_cep);
}

static int compar(const void *xx, const void *yy)
{
  struct pic_cepstre *x,*y;
  struct pic_cepstre **x0,**y0;

  x0=(struct pic_cepstre **)xx;
  y0=(struct pic_cepstre **)yy;
  x=*x0;
  y=*y0;
  return ((y->q > x->q) ? 1 : (y->q == x->q) ? 0 : -1);
}

static int comparFreq(const void *xx, const void *yy)
{
  struct pic_cepstre *x,*y;
  struct pic_cepstre **x0,**y0;

  x0=(struct pic_cepstre **)xx;
  y0=(struct pic_cepstre **)yy;
  x=*x0;
  y=*y0;
  return ((y->q > 0.5 && y->frHz < x->frHz) ? 1 : (y->frHz == x->frHz) ? 0 : -1);
}


/*
 * Cette fonction calcule les n meilleurs pics de la courbe
 * inferieurs a la demi-frequence d'echantillonnage
 * et les range contigument dans resume.
 * Cette fonction retourne de plus la perceptivite maximale 
 * explicable avec les formants.
 */

float les_meilleurs(
	Pics1_Cepstre& resume, /* les extrema	*/
	int n,			/* les n meilleurs maxima */
	int nmax,		/* valeur maximale de l'indice du pic */
   TypeMeilleurPic critere)
{
	int dernier;	/* dernier pic a etudier */
	int i,j;
	float pmax;

	/* on cherche d'abord le dernier pic inferieur a nmax */
	for (dernier = resume.sens;
	     dernier < resume.size();
	     dernier += 2)
	  if (resume[dernier].xi > 3 * nmax) break;

	if (dernier >= resume.size() ) dernier -= 2;
	else if ( resume[dernier].xi> 3 * nmax ) dernier -= 2;

	/* on met en tete de resume les pics interessants */
	for (i = 0, j = resume.sens; j <= dernier; i++, j += 2)
	  resume[i] = resume[j];

	resume.resize(i);

	/* on trie les pics */
	qsort((char *)resume.p, resume.size(), sizeof(struct pic_cepstre *),
         (critere == QualitePicCepstre) ? compar : comparFreq);

	resume.resize(min(n, resume.size()));
	for (i = 0, pmax = 0.;
	     i < resume.size();
	     i++)
	  if (resume[i].xi < LIMFOR3)
	    pmax +=  resume[i].p;

	return(pmax);
}

