/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier format_f.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <dir.h>

#include "format_f.h"
#include "files.h"


struct format *Fmt_par; /* le tableau des formats de lecture 			*/

// ci-dessous doit tre en phase avec les
//   define correspondants
struct format_phon Fmt_phon[]
    = { {"NONE"}, {"CRIN"}, {"NIST"}, {"SAM"}, {"ALIGN"}, {""} };
struct format_text Fmt_text[]
    = { {"NONE"}, {"VECSYS"}, {"NIST"}, {"SAM"}, {"NOISEX"}, {""} };

class alloc_fmt
{ public:
    alloc_fmt();
    ~alloc_fmt();
    void free_champ(champ *Pchamp);
};
alloc_fmt::alloc_fmt()
{
  Fmt_par=(struct format *) malloc(sizeof(struct format));
  strcpy(Fmt_par[0].nfmt,"");
}
alloc_fmt::~alloc_fmt()
{ int i;
  for (i=0;Fmt_par[i].nfmt[0]!=0;i++)
  { free_champ(Fmt_par[i].fmt->frequence);
    free_champ(Fmt_par[i].fmt->nb_bits);
    free_champ(Fmt_par[i].fmt->debut);
    free_champ(Fmt_par[i].fmt->longueur);
    free_champ(Fmt_par[i].fmt->signal_swap);
  }
  free(Fmt_par);
}
void alloc_fmt::free_champ(champ *Pchamp)
{
  switch (Pchamp->type)
  { case ' ' : break;
    case 'n' : free(Pchamp->x.val->pile);
	       free(Pchamp->x.val);
               break;
    case 'c' : free(Pchamp->x.libel);
	       break;
  }
  free(Pchamp);
}
alloc_fmt alloc_fmt_; // seulement pour allocation/dsallocation

/*
 * Cette fonction permet d'initialiser les formats des fichiers
 * de parole que connait Snoori.
 */

void init_fmt()
{
  static ind=0;
  ffblk ffblk;
  int done;
  int i_fmt=0;		/* nombre de formats de fichiers de
                           parole possibles 	*/
  STRING ch;
  char ch1[_MAX_FNAME];

  if (ind!=0) return;
  ind =1;

  strcpy(ch,formatdir); strcat(ch,"*.fmt");
  done=findfirst(ch,&ffblk,0);
  while (!done) {
    _splitpath(ffblk.ff_name,NULL,NULL,ch1,NULL);
	 i_fmt++;
	 Fmt_par=(struct format *)realloc(Fmt_par,(i_fmt+1)*sizeof(struct format));
    //
	 { FILE *ficf;
	
		strcpy(ch,formatdir);
      strcat(ch,ffblk.ff_name);	    
		ficf = fopen(ch,"rt");
		if (debug==5)
		{ char ch1[80];
		  sprintf(ch1,"init_fmt() : fichier %s",ch);
		  Wmsg->wr(ch1);
		}
      Fmt_par[i_fmt-1].fmt = lire_entete(ficf);
      fclose(ficf);
		_splitpath(ffblk.ff_name,NULL,NULL
		  ,Fmt_par[i_fmt-1].nfmt
		  ,NULL);
	 }
	 //
    done=findnext(&ffblk);
  }
  strcpy(Fmt_par[i_fmt].nfmt,"");
}

////////////////////////////////////
// fentre Wselformat
//
// variables de la fentre
//     num_format
//

class TWselformat : public TDialog
{   infoFile *Pfile;
    TComboBox *PTfmt_son;
    TComboBox *PTfmt_phon;
    TComboBox *PTfmt_text;
    TStatic *PTfile_son;
    TStatic *PTfile_phon;
    TStatic *PTfile_text;
  public:
    TWselformat(PTWindowsObject AParent
	       ,infoFile *Pfile
	       , LPSTR name);
    virtual void SetupWindow();
    virtual bool CanClose();
    void SetTexts();

	 // rponses aux vnements
	 /*
	 virtual void IDselectSoundFile(RTMessage)
                     = [ID_FIRST+103];
	 virtual void IDselectPhonFile(RTMessage)
                     = [ID_FIRST+113];
	 virtual void IDselectTextFile(RTMessage)
							= [ID_FIRST+123];
	 */
	 void IDselectSoundFile(WPARAM code);
	 void IDselectPhonFile(WPARAM code);
	 void IDselectTextFile(WPARAM code);
	 DECLARE_RESPONSE_TABLE(TWselformat);
};
DEFINE_RESPONSE_TABLE1(TWselformat,TDialog)
  EV_CHILD_NOTIFY_ALL_CODES(103,IDselectSoundFile),
  EV_CHILD_NOTIFY_ALL_CODES(113,IDselectPhonFile),
  EV_CHILD_NOTIFY_ALL_CODES(123,IDselectTextFile),
END_RESPONSE_TABLE;

TWselformat::TWselformat(PTWindowsObject AParent
                      ,infoFile *Pfile, LPSTR name)
	:TDialog(AParent,name)
{ // initialiser le contenu des contrles
  //   avec les variables correspondantes
  this->Pfile=Pfile;
  PTfmt_son=new TComboBox(this,101,0);
  PTfmt_phon=new TComboBox(this,111,0);
  PTfmt_text=new TComboBox(this,121,0);
  PTfile_son=new TStatic(this,102,MAX_LIGNE);
  PTfile_phon=new TStatic(this,112,MAX_LIGNE);
  PTfile_text=new TStatic(this,122,MAX_LIGNE);
}
void TWselformat::SetupWindow()
{ int i;

  TDialog::SetupWindow();
  //
  for (i=0;strlen(Fmt_par[i].nfmt)!=0;i++)
    PTfmt_son->AddString(Fmt_par[i].nfmt);
  PTfmt_son->SetSelIndex(Pfile->fmt_son);
  //
  for (i=0;strlen(Fmt_phon[i].nom)!=0;i++)
    PTfmt_phon->AddString(Fmt_phon[i].nom);
  PTfmt_phon->SetSelIndex(Pfile->fmt_phon);
  //
  for (i=0;strlen(Fmt_text[i].nom)!=0;i++)
    PTfmt_text->AddString(Fmt_text[i].nom);
  PTfmt_text->SetSelIndex(Pfile->fmt_word);
  //
  SetCaption(ml(4402,"Slecteur de format"));
  ::SetWindowText(GetItemHandle(IDCANCEL),ml(4403,"&Annuler"));
  ::SetWindowText(GetItemHandle(103),ml(4409,"Choisir un fichier"));
  ::SetWindowText(GetItemHandle(113),ml(4409,"Choisir un fichier"));
  ::SetWindowText(GetItemHandle(123),ml(4409,"Choisir un fichier"));
  ::SetWindowText(GetItemHandle(104),ml(4405,"fichier son"));
  ::SetWindowText(GetItemHandle(114),ml(4406,"fichier phonmes"));
  ::SetWindowText(GetItemHandle(124),ml(4407,"fichier mots"));
  ::SetWindowText(GetItemHandle(105),ml(4408,"Format:"));
  ::SetWindowText(GetItemHandle(115),ml(4408,"Format:"));
  ::SetWindowText(GetItemHandle(125),ml(4408,"Format:"));

  SetTexts();
}

bool TWselformat::CanClose()
{ BOOL Valid;
  // mettre  jour les variables en extrayant
  //   leur valeur des contrles,
  //   et aprs avoir ventuellement vrifi
  //   leur validit (variable Valid)
  Pfile->fmt_son=PTfmt_son->GetSelIndex();
  Pfile->fmt_phon=PTfmt_phon->GetSelIndex();
  Pfile->fmt_word=PTfmt_text->GetSelIndex();
  Valid=TRUE;
  return Valid;
}
void TWselformat::SetTexts()
{
  STRING ch;
  strcpy(ch,ml(4410,"nom fichier: "));
  strcat(ch,Pfile->nom);
  strlwr(ch);
  PTfile_son->SetText(ch);
  strcpy(ch,ml(4410,"nom fichier: "));
  strcat(ch,Pfile->nom_man);
  strlwr(ch);
  PTfile_phon->SetText(ch);
  strcpy(ch,ml(4410,"nom fichier: "));
  strcat(ch,Pfile->nomAnnot);
  strlwr(ch);
  PTfile_text->SetText(ch);
}
void TWselformat::IDselectSoundFile(WPARAM)
{ STRING ch;

  x_makepath(ch,Pfile->nom,Pfile->nom,Pfile->nom,"*.*");
  if (SelectFile(this,ch)==IDOK) //:: Pfile->nom);
    strcpy(Pfile->nom,ch);
  SetTexts();
}
void TWselformat::IDselectPhonFile(WPARAM)
{ STRING ch;

  x_makepath(ch,Pfile->nom_man,Pfile->nom_man,
  				(Pfile->nom_man && strlen(Pfile->nom_man)) ? Pfile->nom_man : "*",".*");
  if (SelectFile(this,ch)==IDOK) //:: Pfile->nom_man);
    strcpy(Pfile->nom_man,ch);
  SetTexts();
}
void TWselformat::IDselectTextFile(WPARAM)
{ STRING ch;

  x_makepath(ch,Pfile->nomAnnot,Pfile->nomAnnot,Pfile->nomAnnot,"*.*");
  if (SelectFile(this,ch)==IDOK) //:: Pfile->nomAnnot);
    strcpy(Pfile->nomAnnot,ch);
  SetTexts();
}

int get_format(PTWindowsObject AParent,infoFile& Pfile) //int& f)
{
  int id;
  init_fmt();

  // MessageBox(NULL,"ooo","oooo",MB_OK);

  return (TWselformat(AParent,&Pfile,"Wselformat").Execute());
  /* if ((id=AParent->GetApplication(
              )->ExecDialog(new TWselformat(AParent
			      ,&Pfile
			      ,"Wselformat"))
      )!=IDOK) return id;    */

  return id;
}


////////////////////////////////////
// fentre WselSpeechFormat
//
// variables de la fentre
//     num_format
//

class TWselSpeechFormat : public TDialog
{   infoFile *Pfile;
    TComboBox *PTfmt_son;
    TStatic *PTfile_son;
  public:
    TWselSpeechFormat(PTWindowsObject AParent
	       ,infoFile *Pfile
	       , LPSTR name);
    virtual void SetupWindow();
    virtual bool CanClose();
    void SetTexts();

	 // rponses aux vnements

	 void IDselectSoundFile(WPARAM code);
	 DECLARE_RESPONSE_TABLE(TWselSpeechFormat);
};
DEFINE_RESPONSE_TABLE1(TWselSpeechFormat,TDialog)
  EV_CHILD_NOTIFY_ALL_CODES(103,IDselectSoundFile),
END_RESPONSE_TABLE;

TWselSpeechFormat::TWselSpeechFormat(PTWindowsObject AParent
                      ,infoFile *Pfile, LPSTR name)
	:TDialog(AParent,name)
{ // initialiser le contenu des contrles
  //   avec les variables correspondantes
  this->Pfile=Pfile;
  PTfmt_son=new TComboBox(this,101,0);
  PTfile_son=new TStatic(this,102,MAX_LIGNE);
}
void TWselSpeechFormat::SetupWindow()
{ int i;

  TDialog::SetupWindow();
  //
  for (i=0;strlen(Fmt_par[i].nfmt)!=0;i++)
    PTfmt_son->AddString(Fmt_par[i].nfmt);
  PTfmt_son->SetSelIndex(Pfile->fmt_son);
  SetCaption(ml(4402,"Slecteur de format"));
  ::SetWindowText(GetItemHandle(IDCANCEL),ml(4403,"&Annuler"));
  ::SetWindowText(GetItemHandle(103),ml(4409,"Choisir un fichier"));
  ::SetWindowText(GetItemHandle(104),ml(4405,"fichier son"));
  ::SetWindowText(GetItemHandle(105),ml(4408,"Format:"));

  SetTexts();
}

bool TWselSpeechFormat::CanClose()
{
  bool Valid;
  // mettre  jour les variables en extrayant
  //   leur valeur des contrles,
  //   et aprs avoir ventuellement vrifi
  //   leur validit (variable Valid)
  Pfile->fmt_son=PTfmt_son->GetSelIndex();
  Valid = true;
  return Valid;
}
void TWselSpeechFormat::SetTexts()
{
  STRING ch;
  strcpy(ch,ml(4410,"nom fichier: "));
  strlwr(Pfile->nom);
  strcat(ch,Pfile->nom);
  PTfile_son->SetText(ch);
}
void TWselSpeechFormat::IDselectSoundFile(WPARAM)
{ STRING ch;

  x_makepath(ch,Pfile->nom,Pfile->nom,Pfile->nom,"*.*");
  if (SelectFile(this,ch)==IDOK) //:: Pfile->nom);
    strcpy(Pfile->nom,ch);
  SetTexts();
}

int getSpeechFormat(PTWindowsObject AParent,infoFile& Pfile) //int& f)
{
  int id;
  init_fmt();

  return (TWselSpeechFormat(AParent,&Pfile,"WselSpeechFormat").Execute());

  return id;
}
