/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include "common.h"
#pragma hdrstop


#include "signal.h"
#include "outils.h"     // pour la fonction enMilli et enEcht(temps.cpp)
#include "fftlpc.h"
#include "fft_mayer.h"
#include "phase_unwrapping.h"
#include <math.h>


void affichageSigSimple(
	  SAMPLE *sig,	 /* marques sur le signal temporel       */
	  int n)	       /* echantillon de debut d'affichage 	    */
{
  float vmax = 0;;

  Wact->wr(ml(2101,"draw sound"));
  wsignal->wsig->clear_fenetre();
  wsignal->wsig->setLXcoor(ct.d_a, ct.f_a);
  MgiHue(BLEU, wsignal->wsig);

  // donner les ordonnes  la fenetre signal
  for (int i = 0; i < n; i++) vmax = (abs(sig[i] > vmax)) ? abs(sig[i]) : vmax;
  wsignal->wsig->setLYcoor(-vmax, vmax);

  for (int i = 1; i < n; i++)
    wsignal->wsig->Dline2(wsignal->wsig->LXtoDX(i - 1 + ct.d_a), wsignal->wsig->LYtoDY(sig[i - 1]),
    							  wsignal->wsig->LXtoDX(i + ct.d_a), wsignal->wsig->LYtoDY(sig[i]));

  Wact->wr("");
  InvalidateRect(wsignal->wsig->HWindow,NULL,TRUE);
}

void unwrappingPhase(
  SAMPLE parole[],
  double phase[],
  int fenSignal,
  int ordreFFT)
{
   double *real = new double[fenSignal],
          *imag = new double[fenSignal];

/*  FILE *ff = fopen("G:\\Users\\laprie\\TestGuiKlatt\\Src\\essai\\signalDeuxPolesSautPi","r");
  for(int j = 0; j < fftOrder; j++) { fscanf(ff,"%lf", &real[j]);}
  for(int j = 0; j < fftOrder; j++) { imag[j] = 0.;}
  fclose(ff);
*/

  initham_(NULL, fenSignal);
  ham_sd_(parole, 0, real, NULL, fenSignal);
  for(int j = 0; j < fenSignal; j++) {
    imag[j] = 0.;
  }

  RealPhaseUnwrapping xx(real, fenSignal, ordreFFT);
  xx.get(phase);


/*   FILE *gg = fopen("G:\\Users\\laprie\\TestGuiKlatt\\Src\\essai\\phaseDepliee","w");
  for(int j = 0; j <= fenSignal / 2 ; j++) fprintf(gg,"%lf ", phase[j]);
  fclose(gg); */
  delete [] real;
  delete [] imag;
}

// Un filtre  3 poles
double coefAR[] = {1.00000000000000, -8.812178406628336E-002, 1.40520665823259, -0.364933515310609,
					 0.929841308800229, -0.527577153732348, 0.240149002500000};


void LPCresidu(
  SAMPLE *speech,
  SAMPLE *residu,
  int n,
  int windowLength,
  int offset,
  int LPCorder)
{
  float *h = new float[windowLength],
        *w = new float[windowLength],
        *r = new float[LPCorder + 1],
	     *a = new float[LPCorder + 1],
        *sf = new float[LPCorder + 1],
        *sumHanning = new float[n],
        gain;

  // Initialization of the Hanning window
  for (int i = 0; i < windowLength; i++)
    h[i] = 0.5 * (1. - cos((double)(2. * _PI * i / windowLength))) ;

  // initialization of the residu
  for (int i = 0; i < n; i++) {
    residu[i] = 0;
    sumHanning[i] = 0.;
  }

  for (int i = 0; i < n - windowLength; i += offset) {
    for (int j = 0; j < windowLength; j++) {
      w[j] = speech[i + j] * h[j];
    }

    /* The signal should be filtered at 5kHz and the LPCorder set to 10. Too high a prediction order
       gives rise to "too small" a residue which does not correspond to an approximation of the source; */
    autolpc (w, r, a, sf, &gain, windowLength, LPCorder);

    /* {FILE *f=fopen("coefFiltres","w");
    for (int i = 1; i <= LPCorder; i++) fprintf(f,"%.9f ", a[i]);
    fclose(f);
    }   */
    
    for (int j = LPCorder; j < windowLength; j++) {
      // Subtraction of the prediction
      sumHanning[i + j] += h[j];
      residu[i + j] += w[j];
      for (int k = 1; k <= LPCorder; k++)
        residu[i + j] -= a[k] * w[j - k];
/*      for (int k = 1; k <= 6; k++)             A test for a signal with a known number of poles
        residu[i + j] += coefAR[k] * w[j - k];   */
    }
  }

  // Finally, divide by the sum of Hanning window.  Not necessary if offset is well chosen (1/4 of the window)
  // for (int i = 0; i < n; i++) residu[i] = (fabs(sumHanning[i] > 1e-06)) ? residu[i] / sumHanning[i] : 0.;
  for (int i = 0; i < n; i++) residu[i] /= 2;

  delete [] h;  delete [] w; delete [] r; delete[] a; delete [] sf; delete[] sumHanning;
}

void Wtop::CMGlottalClosureInstants()
{
   affiche_erreur("Not yet implemented");
   return;
/*   SAMPLE *residu = new SAMPLE[ct.f_a - ct.d_a + 1];
	wsignal->aff();
   LPCresidu(&(space[_i(ct.d_a)]), residu, ct.f_a - ct.d_a + 1, 400, 100, 12);
   affichageSigSimple(residu, ct.f_a - ct.d_a + 1);
   delete [] residu;     */
}


