/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier gerefilt.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <string.h>

#include "snoori.h"
#include "gerefilt.h"
#include "filtre.h"
#include "outils.h"
#include "signal.h"

bool yaUnFiltrage=FALSE;
static Position x0Domaine, y0Domaine, // 1er point, fixe
                x1Domaine, y1Domaine; // 2me point, mobile
static bool isDomaine=FALSE; // TRUE s'il y a un domaine trac
static long debut, fin;	 // echantillons de debut et de
                         //         fin de filtrage 
profiltre *filt;
static bool typeFiltre = false; // false == stop, !=0 pass
static int ordre = 20;
static float attenuation = 15.;

static void affDomaine(
      d_planche *fenetre
     ,int  x0, int y0   // un sommet du rectangle
     , int x1, int y1)  // le sommet oppos
{ int xi,yi   // coordonnes les plus petites
     ,xs,ys;  // coordonnes les plus grandes

  xi=min(x0,x1);
  yi=min(y0,y1);
  xs=max(x0,x1);
  ys=max(y0,y1);

  InvertArea(fenetre->HWindow   // left
  		 ,xi, yi 
		 ,1
		 ,ys-yi + 1);
  if (xi!=xs)
    InvertArea(fenetre->HWindow // right
  		 ,xs, yi 
		 ,1
		 ,ys-yi + 1);
  if (xi<=xs-2)
  { InvertArea(fenetre->HWindow // top
  		 ,xi+1, yi 
		 ,xs-xi-1
		 ,1);
    InvertArea(fenetre->HWindow // bottom
  		 ,xi+1, ys 
		 ,xs-xi-1
		 ,1);
  }
}


static void lanceFiltre()
{ float f1, f2;
  profiltre filt;

  filt.newsize(ordre); 
  filt.att = attenuation;
  debut = info_souris(wspectro->wsona, x0Domaine, y0Domaine, &f1);
  fin = info_souris(wspectro->wsona, x1Domaine, y1Domaine, &f2);
  
  if (f1 > f2) {
    float t = f1;
    f1 = f2;
    f2 = t;
  }
  f1 = max(0., f1);
  f2 = min(en_cours.frequence / 2., f2);

  if (debut > fin) {
    long t = debut;
    debut = fin;
    fin = t;
  }

  if (f2 > (en_cours.frequence/2.) * 0.9) {
    strcpy(filt.type, (!typeFiltre) ? "lp" : "hp");
    filt.fc = f1 / en_cours.frequence;
  }
  else if (f1 < EPSILON) {
    strcpy(filt.type, (!typeFiltre) ? "hp" : "lp");
    filt.fc = f2 / en_cours.frequence;
  }
  else  {
    filt.fl = f1 / en_cours.frequence;
    filt.fh = f2 / en_cours.frequence;
    strcpy(filt.type, (!typeFiltre) ? "bs" : "bp");
  }

  desfir(&filt);
  filtrage(&filt, debut, fin);
}

#ifdef VERSIONDANSMENU
/////////////////////////////////////////////
// fentre Wfiltrage
//
// variables de la fentre
//     typeFiltre
//     ordre
//     attenuation
//
static choix choix_type[] = { {0," stop band"}
                             ,{1," pass band"}
			  ,{0,""} };
class TWfiltrage : public TFrameWindow
{ public:
    TWfiltrage(PTWindowsObject AParent);
    virtual void SetupWindow();
    virtual bool CanClose();
	 // rponses aux vnements
	 /*
	 virtual void CMfiltrer(RTMessage)=[ID_FIRST+112];
	 */
	 void CMfiltrer(WPARAM code);
	 DECLARE_RESPONSE_TABLE(TWfiltrage);
};
DEFINE_RESPONSE_TABLE1(TWfiltrage,TFrameWindow)
  EV_CHILD_NOTIFY_ALL_CODES(112,CMfiltrer),
END_RESPONSE_TABLE;

TWfiltrage::TWfiltrage(PTWindowsObject AParent)
	:TFrameWindow(AParent,ml(4502,"Filtrage"))
{ int xx,yy,ww;

  Attr.Style= WS_VISIBLE | WS_POPUPWINDOW | WS_CAPTION;
  xx=2;yy=2;ww=150;
  Attr.X=200;
  Attr.Y=200;
  Attr.W=ww+6;
  this->SetIcon(GetApplication(), IDI_ICON1);
  // initialiser le contenu des contrles
  //   avec les variables correspondantes
  new TWchoix(this,0,xx,yy,ww
	      ,&typeFiltre
	      ,ml(4510,"type:")
	      ,choix_type
	      ,0);                yy+=HNUM;
  new TWint(this,0,xx,yy,ww
	      ,&ordre
	      ,ml(4506,"ordre")
	      ,4,100,4,100
	      ,0);                yy+=HNUM;
  new TWfloat(this,0,xx,yy,ww
	      ,&attenuation
	      ,ml(4507,"attenuation")
	      ,5.,100.,5.,100.
	      ,0);                yy+=HNUM;
  //
  new TStatic(this,-1,ml(4509,"Domaine  dlimiter sur le spectro")
	      ,xx,yy,ww,50,0);   yy+=50;
  //
  yy+=5;
  new TButton(this,112,ml(4508,"&Filtrer")
	      ,xx+20,yy,ww-40,24,FALSE); yy+=24;
  Attr.H=yy+30;
}
void TWfiltrage::SetupWindow()
{ TFrameWindow::SetupWindow();
}
void TWfiltrage::CMfiltrer(WPARAM)
{ if (!isDomaine)
  { affiche_erreur(ml(4501,"Dfinissez d'abord le domaine"));
    return;
  }

  souris(XC_watch);
  lanceFiltre();
  souris(XC_top_left_arrow);
  isDomaine=FALSE;
  ::InvalidateRect(wspectro->wsona->HWindow,NULL,TRUE);
}
bool TWfiltrage::CanClose()
{ yaUnFiltrage=FALSE;
  isDomaine=FALSE;
  active(PPMENUAllStandard);
  ::InvalidateRect(wspectro->wsona->HWindow,NULL,TRUE);
  return TRUE;
}

void Wtop::CMfiltreSignal()
{
  if (!yaUnSignal) return; 
  yaUnFiltrage=TRUE;
  isDomaine=FALSE;
  desactive(PPMENUAllStandard);
  GetApplication()->MakeWindow(
	   new TWfiltrage(PTWtop));

}
#endif

//////////////////////////////////////////////
// traitement de la souris
//
void drawDownDomaine(int event_x,int event_y)
{
  if (!yaUnSignal) return; 

  if (isDomaine) // effacer le domaine s'il en existe un
    affDomaine(wspectro->wsona, x0Domaine, y0Domaine
	       , x1Domaine, y1Domaine);
  x0Domaine = x1Domaine = event_x;
  y0Domaine = y1Domaine = event_y;
  affDomaine(wspectro->wsona, x0Domaine, y0Domaine
	       , x1Domaine, y1Domaine);
  isDomaine=TRUE;
}

void drawMotionDomaine(int event_x,int event_y)
{
  if (!yaUnSignal) return;
  affDomaine(wspectro->wsona, x0Domaine, y0Domaine, x1Domaine, y1Domaine);
  x1Domaine = event_x;
  y1Domaine = event_y;
  affDomaine(wspectro->wsona, x0Domaine, y0Domaine, x1Domaine, y1Domaine);
}

void drawUpDomaine(int ,int) 
{ if (!yaUnSignal) return; 
}

void applyFIR(
  int ordre_,
  float attenuation_,
  bool typeFiltre_)
{
  attenuation = attenuation_;
  ordre = ordre_;
  typeFiltre = typeFiltre_;
  if (!isDomaine) {
    affiche_erreur(ml(4501,"Dfinissez d'abord le domaine"));
    return;
  }

  souris(XC_watch);
  lanceFiltre();
  souris(XC_top_left_arrow);
  isDomaine=FALSE;
  ::InvalidateRect(wspectro->wsona->HWindow,NULL,TRUE);
}
