/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier getphoan.cpp
//

#include "common.h"
#include "signal.h"
#pragma hdrstop

#include "snoori.h"
#include "menu.h"
#include "notetiq.h"
#include "getphoan.h"
// Control ids:
//
const int ID_EXAMPLE_EDIT       = 200,
          IDCHECKBOX  			  = 201;

// Message ids:
//
const int CN_UPDATE             = 300,
		    CN_PHONBUTON			  = 301;

static int numPhonButton = 0;

DEFINE_RESPONSE_TABLE1(TWeditphoan, TEdit)
  EV_WM_KEYDOWN,
END_RESPONSE_TABLE;

void TWeditphoan::EvKeyDown(uint key, uint repeatCount, uint flags)
{
  TEdit::EvKeyDown(key, repeatCount, flags);
  backspace = key == '\b';
  leftArrow = key == 37;
  rightArrow = key == 39;
  endKey = key == '\r';
  NotifyParent(CN_UPDATE); // pour le traitement de la lecture des caracteres d'annotation
}

//
// NotifyParent(). 'easy' method of notifing the parent of an event.
// Notifies immediately
void
TWeditphoan::NotifyParent(int notification)
{
  Parent->SendNotification(Attr.Id, notification, HWindow);
}



DEFINE_RESPONSE_TABLE1(TWgetphoan,TDialog)
  EV_CHILD_NOTIFY(ID_EXAMPLE_EDIT, CN_UPDATE, CmUpdate),
  EV_COMMAND(IDCANCEL, CmCancel),
  EV_COMMAND(IDCHECKBOX, HandleCheckPlayer),
  EV_WM_CLOSE,
//  EV_CHILD_NOTIFY(ID_EXAMPLE_EDIT, CN_PHONBUTON, CmButton),
END_RESPONSE_TABLE;




extern void ecoute(long deb, long fin, int n);
/*
 * Recuperation des codes phonetiques des boutons.
 */
LRESULT TWgetphoan::EvCommand(UINT id, HWND x, UINT y)
{
  TWindow::EvCommand(id, x, y);
  if (  (id>=ID_PHON0) && (id<=ID_PHON0+127) ) {
    capture->SetText(TextOfPhon(id - ID_PHON0));
    executeAnnotation(TextOfPhon(id - ID_PHON0));
  }
  else if (id == ID_PHON0 - 1)
  	ecoute(etiqs.e[numEtiq]->limite, etiqs.e[numEtiq+1]->limite, 1);
  return 0;
}


/*
 * EvCommandEnable  pour les boutons des codes phonetiques
 */
void TWgetphoan::EvCommandEnable(TCommandEnabler& ce)
{
  if (ID_PHON0 - 1 <= ce.Id && ce.Id < ID_PHON0 + 1000) {
    ce.Enable(true);
  }
}

TWgetphoan::TWgetphoan (PTWindowsObject AParent, LPSTR name, int echt, int x, int y, int w,
	      					const char *afftext):
                        TDialog(AParent,name)
{
  int i;

  TWindow::Attr.W = 40 * 8;
  TWindow::Attr.H = nPhonEntry/8*20 + 150;
  numEtiq = etiqs.search(echt);
  butPhon = new TButton*[nPhonEntry];


  for (i = 0; i < nPhonEntry; i++)
     butPhon[i] = new TButton(this, ID_PHON0 + phonEntries[i].item,
     								   phonEntries[i].text, (i%8) * 40, (i/8) * 20, 40, 20, false);
  st0 = new TStatic(this, -1, "st0",20,nPhonEntry/8*20 + 30,160,20,100);
  capture = new TStatic(this, -1, "tstatic0",150,nPhonEntry/8*20 + 30,70,20,100);
  st1 = new TStatic(this, -1, "st1",20,nPhonEntry/8*20 + 50,160,20,100);
  Teditpho = new TWeditphoan(this,ID_EXAMPLE_EDIT,"",150,nPhonEntry/8*20 + 50,70,20);
  butPlay = new TButton(this, ID_PHON0 - 1, "play current segment",
  								150, nPhonEntry/8*20+80,150,20);
  CheckPlayer = new TCheckBox(this, IDCHECKBOX, "Play back", 20, nPhonEntry/8*20+80, 100, 20);
  butCancel = new TButton(this, IDCANCEL, ml(121,"Cancel"),
  								(40*8+6) / 2 - 30, nPhonEntry/8*20+110,60,25);
}

bool TWgetphoan::CanClose()
{
  retour();
  return true;
}

void TWgetphoan::CloseWindow()
{
  TDialog::CloseWindow();
  retour();
}

void TWgetphoan::EvClose()
{
  TDialog::EvClose();
  retour();
}


TWgetphoan::~TWgetphoan()
{
  detruitAnnexes();
}

void TWgetphoan::CmCancel()
{
  TDialog::CmCancel();
  retour();
}

void TWgetphoan::retour()
{
  extern int getPhoanDejaCree;
  wsignal->aff();
  if (!Licensed) {
    active(MENUOpen);
    active(MENUNew);
    active(MENUEcoute1Fois);
    active(MENUEcoute);
    active(MENUPlayFile);
    active(MENUZoomin);
    active(MENUZoomout);
    active(PPMENUSpectro);
  }
  else active(PPMENUAllStandard);
  ::DrawMenuBar(PTWtop->HWindow);
  getPhoanDejaCree = false;
}

void TWgetphoan::detruitAnnexes()
{
  int i;
  for (i = 0; i < nPhonEntry; i++) delete butPhon[i];
  delete [] butPhon;
  delete st0;
  delete st1;
  delete Teditpho;
  delete butPlay;
  delete butCancel;
  delete capture;
  delete CheckPlayer;
}

void TWgetphoan::SetupWindow()
{
  int i;
  TDialog::SetupWindow();

  capture->SetText("");
  st0->SetText("Last annotation");
  st1->SetText("Keyboard input");
  capture->SetWindowFont(queryFont(strlen(PhoneticFont) ? PhoneticFont : "arial",
                         7, 14, 0), false);
  Teditpho->SetWindowFont(queryFont(strlen(PhoneticFont) ? PhoneticFont : "arial",
                         7, 14, 0), false);
  for (i = 0; i < nPhonEntry; i++)
     butPhon[i]->SetWindowFont(queryFont(strlen(PhoneticFont) ? PhoneticFont : "arial",
                         7, 14, 0), false);

  CheckPlayer->SetState(BF_UNCHECKED);
  if ((numEtiq>=0)&&(numEtiq<etiqs.nb-1)) {
    wsignal->aff();
    wsignal->BlueSegment(etiqs.e[numEtiq]->limite , etiqs.e[numEtiq+1]->limite);
  }
  MoveWindow(100, 100, 40 * 8 + 6, nPhonEntry/8*20 + 170, true);

}

/*
 * Handles BN_CLICK notifications for the play back button
 */
void
TWgetphoan::HandleCheckPlayer()
{
/*  if (CheckPlayer->GetCheck() == BF_CHECKED)
    MessageBox("Enabled", "The play back is:", MB_OK);
  else
    MessageBox("Disabled", "The play back is:", MB_OK);
    */
}

void TWgetphoan::UpdateCurrentSegment()
{
  wsignal->aff();
  if (numEtiq >= etiqs.nb-1) numEtiq = max(-1, etiqs.nb-2);
  if ((numEtiq>=0)&&(numEtiq<etiqs.nb-1))
	 wsignal->BlueSegment(etiqs.e[numEtiq]->limite , etiqs.e[numEtiq+1]->limite);
}

/*
 * teste si s est une sous-chaine de t.
 */
static bool sousChaine(char *s, char *t)
{
  int i, len = strlen(s);
  if (len > strlen(t)) return(false);
  for (i = 0; i < len && s[i] == t[i]; i++);
  return(i == len);
}

/*
 * teste si la chaine s est un code (ou un debut) phonetique possible.
 * et retourne le premier code phonetique possible.
 */
static int codePossible(char *s, int *trouve)
{
  int i, nPos, nexact;
  bool exact = false;
  for (i = 0, nPos = 0; i < nPhonEntry; i++)
    if (sousChaine(s, phonEntries[i].text)) {
       *trouve = i;
       if (!strcmp(s, phonEntries[i].text)) {
       	exact = true;
         nexact = i;
       }
       nPos++;
  }
  // Si on a trouve la bonne chaine exactement on conserve le phoneme correct.
  if (exact) *trouve = nexact;
  return(nPos);
}


/*
 * One annotation forwards.
 */
void TWgetphoan::forwards()
{
  if  ((numEtiq>=0)&&(numEtiq<etiqs.nb-2)) numEtiq++;
  if (numEtiq < etiqs.nb-1
      && (etiqs.e[numEtiq]->limite > ct.f_a
          || (abs(etiqs.e[numEtiq+1]->limite - etiqs.e[numEtiq]->limite) < (ct.f_a - ct.d_a) / 3)
              && etiqs.e[numEtiq+1]->limite > ct.f_a)) {
    int echt;
    souris(XC_watch);
    echt= max(0, etiqs.e[numEtiq]->limite - 1000);
    SetScrollSpectroPosition(echt);
    allerA(echt);
    souris(XC_top_left_arrow);
  }
  wsignal->aff();
  if ((numEtiq>=0)&&(numEtiq<etiqs.nb-1)) {
    wsignal->BlueSegment(etiqs.e[numEtiq]->limite , etiqs.e[numEtiq+1]->limite);
    if (CheckPlayer->GetState() == BF_CHECKED
    	  && etiqs.e[numEtiq]->limite >= ct.d_e
        && etiqs.e[numEtiq+1]->limite <= ct.f_e)
    	ecoute(etiqs.e[numEtiq]->limite, etiqs.e[numEtiq+1]->limite, 1);
  }
}

/*
 * Update the annotations with the new one and
 * moves in the signal if needed.
 */
void TWgetphoan::executeAnnotation(
 char *text)
{
  /* The phonetic symbol is stored */
  if  ((numEtiq>=0)&&(numEtiq<etiqs.nb-1)) {
    etiqs.e[numEtiq]->SetText(text);
    aff_etiqs();
  }
  forwards();
}

/*
 * One annotation backwards.
 */
void TWgetphoan::backwards()
{
  if (numEtiq > 0) numEtiq--;
  if (numEtiq >= 0 && etiqs.e[numEtiq]->limite < ct.d_a) {
    int echt;
    souris(XC_watch);
    echt= max(0, etiqs.e[numEtiq]->limite - (int)(((ct.f_a - ct.d_a) * 2./3.)));
    SetScrollSpectroPosition(echt);
    allerA(echt);
    souris(XC_top_left_arrow);
  }
  wsignal->aff();
  if ((numEtiq>=0)&&(numEtiq<etiqs.nb-1)) {
    wsignal->BlueSegment(etiqs.e[numEtiq]->limite , etiqs.e[numEtiq+1]->limite);
    if (CheckPlayer->GetState() == BF_CHECKED
    	  && etiqs.e[numEtiq]->limite >= ct.d_e
        && etiqs.e[numEtiq+1]->limite <= ct.f_e)
    	ecoute(etiqs.e[numEtiq]->limite, etiqs.e[numEtiq+1]->limite, 1);
  }    
}

 /*
 * Delete the current annotation
 * and moves back in the signal if needed.
 */
void TWgetphoan::deleteLastAnnotation()
{
  /* The phonetic symbol is stored */
  if  (numEtiq >=0) {
    etiqs.e[numEtiq]->SetText("");
    aff_etiqs();
  }
  backwards();
}

//
// UpdateTextFields(). Updates text fields that reflex the edit control's state.
//
void
TWgetphoan::UpdateTextFields()
{
  char buf[MAX_LIGNE] = "";
  int lePhon;
  uint sPos, ePos, curLine;

  // make sure all pending messages are processed first
  //
  GetApplication()->PumpWaitingMessages();

  if (Teditpho->endKey) wsignal->aff();
  Teditpho->GetLine(buf, 10, 0);
  if (!strlen(buf)) {
    if (Teditpho->backspace) deleteLastAnnotation();
    else if (Teditpho->rightArrow) forwards();
    else if (Teditpho->leftArrow) backwards();
  }
  else switch (codePossible(buf, &lePhon)) {
    case 0: // la chaine qui vient d'etre rentree est illegale.
      // On regarde si en enlevant le dernier caractere elle est correcte.
     buf[strlen(buf) - 1] = (char) 0;
      if (codePossible(buf, &lePhon)) {
        ePos = strlen(buf);
        executeAnnotation(phonEntries[lePhon].text);
        capture->SetText(phonEntries[lePhon].text);
        Teditpho->DeleteSubText(0, ePos);
      }
      else Teditpho->DeleteLine(0);
      break;
    case 1:  // Un seul code correspond a la chaine, on etiquette le segment.
      executeAnnotation(phonEntries[lePhon].text);
  		capture->SetText(phonEntries[lePhon].text);
  		Teditpho->DeleteLine(0);
      break;
    default: // On ne peut rien dire, on ne fait rien
      break;
  }
}


