/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier piquefor.cpp
//

#include "common.h"

#include <math.h>
#include <sys/types.h>
#include <fstream.h>

#include "complex.h"

#include "snoori.h"
#include "signal.h"
#include "outils.h"
#include "files.h"
#include "spectre.h"
#include "pics.h"
#include "opt_for.h"
#include "wformant.h"
#include "menu.h"
#include "spectro.h"
#include "fftlpc.h"
#include "axes.h"
#include "pitchm.h"
#include "pitchmk.h"
#include "audio.h"
#include "editeur.h"
#include "editer.h"
#include "wopensav.h"        
#include "mccandle.h"
#include "fmaxspec.h"
#include "guiklat.h"
#include "add_files/htmlhelp.h"

////////////////////////////////////////////
Wpiquefor *wpiquefor;
#define DEPFOR 4  // les formants sont echantillonns
                   // toutes les DEPFOR ms
klatt_frame_t defaultFrame = {0, 100, 60, 0, 50, 0, 60, 0, 90, 0, 140, 0, 210, 0, 300,  // Date to F6
			      280, 90, 0, 150, 0, 30, 0, 0, 0, 0, 0, 4003, 0, 4003, 0, 4003, // Nasal zero to B3phz
                              0, 4003, 0, 4003, 0, 4003, 0, 0, 60, 55, 17, 30, 180, // A4 to Rg
                              0, 90, 0, 4003, 0, 90, 0, 4003, 0, 90, 0, 4003}; // F1Nhz to B3Nphz

// The very high values for F1 to F6 bandwidth (4003) are intended to cancel the energy of
// a formant as soon as it is not defined anymore. 4003 (the 3 at the end is just to find
// it easily when the Klatt file is edited by hand.

// ancienne valeur klatt_frame_t defaultFrame = {0, 100, 60, 0, 50, 0, 60, 0, 90, 0, 140, 0, 210, 0, 300,
//				     280, 90, 0, 150, 0, 30, 0, 0, 0, 0,
//				     0, 50, 0, 4003, 0, 90, 0, 140, 0, 180, 0, 210, 0, 0, 60, 55};

char *nomParamKlatt[] = {"Date", "F0hz10", "AVdb", "F1hz", "B1hz", "F2hz", "B2hz",
			 "F3hz", "B3hz", "F4hz", "B4hz", "F5hz", "B5hz", "F6hz", "B6hz",
			 "FNZhz", "BNZhz", "FNPhz", "BNPhz", "ASP", "Kopen", "Aturb",
			 "TLTdb", "AF", "Kskew", "A1", "B1phz", "A2", "B2phz", "A3",
			 "B3phz", "A4", "B4phz", "A5", "B5phz", "A6", "B6phz", "ANP",
			 "AB", "AVpdb", "Gain0", "Rax1000", "Rkx100", "Rgx50",
          "F1Nhz","B1Nhz","A1N","B1Nphz",
          "F2Nhz","B2Nhz","A2N","B2Nphz",
          "F3Nhz","B3Nhz","A3N","B3Nphz"};
char *nomParMajeursKlatt[NBMAJEURS] = {"F0","F1","F2","F3","F4","F5","F6",
			      "F1N","F2N","F3N","FNZ","FNP","Aspiration","Kopen","Aturb",
			      "TLTdB","Friction","Skewness","Bypass","Gain", "Rax1000", "Rkx100", "Rgx50"};
/*
 * Etat dans lequel se trouvent les formants.
 */
int etatParam[NBMAJEURS] = {ACTIF, ACTIF, ACTIF, ACTIF, ACTIF, ACTIF, ACTIF, ACTIF, ACTIF, ACTIF,
			    CACHE, ACTIF, CACHE, CACHE, CACHE, CACHE, ACTIF, CACHE, CACHE, CACHE, CACHE, CACHE, CACHE};

/* tableau donnant les dimensions de chacun des parametres majeurs. */
int dim[NBMAJEURS] = {4,5,5,5,5,5,5,5,5,5,3,4,2,2,2,2,2,2,2,2,2,2,2};

int scalePar[] = {-1, SCALEF0, SCALEDBC, SCALEFr, SCALEBwC, SCALEFr, SCALEBwC,
		SCALEFr, SCALEBwC, SCALEFr, SCALEBwC, SCALEFr, SCALEBwC, SCALEFr, SCALEBwC,
		SCALEFr, SCALEBwC, SCALEFr, SCALEBwC, SCALEDBC, SCALEDNb, SCALEDBC,
		SCALEDBC, SCALEDBC, SCALEDNb, SCALEDBP, SCALEBwP, SCALEDBP, SCALEBwP, SCALEDBP,
		SCALEBwP, SCALEDBP, SCALEBwP, SCALEDBP, SCALEBwP, SCALEDBP, SCALEBwP, SCALEDBP,
		SCALEDBC, SCALEDBP, SCALEDBC, SCALEDNb, SCALEDNb, SCALEDNb,
                SCALEFr, SCALEBwC,SCALEDBP, SCALEBwP,
                SCALEFr, SCALEBwC,SCALEDBP, SCALEBwP,
                SCALEFr, SCALEBwC,SCALEDBP, SCALEBwP};

// array which gives the Klatt parameter from  major and minor index
int nKlatt[NBMAJEURS][LARGBANDEP-FREQUENCE+1] = {
  {F0hz10_, AVdb_, AVpdb_, -1}, /* F0 */
  {F1hz_, A1_, B1hz_, B1phz_}, /* F1 */
  {F2hz_, A2_, B2hz_, B2phz_}, /* F2 */
  {F3hz_, A3_, B3hz_, B3phz_}, /* F3 */
  {F4hz_, A4_, B4hz_, B4phz_}, /* F4 */
  {F5hz_, A5_, B5hz_, B5phz_}, /* F5 */
  {F6hz_, A6_, B6hz_, B6phz_}, /* F6 */
  {F1Nhz_, A1N_, B1Nhz_, B1Nphz_}, /* F1N */
  {F2Nhz_, A2N_, B2Nhz_, B2Nphz_}, /* F2N */
  {F3Nhz_, A3N_, B3Nhz_, B3Nphz_}, /* F3N */
  {FNZhz_, BNZhz_, -1, -1}, /* FNZ */
  {FNPhz_, ANP_, BNPhz_, -1}, /* FNP */
  {ASP_, -1, -1, -1}, /* ASPIRATION */
  {Kopen_, -1, -1, -1}, /* KOPEN */
  {Aturb_, -1, -1, -1}, /* ATURB */
  {TLTdb_, -1, -1, -1}, /* TLTD */
  {AF_, -1, -1, -1}, /* AFRICTION */
  {Kskew_, -1, -1, -1}, /* KSKEW */
  {AB_, -1, -1, -1}, /* ABYPASS */
  {Gain0_, -1, -1, -1}, /* GAIN */
  {Ra_, -1, -1, -1}, /* Ra */
  {Rk_, -1, -1, -1}, /* Rk */
  {Rg_, -1, -1, -1}}; /* Rg */


// array which gives the Klatt parameter for the major index and
//   the current scale.
int nKlattScale[NBMAJEURS][SCALEDNb - SCALEFr + 1] = {
  {-1,		AVpdb_,	-1,	-1,	AVdb_,	F0hz10_,-1}, /* F0 */
  {F1hz_,	A1_,	B1hz_,	B1phz_, -1,	-1,	-1}, /* F1 */
  {F2hz_,	A2_,	B2hz_,	B2phz_, -1,	-1,	-1}, /* F2 */
  {F3hz_,	A3_, 	B3hz_, 	B3phz_, -1, 	-1,	-1}, /* F3 */
  {F4hz_, 	A4_, 	B4hz_, 	B4phz_, -1, 	-1, 	-1}, /* F4 */
  {F5hz_, 	A5_, 	B5hz_, 	B5phz_, -1, 	-1, 	-1}, /* F5 */
  {F6hz_, 	A6_, 	B6hz_, B6phz_, 	-1, 	-1, 	-1}, /* F6 */
  {F1Nhz_,      A1N_,   B1Nhz_, B1Nphz_, -1,	-1,	-1}, /* F1N */
  {F2Nhz_,      A2N_,   B2Nhz_, B2Nphz_, -1,	-1,	-1}, /* F2N */
  {F3Nhz_,      A3N_,   B3Nhz_, B3Nphz_, -1,	-1,	-1}, /* F3N */
  {FNZhz_, 	-1, 	BNZhz_, -1, 	-1, 	-1, 	-1}, /* FNZ */
  {FNPhz_, 	ANP_, 	-1, 	BNPhz_, -1, 	-1, 	-1}, /* FNP */
  {-1, 		-1, 	-1, 	-1, 	ASP_, 	-1, 	-1}, /* ASPIRATION */
  {-1, 		-1, 	-1, 	-1, 	-1, 	-1, 	Kopen_}, /* KOPEN */
  {-1, 		-1, 	-1, 	-1, 	Aturb_, -1, 	-1}, /* ATURB */
  {-1, 		-1, 	-1, 	-1, 	TLTdb_, -1, 	-1}, /* TLTD */
  {-1, 		-1, 	-1, 	-1, 	AF_, 	-1, 	-1}, /* AFRICTION */
  {-1, 		-1, 	-1, 	-1, 	-1, 	-1, 	Kskew_}, /* KSKEW */
  {-1, 		-1, 	-1, 	-1, 	AB_, 	-1, 	-1}, /* ABYPASS */
  {-1, 		-1, 	-1, 	-1, 	Gain0_, -1, 	-1}, /* GAIN */
  {-1, 		-1, 	-1, 	-1, 	-1, 	-1, 	Ra_}, /* Ra */
  {-1, 		-1, 	-1, 	-1, 	-1, 	-1, 	Rk_}, /* Rk */
  {-1, 		-1, 	-1, 	-1, 	-1, 	-1, 	Rg_}}; /* Rg */

static int mineur(int majeur,int scale);

// differentes constantes pour contrler
//   le suivi de formants



static TabPoint<Point5P> TESTPoints(0);//XM static PiqPoint *points;
//XM Pour les tests
PolyPointsMD<TabPoint <Point5P>, Point5P > PPtest(TESTPoints);

int typeScale;    // type de scale used (Frequency ...)

int paramActif;   // active parameter (which is has been read)
int typeAff = LINES;		// type of display for polypoints (squares or lines)
int depFor = DEPFOR;    // pas d'echantillonnage
							   //   des formants en ms

float yToValue(float y, int scale);
Listyl fpattern[NBMAJEURS];

void messagePasDeFormantsSelectionnes()
{ affiche_warning(
      ml(6423,"pas de formants slectionns,\n\
donc pas d'opration  faire"));
}



bool desc_visible;
/////////////////////////////////////
// fenetre Wdescform
/////////////////////////////////////

class TWdescform : public TDialog
{
  public:
    TWdescform(PTWindowsObject AParent, LPSTR name);
    virtual void SetupWindow();
    virtual bool CanClose();
};

TWdescform::TWdescform(PTWindowsObject AParent, LPSTR name)
	:TDialog(AParent,name){
   	desc_visible = TRUE;
}

void TWdescform::SetupWindow(){
	TDialog::SetupWindow();
}

bool TWdescform::CanClose(){
	desc_visible = FALSE;
  return TRUE;
}


static PolyPointsKla *newPPd(int dim)
{                                                     
  switch (dim) {
    case 2:
      return(new PolyPointsMD<TabPoint <Point2P>, Point2P>());
    case 3:
      return(new PolyPointsMD<TabPoint <Point3P>, Point3P>());
    case 4:
      return(new PolyPointsMD<TabPoint <Point4P>, Point4P>());
    case 5:
      return(new PolyPointsMD<TabPoint <Point5P>, Point5P >());
  }
}


bool yaUneFenetrePiquefor = false;



DEFINE_RESPONSE_TABLE1(Wpiquefor,WSignal)
  EV_CHILD_NOTIFY_ALL_CODES(101,IDscrollbar),
  EV_COMMAND(MENUlireFor,CMlireFor),
  EV_COMMAND(MENUpasteKlattFile,CMpasteKlattFile),
  EV_COMMAND(MENUdetruireFor,CMdetruireFor),
  EV_COMMAND(MENUrecalForRACLPC,CMrecalForRACLPC),
  EV_COMMAND(MENUrecalForLISCEP,CMrecalForLISCEP),
  EV_COMMAND(MENUformantTracking,CMinitialFTracks),
  EV_COMMAND(MENUtrackingFromExisting,CMtrackingFromExisting),
  EV_COMMAND(MENUtrackingFromExistingSelected,CMtrackingFromExistingSelected),
  EV_COMMAND(MENUcalculimCepstral,CMcalculImCeps),
  EV_COMMAND(MENUcalculimTrueEnvelope, CMcalculImTrueEnv),
  EV_COMMAND(MENUcalculimLPC,CMcalculImLPC),
  EV_COMMAND(MENUsavePGM,CMsavePGM),
  EV_COMMAND(MENUremoveFirstPoint,CMremoveFirstPoint),
  EV_COMMAND(MENUremoveLastPoint,CMremoveLastPoint),
  EV_COMMAND(MENUlisseFor,CMlisseFor),
  EV_COMMAND(MENUselectF1,CMselectF1),
  EV_COMMAND(MENUselectF2,CMselectF2),
  EV_COMMAND(MENUselectF3,CMselectF3),
  EV_COMMAND(MENUselectF4,CMselectF4),
  EV_COMMAND(MENUselectF5,CMselectF5),
  EV_COMMAND(MENUselectF6,CMselectF6),
  EV_COMMAND(MENUselectF1N,CMselectF1N),
  EV_COMMAND(MENUselectF2N,CMselectF2N),
  EV_COMMAND(MENUselectF3N,CMselectF3N),
  EV_COMMAND(MENUmergeFor,CMmergeFor),
  EV_COMMAND(MENUselectTous,CMselectTous),
  EV_COMMAND(MENUselectAucun,CMselectAucun),
  EV_COMMAND(MENUdescf,CMopcldesc),
  EV_COMMAND(MENUsquares, CMchAffSquares),
  EV_COMMAND(MENUlines, CMchAffLines),
  EV_COMMAND(MENUFrequency, CMchScaleFreq),
  EV_COMMAND(MENUAmpParallel, CMchScaleDBP),
  EV_COMMAND(MENUAmpCascade, CMchScaleDBC),
  EV_COMMAND(MENUBwCascade, CMchScaleBWC),
  EV_COMMAND(MENUBwParallel, CMchScaleBWP),
  EV_COMMAND(MENUFreqF0, CMchScaleF0),
  EV_COMMAND(MENUNumeral, CMchScaleNum),
  EV_COMMAND(MENUpas2ms,CMpas2ms),
  EV_COMMAND(MENUpas4ms,CMpas4ms),
  EV_COMMAND(MENUpas8ms,CMpas8ms),
  EV_COMMAND(MENUpas16ms,CMpas16ms),
  EV_COMMAND(MENUsaveKlatt,CMsaveKlatt),
  EV_COMMAND(MENUsaveKlattLF,CMsaveKlattLF),
  EV_COMMAND(MENUhelpMouse,CMhelpMouse),
  EV_COMMAND(MENUTools,CMTools),
  EV_COMMAND(MENUsynthese,CMsynthese),
  EV_COMMAND(MENUsaveSyntheticSpeech,CMsaveSyntheticSpeech),
  EV_COMMAND(MENUSetParam,CMSetParam),
  EV_COMMAND(MENUgetF0PP,CMcreePPF0),
  EV_COMMAND(MENUFormant,CMFormant),
  EV_COMMAND(MENUSpectro,CMSpectro),
  EV_COMMAND(MENUchangeAnaSpec,CMchangeAnaSpec),
  EV_COMMAND(MENUsettingsKlatt, CMSettingsKlatt),
  EV_COMMAND(MENUsnakeParametersControl, CMSnakeParametersControl),
  EV_COMMAND(MENUsaveFormants, CMsaveFormants),
  EV_COMMAND(MENUreadFormants, CMlectureFicFormants),
  EV_COMMAND(MENUaffFormantsF1F2, CMaffFormantsF1F2),
  EV_COMMAND(MENUcopysynthesis, CMcopySynthesis),
  EV_COMMAND(MENUinitialCurvesLPC, CMchInitialTracksLPC),
  EV_COMMAND(MENUinitialCurvesTrueenv, CMchInitialTracksTrueenv),
END_RESPONSE_TABLE;


Wpiquefor *PTWpiquefor;
//Wdescforms *PTWdescfor;

Wpiquefor::Wpiquefor()
    : WSignal(PTWtop,ml(6413,"Klatt synthesizer"))
{
  ppp = new TabPoint<Point2P>();
  PTWpiquefor = this;
  ::typeScale = SCALEFr;
  typeImageSnake = SnakeImageCepstral;
  typeInitialSnakes = InitialCurvesLPC;
  ctImage.d_e = ctImage.f_e = -1;

  typeFor = FORMANT;
  ppointModif = (PolyPointsKla *) NULL;
  Attr.Style=WS_VISIBLE|WS_POPUPWINDOW|WS_CAPTION
		       |WS_THICKFRAME;
  Attr.X=60;
  Attr.Y=0;
  Attr.W=400;
  Attr.H=300;
  Attr.AccelTable = IDA_FORMANTS;

  ::yaUneFenetrePiquefor = true;
  yaUneFenetreToolsFormants = false;
  tools = (WToolsFormants *) NULL;
  space_lowFreq =new SAMPLEbuf();
  space_highFreq=new SAMPLEbuf();
  space_ener    =new SAMPLEbuf();
  lowFreqFilter =new profiltre();
  highFreqFilter=new profiltre();
}

Wpiquefor::~Wpiquefor()
{
  // DestroyMenu(::GetMenu(HWindow));

  selectPPoint((PolyPointsKla *) NULL, 0);
  affPPoints();

  active(PPMENUAllStandard);
  ::DrawMenuBar(PTWtop->HWindow);
  ::yaUneFenetrePiquefor = false;
  if (yaUneFenetreMccandless) delete wmccandless; wmccandless=NULL;

  if (tools) {
      delete tools;
	   tools = (WToolsFormants *) NULL;
  }
  wspectro->wsona->MoveBarre(-1);
  if (yaUnSignal) wspectro->wvscale->affichage(0., fMaxSpectro, wspectro->wsona->incr, "Hz");
  if (ppp)   delete ppp;    ppp  =NULL;
  if (space_lowFreq)  delete space_lowFreq;  space_lowFreq =NULL;
  if (space_highFreq) delete space_highFreq; space_highFreq=NULL;
  if (space_ener)     delete space_ener;     space_ener    =NULL;
  if (lowFreqFilter)  delete lowFreqFilter;  lowFreqFilter =NULL;
  if (highFreqFilter) delete highFreqFilter; highFreqFilter=NULL;
}



extern void RawtoWav(char* fichierraw,char* fichierwav,float frequence);

//
// Append a zero signal at the end of the shadow file when the user moves
// on the right.
//
void Wpiquefor::lengthenShadow()
{
  // Lengthening the shadow occurs only if the shadow file ends less than
  // one second after last polypoints.
  if (enEcht(dateLastEndPP(true) + 1000) > ct.tf - 1) {
    bool status;
    STRING appendFile, appendRawFile;
    sprintf(appendFile, "%s\\temp\\append.wav", WinSnooriDirectory);
    sprintf(appendRawFile, "%s\\temp\\append.raw", WinSnooriDirectory);
    status = cstRawFile(appendRawFile, (int) (en_cours.frequence * 1));
    if (!status) return;
    RawtoWav(appendRawFile, appendFile, (double) en_cours.frequence);
    unlink(appendRawFile);
    insererUnFichierWAV(appendFile, min(ct.f_e, ct.tf - 1));
    unlink(appendFile);
  }
}

void Wpiquefor::SetupWindow()
{
  wspectro->wsona->MoveBarre(-1);
  initCtrlCommandKlatt();
  if (yaUnSignal == ASIGNAL) pm.currentF0(); // Calculate F0 if need be.
  else initShadow(max(dateLastEndPP(true), DEFAULTDURATION), true, 0, 2000, ctrlCommandKlatt.samplingRate);
  syntheExists = false;
  marquesPitch.newsize(1);
  WSignal::SetupWindow();
  this->AssignMenu("FORMANTS");
  synthesizeAndDisplay(true);
  ctEner.d_e = ctEner.f_e = -1;
  (*space_lowFreq).newsize(1);
  (*space_highFreq).newsize(1);
  correlationSizeMs = 12; // 12 ms for the autocorrelation
  space_ter.newsize(MAXECH);
  initFilters();
  chScaleKlatt();
  chTypeAff();
  chTypeFor();
  chInitialTracks();
  CMcalculImCeps();

  initOptKlattSpec();
  CheckMenuItemPas();
}


void Wpiquefor::aff()
{
  if ( typeFor == FORMANT )
  	affPPoints();                          
  else
  	if ( typeFor == SPECTRO )
   	affPPoints();
}

bool Wpiquefor::CanClose()
{
  return true;
}

void Wpiquefor::CMchAffSquares()
{
 ::typeAff = SQUARES;
 chTypeAff();
}

void Wpiquefor::CMchAffLines()
{
 ::typeAff = LINES;
 chTypeAff();
}

void Wpiquefor::CMsynthese()
{
  synthesizeAndDisplay(false);
}

void Wpiquefor::synthesizeAndDisplay(
  bool newDisplay)
{
  STRING resFile;
  sprintf(resFile, "%s\\temp\\synthe.wav", WinSnooriDirectory);

  synthetise(resFile);

  displaySyntheticSpeech(newDisplay, resFile);
}


// Synthesizes the waveform and saves it.
void Wpiquefor::synthetise(
  char *filename)
{
  extern void RawtoWav(char* fichierraw,char* fichierwav,float frequence);
  STRING commande, resRawFile, klattemp;
  STARTUPINFO siStartInfo;
  PROCESS_INFORMATION piProcInfo;
  DWORD code;

  sprintf(resRawFile, "%s\\temp\\synthe.raw", WinSnooriDirectory);
  sprintf(klattemp, "%s\\temp\\klattemp.kle", WinSnooriDirectory);

  sauvePP(klattemp, 0, ct.tf, true, 0, NPARLE);
  if (ctrlCommandKlatt.output == COMPLETE)
  sprintf(commande, "\"%s\\Klatt\\klatt.exe\" -r %d -i \"%s\\temp\\klattemp.kle\" -o \"%s\\temp\\synthe.raw\" -f %d -s %d -F %d -v %d",
           WinSnooriDirectory, ctrlCommandKlatt.byteArrang, WinSnooriDirectory, WinSnooriDirectory,
           ctrlCommandKlatt.frame, (yaUnSignal == ASHADOW)? ctrlCommandKlatt.samplingRate : ((int) en_cours.frequence),
           ctrlCommandKlatt.flutter, (ctrlCommandKlatt.source == KLATTSOURCE) ? 2 : 4);
  else sprintf(commande, "\"%s\\Klatt\\klatt.exe\" -r %d -i \"%s\\temp\\klattemp.kle\" -o \"%s\\temp\\synthe.raw\" -t %d -f %d -s %d -F %d -v %d",
           WinSnooriDirectory, ctrlCommandKlatt.byteArrang, WinSnooriDirectory, WinSnooriDirectory,
           ctrlCommandKlatt.output,
           ctrlCommandKlatt.frame, (yaUnSignal == ASHADOW)? ctrlCommandKlatt.samplingRate : ((int) en_cours.frequence),
           ctrlCommandKlatt.flutter, (ctrlCommandKlatt.source == KLATTSOURCE) ? 2 : 4);
  memset(&siStartInfo, 0, sizeof(STARTUPINFO));
  siStartInfo.cb = sizeof(STARTUPINFO);
  CreateProcess(NULL, commande, 0, 0, false, DETACHED_PROCESS, 0, 0, &siStartInfo, &piProcInfo);
  for (code = STILL_ACTIVE; code == STILL_ACTIVE;) {
    GetExitCodeProcess(piProcInfo.hProcess,&code);
    Sleep(100);
  }
  RawtoWav(resRawFile, filename, (float) ctrlCommandKlatt.samplingRate);
  unlink(resRawFile);
  unlink(klattemp);
  // resFile MUST remain because it is used while moving with the scrollbar.
}

//  If newDisplay is true it does not display the previous
// zoom if it exists.
void Wpiquefor::displaySyntheticSpeech(
  bool newDisplay,
  char *resFile)
{
  extern void substitueFichier(char *filname, int ou);
  int d, f;

  wspectr->Show(SW_SHOWNORMAL);

  // get the old zoom parameters.
  d = cts.d_a;
  f = cts.f_a;
  // If the sampling frequency changed previous time references are discarded.
  if (newDisplay || fabs(info_en_cours.frequence - ctrlCommandKlatt.samplingRate) > 1) {
    lecture_signal(resFile, 0);
    spectrogram(cts.d_es, cts.f_es, ON_NETTOIE);
  }
  // If there was a zoom it is used.
  else if (cts.modeMemoire == ZOOM) {
    lecture_signal(resFile, cts.d_a);
    SetModeZoom(ZOOM);
    execZoom(d, f);
  }
  // No zoom, no reading of a new file
  else {
    lecture_signal(resFile, cts.d_a);
    spectrogram(cts.d_es, cts.f_es, ON_NETTOIE);
  }

  wspectr->UpdateWindow();

  if (yaUnSignal == ASHADOW) {
    int dateSubstitution = datePremPP(true);
    int dateEndSubstitution = dateLastEndPP(true) + 1000; // + 1000 ms
    if (dateSubstitution > INFINI_I / 2 || dans(dateSubstitution, enMilli(ct.d_a), enMilli(ct.f_a)))
       initShadow(max(enMilli(ct.tf), dateEndSubstitution), true, (syntheExists) ? enMilli(ct.d_a) : 0, (syntheExists) ? enMilli(ct.f_a) : 2000, ctrlCommandKlatt.samplingRate);
    else initShadow(max(enMilli(ct.tf), dateEndSubstitution), true, max(dateSubstitution - 100, 0), max(dateSubstitution - 100, 0) + 2000, ctrlCommandKlatt.samplingRate);

    if (dateSubstitution < INFINI_I) substitueFichier(resFile, enEcht(dateSubstitution));
    affPPoints();
    wspectro->wsona->UpdateWindow();
  }
  syntheExists = true;
  ::PlaySound(resFile, 0, SND_ASYNC | SND_NOWAIT | SND_FILENAME);
}

void Wpiquefor::CMsaveSyntheticSpeech()
{
  STRING filename;

  if (!yaUnSignal) return;

  strcpy(filename,"*.*");
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "*.*", 0,"","");
  strcpy(filename,"*.*");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (TFileSaveDialog(Parent,FileData, 0).DoExecute() == IDOK) {
     strcpy(filename,FileData.FileName);
     if (!isWAV(filename)) {  // forces the name to become a wav name!
       char drive[_MAX_DRIVE], dir[_MAX_DIR], file[_MAX_FNAME], ext[_MAX_EXT];
        _splitpath(filename, drive, dir, file, ext);
        _makepath(filename, drive, dir, file,"wav");
     }
     if (droit_ecrire_message(filename)) synthetise(filename);
  }
}

// Affichage des polypoints correspondant aux formants F1 F2 dans le plan F1 F2.
void Wpiquefor::CMaffFormantsF1F2()
{
  wmccandless = new Wmccandless;
  yaUneFenetreMccandless = true;
  GetApplication()->MakeWindow(wmccandless);
  wf1f2->paint();
  affVoyelles();
}

void Wpiquefor::CMplayKlatt()
{
  if (syntheExists) CMplay();
}

void Wpiquefor::CMzoominKlatt()
{
  if (syntheExists) CMzoomIn();
}

void Wpiquefor::CMzoomoutKlatt()
{
  if (syntheExists) CMzoomOut();
}

void Wpiquefor::CMchScaleFreq()
{
 ::typeScale = SCALEFr;
 chScaleKlatt();
}

void Wpiquefor::CMchScaleDBP()
{
 ::typeScale = SCALEDBP;
 chScaleKlatt();
}

void Wpiquefor::CMchScaleDBC()
{
 ::typeScale = SCALEDBC;
 chScaleKlatt();
}

void Wpiquefor::CMchScaleBWC()
{
 ::typeScale = SCALEBwC;
 chScaleKlatt();
}

void Wpiquefor::CMchScaleBWP()
{
 ::typeScale = SCALEBwP;
 chScaleKlatt();
}

void Wpiquefor::CMchScaleF0()
{
 ::typeScale = SCALEF0;
 chScaleKlatt();
}

void Wpiquefor::CMchScaleNum()
{
 ::typeScale = SCALEDNb;
 chScaleKlatt();
}

void Wpiquefor::chTypeAff()
{
  WORD c=MF_CHECKED, u=MF_UNCHECKED;

  CheckMenuItem(::GetMenu(HWindow), MENUsquares, (::typeAff==SQUARES)?c:u);
  CheckMenuItem(::GetMenu(HWindow), MENUlines,(::typeAff==LINES)?c:u);
  affPPoints();
}

void Wpiquefor::chScaleKlatt()
{
  WORD c=MF_CHECKED, u=MF_UNCHECKED;

  CheckMenuItem(::GetMenu(HWindow), MENUFrequency, (typeScale==SCALEFr)?c:u);
  CheckMenuItem(::GetMenu(HWindow), MENUAmpParallel,(typeScale==SCALEDBP)?c:u);
  CheckMenuItem(::GetMenu(HWindow), MENUAmpCascade,(typeScale==SCALEDBC)?c:u);
  CheckMenuItem(::GetMenu(HWindow), MENUBwCascade,(typeScale==SCALEBwC)?c:u);
  CheckMenuItem(::GetMenu(HWindow), MENUBwParallel,(typeScale==SCALEBwP)?c:u);
  CheckMenuItem(::GetMenu(HWindow), MENUFreqF0,(typeScale==SCALEF0)?c:u);
  CheckMenuItem(::GetMenu(HWindow), MENUNumeral,(typeScale==SCALEDNb)?c:u);
  selectFor(FAUCUN);
  affPPoints();
}

void Wpiquefor::CheckMenuItemPas()
{
  CheckMenuItem(::GetMenu(HWindow),MENUpas2ms,(depFor== 2)?MF_CHECKED:MF_UNCHECKED);
  CheckMenuItem(::GetMenu(HWindow),MENUpas4ms,(depFor== 4)?MF_CHECKED:MF_UNCHECKED);
  CheckMenuItem(::GetMenu(HWindow),MENUpas8ms,(depFor== 8)?MF_CHECKED:MF_UNCHECKED);
  CheckMenuItem(::GetMenu(HWindow),MENUpas16ms,(depFor==16)?MF_CHECKED:MF_UNCHECKED);
  ctrlCommandKlatt.frame = depFor;
  resampleAll();
  affPPoints();
}

//
// Resampling of the polypoints.
//
void Wpiquefor::resampleAll()
{
  int i;
  for (i = 0; i < NBMAJEURS; i++) {
    Item *q;
    for (q = fpattern[i].list; q; q = q->next) q->val->echantillonne(depFor);
  }
}

//
// Selection ou deselection de TOUS les PolyPoints.
//
void Wpiquefor::selectTous (
  bool mode)    // false for unselection, true for selection
{
 int i;
 for (i = 0; i < NBMAJEURS; i++) {
    Item *q;
    for (q = fpattern[i].list; q; q = q->next) q->val->selectionne(mode);
 }
}

// la meme fonction mais pour etre utilisee en dehors de l'editeur de Klatt
void selectTous_(
  bool mode)    // false for unselection, true for selection
{
 int i;
 for (i = 0; i < NBMAJEURS; i++) {
    Item *q;
    for (q = fpattern[i].list; q; q = q->next) q->val->selectionne(mode);
 }
}

//
// Destruction de tous les PolyPoints existants.
//




void Wpiquefor::detruitTousPP()
{
  int i;
  selectTous(true);
  for (i = 0; i < NBMAJEURS; i++) fpattern[i].removeIfSelec();
}

void detruitTousPP_()
{
  int i;
  selectTous_(true);
  for (i = 0; i < NBMAJEURS; i++) fpattern[i].removeIfSelec();
}

bool Wpiquefor::isNoPPSel()
{
  int i;
  bool res = true;
  for (i = 0; i < NBMAJEURS && res; i++) {
    Item *q;
    for (q = fpattern[i].list; q && res ; q = q->next)
      res = !q->val->estSelec();
  }
  return res;
}

void Wpiquefor::CMdetruireFor()
{
  for (int i = 0; i < NBMAJEURS; i++) fpattern[i].removeIfSelec();
  affPPoints();
}


void Wpiquefor::CMrecalForRACLPC()
{
  souris(XC_watch);
  CMrecalFor(SRACLPC);
  souris(XC_top_left_arrow);
}

void Wpiquefor::CMrecalForLISCEP()
{
  souris(XC_watch);
  CMrecalFor(SLISCEP);
  souris(XC_top_left_arrow);
}


void Wpiquefor::CMlisseFor()
{
  int i;

  // Discard any modification on points
  ppointModif = (PolyPointsKla *) NULL;
  for (i = 0; i < NBMAJEURS; i++) {
    Item *q;
    for (q = fpattern[i].list; q; q = q->next)
      if (q->val->estSelec() && mineur(i, ::typeScale) != -1) {
        q->val->spline(mineur(i, ::typeScale));
        q->val->selectionne(false);
      }
  }
  affPPoints();
}

void Wpiquefor::CMremoveFirstPoint()
{
  int i;

  // Discard any modification on points
  ppointModif = (PolyPointsKla *) NULL;
  for (i = 0; i < NBMAJEURS; i++) {
    Item *q;
    for (q = fpattern[i].list; q; q = q->next)
      if (q->val->estSelec() && mineur(i, ::typeScale) != -1)
        q->val->removeFirstPoint();
  }
  affPPoints();
}

void Wpiquefor::CMremoveLastPoint()
{
  int i;

  // Discard any modification on points
  ppointModif = (PolyPointsKla *) NULL;
  for (i = 0; i < NBMAJEURS; i++) {
    Item *q;
    for (q = fpattern[i].list; q; q = q->next)
      if (q->val->estSelec() && mineur(i, ::typeScale) != -1)
        q->val->removeLastPoint();
  }
  affPPoints();
}

//
// Selection avec le menu d'un ou de plusieurs formants.
//
void Wpiquefor::selectFor(int param)
{
  if (param == FAUCUN) selectPPoint((PolyPointsKla *) NULL, 0);
  else for (int i = 0; i < NBMAJEURS; i++) {
    Item *q;
    for (q = fpattern[i].list; q; q = q->next)
      q->val->selectionne((param == i) || (param == FTOUS));
  }
  affPPoints();
}


void Wpiquefor::CMselectF1()
{  selectFor(F1);
}
void Wpiquefor::CMselectF2()
{  selectFor(F2);
}
void Wpiquefor::CMselectF3()
{  selectFor(F3);
}
void Wpiquefor::CMselectF4()
{  selectFor(F4);
}
void Wpiquefor::CMselectF5()
{  selectFor(F5);
}
void Wpiquefor::CMselectF6()
{  selectFor(F6);
}
void Wpiquefor::CMselectF1N()
{  selectFor(F1N);
}
void Wpiquefor::CMselectF2N()
{  selectFor(F2N);
}
void Wpiquefor::CMselectF3N()
{  selectFor(F3N);
}
void Wpiquefor::CMselectTous()
{  selectFor(FTOUS);
}
void Wpiquefor::CMselectAucun()
{  selectFor(FAUCUN);
}

void Wpiquefor::CMpas2ms()
{ depFor=2;
  CheckMenuItemPas();
}
void Wpiquefor::CMpas4ms()
{ depFor=4;
  CheckMenuItemPas();
}
void Wpiquefor::CMpas8ms()
{ depFor=8;
  CheckMenuItemPas();
}
void Wpiquefor::CMpas16ms()
{ depFor=16;
  CheckMenuItemPas();
}



void Wpiquefor::CMSauveKlatt()
{
  souris(XC_watch);

  souris(XC_top_left_arrow);
}

typedef HWND WINAPI (*HtmlHelpFunc)( HWND hwndCaller, LPCSTR pszFile,UINT uCommand, DWORD dwData );

const char pszHelpOCXCtrl [] = "hhctrl.ocx";
void Wpiquefor::CMhelpMouse()
{
  HINSTANCE hHelpOCX = ::LoadLibrary(pszHelpOCXCtrl);
  if (!hHelpOCX) {
    ::MessageBox(HWindow, "Cannot use the html help!", "Error", MB_OK);
   return;
  }

   HtmlHelpFunc HTMLHelp = (HtmlHelpFunc)::GetProcAddress(hHelpOCX, ATOM_HTMLHELP_API_ANSI);

  // Now call HTMLHelp functions:

  HTMLHelp(HWindow, helpfile, HH_DISPLAY_TOPIC, (DWORD) "mouseKlatt.htm");
}



void Wpiquefor::CMTools()
{
   if (!tools) {
     tools = new WToolsFormants(PTWtop, this);
     tools->Create();
   }
}

void piqueforClear()
{
  detruitTousPP_();
}

void Wpiquefor::CMSetParam()
{
  if (!yaUneFenetreSetParam){
    TWParamDef(PTWtop,"WParamDef").Execute();
  }
}

void Wpiquefor::CMopcldesc()
{

}

void Wtop::CMpiquefor()
{
  desactive(PPMENUAllStandard);
  active(MENUAffCoupe);
  active(MENUZoomin);
  active(MENUZoomout);
  active(MENUEcoute1Fois);
  active(MENUSuiviPicsCep);
  active(MENUSuiviPicsTrueEnv);
  active(MENUSuiviRacinesLPC);
  active(MENUconsDECO);
  active(MENUsuppDECO);
  desactive(MENUEditFormants);
  desactive(MENUEtiqExploreCorpus);
  desactive(MENUNoteExploreCorpus);
  ::DrawMenuBar(PTWtop->HWindow);

  yaUneFenetrePiquefor = true;
  GetApplication()->MakeWindow(wpiquefor = new Wpiquefor);
}


DEFINE_RESPONSE_TABLE1(TWpiqueforchoosef,TDialog)
  EV_CHILD_NOTIFY_ALL_CODES(IDOK,IDok),
  EV_CHILD_NOTIFY_ALL_CODES(IDCANCEL,IDcancel),
END_RESPONSE_TABLE;
TWpiqueforchoosef::TWpiqueforchoosef(PTWindowsObject AParent, LPSTR name)
	:TDialog(AParent,name)
{
  int i, j;
  int e = 2; // echelle pour affichage
  ::paramActif = -1;

  height = 20;
 
  // Creation d'un bouton de choix par parametre actif et pour l'echelle courante.
  for (i = 0, j = 0; i < NBMAJEURS; i++)
   if (etatParam[i] == ACTIF && nKlattScale[i][typeScale] != -1) {
    butPar[j] = new TRadioButton(this, ID_KLATT0 + i, nomParMajeursKlatt[i], 20, height,
    80, 15*e, 0);
 	 height += 16 * e;
    j++;
  }
  nParam = j;
  box = new TGroupBox(this, ID_KLATT0 - 1, ml(6421,"choisir un nom"), 4, 4, 112, (nParam * 16)*e+16, 0);
  height += 8;
  ok = new TButton(this, IDOK, "OK", 6, height, 64, 28, false, 0);
  cancel = new TButton(this, IDCANCEL, ml(6422,"&Annuler"), 76, height, 64, 28, false, 0);
  height += 62;
}

TWpiqueforchoosef::~TWpiqueforchoosef()
{
  int i;

  for (i = 0; i < nParam; i++) delete butPar[i];
  delete ok;
  delete cancel;
  delete box;
}

void TWpiqueforchoosef::SetupWindow()
{
  TDialog::SetupWindow();
  TRect rect = GetWindowRect();
  MoveWindow(rect.left, rect.top, 160, height, false);
  SetCaption(ml(6420,"nouveau paramtre"));
//  ::SetWindowText(GetItemHandle(ID_KLATT0 - 1),ml(6421,"choisir un nom"));
//  ::SetWindowText(GetItemHandle(IDCANCEL),ml(6422,"&Annuler"));
}

bool TWpiqueforchoosef::CanClose()
{
  return(true);
}
void TWpiqueforchoosef::IDok(WPARAM)
{
  int i;
  for (i = 0; i < nParam; i++)
    if (butPar[i]->GetCheck()==BF_CHECKED) ::paramActif = butPar[i]->GetId() - ID_KLATT0;
  CloseWindow(IDOK);
}

void TWpiqueforchoosef::IDcancel(WPARAM)
{
  paramActif = -1;
  CloseWindow(IDCANCEL);
}

float trouveDate(int ix, int iy, float *ordonnee)
{
  float x;
  snoiunscalexy(ix, iy, &x, ordonnee, wspectro->wsona);
  return(x);
}

/*
 * Domaine de valeur que peut prendre le parametre a l'ecran.
 */
float Wpiquefor::domyParam(
  int kpar)	/* parametre de Klatt (entre 1 et 40) */
{
  switch (kpar) {
  case F0hz10_: 
    return(BWMAX);
    break;
  case AVdb_: case ASP_: case TLTdb_: case AF_: case A1_: case A2_:
  case A3_: case A4_: case A5_: case A6_: case A1N_ : case A2N_ : case A3N_ :
  case ANP_: case AB_: case AVpdb_: case Gain0_:
    return(2*DBMAX);
    break;
  case F1hz_: case F2hz_: case F3hz_: case F4hz_: case F5hz_: case F6hz_: 
  case FNZhz_: case FNPhz_: case F1Nhz_ : case F2Nhz_ : case F3Nhz_ :
    return(fMaxSpectro);
    break;
  case B1hz_: case B2hz_: case B3hz_: case B4hz_: case B5hz_: case B6hz_ :
  case B1Nhz_ : case B2Nhz_ : case B3Nhz_ : case BNZhz_: case BNPhz_:
  case B1phz_: case B2phz_: case B3phz_: case B4phz_: case B5phz_: 
  case B6phz_: case B1Nphz_ : case B2Nphz_ : case B3Nphz_ :
    return(BWMAX);
    break;
  case Kopen_: case Aturb_: case Kskew_: case Ra_: case Rk_: case Rg_:
    return(100.);
    break;
  }
  return(1.);
}

/*
 * Trouve le PolyPoint auquel appartient un point, et en cas de succes le point
 * pointee par la souris si elle est suffisamment proche d'un point de la courbe.
 */

PolyPointsKla *Wpiquefor::trouvePtFor(
  int ipar,	/* parametre majeur. */
  int date,	/* en ms */
  float y,   // ordonnee exprimee dans la grandeur du polypoints (Hz, dB ...)
  int *indice,
  int focus,	/* parametre mineur. */
  int onForce)		/* FAUX pour chercher parmi les polypoints selectionnes, VRAI pour tous les polypoints. */
{
  PolyPointsKla *bingo;
  Item *p;
  float domyMax = domyParam(nKlatt[ipar][focus]);
  /* correction sur l'ordonnee pour calculer le seuil de proximite sur l'ecran. */
  float correc = (((float) enMilli(ct.f_a - ct.d_a + 1)) / ((float)wspectro->wsona->largeur)) /
		  (domyMax / ((float) wspectro->wsona->hauteur));

  float seuil =  5. * ((float) enMilli(ct.f_a - ct.d_a + 1)) / ((float)wspectro->wsona->largeur);

  for (p = fpattern[ipar].list, bingo = (PolyPointsKla *) NULL; !bingo && p; p = p->next)
    if (p->val->estSelec() || onForce) {
      int imin;
      float distCourbe, distPoint; // On conserve la distance minimale a la courbe et la
      					              // distance minimale aux points de la courbe.
      p->val->distance(date, y, focus, correc, seuil, &distPoint, &distCourbe, &imin);

      if (distCourbe < seuil) bingo = p->val;

      if (distPoint < seuil) *indice = imin;
      else *indice = -1;
    }
  return(bingo);
}


/*
 * Retourne le polyPoint auquel un point appartient.
 */
PolyPointsKla *Wpiquefor::trouvePt (
  int date,	/* point que l'on cherche.  date (ms) et ordonnee entiere dans la fenetre. */
  float ordonnee, /* coordonnee reelle dans la fenetre du spectro (entre 0 et NBCANAUX). */
  int *indice,
  int *param)	/* parametre majeur. */
{
  int i;
  PolyPointsKla *bingo;

  /* D'abord on cherche le point dans les polypoints selectionnes. */
  for (i = 0, bingo = (PolyPointsKla *) NULL; !bingo && i < NBMAJEURS; i++)
    if (etatParam[i] == ACTIF && mineur(i, ::typeScale) != -1) {
      int focus = mineur(i, ::typeScale);
      float y = yToValue(ordonnee, ::typeScale);
      bingo = trouvePtFor (i, date, y, indice, focus, FAUX);
      *param = i;
    }
  if (bingo) return(bingo);
  /* Et si on n'a pas trouve on le cherche parmi tous les polypoints. */
  for (i = 0; !bingo && i < NBMAJEURS; i++) 
    if (etatParam[i] == ACTIF && mineur(i, ::typeScale) != -1) {
      int focus = mineur(i, ::typeScale);
      float y = yToValue(ordonnee, ::typeScale);
      *param = i;
      bingo = trouvePtFor (i, date, y, indice, focus, VRAI);
    }
    
  return (bingo);
}

/*
 * Fin du deplacement des points dans un polypoints.
 * On reechantillonne le PolyPoints.
 */
void Wpiquefor::arretModifPP()
{
  selectPPoint((PolyPointsKla *) NULL, 0);
  if (focusModif == 0) { // dans les autres cas on ne peut que changer la valeur d'un
                    // point mais pas son abscisse donc il n'y a rien a faire.
    //fpattern[paramPtModif] = supprimePP(fpattern[paramPtModif], ppointModif);
    ppointModif->echantillonne(depFor);
  }
  affPPoints();
  ppointModif = (PolyPointsKla *) NULL;
}

/*
 * Mise a jour des polyPoints selectionnes.
 */
void Wpiquefor::selectPPoint(
  PolyPointsKla *ppoint,
  int mode)
{
  if (!mode || !ppoint) {
    int i;
    for (i = 0; i < NBMAJEURS; i++) {
      Item *q;
      for (q = fpattern[i].list; q; q = q->next) q->val->selectionne(false);
    }
  }
  if (ppoint) ppoint->selectionne(true);
}

void Wpiquefor::drawDownPiqPt(int event_x,int event_y)
{
  int indice, param, date;
  float ordonnee;
  PolyPointsKla *pp;
  
  date = enMilli(trouveDate(event_x, event_y, &ordonnee));
  pp = trouvePt(date, ordonnee, &indice, &param);

  if (!pp) {  // c'est la deselection d'un ou de plusieurs polypoints.
    if (ppointModif) arretModifPP();	// il y avait un PP en cours de modification.
    // boutton1Veille = FAUX;
    selectPPoint((PolyPointsKla *) NULL, 0);
    affPPoints();
  }
  else if (pp->estSelec() == false) { // c'est la selection d'un PolyPoint.
    // boutton1Veille = FAUX;
    selectPPoint(pp, 1);
    pp->selectionne(true);
    affPPoints();
  }
  else if (indice > -1) { // c'est le deplacement d'un point d'un PolyPoint selectionne.
    // boutton1Veille = VRAI;
    if (ppointModif && ppointModif != pp)  // on interdit la mise a jour simultanee de plusieurs PP.
      arretModifPP();
    else {
      ppointModif = pp;
      indicePtModif = indice;
      paramPtModif = param;
      yecranModif = valueToFcty(param, ::typeScale);
      focusModif = mineur(param, ::typeScale);
      xPtModif = wspectro->wsona->LXtoDX(enEcht(ppointModif->date(indice)));
      yPtModif = wspectro->wsona->LYtoDY(yecranModif(ppointModif->y(indice,focusModif)));
     if (indice > 0)
        event_x_g = maxi(0,
        						 wspectro->wsona->LXtoDX(maxi(ct.d_es,(float) enEcht(ppointModif->date(indice-1)))));
      else event_x_g = maxi(0, 2 * wspectro->wsona->LXtoDX(maxi(ct.d_es,(float)enEcht(ppointModif->date(indice))))
                              - wspectro->wsona->LXtoDX(maxi(ct.d_es,(float)enEcht(ppointModif->date(indice+1)))));
      if (indice < pp->nbPt - 1)
        event_x_d = mini(wspectro->wsona->largeur-1,
                         wspectro->wsona->LXtoDX(mini(ct.f_es,(float) enEcht(ppointModif->date(indice+1)))));
      else event_x_d = mini(wspectro->wsona->largeur-1,
                            2 * wspectro->wsona->LXtoDX(mini(ct.f_es,(float)enEcht(ppointModif->date(indice))))
                            - wspectro->wsona->LXtoDX(mini(ct.f_es,(float)enEcht(ppointModif->date(indice - 1)))));
    }
  }
}

void Wpiquefor::drawMotionPiqPt(int event_x,int event_y)
{
  HDC hDC;
  LOGPEN MyPen;
  HPEN hMyPen,holdpen;

 // if (!boutton1Veille || !ppointModif) return;
  if (!ppointModif) return;
  if (  (event_x <= event_x_g)
      ||(event_x >= event_x_d) ) return;

  hDC=GetDC(wspectro->wsona->HWindow);
  GetObject(GetStockObject(BLACK_PEN),sizeof(LOGPEN),&MyPen);
  hMyPen=CreatePenIndirect(&MyPen);
  holdpen=(HPEN)SelectObject(hDC,hMyPen);

  SetROP2(hDC,R2_NOT);

  // First the old point and segments connected to this point are erased
  InvertPointSel(hDC, xPtModif, yPtModif);
  if (indicePtModif) {
    MoveToEx(hDC, wspectro->wsona->LXtoDX(enEcht(ppointModif->date(indicePtModif - 1))),
		       wspectro->wsona->LYtoDY(yecranModif(ppointModif->y(indicePtModif - 1, focusModif))),(POINT *) NULL );
    LineTo(hDC, xPtModif, yPtModif);
  }

  if (indicePtModif < ppointModif->nbPt - 1) {
    MoveToEx(hDC, xPtModif, yPtModif, (POINT *) NULL);
    LineTo(hDC,wspectro->wsona->LXtoDX(enEcht(ppointModif->date(indicePtModif + 1))),
           wspectro->wsona->LYtoDY(yecranModif(ppointModif->y(indicePtModif + 1, focusModif))) );
  }

  // Then the new point is displayed.
  // Modifying date is not allowed if the parameter is not a major parameter.
  InvertPointSel(hDC, (focusModif == 0) ? event_x : xPtModif, event_y);

  if (indicePtModif) {
    MoveToEx(hDC,wspectro->wsona->LXtoDX(enEcht(ppointModif->date(indicePtModif - 1))),
			    wspectro->wsona->LYtoDY(yecranModif(ppointModif->y(indicePtModif - 1, focusModif))),(POINT *) NULL );
	 LineTo(hDC, ((focusModif == 0) ? event_x : xPtModif), event_y);
  }
  if (indicePtModif < ppointModif->nbPt - 1) {
    MoveToEx(hDC, ((focusModif == 0) ? event_x : xPtModif), event_y, (POINT *) NULL);
	 LineTo(hDC,wspectro->wsona->LXtoDX(enEcht(ppointModif->date(indicePtModif + 1))),
           wspectro->wsona->LYtoDY(yecranModif(ppointModif->y(indicePtModif + 1, focusModif))));
  }
  SetROP2(hDC,R2_COPYPEN);

  SelectObject(hDC,holdpen);
  DeleteObject(hMyPen);
  ReleaseDC(wspectro->wsona->HWindow,hDC);

  xPtModif = (focusModif == 0) ? event_x : xPtModif;
  yPtModif = event_y;
}

void Wpiquefor::drawUpPiqPt(int, int)
{
//  if (captureForEncours) drawUpPiqFor(widget, event, args, num_args);
//  else
//  if (!boutton1Veille || !ppointModif) return; else
	if (!ppointModif) return;
   else {
    int date;
    float ordonnee;
    date = enMilli(trouveDate(xPtModif,  yPtModif, &ordonnee));
    if (focusModif == 0) *(ppointModif->pdate(indicePtModif)) = date;
    *(ppointModif->py(indicePtModif, focusModif)) = yToValue(ordonnee, ::typeScale);
    affPPoints();
  }
}

void Wpiquefor::drawDown(int event_x,int event_y) {
  wpiquefor->get_ppp()->drawDown(event_x, event_y);
}

void Wpiquefor::drawMotion(int event_x,int event_y) {
  wpiquefor->get_ppp()->drawMotion(event_x, event_y);
}

void Wpiquefor::CMmergeFor()
{
  fusionnePP();
  affPPoints();
}

//
//  Fusion de plusieurs polypoints.
//
void Wpiquefor::fusionnePP()
{
  int i;
  // First, we discard any polypoint modification
  ppointModif = (PolyPointsKla *) NULL;
  for (i = 0; i < NBMAJEURS; i++) {
    Item *pp;
    PolyPointsKla *npp;
    int nbPP;
    /* On saute les premiers pp non selectionnes. */
    for (pp = fpattern[i].list; pp && !pp->val->estSelec(); pp = pp->next);

    npp = newPPd(dim[i]);
    for (nbPP = 0; pp && pp->val->estSelec(); pp = pp->next) {
      int j;
      nbPP++;
      for (j = 0; j < pp->val->nbPt; j++) {
        float t[LARGBANDEP - FREQUENCE + 1];
        for (int k = 0; k < dim[i] - 1; k++) t[k] = pp->val->y(j,k);
        npp->push(pp->val->date(j), t);
      }
    }
    if (nbPP > 1) {	/* Il doit effectivement y avoir fusion. */
      npp->echantillonne(depFor);
      fpattern[i].removeIfSelec();  // suppression des polypoints a fusionner.
      fpattern[i].insertSorted(npp);
    }
    else delete npp;
  }
}


//
// Resolution des conflits crees par le polypoints qui vient d'etre dessine.
//
void Wpiquefor::resolConflits(
  int iKlatt)	/* parametre de Klatt (entre 1 et 40) */
{
  Item *pp;
  int debNvpp, finNvpp;
  int mi, mj, dim;

  mj = majeurMineur(iKlatt, &mi, &dim);
  if (!(*ppp).ni) return;
  debNvpp = (*ppp)[0].date;
  finNvpp =  (*ppp)[(*ppp).ni - 1].date;
  /* D'abord on resoud les conflits en
     modifiant la frequence, la largeur de bande ou l'amplitude 
     d'un ou de plusieurs polypoints. */
  for (pp = fpattern[mj].list; pp; pp = pp->next) {
    int debpp = pp->val->debut(),
      finpp = pp->val->fin();

    if (dans(debNvpp, debpp, finpp) && dans(finNvpp, debpp, finpp)) { /* inclusion du nvpp dans pp. */
      int i, j;
      j = pp->val->indPP((*ppp)[0].date);

      if (j != -1) for (i = 0; i < (*ppp).ni; i++, j++) {
	     *(pp->val->py(j, mi)) = (*ppp)[i].y[0];
	     (*ppp)[i].y[0] = -INFINI_I;		/* on marque les points qui ont ete utilises pour les corrections. */
      }
    }
    else if (debNvpp < finpp && finpp < finNvpp) {	/* recouvrement sur la fin du pp. */
      int i = (debNvpp < debpp) ? (*ppp).indPP(pp->val->debut()) : 0,  /* dans (*ppp) */
	  j = (debNvpp < debpp) ? 0 : pp->val->indPP((*ppp)[0].date); /* dans pp */

      if (j != -1 && i != -1) for (; i < (*ppp).ni && j < pp->val->nbPt; i++, j++) {
        *(pp->val->py(j, mi)) = (*ppp)[i].y[0];
	     (*ppp)[i].y[0] = -INFINI_I;		/* on marque les points qui ont ete utilises pour les corrections. */
      }
    }
    else if (debNvpp < debpp && debpp < finNvpp) {   /* recouvrement sur le debut du pp. */
      int i = (*ppp).indPP(pp->val->debut()), j;
      if (i != -1) for (j = 0;  i < (*ppp).ni && j < pp->val->nbPt; i++, j++) {
        *(pp->val->py(j, mi)) = (*ppp)[i].y[0];
	     (*ppp)[i].y[0] = -INFINI_I;		/* on marque les points qui ont ete utilises pour les corrections. */
      }
    }
  }
}

TabPoint<Point2P>*  Wpiquefor::extractOneNewPP()
{
   /* on vient de resoudre les conflits avec le nouveau polypoints
      maintenant on sauvegarde les points qui n'ont pas servi
      aux corrections. */
  int i, nbPoints;

  TabPoint<Point2P> *sousppp = new TabPoint<Point2P>(0);

  for (i = 0; i < (*ppp).ni && (*ppp)[i].y[0] < -INFINI_I / 2.; i++); /* on saute les premiers points nuls. */

  /* on sauve les points qui restent valides. */
  nbPoints = 0;

  for (; i < (*ppp).ni && (*ppp)[i].y[0] > -INFINI_I / 2.; i++, nbPoints++) {
    sousppp->TabDessin = (Point2P *) realloc((void *)sousppp->TabDessin, (nbPoints + 1) * sizeof(Point2P));
    sousppp->TabDessin[nbPoints].date = (*ppp)[i].date;
    sousppp->TabDessin[nbPoints].y[0] = (*ppp)[i].y[0];
    (*ppp)[i].y[0] = -INFINI_I;
  }
  sousppp->p = sousppp->TabDessin;
  sousppp->ni = nbPoints;
  sousppp->TabDessin = (Point2P *) NULL;
  sousppp->nid = 0;

  return(sousppp);
}

//
// Validates (*ppp) as a new Polypoints
//
void Wpiquefor::validatePPP()
{
  int mi, dimension, souspppNbPoints = 0;
  TabPoint<Point2P> *sousppp;
  PolyPointsKla *npp;
  int focus = mineur(::paramActif, ::typeScale);

  // First, copy the value of points modified by the new polypoints entered.
  resolConflits(nKlattScale[::paramActif][::typeScale]);

  majeurMineur(nKlattScale[::paramActif][::typeScale], &mi, &dimension);
  if (mi != 0) (*ppp).newsize(0);

  // Then save in new polypoints with points left.
  do {
    sousppp = extractOneNewPP();
    souspppNbPoints = sousppp->ni;
    if (sousppp->ni) {
      switch (dim[::paramActif]) {
      case 2:
        npp = new PolyPointsMD<TabPoint <Point2P>, Point2P >(*sousppp);
      break;
      case 3:
        npp = new PolyPointsMD<TabPoint <Point3P>, Point3P >(*sousppp, focus);
      break;
      case 4:
        npp = new PolyPointsMD<TabPoint <Point4P>, Point4P >(*sousppp, focus);
      break;
      case 5:
        npp = new PolyPointsMD<TabPoint <Point5P>, Point5P >(*sousppp, focus);
      break;
      }
      delete sousppp;
      npp->setDefaultValuesKla(nKlattScale[::paramActif][::typeScale], focus);
      fpattern[::paramActif].insertSorted(npp);
    }
  } while (souspppNbPoints);
}

void Wpiquefor::drawUp(int event_x,int event_y) {
  wpiquefor->get_ppp()->drawUp(event_x, event_y);
  int r = TWpiqueforchoosef(PTWtop,"Wpiqueforchoosef").Execute();
  if (r == IDOK && ::paramActif != -1) {
    for (int j = 0; j < (*ppp).ni; j++) {
      (*ppp)[j].y[0] = yToValue((*ppp)[j].y[0], ::typeScale);
      (*ppp)[j].date = enMilli((*ppp)[j].date);
    }
    (*ppp).echantillonnePP(depFor);
    validatePPP();
  }
  affPPoints();
}


void Wpiquefor::affPPoints()
{  
  /* D'abord on efface tous les PolyPoints affiches. */
  wspectro->wsona->spectro_nettoyer(0,INFINI_I);

  /* On Affiche l'echelle */
  switch (typeScale) {
  case SCALEFr :
    affEchelle(0., fMaxSpectro, wspectro->wsona->incr, "Hz");
    break;
  case SCALEBwC :
    affEchelle(0., BWMAX, 100., "Bw cascade");
    break;
  case SCALEBwP :
    affEchelle(0., BWMAX, 100., "Bw parallel");
    break;
  case SCALEDBC :
    affEchelle(DBMIN, DBMAX, 20., "dB cascade");
    break;
  case SCALEDBP :
    affEchelle(DBMIN, DBMAX, 20., "dB parallel");
    break;
  case SCALEDNb :
    affEchelle(0., 100., 10., "Numeral");
    break;
  case SCALEF0 :
    affEchelle(0., 800., 100., "Hz(F0)");
    break;
  } 

  for (int i = 1; i < NPARLE; i++) {
    int mi, mj, dim;
    mj = majeurMineur(i, &mi, &dim);
    if (etatParam[mj] != CACHE && scalePar[i] == ::typeScale) {
      YFCT yecran = valueToFcty(mj, typeScale);
      int ligne = (etatParam[mj] == LIGNE);

      for (Item *q = fpattern[mj].list; q; q = q->next)
	     if (dans(q->val->debut(), enMilli(ct.d_a), enMilli(ct.f_a))
	    || dans(q->val->fin(), enMilli(ct.d_a), enMilli(ct.f_a))
	    || dans(enMilli(ct.d_a), q->val->debut(), q->val->fin())
	    || dans(enMilli(ct.f_a), q->val->debut(), q->val->fin()))
       q->val->aff(mi, yecran, q->val->estSelec(), ::typeAff, couleurParam(mj), ligne);
    }
  }
  ::InvalidateRect(wspectro->wsona->HWindow,NULL,true);
}

void Wpiquefor::affEchelle(
  float vmin,
  float vmax,
  float incr,
  char *texte)
{
  wspectro->wvscale->affichage(vmin, vmax, incr, texte);
}


/*
 * Calcul du parametre mineur en fonction du parametre majeur
 * et du type d'echelle.
 */
static int mineur(
  int majeur,
  int scale)
{
  switch(majeur) {
  case ASPIRATION: case ATURB: case TLTDB: case AFRICTION:
  case ABYPASS:  case GAIN:
    if (scale == SCALEDBC) return(0);
    break;
  case KOPEN: case KSKEW: case RaLF: case RkLF: case RgLF:
    if (scale == SCALEDNb) return(0);
    break;
  case F0:
    switch(scale) {
    case SCALEF0: return(FREQUENCE);
    case SCALEDBP: return(AMPLITUDEF0P);
    case SCALEDBC: return(AMPLITUDE);
    }
  case FNZ:
    switch(scale) {
    case SCALEFr: return(FREQUENCE);
    case SCALEBwC: return(1);
    }
  case FNP:
    switch(scale) {
    case SCALEFr: return(FREQUENCE);
    case SCALEDBP: return(AMPLITUDE);
    case SCALEBwC: return(LARGBANDEC);
    }
  case F1: case F2: case F3: case F4: case F5: case F6:case F1N :
  case F2N : case F3N :
    switch(scale) {
    case SCALEFr: return(FREQUENCE);
    case SCALEBwC: return(LARGBANDEC);
    case SCALEBwP: return(LARGBANDEP);
    case SCALEDBP: return(AMPLITUDE);
    }
  }
  return(-1);
}

/*
 * Indice du polypoint et de la coordonnee du parametre dans le polypoints, 
 * et dimension du polypoints associe.
 */
int majeurMineur(
  int ipar,
  int *mineur,
  int *dim)
{
  switch (ipar) {
  case Gain0_: case AF_: case  ASP_: case  Kopen_: case  Aturb_: case  Kskew_:
  case  AB_: case TLTdb_: case  F0hz10_: case  F1hz_: case  F2hz_: case  F3hz_: 
  case  F4hz_: case  F5hz_: case  F6hz_: case  FNZhz_: case  FNPhz_: case Ra_:
  case Rk_: case Rg_: case F1Nhz_ : case F2Nhz_ : case F3Nhz_ :
    *mineur = 0;
    break;
  case AVdb_: case  A1_: case  A2_: case  A3_: case  A4_: case  A5_: 
  case  A6_: case  ANP_: case  BNZhz_: case A1N_ : case A2N_ : case A3N_ :
    *mineur = 1;
    break;
  case B1hz_: case  B2hz_: case B3hz_: case B4hz_: case B5hz_: case B6hz_: 
  case BNPhz_: case  AVpdb_: case B1Nhz_ : case B2Nhz_ : case B3Nhz_ : 
    *mineur = 2;
    break;
  case B1phz_: case  B2phz_: case  B3phz_: case  B4phz_: case  B5phz_: 
  case  B6phz_: case B1Nphz_ : case B2Nphz_ : case B3Nphz_ :
    *mineur = 3;
    break;
  }
  switch (ipar) {
  case F0hz10_: case  AVdb_: case  AVpdb_:
    *dim = 4;
    return(F0);
  case F1hz_: case  A1_: case  B1hz_: case  B1phz_:
    *dim = 5;
    return(F1);
 case F2hz_: case  A2_: case  B2hz_: case  B2phz_:
    *dim = 5;
    return(F2);
  case F3hz_: case  A3_: case  B3hz_: case  B3phz_:
    *dim = 5;
    return(F3);
  case F4hz_: case  A4_: case  B4hz_: case  B4phz_:
    *dim = 5;
    return(F4);
  case F5hz_: case  A5_: case  B5hz_: case  B5phz_:
    *dim = 5;
    return(F5);
  case F6hz_: case  A6_: case  B6hz_: case  B6phz_:
    *dim = 5;
    return(F6);

    case F1Nhz_: case  A1N_: case  B1Nhz_: case  B1Nphz_:
    *dim = 5;
    return(F1N);
      case F2Nhz_: case  A2N_: case  B2Nhz_: case  B2Nphz_:
    *dim = 5;
    return(F2N);
      case F3Nhz_: case  A3N_: case  B3Nhz_: case  B3Nphz_:
    *dim = 5;
    return(F3N);

  case FNZhz_: case  BNZhz_:
    *dim = 3;
    return(FNZ);
  case FNPhz_: case  BNPhz_: case  ANP_:
    *dim = 4;
    return(FNP);
  case ASP_:
    *dim = 2;
    return(ASPIRATION);
  case TLTdb_:
    *dim = 2;
    return(TLTDB);
  case AF_:
    *dim = 2;
    return(AFRICTION);
  case Gain0_:
    *dim = 2;
    return(GAIN);
  case Kopen_:
    *dim = 2;
    return(KOPEN);
  case Aturb_:
    *dim = 2;
    return(ATURB);
  case Kskew_:
    *dim = 2;
    return(KSKEW);
  case AB_:
    *dim = 2;
    return(ABYPASS);
  case Ra_:
    *dim = 2;
    return(RaLF);
   case Rk_:
    *dim = 2;
    return(RkLF);
   case Rg_:
    *dim = 2;
    return(RgLF);
   }
  return(-1);
}

/*
 * Indique la couleur d'un parametre.
 */
COLORREF couleurParam(
  int param)
{
  switch (param) {
  case F0: return(JAUNEsombre);
  case F1: return(ROUGE);
  case F2: return(MAGENTA);
  case F3: return(BLEU);
  case F4: return(VERT);
  case F5: return(JAUNE);
  case F6: return(CYAN);
  case FNZ: return(VERTsombre);
  case FNP: return(ROUGEsombre);
  case ASPIRATION: return(JAUNEsombre);
  case KOPEN: return(CYANsombre);
  case ATURB: return(BLEUsombre);
  case TLTDB: return(MAGENTAsombre);
  case AFRICTION: return(MAGENTA);
  case KSKEW: return(BLEU);
  case ABYPASS: return(VERT);
  case GAIN: return(CYAN);
  case RaLF: return(DARKORANGE);
  case RkLF: return(GRISsombre);
  case RgLF: return(VERT);
  case F1N: return(DARKORANGE);
  case F2N: return(MAGENTAsombre);
  case F3N: return(BLEUsombre);
  }
  return(ROUGE);
}

/*
 * Passage de F0 -> canaux
 */
float f0ToEcr(float f)
{
  return(f / F0KLATTMAX * NB_CANAUX);
}

/*
 * Passage reperage spectro -> amplitude en dB.
 */
float ecrToFo (float f)
{
    return (f / NB_CANAUX * F0KLATTMAX);
}

/*
 * Passage d'un nombre -> canaux
 */
float nbToEcr(float f)
{
  return(f / NPURMAX * NB_CANAUX);
}

/*
 * Passage reperage spectro -> amplitude en dB.
 */
float ecrToNb (float f)
{
    return (f / NB_CANAUX * NPURMAX);
}


/*
 * Passage largeur de bande en Hz --> canaux.
 */

float bwToEcr (float f)

{
    return (f / BWMAX * NB_CANAUX);
}

/*
 * Passage largeur de bande en Hz --> canaux.
 */

float ecrToBw (float f)
{
    return (f / NB_CANAUX * BWMAX);
}

/*
 * Passage amplitude en dB --> canaux.
 */
float dbToEcr (float f)
{
    return ((f - DBMIN)/ (DBMAX - DBMIN) * NB_CANAUX);
}

/*
 * Passage reperage spectro -> amplitude en dB.
 */
float ecrToDb (float f)
{
    return (f / NB_CANAUX * (DBMAX -DBMIN) + DBMIN);
}

/*
 * Retourne la fonction de conversion des donnees du parametres vers
 * les coordonnees de la fenetre.
 */
YFCT Wpiquefor::valueToFcty(
  int majeur,
  int scale)
{
  YFCT fct;
  switch (scale) {
  case SCALEFr: fct = freqToEcr; break;
  case SCALEBwC: case SCALEBwP: fct = bwToEcr; break;
  case SCALEDBC: case SCALEDBP: fct = dbToEcr; break;
  case SCALEF0: fct = f0ToEcr; break;
  case SCALEDNb: fct = nbToEcr; break;
  default: fct = (YFCT) NULL;
  }
  return(fct);
}

float yToValue(
  float y,
  int scale)
{
  switch (scale) {
  case SCALEFr: return(canToHz(y));
  case SCALEBwC: case SCALEBwP: return(ecrToBw(y));
  case SCALEDBC: case SCALEDBP: return(ecrToDb(y));
  case SCALEF0: return(ecrToFo(y));
  case SCALEDNb: return(ecrToNb(y));
  }
  return(0);
}


void translatePPselectionne(int incr)
{
  for (int i = 0; i < NBMAJEURS; i++) {
   Item *q;
   for (q = fpattern[i].list; q; q = q->next)
      if (q->val->estSelec() && mineur(i, ::typeScale) != -1)
        q->val->translateKla(i, incr);
  }
}

// Tests whether at least one parameter is not null.
int Wpiquefor::oupp(
  Item *ppfi[])
{
  int res;
  int i;
  for (i = 0, res = 0; i < NBMAJEURS; i++)
    res = (res || (ppfi[i] != (Item *) NULL));
  return(res);
}

//  returns the time (in milliseonds) at which the first polypoints begins.
//  If force is false, only the polypoints selected are considered.
int Wpiquefor::datePremPP(bool force)
{
  int i, dateMin;

  for (i = 0, dateMin = INFINI_I; i < NBMAJEURS; i++) {
    Item *q;
    for (q = fpattern[i].list; q && (!q->val->estSelec() && !force); q = q->next);
    dateMin =  (q && q->val->debut() < dateMin) ? q->val->debut() : dateMin;
  }
  return(dateMin);
}

//  returns the time (in milliseonds) at which last polypoint ends.
//  If force is false, only the polypoints selected are considered.
int Wpiquefor::dateLastEndPP(bool force)
{
  int i, dateMax;

  for (i = 0, dateMax = -INFINI_I; i < NBMAJEURS; i++) {
    Item *q;
    for (q = fpattern[i].list; q; q = q->next) {
      if ((q->val->estSelec() || force) && q->val->fin() > dateMax)
        dateMax = q->val->fin();
    }
  }
  return(dateMax);
}

/*
 * Sauvegarde des polypoints correspondants aux formants.
 * Attention, on suppose que l'utilisateur ne fait pas n'importe quoi, et qu'il
 * n'y a pas plus d'un polypoint par formant. Si c'est le cas, Snoori
 * ne sauvegarde que le premier des polypoints en conflit.
 * Returns the date (in ms) of the last frame saved.
 */

int Wpiquefor::sauvePP (
  char *filename,
  int debutEcht,	/* echantillons de debut et de fin de sauvegarde (0 et ct.tf si on veut tout sauver) */
  int finEcht,	/* echantillons de debut et de fin de sauvegarde (0 et ct.tf si on veut tout sauver) */
  bool force,  /* true si on force la sauvegarde meme si les PolyPoints ne sont pas selectionnes. */
  int klatt,	/* 0 si c'est une sauvegarde normale,
		   1 si c'est pour le synthesiseur de Klatt,
		   2 si c'est pour le synthe de Klat modifie,
		   3 si c'est le squelette des formants. Dans ce cas on
		   sauve 1 si le formant ou la grandeur est selectionnee, O sinon. */
  int Npar) // NPAR when the LF source is used, NPARKLATT otherwise.
{
  FILE *f;
  Item *ppfi[NBMAJEURS];
  int i, date, dateMin = INFINI_I, indfi[NBMAJEURS],
    debut = enMilli(debutEcht), fin = enMilli(finEcht);
  int *defFrame = (int *) (&defaultFrame);

  /* On recherche la date du premier point a sauver, car la date de debut de sauvegarde
     peut etre differente de la date de debut du polypoint representant un formant. */
  if (!debut) for (i = 0; i < NBMAJEURS; i++) {
    Item *q;
    for (q = fpattern[i].list; q && (!q->val->estSelec() && !force); q = q->next);
    ppfi[i] = q;
    dateMin =  (q && q->val->debut() < dateMin) ? q->val->debut() : dateMin;
    indfi[i] = 0;
  }
  else for (i = 0; i < NBMAJEURS; i++) {
    Item *q;
    for (q = fpattern[i].list; q && (q->val->fin() <= debut || (!q->val->estSelec() && !force));
         q = q->next);
    ppfi[i] = q;
    if (q) {
      int j;
      for (j = 0; j < q->val->nbPt && q->val->date(j) < debut; j++);
      indfi[i] = (j > 0) ? j - 1 : j;
      dateMin = (q->val->date(indfi[i]) < dateMin) ? q->val->date(indfi[i]) : dateMin;
    }
  }

  f = fopen(filename, "w");
  if (!f) {
    affiche_erreur(ml(6403,"sauvePP : sauvegarde des formants impossible"));
    return 0;
  }

  if (klatt != 5) {
    if (Npar == NPARKLATT) fprintf(f,".kla\n");         // write the magic number
    else fprintf(f, ".kle\n");
  }

  if (klatt == 4) {  /* pour le calcul de l'energie. */
    cF0 = min(IROUND(freqToEcr(1000.)), NB_CANAUX - 1);
    cbas =  min(IROUND(freqToEcr(2000.)), NB_CANAUX - 1);
    chaut = min(IROUND(freqToEcr(6000.)), NB_CANAUX - 1);
  }
  /* Echantillonnage des formants toutes les depFor ms */
  for (date = dateMin - (dateMin % depFor); date < fin && oupp(ppfi); date += depFor) {
    klatt_frame_t fframe;
    int *frame = (int *) (&fframe);

    frame[Date_] = date;
    for (i = 0; i < NBMAJEURS; i++) {
      // There is no formant track for this formant or the date is before the first track of this formant
      if (!ppfi[i] || date < ppfi[i]->val->debut()) {
	     for (int j = 0; j <= LARGBANDEP; j++) if (nKlatt[i][j] != -1)
          // In this case the formant was not defined, default values are used.
	       frame[nKlatt[i][j]] = (klatt == 3) ? 0 : defFrame[nKlatt[i][j]];
      }
      else {
	     /* On recherche les points qui encadrent la date d'echantillonnage. */
	     while (indfi[i] < ppfi[i]->val->nbPt - 1 && date > ppfi[i]->val->date(indfi[i]+1)) indfi[i]++;
        if (indfi[i] == (ppfi[i]->val->nbPt - 1)) {
	       /* si le dernier point apparait tres peu de temps apres la
	          date on le conserve. */
          if (date < ppfi[i]->val->fin() + depFor / 3) {
            for (int j = 0; j <= LARGBANDEP; j++) if (nKlatt[i][j] != -1)
	           frame[nKlatt[i][j]] = (klatt == 3) ? 1 : ppfi[i]->val->y(ppfi[i]->val->nbPt - 1, j);
	       }
	       else {
	         // In this case the formant is just finished, default values are used.
	         for (int j = 0; j <= LARGBANDEP; j++) if (nKlatt[i][j] != -1)
	           frame[nKlatt[i][j]] = (klatt == 3) ? 0 : defFrame[nKlatt[i][j]];
	       }
	       ppfi[i] = ppfi[i]->next;
          if (!force) for (; ppfi[i] && !ppfi[i]->val->estSelec(); ppfi[i] = ppfi[i]->next);
	       indfi[i] = 0;
	     }
	     else { /* Cas banal ou il faut au pire interpoler */
	       if (date == ppfi[i]->val->date(indfi[i])) {
	         for (int j = 0; j <= LARGBANDEP; j++) if (nKlatt[i][j] != -1)
	           frame[nKlatt[i][j]] = (klatt == 3) ? 1 : ppfi[i]->val->y(indfi[i], j);
	       }
	       else {
	         float r = ((float)(date - ppfi[i]->val->date(indfi[i]))) /
                      ((float)(ppfi[i]->val->date(indfi[i]+1) - ppfi[i]->val->date(indfi[i])));
            for (int j = 0; j <= LARGBANDEP; j++) if (nKlatt[i][j] != -1)
	           frame[nKlatt[i][j]] = (klatt == 3) ? 1 :
	                                 (ppfi[i]->val->y(indfi[i], j)
                                   + r * (ppfi[i]->val->y(indfi[i]+1, j) - ppfi[i]->val->y(indfi[i], j)));
	       }
	     }
      }
    }
    if (!klatt || klatt == 3) sauveFrameKlatt(f, fframe, Npar);
    else if (klatt == 5) {
      int i;
      fprintf(f,"date %ld (%ld ms) ",
	      date, enMilli(fframe.Date));
      for (i = F1; i <= F6; i++)
        fprintf(f, "F%1d=%.5g %.5g %.5g ", i,
              (float) frame[nKlatt[i][FREQUENCE]],
              (float) frame[nKlatt[i][AMPLITUDE]],
              (float) frame[nKlatt[i][LARGBANDEP]]);
      fprintf(f, "\n");
    }
    else if (klatt == 4) { /* On fixe le parametre de bruit de friction
			                   pour calculer le signal d'excitation. */
        if (fframe.F0hz10 < 10) {
	       fframe.AF = defaultFrame.AVpdb;
	       fframe.AVpdb = defaultFrame.AVpdb;
        }
        else {
          float lowFreqEnergy, highFreqEnergy, logb;
          float voicingRatio = voicingFricationRatio(fframe.F0hz10, max(enEcht(date - correlationSizeMs / 2), 0),
        														  lowFreqEnergy, highFreqEnergy);
          logb = defaultFrame.AVpdb - 10. * log10(lowFreqEnergy + highFreqEnergy + EPSILON);
          fframe.AVpdb = logb + 10. * log10(lowFreqEnergy + voicingRatio * highFreqEnergy + EPSILON);
          fframe.AF = logb + 10. * log10((1. - voicingRatio) * highFreqEnergy + EPSILON);
        }
        sauveFrameKlatt(f, fframe, Npar);
    }
  }
  fclose(f);
  return(date);
}


/*
 *  Initialization of the filters used to compute the frication/voicing ratio.
 */
void Wpiquefor::initFilters()
{
  (*lowFreqFilter).newsize(41);
  (*lowFreqFilter).att = 30.;
  strcpy((*lowFreqFilter).type, "lp");
  (*lowFreqFilter).fc = 1000. / en_cours.frequence;
  desfir(&(*lowFreqFilter));

  (*highFreqFilter).newsize(41);
  (*highFreqFilter).att = 30.;
  strcpy((*highFreqFilter).type, "bp");
  (*highFreqFilter).fl = min(2000., en_cours.frequence) / en_cours.frequence;
  (*highFreqFilter).fh = min(4000., en_cours.frequence) / en_cours.frequence;
  desfir(&(*highFreqFilter));
}

/*
 * Calcul de la part de l'energie au-dessus de 4000 Hz (qui correspond
 * essentiellement a de la friction) sur l'energie totale.
 */
void Wpiquefor::calculEnerFric(
  SAMPLEbuf& parole,
  int echt,
  float &lowFreqEnergy,
  float &highFreqEnergy)
{
  int c, i, j;
  FLOATbuf spectre(512,"calcul energie dans calculEnerfric");
  float energieb, energieh;

  for (j = 0; j < 128; j++) spectre[j] = 0.;
  // Calcul sur 4 spectres calculs sur des fentres voisines.
  for (i = 0; i < 4; i++) {
    FLOATbuf nspectre(512,"nspectre : calcul energie dans calculEnerfric");
    specFft(parole, echt + i * 64, (!i) ? spectre : nspectre, true, false);
    if (i) {
      for (j = 0; j < cF0; j++) spectre[j] = max(nspectre[j], spectre[j]);
      for (j = cbas; j <= chaut; j++) spectre[j] += nspectre[j];
    }
  }
  for (i = cbas; i <= chaut; i++) spectre[i] /= 4;
  for (c = 0, energieb = 0.; c <= cbas; c++)
    energieb += max(spectre[c] - (SILENCE_SPECTRO - 10.), 0);
  for (c = cbas, energieh = 0.; c <= chaut; c++)
    energieh += max(spectre[c] - (SILENCE_SPECTRO - 10.), 0);
  lowFreqEnergy = energieb;
  highFreqEnergy = energieh;
}


/*
 * Computes the best autocorrelation in the vicinity of F0.
 */
float Wpiquefor::voicingRatio(
  SAMPLEbuf &speech,
  float f0,
  int windowSize,
  int echt)
{
  int T0, shift;
  float bestAutoCorrelation = -INFINI_F;

  T0 = max(2, IROUND(en_cours.frequence / f0));
  for (shift = T0 - 10; shift <= T0 + 10; shift++) {
    int i;
    float s, sx, sy;
    for (i = 0, s = 0., sx = 0., sy = 0.; i < windowSize; i++) {
      s += speech[i + echt] * speech[i + echt + shift];
      sx += speech[i + echt] * speech[i + echt];
      sy += speech[i + echt + shift] * speech[i + echt + shift];
    }
    if (fabs(sx) > EPSILON && fabs(sy) > EPSILON) {
      float x = s / (sqrt(sx) * sqrt(sy));
      bestAutoCorrelation = (x > bestAutoCorrelation) ? x : bestAutoCorrelation;
    }
  }
  return((bestAutoCorrelation > 0.) ? bestAutoCorrelation : 0.);
}

/*
 * Computes the normalized autocorrelation of the signal between 2000 and 4000Hz.
 * The autocorrelation is computed for F0, a weak autocorrelation thus means that the signal
 * is noised.
 */
float Wpiquefor::voicingFricationRatio(
  float f0,
  int echt,
  float &lowFreqEnergy,
  float &highFreqEnergy)
{
  if (echt > (ct.tf - FENFRIC)) return(0);

  initSpecFft(512, 512); // ATTENTION il faudra rgler la fentre et l'ordre.
  /* Si le signal est disponible il suffit de calculer enerFric */
  if (dans(echt, ctEner.d_e, ctEner.f_e)
      && dans(echt + FENFRIC, ctEner.d_e, ctEner.f_e)) {
    calculEnerFric(*space_ener, echt - ctEner.d_e, lowFreqEnergy, highFreqEnergy);
    return voicingRatio((*space_highFreq), f0, enEcht(correlationSizeMs), echt - ctEner.d_e);
  }
  else { /* Il faut relire du signal. */
    int nbEchtMax = NB_SPECTRES_SUIVI * enEcht(depFor);

    ctEner = ct;
    ctEner.d_es = ctEner.d_a = ctEner.d_e = echt;
    ctEner.f_es = ctEner.f_a = ctEner.f_e = min(ctEner.tf, ctEner.d_e + min(MAXECH / 6, nbEchtMax) - 1);
    (*space_lowFreq).newsize(ctEner.f_e - ctEner.d_e + 1 + 1);
    (*space_highFreq).newsize(ctEner.f_e - ctEner.d_e + 1 + 1);
    space_ener->newsize(ctEner.f_e - ctEner.d_e + 1 + 1);
    lireSignal(en_cours, en_cours.depl + sizeof(SAMPLE) * ctEner.d_e, (*space_lowFreq), 0,
	       ctEner.f_e - ctEner.d_e + 1);
    lireSignal(en_cours, en_cours.depl + sizeof(SAMPLE) * ctEner.d_e, (*space_highFreq), 0,
	       ctEner.f_e - ctEner.d_e + 1);
    lireSignal(en_cours, en_cours.depl + sizeof(SAMPLE) * ctEner.d_e, *space_ener, 0,
	       ctEner.f_e - ctEner.d_e + 1);

    filtrer((*space_lowFreq), (*space_lowFreq), 0, ctEner.f_e - ctEner.d_e, (*lowFreqFilter));
    filtrer((*space_highFreq), (*space_highFreq), 0, ctEner.f_e - ctEner.d_e, (*highFreqFilter));
/*   to check weather the filered signal was ok
    {
      FILE *f = fopen("trucHigh.raw","wb");
      for (int i = 0; i < ctEner.f_e - ctEner.d_e; i++) fwrite(&(*space_highFreq)[i], sizeof(SAMPLE), 1, f);
      fclose(f);
    }
*/
    calculEnerFric(*space_ener, echt - ctEner.d_e, lowFreqEnergy, highFreqEnergy);
    return voicingRatio((*space_highFreq), f0, enEcht(correlationSizeMs), echt - ctEner.d_e);
  }
}

void Wpiquefor::sauveFrameKlatt(
  FILE *f,
  klatt_frame_t frame,
  int Npar)
{
  int *frame_ptr = (int *) (&frame);
  int par_count;

  frame.F0hz10 *= 10;
  for (par_count = 0; par_count < Npar; par_count++) {
    int x = *(frame_ptr++);
    fprintf(f,"%d ", max(x, 0));
  }
  fprintf(f, "\n");
}

void Wpiquefor::CMsaveKlatt()
{
  STRING filename;

  if (!yaUnSignal) return;

  strcpy(filename,"*.*");
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "*.*", 0,"","");
  strcpy(filename,"*.*");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (TFileSaveDialog(Parent,FileData, 0).DoExecute() == IDOK) {
     strcpy(filename,FileData.FileName);
     if (droit_ecrire_message(filename))
        sauvePP(filename, 0, ct.tf,true,0, NPARKLATT);
  }
}

void Wpiquefor::CMsaveKlattLF()
{
  STRING filename;

  if (!yaUnSignal) return;

  strcpy(filename,"*.*");
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "*.*", 0,"","");
  strcpy(filename,"*.*");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (TFileSaveDialog(Parent,FileData, 0).DoExecute() == IDOK) {
     strcpy(filename,FileData.FileName);
     if (droit_ecrire_message(filename))
        sauvePP(filename, 0, ct.tf,true,0, NPARLE);
  }
}
void Wpiquefor::CMsaveFormants()
{
  STRING filename;

  if (!yaUnSignal) return;

  strcpy(filename,"*.*");
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "*.*", 0,"","");
  strcpy(filename,"*.*");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (TFileSaveDialog(Parent,FileData, 0).DoExecute() == IDOK) {
     strcpy(filename,FileData.FileName);
     if (droit_ecrire_message(filename))
        sauvePP(filename, 0, ct.tf, true, 5, NPAR);
  }
}

/*
 * Lecture des polyPoints d'un fichier (appele dans selecteur.c) .
 */
void Wpiquefor::CMlireFor()
{
  lireForKlattOrLF();
}

void Wpiquefor::lireForKlattOrLF()
{
  STRING filename;
  int isRelocatedAt = 0;

  strcpy(filename,"*.*");
  TOpenSaveDialog::TData FileData(OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	      "Klatt files (*.kl?)|*.kl?|All Files|*.*",0,"","");

  strcpy(filename,"");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (WFileOpenDialog(Parent, isRelocatedAt, FileData, IDD_KLATT).DoExecute() == IDOK) {
     strcpy(filename,FileData.FileName);
     if (is_readable(filename) == 1) {
       int deb, fin;
       dateDebFinFilePP(filename, &deb, &fin);
       if (deb == -1 && fin == -1) return;
/*       if (yaUnSignal == ASHADOW && isRelocatedAt + deb > DEFAULTDURATION) {
         affiche_erreur(ml(6460, "Choisissez une date de lecture plus petite"));
         return;
       } that should be allowable.  */
       detruitTousPP();
       isRelocatedAt = max(isRelocatedAt, -deb); // to prevent the creation of polypoints before 0
       lirePP(filename, isRelocatedAt, fpattern);
       if (yaUnSignal == ASHADOW) {
    		int dateSubstitution = datePremPP(true);
    		int dateEndSubstitution = dateLastEndPP(true) + 1000; // + 1000 ms

    		if (dans(dateSubstitution, enMilli(ct.d_a), enMilli(ct.f_a)))
       		initShadow(max(enMilli(ct.tf), dateEndSubstitution), false, (syntheExists) ? enMilli(ct.d_a) : 0, (syntheExists) ? enMilli(ct.f_a) : 2000, ctrlCommandKlatt.samplingRate);
         else initShadow(max(enMilli(ct.tf), dateEndSubstitution), false, max(dateSubstitution - 100, 0), max(dateSubstitution - 100, 0) + 2000, ctrlCommandKlatt.samplingRate);

         synthesizeAndDisplay(true);
       }
       else affPPoints();
     }
  }
}

/*
 * Lecture des polyPoints d'un fichier avec les parametres de source de Liljencrants et Fant (appele dans selecteur.c) .
 */
void Wpiquefor::CMlireForLF()
{
  lireForKlattOrLF();
}

/*
 * Insert a Klatt file somewhere in the current polypoints.
 */
void Wpiquefor::CMpasteKlattFile()
{
  STRING filename;
  int isRelocatedAt = 0;
  Listyl tmpFPattern[NBMAJEURS];

  strcpy(filename,"*.*");
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "Kla files (*.kle)|*.kle;*.bmp|All Files|*.*",0,"","");
  strcpy(filename,"");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  // Initialize an empty fpattern.
  for (int i = 0; i < NBMAJEURS; i++) tmpFPattern[i].list = (Item *) NULL;

  if (WFileOpenDialog(Parent, isRelocatedAt, FileData, IDD_KLATT).DoExecute() == IDOK) {
     strcpy(filename,FileData.FileName);
     if (is_readable(filename) == 1) {
       int deb, fin;
       dateDebFinFilePP(filename, &deb, &fin);
       if (deb == -1 && fin == -1) return;
       if (enEcht(isRelocatedAt + deb) > ct.tf + enEcht(1000)) {
         affiche_erreur(ml(6461, "Insertion au-del de la fin du fichier actuel"));
         return;
       }
       lirePP(filename, isRelocatedAt, tmpFPattern);
       updateWithNewFpattern(tmpFPattern);
       if (yaUnSignal == ASHADOW) {
    		int dateSubstitution = isRelocatedAt + deb;
    		int dateEndSubstitution = dateLastEndPP(true) + 1000; // + 1000 ms

    		if (dans(dateSubstitution, enMilli(ct.d_a), enMilli(ct.f_a)))
       		initShadow(max(enMilli(ct.tf), dateEndSubstitution), false, (syntheExists) ? enMilli(ct.d_a) : 0, (syntheExists) ? enMilli(ct.f_a) : 2000, ctrlCommandKlatt.samplingRate);
         else initShadow(max(enMilli(ct.tf), dateEndSubstitution), false, max(dateSubstitution - 100, 0), max(dateSubstitution - 100, 0) + 2000, ctrlCommandKlatt.samplingRate);

         synthesizeAndDisplay(true);
       }
       else affPPoints();
     }
  }
}


bool Wpiquefor::lireFrameKlatt(
  FILE *f, klatt_frame_t& frame,
  int npar)
{
  int *frame_ptr = (int *) (&frame);
  int par_count;
  bool ok;

  for (par_count = 0, ok = true; ok && par_count < npar; par_count++) {
    int x, n;
    n = fscanf(f,"%d", &x);
    ok = n > 0;
    *frame_ptr++ = x;
  }
  if (par_count < npar) return(false);
  // complete with default values when Ra, Rk and Rg are nor given.
  if (npar < NPARLE) {
    frame.Ra = defaultFrame.Ra;
    frame.Rk = defaultFrame.Rk;
    frame.Rg = defaultFrame.Rg;
    frame.F1Nhz = defaultFrame.F1Nhz;
    frame.B1Nhz = defaultFrame.B1Nhz;
    frame.A1N = defaultFrame.A1N;
    frame.B1Nphz = defaultFrame.B1Nphz;
    frame.F2Nhz = defaultFrame.F2Nhz;
    frame.B2Nhz = defaultFrame.B2Nhz;
    frame.A2N = defaultFrame.A2N;
    frame.B2Nphz = defaultFrame.B2Nphz;
    frame.F3Nhz = defaultFrame.F3Nhz;
    frame.B3Nhz = defaultFrame.B3Nhz;
    frame.A3N = defaultFrame.A3N;
    frame.B3Nphz = defaultFrame.B3Nphz;
  }
  frame.F0hz10 /= 10;

  return(true);
}

/*
 * reading one line of the formant file
 */
void Wpiquefor::readFormantFrame(
  STRING buf, int& date, int& datems, float fi[NBFORKLATT_OLD],
  float db[NBFORKLATT_OLD], float bw[NBFORKLATT_OLD])
{
  char *token;
  int i;

  strtok(buf, " ");
  token = strtok(NULL, " ");
  date = atoi(token);
  token = strtok(NULL, " ()ms");
  datems =  atoi(token);
  for (i = 0; i < NBFORKLATT_OLD; i++) {
    token = strtok(NULL, " ()ms");
    fi[i] = atof(token+3);
    db[i] = atof(strtok(NULL, " "));
  	 bw[i] = atof(strtok(NULL, " "));
  }
}

/*
 * Lecture des anciens fichiers de formants
 */
void Wpiquefor::lectureFicFormants(
  char *filename,
  Listyl newFpattern[NBMAJEURS])
{
  ifstream f;
  int i, indfi[NBFORKLATT_OLD];
  PolyPointsKla *pointfi[NBFORKLATT_OLD];

  f.open(filename);
  if (!f) {
    affiche_erreur(ml(6406,"lirePP : lecture des formants impossible"));
    return;
  }

  for (i = 0; i < NBFORKLATT_OLD; i++) indfi[i] = -1;
  for (i = 0; i < NBFORKLATT_OLD; i++)
    pointfi[i] = newPPd(dim[i+1]);


  while (!f.eof() && !f.fail() && !f.bad()) {
    STRING buf;
    float fi[NBFORKLATT_OLD],db[NBFORKLATT_OLD],bw[NBFORKLATT_OLD];
    int date, datems;
    f.getline(buf, MAX_LIGNE, '\n');
    if (strlen(buf)) {
      readFormantFrame(buf, date, datems, fi, db, bw);

    	for (i = 0; i < NBFORKLATT_OLD; i++) {
      	if (indfi[i] == -1 && fi[i] > 0) indfi[i] = 0;
         /* Si on a lu plus de 4 valeurs par defaut on termine la construction
            du polypoints. */
         if (indfi[i] > 0 && fabs(fi[i]) < EPSILON) {
           pointfi[i]->subpp(indfi[i]);
           pointfi[i]->echantillonne(depFor);
           if (pointfi[i]->nbPt) newFpattern[i + 1].insertSorted(pointfi[i]);
           else delete(pointfi[i]);
           pointfi[i] = newPPd(dim[i + 1]);
	        indfi[i] = -1;
         }
         else if (indfi[i] > -1) {
           int j;
           float t[LARGBANDEP - FREQUENCE + 1];

           for (j = 0; j <= LARGBANDEP; j++) t[j] = 0.;
           t[FREQUENCE] = fi[i];
           t[AMPLITUDE] = db[i];
           t[LARGBANDEP] = bw[i];

           indfi[i]++;
           switch (dim[i+1]) {
             case 2 : pointfi[i]->push(date, t);  break;
             case 3 : pointfi[i]->push(date, t);  break;
             case 4 : pointfi[i]->push(date, t);  break;
             case 5 : pointfi[i]->push(date, t);  break;
          }
        }
      }
    }
  }

  /* On acheve la creation des polyPoints en cours de construction. */
  for (i = 0; i < NBFORKLATT_OLD; i++) if (indfi[i] > 0) {
    pointfi[i]->subpp(min(indfi[i], indfi[i]));
    pointfi[i]->echantillonne(depFor);
    if (pointfi[i]->nbPt) newFpattern[i + 1].insertSorted(pointfi[i]);
    else delete(pointfi[i]);
  }
  else delete(pointfi[i]);
  f.close();
}


/*
 * Lecture des polyPoints d'un fichier de formants ancien style.
 */
void Wpiquefor::CMlectureFicFormants()
{
  STRING filename;
  int isRelocatedAt = 0;

  strcpy(filename,"*.*");
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "Formant files (*.for)|*.for;*.bmp|All Files|*.*",0,"","");
  strcpy(filename,"");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (WFileOpenDialog(Parent, isRelocatedAt, FileData, IDD_KLATT).DoExecute() == IDOK) {
     strcpy(filename,FileData.FileName);
     if (is_readable(filename) == 1) {
       detruitTousPP();
       lectureFicFormants(filename, fpattern);
       if (yaUnSignal == ASHADOW) {
    		int dateSubstitution = datePremPP(true);
    		int dateEndSubstitution = dateLastEndPP(true) + 1000; // + 1000 ms

    		if (dans(dateSubstitution, enMilli(ct.d_a), enMilli(ct.f_a)))
       		initShadow(max(enMilli(ct.tf), dateEndSubstitution), false, (syntheExists) ? enMilli(ct.d_a) : 0, (syntheExists) ? enMilli(ct.f_a) : 2000, ctrlCommandKlatt.samplingRate);
         else initShadow(max(enMilli(ct.tf), dateEndSubstitution), false, max(dateSubstitution - 100, 0), max(dateSubstitution - 100, 0) + 2000, ctrlCommandKlatt.samplingRate);

         synthesizeAndDisplay(true);
       }
       else affPPoints();
     }                  
  }
}

//
// Lecture des polyPoints d'un fichier.
//
void Wpiquefor::lirePP(
  char *filename,
  int offset,  	// time offset in ms to relocate the polypoints.
  Listyl newFpattern[NBMAJEURS])
{
  FILE *f;
  int i, indfi[NBMAJEURS],
      ldef[NBMAJEURS];	/* indice du premier point egal a la valeur par defaut. */
  PolyPointsKla *pointfi[NBMAJEURS];
  char magicCode[5];
  int npar;

  klatt_frame_t frame;

  for (i = 0; i < NBMAJEURS; i++)
    pointfi[i] = newPPd(dim[i]);

  f = fopen(filename, "r");
  if (!f) {
    affiche_erreur("lirePP : lecture des formants impossible");
    return;
  }

  fscanf(f, "%4s\n", magicCode);
  if (!strcmp(magicCode, ".kla")) npar = NPARKLATT;
  else if (!strcmp(magicCode, ".kle")) npar = NPARLE;
  else if (!strcmp(magicCode, ".klf")) npar = NPAR;
  else {
    affiche_erreur("lirePP: ce n'est pas un fichier pour le synthe de Klatt");
    fclose(f);
    return;
  }

  for (i = 0; i < NBMAJEURS; i++) {
    indfi[i] = -1;
    ldef[i] = INFINI_I/2;
  }

  while (lireFrameKlatt(f, frame, npar)) {
    int *frami = (int *) &frame,
        *dframe = (int *) &defaultFrame;

    frame.Date += offset;

    for (i = 0; i < NBMAJEURS; i++) {
      /* Des qu'on lit une valeur differente de la valeur par defaut on commence
	      a enregistrer les points lus. */
      if (indfi[i] == -1 && frami[nKlatt[i][0]] != dframe[nKlatt[i][0]]) indfi[i] = 0;

      if (indfi[i] > 0 && frami[nKlatt[i][0]] == dframe[nKlatt[i][0]]) {
        pointfi[i]->subpp(indfi[i]);
        pointfi[i]->echantillonne(depFor);

        if (pointfi[i]->nbPt) newFpattern[i].insertSorted(pointfi[i]);
        else delete(pointfi[i]);
        pointfi[i] = newPPd(dim[i]);
	     indfi[i] = -1;
      }
      else if (indfi[i] > -1) {
        int j, k;
        float t[LARGBANDEP - FREQUENCE + 1];

        for (j = 0, k = 0; j <= LARGBANDEP; j++) if (nKlatt[i][j] > -1)
          t[k++] = frami[nKlatt[i][j]];
        indfi[i]++;
        switch (dim[i]) {
          case 2 : pointfi[i]->push(frame.Date, t);  break;
          case 3 : pointfi[i]->push(frame.Date, t);  break;
          case 4 : pointfi[i]->push(frame.Date, t);  break;
          case 5 : pointfi[i]->push(frame.Date, t);  break;
        }
      }
    }
  }

  /* On acheve la creation des polyPoints en cours de construction. */
  for (i = 0; i < NBMAJEURS; i++) if (indfi[i] > 0) {
    pointfi[i]->subpp(min(indfi[i], ldef[i]));
    pointfi[i]->echantillonne(depFor);
    if (pointfi[i]->nbPt) newFpattern[i].insertSorted(pointfi[i]);
    else delete(pointfi[i]);
  }
  else delete(pointfi[i]);
  fclose(f);
}

// Update the current fpattern with the polypoints read from the file.
void Wpiquefor::updateWithNewFpattern(
  Listyl newFpattern[NBMAJEURS])
{
 int i;
 for (i = 0; i < NBMAJEURS; i++) {
   Item *q;
   for (q = newFpattern[i].list; q; q = q->next)
     fpattern[i].insertSorted(resolConflictPP(q->val, i));
 }
 // destroys the temporary fpattern read from the file.
 for (i = 0; i < NBMAJEURS; i++) {
   Item *q;
   for (q = newFpattern[i].list; q; q = q->next) q->val->selectionne(true);
   newFpattern[i].removeIfSelec();
 }
}

//
// Solve conflicts generated by polypoints npp. Unlike the function resolConflit
// npp is a complete polypoints, i.e. all the coordinates are non-zero.
//
PolyPointsKla* Wpiquefor::resolConflictPP(
  PolyPointsKla *npp,
  int mj)	/* parametre majeur (entre 1 et NBMAJEURS) */
{
  Item *pp;
  int debNvpp, finNvpp;
  int mi, dimy = dim[mj] - 1,
      ideb, ifin;

  // Before solving conflicts the polypoints must be sampled because it
  // comes from a file which possibily has been recorded with a depFor diffirent
  // from the current value.
  npp->echantillonne(depFor);
  if (!npp->nbPt) return((PolyPointsKla *) NULL);
  debNvpp = npp->debut();
  finNvpp =  npp->fin();
  /* D'abord on resoud les conflits en
     modifiant la frequence, la largeur de bande ou l'amplitude 
     d'un ou de plusieurs polypoints. */
  for (pp = fpattern[mj].list; pp; pp = pp->next) {
    int debpp = pp->val->debut(),
      finpp = pp->val->fin();

    if (dans(debNvpp, debpp, finpp) && dans(finNvpp, debpp, finpp)) { /* inclusion du nvpp dans pp. */
      int i, 
	   j = pp->val->indPP(npp->debut());

      if (j != -1) for (i = 0; i < npp->nbPt; i++, j++) {
	     for (mi = 0; mi < dimy; mi++) *(pp->val->py(j, mi)) = *(npp->py(i, mi));
	     *(npp->py(i, 0)) = -INFINI_I;		/* on marque les points qui ont ete utilises pour les corrections. */
      }
    }
    else if (debNvpp < finpp && finpp < finNvpp) {	/* recouvrement sur la fin du pp. */
      int i = (debNvpp < debpp) ? npp->indPP(pp->val->debut()) : 0,  /* dans (*ppp) */
	       j = (debNvpp < debpp) ? 0 : pp->val->indPP(npp->debut()); /* dans pp */

      if (j != -1 && i != -1) for (; i < npp->nbPt && j < pp->val->nbPt; i++, j++) {
        for (mi = 0; mi < dimy; mi++) *(pp->val->py(j, mi)) = *(npp->py(i, mi));
	     *(npp->py(i, 0)) = -INFINI_I;		/* on marque les points qui ont ete utilises pour les corrections. */
      }
    }
    else if (debNvpp < debpp && debpp < finNvpp) {   /* recouvrement sur le debut du pp. */
      int i = npp->indPP(pp->val->debut()), j;
      if (i != -1) for (j = 0;  i < npp->nbPt && j < pp->val->nbPt; i++, j++) {
        for (mi = 0; mi < dimy; mi++) *(pp->val->py(j, mi)) = *(npp->py(i, mi));
	     *(npp->py(i, 0)) = -INFINI_I;		/* on marque les points qui ont ete utilises pour les corrections. */
      }
    }
  }
  // on vient de resoudre les conflits avec le nouveau polypoints
  // maintenant on sauvegarde les points qui n'ont pas servi aux corrections.
  // skip the first null points to find the first point to be kept.
  for (ideb = 0; ideb < npp->nbPt && *(npp->py(ideb, 0)) < -INFINI_I / 2.; ideb++);
  // search for the last point.
  for (ifin = ideb; ifin < npp->nbPt && *(npp->py(ifin, 0)) > -INFINI_I / 2.; ifin++);
  // extract the sub-polypoints which remains.
  return(npp->subPPKla(ideb, ifin - 1));
}


//
//  Finds the beginning and the end time in a Klatt parameter file.
//
void Wpiquefor::dateDebFinFilePP(
  char *filename,
  int *deb,
  int *fin)
{
  FILE *f = fopen(filename, "r");
  klatt_frame_t frame;
  char magicCode[5];
  int npar;

  *deb = -1;
  *fin = -1;
  if (!f) {
    affiche_erreur("dateDebFinFilePP : unable to open the Klatt file");
    return;
  }

  fscanf(f, "%4s\n", magicCode);
  if (!strcmp(magicCode, ".kla")) npar = NPARKLATT;
  else if (!strcmp(magicCode, ".kle")) npar = NPARLE;
  else if (!strcmp(magicCode, ".klf")) npar = NPAR;
  else {
    affiche_erreur(ml(6462, "ce n'est pas un fichier pour le synthtiseur de Klatt"));
    fclose(f);
    return;
  }

  while (lireFrameKlatt(f, frame, npar)) {
    if (*deb < 0) *deb = frame.Date;
    *fin = frame.Date;
  }
  fclose(f);
}

void Wpiquefor::chTypeFor()
{
  WORD c=MF_CHECKED, u=MF_UNCHECKED;

  CheckMenuItem(::GetMenu(HWindow), MENUFormant, (typeFor==FORMANT)?c:u);
  CheckMenuItem(::GetMenu(HWindow), MENUSpectro,(typeFor==SPECTRO)?c:u);
}

void Wpiquefor::CMFormant()
{
 	typeFor = FORMANT;
 	chTypeFor();
}

void Wpiquefor::CMSpectro()
{
	typeFor = SPECTRO;
 	chTypeFor();
}

//
// determination des dates de debut et de fin du domaine couvert par
// les polypoints.
//
void Wpiquefor::dateDebFinKlatt(
  int *deb,
  int *fin)
{
  int i, dateMin = ct.tf,
      dateMax = 0;

  for (i = 0; i < NBMAJEURS; i++) {
    Item *q;
    for (q = fpattern[i].list; q; q = q->next) {
      dateMin =  (q->val->debut() < dateMin) ? q->val->debut() : dateMin;
      dateMax =  (q->val->fin() > dateMax) ? q->val->fin() : dateMax;
    }
  }
  *deb = dateMin;
  *fin = dateMax;
}


//
// Creation des polypoints qui correspondent au pitch calcule
// automatiquement.
//
void Wpiquefor::CMcreePPF0()
{
  int date, deb, fin;

  if (yaUnSignal != ASIGNAL) {
    affiche_erreur("Only for a real speech signal");
	 return ;
  }
  dateDebFinKlatt(&deb, &fin);
  (*ppp).newsize(0); // Re-initialization of the tabpoint used for capturing a new parameter polypoints.
  if (deb >= fin) return;
  for (date = deb - (deb % depFor); date < fin; date += depFor) 
    (*ppp).push(Point2P(date, (int) pm.getReadf0(enEcht(date))));
  ::paramActif = F0;
  ::typeScale = SCALEF0;
  validatePPP();
  chScaleKlatt();
}

/*
 * Recalage des PolyPoints sur les pics de spectres les plus proches.
 */
void Wpiquefor::CMrecalFor(int mode)
{
  STRING ficTempKlatt,
       ficSquelette,
       ficTempKlattCorrige;

  // On s'assure qu'il y a au moins un
  //   formant selectionne
  if (isNoPPSel()) {
    messagePasDeFormantsSelectionnes();
    return;
  }

  strcat(strcpy(ficTempKlatt, tempdir), tmpnam((char *) NULL));
  strcat(strcpy(ficSquelette, tempdir), tmpnam((char *) NULL));
  strcat(strcpy(ficTempKlattCorrige, tempdir), tmpnam((char *) NULL));
  sauvePP(ficTempKlatt, 0, ct.tf, true, 0, NPARLE);
  sauvePP(ficSquelette, 0, ct.tf, false, 3, NPARLE);
  recalPPSpectre(ficTempKlatt, ficSquelette, ficTempKlattCorrige, mode);

  detruitTousPP();
  lirePP(ficTempKlattCorrige, 0, fpattern);
  affPPoints();
  unlink(ficTempKlatt);
  unlink(ficSquelette);
  unlink(ficTempKlattCorrige);
}


/*
 * Recalage des points d'un fichier de points de formants par rapport au pic spectral le plus proche.
 */
void Wpiquefor::recalPPSpectre (
    char *nomFicFormant,
    char *nomSquelette,
    char *nomFicFormantCorrige,
    int mode)			/* Lpc ou Cepstre suivant le choix de l'utilisateur. */
{
  FILE *f, *g, *h;
  char magicCode[5];
  float deviation[NBFORKLATT_EXT],	/* ecart entre le formant d'origine et le formant recale. */
        db_[NBFORKLATT_EXT];			/* derniere valeur de l'amplitude calculee. */
  int nbPtFor[NBFORKLATT_EXT];	/* nombre de points par formant. */
  int i,
      /* Nombre d'echantillons correspondant au maximum de spectres qui peuvent etre calcules. */
      nbEchtMax = NB_SPECTRES_SUIVI * enEcht(depFor);
  struct ctrl_temps  ctloc;
  SAMPLEbuf space_bis(1, "space_bis to register formants");
    // pour suivrePicsCepstre() et suivreRacLpc()
  FLOATbuf **spectreCep;
  klatt_frame_t squelette;
  int startSkeleton = INFINI_I;

  for (i = 0; i < NBFORKLATT_EXT; i++) db_[i] = 0.;

  f = fopen(nomFicFormant, "rt");
  if (f) fscanf(f, "%4s\n", magicCode);
  if (!f) {
    affiche_erreur(ml(6408,"recalPPSpectre : le fichier de formants n'existe pas "));
    return;
  }
  if (nsof(nomFicFormant) <= 5 || strcmp(magicCode, ".kle")) {
    affiche_erreur(ml(6409,"recalPPSpectre : pas de formant dans ce fichier"));
    fclose(f);
    return;
  }

  h = fopen(nomSquelette, "rt");
  if (h) fscanf(h, "%4s\n", magicCode);
  if (!h || strcmp(magicCode, ".kle")) {
    affiche_erreur(ml(6414,"recalPPSpectre : le fichier squelette n'existe pas "));
    return;
  }

  g = fopen(nomFicFormantCorrige, "w");
  if (!g) {
    fclose(f);
    affiche_erreur(ml(6410,"recalPPSpectre : creation du fichier temporaire de formant impossible "));
    return;
  }
  fprintf(g,".kle\n");         // write the magic number

  ctloc.d_e = ctloc.f_e = -1;

  for (i = 0; i < NBFORKLATT_EXT; i++) {
    deviation[i] = 200.;	/* ecart initial autorise. */
    nbPtFor[i] = 0;
  }

  spectreCep =(FLOATbuf**) malloc(sizeof(FLOATbuf *));
  *spectreCep = (FLOATbuf *) new FLOATbuf(512 / 2 + 1, "spectreCep");
  if (mode == SLISCEP && enEcht(optKlattSpec.fenCepstre) > 512)
    optKlattSpec.fenCepstre = enMilli(512);

  if (lireFrameKlatt(h, squelette, NPARLE)) {
    startSkeleton = squelette.Date;
  }

  while (!feof(f)) {
    float fi[NBFORKLATT_EXT], bw[NBFORKLATT_EXT], db[NBFORKLATT_EXT], ok[NBFORKLATT_EXT];
    float spectre[512];
    bool skeletonDefined;
    int ifor, date;
    racines1_lpc res_lpc;
    Pics1_Cepstre picsCepstre;
    // struct extrema picsCepstre;
    klatt_frame_t frame;

    if (!lireFrameKlatt(f, frame, NPARLE)) break;
    if (frame.Date < startSkeleton) skeletonDefined = false;
    else if (frame.Date == startSkeleton) skeletonDefined = true;
    else skeletonDefined = lireFrameKlatt(h, squelette, NPARLE);

    fi[0] = frame.F1hz; bw[0] = frame.B1phz; db[0] = frame.A1;
    fi[1] = frame.F2hz; bw[1] = frame.B2phz; db[1] = frame.A2;
    fi[2] = frame.F3hz; bw[2] = frame.B3phz; db[2] = frame.A3;
    fi[3] = frame.F4hz; bw[3] = frame.B4phz; db[3] = frame.A4;
    fi[4] = frame.F5hz; bw[4] = frame.B5phz; db[4] = frame.A5;
    fi[5] = frame.F6hz; bw[5] = frame.B6phz; db[5] = frame.A6;
    fi[6] = frame.F1Nhz; bw[6] = frame.B1Nphz; db[6] = frame.A1N;
    fi[7] = frame.F2Nhz; bw[7] = frame.B2Nphz; db[7] = frame.A2N;
    fi[8] = frame.F3Nhz; bw[8] = frame.B3Nphz; db[8] = frame.A3N;

    date = enEcht(frame.Date);

    for(ifor = 0; ifor < NBFORKLATT_EXT; ifor++) ok[ifor] = FAUX;

    if (date > ct.tf) break;

    /* Eventuellement on lit de la parole si on sort du buffer en memoire. */
    if (!dans(date + max(enEcht((int)optKlattSpec.fenLPC), enEcht((int)optKlattSpec.fenCepstre)), ctloc.d_e, ctloc.f_e)) {
      ctloc.d_e = max(date  - ((((mode) == SRACLPC) ? enEcht((int)optKlattSpec.fenLPC) : enEcht((int)optKlattSpec.fenCepstre))) / 2, 0);
      ctloc.d_e = max(0, ctloc.d_e);
      /* On lit un buffer plus petit que ce que l'on pourrait lire car les analyses spectrales
         peuvent prendre du temps. */
      ctloc.f_e = min(ct.tf, ctloc.d_e + min(MAXECH / 6, nbEchtMax) - 1);
      if (ctloc.f_e <= ctloc.d_e)  {
         affiche_erreur("recalPPSpectre : unconsistent Klat parameter file");
         return;
      }
      space_bis.newsize(ctloc.f_e - ctloc.d_e + 1 + 1);
      lireSignal(en_cours, en_cours.depl + sizeof(SAMPLE) * ctloc.d_e, space_bis, 0,
	       ctloc.f_e - ctloc.d_e + 1);
    }

    /* Les spectres sont calcules avec preaccentuation pour ne pas decaler
       F1 vers les basses frequences. */
    switch (mode) {
    case SRACLPC:
      spectreLPC(spectre, res_lpc, &space_bis[0], date - enEcht((int)optKlattSpec.fenLPC) / 2 - ctloc.d_e,
      			  enEcht((int)optKlattSpec.fenLPC), 512,
                 true, ordre_lpc_msTOecht(optKlattSpec.or_lpc), 1.);
      break;
    case SLISCEP: {
      float min_cep, max_cep;
      cepstre(spectreCep, &space_bis[0], date - enEcht((int)optKlattSpec.fenCepstre) / 2 - ctloc.d_e, 1, 0, enEcht((int)optKlattSpec.fenCepstre),
              512, 512, (int)enEcht(optKlattSpec.nb_cep[0]), (int)enEcht(optKlattSpec.nb_cep[1]), 1, true, false);
      ex_extr(**spectreCep, picsCepstre, 1, 512 / 2 - 1);
      /* recuperation du min et du max de la fft lissee
         correspondant a ce cepstre */
      min_cep = min_f(**spectreCep, 1, 512 / 2 - 1);
      max_cep = max_f(**spectreCep, 1, 512 / 2 - 1);
      estime(picsCepstre, min_cep, max_cep);
      les_meilleurs(picsCepstre, 9, 512 / 2, QualitePicCepstre);
      }
      break;
    }

    for (i = 0; i < NBFORKLATT_EXT; i++) if (fi[i] > 0.) {
      int j, jmin;
      switch (mode) {
        case SRACLPC : {
          for (j = 0, jmin = 0; j < res_lpc.size(); j++)
            jmin = (   fabs(res_lpc[j].fr - fi[i]) <
		       fabs(res_lpc[jmin].fr - fi[i])
                    && res_lpc[j].lar < 700.) ? j : jmin;
          if (res_lpc.size() && fabs(res_lpc[jmin].fr - fi[i]) < max(500.,deviation[i] * 2.5)) {
	         deviation[i] =  ((deviation[i] * nbPtFor[i])
            				  + fabs(res_lpc[jmin].fr - fi[i])) / (nbPtFor[i]+1);
	    		nbPtFor[i]++;
            fi[i] = res_lpc[jmin].fr;
	    		bw[i] = res_lpc[jmin].lar;
	   		 db_[i] = db[i] = res_lpc[jmin].db;
	    		ok[i] = VRAI;
	  		 }
		  }
        break;
	   case SLISCEP : {
        for (j = 1, jmin = 0; j < picsCepstre.size(); j++)
          jmin = (fabs(picsCepstre[j].frHz - fi[i]) < fabs(picsCepstre[jmin].frHz - fi[i])) ? j : jmin;
        if (   picsCepstre.size()
            && fabs(picsCepstre[jmin].frHz - fi[i]) < max(500.,deviation[i] * 2.5)) {
          deviation[i] =  ((deviation[i] * nbPtFor[i])
        					 + fabs(picsCepstre[jmin].frHz - fi[i])) / (nbPtFor[i]+1);
          nbPtFor[i]++;
          fi[i] = picsCepstre[jmin].frHz;
          bw[i] =  picsCepstre[jmin].bw / 2.; /* approximatif car c'est a moins 6 dB et sur un pic "lisse" */
          db_[i] = db[i] = picsCepstre[jmin].v;
          ok[i] = VRAI;
        }
      }
      break;
     }
    }
    else {
      bw[i] = 50. + i*i * 10.;
      db[i] = 0.;
    }
    
    /* On ne conserve pas les amplitudes qui doivent etre recalees avec
       recalCopySynthesis. */
    /* for (ifor = 0; ifor < NBFORKLATT_EXT; ifor++)
       if (!ok[ifor]) db[ifor] = db_[ifor]; */

    if (skeletonDefined && squelette.F1hz) {
      frame.F1hz = fi[0]; 
      // frame.B1phz = bw[0];
      /* frame.A1 = db[0]; */
    }
    if (skeletonDefined && squelette.F2hz) {
      frame.F2hz = fi[1]; 
      // frame.B2phz = bw[1];
      /* frame.A2 = db[1]; */
    }
    if (skeletonDefined && squelette.F3hz) {
      frame.F3hz = fi[2]; 
      // frame.B3phz = bw[2];
      /* frame.A3 = db[2]; */
    }
    if (skeletonDefined && squelette.F4hz) {
      frame.F4hz = fi[3]; 
      // frame.B4phz = bw[3];
      /* frame.A4 = db[3]; */
    }
    if (skeletonDefined && squelette.F5hz) {
      frame.F5hz = fi[4]; 
      // frame.B5phz = bw[4];
      /* frame.A5 = db[4]; */
    }
    if (skeletonDefined && squelette.F6hz) { 
      frame.F6hz = fi[5]; 
      // frame.B6phz = bw[5];
      /* frame.A6 = db[5]; */
    }

    if (skeletonDefined && squelette.F1Nhz) {
      frame.F1Nhz = fi[6];
    }

    if (skeletonDefined && squelette.F2Nhz) {
      frame.F2Nhz = fi[7];
    }

    if (skeletonDefined && squelette.F3Nhz) {
      frame.F3Nhz = fi[8];
    }

    sauveFrameKlatt(g, frame, NPARLE);
  }
  fclose(f);
  fclose(g);
  fclose(h);
  delete *spectreCep;
  free((char *) spectreCep);
}


/*
 * Recalage des PolyPoints pour la synthese de Klatt
 */
void Wpiquefor::CMcopySynthesis()
{
  STRING ficTempKlatt,
         ficSquelette,
         ficTempExcit,
         ficTempKlattCorrige;
  int i, auMoinsUnPP, dateEndSkeleton;

  STARTUPINFO siStartInfo;
  PROCESS_INFORMATION piProcInfo;
  DWORD code;
  STRING commande;

  /* On s'assure qu'il y a au moins un formant selectionne. */
  for (i = 0, auMoinsUnPP = 0; !auMoinsUnPP && i < NBFORKLATT_EXT; i++) {
    Item *q;
    for (q = fpattern[i].list; q && !auMoinsUnPP; q = q->next) auMoinsUnPP = q->val->estSelec();
  }
  if (!auMoinsUnPP) return;

  strcat(strcpy(ficTempKlatt, tempdir), tmpnam((char *) NULL));
  strcat(strcpy(ficSquelette, tempdir), tmpnam((char *) NULL));
  strcat(strcpy(ficTempKlattCorrige, tempdir), tmpnam((char *) NULL));
  strcat(strcpy(ficTempExcit, tempdir), tmpnam((char *) NULL));

  sauvePP(ficTempKlatt, 0, ct.tf, true, 4, NPARLE);
  dateEndSkeleton = sauvePP(ficSquelette, 0, ct.tf, false, 3, NPARLE);

  sprintf(commande, "\"%s\\Klatt\\klatt.exe\" -r %d -i \"%s\" -o \"%s\" -t 7 -f %d -s %d -v %d",
           WinSnooriDirectory, ctrlCommandKlatt.byteArrang, ficTempKlatt, ficTempExcit,
           ctrlCommandKlatt.frame,  (int) en_cours.frequence, (ctrlCommandKlatt.source == KLATTSOURCE) ? 2 : 4);
  memset(&siStartInfo, 0, sizeof(STARTUPINFO));
  siStartInfo.cb = sizeof(STARTUPINFO);
  CreateProcess(NULL, commande, 0, 0, false, DETACHED_PROCESS, 0, 0, &siStartInfo, &piProcInfo);
  for (code = STILL_ACTIVE; code == STILL_ACTIVE;) {
    GetExitCodeProcess(piProcInfo.hProcess,&code);
    Sleep(100);
  }

//  recalCopySynthesis(ficTempKlatt, ficSquelette, ficTempExcit, ficTempKlattCorrige, SRACLPC);
  recalCopySynthesis(ficTempKlatt, ficSquelette, ficTempExcit, ficTempKlattCorrige, SPITCHSYNC, dateEndSkeleton);

  detruitTousPP();
  lirePP(ficTempKlattCorrige, 0, fpattern);

  affPPoints();

  unlink(ficSquelette);
  unlink(ficTempKlattCorrige);
  unlink(ficTempExcit);
  unlink(ficTempKlatt);
}



#define NASAL                                                      
#define FENFFT512    512
/* #define OPTIMISE */
/*
 * Reglage des parametres pour la synthese par copie.
 */
void Wpiquefor::recalCopySynthesis (
    char *nomFicFormant, 
    char *nomSquelette,
    char *nomExcitation,
    char *nomFicFormantCorrige,
    int mode,  /* Lpc, Cepstre ou Halle suivant le choix de l'utilisateur. */
    int dateEndSkeleton) // date in ms
{  
  FILE *f, *g, *h;
  char magicCode[5];
  int i, dateDebExcit = -1,
      /* Nombre d'echantillons correspondant au maximum de spectres qui peuvent etre calcules. */
      nbEchtMax = NB_SPECTRES_SUIVI * enEcht(depFor),
      nbEchtFen;
  struct ctrl_temps  ctloc, ctExcit;
  struct infoFile en_coursExcit;
  int fenCepstreIter = 32;
  SAMPLEbuf space_bis(1, "space_bis for copy synthesis");
  FLOATbuf **spectreCep;
  klatt_frame_t squelette;
  int startSkeleton = INFINI_I;

  strcpy(en_coursExcit.nom, nomExcitation);
  en_coursExcit.nb_bits = 16;
  en_coursExcit.depl = 0;

  en_coursExcit.swap = SWAPINFILE;

  f = fopen(nomFicFormant, "rt");
  if (f) fscanf(f, "%4s\n", magicCode);
  if (!f) {
    affiche_erreur(ml(6408,"recalPPSpectre : le fichier de formants n'existe pas "));
    return;
  }
  if (nsof(nomFicFormant) <= 5 || strcmp(magicCode, ".kle")) {
    affiche_erreur(ml(6409,"recalPPSpectre : pas de formant dans ce fichier"));
    fclose(f);
    return;
  }

  h = fopen(nomSquelette, "rt");
  if (h) fscanf(h, "%4s\n", magicCode);
  if (!h || strcmp(magicCode, ".kle")) {
    affiche_erreur(ml(6414,"recalPPSpectre : le fichier squelette n'existe pas "));
    return;
  }

  g = fopen(nomFicFormantCorrige, "w");
  if (!g) {
    fclose(f);
    affiche_erreur("recalPPSpectre : creation du fichier temporaire de formant impossible ");
    return;
  }
  fprintf(g,".kle\n");         // write the magic number
 
  ctloc.d_e = ctloc.f_e = -1;
  ctExcit.d_e = ctExcit.f_e = -1;
  ctExcit.tf = nsof(nomExcitation) / sizeof(SAMPLE);

  spectreCep =(FLOATbuf**) malloc(sizeof(FLOATbuf *));
  *spectreCep = (FLOATbuf *) new FLOATbuf(512 / 2 + 1, "spectreCep");
  if (mode == SLISCEP && enEcht(optKlattSpec.fenCepstre) > 512)
    optKlattSpec.fenCepstre = enMilli(512);

  switch(mode) {
  case SRACLPC: nbEchtFen = enEcht(optKlattSpec.fenLPC);
    break;
  case SLISCEP: nbEchtFen = enEcht(optKlattSpec.fenCepstre);
    break;
  case SPITCHSYNC : nbEchtFen = enEcht(optKlattSpec.fenCepstre);
    break;
  }

  if (lireFrameKlatt(h, squelette, NPARLE)) {
    startSkeleton = squelette.Date;
  }
  Wact->showpourcent("Recal copy synthesis");
  while (!feof(f)) {
    float fi[NBFORKLATT_EXT+1], bw[NBFORKLATT_EXT+1], db[NBFORKLATT_EXT+1];
    int date;
    bool skeletonDefined;
    klatt_frame_t frame;

    if (!lireFrameKlatt(f, frame, NPARLE)) break;
    if (frame.Date < startSkeleton) skeletonDefined = false;
    else if (frame.Date == startSkeleton) skeletonDefined = true;
    else skeletonDefined = lireFrameKlatt(h, squelette, NPARLE);

    date = enEcht(frame.Date);
    if (dateDebExcit < 0) dateDebExcit = date;

    if (!skeletonDefined) sauveFrameKlatt(g, frame, NPARLE);
    else {
      int dateExcit;
      float spectreExcitation[FENFFT512],
            spectreOrigine[FENFFT512];
      float spectresOrigineFi[F3N - F2 + 1][FENFFT512],
            *spectresOrigine[F3N - F1 + 1];
      spectresOrigine[0] = spectreOrigine;    // for F1
      for (int i = F2 - F1; i <= F3N - F1; i++) spectresOrigine[i] = spectresOrigineFi[i - 1];
      racines1_lpc res_lpc;

      /* fi[0] represente le formant nasal qui est utilise pour controler
       les premiers harmoniques. Ce formant est cree s'il n'existait pas
       auparavant. */
#ifdef NASAL
      fi[0] = (frame.FNPhz) ? frame.FNPhz : 200.; bw[0] = frame.BNPhz; db[0] = frame.ANP;
#else
      fi[0] = 0.; bw[0] = frame.BNPhz; db[0] = frame.ANP;
#endif
      fi[1] = frame.F1hz; bw[1] = frame.B1phz; db[1] = frame.A1;
      fi[2] = frame.F2hz; bw[2] = frame.B2phz; db[2] = frame.A2;
      fi[3] = frame.F3hz; bw[3] = frame.B3phz; db[3] = frame.A3;
      fi[4] = frame.F4hz; bw[4] = frame.B4phz; db[4] = frame.A4;
      fi[5] = frame.F5hz; bw[5] = frame.B5phz; db[5] = frame.A5;
      fi[6] = frame.F6hz; bw[6] = frame.B6phz; db[6] = frame.A6;
      fi[7] = frame.F1Nhz; bw[7] = frame.B1Nphz; db[7] = frame.A1N;
      fi[8] = frame.F2Nhz; bw[8] = frame.B2Nphz; db[8] = frame.A2N;
      fi[9] = frame.F3Nhz; bw[9] = frame.B3Nphz; db[9] = frame.A3N;


      dateExcit = date - dateDebExcit;
      Wact->pourcent(date - enEcht(startSkeleton), enEcht(dateEndSkeleton) - enEcht(startSkeleton));

      if (date  + nbEchtFen > ct.tf) break;

      /* Eventuellement on lit de la parole si on sort du buffer en memoire. */
      if (!dans(date + nbEchtFen, ctloc.d_e, ctloc.f_e)) {
        ctloc.d_e = max(date  - nbEchtFen / 2, 0);
        ctloc.d_e = max(0, ctloc.d_e);

        /* On lit un buffer plus petit que ce que l'on pourrait lire car les analyses spectrales
           peuvent prendre du temps. */
        ctloc.f_e = min(ct.tf, ctloc.d_e + min(MAXECH / 6, nbEchtMax) - 1);
        if (ctloc.f_e <= ctloc.d_e)  {
           affiche_erreur("recalCopySynthesis: unconsistent formant file");
           return;
        }
        space_bis.newsize(ctloc.f_e - ctloc.d_e + 1 + 1);
        lireSignal(en_cours, en_cours.depl + sizeof(SAMPLE) * ctloc.d_e, space_bis, 0,
      		    	 ctloc.f_e - ctloc.d_e + 1);
        if (mode == SPITCHSYNC) {
          int i, n, premier_pic_Maxmin =0;
          float betaPitch =200.;

          calculePitch(&n, space_bis, ctloc, typeLoc);

          marquesPitch.newsize(ctloc.f_e - ctloc.d_e);
          for (i=0 ; i < ctloc.f_e - ctloc.d_e ; i++) marquesPitch[i]=0;

          Pitch_Marqueur(marquesPitch, space_bis, ctloc, ctloc.d_e, ctloc.f_e, betaPitch,
   				   	     enEcht(10), (struct ProgDynPoint *) NULL, &premier_pic_Maxmin);
          i=0;
          while (marquesPitch[i]!=0) i++;
          nbMarques = i;
          marquesPitch.newsize(i);
        }
      }

      /* Eventuellement on lit le signal d'excitation si on sort du buffer en memoire */
      if (!dans(dateExcit + nbEchtFen, ctExcit.d_e, ctExcit.f_e)) {
        ctExcit.d_e = max(dateExcit  - nbEchtFen / 2, 0);

        /* On lit un buffer plus petit que ce que l'on pourrait lire car les analyses spectrales
           peuvent prendre du temps. */
        ctExcit.f_e = min(ctExcit.tf, ctExcit.d_e + min(MAXECH / 6, nbEchtMax) - 1);
        if (ctExcit.f_e <= ctExcit.d_e)  {
           affiche_erreur("recalCopySynthesis: unconsistent excitation file");
           return;
        }
        space_ter.newsize(ctExcit.f_e - ctExcit.d_e + 1 + 1);
        lireSignal(en_coursExcit, sizeof(SAMPLE) * ctExcit.d_e, space_ter, 0,
      	          ctExcit.f_e - ctExcit.d_e + 1);
      }

      switch (mode) {
      case SRACLPC :
        spectreLPC(spectreOrigine, res_lpc, &space_bis[0], date - nbEchtFen / 2 - ctloc.d_e,
      	   		  nbEchtFen, 512,
                    true, ordre_lpc_msTOecht(optKlattSpec.or_lpc), 1.);
        spectreLPC(spectreExcitation, res_lpc, &space_ter[0], dateExcit - nbEchtFen / 2 - ctExcit.d_e,
      			    nbEchtFen, 512,
                   true, ordre_lpc_msTOecht(optKlattSpec.or_lpc), 1.);
        break;
      case SPITCHSYNC : {
		  int m = chercherMarque(date);
        int fF3N = F3N - F1 + 1;
   	  int echt, echt0, echt1,  echtMaxEner[F3N - F1 + 1], freqMaxEner[F3N - F1 + 1];
   	  float eForMax[F3N - F1 + 1];
		  if (m != -1) {
          if (!m) {
        	   echt0 = abs(marquesPitch[0]);
            echt1 = abs(marquesPitch[1]);
          }
          else {
            echt0 = abs(marquesPitch[m - 1]);
            echt1 = abs(marquesPitch[m]);
          }
        }

        for (int iFor = 0; iFor < fF3N; iFor++) {
			 echtMaxEner[iFor] = echt0;
          eForMax[iFor] = - INFINI_I;
        }

        for  (echt = echt0; echt < echt1; echt += 4) {
           spectreFft(spectreOrigine, space_bis, max(echt - ctloc.d_e, 0), false, FENFFT512, enEcht(4));
           for (int iFor = 0; iFor < F3N - F1 + 1; iFor++) {
             int k = IROUND(fi[1 + iFor] / en_cours.frequence * FENFFT512);
             int km, kmax,
                 k0 = max(0, IROUND((fi[1 + iFor] - 100.)/ en_cours.frequence * FENFFT512)),
                 k1 = min(FENFFT512 / 2, IROUND((fi[1 + iFor] - 100.)/ en_cours.frequence * FENFFT512));
             float smax;
             for (km = k0, smax = 0., kmax = k0; km < k1; km++) if (spectreOrigine[km] > smax) {
               smax = spectreOrigine[km];
               kmax = km;
             }
             if (smax > eForMax[iFor]) {
               eForMax[iFor] = smax;
               echtMaxEner[iFor] = echt;
               freqMaxEner[iFor] = kmax;
             }
           }
         }
         // Once the sample that gives the max of energy one computes the spectrogram that will be used to computes the
         // energies of formants
         for(int i = 0; i < F3N - F1 + 1; i++) if (fi[i + 1] > 0.)
           spectreFft(spectresOrigine[i], space_bis, max(echtMaxEner[i] - ctloc.d_e, 0), false, 512, enEcht(4));

	      halle(spectreExcitation, &space_ter[0], max(dateExcit - nbEchtFen / 2 - ctExcit.d_e, 0), 512, nbEchtFen,
               (int)enEcht(opt_for.nb_cep[0]), (int)enEcht(opt_for.nb_cep[1]), 0., 9, false);
        }

#ifdef UIY
      case SLISCEP: {
        float min_cep, max_cep;
        cepstre(spectreOrigine, &space_bis[0], date - nbEchtFen / 2 - ctloc.d_e, 1, 0, enEcht((int)optKlattSpec.fenCepstre),
                512, 512, (int)enEcht(optKlattSpec.nb_cep[0]), (int)enEcht(optKlattSpec.nb_cep[1]), 1, true, false);
        cepstre(spectreExcitation, &space_ter[0], dateExcit - nbEchtFen / 2 - ctExcit.d_e, 1, 0, enEcht((int)optKlattSpec.fenCepstre),
                512, 512, (int)enEcht(optKlattSpec.nb_cep[0]), (int)enEcht(optKlattSpec.nb_cep[1]), 1, true, false);
        }
        break;
      case SCEPITER :
        halle(spectreOrigine, &space_bis[max(date - nbEchtFen / 2 - ctloc.d_e, 0)], 512,
	           ROUND(en_cours.frequence / 1000. * t_cep[0]),
	           ROUND(en_cours.frequence / 1000. * t_cep[1]), 0, 9, FAUX);
        halle(spectreExcitation, &space_ter[max(dateExcit - nbEchtFen / 2 - ctExcit.d_e, 0)], 512,
	           ROUND(en_cours.frequence / 1000. * t_cep[0]),
	           ROUND(en_cours.frequence / 1000. * t_cep[1]), 0, 9, FAUX);
        break;
#endif
      }
    
      /* Reglage conjoint du formant nasal et de F1 */
      if (fi[1] > 0.) {
#ifdef NASAL
        /* pour l'instant on fixe arbitrairement les largeurs de bande. */
        bw[1] = (fi[1] < 300) ? 40 : 60.;

        /* la largeur de bande du formant nasal est imposee ! */
        if (fi[1] < 300) {
			  /* Si la frequence de F1 est tres faible, on reduit la largeur de
	   		  bande de FN. */
			  fi[0] = max(100.,fi[1] - 100.);
			  if (fi[1] < 200) bw[0] = 40.;
			  else bw[0] = 40 + (fi[1] - 200) / (300 - 200) * 110.;
     	   }
         else bw[0] = defaultFrame.BNPhz;
         for (i = 0; i < 2; i++) {
			  int k = IROUND(fi[i] / en_cours.frequence * FENFFT512);

           // essai simpliste
           /*
            *   |H(z0)|^2 =  (a0 * H0(z0) + a1 * H1(z0)) * conjugue(a0 * H0(z0) + a1 * H1(z0));
            *   |H(z1)|^2 =  (a0 * H0(z1) + a1 * H1(z1)) * conjugue(a0 * H0(z1) + a1 * H1(z1));
            *   z0 est la frequence de FO (le formant nasal) et z1 celle de F1
            * |H(z0)|^2 et |H(z1)|^2 sont les valeurs mesures sur la signal original.
            * H0(z0), H0(z1), H1(z0), H1(z1) sont des complexes et ya pu k'a resoudre avec Maple avec un systeme du genre :
              E1:=a00 * x0^2 + a10 * x1^2 + a20 * x0 * x1 = b0;
              E2:=a01 * x0^2 + a11 * x1^2 + a21 * x0 * x1 = b1;
              klatt := solve({E1,E2}, {x0, x1});
              xx:=[klatt[1],klatt[2]];
              with(CodeGeneration);
             C(xx);
            * mais les constantes a20 et a21 sont des complexes, et le systeme a plusieurs solutions. Maple utilise plusieurs RootOf et je
            * n'ai pas eu le temps d'aller plus loin.
            * donc je fais un choix simpliste consistant  dire qu'il n'y a pas d'interaction entre les deux.
            * La version de 98 est fausse car les produits croises taient oublies....
            */
			  db[i] = spectreOrigine[k]
                   - (spectreExcitation[k] - 10. * log10(2. * (1. - cos(2.* _PI * fi[i] / en_cours.frequence))))
                   - amplitudeFormant(fi[i], bw[i], fi[i]) + 60
         		    + ((mode == SPITCHSYNC) ? 12. : 0);
         }
#else
           int k = IROUND(fi[1] / en_cours.frequence * FENFFT512);
			  /* On supprime la derivation faite dans le synthetiseur de Klatt. */
			  db[1] = spectreOrigine[k]
								- (spectreExcitation[k] - 10. *
			   					log10(2. * (1. - cos(2.* _PI * fi[1] / en_cours.frequence))))
                        - amplitudeFormant(fi[1], bw[1], fi[1]) + 60
         		   + ((mode == SPITCHSYNC) ? 12. : 0);
#endif
       	}

    	  /* ON traite les autres formants et on ne fait le reglage que si le formant est defini. */
        for (i = 2; i < NBFORKLATT_EXT+1; i++) if (fi[i] > 0.) {
     	    int k, k0, k1, km, kmax;
     	    float smax,
                enerOrigine,
			  	    enerExcit,
                enerFormant;

          k0 = max(0, IROUND((fi[i] - 100.)/ en_cours.frequence * FENFFT512)),
          k1 = min(FENFFT512 / 2, IROUND((fi[i] - 100.)/ en_cours.frequence * FENFFT512));

          for (km = k0, smax = 0., kmax = k0; km < k1; km++) if (spectresOrigine[i - F1][km] > smax) {
             smax = spectresOrigine[i - F1][km];
             kmax = km;
          }

          k = km;
          enerOrigine = spectresOrigine[i - F1][k];
          enerExcit = spectreExcitation[k];
          bw[i] = 50. + i*i * 10.;	/* pour l'instant on fixe arbitrairement les largeurs de bande. */

		    enerFormant = amplitudeFormant(fi[i], bw[i], fi[i]);

		    db[i] = enerOrigine - enerExcit - enerFormant + 60.
         		   + ((mode == SPITCHSYNC) ? 12. : 0);
    	  }
        else {
          bw[i] = 50. + i*i * 10.;
          db[i] = 0.;
        }

      /* On traite F1 ET FN ensemble. */
      if (skeletonDefined && squelette.F1hz) {
        frame.FNPhz = fi[0];
        frame.BNPhz = bw[0];
        frame.ANP = db[0];
        frame.F1hz = fi[1];
        frame.B1phz = bw[1];
        frame.A1 = db[1];
      }
      if (skeletonDefined && squelette.F2hz) {
        frame.F2hz = fi[2];
        frame.B2phz = bw[2];
        frame.A2 = db[2];
      }
      if (skeletonDefined && squelette.F3hz) {
        frame.F3hz = fi[3];
        frame.B3phz = bw[3];
        frame.A3 = db[3];
      }
      if (skeletonDefined && squelette.F4hz) {
        frame.F4hz = fi[4];
        frame.B4phz = bw[4];
        frame.A4 = db[4];
      }
      if (skeletonDefined && squelette.F5hz) {
        frame.F5hz = fi[5];
        frame.B5phz = bw[5];
        frame.A5 = db[5];
      }
      if (skeletonDefined && squelette.F6hz) {
        frame.F6hz = fi[6];
        frame.B6phz = bw[6];
        frame.A6 = db[6];
      }
      if (skeletonDefined && squelette.F1Nhz) {
        frame.F1Nhz = fi[7];
        frame.B1Nphz = bw[7];
        frame.A1N = db[7];
      }
      if (skeletonDefined && squelette.F2Nhz) {
        frame.F2Nhz = fi[8];
        frame.B2Nphz = bw[8];
        frame.A2N = db[8];
      }
      if (skeletonDefined && squelette.F3Nhz) {
        frame.F3Nhz = fi[9];
        frame.B3Nphz = bw[9];
        frame.A3N = db[9];
      }
      sauveFrameKlatt(g, frame, NPARLE);
    }
  }
  Wact->pourcent(1,1);
  Wact->hidepourcent();
  fclose(f);
  fclose(g);
  fclose(h);
}

int Wpiquefor::chercherMarque(
  int echt)
{
  int m;
  for (m = 0; m < nbMarques && fabs(marquesPitch[m]) < echt; m++);
  if (m < nbMarques && fabs(marquesPitch[m]) >= echt) return (m);
  else return(-1);
}

/*
 * Amplitude de la fonction de transfert d'un formant.
 */
float Wpiquefor::amplitudeFormant(
  float Fi,	/* frequence du formant. */
  float bw,	/* largeur de bande du formant */
  float f)	/* frequence ou calculer l'amplitude. */
{
  float b = 2.* exp(- _PI * bw / en_cours.frequence) * cos(2.* _PI * Fi / en_cours.frequence),
        c = -exp(-2. * _PI * bw / en_cours.frequence),
        a = 1 - b - c;
  fcomplex zinv = Cinv(CComplex(cos(2. * _PI * f / en_cours.frequence), sin(2. * _PI * f / en_cours.frequence))),
    h = RCmul(a, Cinv(Csub(Csub(CComplex(1.,0.), RCmul(b, zinv)), RCmul(c, Cmul(zinv,zinv)))));
  return(20. * log10(Cabs(h)));
}


void initfpattern(){
   int i;
   for (i = 0; i < NBMAJEURS; i++) {
    	Item *q;
    	for (q = fpattern[i].list; q; q = q->next) q->val->selectionne(true);
   }
	for (i = 0; i < NBMAJEURS; i++) fpattern[i].removeIfSelec();
}

void Wpiquefor::CMinitialFTracks()
{
  if (typeImageSnake == SnakeImageCepstral) CMcalculImCeps();
  else if (typeImageSnake == SnakeImageTrueEnv) CMcalculImTrueEnv();
  else CMcalculImLPC();
  if (ct.debRegion > ct.finRegion)  {
    int start = max(ct.d_e, ct.d_a),
        end = min(ct.f_e, ct.f_a);
    if (end > start) suivreFormantsSimultane(start, end, true, false, typeInitialSnakes);
  }
  else suivreFormantsSimultane(ct.debRegion, ct.finRegion, true, false, typeInitialSnakes);
}

void Wpiquefor::CMtrackingFromExistingSelected()
{
  if (typeImageSnake == SnakeImageCepstral) CMcalculImCeps();
  else if (typeImageSnake == SnakeImageTrueEnv) CMcalculImTrueEnv();
  else CMcalculImLPC();
  if (ct.debRegion > ct.finRegion)  {
    int start = max(ct.d_e, ct.d_a),
        end = min(ct.f_e, ct.f_a);
    if (end > start) suivreFormantsSimultane(start, end, false, true, typeInitialSnakes);
  }
  else suivreFormantsSimultane(ct.debRegion, ct.finRegion, false, true, typeInitialSnakes);
}

void Wpiquefor::CMtrackingFromExisting()
{
  if (typeImageSnake == SnakeImageCepstral) CMcalculImCeps();
  else if (typeImageSnake == SnakeImageTrueEnv) CMcalculImTrueEnv();
  else CMcalculImLPC();
  if (ct.debRegion > ct.finRegion)  {
    int start = max(ct.d_e, ct.d_a),
        end = min(ct.f_e, ct.f_a);
    if (end > start) suivreFormantsSimultane(start, end, false, false, typeInitialSnakes);
  }
  else suivreFormantsSimultane(ct.debRegion, ct.finRegion, false, false, typeInitialSnakes);
}


void Wpiquefor::CMcalculImCeps()
{
  typeImageSnake = SnakeImageCepstral;
  chCalculIm();
  calculImCeps(&space[0], ct.d_e, ct.d_a - ct.d_e, min(ct.f_e, ct.f_a) - ct.d_e, en_cours.nom, false);
}

void Wpiquefor::CMcalculImTrueEnv()
{
  typeImageSnake = SnakeImageTrueEnv;
  chCalculIm();
  calculImCeps(&space[0], ct.d_e, ct.d_a - ct.d_e, min(ct.f_e, ct.f_a) - ct.d_e, en_cours.nom, true);
}

void Wpiquefor::CMcalculImLPC()
{
  typeImageSnake = SnakeImageLPC;
  chCalculIm();
  calculImLPC(&space[0], ct.d_e, ct.d_a - ct.d_e, min(ct.f_e, ct.f_a) - ct.d_e, en_cours.nom);
}

void Wpiquefor::CMsavePGM()
{
  STRING filename;

  if (!yaUnSignal) return;

  strcpy(filename,"*.*");
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "*.*", 0,"","");
  strcpy(filename,"*.*");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (TFileSaveDialog(Parent,FileData, 0).DoExecute() == IDOK) {
     strcpy(filename,FileData.FileName);
     if (droit_ecrire_message(filename))
        saveImSpectroSnakePGM(filename);
  }
}

void Wpiquefor::chCalculIm()
{
  WORD c=MF_CHECKED, u=MF_UNCHECKED;

  CheckMenuItem(::GetMenu(HWindow), MENUcalculimCepstral, (typeImageSnake == SnakeImageCepstral)?c:u);
  CheckMenuItem(::GetMenu(HWindow), MENUcalculimTrueEnvelope, (typeImageSnake == SnakeImageTrueEnv)?c:u);
  CheckMenuItem(::GetMenu(HWindow), MENUcalculimLPC,(typeImageSnake == SnakeImageLPC)?c:u);
}

void Wpiquefor::CMchInitialTracksLPC()
{
  typeInitialSnakes = InitialCurvesLPC;
  chInitialTracks();
}

void Wpiquefor::CMchInitialTracksTrueenv()
{
  typeInitialSnakes = InitialCurvesTrueenv;
  chInitialTracks();
}

void Wpiquefor::chInitialTracks()
{
  WORD c=MF_CHECKED, u=MF_UNCHECKED;

  CheckMenuItem(::GetMenu(HWindow), MENUinitialCurvesLPC, (typeInitialSnakes == InitialCurvesLPC)?c:u);
  CheckMenuItem(::GetMenu(HWindow), MENUinitialCurvesTrueenv, (typeInitialSnakes == InitialCurvesTrueenv)?c:u);
}

//
//  Affichage dans le plan F1-F2 des PolyPoints.
//
void Wpiquefor::affPPF1F2(
  long debRegion,
  long finRegion,
  d_planche *fenetre,
  COLORREF couleur)
{
  ifstream f;
  STRING ficTempFormant;
  float F1old, F2old;

  strcat(strcpy(ficTempFormant, tempdir), tmpnam((char *) NULL));
  sauvePP(ficTempFormant, debRegion, min(ct.tf, finRegion) , true, 5, NPARLE);

  snoiwidth(3);
  f.open(ficTempFormant);
  if (!f) {
    affiche_erreur(ml(6405,"affPPF1F2() : lecture des formants impossible"));
    return;
  }

  F1old = F2old = -1.;
  while (!f.eof()) {
    STRING buf;
    float fi[NBFORKLATT_EXT],db[NBFORKLATT_EXT],bw[NBFORKLATT_EXT];
    int date, datems;
    f.getline(buf, MAX_LIGNE, '\n');
    if (strlen(buf)) {
      readFormantFrame(buf, date, datems, fi, db, bw);
      if (fi[0] > EPSILON && fi[1] > EPSILON) {
        if (F1old > 0.)
			 snortl(F2old, F1old, fi[1], fi[0], fenetre);
     	  else {
          MgiHue(VERT, fenetre);
          float xl,yl; // largeur du trac (environ 3 pixels)

			 xl=fabs(fenetre->DXtoLX(0)-fenetre->DXtoLX(8));
			 yl=fabs(fenetre->DYtoLY(0)-fenetre->DYtoLY(8));
			 fenetre->Lellipse(fi[1], fi[0], xl/2,yl/2); //25., 25.);
			 MgiHue(couleur, fenetre);
      	}
       F1old = fi[0];
       F2old = fi[1];
      }
    }
  }
  f.close();
  ::InvalidateRect(fenetre->HWindow,NULL,TRUE);
  remove(ficTempFormant);
}

#ifdef UIY
class TWFileOpenSave : public TOpenSaveDialog
{
  public:
    TWdescform(PTWindowsObject AParent, LPSTR name);
    virtual void SetupWindow();
    virtual bool CanClose();
};
#endif

