/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier guiklat.h
//

#ifndef _GUIKLAT_H
#define _GUIKLAT_H

#include <windows.h>
#include "common.h"
#include "filtre.h"
#include "SuiviSnake/snake.h"
#include "SuiviSnake/calculim.h"
#include "SuiviSnake/suiviSnake.h"
#define FENFRIC 256

#define ID_KLATT0	11000   // id du premier param de Klatt

#define NBFORKLATT_EXT	9   // The new number of formants (with the 3 additional nasal formants)  EXT for extended
#define NBFORKLATT_OLD  6 // The old number of formants (used in functions for compatibility)

#define DEFAULTDURATION  3500		//default duration (in ms) when no speech signal is available

#define SCALEFr		0
#define SCALEDBP		1
#define SCALEBwC		2
#define SCALEBwP		3
#define SCALEDBC		4
#define SCALEF0		5
#define SCALEDNb		6

#define F0KLATTMAX	800.	/* valeur maximale de F0. */
#define NPURMAX		100.	/* valeur maximale d'un nombre sans grandeur (Nopen par exemple) */
#define DBMIN 			-20.
#define DBMAX 			120.
#define BWMAX 			700.

#define FORMANT 0
#define SPECTRO 1

#define SRACLPC		0  // type of registering
#define SLISCEP		1
#define SCEPITER		2
#define SPITCHSYNC	3

/* indices majeurs (les indices des polypoints) */
#define F0		0
#define F1 		1
#define F2 		2
#define F3 		3
#define F4 		4
#define F5 		5
#define F6 		6
#define F1N		7 
#define F2N		8
#define F3N		9
#define FNZ		10
#define FNP		11
#define ASPIRATION	12
#define KOPEN		13
#define ATURB		14
#define TLTDB		15
#define AFRICTION	16
#define KSKEW		17
#define ABYPASS		18
#define GAIN		19
#define RaLF      20
#define RkLF      21
#define RgLF      22
#define NBMAJEURS 	23

#define FTOUS		-1
#define FAUCUN 	-2

/* indices mineurs (les champs des majeurs) */
#define DATE		   0
#define FREQUENCE 	0
#define AMPLITUDE	   1
#define LARGBANDEC	2
#define LARGBANDEP	3
#define AMPLITUDEF0P	2	/* pour F0 */
#define LARGBANDECFNZ	1  /* pour FNZ */

// Constants to control the Klatt command line
#define PARALLEL 		0
#define CASCPARALL	1

#define VOICING   	1
#define SASPIRATION  2   // source aspiration to avoid confusion with aspiration above
#define FRICATION    3
#define CASCADEOUT   4
#define PARALLOUT    5
#define BYPASS       6
#define ALLEXCIT     7
#define COMPLETE     8

#define LOWBYTE		1
#define HIGHBYTE		2

#define KLATTSOURCE	1
#define LFSOURCE		2

#define SAM8000 		1
#define  SAM11025 	2
#define SAM16000		3
#define SAM22050 		4

#include "toolsformants.h"

typedef struct
{
  int Date;   /* Date of the frame in ms.                         */
  int F0hz10; /* Voicing fund freq in Hz                          */
  int AVdb;   /* Amp of voicing in dB,            0 to   70       */
  int F1hz;   /* First formant freq in Hz,        200 to 1300     */
  int B1hz;   /* First formant bw in Hz,          40 to 1000      */
  int F2hz;   /* Second formant freq in Hz,       550 to 3000     */
  int B2hz;   /* Second formant bw in Hz,         40 to 1000      */
  int F3hz;   /* Third formant freq in Hz,        1200 to 4999    */
  int B3hz;   /* Third formant bw in Hz,          40 to 1000      */
  int F4hz;   /* Fourth formant freq in Hz,       1200 to 4999    */
  int B4hz;   /* Fourth formant bw in Hz,         40 to 1000      */
  int F5hz;   /* Fifth formant freq in Hz,        1200 to 4999    */
  int B5hz;   /* Fifth formant bw in Hz,          40 to 1000      */
  int F6hz;   /* Sixth formant freq in Hz,        1200 to 4999    */
  int B6hz;   /* Sixth formant bw in Hz,          40 to 2000      */
  int FNZhz;  /* Nasal zero freq in Hz,           248 to  528     */
  int BNZhz;  /* Nasal zero bw in Hz,             40 to 1000      */
  int FNPhz;  /* Nasal pole freq in Hz,           248 to  528     */
  int BNPhz;  /* Nasal pole bw in Hz,             40 to 1000      */
  int ASP;    /* Amp of aspiration in dB,         0 to   70       */
  int Kopen;  /* # of samples in open period,     10 to   65      */
  int Aturb;  /* Breathiness in voicing,          0 to   80       */
  int TLTdb;  /* Voicing spectral tilt in dB,     0 to   24       */
  int AF;     /* Amp of frication in dB,          0 to   80       */
  int Kskew;  /* Skewness of alternate periods,   0 to   40 in sample#/2  */
  int A1;     /* Amp of par 1st formant in dB,    0 to   80       */
  int B1phz;  /* Par. 1st formant bw in Hz,       40 to 1000      */
  int A2;     /* Amp of F2 frication in dB,       0 to   80       */
  int B2phz;  /* Par. 2nd formant bw in Hz,       40 to 1000      */
  int A3;     /* Amp of F3 frication in dB,       0 to   80       */
  int B3phz;  /* Par. 3rd formant bw in Hz,       40 to 1000      */
  int A4;     /* Amp of F4 frication in dB,       0 to   80       */
  int B4phz;  /* Par. 4th formant bw in Hz,       40 to 1000      */
  int A5;     /* Amp of F5 frication in dB,       0 to   80       */
  int B5phz;  /* Par. 5th formant bw in Hz,       40 to 1000      */
  int A6;     /* Amp of F6 (same as r6pa),        0 to   80       */
  int B6phz;  /* Par. 6th formant bw in Hz,       40 to 2000      */
  int ANP;    /* Amp of par nasal pole in dB,     0 to   80       */
  int AB;     /* Amp of bypass fric. in dB,       0 to   80       */
  int AVpdb;  /* Amp of voicing,  par in dB,      0 to   70       */
  int Gain0;  /* Overall gain, 60 dB is unity,    0 to   60       */
  int Ra;	  /* Ra (LF source), x 100				0 to 20			 */
  int Rk;     /* Rk (LF source), x 100				0 to 70         */
  int Rg;     /* Rg (LF source), x 100		      50 to 200       */

  int F1Nhz;   
  int B1Nhz;
  int A1N;
  int B1Nphz;

  int F2Nhz;
  int B2Nhz;
  int A2N;
  int B2Nphz;

  int F3Nhz;
  int B3Nhz;
  int A3N;
  int B3Nphz;
 } klatt_frame_t, *klatt_frame_ptr;

#define Date_	0	/* Date of the frame in ms.                         */
#define F0hz10_	1	/* Voicing fund freq in Hz                          */
#define AVdb_	2	/* Amp of voicing in dB,            0 to   70       */
#define F1hz_	3   	/* First formant freq in Hz,        200 to 1300     */
#define B1hz_	4   	/* First formant bw in Hz,          40 to 1000      */
#define F2hz_	5   	/* Second formant freq in Hz,       550 to 3000     */
#define B2hz_	6   	/* Second formant bw in Hz,         40 to 1000      */
#define F3hz_	7   	/* Third formant freq in Hz,        1200 to 4999    */
#define	B3hz_	8   	/* Third formant bw in Hz,          40 to 1000      */
#define F4hz_	9   	/* Fourth formant freq in Hz,       1200 to 4999    */
#define B4hz_	10   	/* Fourth formant bw in Hz,         40 to 1000      */
#define F5hz_	11   	/* Fifth formant freq in Hz,        1200 to 4999    */
#define B5hz_	12   	/* Fifth formant bw in Hz,          40 to 1000      */
#define F6hz_	13   	/* Sixth formant freq in Hz,        1200 to 4999    */
#define B6hz_	14   	/* Sixth formant bw in Hz,          40 to 2000      */
#define FNZhz_	15  	/* Nasal zero freq in Hz,           248 to  528     */
#define BNZhz_	16  	/* Nasal zero bw in Hz,             40 to 1000      */
#define FNPhz_	17  	/* Nasal pole freq in Hz,           248 to  528     */
#define	BNPhz_	18  	/* Nasal pole bw in Hz,             40 to 1000      */
#define ASP_	19    	/* Amp of aspiration in dB,         0 to   70       */
#define Kopen_	20  	/* # of samples in open period,     10 to   65      */
#define Aturb_	21  	/* Breathiness in voicing,          0 to   80       */
#define TLTdb_	22  	/* Voicing spectral tilt in dB,     0 to   24       */
#define AF_	23     	/* Amp of frication in dB,          0 to   80       */
#define Kskew_	24  	/* Skewness of alternate periods,   0 to   40 in sample#/2  */
#define A1_	25     	/* Amp of par 1st formant in dB,    0 to   80       */
#define B1phz_	26  	/* Par. 1st formant bw in Hz,       40 to 1000      */
#define A2_	27     	/* Amp of F2 frication in dB,       0 to   80       */
#define B2phz_	28  	/* Par. 2nd formant bw in Hz,       40 to 1000      */
#define A3_	29     	/* Amp of F3 frication in dB,       0 to   80       */
#define B3phz_	30  	/* Par. 3rd formant bw in Hz,       40 to 1000      */
#define A4_	31     	/* Amp of F4 frication in dB,       0 to   80       */
#define B4phz_	32  	/* Par. 4th formant bw in Hz,       40 to 1000      */
#define A5_	33     	/* Amp of F5 frication in dB,       0 to   80       */
#define B5phz_	34  	/* Par. 5th formant bw in Hz,       40 to 1000      */
#define A6_	35     	/* Amp of F6 (same as r6pa),        0 to   80       */
#define B6phz_	36  	/* Par. 6th formant bw in Hz,       40 to 2000      */
#define ANP_	37    	/* Amp of par nasal pole in dB,     0 to   80       */
#define AB_	38     	/* Amp of bypass fric. in dB,       0 to   80       */
#define AVpdb_	39  	/* Amp of voicing,  par in dB,      0 to   70       */
#define Gain0_	40  	/* Overall gain in dB.				    */
#define Ra_ 41  		/* Ra (LF source), x 100				0 to 20			 */
#define Rk_	42  		/* Rk (LF source), x 100				0 to 70         */
#define Rg_	43  		/* Rg (LF source), x 100		      50 to 200       */

#define F1Nhz_	44
#define B1Nhz_	45
#define A1N_   46
#define B1Nphz_  47

#define F2Nhz_	48
#define B2Nhz_	49
#define A2N_	50
#define B2Nphz_	51

#define F3Nhz_	52
#define B3Nhz_ 53
#define A3N_	54
#define B3Nphz_ 55

#define NPAR	44
#define NPARLE 56
#define NPARKLATT 41

#include "PolyPoint.h"
#include "polyklat.h"
#include "listyl.h"
#include "class_sig.h"
#include "mccandle.h"

extern Listyl fpattern[NBMAJEURS];


struct optKlattSpec {
  float or_lpc;    	// ordre de lpc donn en ms
  float fenLPC;	 	// fenetre LPC en ms
  float nb_cep[2]; 	// parametres du liftrage cepstral
  float fenCepstre; 	// fenetre cepstrale en ms
};

// Parameters given to control the synthesizer from the command line
struct ctrlCommandKlatt {
  int samplingRate;
  int branch;
  int frame;
  int output;
  int source; // 0 Klatt 1 Liljencrants-Fant
  int flutter;
  int byteArrang;
};

class Wpiquefor : public WSignal
{
    int typeFor;
    char *menu;
    TabPoint<Point2P> * ppp;       // Polypoints being read.
    PolyPointsKla     * ppointModif;     // PolyPoints being modified.
    struct ctrl_temps ctImage;
	 bool syntheExists;
    bool yaUneFenetreToolsFormants;
    Wmccandless *wmccandless;
    SAMPLEbuf * space_lowFreq, * space_highFreq, * space_ener;
    class profiltre *lowFreqFilter, *highFreqFilter;
    int correlationSizeMs;
    int cbas, chaut, cF0;
    SAMPLEbuf space_ter;
    INTbuf marquesPitch;
    int nbMarques;

  public:
    WToolsFormants *tools;
   // modified by ghazi
    TabPoint<Point2P> * get_ppp() { return this->ppp;}
    void removeWToolsFormants()
    {
    	if (tools) delete tools;
      tools=NULL;
    }

    int xPtModif;       	      // Coodinates of the point being tracked.
	 int yPtModif;
	 int indicePtModif;
    int paramPtModif;
    YFCT yecranModif;
	 int focusModif;
    int event_x_g;       			// X Boundaries of the domain where the point
    int event_x_d;               // can be moved.

    TypeSnakeImage typeImageSnake;
    TypeInitialSnakes typeInitialSnakes;

    struct ctrl_temps ctEner;  // dedicated to the computation of energy and voicing/frication decision
    struct optKlattSpec optKlattSpec;
    struct ctrlCommandKlatt ctrlCommandKlatt;


   // Listyl fpattern[NBMAJEURS];
    Wpiquefor();
    ~Wpiquefor();
    void BuildMenu();
    void CheckMenuItemPas();
    void CheckMenuItemType();
    virtual void SetupWindow();
    virtual bool CanClose();
	 // rponses aux vnements commande menu
	 void CMdetruireFor();
	 void CMlireFor();
    void CMlireForLF();
    void CMpasteKlattFile();
	 void CMsaveKlatt();
	 void CMsaveKlattLF();
    void CMrecalForRACLPC();
	 void CMrecalForLISCEP();
	 void CMlisseFor();
    void CMinitialFTracks();
    void CMtrackingFromExisting();
    void CMtrackingFromExistingSelected();
    void CMcalculImCeps();
    void CMcalculImTrueEnv();
    void CMcalculImLPC();
    void CMsavePGM();
    void CMremoveFirstPoint();
    void CMremoveLastPoint();
	 void CMselectF1();
	 void CMselectF2();
	 void CMselectF3();
	 void CMselectF4();
	 void CMselectF5();
	 void CMselectF6();
    void CMselectF1N();
    void CMselectF2N();
    void CMselectF3N();
	 void CMselectTous();
	 void CMselectAucun();
    void CMmergeFor();
	 void CMpas2ms();
	 void CMpas4ms();
	 void CMpas8ms();
	 void CMpas16ms();
    void CMTypeFreq();
    void CMSauveKlatt();
    void CMsaveSyntheticSpeech();
	 void CMhelpMouse();
	 void CMchScaleFreq();
	 void CMchScaleDBP();
	 void CMchScaleDBC();
	 void CMchScaleBWC();
	 void CMchScaleBWP();
	 void CMchScaleF0();
	 void CMchScaleNum();
    void CMchAffSquares();
	 void CMchAffLines();
    void CMopcldesc();
    void CMsynthese();
    void CMcreePPF0();
    void CMplayKlatt();
    void CMzoominKlatt();
    void CMzoomoutKlatt();
    void CMchangeAnaSpec();
    void CMSettingsKlatt();
    void CMsaveFormants();
    void CMlectureFicFormants();
    void CMaffFormantsF1F2();
    void CMcopySynthesis();
    void CMSnakeParametersControl();
    void CMchInitialTracksLPC();
    void CMchInitialTracksTrueenv();
	 void aff();
    void drawDown(int event_x,int event_y);
    void drawMotion(int event_x,int event_y);
    void drawUp(int event_x,int event_y);

    void drawDownPiqPt(int event_x,int event_y);
    void drawMotionPiqPt(int event_x,int event_y);
    void drawUpPiqPt(int event_x,int event_y);

    PolyPointsKla *Wpiquefor::trouvePt(int date, float ordonnee, int *indice, int *param);
    PolyPointsKla *trouvePtFor(int ipar, int date, float y, int *indice, int focus, int onForce);

    void CMTools();
    void CMSetParam();
    void CMFormant();
    void CMSpectro();
    void CMrecalFor(int mode);


    void initOptKlattSpec();
    void initCtrlCommandKlatt();
    void initFilters();
    void chTypeFor();
    void chScaleKlatt();
    void chTypeAff();
    void chCalculIm();
    void chInitialTracks();
    void selectFor(int);
    void synthetise(char *filename);
    void synthesizeAndDisplay(bool newDisplay);
    void displaySyntheticSpeech(bool newDisplay, char *resFile);
    void readFormantFrame(STRING buf, int& date, int& datems, float fi[NBFORKLATT_OLD],
  			float db[NBFORKLATT_OLD], float bw[NBFORKLATT_OLD]);
    void lectureFicFormants(char *filename, Listyl newFpattern[NBMAJEURS]);
    void affEchelle(float, float, float, char *);
    int datePremPP(bool force);
    int dateLastEndPP(bool force);
    void lengthenShadow();

    void affPPoints();
    void detruitTousPP();     
    void fusionnePP();
    void selectTous();
    bool isNoPPSel();
    void resolConflits(int iKlatt);
	 void selectTous (bool mode);
    void resampleAll();
    void selectPPoint(PolyPointsKla *ppoint, int mode);
    YFCT valueToFcty(int majeur, int scale);
	 float domyParam(int kpar);
    void arretModifPP();
    void selectPPoint();
    int oupp(Item *ppfi[]);
    void sauveFrameKlatt(FILE *f, klatt_frame_t frame, int Npar);
    void lireForKlattOrLF();
    int sauvePP(char *filename, int debutEcht, int finEcht, bool force, int klatt, int Npar);
    bool lireFrameKlatt(FILE *f, klatt_frame_t& frame, int npar);
    void lirePP (char *filename, int offset, Listyl newFpattern[NBMAJEURS]);
    void dateDebFinKlatt(int *deb, int *fin);
    void dateDebFinFilePP(char *filename, int *deb, int *fin);
    void validatePPP();
    void updateWithNewFpattern(Listyl newFpattern[NBMAJEURS]);
    void affPPF1F2(long debRegion, long finRegion, d_planche *fenetre, COLORREF couleur);
    void recalPPSpectre (char *nomFicFormant, char *nomSquelette, char *nomFicFormantCorrige, int mode);
    void recalCopySynthesis (char *nomFicFormant, char *nomSquelette, char *nomExcitation,
                             char *nomFicFormantCorrige, int mode, int dateEndSkeleton);
    PolyPointsKla* resolConflictPP(PolyPointsKla *npp, int mj);
    TabPoint<Point2P>*  Wpiquefor::extractOneNewPP();
    float voicingFricationRatio(float f0, int echt, float &lowFreqEnergy, float &highFreqEnergy);
    float voicingRatio(SAMPLEbuf &speech, float f0, int windowSize, int echt);
    void calculEnerFric(SAMPLEbuf& parole, int echt, float &lowFreqEnergy, float &highFreqEnergy);
    float amplitudeFormant(float Fi, float bw, float f);
    int chercherMarque(int echt);
    // rponses aux vnements
	 /*
	 virtual void WMSize(RTMessage) = [WM_FIRST+WM_SIZE];
	 */
	 void EvSize(uint sizeType, TSize &size);
	 DECLARE_RESPONSE_TABLE(Wpiquefor);
};


// fichier piquefor.cpp
extern bool yaUneFenetrePiquefor;

extern void drawUpPiqSelect(int event_x,int event_y);

extern void drawDownPiqPt(int event_x,int event_y);
extern void drawMotionPiqPt(int event_x,int event_y);
extern void drawUpPiqPt(int event_x,int event_y);

void affPPF1F2(long debRegion,long finRegion,
	      d_planche *fenetre,
               COLORREF couleur);
void piqueforClear();
extern int majeurMineur(int ipar,int *mineur,int *dim);

extern void translatePPselectionne(int incr);

/////////////////////////////////////////////
// fentre Wpiqueforchoosef
//
class TWpiqueforchoosef : public TDialog
{
	 TRadioButton *butPar[NBMAJEURS];
    int nParam;
    int height;
    TGroupBox *box;
    TButton *ok, *cancel;
 public:
    TWpiqueforchoosef(PTWindowsObject AParent, char *title);
    ~TWpiqueforchoosef();
    void IDok(WPARAM code);
	 void IDcancel(WPARAM code);

    virtual void SetupWindow();
    virtual bool CanClose();
    DECLARE_RESPONSE_TABLE(TWpiqueforchoosef);
};

extern COLORREF couleurParam(int param);

#endif
