/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier hamming.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include "fftlpc.h"

#define DEUX_PI			(2 * _PI)

static FLOATbuf hamming(TAILLE_MAX_HAMMING,"hamming");
static FLOATbuf hanning(TAILLE_MAX_HAMMING,"hanning");
static FLOATbuf hammingDeriv(TAILLE_MAX_HAMMING, "hamming driv");
static FLOATbuf hammingTimeMul(TAILLE_MAX_HAMMING, "hamming multipli par temps");

static float hamming_[TAILLE_MAX_HAMMING];
static double hamming_d[TAILLE_MAX_HAMMING];

static long nombre_de_points;

/*
 * Cette fonction initialise une fenetre de Hamming.
 * Si le pointeur sur le tableau est different de NULL
 * on initialise une fenetre qui n'est pas la fenetre 
 * privee de ce module.
 * Si le pointeur est egal a NULL on initialise la 
 * fenetre hamming (declaree dans ce module).
 */
                                                                        
void initham(
     FLOATbuf *h,
     long nb_points)
{
  int i;
  float c1 = 0.54,
  c2 = 0.46;
  float *pham;

  if (nb_points>TAILLE_MAX_HAMMING) {
    affiche_bug("initham(): too many points");
    return;
  }

  nombre_de_points = nb_points;
  pham= (float *)((h == NULL) ? &hamming[0] : &(*h)[0]);
  for (i = 0; i < nb_points; i++)
    pham[i]
      =(float)(c1-c2*cos( (double)(DEUX_PI*i/nb_points) )) ;
}

void initham_d(
     double *h,
     long nb_points)
{
  int i;
  double c1 = 0.54,
  c2 = 0.46;
  double *pham;

  if (nb_points > TAILLE_MAX_HAMMING) {
    affiche_bug("initham(): too many points");
    return;
  }

  nombre_de_points = nb_points;
  pham = (double *)((h == NULL) ? hamming_d : h);
  for (i = 0; i < nb_points; i++)
    pham[i] =(double)(c1-c2*cos( (double)(DEUX_PI*i/nb_points))) ;
}

void initham_(
     float *h,
     long nb_points)
{
  int i;
  float c1 = 0.54,
  c2 = 0.46;
  float *pham;

  if (nb_points > TAILLE_MAX_HAMMING) {
    affiche_bug("initham(): too many points");
    return;
  }

  nombre_de_points = nb_points;
  pham = (float *)((h == NULL) ? hamming_ : h);
  for (i = 0; i < nb_points; i++)
    pham[i] =(float)(c1-c2*cos( (double)(DEUX_PI*i/nb_points))) ;
}

// initialization of the hamming window derivated
void initHamDeriv (
    FLOATbuf *h,
    int nb_points)
{
  int i;

  float c1 = 0.54,
	     c2 = 0.46;
  float *hamDeriv = &(*h)[0];
  if (nb_points>TAILLE_MAX_HAMMING) {
	affiche_bug("initHamDeriv(): too many points");
	return;
  }

  nombre_de_points = nb_points;
  hamDeriv = (hamDeriv == NULL) ? &hammingDeriv[0] : hamDeriv;
  for (i = 0; i < nb_points; i++)
	 hamDeriv[i] = (float) (c2 * DEUX_PI / nb_points * sin( (double)  (DEUX_PI * i / nb_points) )) ;
}

// initialization of the hamming window multiplied by time
void initHamTimeMul(
    FLOATbuf *h,
    int nb_points)
{
    int i;

    float c1 = 0.54,
	       c2 = 0.46;
    float *hamTime = &(*h)[0];
    if (nb_points > TAILLE_MAX_HAMMING) {
	   affiche_bug("initHamTime(): too many points");
   	return;
    }

    nombre_de_points = nb_points;
    hamTime = (hamTime == NULL) ? &hammingTimeMul[0] : hamTime;
    for (i = 0; i < nb_points; i++)
	   hamTime[i] = (float) (i * (c1 - c2 * cos( (double)  (DEUX_PI * i / nb_points)) )) ;
}

void inithan(
     FLOATbuf *h,
     long nb_points)
{
  int i;
  float c1 = 0.5,
  c2 = 0.5;
  float *phan;

  if (nb_points>TAILLE_MAX_HAMMING)
  { affiche_bug("inithan(): too many points");
    return;
  }
  
  nombre_de_points = nb_points;	
  phan= (float *)((h == NULL) ? &hanning[0] : &(*h)[0]);
  for (i = 0; i < nb_points; i++)
    phan[i]
      = 0.5 * (1. - cos( (double)(DEUX_PI*i/nb_points) )) ;
}


/*
 * Preaccentuation du signal
 */
void preaccentuation(
     SAMPLEbuf& cnv,
     long cnv_debut,
     float *res,
     long nb_points)
{
  int i;
  SAMPLE huge *pcnv;

  pcnv=(SAMPLE huge *)&cnv[cnv_debut];

  for (i = 0; i < nb_points; i++)
    res[i] = (float) (pcnv[i+1] - pcnv[i]);
  return;
}

// Application d'une fentre d'analyse quelconque
void applyWindow(
     float *speech,
     float *result,
     FLOATbuf *window,
     long nb_points)
{
  int i;
  float *pwindow;

  pwindow = (float *) (&(*window)[0]);

  for (i = 0; i < nb_points; i++)
    result[i] = (float) (speech[i] * pwindow[i]);

  return;
}

/*
 * Cette fonction calcule simultanement 
 * la fenetre de hamming sur un signal. Comme la fonction
 * d'initialisation
 * si le pointeur sur la fenetre est NULL on travaille
 * sur la fenetre privee
 * et dans de cas on ne tient pas compte de nb_points.
 */

/* donnees short, resultat float */
void ham_sf(
     SAMPLEbuf& cnv,
     long cnv_debut,
     FLOATbuf& res,
     FLOATbuf *h,
     long nb_points)
{ int i;
  float *pham,*pres;
  SAMPLE huge *pcnv;

  pham= (float *)((h == NULL) ? &hamming[0] : &(*h)[0]);
  pcnv=(SAMPLE huge *)&cnv[cnv_debut];
  pres=(float *)&res[0];

  for (i=0;
       i< ((h==NULL) ? nombre_de_points : nb_points);
       i++) 
    pres[i] = (float) (pcnv[i] * pham[i]);
  return;
}

/* donnees short, resultat float */
void ham_sf_(
     short *cnv,
     long cnv_debut,
     float *res,
     float *h,
     long nb_points)
{
  int i;
  float *ham = ((h == NULL) ? hamming_ : h);
  for (i=0; i< ((h==NULL) ? nombre_de_points : nb_points); i++)
    res[i] = (float) (cnv[i] * ham[i]);
}

/* donnees float, resultat float */
void ham_ff(
     FLOATbuf& cnv,
     FLOATbuf& res,
     FLOATbuf *h,
     long nb_points)
{
  int i;
  float *pham,*pres,*pcnv;

  pham= (float *)((h==NULL) ? &hamming[0] : &(*h)[0]);
  pcnv=(float *)&cnv[0];
  pres=(float *)&res[0];

  for (i = 0;
       i< ((h==NULL) ? nombre_de_points : nb_points);
       i++)
    pres[i] = pcnv[i] * pham[i];
}

/* donnees float, resultat float */
void ham_ff_(
     float *cnv,
     float *res,
     float *h,
     long nb_points)
{
  float *ham = (h == NULL) ? hamming_ : h;

  for (int i = 0; i < ((h==NULL) ? nombre_de_points : nb_points); i++)
    res[i] = cnv[i] * ham[i];
}

/*
 * Preaccentuation du signal
 */

void preac_f(
     SAMPLEbuf& cnv,
     long cnv_debut,
     FLOATbuf& res,
     long nb_points)
{
  SAMPLE huge *pcnv;
  float *pres;

  pres=(float *)&res[0];
  pcnv=(SAMPLE huge *)&cnv[cnv_debut];

  for (int i=0; i<nb_points; i++)
    pres[i] = (float) (pcnv[i+1] - pcnv[i]);
}

void preac_f_(
     SAMPLE *cnv_,
     long cnv_debut,
     float *res,
     long nb_points)
{
  SAMPLE *cnv=(SAMPLE  *) (cnv_ + cnv_debut);
  for (int i = 0; i < nb_points; i++)
    res[i] = (float) (cnv[i+1] - cnv[i]);
}

void preac_d_(
     SAMPLE *cnv_,
     long cnv_debut,
     double *res,
     long nb_points)
{
  SAMPLE *cnv =(SAMPLE  *) (cnv_ + cnv_debut);
  for (int i = 0; i < nb_points; i++)
    res[i] = (double) (cnv[i+1] - cnv[i]);
}

/* donnees double, resultat double */
void ham_dd_(
     double *cnv,
     double *res,
     float *h,
     long nb_points)
{
  float *ham = (h == NULL) ? hamming_ : h;

  for (int i = 0; i < ((h==NULL) ? nombre_de_points : nb_points); i++)
    res[i] = cnv[i] * ham[i];
}

/* donnees short, resultat float */
void ham_sd_(
     short *cnv,
     long cnv_debut,
     double *res,
     float *h,
     long nb_points)
{
  int i;
  float *ham = ((h == NULL) ? hamming_ : h);
  for (i=0; i< ((h==NULL) ? nombre_de_points : nb_points); i++)
    res[i] = (double) (cnv[i] * ham[i]);
}
