/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include "common.h"
#pragma hdrstop

#include "fftlpc.h"

/* if RADIX2 is not defined (see plo.h) FFT's written by Mayer are used. They
   are subtentially faster than radix-2 FFT's.
   As there is a possible patent problem you are given the choice... */
//#define RADIX2
#ifdef RADIX2
/* Structure contenant les tables auxiliaires necessaires aux calculs de FFT. */
typedef struct {
  int *tabj1_, *tabj2_, *tabjn_;
  double *tabcos_, *tabsin_;
  bool init;
} TabAuxFFT;

/* Structure contenant les tables auxiliaires necessaires aux calculs des FFT reelles. */
typedef struct {
  double *tabCos, *tabSin;
  bool init;
} TabAuxFFTReelles;

static int bitReverseFft (int x, int nBits);
static void initMask (int n);
static int mask[32];                /* Ne peut faire les FFT "que" jusqu'a l'ordre 2^32 */


#define ENTREE_FFTC  \
                 float *data;                       \
		 data=(float *)&data_[0];           \
		 for (i = 1; i  <= N; i++)          \
                 { x[i] = data[(i - 1) << 1];	    \
		   y[i] = data[((i - 1) << 1) + 1]; \
		 }
#define ENTREE_FFTIC ENTREE_FFTC

#define SORTIE_FFTC \
                 for (i = 0; i < N; i++)            \
                 { j1=AUX.tabjn_[i + 1];		    \
                   data[i << 1] = x[j1];	    \
		   data[(i << 1) + 1] = y[j1];	    \
	         }
#define SORTIE_FFTIC \
                 for (i = 0; i < N; i++)            \
                 { j1=AUX.tabjn_[i + 1];		    \
                   data[i << 1] = x[j1] / N;	    \
		   data[(i << 1) + 1] = y[j1] / N;  \
	         }


/***************************************************************/
/*			128				       */
/***************************************************************/
#define N	128
#define LOG2N	7
#define AUX TFFTC128
#define TYPE double

/* donnees et fft calculees    */
#define ENTETE void fftc128 (FLOATbuf& data_)


#define LIBERE void free_fftc128 ()
#define MESFFT "fftc128"
#define MESINIT "init_fftc128"
#define MESFREE "free_fftc128"
#define ENTREE ENTREE_FFTC
#define SORTIE SORTIE_FFTC

#define FFTINIT initFftc128

#include "fftproto.h"

#define N	128
#define LOG2N	7
#define AUX TFFTIC128
#define TYPE double

/* donnees et fft calculees    */
#define ENTETE void fftic128 (FLOATbuf& data_)


#define LIBERE void free_fftic128 ()
#define MESFFT "fftic128"
#define MESINIT "init_fftic128"
#define MESFREE "free_fftic128"
#define ENTREE ENTREE_FFTC
#define SORTIE SORTIE_FFTIC

#define FFTINIT	initFfti128
#include "fftproto.h"


/***************************************************************/
/*			256				       */
/***************************************************************/
#define N	256
#define LOG2N	8
#define AUX TFFTC256
#define TYPE double

/* donnees et fft calculees    */
#define ENTETE void fftc256 (FLOATbuf& data_)
#define LIBERE void free_fftc256 ()
#define MESFFT "fftc256"
#define MESINIT "init_fftc256"
#define MESFREE "free_fftc256"
#define ENTREE ENTREE_FFTC
#define SORTIE SORTIE_FFTC


#define FFTINIT initFftc256
#include "fftproto.h"


#define N	256
#define LOG2N	8
#define AUX TFFTIC256
#define TYPE double

/* donnees et fft calculees    */
#define ENTETE void fftic256 (FLOATbuf& data_)
#define LIBERE void free_fftic256 ()
#define MESFFT "fftic256"
#define MESINIT "init_fftic256"
#define MESFREE "free_fftic256"
#define ENTREE ENTREE_FFTC
#define SORTIE SORTIE_FFTIC


#define FFTINIT initFfti256
#include "fftproto.h"

#define N               128
#define AUX 		TFFTR256
#define ENTETE 		fftr256
#define FFTCOMPLEXE 	fftc128
#define FFTCOMPINV  	fftic128
#define LIBERE	    	free_fftr256
#define REALFTINIT  	init_fftr256
#include "fftr.h"


/***************************************************************/
/*			512				       */
/***************************************************************/
#define N	512
#define LOG2N	9
#define AUX TFFTC512
#define TYPE double

/* donnees et fft calculees    */
#define ENTETE void fftc512 (FLOATbuf& data_)
#define LIBERE void free_fftc512 ()
#define MESFFT "fftc512"
#define MESINIT "init_fftc512"
#define MESFREE "free_fftc512"
#define ENTREE ENTREE_FFTC
#define SORTIE SORTIE_FFTC

#define FFTINIT initFftc512
#include "fftproto.h"

#define N	512
#define LOG2N	9
#define AUX TFFTIC512
#define TYPE double

/* donnees et fft calculees    */
#define ENTETE void fftic512 (FLOATbuf& data_)
#define LIBERE void free_fftic512 ()
#define MESFFT "fftic512"
#define MESINIT "init_fftic512"
#define MESFREE "free_fftic512"
#define ENTREE ENTREE_FFTC
#define SORTIE SORTIE_FFTIC

#define FFTINIT initFfti512
#include "fftproto.h"

#define N               256
#define AUX 		TFFTR512
#define ENTETE 		fftr512
#define FFTCOMPLEXE 	fftc256
#define FFTCOMPINV  	fftic256
#define LIBERE	    	free_fftr512
#define REALFTINIT  	init_fftr512
#include "fftr.h"
/***************************************************************/
/*			1024				       */
/***************************************************************/
#define N	1024
#define LOG2N	10
#define AUX TFFTC1024
#define TYPE double

/* donnees et fft calculees    */
#define ENTETE void fftc1024 (FLOATbuf& data_)
#define LIBERE void free_fftc1024 ()
#define MESFFT "fftc1024"
#define MESINIT "init_fftc1024"
#define MESFREE "free_fftc1024"
#define ENTREE ENTREE_FFTC
#define SORTIE SORTIE_FFTC

#define FFTINIT initFftc1024
#include "fftproto.h"

#define N	1024
#define LOG2N	10
#define AUX TFFTIC1024
#define TYPE double

/* donnees et fft calculees    */
#define ENTETE void fftic1024 (FLOATbuf& data_)
#define LIBERE void free_fftic1024 ()
#define MESFFT "fftic1024"
#define MESINIT "init_fftic1024"
#define MESFREE "free_fftic1024"
#define ENTREE ENTREE_FFTC
#define SORTIE SORTIE_FFTIC

#define FFTINIT initFfti1024
#include "fftproto.h"

#define N               512
#define AUX 		TFFTR1024
#define ENTETE 		fftr1024
#define FFTCOMPLEXE 	fftc512
#define FFTCOMPINV  	fftic512
#define LIBERE	    	free_fftr1024
#define REALFTINIT  	init_fftr1024
#include "fftr.h"

/***************************************************************/
/*			2048				       */
/***************************************************************/
#define N	2048	
#define LOG2N	11
#define AUX TFFTC2048
#define TYPE double

/* donnees et fft calculees    */
#define ENTETE void fftc2048 (FLOATbuf& data_)
#define LIBERE void free_fftc2048 ()
#define MESFFT "fftc2048"
#define MESINIT "init_fftc2048"
#define MESFREE "free_fftc2048"
#define ENTREE ENTREE_FFTC
#define SORTIE SORTIE_FFTC

#define FFTINIT initFftc2048
#include "fftproto.h"

#define N               1024
#define AUX 		TFFTR2048
#define ENTETE 		fftr2048
#define FFTCOMPLEXE 	fftc1024
#define FFTCOMPINV  	fftic1024
#define LIBERE	    	free_fftr2048
#define REALFTINIT  	init_fftr2048
#include "fftr.h"

/***************************************************************/
/*			4096				       */
/***************************************************************/
#define N	4096	
#define LOG2N	12
#define AUX TFFTC4096
#define TYPE double

/* donnees et fft calculees    */
#define ENTETE void fftc4096 (FLOATbuf& data_)
#define LIBERE void free_fftc4096 ()
#define MESFFT "fftc4096"
#define MESINIT "init_fftc4096"
#define MESFREE "free_fftc4096"
#define ENTREE ENTREE_FFTC
#define SORTIE SORTIE_FFTC

#define FFTINIT initFftc4096
#include "fftproto.h"

#define N               2048
#define AUX 		TFFTR4096
#define ENTETE 		fftr4096
#define FFTCOMPLEXE 	fftc2048
#define FFTCOMPINV  	fftic2048
#define LIBERE	    	free_fftr4096
#define REALFTINIT  	init_fftr4096
#include "fftr.h"

/*
 * 
 */
static void initMask (int n)
{
  int i, m;

  for (i = 0, m = 1; i < n; i++, m <<= 1) mask[i] = m;
}


/*
 * Bit reversal for FFT.
 * bitReverseFft(x) =  bitReverse(x - 1) + 1
 */

static int bitReverseFft (
  int x,	/* to be bit reversed */
  int nBits	/* maximum number of bits to be reversed */
  )

{
  int i;
  int y = 0;	/* result */

  if (x < 1 || x > (mask[nBits - 1] << 1)) {
    STRING ch;
    sprintf(ch,"bug in bitReverseFft() in fftproto.h\nx=%d out of range", x);
    fatal_error(ch);
    return(-1);
  }

  for (i = 0; i < nBits; i++) 
    y |= ((x - 1) & mask[i]) ? mask[nBits - 1 - i] : 0;

  return (y + 1);
}

#else
#include <math.h>
extern void realfft(int n, double *real);
extern void realifft(int n, double *real);
extern void fft(int n, double *real, double *imag);
extern void ifft(int n, double *real, double *imag);
/* FFT de Mayer */
/* le contenu est inutile mais evite au compilo de raler. */

/* le contenu est inutile mais evite au compilo de raler. */
#define INITREALFFT(nom)	\
void nom ()			\
{				\
  int x = 1;			\
  x = x + 2;			\
}

#define REALFFT(nom, N) 	\
void nom (FLOATbuf& data_, int sens)  \
{				\
  double x[N];			\
  int i;			\
  float *data; \
  data=(float *)&data_[0]; \
  for (i = 0; i  < N; i++)	\
    x[i] = data[i];		\
  if (sens == 1) realfft(N, x);	\
  else realifft(N, x);		\
  data[0] = x[0]; data[1] = 0.;	\
  for (i = 1; i < N / 2; i++) { \
    data[i << 1] = x[i]; 	\
    data[(i << 1) + 1] = x[N - i];\
  }				\
}

#define FFT(nom, N) \
void nom (FLOATbuf& data_) \
{ \
  float *data; \
  data=(float *)&data_[0]; \
  double real[N], imag[N];	\
  int i;			\
  for (i = 0; i  < N; i++) {	\
    real[i] = data[i << 1];	\
    imag[i] = data[(i << 1)+1];	\
  }				\
  fft(N, real, imag);		\
  for (i = 0; i < N; i++) {	\
    data[i << 1] = real[i];	\
    data[(i << 1)+1] = imag[i];	\
  }				\
}

#define INUTILEFFT(nom)		\
void nom()			\
{				\
}

#define IFFT(nom, N) 		\
void nom (FLOATbuf& data_)		\
{				\
  double real[N], imag[N];	\
  float *data=(float *)&data_[0]; \
  int i;			\
  for (i = 0; i  < N; i++) {	\
    real[i] = data[i << 1];	\
    imag[i] = data[(i << 1)+1];	\
  }				\
  ifft(N, real, imag);		\
  for (i = 0; i < N; i++) {	\
    data[i << 1] = real[i] / N;	\
    data[(i << 1)+1] = imag[i] / N;\
  }				\
}
FFT(fftc128, 128)
IFFT(fftic128, 128)
FFT(fftc256, 256)
IFFT(fftic256, 256)
FFT(fftc512, 512)
IFFT(fftic512, 512)
FFT(fftc1024, 1024)
IFFT(fftic1024, 1024)
FFT(fftc2048, 2048)
IFFT(fftic2048, 2048)
FFT(fftc4096, 4096)
IFFT(fftic4096, 4096)
REALFFT(fftr256, 256)
REALFFT(fftr512, 512)
REALFFT(fftr1024, 1024)
REALFFT(fftr2048, 2048)
REALFFT(fftr4096, 4096)
INUTILEFFT(init_fftc4096)
INUTILEFFT(init_fftc2048)
INUTILEFFT(init_fftc1024)
INUTILEFFT(init_fftc512)
INUTILEFFT(init_fftc256)
INUTILEFFT(init_fftc128)
INUTILEFFT(init_fftic2048)
INUTILEFFT(init_fftic1024)
INUTILEFFT(init_fftic512)
INUTILEFFT(init_fftic256)
INUTILEFFT(init_fftic128)
INUTILEFFT(init_fftr256)
INUTILEFFT(init_fftr512)
INUTILEFFT(init_fftr1024)
INUTILEFFT(init_fftr2048)
INUTILEFFT(init_fftr4096)
INUTILEFFT(free_fftc4096)
INUTILEFFT(free_fftc2048)
INUTILEFFT(free_fftc1024)
INUTILEFFT(free_fftc512)
INUTILEFFT(free_fftc256)
INUTILEFFT(free_fftc128)
INUTILEFFT(free_fftic2048)
INUTILEFFT(free_fftic1024)
INUTILEFFT(free_fftic512)
INUTILEFFT(free_fftic256)
INUTILEFFT(free_fftic128)
INUTILEFFT(free_fftr2048)
INUTILEFFT(free_fftr1024)
INUTILEFFT(free_fftr512)
INUTILEFFT(free_fftr256)
#endif





