/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier lireman.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <string.h>

#include "notetiq.h"
#include "menu.h"
#include "outils.h"
#include "format_f.h"

// les tiquettes sont ignores au-del de 32000
//   environ par fichier


/*
 * Cette fonction permet de lire les fichiers
 *   d'etiquettes phonetiques provenant d'un fichier d'Alignement.
 */

static void liremanAlign(
     notetiqs *et,
     infoFile& infofile)  // fichier  lire
{
  char nfic[MAX_LIGNE],s[MAX_LIGNE], ligne[MAX_LIGNE];
  int nbRates = 0;
  FILE *fic;
  int etiq;
  int debut_ms, fin_ms;
  char phon[5];
  bool ok = true;

  strcpy(nfic,infofile.nom_man);
  et->clear();

  fic = fopen(nfic,"rt");
  if (!fic)
  { affiche_erreur(ml(4701,"Fichier d'tiquettes inexistant"));
    return;
  }

  fgets(ligne, MAX_LIGNE, fic);
  if(strncmp(ligne, ".ali", 4))
  { affiche_erreur(ml(4701,"Fichier d'tiquettes inexistant"));
    return;
  }
  fgets(ligne, MAX_LIGNE, fic); //pour frec_echt ...

  fgets(ligne, MAX_LIGNE, fic);
  while (!feof(fic)& ok) {

  		sscanf(ligne,"%d %d %s\n",&debut_ms,&fin_ms, phon);
      if ( ( etiq=et->addS(enEcht(debut_ms)) ) >= 0 ){
      	strcpy(s, phon);

         if(strncmp(s, "FIN", 3)){
         	if ( phon_TextToi(s)<0
                && (strncmp(s, "#", 1)
                    && strncmp(s, "vcl", 3)
                    && strncmp(s, "cl", 2))
                )
                nbRates++;
			 		else et->e[etiq]->SetText(s);
            
         }
         else
         	ok=false;
     	}

      fgets(ligne, MAX_LIGNE, fic);

  }

  fclose(fic);
  if (nbRates)
  { char ch[80];
    sprintf(ch, ml(4703,"Certains symboles phontiques (%d) sont inconnus"), nbRates);
    affiche_erreur(ch);
  }
}


/*
 * Cette fonction permet de lire les fichiers
 *   d'etiquettes phonetiques du Crin.
 */

static void liremanCrin( 
     notetiqs *et,
     infoFile& infofile)  // fichier  lire
{
  char fret[MAX_LIGNE];			/* frequence d'echantillonnage  */
  char nfic[MAX_LIGNE],s[MAX_LIGNE];
  int i = 1, nbRates = 0;
  FILE *fic;
  int etiq;
  
  strcpy(nfic,infofile.nom_man); 
  et->clear();
  
  fic = fopen(nfic,"rt");
  if (!fic)
  { affiche_erreur(ml(4701,"Fichier d'tiquettes inexistant"));
    return;
  }
  
  fscanf(fic,"%s\n",s); 
  fscanf(fic,"%s\n",fret);
  while (!feof(fic)) {
    /* Cette ligne ne marche plus depuis le passage au compilateur 1.3 :
     *  fscanf(fic,"%*[{]%*[ ]%[^}]%*[},]%d%*[,]%d%*[,]%d%*[, (]%d%*[,]%d%*[)]\n",
     *	 s,&d,&f,&muet,&muet,&muet);
     * Si elle remarchait il faudrait modifier ce qui suit.
     */
    { long l;
      char ligne[MAX_LIGNE]; 
      char *p;

      fgets(ligne, MAX_LIGNE, fic);
      if (feof(fic)) break;
      p=strtok(ligne,"{}");
      if (p!=NULL)
      { if (strcmp(p,"  ")) sscanf(p,"%s",s); 
        else strcpy(s,"");
        p=strtok((char *) NULL,",");
      }
      if (p!=NULL) p=strtok((char *) NULL,",");
      if (p!=NULL) p=strtok((char *) NULL,",");
      if (p!=NULL) p=strtok((char *) NULL,"(");
      if (p!=NULL) p=strtok((char *) NULL,",");
      if (p!=NULL) { l=atol(p)/2;
               	     p=strtok((char *) NULL,")");
                   }
      if (p!=NULL) { if ((etiq=et->addS(l))>=0)
		     { if (et->addS(atol(p)/2)>=0)
		       { if (phon_TextToi(s)<0) nbRates++;
			 else et->e[etiq]->SetText(s);
                       }
                     }
		   }
    }
    if (feof(fic)) break;
    i++;
  }
  
  fclose(fic);
  if (nbRates)
  { char ch[80];
    sprintf(ch, ml(4703,"Certains symboles phontiques (%d) sont inconnus"), nbRates);
    affiche_erreur(ch);
  }
}



/*
 * Lecture de l'etiquetage ortographique d'un fichier SAM.
 */
static void liremanSam( 
     notetiqs *et,
     infoFile& infofile)  // fichier  lire
{
  FILE *f = fopen(infofile.nom_man, "rt");
  int i, nbRates = 0,
      elf, numLigne;
  char ch[80];
  int etiq;

  et->clear();

  if (!f)
  { sprintf(ch,ml(4704,"Fichier Sam inaccessible %s"), infofile.nom_man);
    affiche_erreur(ch);
    return;
  }

  for (i = 1, numLigne = 0, elf = 0;
       !feof(f) && !elf; 
       numLigne++)
  { char ligne[MAX_LIGNE], *debLigne, *p
        ,ligne0[MAX_LIGNE]; 
    int errorDetected = FAUX;
    fgets(ligne0, MAX_LIGNE, f);
    strcpy(ligne,ligne0);
    p = ligne;
    debLigne = readAField(&p, ':');
    elf = (debLigne && !strcmp(debLigne, "ELF"));
    if (!elf) {
      if (!strcmp(debLigne, "LBB")) {
	char *field;
	field = readAField(&p, ',');
	if (field)
	{ if ((etiq=et->addS(atol(field)))<0)
	    errorDetected = VRAI;
	}
	else errorDetected = VRAI;
	readAField(&p, ',');
	if (!errorDetected) {
	  field = readAField(&p, ',');
	  if (field)
	  { if (et->addS(atol(field))<0)
	      errorDetected = VRAI;
	  }  
	  else errorDetected = VRAI;
	}
	if (!errorDetected) {
	  readAField(&p, ',');
	  suppblanc(suppcrnl(p));
	  if (phon_TextToi(p)<0) nbRates++;
          else et->e[etiq]->SetText(p);
	}
	i++;
      }
      if (errorDetected)
      {	affiche_erreur_LigneFichier(ligne, infofile.nom_man);
	fclose(f);
	return;
      }
    }
  }

  fclose(f);
  if (nbRates)
  { sprintf(ch, ml(4703,"Certains symboles phontiques (%d) sont inconnus"), nbRates);
    affiche_erreur(ch);
  }
}

/*
 * Lecture de l'etiquetage ortographique d'un fichier NIST.
 */
static void liremanNist( 
     notetiqs *et,
     infoFile& infofile)  // fichier  lire
{
  FILE *f = fopen(infofile.nom_man, "rt");
  int i = 1, nbRates = 0;
  char ch[80];
  int etiq;

  et->clear();

  if (!f)
  { sprintf(ch,ml(4705,"Fichier Nist inaccessible %s"), infofile.nom_man);
    affiche_erreur(ch);
    return;
  }

  do {
    char ligne[MAX_LIGNE], *p;
    p = fgets(ligne, MAX_LIGNE, f);
    if (p) {
      char *field = readAField(&p, ' ');
      if (field &&(etiq = et->addS(atol(field))) >=0) {
        field = readAField(&p, ' ');
        if (field && et->addS(atol(field))>=0) {
          suppcrnl(p);
          if (phon_TextToi(p)<0) nbRates++;
          else et->e[etiq]->SetText(p);
	      i++;
		  }
      }
    }
  }
  while (!feof(f)); 

  /*
  if (i == NB_SEG - 1) 
    affiche_erreur(ml(4706,"Fichier trop grand, lecture arrte"));
  */

  fclose(f);
}

void lireman( 
     notetiqs *et,
     infoFile& infofile)  // fichier  lire
{
  switch (infofile.fmt_phon) {
    case FMTPHON_ALIGN :
      liremanAlign(et, infofile);
      break;
    case FMTPHON_SAM :
      liremanSam(et, infofile);
      break;
    case FMTPHON_NIST :
      liremanNist(et, infofile);
      break;
    case FMTPHON_CRIN :
    default :
      liremanCrin(et, infofile);
      break;
  }
}




