/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier lpc.cpp
//
#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>

#include "fftlpc.h"
#include "signal.h" // pour en_cours.frequence

#define SECT 128
#undef PI
#define PI 3.1415926535898
#define NMAX 512      /* maximum point for fft */

/*
 * lpc sub-program :
 */

static float t[SECT],
             kk[SECT];

void autolpc (
	float *signal, 	// signal array
	float *r, 	// autoc
	float *a, 	//     lpc coeff. and air
	float *sf,	//         air (areas ??)
			//             of vocal tract
	float *e,       // gain factor 
	int n,      	// Nb elements in the array
   int p)          //     and order of lpc
/****************************************************************/
/*     computes linear prediction coefficients by durbin's      */
/*     autocorrelation  methode in a[1..p]                      */
/*     definition:                                              */
/*         1) h(z)=g/a(z);    2) a(z)=1-sigma(ak*z**(-k))       */
/*                                      k[1..p]                 */
/*         3) g*g=r(0)-sigma(ak*r(k))                           */
/*                     k[1..p]                                  */
/*         4) sf : forme du conduit vocal                       */
/****************************************************************/
{ 
	register m, i, j;
	float    as, k;

	/*compute the autocorrelation vector*/
	for( i = 0; i<= p; i++) {
	  r[i] = 0;
	  for (m = 0 ; m<= n - 1 - i ; m++)
	    r[i] += (signal[m] * signal[i + m]);
	}
	/*solve the matrix equition by durbin's methode*/
	*e = (r[0] < EPSILON) ? 1 : r[0];
	for (i = 1; i<= p; i++) {
		k = r[i];
		for (j = 1 ;j<= i - 1;j++)
			k += (- r[i - j] * a[j]);
		k  /= *e;
		kk[i] = -k;
		a[i] = k;
		for (j = 1; j<= i - 1 ; j++)
			t[j] = a[j] - k * a[i - j];
		for (j = 1 ; j<=i - 1 ; j++)
			a[j] = t[j];
		*e = (//(double)
		      (1) - k * k) * *e;
	}
	as = //(double)
	     (0.5);
	for( i = p; i>=1;i--) {
		as *= (//(double)
		       (1) + kk[i]
		      ) / (//(double)
		           (1) - kk[i]);
		sf[i] = as;
	}
	a[0] = 1.0;
} /*autolpc*/

////////////////////////////////////////
/*
// procdure donnant pour la lpc l'ordre
// raisonnable en standard
int ordre_lpc_std()
{ return max(ORDRE_LPC_MIN
             ,min(ORDRE_LPC_MAX,
	 IROUND(2+(en_cours.frequence+10.0)/1000.0)
	 ));
}
*/
int ordre_lpc_msTOecht(float ordre_ms)
// renvoie toujours un ordre pair
{ int i;
  i=     max(ORDRE_LPC_MIN
             ,min(ORDRE_LPC_MAX,
	 IROUND(2+ordre_ms
		  *(en_cours.frequence+10.0)
		  /1000.0)
	 ));
  return i+i%2;
}

