/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier mccandle.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>

#include "fftlpc.h"
#include "spectre.h"
#include "outils.h"
#include "wformant.h"
#include "opt_for.h"
#include "menu.h"
#include "signal.h" //pour en_cours 1 fois
#include "notetiq.h"
#include "spectro.h"
#include "pitchm.h"
#include "mccandle.h"
#include "polypoint.h"
#include "guiklat.h"
#include "energies.h"
#include "fmaxspec.h"

bool yaUneFenetreMccandless = false;

#define NBFOR	4	/* nombre de formants suivis */
static float domFor[NBFOR][2] = {{180., 950.},
				 {600., 2650.},
				 {1800., 4000.},
				 {3000., 5000.}};

/////////////////////////////////////////////
struct F1F2F3
{ long echt;
  int f1,f2,f3;
};
class F1F2F3S  // tous les pics
{ public:
    int ni;
    F1F2F3 **p;

    F1F2F3S();
    ~F1F2F3S();
    void clear();
    int size() { return ni; }
    void resize(int nb);
    F1F2F3 & operator[](int i) { return *p[i]; }
};
F1F2F3S::F1F2F3S()
{ ni=0;
  p = (F1F2F3 **) NULL;
}
F1F2F3S::~F1F2F3S()
{ clear();
  if (p) free(p);
  p = (F1F2F3 **) NULL;
}
void F1F2F3S::clear()
{ int il;
  for (il=0;il<ni;il++) delete p[il];
  ni=0;
  if (p) free(p);
  p = (F1F2F3 **) NULL;
}
void F1F2F3S::resize(int nb)
{ int i;
  if (nb==0) { clear(); return; }
  if (nb>this->ni)
  { p=(F1F2F3 **) realloc(p,((long)nb)*sizeof(F1F2F3 *));
    for (i=this->ni;i<nb;i++)
      p[i]=new F1F2F3();
  } 
  if (nb<this->ni)
  { for (i=this->ni-1;i>=nb;i--)
      delete p[i];
    p=(F1F2F3 **)realloc(p,((long)nb)*sizeof(F1F2F3 *));
  }
  this->ni=nb;
}
F1F2F3S f123s;

/////////////////////////////////////////////
/* liste des candidats formants utilisant un pic */
struct candPic {
	int c;
	struct candPic *reste;
};

struct pic {
	float fr, bw;		/* frequence et largeur de bande */
	struct candPic *cliS;	/* liste des candidats formants
	                             utilisant ce pic */
};

/* liste de points dans le plan F1-F2 */
struct point {
  float x, y;
  struct point *reste;
};

static float f2_, f1_, f2__, f1__;
static int premFois;
static struct point *cheminF1F2 = (struct point *) NULL;
static float EST[NBFOR];	/* l'estimation des formants */



/*
 * Egalite de deux candidat formants.
 */
static int egalS(
     struct candPic *x,
     struct candPic *y)
{
  return(x->c == y->c);
}


//Ajoute_queue(ajtqS,struct candPic)
static struct candPic *ajtqS(
	struct candPic *l,
	struct candPic *y)
{
	struct candPic *p = l;
	if (!p) return(y);
	if (!y) return(l);
	while (p->reste) p = p->reste;
	p->reste = y;
	return(l);
}
//Longueur_liste(lgS,struct candPic)
static int lgS(
	struct candPic *x)
{							
	struct candPic *p = x;
	int l = 0;					
	while (p) {					
	      l++;					
	      p = p->reste;				
	}						
	return(l);					
}	
//Detruit_liste(detruitLisCan,struct candPic,)
static struct candPic *detruitLisCan(		  
	struct candPic *x)				  
{						  
	struct candPic *p = x;			  
	if (!p) return((struct candPic *) NULL);  
	p->reste = detruitLisCan(p->reste);	  
	;					
	free((char *) p);			
	return((struct candPic *) NULL); 	
}
//Supprime_liste(supprimLisCan, struct candPic, egalS, free)
static struct candPic *supprimLisCan(			       
	struct candPic *l,
	struct candPic *x)						       
{									
	if (!x) return(l);						
	else if (!l) {							
		free((char *) x);					
		return(l);						
	}								
	else if (egalS(l, x)) {					
		struct candPic *p = l->reste;			
		l->reste = (struct candPic *) NULL;		
		free(l); free((char *) x);			
		return(p);					
	}							
	else {							
		struct candPic *p1, *p2;			      
		for (p1 = l, p2 = l->reste; p2 && !egalS(x, p2);) {   
		    p1 = p2;					      
		    p2 = p2->reste;				      
		}						      
		free((char *) x);				      
		if (p2) {					      
			   p1->reste = p2->reste;		      
			   free(p2);				      
		}						      
		return(l);					      
	}							   
}

//Ajoute_queue(ajtqPoint,struct point)
static struct point *ajtqPoint(
	struct point *l,
	struct point *y)
{
	struct point *p = l;
	if (!p) return(y);
	if (!y) return(l);
	while (p->reste) p = p->reste;
	p->reste = y;
	return(l);
}
//Detruit_liste(detruitLisPoint,struct point,)
static struct point *detruitLisPoint(
	struct point *x)
{
	struct point *p = x, *ptemp;
	/* version mangeant de la pile,
	//   et donc abandonne
	if (!p) return((struct point *) NULL);
	p->reste = detruitLisPoint(p->reste);
	;
	free((char *) p);
	*/
	while (p)
	{ ptemp=p->reste;
	  free((char *)p);
          p=ptemp;
        }
	return((struct point *) NULL);
}

/*
 * Creation d'un nouveau point.
 */
static struct point *nvPt(
	float x, float y)
{
  struct point *p = ALLOUER(struct point);
  p->x = x;
  p->y = y;
  p->reste = (struct point *) NULL;
  return(p);
}

/*
 * Creation d'un nouvel element dans une liste de formant candidats.
 */
static struct candPic *nvS(
     int x) 	/* numero du candidat formant */
{
  struct candPic *p = (struct candPic *) malloc (
			  (unsigned int) (sizeof(struct candPic)));
  p->c = x;
  p->reste = (struct candPic *) NULL;
  return(p);
}

/*
 * Les etapes 1 a 4 de l'algorithme de l'algorithme de Mc Candless.
 */
static int step1_4(
     float *ESTim, 
     float *fS, 
     float *fpos,
     float *famp,
     float largBand)
{
  int j, iFor, iPic;
  struct pic lesPics[NBFOR];
  int S[NBFOR];
  int nbForm
     ,nbPic;
  
  nbForm = (int)fpos[0];

  for (j = 0; j < NBFOR; j++) S[j] = -1;
  
  
  /* step 1: selection des racines dans le bon intervalle de frequences */
  for (j = 1, iPic = 0; iPic < NBFOR && j <= nbForm; j++)
  {
    if (dans(fpos[j], 150., 3800.) && famp[j] < largBand)
    {
      lesPics[iPic].fr = fpos[j];
      lesPics[iPic].bw = famp[j];
      lesPics[iPic].cliS = (struct candPic *) NULL;
      iPic++;
    }
  }
  nbPic = iPic;
  
  /* step 2: choix d'un pic pour chaque formant */
  /* Le probleme c'est qu'il faut apres cette etape avoir
     un pic par formant,
     ce qui n'est pas forcement possible. Je verifie grossierement
     la frequence mais je suis tout de meme oblige d'affecter
     un pic au formant
     meme si cela ne devrait pas etre le cas. Bref, l'algo
     de McCandless n'est
     pas terrible! */
  for (iFor = 0; iFor < NBFOR; iFor++) {
    int i, lePic = -1;
    float dmin = INFINI_F;
    for (i = 1; i < nbPic; i++) 
      if  (   dans(lesPics[i].fr, domFor[iFor][0], domFor[iFor][1]) 
	   && fabs(lesPics[i].fr - ESTim[iFor]) < dmin) {
	lePic = i;
	dmin = fabs(lesPics[i].fr - ESTim[iFor]);
      }
    if (lePic != -1) {
      lesPics[lePic].cliS = ajtqS(lesPics[lePic].cliS, nvS(iFor));
      S[iFor] = lePic;
    }
    else S[iFor] = -1;
  }
  
  /* step 3: suppression des candidats multiples pour un pic */
  for (iPic = 0; iPic < nbPic; iPic++) if (lgS(lesPics[iPic].cliS))
  {
    int meilleurS = -1;
    float ESTmeilleur = INFINI_F / 2;
    struct candPic *p = lesPics[iPic].cliS;
    while (p) {
      if ( fabs(ESTim[p->c] - lesPics[iPic].fr)
	   < fabs(ESTmeilleur - lesPics[iPic].fr))
      {
	meilleurS = p->c;
	ESTmeilleur = ESTim[p->c];
      }
      p = p->reste;
    }
    if (ESTmeilleur < INFINI_F / 4) {
      struct candPic *q = lesPics[iPic].cliS;
      while (q) {
	S[q->c] = -1;
	q = q->reste;
      }
      lesPics[iPic].cliS = ajtqS(detruitLisCan(lesPics[iPic].cliS), 
			       nvS(meilleurS));
      S[meilleurS] = iPic;
    }
  }

  /* step 4: cas des pics non assignes */
  /* Attention ce pas est corrige d'apres:
   * "Formant extraction algorithm in error",
   * Michael Wagner
   * IEEE ASSP Vol. 30 No 3, june 1982, page 520
   * En plus on fait un leger controle de la frequence des formants.
   */
  for (iPic = 0; iPic < nbPic; iPic++) if (!lesPics[iPic].cliS)
  {
    if (S[iPic] < 0 && dans(lesPics[iPic].fr, domFor[iPic][0]
			    , domFor[iPic][1]))
    {
      S[iPic] = iPic;
      lesPics[iPic].cliS = nvS(iPic);
    }
    else if (S[iPic] >= 0 && lesPics[S[iPic]].bw > 2. * lesPics[iPic].bw) {
      if (S[iPic + 1] < 0) {
	if (S[iPic] < 0 || lesPics[iPic].fr < lesPics[S[iPic]].fr)
	{
 	  if ((S[iPic] < 0 || dans(lesPics[S[iPic]].fr, domFor[iPic+1][0]
	                           , domFor[iPic+1][1]))
	      && dans(lesPics[iPic].fr, domFor[iPic][0]
		      , domFor[iPic][1]))
	  {
	    S[iPic + 1] = S[iPic];
	    lesPics[S[iPic+1]].cliS = ajtqS(lesPics[S[iPic+1]].cliS
	                                    , nvS(iPic+1));
	    lesPics[S[iPic]].cliS = supprimLisCan(lesPics[S[iPic]].cliS
	                                          , nvS(iPic));
	  }
	}
	else
	{
	  if (dans(lesPics[iPic].fr, domFor[iPic+1][0]
	           , domFor[iPic+1][1])) {
	    S[iPic + 1] = iPic;
	    lesPics[iPic].cliS = nvS(iPic+1);
	  }
	}
      }
      else if (iPic - 1 >= 0 && S[iPic - 1] < 0) {	
	if (S[iPic] < 0 || lesPics[iPic].fr > lesPics[S[iPic]].fr)
	{
 	  if ((S[iPic] < 0 || dans(lesPics[S[iPic]].fr, domFor[iPic-1][0]
	                           , domFor[iPic-1][1]))
	      && dans(lesPics[iPic].fr, domFor[iPic][0]
	              , domFor[iPic][1])) {
	    S[iPic - 1] = S[iPic];
	    lesPics[S[iPic-1]].cliS = ajtqS(lesPics[S[iPic-1]].cliS
	                                    , nvS(iPic-1));
	    lesPics[S[iPic]].cliS = supprimLisCan(lesPics[S[iPic]].cliS
	                                          , nvS(iPic));
	  }
	}
	else {
	  if (dans(lesPics[iPic].fr, domFor[iPic-1][0]
		   , domFor[iPic-1][1]))
	  {
	    S[iPic - 1] = iPic;
	    lesPics[iPic].cliS = nvS(iPic-1);
	  }
	}
      }
    }
  }

  for (j = 0; j < NBFOR; j++)
    fS[j] = (S[j] >= 0) ? lesPics[S[j]].fr : ESTim[j];
  return(S[0] > -1 && S[1] > -1 && S[2] > -1);
}

/*
 * Affichage des formants.
 */
static struct point *afficheForF1F2(
     float f1, float f2,
     struct point *l)	/* liste des points du chemin deja stocke */
{
  struct point *lisPoint;
  if (!premFois) {
    premFois++;
    f2_ = f2;
    f1_ = f1;
    lisPoint = ajtqPoint(l, nvPt(f2, f1));
  }
  else if (premFois == 1) {
    premFois++;
    lisPoint = ajtqPoint(l, nvPt((f2_ * 3. + f2 * 4.) / 7.,
				 (f1_ * 3. + f1 * 4.) / 7.));
    f2__ = f2_;
    f1__ = f1_;
    f2_ = (f2_ * 3. + f2 * 4.) / 7.;
    f1_ = (f1_ * 3. + f1 * 4.) / 7.;
  }
  else {
    float f2temp, f1temp;
    lisPoint = ajtqPoint(l, nvPt((f2__ + f2_ * 2. + f2 * 4.) / 7.,
				 (f1__ + 2. * f1_ + 4. * f1) / 7.));
    f2temp = f2__;
    f1temp = f1__; 
    f2__ = f2_;
    f1__ = f1_;
    f2_ = (f2temp + f2_ * 2. + f2 * 4.) / 7.;
    f1_ = (f1temp + 2. * f1_ + 4. * f1) / 7.;
  }
  return(lisPoint);
}

/*
 * Affichage des formants sur le spectrogramme.
 */

static void afficheFor(
     float f1, float f2, float f3,
     long echt, float fret,
     d_planche *fenetre)
{ float df; // delta frquence par canal d'affichage
  float larg; // largeur du trac (environ 3 pixels)

  larg=fabs(fenetre->DYtoLY(0)-fenetre->DYtoLY(3));
  // bug corrig : fret rajout
  df = fMaxSpectro / NB_CANAUX;
  MgiHue(ROUGE, fenetre);
  fenetre->Lellipse(echt, f1/df+1., 128.,larg/2);
  MgiHue(MAGENTA, fenetre);
  fenetre->Lellipse(echt, f2/df+1., 128.,larg/2);
  MgiHue(BLEU, fenetre);
  fenetre->Lellipse(echt, f3/df+1., 128.,larg/2);
}

/*
 * Affiche un chemin dans le plan F1-F2.
 */
static struct point *afficheChemin(
     struct point *l,
     d_planche *fenetre,
     COLORREF couleur)
{
  struct point *p;

  fenetre->SetPenWidth(3);
  MgiHue(couleur, fenetre);
  for(p = l; p && p->reste; p = p->reste)
    fenetre->Lline2(p->x,p->y
                           ,p->reste->x,p->reste->y);
  fenetre->ResetPenWidth();
  InvalidateRect(fenetre->HWindow,NULL,TRUE);
  UpdateWindow(fenetre->HWindow);
  return(detruitLisPoint(l));
}

/*
 * Iteration du suivi sur tout le domaine a etudier.
 */
static void suivi(
     long deb,
     long fin,
     SAMPLEbuf& cnv,
     float fret,
     d_planche *fenetre)
{
  long iEcht;
  int sens;
  int fen_signal = FENFFT256;
  int depl = OFFSETFFT256;
  int or_lpc = ordre_lpc_msTOecht(opt_for.or_lpc);
  float *r = new float[or_lpc + 1],
	     *a = new float[or_lpc + 1],
	     *sf= new float[or_lpc + 1],
        *xr = new float[fen_signal],
        gain,
        *coef = new float[or_lpc + 1], // coefficients du denominateur de la fct de trandfert
        *fpos = new float[or_lpc + 1], // nombre, puis frequences des formants
        *fbw = new float[or_lpc + 1];  // nombre, puis largeur de bande des formants

  int ok, iEst;
  float fS[NBFOR];
  float xF; // chantillon pour affichage
  int nb, ni;

  EST[0] = 320.;
  EST[1] = 1440.;
  EST[2] = 2760.;
  EST[3] = 3200.;

  initham_((float *) NULL, fen_signal);
  sens = (fin < deb) ? -1 : 1;
  //or_lpc = min(ORDRE_MAX_LPC - 1,opt_for.or_lpc);

  nb=(int)(max(0,min(1000,
        (labs(fin-deb)-fen_signal)/depl ))); // revoir le 1000
  f123s.resize(nb);

  for (ni = 0; ni < nb; ni++) {
    iEcht = deb + ni * sens * ((long)depl) +( (sens<0) ? -fen_signal : 0 );

    preac_f_(&cnv[0], _i(iEcht), xr, fen_signal);
    ham_ff_(xr, xr, NULL, fen_signal);

    autolpc(xr, r, a, sf, &gain, fen_signal, or_lpc);
    coef[0] = 1.;
    for (int j = 1;  j <= or_lpc; j++)  coef[j] = -a[j];

    fband(or_lpc, coef, (float) fret, LARG_BAND_MAX, 0.0,(float) (fret/2.0), fpos, fbw);

    ok = step1_4(EST, fS, fpos, fbw, 400.) || step1_4(EST, fS, fpos, fbw, 600.);
    if (!ok) Wmsg->wr("mccandless *");
    for (iEst = 0; iEst < NBFOR; iEst++)
      EST[iEst] = fS[iEst];
    xF = (float) iEcht + fen_signal/2;	

    cheminF1F2 = afficheForF1F2(EST[0], EST[1], cheminF1F2);
    afficheFor(EST[0], EST[1], EST[2], xF, fret, fenetre);
    f123s[ni].echt=xF;
    f123s[ni].f1=EST[0];
    f123s[ni].f2=EST[1];
    f123s[ni].f3=EST[2];
  }
  f123s.resize(ni);
  delete [] r;
  delete [] a;
  delete [] sf;
  delete [] xr;
  delete [] coef;
  delete [] fpos;
  delete [] fbw;

}
static void logs_mccandless()
{
  STRING ch;
  int i;
  long echt;
  int etiq,note;

  logs.open1();
  logs.wr1(ml(5202,">> formants F1/F2 mccandless"));
  sprintf(ch,ml(5203,"fichier : %s"),en_cours.nom);
  logs.wr1(ch);
  logs.wr1(ml(5204,"Temps\tTemps\tF0\tener\tF1\tF2\tF3\tPhoneme\tWord"));
  logs.wr1(ml(5205,"[ms]\t[echt]\t[Hz]\t[dB]\t[Hz]\t[Hz]\t[Hz]"));
  initCalculEnergie();
  for (i=0;i<f123s.size();i++) {
    echt=f123s[i].echt;
    etiq=etiqs.search(echt);
	 note=notes.search(echt);
    sprintf(ch,"%ld\t%ld\t%d\t%4.0f\t%d\t%d\t%d\t%s\t%s",
		      enMilli(echt),echt,
      		max(pm.getf0(echt), pm.getReadf0(echt)),
            enerBandeFminFmaxCentre(echt),
	   		f123s[i].f1,f123s[i].f2,f123s[i].f3,
	   		etiq>=0 ? etiqs.Text(etiq) : "",
	   		note>=0 ? notes.Text(note) : "" );
    logs.wr1(ch);
  }
  logs.close1();
  freeSpecFft();
}

/*
 * Cette fonction affiche les deux formants F1 F2 dans le
 * plan F1 F2 pour un
 * morceau de spectro (algorithme de McCandless).
 */
void mccandless(
     long deb, long fin,
     float fret,
     d_planche *fenf1f2,
     d_planche *fenspectro,
     SAMPLEbuf& cnv,
     COLORREF couleur)
{
  premFois = 0;
  suivi(deb, fin, cnv, fret, fenspectro);
  cheminF1F2 = afficheChemin(cheminF1F2, fenf1f2, couleur);
  logs_mccandless();
}



Wmccandless::Wmccandless()
    : TFrameWindow(PTWtop,ml(5207,"formants F1 F2 F3"))
{ Attr.Style=WS_VISIBLE|WS_POPUPWINDOW|WS_CAPTION
		       |WS_THICKFRAME;
  Attr.X=60;
  Attr.Y=0;
  Attr.W=250;
  Attr.H=200;
  this->SetIcon(GetApplication(), IDI_ICON1);
  //
  wf1f2=new d_planche_f1f2(this);
  menu=(char *)malloc(1);
}
Wmccandless::~Wmccandless()
{
  DestroyMenu(::GetMenu(HWindow));
  free(menu);

  if (!yaUneFenetrePiquefor) active(PPMENUAllStandard);
  ::DrawMenuBar(PTWtop->HWindow);
  yaUneFenetreMccandless = false;
}
void Wmccandless::SetupWindow()
{ //HMENU hmenu;
  TFrameWindow::SetupWindow();
  //BuildMenu();
  //hmenu = LoadMenuIndirect(menu);
  //::SetMenu(HWindow, hmenu);
  this->AssignMenu("F1F2F3");
}

void Wmccandless::EvSize(uint, TSize &)
{ RECT rect;

  Attr.H=max(100,Attr.H);
  Attr.W=max(100,Attr.W);
  ::GetClientRect(HWindow,&rect);
  //
  ::MoveWindow(wf1f2->HWindow,0,0
		 ,rect.right,rect.bottom,FALSE);
  affv(wf1f2);
  ::InvalidateRect(HWindow,NULL,TRUE);
}
void Wmccandless::CMhelpMouse()
{ ::WinHelp(PTWtop->HWindow,helpfile,HELP_KEY
	  ,chtoul("mousehelp_f1f2f3"));
}

void Wtop::CMaffFormants()
{
  desactive(PPMENUAllStandard);
  ::DrawMenuBar(PTWtop->HWindow);
  yaUneFenetreMccandless = true;

  GetApplication()->MakeWindow(new Wmccandless);
  wf1f2->paint();
  affVoyelles();
}
