/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier menu.cpp
//

#include "common.h"
#pragma hdrstop

#include "snoori.h"
#include "format_f.h" // pour formatdir[]
#include "menu.h"

MenuSquel *squelMenuPhoneme, *squelMenuClasses;

void detruitMenusLangue()
{
  delete squelMenuPhoneme;
  delete squelMenuClasses;
}

//
// Function called when button needs to be drawn
//
void
Wdonnees_etiquetage::ODADrawEntire(DRAWITEMSTRUCT far& drawInfo)
{
  HGDIOBJ oldHFont;

  TDC drawDC(drawInfo.hDC);
  TRect itemRect(drawInfo.rcItem.left, drawInfo.rcItem.top,
                 drawInfo.rcItem.right, drawInfo.rcItem.bottom);
  TBrush fillBrush(TColor::LtGray);
  drawDC.FillRect(itemRect, fillBrush);

  oldHFont = SelectObject(drawDC, queryFont(strlen(PhoneticFont) ? PhoneticFont : "arial", 7, 14, 0));
  drawDC.DrawText(((char *)drawInfo.itemData), -1, itemRect, DT_LEFT | DT_VCENTER | DT_SINGLELINE);
  SelectObject(drawDC, oldHFont);
  if (drawInfo.itemState & ODS_FOCUS)
    drawDC.DrawFocusRect(itemRect);
  if (drawInfo.itemState & ODS_SELECTED)
    drawDC.InvertRect(itemRect);
}

//
// Function called when button gains or loses focus
//
void
Wdonnees_etiquetage::ODAFocus(DRAWITEMSTRUCT far& drawInfo)
{
}

//
// Function called when button's selection status changes
//
void
Wdonnees_etiquetage::ODASelect(DRAWITEMSTRUCT far& drawInfo)
{
  TDC drawDC(drawInfo.hDC);
  TRect itemRect(drawInfo.rcItem.left, drawInfo.rcItem.top,
                 drawInfo.rcItem.right, drawInfo.rcItem.bottom);
  drawDC.InvertRect(itemRect);
}


MenuSquel::MenuSquel()
{
  content = (struct menuItem *) NULL;
  nItem = 0;
}


MenuSquel::~MenuSquel()
{
  if (!nItem) return;
  for(int i = 0; i < nItem; i++) {
    if (content[i].flags && MF_POPUP) delete content[i].subm;
    free((char *) content[i].text);
  }
  free((char *) content);
}

/*int MenuSquel::nbItem()
{
	return nItem;
}
struct menuItem *MenuSquel::contents()
{
	return content;
}  */

void MenuSquel::addMenuItem(uint flags, uint item, char *text, class MenuSquel *subm)
{
  struct menuItem *nvItem;
  char *p, *pp;
  content = (struct menuItem *) realloc(content, sizeof(menuItem) * (nItem + 1));
  nvItem = content + nItem;
  nvItem->flags = flags;
  nvItem->item = item;
  p = nvItem->text = strdup(text);
#ifdef UIY
  // braces are replaced by brackets for clarity sake in menues.
  while (p && (pp = strpbrk(p, "{}"))) {
    if (*pp == '{') *pp = '(';
    else *pp = ')';
    p = pp;
  }
#endif
  if (flags && MF_POPUP) nvItem->subm = subm;
  else nvItem->subm = (class MenuSquel *) NULL;
  nItem++;
}

#ifdef UIY
static TPopupMenu **popupMenus = (class TPopupMenu **) NULL;
static int nbPopupMenu = 0;

void destroyPopupMenus()
{
  int i;
  for (i = 0; i < nbPopupMenu; i++)
    delete(popupMenus[i]);
  free((char *) popupMenus);
}
#endif

TPopupMenu *construitMenu(MenuSquel *squel)
{
	int i;
   TPopupMenu *nm = new TPopupMenu();

#ifdef UIY
   // Memorize the creation of this PopuMenu because the destructor
   // does not work recursively.
   popupMenus = (TPopupMenu **) realloc(popupMenus, sizeof(TPopupMenu*) * (nbPopupMenu + 1));
   popupMenus[nbPopupMenu++] = nm;
#endif
   if (!squel->nItem) return (TMenu *) NULL;
   for (i = 0; i < squel->nItem; i++) {
      if (!(squel->content[i].flags & MF_POPUP)) {
      	nm->AppendMenu(squel->content[i].flags, squel->content[i].item,
                   	   squel->content[i].text);
      }
      else {
        TPopupMenu *nvm;
        nvm = construitMenu(squel->content[i].subm);
        nm->AppendMenu(squel->content[i].flags, (uint) nvm->GetHandle(),
                   	   squel->content[i].text);
      }
   }
   return nm;
}

/*
 * Construction de la table des phonemes.
 * Cette table est utilisee pour la lecture des phonemes
 * quand l'etiquetage se fait par le clavier.
 * C'est deja probablement deja fait mais pas commente.
 */
int nPhonEntry = 0;
struct phonEntry *phonEntries = (struct phonEntry *) NULL;

void destroyPhonEntries()
{
	for(int i = 0; i < nPhonEntry; i++) free((char *) phonEntries[i].text);
   free((char *) phonEntries);
}

static struct phonEntry *AppendPhon(struct phonEntry **t, uint item, char *text)
{
  struct phonEntry *nvEntry;
  *t = (struct phonEntry*) realloc((char *) *t, sizeof(struct phonEntry) * (nPhonEntry + 1));
  nvEntry = *t + nPhonEntry;
  nvEntry->item = item;
  nvEntry->text = strdup(text);
  nPhonEntry++;
}

static void construitTablePhon_(MenuSquel *squel)
{
	int i;
   for (i = 0; i < squel->nItem; i++) {
      if (!(squel->content[i].flags & MF_POPUP)) {
      	if (squel->content[i].item >= ID_PHON0)
         	AppendPhon(&phonEntries, squel->content[i].item - ID_PHON0,
            			  TextOfPhon(squel->content[i].item - ID_PHON0));
      }
      else construitTablePhon_(squel->content[i].subm);
   }
}

void construitTablePhon(MenuSquel *squel)
{
  nPhonEntry = 0;
  phonEntries = (struct phonEntry *) NULL;
  construitTablePhon_(squel);
}


int GetPhonemeByMenu(TPoint& point)
// en retour
//     ==0   phonme "rien"
//     >0    numro de phonme
//     -128  annuler (pas de phonme)
{
  wetiquetage->ClientToScreen(point);
  wetiquetage->num_phon = -128;
  wetiquetage->PopupMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON,
					 point,0, wetiquetage->HWindow);
  wetiquetage->GetApplication()->PumpWaitingMessages();
  return wetiquetage->num_phon;
}

int GetPhonemeOrClassByMenu()
// en retour
//     ==0   phonme "rien"
//     >0    numro de phonme
//     <0    moins (ngatif) numro de classe
//     -128  annuler (pas de phonme)
{
  TPoint point(100,0);
  wetiquetage->num_phon = -128;
  wetiquetage->PopupMenuClasses->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON,
					 point,0, wetiquetage->HWindow);
  wetiquetage->GetApplication()->PumpWaitingMessages();
  return wetiquetage->num_phon;

}

class langue
{ public:
    //bool yaUneLangue; // TRUE si oui
    ////////////////////////////////////////////////////
    // stockage des textes
    //
    // les textes sont un nom de phonme (ex: u)
    //       ou un nom de classe (ex: /V/)
    //       ou le texte destin  apparatre dans les menus
    //                     (ex: sOUpe
    //                          voyelle)
    //
    // le texte "" a pour numro phon 0
    //
    char *ttext; // stockage des chanes textes
    int ntext; // longueur de ttext

    //////////////////////////////////////////////////
    // stockage des listes de phonmes
    //
    // pour des listes de phonmes pures
    //   ou des listes de phonmes etou de classes
    //   ou idem etou des souslistes pour les submenus
    //
    // ==> mis dans ttext
	// >0   phoneme
        // <0   classe
        // ==0  fin de liste

    /////////////////////////////////////////////
    // menus
    char *menuphon; // indice du menu normal dans tmenu
    int nmenuphon;
    char *menuclass; // indice du menu avec classes dans tmenu 
    int nmenuclass;

    ////////////////////////////////////////
    // stockage des phonmes
    //
    struct phon
    { int iname;  // phonme (indice dans ttext)
      int itext;  // texte pour le menu (idem)
    };
    phon *tphon; // table des phonmes 
    int nphon; // nombre de phonmes dans tphon
    //////////////////////////////
    // liste des voyelles pour la fentre formant
    struct F1F2vowel { char phon; int f1; int f2; };
    F1F2vowel *tF1F2vowel;
    int nF1F2vowel;

    ////////////////////////////////////////
    // stockage des classes
    //
    struct classphon
	 { int iname; // classe (indice dans ttext)
		int itext; // texte pour le menu (idem)
		int ilist; // liste phonmes et/ou de classes
						//      (indice dans ttext)
	 };
	 classphon *tclassphon; // table des classes de phonmes
	 int nclassphon; // nombre de classes

	 langue();
	 ~langue();
	 bool init();
	 void clear();
	 int GetString(char *pc,int& ipc,char *string);
	 int GetString1(char *pc,int& ipc,char *string);
	 int GetStringF(FILE *fichier,int& nligne
							,char *ligne,int& iligne
							,char *string);
	 int addtext(char *text);
	 char phon_TextToi(char *text);
	 char class_TextToi(char *text);
    bool comp_phon(char *phon1, char *phon2);
	 MenuSquel *BuildSubMenu(MenuSquel *squelDeb, char *pc,int& ip, int npcmax,
    							    bool AllowClasses, char *nomSubMenu);
	 MenuSquel *BuildMenu(FILE *fichier,int& nligne,bool AllowClasses);
	 bool NewLanguage(char *filename);
};
langue langue;

langue::langue()
{ ttext=(char *)malloc(ntext=1); 
  tphon=(phon *)malloc((nphon=1)*sizeof(phon));
  tclassphon=(classphon *)malloc((nclassphon=1)*sizeof(classphon));
  tF1F2vowel=(F1F2vowel *)malloc(nF1F2vowel=0);
  menuphon=(char *)malloc(1);
  menuclass=(char *)malloc(1); 

  ttext[0]=0;
  tphon[0].iname=0;
  tphon[0].itext=0;
}
langue::~langue()
{ free(ttext);
  free(tphon);
  free(tclassphon);
  free(tF1F2vowel);
  free(menuphon);
  free(menuclass);
}
bool langue::init()
{ char ch[255];
  strcpy(ch,formatdir);
  strcat(ch,LanguageFile);
  return NewLanguage(ch);
}

bool init_langue()
{
	return langue.init();
}

void langue::clear()
{ ttext=(char *)realloc(ttext,ntext=1);
  tphon=(phon *)realloc(tphon,(nphon=1)*sizeof(phon));
  tclassphon=(classphon *)realloc(tclassphon,(nclassphon=1)
                                    *sizeof(classphon));
  menuphon=(char *)realloc(menuphon,1); 
  menuclass=(char *)realloc(menuclass,1);
}
static bool GetLigne(FILE *fichier,int& nligne
						  ,char *ligne,int& iligne)
// en retour TRUE ok
//           FALSE erreur ou fin de fichier
//                     donc pas de ligne
{ int imax=254;
  char ch[255];

  iligne=0;
  for (;;)
  { if (feof(fichier))
	 { // erreur fin de fichier
		Wmsg->show();
		Wmsg->wr(ml(5310,"unexpected end of file"));
		return FALSE;
	 }
	 fgets(ligne,imax+1,fichier);
	 nligne++;
	 if (ligne[0]!=';') break;
  }
  if (strlen(ligne)>=imax-1)
  { // erreur ligne trop longue
	 Wmsg->show();
	 sprintf(ch,ml(5311,"line too long (more than %d characters)"),imax);
	 Wmsg->wr(ch);
	 strcpy(ch,"... ");
	 strncat(ch,ligne,30);
	 strcat(ch,"...");
	 Wmsg->wr(ch);
	 return FALSE;
  }
  return TRUE;
}

int langue::GetString(char *pc,int& ipc
							,char *string)
// retour >0 il y a un string
//        0 erreur
//        -1 il n'y a plus de string disponible
//              et on a rencontr un }
//        -2 il n'y a plus de string disponible
//              et on est arriv  la fin de la ligne
// seul le cas 0 produit des messages, les messages des
//   cas -1 et -2 tant laisss  l'appelant
// ipc pointe sur le 1er caractre aprs le string
// cas particulier:
//   lorsqu'un caractre \ apparait dans la chane,
//   le caractre suivant est pris en compte
//   quel qu'il soit
{ int istring;
  char c;

  strcpy(string,"");
  for (;;)
  { switch (pc[ipc])
	 { case 0  :return -2;
		case ']':// cas normal o on arrive en bout des strings
					// c'est l'appelant qui traite l'erreur le cas chant
					return -1;
		case '"':istring=0;
			 for (;istring<254;)
			 { ipc++;
				switch (pc[ipc])
				{ case 0:Wmsg->show();
							Wmsg->wr(ml(5312,"unterminated string"));
							return 0;
				  case '"': ipc++; return 1;
				  case '\\': ipc++;
				  default: c=pc[ipc]; break;
				}
				string[istring]=c; istring++;
				string[istring]=0;
			 }
			 Wmsg->show();
			 Wmsg->wr(ml(5313,"string too long, more than 255 characters"));
			 return 0;
		default:ipc++; break;
	 }
  }
}
int langue::GetString1(char *pc,int& ipc
							,char *string)
// get 1 et 1 seul string.
// retour >0 il y a un string
//        <=0 erreur
// ipc pointe sur le 1er caractre aprs le string
// cas particulier:
//   lorsqu'un caractre \ apparait dans la chane,
//   le caractre suivant est pris en compte
//   quel qu'il soit
{ int ret;

  ret=GetString(pc,ipc,string);
  if (ret<=0)
  { Wmsg->show();
	 Wmsg->wr(ml(5314,"missing string"));
  }
  return ret;
}
int langue::GetStringF(FILE *fichier,int& nligne
							,char *ligne,int& iligne
							,char *string)
// get 1 ou plusieurs strings sur la ligne courante
//   du fichier ou les lignes suivantes qui sont
//   appeles automatiquement si besoin.
// retour 0 erreur
//        >0 il y a un string
//        <0 il n'y a plus de string disponible
// ipc pointe sur le 1er caractre aprs le string
// cas particulier:
//   lorsqu'un caractre \ apparait dans la chane,
//   le caractre suivant est pris en compte
//   quel qu'il soit
{ int ret;

  for (;;)
  { ret=GetString(ligne,iligne,string);
	 if (ret!=-2) return ret;
	 if (!GetLigne(fichier,nligne,ligne,iligne))
	 { Wmsg->show();
		Wmsg->wr(ml(5315,"error while expecting a string"));
		return 0;
	 }
  }
}

int langue::addtext(char *text)
{ int n,ret;
  ret=ntext;
  ttext=(char *)realloc(ttext,n=ntext+strlen(text)+1);
  strcpy(ttext+ntext,text);
  ntext=n;
  return ret;
}
char langue::phon_TextToi(char *text)
// retour <0 si pas trouv
{ char i;
  char ret;
  for (i=0,ret=-1;(i<nphon)&&(ret<0);i++)
    if (strcmp(text,ttext+tphon[i].iname)==0) ret=i;
  return ret;
}
char langue::class_TextToi(char *text)
// retour <0 si pas trouv
//        et jamais 0
{ char i;
  char ret;
  for (i=1,ret=-1;i<nclassphon;i++)
	 if (strcmp(text,ttext+tclassphon[i].iname)==0) ret=i;
  return ret;
}
bool langue::comp_phon(char *phon1, char *phon2)
// retour TRUE s'il y a un phonme commun
//     entre phon1 et phon2 qui peuvent chacun tre
//     une classe ou un phonme
{ char i,c;
  int j;
  if (phon1[0]!='[')
  { if (phon2[0]!='[')
      return (strcmp(phon1,phon2)==0);
    else
      return comp_phon(phon2,phon1);
  }
  else
  { i=class_TextToi(phon1);
    if (i<0) return FALSE;
    for (j=tclassphon[i].ilist;(c=ttext[j])!=0;j++)
    { if (c>=0)
      {	if (comp_phon(phon2,ttext+tphon[c].iname))
	  return TRUE;
      }
      else
      	if (comp_phon(phon2,ttext+tclassphon[-c].iname))
	  return TRUE;
    }
    return FALSE;
  }
}

MenuSquel *langue::BuildSubMenu(MenuSquel *squelDeb, char *pc,int& ip,int npcmax,bool AllowClasses, char *nomSubMenu)
{
  int id,iphon;
  bool premFois = true;
  int ind;
  bool iend;
  char ch[255];
  MenuSquel *squel = (squelDeb) ? squelDeb : new MenuSquel();

  // dpasser d'abord le 1er {
  while (pc[ip]!='[') ip++; ip++;
  //
  do
  {
	 if (GetString1(pc,ip,ch)<=0)
	 { Wmsg->show();
		Wmsg->wr(ml(5316,"error submenu without any string"));
		return (TMenu *) NULL;
	 }
    if (premFois) {
    	strcpy(nomSubMenu, ch);
      premFois = false;
    }
    ind=0;
	 do
	 { iend=FALSE;
		switch (pc[ip])
      { case ',':
		  case ']':
			 if ( (!AllowClasses) || (ch[0]!='[') )
			 { iphon=phon_TextToi(ch);
				if (iphon<0)
				{ char ch1[255];
				  Wmsg->show();
				  sprintf(ch1,ml(5317,"the string \"%s\" is not a phoneme"),ch);
				  Wmsg->wr(ch1);
				  return (TMenu *) NULL;
				}
				id=ID_PHON0+iphon;
						strcat(ch,"    ");
				strcat(ch,ttext+tphon[iphon].itext);
			 }
			 else
			 { int i,j,k;
				iphon=class_TextToi(ch);
				if (iphon<0)
				{ char ch1[255];
				  Wmsg->show();
				  sprintf(ch1,ml(5318,"the string \"%s\" is not a class"),ch);
				  Wmsg->wr(ch1);
				  return (TMenu *) NULL;
				}
				id=ID_PHON0-iphon;
						strcat(ch,"    ");
				strcat(ch,ttext+tclassphon[iphon].itext);
				strcat(ch,"  [");
				for (i=tclassphon[iphon].ilist,k=0;
					  ttext[i]!=0;
					  i++,k++)
				{ if (k!=0) strcat(ch,",");
				  if ((j=ttext[i])>0)
					 strcat(ch,ttext+tphon[j].iname);
				  else
					 strcat(ch,ttext+tclassphon[-j].iname);
				}
				strcat(ch,"]");
			 }
			 // rajouter lment normal au menu
			 squel->addMenuItem(MF_STRING | MF_OWNERDRAW,id,ch, (class MenuSquel *) NULL);
			 ind=1;
		    break;
		  case '[':
			 { int ip0=ip;
            char nom[255];
            MenuSquel *sousmenu;
            sousmenu = BuildSubMenu((MenuSquel *)NULL, pc,ip,npcmax,AllowClasses, nom);
				if (sousmenu) {
            	squel->addMenuItem(MF_POPUP, (uint)0, ch, sousmenu);
            }
				else {
              char ch1[255],ch2[255];
				  strncpy(ch1,pc+ip0,30); ch1[30]=0;
				  sprintf(ch2,ml(5319,"error in submenu %s..."),ch1);
				  Wmsg->show();
				  Wmsg->wr(ch2);
				  return (TMenu *) NULL;
				}
			 }
			 while ((pc[ip]!=',')&&(pc[ip]!=']')) ip++;
			 ind=1;
			 break;
		}
		if (pc[ip]==']') iend=TRUE;
      ip++;
    }while ((ip<npcmax)&&(ind==0));
  }while ((ip<npcmax)&&(iend!=TRUE));
  return squel;
}

MenuSquel *langue::BuildMenu(FILE *fichier,int& nligne,bool AllowClasses)
{
  char *pc; // texte original complet du menu
  int ip=0; // indice courant sur pc
  int npc=0,npcmax; // longueur du texte pc
  char c;
  bool ret=FALSE;
  MenuSquel *squel = new MenuSquel();

  squel->addMenuItem(MF_STRING,ID_PHON0-128,ml(5304,"Cancel"),
  							(class MenuSquel *) NULL);
  // lecture du menu dans le fichier
  pc=(char *)malloc(npc+1);
  pc[0]=0;
  do
  { int i;
	 pc=(char *)realloc(pc,npc+255);
	 pc[npc]=0;
	 fgets(pc+npc,255,fichier);
	 nligne++;
	 c=pc[npc]; npcmax=npc;
	 i=npc+strlen(pc+npc);
	 if (i>0)
		if (pc[i-1]==0xa) pc[i-1]=0;
	 npc+=strlen(pc+npc);
  } while ((c!=']')
           &&(!feof(fichier)));
  if (!feof(fichier))
  { int ip0=ip;
    char nom[255];
	 if (BuildSubMenu(squel,pc,ip,npcmax,AllowClasses,nom))
  		ret=TRUE;
	 else
			{ char ch1[255],ch2[255];
			  strncpy(ch1,pc+ip0,30); ch1[30]=0;
			  sprintf(ch2,ml(5320,"error in menu %s..."),ch1);
			  Wmsg->show();
			  Wmsg->wr(ch2);
			}
  }
  free(pc);
  //
  return squel;

}
static void errorNewLanguage(int& nligne,char *ligne)
// affiche l'erreur
// et met nligne  -1
{ char ch[300];
  sprintf(ch,ml(5305,"erreur de lecture du fichier langage\n\
 la ligne %d\n"),nligne);
  strncat(ch,ligne,40);
  strcat(ch," ...");
  strcat(ch,ml(5321,"\nvoir dtails dans la fenetre message"));
  affiche_erreur(ch);
  nligne=-1;
}
bool langue::NewLanguage(char *filename)
{
  int i;
  FILE *fichier;
  char ligne[255];
  int nligne=0; // numro de ligne

  clear();
  fichier=fopen(filename,"rt");
  if (!fichier) {
    fatal_error("Unable to open the phonetic description file (language file)");
    return false;
  }
  while ((!feof(fichier))&&(nligne>=0))
  { fgets(ligne,254,fichier);
	 nligne++;
	 i=strlen(ligne);
	 if (i>0)
		if (ligne[i-1]==0xa) ligne[i-1]=0;
	 if (strnicmp(ligne,"phon",4)==0)
	 { int i=0; // pointeur dans la ligne
		char pname[255],ptext[255];

		if (  (GetString1(ligne,i,pname)>0)
			 &&(GetString1(ligne,i,ptext)>0) )
		{ if (*pname==0)
		  { // cas particulier du phonme "rien"
			 tphon[0].itext=addtext(ptext);
		  }
		  else
		  { // cas gnral
			 tphon=(phon *)realloc(tphon
					 ,(nphon+1)*sizeof(phon));
			 phon *p;
			 p=&tphon[nphon];
			 p->iname=addtext(pname);
			 p->itext=addtext(ptext);
			 nphon++;
		  }
		}
		else
		  errorNewLanguage(nligne,ligne);
	 }
	 else if (strnicmp(ligne,"class",5)==0)
	 { char ip[255];
      int iip=0;
		int ind;
		int i=0;
		char pname[255],ptext[255];
		char ch[255];

		if (  (GetString1(ligne,i,pname)>0)
			 &&(GetString1(ligne,i,ptext)>0) )
		  { while ((i<254)&&(ligne[i]!='[')) i++; i++;
			 iip=0;
			 while ((ind=GetStringF(fichier,nligne,ligne,i,ch))>=0)
					  // ind ==0 erreur, >0 il y a un string  <0 fin
			 { int iphon=-1;
				if (ind>0)
				{ ip[iip]= (ch[0]=='[')
							 ? -(iphon=class_TextToi(ch))
							 :  (iphon=phon_TextToi(ch));
				  iip++;
				}
				if ((ind==0)||(iphon<0))
				{
              errorNewLanguage(nligne,ligne);
				  i=INFINI_I;
				  break;
				}
			 }
			 ip[iip]=0;
			 { tclassphon=(classphon *)realloc(tclassphon
				 ,(nclassphon+1)*sizeof(classphon));
				classphon *p;
				p=&tclassphon[nclassphon];
				p->iname=addtext(pname);
				p->itext=addtext(ptext);
				p->ilist=addtext(ip);
		      nclassphon++;
			 }
	}
	else
	  errorNewLanguage(nligne,ligne);
	 }
	 else if (strnicmp(ligne,"menuphon",8)==0)
	 {
      squelMenuPhoneme = BuildMenu(fichier,nligne,FALSE);
       if (!squelMenuPhoneme)
		  errorNewLanguage(nligne,ligne);
	 }
	 else if (strnicmp(ligne,"menuclass",9)==0)
	 {
    	squelMenuClasses = BuildMenu(fichier,nligne,TRUE);
		if (!squelMenuClasses)
         errorNewLanguage(nligne,ligne);
	 }
	 else if (strnicmp(ligne,"F1F2vowel",9)==0)
	 { int i=0; // pointeur dans la ligne
		int j,iphon;
		char pname[255];
		if (GetString1(ligne,i,pname)>0)
		{ iphon=phon_TextToi(pname);
		  if (iphon>=0)
		  { tF1F2vowel=(F1F2vowel *)realloc(tF1F2vowel
				 ,(nF1F2vowel+1)*sizeof(F1F2vowel));
			 j=nF1F2vowel;
			 tF1F2vowel[j].phon=iphon;
			 sscanf(ligne+i,"%d %d"
				,&tF1F2vowel[j].f1,&tF1F2vowel[j].f2);
			 nF1F2vowel++;
		  }
		  else
			 errorNewLanguage(nligne,ligne);
		}
	 }
  }
  fclose(fichier);
  return (nligne>=0);
}




char *TextOfPhon(int phon)
{ char *p;
  if (phon>=0)
	 p=&langue.ttext[langue.tphon[phon].iname];
  else
	 p=&langue.ttext[langue.tclassphon[-phon].iname];
  return p;
}
int phon_TextToi(char *text)
{ return langue.phon_TextToi(text);
}
bool comp_phon(char *phon1, char *phon2)
// retour TRUE s'il y a un phonme commun
//     entre phon1 et phon2 qui peuvent chacun tre
//     une classe ou un phonme
{ return langue.comp_phon(phon1,phon2);
}
void get_F1F2vowel(int i,int& phon,int& f1,int& f2)
// en retour phon,f1,f2
// et si i hors limite, alors phon=0
{ if ((i>=0)&&(i<langue.nF1F2vowel))
  { phon=langue.tF1F2vowel[i].phon;
    f1=langue.tF1F2vowel[i].f1;
    f2=langue.tF1F2vowel[i].f2;
  }
  else phon=0;
}
