/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier noise.cpp                                            
//

#include "common.h"
#pragma hdrstop

#include "snoori.h"
#include "files.h"
#include "outils.h"
#include "signal.h"
#include "editer.h"

/*
 *  Computes the average level fo the speech signal.
 */
double getSignaldB(
  long deb,
  long fin)
{
  int il;
  double e;
  for (il = _i(deb), e = 0.; il <= _i(fin); il++) e += SQR(space[il]);
  if (fabs(e) < EPSILON) {
    affiche_erreur(ml(9111,"Speech level set to the default value 60 dB."));
    return(60.);
  }
  return(10. * log(e / (_i(fin) -_i(deb) + 1)) / log(10.));
}

/*
 *  Add noise to speech.
 */
double addNoise(
  long deb, long fin,
  double signalDB,
  double SNRdB)
{
   int i, init;
   long tailleRegion = fin - deb + 1;
   struct time essai;
	double bruit, speech, alpha ; // coefficient de bruit pour obtenir un signal au SNR desire
   double srbdBCalcul;

   alpha = sqrt(6. / (RAND_MAX * RAND_MAX) * exp(log(10.) * (signalDB - SNRdB) / 10.));
   // La valeur moyenne de k^2, 0<= k <= RAND_MAX est RAND_MAX ^2 / 3.
   // Celle de  -RAND_MAX <= k  <= RAND_MAX est RAND_MAX ^2 / 6.

  if (!droit_ecrire_message(en_cours.nom)) return(INFINI_F / 2.);

  if (deb > fin || !dans(deb, ct.d_e, ct.f_e)
		|| !dans(fin, ct.d_e, ct.f_e)) {
    affiche_erreur(ml(9110,"La rgion  bruiter mal dfinie"));
    return(INFINI_F / 2.);
  }

  infoEdit.derOp = NOISE;
  infoEdit.debut = deb;
  infoEdit.fin = fin;
  strcat(strcpy(infoEdit.cut.nom,tempdir),"cutfile_.wa$");
  infoEdit.cut.nb_bits = NB_BITS;
  infoEdit.cut.depl = 0;
  infoEdit.cut.swap = SWAPINFILE;

  /* On sauve le signal qui va etre coupe */
  copieDansCutFile(&infoEdit);

  /*   Initialisation de la fonction rand()   */ // VOIR PORTABILITE
  gettime(&essai);
  init = (int)(essai.ti_sec * essai.ti_min * essai.ti_hour);  // avantage de changer souvent !
  srand(init);

  /*   Adjonction du bruit au signal    */
  for (i = _i(deb), bruit = 0., speech = 0.; i <= _i(fin); i++) {
    float x = alpha * (rand() - rand());
    speech += SQR(space[i]);
    bruit += SQR(x);
    space[i] += x;
  }

  // calcul du rapport signal sur bruit.
  srbdBCalcul = 10 * log10((speech + EPSILON) / (bruit + EPSILON));

  // bout avant la zone amortie : conserv en place
  ajtBoutSpectro(ct.d_a, infoEdit.debut - 1, ct.d_a, infoEdit.debut - 1);  
  // bout aprs la zone amortie : conserv en place
  ajtBoutSpectro(infoEdit.debut + tailleRegion + 1, ct.f_a,
		 infoEdit.debut + tailleRegion + 1, ct.f_a);
  // bout amorti : recalcul
  ajtBoutSpectro(-1, -1, infoEdit.debut, infoEdit.debut + tailleRegion);

  wsignal->aff();
  wsignal->paint();
  spectroRapide(wspectro->wsona);
  
  infoEdit.yaDesModifsSignal = VRAI;
  ct.debRegion = INFINI_L;
  ct.finRegion = -INFINI_L;

  return(srbdBCalcul);
}