/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier notetiq.cpp
//

#include "common.h"
#pragma hdrstop

#include "audio.h"
#include "signal.h"
#include "notetiq.h"

// NB: aucun contrle n'est fait sur le nombre d'lments
//   dans un tiquetage. Actuellement, il ne faut pas
//   que ce nombre dpasse 32K

#define NOTETIQ_TEXT_MAXLEN 40

notetiqs etiqs;
notetiqs notes;

bool isEtiqModifiable=TRUE;
bool isNoteModifiable=TRUE;

///////////////////////////////////////
// classe notetiq
//
notetiq::notetiq()
{
	text= new char[1];
  	text[0]=0;
}
notetiq::~notetiq()
{
	delete [] text;
}
char *notetiq::Text()
{
	return text;
}
void notetiq::SetText(char *Text)
{
  delete [] text;
  text = new char[strlen(Text)+1];
  strcpy(text,Text);
}

//////////////////////////////////////
// classe notetiqs
//
notetiqs::notetiqs()
{
	e = (notetiq **) NULL;
  	nb=0;
}
notetiqs::~notetiqs()
{
	clear();
   if (e) delete[] e;
  	e = (notetiq **) NULL;
}

void notetiqs::clear()
{ int i;

  for (i=0;i<nb;i++) delete e[i];
  if (e) delete[] e;
  e = (notetiq **) NULL;
  nb = 0;
}
int notetiqs::addX(long echt)
// en retour
//   >=0  ok, retour donne le numro de l'insertion
//   <0  insertion refuse,
//         echt existe dj en tant que limite
//         ou nombre maximum de limites atteint
{ int n;

  n=search(echt);
  if ((n>=0)&&(e[n]->limite==echt))
    return -1; // l'insertion existe dj
  return sp_add(n,echt);
}
int notetiqs::addS(long echt)
// addS() == addX() suivi de search()
//   diffre de addX() par la valeur retourne dans
//   le cas o la limite  rajouter se trouve dj
//   dans l'objet notetiqs 
// en retour
//   >=0  ok, retour donne le numro de la limite
//        aprs insertion
//   <0  insertion refuse,
//         nombre maximum de limites atteint
{ int n;

  n=search(echt);
  if ((n>=0)&&(e[n]->limite==echt))
    return n; // l'insertion existe dj
  return sp_add(n,echt);
}
int notetiqs::sp_add(int n,long echt)
// sous-programme commun  addX() et addS()
//   faisant l'insertion effective  la nime
//   limite
// en retour,
//   si ok, n
//   si ko, -1
{ int i;
  notetiq  **ee;

  if (nb>=INFINI_I-4) return -1;
  n++;
  // alloue la nouvelle mmoire ncessaire
  ee = new notetiq*[nb+1];

  for (i=0;i<n;i++) ee[i]=e[i];
  // fait de la place pour le nouvelle lment
  for (i=nb-1;i>=n;i--) ee[i+1]=e[i];
  // cre le nouvel lment
  ee[n]=new notetiq;
  ee[n]->limite=echt;
  // termine en mettant  jour le nombre d'lments
  nb++;
  delete[]e;
  e = ee;
  //
  return n;
}
int notetiqs::add_mt(long echt)
{ int ret;

  ret=addX(echt);
  if ((ret>0)&&(ret<nb-1))
  { if ( (e[ret-1]->limite+e[ret+1]->limite)
	 >(2*e[ret]->limite) )
    { e[ret]->SetText(e[ret-1]->Text());
      e[ret-1]->SetText("");
    }  
  }
  return ret;
}
void notetiqs::detach(long echt)
// efface la limite la plus proche de echt
{ int i,n;

  if (nb == 0) return;    // rien a faire si aucune limite existe

  // trouver d'abord l'indice de la limite la plus proche
  n=nearestLimit(echt);

  // dplacer le texte dans l'lment prcdent
  //   si c'est possible
  if ( (n>0)&&(strlen(e[n]->Text())!=0)
	    &&(strlen(e[n-1]->Text())==0) )
  { e[n-1]->SetText(e[n]->Text());
    e[n]->SetText("");
  }

  // cas du dernier lment
  if ((n>0)&&(n==nb-1)&&(strlen(e[n-1]->Text())!=0))
    n=-1;

  // si le texte n'est pas vide, on ne dtachera pas
  if ( (n>0)&&(strlen(e[n]->Text())!=0) ) n=-1;

  // en final, dtacher ou sortir un message d'erreur 
  if (n<0)
  { affiche_erreur(ml(5601,"effacer le texte\n\
 droite ou  gauche\n\
avant d'effacer la limite"));
  }
  else {
   // delete l'annotation  enlever
    notetiq **ee;
    delete e[n];

    ee = new notetiq*[nb-1];
    for (i=0;i<n;i++) ee[i]=e[i];
    // supprime la place de l'annotation enleve
    for (i=n;i<nb-1;i++) ee[i]=e[i+1];
    delete [] e; // dsalloue la mmoire devenue non ncessaire
	 e = ee;
    // termine en mettant  jour le nombre d'annotations
    nb--;
  }
}
int notetiqs::search(long echt)
// renvoie le numro de notetiq correspondant  echt
//     on appelle ci-dessous retour la variable retourne
//  retour==-1     avant le premier lment
//  0<=retour<nb     dans l'lment retour
//                      echt>=e[retour]->limite
//                   && ( (retour!=nb-1)
//                       && (ech<e[retour+1]->limite) )
//  retour==nb-1   aprs (ou dans) le dernier lment 
{ int n;

  for (n=nb-1;(n>=0)&&(e[n]->limite>echt);n--) ;
  return n;
}
int notetiqs::nearestLimit(long echt)
// renvoie le numro de la limite la plus proche de echt
// on garantit que le retour correspond bien
//    une limite existante.
// quand il n'y a aucune limite (nb==0), le rsultat
//   est imprvisible.
{ int n;

  n=search(echt);
  if (n<0) n=0;
  else
  { if ( (n<nb-1)
	 &&(echt>(e[n]->limite+e[n+1]->limite)/2) )
      n++;
  }
  return n;  
}
void notetiqs::shift(long echt)
// modifie la limite
//   sans dpasser celles des lments
//   prcdent et suivant
{ if (nb==0) return;
  e[nearestLimit(echt)]->limite=echt;
}
void notetiqs::SetText(int i,char *texte)
{ // contrle
  if ((i<0)||(i>=nb)) 
  { affiche_bug("notetiqs::SetText(n,texte) : n out of range");
    return;
  }
  //
  e[i]->SetText(texte);
}
char *notetiqs::Text(int i)
{ static char vide[]="";
  // contrle
  if ((i<0)||(i>=nb)) 
  { affiche_bug("notetiqs::Text(n) : n out of range");
    return vide;
  }
  //
  return e[i]->Text();
}
void notetiqs::edit_copy(long echtdebut,long echtfin,notetiqs& dst)
{ int i,j;
  long echtm; // milieu du segment

  for (i=0;i<nb;i++)
  { if (i==nb-1)
    { // dernire limite
      if (  (  (e[i]->limite>echtdebut)
	     ||(e[i]->limite<echtfin) ) )
      { //
	dst.addS(max(0,e[i]->limite-echtdebut));
      }
    }
    else
    { // non dernire limite
      echtm=(e[i]->limite+e[i+1]->limite)/2;
      if (  (echtm>echtdebut)
	  &&(echtm<echtfin)
	  &&(  (  (e[i]->limite>echtdebut)
                &&(e[i]->limite<echtfin) )
	     ||(  (e[i+1]->limite>echtdebut)
	        &&(e[i+1]->limite<echtfin) ) ) )
      { //
	j=dst.addS(max(0,e[i]->limite-echtdebut));
	dst.e[j]->SetText(e[i]->Text());
      }
    }
  }
}
void notetiqs::edit_cut(long echtdebut,long echtfin,notetiqs& dst)
{ edit_copy(echtdebut,echtfin,dst);
  edit_delete(echtdebut,echtfin);
}
void notetiqs::edit_paste(long echt,long length,notetiqs& src)
{ int i,j;
  long echtm; // milieu du segment

  // s'il existe un segment avec texte
  // recouvrant echt, alors rajouter limite en echt
  i=search(echt);
  if ( (i>0)&&(i<nb-1) )
  {
    if (  (e[i]->limite<echt)
	&&(e[i+1]->limite>echt) )
    { addS(echt);
      echtm=(e[i]->limite+e[i+1]->limite)/2;
      if (echtm<echt)
	i++;
      else
      { SetText(i+1,e[i]->Text());
	SetText(i,"");
      }
    }
  }
  // dcaler aprs echt pour faire la place  src
  for (i++;i<nb;i++) e[i]->limite+=length;
  // rajouter ce qui vient de src
  for (i=0;i<src.nb;i++)
  { j=addS(echt+src.e[i]->limite);
    e[j]->SetText(src.e[i]->Text());
  }
}
void notetiqs::edit_delete(long echtdebut,long echtfin)
{ int i;
  long echtm; // milieu du segment

  for (i=0;i<nb;i++)
  { if (i==nb-1)
    { // dernire limite
      if ( e[i]->limite>echtdebut )
      { //
	e[i]->limite=max(echtdebut
	         ,e[i]->limite-(echtfin-echtdebut));
      }
    }
    else
    { // non dernire limite
      echtm=(e[i]->limite+e[i+1]->limite)/2;
      if (  (e[i]->limite<echtdebut)
	  &&(e[i+1]->limite>echtdebut)
	  &&(e[i+1]->limite<echtfin) )
      { // segment  cheval sur echtdebut
	// ne dbordant pas aprs echtfin
	if ( echtm>echtdebut ) SetText(i,"");
        addS(echtdebut);
      }
      else if (  (e[i]->limite>echtdebut)
	       &&(e[i+1]->limite<=echtfin) )
      { // segment compris dans echtdebut/echtfin
	e[i]->SetText("");
        detach(e[i]->limite);
        i--;
      }   
      else if (  (e[i]->limite>echtdebut)
	       &&(e[i]->limite<echtfin)
	       &&(e[i+1]->limite>echtfin) )
      { // segment  cheval sur echtfin
	// ne dbordant pas avant echtdebut
        e[i]->limite=echtdebut;
        if ( echtm<echtfin ) SetText(i,"");
      }
      else if ( e[i]->limite>=echtfin )
      { // segment entirement aprs echtfin
	e[i]->limite-=echtfin-echtdebut;
      }
    }
  }
}

///////////////////////////////////////////////////
// affichage dans les fentres phonmes et mots
//
//
// procdure commune d'affichage
//
static void affNotetiqFen(
        notetiqs& ne,
        const char *fontName,
	d_planche *fenetre) /* fenetre d'affichage 			      */
{
  long premEchtAff=ct.d_a
      ,derEchtAff=ct.f_a;
  int i; //, y;

  SetTextAlign(fenetre->hDCcomp,TA_CENTER|TA_BOTTOM);
  snovcoor((float)premEchtAff, 0.
           ,(float) derEchtAff + 1, 1., fenetre); 
  MgiHue(BLANC, fenetre);
  snorbox((float) premEchtAff, 0.
          , (float) derEchtAff + 1, 1.0, fenetre);

  fenetre->SetPlancheFont(queryFont(fontName, 7, 14, 0));

  /* il faut que l'etiquette apparaisse dans la fenetre d'affichage */
  for (i = 0; i < ne.nb; i++) {
    float echtMilieu;

      // affichage de la limite
      if ( dans(ne.e[i]->limite, ct.d_a, ct.f_a)) {
        MgiHue(NOIR, fenetre);
	     snol(   (float) ne.e[i]->limite
	       ,(float) 0.0
	       ,(float) ne.e[i]->limite
	       ,(float) 1.0, fenetre);
      }
      // affichage du texte
      /* on enleve longueur / 2 * 7 car c'est la largeur
           de la moitie du texte */
      if (i!=ne.nb-1) {
        echtMilieu = (ne.e[i]->limite
                     + ne.e[i+1]->limite) / 2. ;
        if (dans(echtMilieu, premEchtAff, derEchtAff)) {
	       MgiHue(NOIR, fenetre);
          snorgfs(echtMilieu , 0.0, ne.e[i]->Text(), fenetre);
        }
      }
  }
  CopyArea_Dbw(fenetre, 0, 0,
	    fenetre->largeur, fenetre->hauteur, 0, 0,
	    GXcopy);
}
////////////////////////////////////////////
// Affichage de l'tiquetage orthographique
//
void aff_notes()
{
  affNotetiqFen(notes,"arial", wannotations);
}
////////////////////////////////////////////
// affichage de l'tiquetage phontique
//
void aff_etiqs()
{
  affNotetiqFen(etiqs,strlen(PhoneticFont) ? PhoneticFont : "arial", wetiquetage);
}

//////////////////////////////////////////////////
static void erreur_readnotetiqs(char *message)
{ char ch[200];
  strcpy(ch,ml(5606,"lecture de fichier .WAV mots ou phonmes\n"));
  strcat(ch,message);
  affiche_erreur(ch);
}
void readnotetiqs(char *soundName,notetiqs& ne
                         ,long depl,long longueur)
// attention aux fichiers contenant un grand nombre d'tiquettes
//   (nombre>INFINI_I). Au del de ce nombre, les tiquettes
//   sont ignores. Les performances sont mauvaises (et pourraient
//   tre amliores) lorsque le nombre d'tiquettes grandi.
//   Il est conseill de ne pas dpasser quelques centaines
//   d'tiquettes.
{ int j;
  HMMIO hmmio;
  long nbBytes;
  long realOffset;
  char proc[]="readnotetiqs()";
  char *p;
  long ip; // indice lment  traiter dans le buffer p
  int ind; // flip-flop limite et texte
	   //   dans le fichier, on a toujours
	   //     une limite suivi d'un texte
	   //     et un texte suivi d'une limite
	   // ==0 : limite  lire
	   // >0 : texte  lire
	   // <0 : erreur, arrter la lecture
  long offset;
  int it; // indice dernire limite entre dans notetiqs.

  ne.clear();

  if (longueur==0) return;

  if (erboxmmioOpen(1,proc,soundName
	,hmmio=mmioOpen(soundName,NULL,MMIO_READ))
	!=0) return;

  offset=depl;
  realOffset=mmioSeek(hmmio,offset,SEEK_SET);
  if (realOffset != offset)
  { affiche_bug("readnotetiqs() : offset out of file");
  }

  // lecture de l'tiquetage
  p = new char[longueur];
      // revoir le cas des trs grandes longueurs
      //   actuellement, on suppose que la mmoire est
      //   suffisamment grande pour l'tiquetage qui
      //   est lu en une seule fois
      if (erboxmmioRead(3,proc,soundName
	     ,nbBytes
	       = mmioRead(hmmio,(HPSTR)(p)
			  ,longueur))
         !=0 ) ind=-1;
      else
      { // vrifier que le nombre d'octets lus est 
	//     suffisant si le nombre d'octets demands 
        //     n'a pas t fourni en totalit
	if (nbBytes!=longueur)
	{ // erreur
	  erreur_readnotetiqs(
	    ml( 5602,
	    "readnotetiqs() : unexpected number of read bytes"));
	  ind=-1;
	}
      }
      erboxmmioClose(4,proc,soundName
		     ,mmioClose(hmmio,0));
  //
  ip=0; // pointeur dans le buffer de lecture
  ind=0; // on commence toujours par une limite
  while ((ip<longueur)&&(ind>=0))
  { switch (ind)
    { case 0:// on lit une limite
	     it=ne.addS(*((long *)(p+ip)));
		// revoir pour la performance
	     ip+=4; 
             //
	     ind=1;
             break;
      case 1:// on lit un texte
	     j=strlen((char *)(p+ip))+1; // dplacement  effectuer
	     if (  (ip+j>longueur) // scurit
		 ||(j>NOTETIQ_TEXT_MAXLEN) )
	     { erreur_readnotetiqs(
	         ml(5603,"readnotetiqs() : text too long"));
	       ind =-1;
	     }
             else
	     { if (it>=0) ne.SetText(it,(char *)(p+ip));
	       ip+=j; 
	       ind=0;
	     }
	     break;
    }
  }
  // vrifier qu'on a termin par une limite
  if (ind==0)
    erreur_readnotetiqs(
      ml(5604,"readnotetiqs() : missing end limit"));
  // et aussi qu'on a tout trait
  if (ip!=longueur)
    erreur_readnotetiqs(
      ml(5605,"readnotetiqs() : length error"));

  delete []p;
  return;
}



