/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier opt_for.cpp
//

#include "common.h"
#pragma hdrstop

#include "polypoint.h"
#include "guiklat.h"
#include "fftlpc.h"
#include "outils.h"
extern Wpiquefor *wpiquefor;

void Wpiquefor::initOptKlattSpec()
{
  optKlattSpec.or_lpc = ORDRE_LPC_STD_MS;
  optKlattSpec.nb_cep[0] = NB_CEP_STD1_SEC * 1000.;
  optKlattSpec.nb_cep[1] = NB_CEP_STD2_SEC * 1000.;
  optKlattSpec.fenLPC = 16.;
  optKlattSpec.fenCepstre = 32.;
  if (enEcht(optKlattSpec.fenCepstre) > 512)
    optKlattSpec.fenCepstre = enMilli(512);
}

///////////////////////////////////////////////
// fenetre de modifications des options
// applicables au menu KlattSpecs

// variables modifies par la fentre
//   optKlattSpec
// 

class TWoptionKlattSpecs : public TFrameWindow
{
   int ticket,xx,yy,ww;
  public:
    TWoptionKlattSpecs(PTWindowsObject AParent);
    virtual void SetupWindow();
    virtual bool CanClose();
    void myTStatic(char *texte);
	 // rponses aux vnements
	 /*
	 virtual void controle_nb_cep0(RTMessage)=[WM_USER+2];
    virtual void controle_nb_cep1(RTMessage)=[WM_USER+3];
	 virtual void defauts(RTMessage)=[ID_FIRST+101];
	 virtual void details(RTMessage)=[ID_FIRST+102];
	 */
	 LRESULT controle_nb_cep0(WPARAM,LPARAM);
	 LRESULT controle_nb_cep1(WPARAM,LPARAM);
	 LRESULT controle_fenCepstre(WPARAM,LPARAM);
	 void IDdefauts(WPARAM code);
	 void IDdetails(WPARAM code);
	 DECLARE_RESPONSE_TABLE(TWoptionKlattSpecs);
};
DEFINE_RESPONSE_TABLE1(TWoptionKlattSpecs,TFrameWindow)
  EV_MESSAGE(WM_USER+2,controle_nb_cep0),
  EV_MESSAGE(WM_USER+3,controle_nb_cep1),
  EV_MESSAGE(WM_USER+4,controle_fenCepstre),
  EV_CHILD_NOTIFY_ALL_CODES(101,IDdefauts),
  EV_CHILD_NOTIFY_ALL_CODES(102,IDdetails),
END_RESPONSE_TABLE;

TWoptionKlattSpecs::TWoptionKlattSpecs(PTWindowsObject AParent)
	:TFrameWindow(AParent,"")
{
  int w2;
  char ch[80];

  ticket=GetTicket();
  wsprintf(ch,ml(6426,"Options spectrales pour Klatt %d."),ticket);
  SetCaption(ch);
  DisableAutoCreate();
  Attr.Style = WS_VISIBLE | WS_POPUPWINDOW | WS_CAPTION;
  Attr.X = 50;
  Attr.Y = 50;
  Attr.W = 250;
  this->SetIcon(GetApplication(), IDI_ICON1);

  // initialiser le contenu des contrles
  //   avec les variables correspondantes
  xx=2;yy=2;ww=Attr.W-6;

  myTStatic(ml(6428,"LPC"));
  // or_lpc
  new TWfloat(this,0,xx,yy,ww 
	      ,&wpiquefor->optKlattSpec.or_lpc,ml(5704,"ordre lpc")
	      ,ORDRE_LPC_MIN_MS,ORDRE_LPC_MAX_MS
	      ,ORDRE_LPC_MIN_MS,ORDRE_LPC_MAX_MS
	      ,ticket);        yy+=HNUM;
  // fenLPC
  new TWfloat(this,0,xx,yy,ww
	      ,&wpiquefor->optKlattSpec.fenLPC,ml(6427,"Fentre LPC (ms)")
	      ,1.,64.,1.,64.
	      ,ticket);        yy+=HNUM;

  //
  myTStatic(ml(6429,"Cepstral analysis"));
  // nb_cep[0]
  new TWfloat(this,2,xx,yy,ww 
	      ,&wpiquefor->optKlattSpec.nb_cep[0],ml(5706,"dbut du liftrage")
	      ,0.,5.,0.,5.
	      ,ticket);        yy+=HNUM;
  // nb_cep[1]
  new TWfloat(this,3,xx,yy,ww 
	      ,&wpiquefor->optKlattSpec.nb_cep[1],ml(5707,"fin du liftrage")
	      ,0.,25.,0.,25.
	      ,ticket);        yy+=HNUM;
  new TWfloat(this,0,xx,yy,ww
	      ,&wpiquefor->optKlattSpec.fenLPC,ml(6430,"Fentre cepstre (ms)")
	      ,16.,64.,16.,64.
	      ,ticket);        yy+=HNUM;
  //
  yy+=10;
  w2=Attr.W/2;
  new TButton(this,101,ml(5708,"Dfauts")
              ,   5,yy,w2-10,24,FALSE);
  new TButton(this,102,ml(5709,"Dtails")
	      ,w2+3,yy,w2-10,24,FALSE);
  yy+=HNUM;
  Attr.H = yy+30;
}
void TWoptionKlattSpecs::SetupWindow()
{
  TFrameWindow::SetupWindow();
}
bool TWoptionKlattSpecs::CanClose()
{
  active(MENUchangeAnaSpec);
  return true;
}
void TWoptionKlattSpecs::myTStatic(char *texte)
{
  TStatic *PTS;
  PTS=new TStatic(this,-1,texte
		  ,xx,yy+2,ww,HNUM-2,0);
  yy+=HNUM;
  PTS->Attr.Style&=~SS_LEFT;
  PTS->Attr.Style|=SS_CENTER;
}
LRESULT TWoptionKlattSpecs::controle_nb_cep0(WPARAM,LPARAM)
{
  wpiquefor->optKlattSpec.nb_cep[0] = min(wpiquefor->optKlattSpec.nb_cep[0]
				 ,wpiquefor->optKlattSpec.nb_cep[1]);
  ::InvalidateRect(HWindow,NULL,true);
  return 0;
}
LRESULT TWoptionKlattSpecs::controle_nb_cep1(WPARAM,LPARAM)
{
  wpiquefor->optKlattSpec.nb_cep[1] = max(wpiquefor->optKlattSpec.nb_cep[0]
				 ,wpiquefor->optKlattSpec.nb_cep[1]);
  ::InvalidateRect(HWindow,NULL,true);
  return 0;
}
LRESULT TWoptionKlattSpecs::controle_fenCepstre(WPARAM,LPARAM)
{
  if (enEcht(wpiquefor->optKlattSpec.fenCepstre) > 512)
    wpiquefor->optKlattSpec.fenCepstre = enMilli(512);
  ::InvalidateRect(HWindow,NULL,true);
  return 0;
}
void TWoptionKlattSpecs::IDdefauts(WPARAM)
// doit tre cohrent avec l'init de optKlattSpec
{
  wpiquefor->initOptKlattSpec();
  ::InvalidateRect(HWindow,NULL,true);
}
void TWoptionKlattSpecs::IDdetails(WPARAM)
{ CHARbuf buf(1,"");
  char ch[80];

  sprintf(ch,ml(6431,"\nFentre LPC (echt)=%.2f")
				  ,enEchtf(wpiquefor->optKlattSpec.fenLPC));
  buf.cpy(ch);
  sprintf(ch,ml(5713,"\nordre lpc (echt)=%d")
			,ordre_lpc_msTOecht(wpiquefor->optKlattSpec.or_lpc));
  buf.cat(ch);
  sprintf(ch,ml(6432,"\nFentre cepstrale (echt) =%f")
			,enEchtf(wpiquefor->optKlattSpec.fenCepstre));
  buf.cat(ch);
  sprintf(ch,ml(5714,"\ndbut du liftrage (echt)=%.2f")
			,enEchtf(wpiquefor->optKlattSpec.nb_cep[0]));
  buf.cat(ch);
  sprintf(ch,ml(5715,"\nfin du liftrage (echt)=%.2f")
			,enEchtf(wpiquefor->optKlattSpec.nb_cep[1]));
  buf.cat(ch);
  ::MessageBox(HWindow
		  ,(char *)&buf[0],ml(5709,"details")
		  ,MB_OK | MB_TASKMODAL );
}

void Wpiquefor::CMchangeAnaSpec()
// fonction d'activation de la fentre dialogue
{
  GetModule()->MakeWindow(
		new TWoptionKlattSpecs(this) );
		// "nom" est le nom de la fentre
		//   dialog dans le fichier .rc
  desactive(MENUchangeAnaSpec);
}

