/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier optproso.cpp
//

#include "common.h"
#pragma hdrstop

#include "menu.h"
#include "spectro.h"
#include "pitchm.h"
#include "signal.h"
#include "outils.h"     // pour la fonction enMilli et enEcht(temps.cpp)
#include "files.h"
#include "pitchmk.h"
#include "optproso.h"

static float timeFactor_ = 1.5;
static float F0Factor_ = 1.;

///////////////////////////////////////////////
// fenetre de modifications des options
// de PSOLA

DEFINE_RESPONSE_TABLE1(TWTimeF0,TDialog)
  EV_CHILD_NOTIFY_ALL_CODES(IDOK,IDOk),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_SCROLLBAR3,IDscrollTime),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_SCROLLBAR4,IDscrollF0),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON3,EVChModSynt),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON4,EVChModSynt),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON13,EVChModPhaseCorrec),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON14,EVChModPhaseCorrec),
  EV_WM_CLOSE,
END_RESPONSE_TABLE;


TWTimeF0::TWTimeF0(PTWindowsObject AParent, LPSTR name)
	:TDialog(AParent,name)
{
  PTpsola = new TRadioButton(this,IDC_RADIOBUTTON3,0);
  PTvocoder = new TRadioButton(this,IDC_RADIOBUTTON4,0);
  PTNocorrec = new TRadioButton(this,IDC_RADIOBUTTON13,0);
  PTPhaseLock = new TRadioButton(this,IDC_RADIOBUTTON14,0);
  PTCancel = new TButton(this,IDCANCEL);
  PTApply = new TButton(this, IDOK);
  PTTimeFactor = new TScrollBar(this, IDC_SCROLLBAR3);
  PTF0Factor = new TScrollBar(this, IDC_SCROLLBAR4);
  PTTextTime =  new TStatic(this, IDC_STATICTEXT30);
  PTTextF0 = new TStatic(this, IDC_STATICTEXT3);
  PTgroupVocoder = new TGroupBox(this, IDC_GROUPBOX12);
}

bool TWTimeF0::CanClose()
{
  F0Factor_ = F0Factor;
  timeFactor_ = timeFactor;  
  return true;
}

TWTimeF0::~TWTimeF0()
{
  delete PTpsola;
  delete PTvocoder;
  delete PTCancel;
  delete PTApply;
  delete PTTimeFactor;
  delete PTF0Factor;
  delete PTTextTime;
  delete PTTextF0;
  delete PTNocorrec;
  delete PTPhaseLock;
  delete PTgroupVocoder;
}

void TWTimeF0::SetupWindow()
{
	STRING s;

   TDialog::SetupWindow();

   typeSynthese = PSOLA;
   withPhaseLocking = true;
   F0Factor = F0Factor_;
   timeFactor = timeFactor_;
   if (typeSynthese == PSOLA) PTpsola->SetCheck(BF_CHECKED);
   else PTvocoder->SetCheck(BF_CHECKED);
   sprintf(s,"%.0f", F0Factor);
   PTTextF0->SetText(s);

   if (withPhaseLocking) PTPhaseLock->SetCheck(BF_CHECKED);
   else PTNocorrec->SetCheck(BF_CHECKED);

   PTTimeFactor->SetRange(50, 300, true);
   PTTimeFactor->SetPosition((int) (timeFactor * 100));
   sprintf(s,"%.2f", timeFactor);
   PTTextTime->SetText(s);

   PTF0Factor->SetRange(50, 250, true);
   PTF0Factor->SetPosition((int) (F0Factor * 100));
   sprintf(s, "%.2f", F0Factor);
   PTTextF0->SetText(s);
   disableVocoder();
}

void TWTimeF0::EVChModSynt(WPARAM code)
{
  if (PTpsola->GetCheck() == BF_CHECKED) {
    disableVocoder();
    disablePsola();
    typeSynthese = PSOLA;
  }
  if (PTvocoder->GetCheck() == BF_CHECKED) {
    disablePsola();
    disableVocoder();
    typeSynthese = VOCODER;
  }
}

void TWTimeF0::EVChModPhaseCorrec(WPARAM code)
{
  withPhaseLocking = PTPhaseLock->GetCheck() == BF_CHECKED;
}

void TWTimeF0::IDOk(WPARAM code)
{
  if (typeSynthese == PSOLA) pitchScaleMod(F0Factor, timeFactor);
  else vocoderLaroche(timeFactor, withPhaseLocking);
}

void TWTimeF0::IDscrollTime(WPARAM code)
{
  int pos;
  STRING s;
  pos = PTTimeFactor->GetPosition();
  timeFactor = (float) pos / 100.;
  sprintf(s, "%.2f", timeFactor);
  PTTextTime->SetText(s);
}

void TWTimeF0::IDscrollF0(WPARAM code)
{
  int pos;
  STRING s;
  pos = PTF0Factor->GetPosition();
  F0Factor = (float) pos / 100.;
  sprintf(s, "%.2f", F0Factor);
  PTTextF0->SetText(s);
}

#define INVERSE(X) oldStyle = X->GetStyle(); X->TWindow::SetWindowLong(GWL_STYLE, oldStyle ^= (WS_DISABLED));
void TWTimeF0::disableVocoder()
{
  uint32 oldStyle;
  INVERSE(PTNocorrec);
  INVERSE(PTPhaseLock);
  INVERSE(PTgroupVocoder);
  ::InvalidateRect(HWindow,NULL,TRUE);
  UpdateWindow();
 }

void TWTimeF0::disablePsola()
{
  uint32 oldStyle;
  INVERSE(PTF0Factor);
  INVERSE(PTTextF0);
  ::InvalidateRect(HWindow,NULL,TRUE);
  UpdateWindow();
}

void Wtop::CMPitchScaleMod()
// fonction d'activation de la fentre dialogue
{
  PTWtop->GetModule()->ExecDialog(new TWTimeF0(PTWtop,"WTimeF0"));
}


#ifdef UIY
class TWoptionsProsodie : public TFrameWindow
{   int ticket,xx,yy,ww;
  public:
    TWoptionsProsodie(PTWindowsObject AParent);
    virtual void SetupWindow();
    virtual bool CanClose();
    void myTStatic(char *texte);

	 LRESULT controle_timeFactor(WPARAM,LPARAM);
	 LRESULT controle_f0Factor(WPARAM,LPARAM);
	 DECLARE_RESPONSE_TABLE(TWoptionsProsodie);
};

DEFINE_RESPONSE_TABLE1(TWoptionsProsodie,TFrameWindow)
  EV_MESSAGE(WM_USER+2,controle_timeFactor),
  EV_MESSAGE(WM_USER+3,controle_f0Factor),
END_RESPONSE_TABLE;

TWoptionsProsodie::TWoptionsProsodie(PTWindowsObject AParent)
	:TFrameWindow(AParent,"")
{ 
  char ch[80];

  ticket=GetTicket();
  wsprintf(ch,ml(6942,"Options des modifications temps-F0"),ticket);
  SetCaption(ch);
  DisableAutoCreate();
  Attr.Style = WS_VISIBLE | WS_POPUPWINDOW | WS_CAPTION;
  Attr.X = 50;
  Attr.Y = 50;
  Attr.W = 250;
  this->SetIcon(GetApplication(), IDI_ICON1);

  // initialiser le contenu des controles
  //   avec les variables correspondantes
  xx=2;yy=2;ww=Attr.W-6;

  new TWfloat(this,2,xx,yy,ww
	      ,&optProso.timeFactor,ml(6940,"Facteur temporel")
	      ,0.5,3.,0.5,3.
	      ,ticket);        yy+=HNUM;

  new TWfloat(this,3,xx,yy,ww
	      ,&optProso.f0Factor,ml(6941,"Facteur pour F0")
	      ,0.5,2.5,0.5,2.5
	      ,ticket);        yy+=HNUM;
  Attr.H = yy+30;
}

void TWoptionsProsodie::SetupWindow()
{ TFrameWindow::SetupWindow();
}

bool TWoptionsProsodie::CanClose()
{ active(MENUChOptProso);
  return TRUE;
}


LRESULT TWoptionsProsodie::controle_timeFactor(WPARAM,LPARAM)
{
  ::InvalidateRect(HWindow,NULL,TRUE);
  return 0;
}

LRESULT TWoptionsProsodie::controle_f0Factor(WPARAM,LPARAM)
{
  ::InvalidateRect(HWindow,NULL,TRUE);
  return 0;
}


void Wtop::CMChOptProso()
// fonction d'activation de la fenetre dialogue
{
  GetModule()->MakeWindow(
		new TWoptionsProsodie(this) );
		// "nom" est le nom de la fenetre
		//   dialog dans le fichier .rc
  desactive(MENUChOptProso);
}

#endif
