/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier outils.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include "outils.h"
#include "signal.h"


/*
 * Dans ct toutes les origines sont relatives au debut du fichier,
 * cette fonction permet de se reperer par rapport au debut 
 * du stockage.
 */
long _i(long x) { return (x - ct.d_e); }

/****************************************************************/
/* Cette fonction calcule le maximum d'un tableau 		*/
/****************************************************************/

float max_f(
	FLOATbuf& t_,	/* tableau sur lequel porte la recherche */
	int debut,	/* indice de debut et fin de recherche	*/
        int fin)
{ float *t=(float *)&t_[0];
	float M = -INFINI_F;
        int i;
        for (i=debut;i<=fin;i++) M=max(M,t[i]);
	return(M);
}
/****************************************************************/
/* Cette fonction calcule le minimum d'un tableau 		*/
/****************************************************************/

float min_f(
	FLOATbuf& t_,	/* tableau sur lequel porte la recherche */
	int debut,	/* indice de debut et fin de recherche	*/
        int fin)
{ float *t=(float *)&t_[0];
	float M = INFINI_F;
	int i;
        for (i=debut;i<=fin;i++) M=min(M,t[i]);
	return(M);
}

float max_tf(
	float *t,	/* tableau sur lequel porte la recherche */
	int debut,	/* indice de debut et fin de recherche	*/
   int fin)
{
	float M = -INFINI_F;
   for (int i = debut;i <= fin; i++) M = max(M,t[i]);
	return(M);
}

double max_td(
	double *t,	/* tableau sur lequel porte la recherche */
	int debut,	/* indice de debut et fin de recherche	*/
   int fin)
{
	double M = -INFINI_D;
   for (int i = debut;i <= fin; i++) M = max(M,t[i]);
	return(M);
}

/****************************************************************/
/* Cette fonction calcule le minimum d'un tableau 		*/
/****************************************************************/

float min_ti(
	int *t,	/* tableau sur lequel porte la recherche */
	int debut,	/* indice de debut et fin de recherche	*/
   int fin)
{
	int M = INFINI_I;
   for (int i = debut; i <= fin; i++) M = min(M,t[i]);
	return(M);
}

float max_ti(
	int *t,	/* tableau sur lequel porte la recherche */
	int debut,	/* indice de debut et fin de recherche	*/
   int fin)
{
	int M = -INFINI_I;
   for (int i = debut;i <= fin; i++) M = max(M,t[i]);
	return(M);
}

/****************************************************************/
/* Cette fonction calcule le minimum d'un tableau 		*/
/****************************************************************/

float min_tf(
	float *t,	/* tableau sur lequel porte la recherche */
	int debut,	/* indice de debut et fin de recherche	*/
   int fin)
{
	float M = INFINI_F;
   for (int i = debut; i <= fin; i++) M = min(M,t[i]);
	return(M);
}

double min_td(
	double *t,	/* tableau sur lequel porte la recherche */
	int debut,	/* indice de debut et fin de recherche	*/
   int fin)
{
	double M = INFINI_D;
   for (int i = debut; i <= fin; i++) M = min(M,t[i]);
	return(M);
}

/*
//*********************************************************************
// Cette fonction calcul et localise le minimum d'un tableau de float *
//*********************************************************************
float min_fl(
	float t[],	// tableau a examiner		    
	int debut,	// indices extremes de la recherche 
	int fin,
	int *ou)	// emplacement du minimum 	    
{
	     float *p;
	     float m = INFINI_F;
	     int im;
	     for (p=(float *)(t+debut);p<=(float *)(t+fin);p++) 
		 if (*p<m) {
		    m = *p;
		    im = p - t;
		 }
	     *ou = im;
	     return(m);
}
*/

//*********************************************************************
// Cette fonction calcul et localise le maximum d'un tableau de float *
//*********************************************************************
// attention, pas homogne avec max_sl
float max_fl(
	float *t,	// tableau a examiner
	int debut,	// indices extremes de la recherche
        int fin,
	int *ou)	// emplacement du maximum
{
  float m = -INFINI_F;
  int im;

  for (int i=debut;i<=fin;i++)
    if (t[i] > m) { m = t[i]; im = i; }
  *ou = im;
  return(m);
}

double max_dl(
	double *t,	// tableau a examiner
	int debut,	// indices extremes de la recherche
        int fin,
	int *ou)	// emplacement du maximum
{
  double m = -INFINI_D;
  int im;

  for (int i=debut;i<=fin;i++)
    if (t[i] > m) { m = t[i]; im = i; }
  *ou = im;
  return(m);
}

/*
 * Copie d'un bloc de SAMPLE's
 */
void blkcpy(
     SAMPLEbuf& dest_,
     long dest_debut,
     SAMPLEbuf& ori_,
     long ori_debut,
     long n)
     
{ SAMPLE huge *dest=(SAMPLE huge *)&dest_[0];
  SAMPLE huge *ori=(SAMPLE huge *)&ori_[0];
  long i,j,k;

  for (i=0,j=dest_debut,k=ori_debut;
       i<n; i++,j++,k++)      dest[j]=ori[k];
}

void swap(char huge *t,long n)  /* n en caracteres */
// inverse les octets des mots d'un buffer *int
{ union uswap { unsigned char c[2];
		short i;
	      } u, huge *p; 

  for (p = (union uswap huge *)t;
       n>0; 
       p++,n-=2)
  { u = *p;  
    p->c[0] = u.c[1];
    p->c[1] = u.c[0];
  }
}

/*
 * Cette fonction calcule le secteur et la frequence
 * qu'indique la souris.
 */
long info_souris(
     d_planche *fen_but,
     int x,		// position de la souris par rapport
     int y,             //           a la fenetre 
     float *frequence)	// frequence pointee par la souris 
{
  float xr,yr;

  snoiunscalexy(x, y, &xr, &yr, fen_but);
  *frequence = canToHz (yr);

  return xr;
}

///////////////////////////////////////////////
//
// Suppression des caracteres egaux a c a la fin de la chaine de caracteres s.
//
char *suppFin( 
	char *s,
	char c)
{
  char *x;
  for (x = s + strlen(s) - 1;
       x >= s && *x == c; x--) *x = '\0';
  return(s);
}

//
// Suppression des caracteres c en debut de chaine de caracteres.
//
char *suppDebut( 
	char *s,
	char c)
{
  char *x, *p;
  for (x = s; *x && *x == c;) {
    for (p = x; *p; p++) *p = *(p + 1);
  }
  return(s);
}

//
// Lecture d'un champ delimite par un caractere
// separateur dans une chaine de caracteres.
//
static STRING _bufReadAField;
char *readAField( 
	char **s,
     	char separator)
{
   char *p;
   p = strchr(*s, separator);
   if (!p) return((char *) NULL);
   *p = '\0';
   strncpy(_bufReadAField, *s, MAX_LIGNE);
   _bufReadAField[MAX_LIGNE - 1] = '\0';
   *p = separator;
   *s = p + 1;
   return(_bufReadAField);
}

//
// Suppression des retour chariot et saut de ligne
//   en fin de chaine de caracteres.
//
char *suppcrnl( 
	char *s)
{ char *x;

  for (x = s + strlen(s) - 1;
       x >= s && ((*x=='\n')||(*x=='\r'));
       x--) *x = '\0';
  return(s);
}

//
// Suppression des blancs d'une chaine de caracteres.
//
char *suppblanc( 
	char *s)
{ char *x, *p;

  for (x = s; *x;)
  { if (*x == ' ') for (p = x; *p; p++) *p = *(p + 1);
    if (*x) x++;
  }
  return(s);
}
