/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier paraceps.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "snoori.h"
#include "signal.h"
#include "outils.h"
#include "wcoupe.h"

// variables modifies par la fentre
//   (selon le paramtre d'appel p)
//   p->fen_signal
//   p->fen_fft1
//   p->liftrage[0]
//   p->liftrage[1]
//   p->fen_fft2
//   p->niv
//   p->isPreac

static choix choix_fen_fft[] = { {256,"256"},{512,"512"}
			  ,{0,""} };

class TWparaLisCep : public TWparaCoupe
{   paramCoupeLisCep *param;

  public:
	 TWparaLisCep(PTWindowsObject AParent,char *titre
		  ,float fen_signal_min,float fen_signal_max
		  ,paramCoupeLisCep *p);
	 // rponses aux vnements
	 /*
	 virtual void controle_fen_fft1(RTMessage)=[WM_USER+2];
	 virtual void controle_fen_signal(RTMessage)=[WM_USER+3];
	 virtual void controle_debut_liftrage(RTMessage)=[WM_USER+4];
	 virtual void controle_fin_liftrage(RTMessage)=[WM_USER+5];
	 */
	 LRESULT controle_fen_fft1(WPARAM,LPARAM);
	 LRESULT controle_fen_fft2(WPARAM,LPARAM);
	 LRESULT controle_fen_signal(WPARAM,LPARAM);
	 LRESULT controle_debut_liftrage(WPARAM,LPARAM);
	 LRESULT controle_fin_liftrage(WPARAM,LPARAM);
	 DECLARE_RESPONSE_TABLE(TWparaLisCep);
    private:
      TWint *TWfft2;
};
DEFINE_RESPONSE_TABLE1(TWparaLisCep,TWparaCoupe)
  EV_MESSAGE(WM_USER+2,controle_fen_fft1),
  EV_MESSAGE(WM_USER+3,controle_fen_fft2),
  EV_MESSAGE(WM_USER+4,controle_fen_signal),
  EV_MESSAGE(WM_USER+5,controle_debut_liftrage),
  EV_MESSAGE(WM_USER+6,controle_fin_liftrage),
END_RESPONSE_TABLE;

TWparaLisCep::TWparaLisCep(PTWindowsObject AParent
						,char *titre
		  ,float fen_signal_min,float fen_signal_max
		  ,paramCoupeLisCep *p)
	:TWparaCoupe(AParent,titre,p)
{
  param=p;
  // initialiser le contenu des contrles
  //   avec les variables correspondantes
  // affichage des cepstres avant la dernire transforme de Fourier
  new TWchoix(this,0,xx,yy,ww
	      ,&param->beforeLastSpectrum
	      ,ml(6019, "cepstrum coefficients")
	      ,choix_YESNO 
	      ,ticket);                yy+=HNUM;
  // fenetre signal
  new TWfloat(this,4,xx,yy,ww
			,&param->fen_signal
			,ml(6002,"fenetre_signal=")
			,fen_signal_min,fen_signal_max
			,fen_signal_min,fen_signal_max
			,ticket);                 yy+=HNUM;
  // ordre premire fft pour le cepstre
  myTStatic(ml(6003,"1re fft de calcul du cepstre"));
  new TWint(this,2,xx,yy,ww
			,&param->fen_fft1
			,ml(6004,"log2(fenetre_fft1)="),8,11,8,11
			,ticket);                 yy+=HNUM;
  // ordre deuxime fft pour le cepstre
  myTStatic(ml(6020,"2me fft de calcul du cepstre"));
  TWfft2 = new TWint(this,3,xx,yy,ww
			,&param->fen_fft2
			,ml(6021,"log2(fenetre_fft2)="),8,10,8,10
			,ticket);                 yy+=HNUM;
  // liftrage
  myTStatic(ml(6005,"liftrage"));
  new TWfloat(this,5,xx,yy,ww
	      ,&param->debut_liftrage
	      ,ml(6006,"dbut (en sec)=")
	      ,0.,5.,0.,5.
	      ,ticket);                 yy+=HNUM;
  new TWfloat(this,6,xx,yy,ww
	      ,&param->fin_liftrage
	      ,ml(6007,"fin (en sec)=")
	      ,0.,10.,0.,10.
	      ,ticket);                 yy+=HNUM;
  // ordre fft finale
  myTStatic(ml(6008,"fft finale"));
  new TWint(this,0,xx,yy,ww
	      ,&param->fen_fft3
	      ,ml(6009,"log2(fenetre_fft3)="),8,11,8,11
	      ,ticket);                 yy+=HNUM;
  FinirCreate();
}

LRESULT TWparaLisCep::controle_fen_signal(WPARAM,LPARAM)
{ if ((1 << param->fen_fft1) <enEchtf(param->fen_signal))
  { param->fen_signal=enMillif(1 << param->fen_fft1);
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  return 0;
}
LRESULT TWparaLisCep::controle_fen_fft1(WPARAM,LPARAM)
{
  if ((1 << param->fen_fft1) <enEchtf(param->fen_signal))
  { while ((1 << param->fen_fft1) <enEchtf(param->fen_signal))
		param->fen_fft1++;
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  if (param->fen_fft2 > param->fen_fft1)
  { while (param->fen_fft1 < param->fen_fft2)
		param->fen_fft1++;
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  return 0;
}
LRESULT TWparaLisCep::controle_fen_fft2(WPARAM,LPARAM)
{ if (param->fen_fft2 > param->fen_fft1)
  {
    param->fen_fft2 = param->fen_fft1;
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  return 0;
}
LRESULT TWparaLisCep::controle_debut_liftrage(WPARAM,LPARAM)
{ param->debut_liftrage=max(0
								,min(param->debut_liftrage
				 ,param->fin_liftrage));
  ::InvalidateRect(HWindow,NULL,TRUE);
  return 0;
}
LRESULT TWparaLisCep::controle_fin_liftrage(WPARAM,LPARAM)
{ param->fin_liftrage=   max(param->debut_liftrage
									 ,param->fin_liftrage);
  ::InvalidateRect(HWindow,NULL,TRUE);
  return 0;
}

void Wcoupe::IDparaLisCep1(WPARAM) //RTMessage)
{ if (pLisCep1->isOpen) return;
  GetModule()->MakeWindow(
		new TWparaLisCep(this
			,ml(6010,"paramtres LisCep1")
				 ,0.5,64.,pLisCep1));
}

void Wcoupe::IDparaLisCep2(WPARAM) //RTMessage)
{ if (pLisCep2->isOpen) return;
  GetModule()->MakeWindow(
	   new TWparaLisCep(this
		   ,ml(6011,"paramtres LisCep2")
		       ,1.,128.,pLisCep2));
}
