/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

                             // fichier editMngmt.cpp
//

#include "common.h"
#include "signal.h"
#include "editer.h"
#include "editmngmt.h"
#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "snoori.h"
#include "signal.h"
#include "outils.h"

#include "wcoupe.h"

class TWCriticalBandManager : public TDialog
{
  paramCoupeCriticalBands *param;

  public:
    TWCriticalBandManager(PTWindowsObject AParent, LPSTR name, paramCoupeCriticalBands *p);
    ~TWCriticalBandManager();

   TRadioButton *PTPreemphasis;
   TRadioButton *PTNoPreemphasis;
   TScrollBar *PTFenSignal;
   TScrollBar *PTFenFFT;
   TScrollBar *PTNbBands;
   TScrollBar *PTPredictionOrder;
   TRadioButton *PTCompletionStrategyASR;
   TRadioButton *PTCompletionStrategySpectro;
   TRadioButton *PTFrequencyScaleBark;
   TRadioButton *PTFrequencyScaleMel;
   TRadioButton *PTFrequencyScaleMely;
   TRadioButton *PTCepsSmoothBeforeBand;
   TRadioButton *PTNoCepsSmoothBeforeBand;
   TRadioButton *PTNumberDepdendentWidth;
   TRadioButton *PTConstantWidth;
   
   TCheckBox  *PTIntensityConversion;
   TScrollBar *PTLifteringOrder;
   TScrollBar *PTdBLevel;
   TGroupBox *PTgroupcepsSmoothBefore;
   TGroupBox *PTgroupCepstral;
   TGroupBox *PTgroupPLP;

   TRadioButton *PTPLP;
   TRadioButton *PTCepstral;
   TRadioButton *PTNothing;

   TStatic *PTTextCepsSmoothBefore;
   TStatic *PTTextFenWindow;
   TStatic *PTTextFen_FFT;                                                     
   TStatic *PTTextNbBands;
   TStatic *PTTextLiftering;
   TStatic *PTTextdBLevel;
   TStatic *PTTextPredictionOrder;
   TStatic *PTTextPredictionOrderVal;

   TButton *PTCancel;
   TButton *PTOk;

   void checkParams();
   virtual void SetupWindow();
   virtual bool CanClose();
   virtual void EvClose();
   void IDOk(WPARAM code);
   void IDCancel(WPARAM code);

   void disableCepstral();
   void checkMelFilterWidthParam(bool mute);
   void disablePLP();
   void IDscrollFen_FFT(WPARAM);
   void IDscrollFenSignal(WPARAM);
   void IDscrollNbBands(WPARAM);
   void IDscrollLiftering(WPARAM);
   void IDscrolldBLevel(WPARAM);
   void IDscrollPLPOrder(WPARAM);

   void EVChPreemphasis(WPARAM code);
   void EVChCompletionStrategy(WPARAM code);
   void EVChScale(WPARAM code);
   void EVChBeforeBandIntegration(WPARAM code);
   void EVChWithLiftering(WPARAM code);
   void EVChAnalysis(WPARAM code);
   void EVChWidthStrategy(WPARAM code);
   void EVChIntensityConv(WPARAM code);
   DECLARE_RESPONSE_TABLE(TWCriticalBandManager);
};

DEFINE_RESPONSE_TABLE1(TWCriticalBandManager,TDialog)
  EV_CHILD_NOTIFY_ALL_CODES(IDOK,IDOk),
  EV_CHILD_NOTIFY_ALL_CODES(IDCANCEL,IDCancel),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_SCROLLBAR5,IDscrollFenSignal),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_SCROLLBAR6,IDscrollFen_FFT),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_SCROLLBAR7,IDscrollNbBands),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_SCROLLBAR8,IDscrollLiftering),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_SCROLLBAR9,IDscrolldBLevel),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_SCROLLBAR10,IDscrollPLPOrder),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON21,EVChCompletionStrategy),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON22,EVChCompletionStrategy),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON23,EVChScale),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON24,EVChScale),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON25,EVChScale),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON35,EVChBeforeBandIntegration),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON36,EVChBeforeBandIntegration),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON30,EVChAnalysis),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON31,EVChAnalysis),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON32,EVChAnalysis),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON33,EVChWidthStrategy),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_RADIOBUTTON34,EVChWidthStrategy),
  EV_CHILD_NOTIFY_ALL_CODES(IDC_CHECKBOX2,EVChIntensityConv),
  EV_WM_CLOSE,
END_RESPONSE_TABLE;


TWCriticalBandManager::TWCriticalBandManager(PTWindowsObject AParent, LPSTR name, paramCoupeCriticalBands *p)
	:TDialog(AParent,name)
{
  param = p;
  param->PTW =(PTWindowsObject)this;

  PTPreemphasis = new TRadioButton(this,IDC_RADIOBUTTON19,0);
  PTNoPreemphasis = new TRadioButton(this,IDC_RADIOBUTTON20,0);
  PTFenSignal = new TScrollBar(this, IDC_SCROLLBAR5);
  PTFenFFT = new TScrollBar(this, IDC_SCROLLBAR6);
  PTNbBands = new TScrollBar(this, IDC_SCROLLBAR7);
  PTCompletionStrategyASR = new TRadioButton(this,IDC_RADIOBUTTON21,0);
  PTCompletionStrategySpectro = new TRadioButton(this,IDC_RADIOBUTTON22,0);
  PTFrequencyScaleBark = new TRadioButton(this,IDC_RADIOBUTTON23,0);
  PTFrequencyScaleMel = new TRadioButton(this,IDC_RADIOBUTTON24,0);
  PTFrequencyScaleMely = new TRadioButton(this,IDC_RADIOBUTTON25,0);
  PTTextCepsSmoothBefore  =  new TStatic(this, IDC_STATICTEXT18);
  PTCepsSmoothBeforeBand = new TRadioButton(this,IDC_RADIOBUTTON35,0);
  PTNoCepsSmoothBeforeBand = new TRadioButton(this,IDC_RADIOBUTTON36,0);
  PTLifteringOrder = new TScrollBar(this, IDC_SCROLLBAR8);
  PTdBLevel = new TScrollBar(this, IDC_SCROLLBAR9);

  PTConstantWidth = new TRadioButton(this,IDC_RADIOBUTTON33,0);
  PTNumberDepdendentWidth = new TRadioButton(this,IDC_RADIOBUTTON34,0);

  PTTextFenWindow =  new TStatic(this, IDC_STATICTEXT29);
  PTTextFen_FFT = new TStatic(this, IDC_STATICTEXT30);
  PTTextNbBands = new TStatic(this, IDC_STATICTEXT34);
  PTTextLiftering = new TStatic(this, IDC_STATICTEXT19);
  PTTextdBLevel = new TStatic(this, IDC_STATICTEXT22);

  PTgroupcepsSmoothBefore = new TGroupBox(this, IDC_GROUPBOX18,0);
  PTPLP = new TRadioButton(this,IDC_RADIOBUTTON30,0);
  PTCepstral = new TRadioButton(this,IDC_RADIOBUTTON31,0);
  PTNothing = new TRadioButton(this,IDC_RADIOBUTTON32,0);
  PTgroupCepstral = new TGroupBox(this, IDC_GROUPBOX17);

  PTgroupPLP = new TGroupBox(this, IDC_GROUPBOX7);
  PTTextPredictionOrder = new TStatic(this, IDC_STATICTEXT38);
  PTTextPredictionOrderVal = new TStatic(this, IDC_STATICTEXT39);
  PTPredictionOrder = new TScrollBar(this, IDC_SCROLLBAR10);
  PTIntensityConversion = new TCheckBox(this,IDC_CHECKBOX2,0);

  PTCancel = new TButton(this,IDCANCEL);
  PTOk = new TButton(this,IDOK);
}


bool TWCriticalBandManager::CanClose()
{
  param->isOpen = false;
  return true;
}


void TWCriticalBandManager::EvClose()
{
  TDialog::EvClose();
  param->isOpen = false;
}

TWCriticalBandManager::~TWCriticalBandManager()
{
  delete PTPreemphasis;
  delete PTNoPreemphasis;
  delete PTFenSignal;
  delete PTFenFFT;
  delete PTNbBands;
  delete PTCompletionStrategyASR;
  delete PTCompletionStrategySpectro;
  delete PTFrequencyScaleBark;
  delete PTFrequencyScaleMel;
  delete PTFrequencyScaleMely;
  delete PTCepsSmoothBeforeBand;
  delete PTNoCepsSmoothBeforeBand;
  delete PTLifteringOrder;
  delete PTdBLevel;

  delete PTTextFenWindow;
  delete PTTextFen_FFT;
  delete PTTextNbBands;
  delete PTTextLiftering;
  delete PTTextdBLevel;

  delete PTgroupPLP;
  delete PTTextPredictionOrder;
  delete PTTextPredictionOrderVal;
  delete PTPredictionOrder;
  delete PTIntensityConversion;
  delete PTCancel;
  delete PTOk;

  delete PTConstantWidth;
  delete PTNumberDepdendentWidth;

  param->isOpen = false;
}

void TWCriticalBandManager::SetupWindow()
{
	STRING s;
   TDialog::SetupWindow();

   // param->analysis = 0; pour tester
   switch (param->analysis) {
     case 0: PTNothing->SetCheck(BF_CHECKED);
     disablePLP();
     disableCepstral();
     break;
     case 1: PTPLP->SetCheck(BF_CHECKED);
     disableCepstral();
     break;
     case 2: PTCepstral->SetCheck(BF_CHECKED);
     disablePLP();
     break;
   }
   if (param->isPreac) PTPreemphasis->SetCheck(BF_CHECKED);
   else PTNoPreemphasis->SetCheck(BF_CHECKED);
   PTIntensityConversion->SetCheck((param->intensityConversion) ? BF_CHECKED : BF_UNCHECKED);
   if (param->completionStrategy) PTCompletionStrategyASR->SetCheck(BF_CHECKED);
   else PTCompletionStrategySpectro->SetCheck(BF_CHECKED);
   switch (param->frequencyScale) {
      case Bark:
        PTFrequencyScaleBark->SetCheck(BF_CHECKED);
        break;
      case Mel:
        PTFrequencyScaleMel->SetCheck(BF_CHECKED);
        break;
      case Mely:
        PTFrequencyScaleMely->SetCheck(BF_CHECKED);
        break;
   }

   checkMelFilterWidthParam(true);
   if (param->melWidthStrategy == ConstantMelWidth) PTConstantWidth->SetCheck(BF_CHECKED);
   else PTNumberDepdendentWidth->SetCheck(BF_CHECKED);

   if (param->withCepstralSmoothing) {
     PTCepsSmoothBeforeBand->SetCheck(BF_CHECKED);
//     PTNoCepsSmoothBeforeBand->SetCheck(BF_UNCHECKED);
   }
   else {
     PTNoCepsSmoothBeforeBand->SetCheck(BF_CHECKED);
//     PTCepsSmoothBeforeBand->SetCheck(BF_UNCHECKED);
   }
   
   PTCancel = new TButton(this,IDCANCEL);
   PTOk = new TButton(this,IDOK);

   PTFenFFT->SetRange(1, 10, true);
   PTFenFFT->SetPosition((int) param->fen_fft);
   sprintf(s,"%d", param->fen_fft);
   PTTextFen_FFT->SetText(s);

   PTFenSignal->SetRange(1, 64, true);
   PTFenSignal->SetPosition((int) param->fen_signal);
   sprintf(s, "%.0f", param->fen_signal);
   PTTextFenWindow->SetText(s);

   PTNbBands->SetRange(1, 1024, true);
   PTNbBands->SetPosition((int) param->nbBands);
   sprintf(s,"%d", param->nbBands);
   PTTextNbBands->SetText(s);

   PTLifteringOrder->SetRange(1, 1024, true);
   PTLifteringOrder->SetPosition((int) param->nbMelLiftering);
   sprintf(s,"%d", param->nbMelLiftering);
   PTTextLiftering->SetText(s);

   PTdBLevel->SetRange(0, 60, true);
   PTdBLevel->SetPosition((int) (param->niv + 30));
   sprintf(s,"%.0f", param->niv);
   PTTextdBLevel->SetText(s);

   PTPredictionOrder->SetRange(1, 64, true);
   PTPredictionOrder->SetPosition(param->PLPorder);
   sprintf(s,"%d", param->PLPorder);
   PTTextPredictionOrderVal->SetText(s);

   param->isOpen = true;
}

void TWCriticalBandManager::IDOk(WPARAM)
{
  TDialog::CmOk();
  param->isOpen = false;
}

void TWCriticalBandManager::IDCancel(WPARAM)
{
 TDialog::CmCancel();
 param->isOpen = false;
}

void TWCriticalBandManager::IDscrolldBLevel(WPARAM)
{
  int pos;
  STRING s;
  pos = PTdBLevel->GetPosition();
  param->niv = pos - 30.;
  PTdBLevel->SetPosition((int) (param->niv + 30.));
  sprintf(s, "%.0f", param->niv);
  PTTextdBLevel->SetText(s);

}

void TWCriticalBandManager::IDscrollFen_FFT(WPARAM)
{
  int pos;
  STRING s;
  pos = PTFenFFT->GetPosition();
  param->fen_fft = pos;
  checkParams();
}

void TWCriticalBandManager::IDscrollFenSignal(WPARAM)
{
  int pos;
  pos = PTFenSignal->GetPosition();
  param->fen_signal = (float) pos;
  checkParams();
}

void TWCriticalBandManager::IDscrollNbBands(WPARAM)
{
  int pos;
  pos = PTNbBands->GetPosition();
  param->nbBands = pos;
  checkParams();
}

void TWCriticalBandManager::IDscrollLiftering(WPARAM)
{
  int pos;
  pos = PTLifteringOrder->GetPosition();
  param->nbMelLiftering = pos;
  checkParams();
}

void TWCriticalBandManager::IDscrollPLPOrder(WPARAM)
{
  int pos;
  STRING s;
  pos = PTPredictionOrder->GetPosition();
  param->PLPorder = pos;
  checkParams();
}

  // if the analysis is not PLP and the width strategy sets width from the number of filter,
  // i.e. the strategy of MFCC that sets the width from the number of filters there is a risk of
  // using a width of 0 if the number of filters is too large.
void TWCriticalBandManager::checkMelFilterWidthParam(
  bool mute)
{
   if (param->melWidthStrategy == VariableMelWidth && param->analysis != 1) {
     if (!checkMelFilterWidth(param->frequencyScale, param->nbBands, 1 << param->fen_fft, en_cours.frequence)) {
        param->melWidthStrategy = ConstantMelWidth;
        PTConstantWidth->SetCheck(BF_CHECKED);
        PTNumberDepdendentWidth->SetCheck(BF_UNCHECKED);
        if (!mute) affiche_erreur("Width strategy set to constant width because the width is too small.\n Decrease the number of bands");
     }
   }
}

void TWCriticalBandManager::checkParams()
{
  STRING s;
  if (param->analysis == 2) {
    if (param->nbMelLiftering  > (1 << param->fen_fft))
      param->nbMelLiftering = 1 << param->fen_fft;
    if (param->nbMelLiftering  > param->nbBands)
      param->nbMelLiftering = param->nbBands;
  }

  if (param->nbBands > (1 << (param->fen_fft - 1)))
    param->nbBands = 1 << (param->fen_fft - 1);

  if (param->analysis == 1) {
    if (param->nbBands < param->PLPorder) {
      param->nbBands = param->PLPorder;
      if (param->nbBands > (1 << (param->fen_fft - 1))) {
       param->nbBands = param->PLPorder = 1 << (param->fen_fft - 1);
      }
    }
  }

  if ((1 << param->fen_fft) < enEcht(param->fen_signal)) {
    for (; (1 << param->fen_fft) < enEcht(param->fen_signal) && param->fen_fft < 10;  param->fen_fft++);
    if ((1 << param->fen_fft) < enEcht(param->fen_signal)) param->fen_signal = enMilli(1 << param->fen_fft);
  }
  
  if (param->nbFreqSamplesDisplayed > (1 << (param->fen_fft - 1)))
    param->nbFreqSamplesDisplayed = 1 << (param->fen_fft - 1);

  checkMelFilterWidthParam(false);

  PTLifteringOrder->SetPosition(param->nbMelLiftering);
  sprintf(s, "%d", param->nbMelLiftering);
  PTTextLiftering->SetText(s);

  PTNbBands->SetPosition(param->nbBands);
  sprintf(s, "%d", param->nbBands);
  PTTextNbBands->SetText(s);

  PTFenFFT->SetPosition(param->fen_fft);
  sprintf(s, "%d", param->fen_fft);
  PTTextFen_FFT->SetText(s);

  PTFenSignal->SetPosition(param->fen_signal);
  sprintf(s, "%d", (int) param->fen_signal);
  PTTextFenWindow->SetText(s);

  PTPredictionOrder->SetPosition(param->PLPorder);
  sprintf(s, "%d", param->PLPorder);
  PTTextPredictionOrderVal->SetText(s);
}


void TWCriticalBandManager::EVChAnalysis(WPARAM code)
{
   if (PTCepstral->GetCheck() == BF_CHECKED) {
     if (param->analysis != 2) {
       if (param->analysis) disablePLP();
       param->analysis = 2;
       param->withDctIdctLiftering = true;
       disableCepstral();
     }
   }
   if (PTNothing->GetCheck() == BF_CHECKED)  {
     if (param->analysis != 0) {
       if (param->analysis == 2) disableCepstral();
       if (param->analysis == 1) disablePLP();
       param->analysis = 0;
       param->withDctIdctLiftering = false;
     }
   }
   if (PTPLP->GetCheck() == BF_CHECKED)  {
     if (param->analysis != 1) {
       if (param->analysis) disableCepstral();
       param->analysis = 1;
       disablePLP();
     }
   }
}

void TWCriticalBandManager::EVChWidthStrategy(WPARAM code)
{
  param->melWidthStrategy = (PTConstantWidth->GetCheck() == BF_CHECKED) ? ConstantMelWidth : VariableMelWidth;
  checkMelFilterWidthParam(false);
  ::InvalidateRect(HWindow,NULL,TRUE);
  UpdateWindow();
}

void TWCriticalBandManager::EVChIntensityConv(WPARAM code)
{
  param->intensityConversion = PTIntensityConversion->GetCheck() == BF_CHECKED;
}

void TWCriticalBandManager::EVChPreemphasis(WPARAM code)
{
   param->isPreac = PTPreemphasis->GetCheck() == BF_CHECKED;
}

void TWCriticalBandManager::EVChCompletionStrategy(WPARAM code)
{
  param->completionStrategy = PTCompletionStrategyASR->GetCheck() == BF_CHECKED;
}

void TWCriticalBandManager::EVChScale(WPARAM code)
{
  if (PTFrequencyScaleMel->GetCheck() == BF_CHECKED) param->frequencyScale = Mel;
  else if (PTFrequencyScaleBark->GetCheck() == BF_CHECKED) param->frequencyScale = Bark;
  else param->frequencyScale = Mely;
}

void TWCriticalBandManager::EVChBeforeBandIntegration(WPARAM code)
{
  param->withCepstralSmoothing = PTCepsSmoothBeforeBand->GetCheck() == BF_CHECKED;
}

#define INVERSE(X) oldStyle = X->GetStyle(); X->TWindow::SetWindowLong(GWL_STYLE, oldStyle ^= (WS_DISABLED));
void TWCriticalBandManager::disableCepstral()
{
  uint32 oldStyle;
  INVERSE(PTTextCepsSmoothBefore);
  INVERSE(PTCepsSmoothBeforeBand);
  INVERSE(PTNoCepsSmoothBeforeBand);
//  if (param->withCepstralSmoothing) PTCepsSmoothBeforeBand->SetCheck(BF_CHECKED);
//  else PTNoCepsSmoothBeforeBand->SetCheck(BF_CHECKED);
  INVERSE(PTTextLiftering);
  INVERSE(PTLifteringOrder);
  INVERSE(PTgroupCepstral);
  INVERSE(PTgroupcepsSmoothBefore);
  ::InvalidateRect(HWindow,NULL,TRUE);
  UpdateWindow();
}

void TWCriticalBandManager::disablePLP()
{
  uint32 oldStyle;
  INVERSE(PTgroupPLP);
  INVERSE(PTTextPredictionOrder);
  INVERSE(PTTextPredictionOrderVal);
  INVERSE(PTPredictionOrder);
  INVERSE(PTIntensityConversion);
  ::InvalidateRect(HWindow,NULL,TRUE);
  UpdateWindow();
}


//////////////////////////////////////////////////
// commandes de cration de la fentre de paramtres
//

void Wcoupe::IDparaCriticalBands(WPARAM) //RTMessage)
{
  if (pCriticalBands->isOpen) return;
  GetModule()->MakeWindow(
		new TWCriticalBandManager(this, "WCriticalBands", pCriticalBands));
}


