/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier partruenv.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "snoori.h"
#include "signal.h"
#include "outils.h"
#include "wcoupe.h"

// variables modifies par la fentre
//   (selon le paramtre d'appel p)
//   p->fen_signal
//   p->fen_fft
//   p->liftrage[0]
//   p->liftrage[1]
//   p->alpha
//   p->nbIter
//   p->niv
//   p->isPreac

static choix choix_fen_fft[] = { {256,"256"},{512,"512"}
			  ,{0,""} };

class TWparaTrueEnv : public TWparaCoupe
{   paramCoupeTrueEnv *param;

  public:
	 TWparaTrueEnv(PTWindowsObject AParent,char *titre
		  ,float fen_signal_min,float fen_signal_max
		  ,paramCoupeTrueEnv *p);
	 // rponses aux vnements
	 LRESULT controle_fen_fft(WPARAM,LPARAM);
	 LRESULT controle_fen_signal(WPARAM,LPARAM);
	 LRESULT controle_debut_liftrage(WPARAM,LPARAM);
	 LRESULT controle_fin_liftrage(WPARAM,LPARAM);
	 LRESULT controle_alpha(WPARAM,LPARAM);
	 LRESULT controle_nbIter(WPARAM,LPARAM);
	 DECLARE_RESPONSE_TABLE(TWparaTrueEnv);
};
DEFINE_RESPONSE_TABLE1(TWparaTrueEnv,TWparaCoupe)
  EV_MESSAGE(WM_USER+2,controle_fen_fft),
  EV_MESSAGE(WM_USER+3,controle_fen_signal),
  EV_MESSAGE(WM_USER+4,controle_alpha),
  EV_MESSAGE(WM_USER+4,controle_nbIter),
  EV_MESSAGE(WM_USER+4,controle_debut_liftrage),
  EV_MESSAGE(WM_USER+5,controle_fin_liftrage),
END_RESPONSE_TABLE;

TWparaTrueEnv::TWparaTrueEnv(PTWindowsObject AParent
						,char *titre
		  ,float fen_signal_min,float fen_signal_max
		  ,paramCoupeTrueEnv *p)
	:TWparaCoupe(AParent,titre,p)
{
  param=p;
  // initialiser le contenu des contrles
  //   avec les variables correspondantes
  // fenetre signal
  new TWfloat(this,3,xx,yy,ww
			,&param->fen_signal
			,ml(6002,"fenetre_signal=")
			,fen_signal_min,fen_signal_max
			,fen_signal_min,fen_signal_max
			,ticket);                 yy+=HNUM;
  // ordre fft pour le cepstre
  myTStatic(ml(6012,"fft de calcul de l'enveloppe vraie"));
  new TWint(this,2,xx,yy,ww
			,&param->fen_fft
			,ml(6013,"log2(fenetre_fft)="),8,11,8,11
			,ticket);                 yy+=HNUM;
  // liftrage
  myTStatic(ml(6005,"liftrage"));
  new TWfloat(this,4,xx,yy,ww
	      ,&param->debut_liftrage
	      ,ml(6006,"dbut (en sec)=")
	      ,0.,5.,0.,5.
	      ,ticket);                 yy+=HNUM;
  new TWfloat(this,5,xx,yy,ww
	      ,&param->fin_liftrage
	      ,ml(6007,"fin (en sec)=")
	      ,0.,10.,0.,10.
	      ,ticket);                 yy+=HNUM;
  // alpha
  myTStatic(ml(6014,"Alpha (convergence)"));
  new TWfloat(this,5,xx,yy,ww
	      ,&param->alpha
	      ,ml(6015,"Alpha=")
	      ,0.,10.,0.,10.
	      ,ticket);                 yy+=HNUM;
  // nbIter
  myTStatic(ml(6016,"Nombre d'itrations"));
  new TWint(this,0,xx,yy,ww
	      ,&param->nbIter
	      ,ml(6017,"Nb. itrations="),1,20,1,20
	      ,ticket);                 yy+=HNUM;
  FinirCreate();
}

LRESULT TWparaTrueEnv::controle_fen_signal(WPARAM,LPARAM)
{
 if ((1 << param->fen_fft) <enEchtf(param->fen_signal))
  { param->fen_signal=enMillif(1 << param->fen_fft);
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  return 0;
}

LRESULT TWparaTrueEnv::controle_fen_fft(WPARAM,LPARAM)
{ if ((1 << param->fen_fft) <enEchtf(param->fen_signal))
  { while ((1 << param->fen_fft) <enEchtf(param->fen_signal))
		param->fen_fft++;
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  return 0;
}

LRESULT TWparaTrueEnv::controle_debut_liftrage(WPARAM,LPARAM)
{ param->debut_liftrage=max(0
								,min(param->debut_liftrage
				 ,param->fin_liftrage));
  ::InvalidateRect(HWindow,NULL,TRUE);
  return 0;
}

LRESULT TWparaTrueEnv::controle_fin_liftrage(WPARAM,LPARAM)
{ param->fin_liftrage=   max(param->debut_liftrage
									 ,param->fin_liftrage);
  ::InvalidateRect(HWindow,NULL,TRUE);
  return 0;
}

LRESULT TWparaTrueEnv::controle_alpha(WPARAM,LPARAM)
{
  ::InvalidateRect(HWindow,NULL,TRUE);
  return 0;
}

LRESULT TWparaTrueEnv::controle_nbIter(WPARAM,LPARAM)
{
  ::InvalidateRect(HWindow,NULL,TRUE);
  return 0;
}

void Wcoupe::IDparaTrueEnv(WPARAM) //RTMessage)
{ if (pTrueEnv->isOpen) return;
  GetModule()->MakeWindow(
		new TWparaTrueEnv(this
			,ml(6018,"paramtres True enveloppe")
				 ,0.5,64.,pTrueEnv));
}

