/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier pics.h
//

#ifndef _PICS_H
#define _PICS_H

#define PICS_NB_SPECTRES_MAX 16383
    // limite en particulier les tables pointeur
    //   de spectres
enum TypeMeilleurPic{FrequencePicCepstre, QualitePicCepstre};

// pics de cepstre
#define NB_MAX_EXTRM	100	/* nombre maximum d'extrema */
struct pic_cepstre // 1 pic de cepstre
{ int xi;	/* abscisse du point */
  float frHz,   /* frequence du point en Hz */
        v,	/* amplitude de ce point */
        q,	/* qualite du pic */
        p,	/* perceptivite du pic */
        bw;	/* largeur de bande */
};
class Pics1_Cepstre // pics en un point
{ public:
    long echt;
    int ni;
    int sens;
    struct pic_cepstre **p;

    Pics1_Cepstre();
    ~Pics1_Cepstre();
    void clear();
    int size() { return ni; }
    void resize(int nb);
    pic_cepstre & operator[](int i) { return *p[i]; }
};
class Pics_Cepstre  // tous les pics
{ public:
    int ni;
    Pics1_Cepstre **p;

    Pics_Cepstre();
    ~Pics_Cepstre();
    void clear();
    int size() { return ni; }
    void resize(int nb);
    Pics1_Cepstre & operator[](int i) { return *p[i]; }
};
// racines de LPC
struct racine_lpc { int val;	// frequence en canal
		float lar;	// largeur de bande 
		float fr;	// frequence en Hertz 
		float db;	// energie 
};
class racines1_lpc // racines en un point
{ public:
    long echt;
    int ni; 
    struct racine_lpc **p;
    int order;
    double *ak;   // coefficients of the denominator ak[0] is the gain.
    					// the # of ak is order.
    racines1_lpc();
    ~racines1_lpc();
    void clearRoots();
    void clearAk();
    int size() { return ni; }
    void resize(int nb);
    void resizeOrder(int order);
    racine_lpc & operator[](int i) { return *p[i]; }
};
class racines_lpc  // toutes les racines
{ public:
    int ni; 
    racines1_lpc **p;

    racines_lpc();
    ~racines_lpc();
    void clear();
    int size() { return ni; }
    void resize(int nb);
    racines1_lpc & operator[](int i) { return *p[i]; }
};

// dans le fichier formants.cpp
extern int ex_extr(
	FLOATbuf& x,		// courbe a traiter
	Pics1_Cepstre& ex, 	// courbe representee par
	                        //      ses extrema	
	int debut,		// debut et
	int fin);                //    fin de la recherche 
extern void estime(
	Pics1_Cepstre& resume, // extrema de la courbe
	float min_cep,	        // valeurs extremes de la 
	float max_cep);         //     fft et du cepstre 
extern float les_meilleurs(
	Pics1_Cepstre& resume, // les extrema
	int n,			/* les n meilleurs maxima */
	int nmax,		/* valeur maximale de l'indice du pic */
   TypeMeilleurPic critere);  /* critere de comparaison */
   
// dans le fichier pics_spe.cpp
extern Pics_Cepstre * pics_cepstre;
extern racines_lpc  * les_raclpc;
//
extern bool suivrePicsCepstre(
     SAMPLEbuf& parole,
     int debut, 	// echantillons de debut et
     int fin,	        //          fin de suivi
     int n,		/* nombre de pics a suivre	     */
     int nmax,		/* dernier canal du spectre ou peut
                           se situer un pic 
			   ( <=> frequence maximale )	*/
     int nb_cep0,	// limite inf et sup
     int nb_cep1,       //   du liftrage
     float attenuation,	/* attenuation spectrale (dB) , a lieu
                           si l'attenuation est differente de 0. */
     int nbEchtDeplac,	/* nombre d'echantillons de deplacements
			   entre deux spectres */
     int& nbSpectres, // nombre de spectres calculs
     bool withTrueEnvelope);
extern void freePicsCepstre();
extern void affichePicsCepstre( 
	int nb, 	/* nombre de spectres calcules. */
	int nbEchtDeplac,	/* nombre d'echantillons de deplacements entre deux spectres */
	int  n,		/* nombre de pics a suivre	*/
	int debut,	/* echantillon de debut de suivi */
   float scale,
	d_planche *fenetre);	 /* fenetre d'affichage */
extern void logPicsCepstre();
extern bool suivreRacLpc(
     SAMPLEbuf& parole,
     long debut,	// debut et 
     long fin,   	//     fin de suivi de lpc
     int ordre_lpc,	/* ordre des lpc */
     float largeur,	/* largeur maximale de bande */
     float attenuation,	/* attenuation spectrale (en dB) */
     int nbEchtDeplac,	/* nombre d'echantillons de
                             deplacement */
     int& nbSpectres); // nombre de spectres calculs
extern void freeRacLpc();
extern void affRacLpc(
	long nb, 	/* nombre de spectres calcules. */
	long nbEchtDeplac,	/* nombre d'echantillons de deplacements entre deux spectres */
	float largeur,	/* largeur maximale de bande */
	long debut,	/* echantillon de debut de suivi */
	d_planche *fenetre);	 /* fenetre d'affichage */
extern void logRacLpc();
extern void spectreLPC(
	float spectre_[],
   racines1_lpc &res_lpc,
	SAMPLE *parole,
	long parole_debut,
   int fen_signal,
   int fen_fft,
	int preaccentuation,
	int ordre,
	float rayon);
void spectreFft(
   float spectre[],
	SAMPLEbuf& parole,
   long parole_debut,
	int preaccentuation,
	int ordre,
	int fen_signal);
#endif
