/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier pics_spe.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>

#include "pics.h"
#include "notetiq.h"
#include "signal.h"
#include "fftlpc.h"
#include "spectre.h"
#include "spectro.h"
#include "pitchm.h"
#include "outils.h"
#include "energies.h"
#include "fmaxspec.h"
#include "fft_mayer.h"

//////////////////////////////////////////////////////
// variables et tableaux pour les pics de cepstres
// 
#define DEFAULT_TRUEENV_ORDER 512

static FLOATbuf **t_suivi; // array of cepstrally smoothed spectra
	   // t_suivi is a pointer on an array of pointers of FLOATbuf
	   // The size of each FLOATbuf order / 2
static int nt_suivi; // taille de t_suivi

///////////////////////////////////////////////
// variables et tableaux des pics de cepstre
//
Pics1_Cepstre::Pics1_Cepstre()
{
  ni=0;
  p = (struct pic_cepstre **)NULL;
}

Pics1_Cepstre::~Pics1_Cepstre()
{
  clear();
}
void Pics1_Cepstre::clear()
{
  if (p) {
  	for (int i=0;i<ni;i++) delete p[i];
  	free(p);
  }
  ni=0;

  p = (struct pic_cepstre **)NULL;
}

void Pics1_Cepstre::resize(int nb)
{
  if (nb==0) { clear(); return; }
  if (nb>this->ni) {
    p=(pic_cepstre **) realloc(p,((long)nb)*sizeof(pic_cepstre *));
    for (int i=this->ni;i<nb;i++)
      p[i]=new pic_cepstre();
  }
  if (nb<this->ni) {
    for (int i=this->ni-1;i>=nb;i--)
      delete p[i];
    p=(pic_cepstre **)realloc(p,((long)nb)*sizeof(pic_cepstre *));
  }
  this->ni=nb;
}



Pics_Cepstre::Pics_Cepstre()
{
  ni=0;
  p = (struct Pics1_Cepstre **) NULL;
}
Pics_Cepstre::~Pics_Cepstre()
{
	clear();
}
void Pics_Cepstre::clear()
{
  if (p) {
	  for (int il=0;il<ni;il++) delete p[il];
     free(p);
  }
  ni=0;
  p = (struct Pics1_Cepstre **) NULL;
}
void Pics_Cepstre::resize(int nb)
{
   if (nb==0) { clear(); return; }
   if (nb>this->ni) {
    p=(Pics1_Cepstre **)realloc(p,((long)nb)*sizeof(Pics1_Cepstre *));
    for (int i=this->ni;i<nb;i++)
      p[i]=new Pics1_Cepstre();
  }
  if (nb<this->ni) {
    for (int i=this->ni-1;i>=nb;i--)
      delete p[i];
    p=(Pics1_Cepstre **)realloc(p,((long)nb)*sizeof(Pics1_Cepstre *));
  }
  this->ni=nb;
}

Pics_Cepstre * pics_cepstre=NULL;

///////////////////////////////////////////////
// variables et tableaux des racines de lpc
//
racines1_lpc::racines1_lpc()
{
  ni = 0;
  order = 0;
  p = (struct racine_lpc **) NULL;
  ak = (double *) NULL;
}
racines1_lpc::~racines1_lpc()
{
  clearRoots();
}
void racines1_lpc::clearRoots()
{
  if (p)
  {
  	for (int i=0; i<ni; i++) delete p[i];
   free(p);
  }
  ni = 0;
  p = (struct racine_lpc **) NULL;
}
void racines1_lpc::clearAk()
{
  order = 0;
  if (ak) free(ak);
  ak = (double *) NULL;
}

void racines1_lpc::resizeOrder(int order)
{
  // + 1 for the first coef. 1 - sum (ak * z^(-k))
  ak = (double *) realloc(ak, ((long)order + 1) * sizeof(double));
  this->order = order;
  if (!order) ak = (double *) NULL;
}

void racines1_lpc::resize(int nb)
{
  if (nb == 0) {
    clearRoots();
    return;
  }
  if (nb>this->ni) {
    p = (racine_lpc **) realloc(p, ((long)nb) * sizeof(racine_lpc *));
    for (int i = this->ni; i < nb; i++)
      p[i] = new racine_lpc;
  }
  if (nb < this->ni) {
    for (int i = this->ni-1; i >= nb; i--) delete p[i];
    p = (racine_lpc **)realloc(p, ((long)nb) * sizeof(racine_lpc *));
  }
  this->ni=nb;
}

racines_lpc::racines_lpc()
{
  ni=0;
  p = (struct racines1_lpc **) NULL;
}
racines_lpc::~racines_lpc()
{
  clear();
  if (p) free(p);
  p = (struct racines1_lpc **) NULL;
}
void racines_lpc::clear()
{
  if (p) {
    for (int i=0;i<ni;i++) delete p[i];
    free(p);
  }
  ni=0;
  p = (struct racines1_lpc **) NULL;
}
void racines_lpc::resize(int nb)
{
  if (nb==0) { clear(); return; }
  if (nb>this->ni) {
    p=(racines1_lpc **)realloc(p,((long)nb)*sizeof(racines1_lpc *));
    for (int i=this->ni;i<nb;i++)
      p[i]=new racines1_lpc();
  } 
  if (nb<this->ni) {
    for (int i=this->ni-1;i>=nb;i--)
      delete p[i];
    p=(racines1_lpc **)realloc(p,((long)nb)*sizeof(racines1_lpc *));
  }
  this->ni=nb;
}

racines_lpc * les_raclpc = NULL;

////////////////////////////////////////////////
// allocation des gros buffers de travail
//     pour t_suivi
//   allocbuf(nb) rserve un buffer de nb lments
//   freebuf() les libre
//
// utilisation:
//     pour l'appelant:
//       - l'allocation des buffers est automatique.
//         L'appelant n'a donc pas  s'en occuper.
//       - la libration des buffers est obligatoire, et
//          la charge de l'appelant. Elle se fait
//         par freePicsCepstre() ou freeRacLpc() qui sont
//         quivalents  freebuf()
static bool indalloc = false; // ==TRUE  buffers rservs
static void freebuf()
{
  //if (!indalloc) return;
  indalloc = false;

  if (t_suivi) {
	  for (int i = 0; i < nt_suivi; i++) delete t_suivi[i];
	  free(t_suivi);
     t_suivi=NULL;
  }
  if (pics_cepstre) delete pics_cepstre;
  pics_cepstre = NULL;

  if (les_raclpc) delete les_raclpc;
  les_raclpc = NULL;
}

static void allocbuf(
  int nb,
  int vectorSize)
{
  freebuf();

  nt_suivi = nb;
  indalloc = true;

  pics_cepstre = new Pics_Cepstre();
  les_raclpc   = new racines_lpc() ;

  // pour suivrePicsCepstre() et suivreRacLpc()
  t_suivi=(FLOATbuf**)malloc(((long)nb)*sizeof(FLOATbuf *) );
  for (int i = 0; i < nb; i++)
    t_suivi[i] =(FLOATbuf *) new FLOATbuf(vectorSize + 1, "t_suivi");
}

//////////////////////////////////////////////////
// fonction  appeler si on veut dsallouer des formants
void pics_spe____done_free_buffers()
{
	freebuf();
}


//////////////////////////////////////////////////
// sous-programmes communs
//

// Calcul du spectre d'attenuation
static void spectreAttenuation(
	float attenuation, float spectre[])
{
  int i2900 = (int) freqToEcr(2900.),
      i3200 = (int) freqToEcr(3200.),
      i;

  /* Calcul du spectre a ajouter */
  for (i = 0; i < i2900; i++) spectre[i] = 0.;
  for (i = i2900; i <= i3200; i++)
    spectre[i] = attenuation * ((float) i - i2900) / ((float) i3200 - i2900);
  for (i = i3200; i < NB_CANAUX; i++)
    spectre[i] = attenuation;
}


/*
 * Attenuation de la zone correspondant a F4. Par precaution
 * on attenue aussi la
 * partie du spectrogramme au-dela de F4. L'attenuation
 * a la forme suivante:
 *	\
 *	 \
 *	  \
 *	   \__________________
 *   2900 3200		    8000
 */
static void attenuationF4Spectro(
     float attenuation,
     FLOATbuf **spectro,
     int nbSpectres)
{
  int i2900 = (int) freqToEcr(2900.);
  float spectreAttenue[NB_CANAUX], *pspectro;

  spectreAttenuation(attenuation, spectreAttenue);
  for (int i = 0; i < nbSpectres; i++) {
    pspectro=(float *)&(*spectro[i])[0];
    for (int j = i2900; j < NB_CANAUX; j++)
      pspectro[j] -= spectreAttenue[j];
  }
}

//#define COEF_1_CEP   0		/* first frequency sample in the cepstrally smoothed spectrum to consider */
//#define COEF_DER_CEP 127   /* last frequency sample in the cepstrally smoothed spectrum to consider */

////////////////////////////////////////////////////////
// fonction principale de dtection de pics de cepstre
//
// Cette fonction permet de suivre les pics des fft
//   de cepstres sur un domaine. Ce domaine est limit.
//
bool suivrePicsCepstre(
     SAMPLEbuf& parole,
     int debut, 	// echantillons de debut et
     int fin_,	        //          fin de suivi
     int n,		/* nombre de pics a suivre	     */
     int nmax,		/* dernier canal du spectre ou peut
                           se situer un pic
			   ( <=> frequence maximale )	*/
     int nb_cep0,	// limite inf et sup
     int nb_cep1,       //   du liftrage
     float attenuation,	/* attenuation spectrale (dB) , a lieu
                           si l'attenuation est differente de 0. */
     int nbEchtDeplac,	/* nombre d'echantillons de deplacements
			   entre deux spectres */
     int& nbSpectres,  // en retour, nombre de spectres calculs
     bool withTrueEnvelope)
// en retour,
//   TRUE  normal
//   FALSE  calculs interrompus au clavier
// si au dpart nbSpectres==1
//   on calcule 1 spectre et 1 seul
//   et alors fin_ est inutilis
{
  int i, status, nb,
      order,
      windowSize,
      deb,fin;
  bool ret;

  deb = _i(debut);

  if (withTrueEnvelope)
    order = windowSize = DEFAULT_TRUEENV_ORDER;
  else order = windowSize = 512;           // should be changed it is a very very old historical residue

  if (nbSpectres == 1) {
    fin = debut + windowSize;
    nb = 1;
  }
  else {
    fin = min(fin_, debut + (NB_CEPSTRE - 1) * nbEchtDeplac + windowSize - 1); /* pour ne pas deborder */
//    nb = (int)min(PICS_NB_SPECTRES_MAX, (fin - debut + 1 - windowSize) / nbEchtDeplac + 1);
    nb = (int)min(PICS_NB_SPECTRES_MAX, (fin - debut + 1) / nbEchtDeplac + 1);
  }
  while ((debut+ (nb-1) * nbEchtDeplac + windowSize) >= ct.f_e) nb--;
  if (nb<=0) {
    nbSpectres=0;
    return TRUE;
  }
  allocbuf(nb, order / 2);

  if (!withTrueEnvelope) {
    nb = cepstre512(t_suivi, &parole[0], deb, nb, nbEchtDeplac, nb_cep0, nb_cep1, 1, VRAI);
    order = 512;
  }
  else {
    order = DEFAULT_TRUEENV_ORDER;
    for (int l = 0; l < nb; l++) {
      float spectre[DEFAULT_TRUEENV_ORDER / 2];
      halle(spectre, &parole[deb], l * nbEchtDeplac, DEFAULT_TRUEENV_ORDER, DEFAULT_TRUEENV_ORDER,
           nb_cep0, nb_cep1, 0., 6, true);
               /* tranfert des resultats */
      for (int j = 0; j < order / 2; j++) (*t_suivi[l])[j] = spectre[j];
    }
  }

  if (fabs(attenuation) > EPSILON) 
    attenuationF4Spectro(attenuation, t_suivi, nb);

  pics_cepstre->resize(nb);
//  if (nb!=1) Wact->showpourcent(ml(6301,"peaks"));
//  GetAsyncKeyState(VK_ESCAPE);
  for (i = 0, status = true; status && (i<nb)&&((ret=((GetAsyncKeyState(VK_ESCAPE)&1)==0))==TRUE); i++) {
    float min_cep, max_cep;

//    if (nb!=1) Wact->pourcent(i,nb);
    (*pics_cepstre)[i].echt = i * nbEchtDeplac + debut + windowSize / 2;
    /* attention modifier mettre un ()?: pour coefAttenue */
    status = ex_extr(*t_suivi[i], (*pics_cepstre)[i], 0, order / 2 -1);
    /* recuperation du min et du max de la fft lissee
       correspondant a ce cepstre */
    min_cep = min_f(*t_suivi[i], 0, order / 2 -1);
    max_cep = max_f(*t_suivi[i], 0, order / 2 -1);
    estime((*pics_cepstre)[i] ,min_cep, max_cep);
    les_meilleurs( (*pics_cepstre)[i], n, nmax, FrequencePicCepstre);
  }
  if (!status) affiche_bug("suivrePicsCepstre() : too many extrema because liftering coefficients are not appropriate");
//  if (nb!=1) Wact->pourcent(1,1);
  pics_cepstre->resize(i);
  nbSpectres=i;
  return ret;
}
void freePicsCepstre()
{
  freebuf();
}

/*
 * Affichage des pics de spectres lisses cepstralement.
 */
void affichePicsCepstre( 
	int nb, 	/* nombre de spectres calcules. */
	int nbEchtDeplac,	/* nombre d'echantillons de deplacements entre deux spectres */
	int  n,		/* nombre de pics a suivre	*/
	int , //debut,	/* echantillon de debut de suivi */
   float scale,
	d_planche *fenetre)	 /* fenetre d'affichage */
{  
  for (int i = 0; i < nb; i++) {
    float e,f;
    for (int j = 0; j < min(n, (*pics_cepstre)[i].size()); j++) {
      // on n'affiche pas les pics trs faibles
        MgiHue( ((*pics_cepstre)[i][j].q > 0.5) ? ROUGE : NOIR, fenetre);
        e = (*pics_cepstre)[i].echt;
        f = (*pics_cepstre)[i][j].xi / 3. / fMaxSpectro * en_cours.frequence / 2. * scale;
        snorbox(e - nbEchtDeplac/2, f - 0.5, e + nbEchtDeplac/2, f + 0.5, fenetre);
      }
  }
}

void logPicsCepstre()
{
  int i,j;
  char ch[1000], ch1[1000], ch2[1000], ch3[1000];
  long e;
  pic_cepstre *pex;
  int etiq, note;

  if (!pics_cepstre) return;
  logs.open1();
  logs.wr1(ml(6303,">> dtection de pics de cepstre"));
  sprintf(ch,ml(6304,"file : %s"),en_cours.nom);
  logs.wr1(ch);
  logs.wr1(ml(6305,"Temps\tTemps\tF0\tener\tPhonme\tMot\tFreq.\tBande\tAmpl.\tPercep ..."));
  logs.wr1(ml(6306,"\t\t\t\\t\t\t\tLargeur\t\t-tivit"));
  logs.wr1(ml(6307,"[ms]\t[echt]\t[F0]\t[dB]\t[phon]\t[mot]\t[Hz]\t[Hz]\t[dB]\t[dB] ..."));
  initCalculEnergie();
  for (i = 0; i < pics_cepstre->size(); i++) {
    int f0, datems;
    float ener;
    e = (*pics_cepstre)[i].echt;
    etiq = etiqs.search(e);
	 note = notes.search(e);
    ener = enerBandeFminFmaxCentre(e);
    f0 = max(pm.getf0(e), pm.getReadf0(e));
    datems = enMilli(e);
    sprintf(ch1,"%ld\t%ld\t%d\t%4.0f\t%s\t%s",
		  datems, e,
		  f0,
        ener,
        etiq >= 0 ? etiqs.Text(etiq) : "",
        note >= 0 ? notes.Text(note) : "");
    sprintf(ch2, "");
    for (j = 0; j < (*pics_cepstre)[i].size(); j++) if ( (*pics_cepstre)[i][j].q > 0.5) {
      pex = &((*pics_cepstre)[i][j]);
      sprintf(ch3,"\t%u\t%u\t%d\t%d",
   		     (unsigned int)pex->frHz,
		        (unsigned int)pex->bw,
		        (int)pex->v,
              (int)pex->p);
      strcat(ch2, ch3);
    }
    sprintf(ch,"%s %s", ch1, ch2);
    logs.wr1(ch);
  }
  logs.close1();
  freeSpecFft();
}

//////////////////////////////////////////////////////
// fonction principale de dtection de racines de lpc
//
#define FFTORDER_LPC			256
#define WINDOW_LPC_MS 		16
  // sert  la fois de fentre signal et fentre fft
//
// Cette fonction suit les solutions d'une lpc.
//
bool suivreRacLpc(
     SAMPLEbuf& parole,
	  long debut,	   // debut et
     long fin_,   	//     fin de suivi de lpc
     int ordre_lpc,	/* ordre des lpc */
     float largeur,	/* largeur maximale de bande */
     float attenuation,	/* attenuation spectrale (en dB) */
     int nbEchtDeplac,	/* nombre d'echantillons de
                             deplacement */
     int& nbSpectres) // nombre de spectres calculs
// en retour,
//   TRUE  normal
//   FALSE  calculs interrompus au clavier
// si au dpart nbSpectres==1
//   on calcule 1 spectre et 1 seul
//   et alors fin_ est inutilis
{
  int deb, fin, nb, i,
      windowSize = enEcht(WINDOW_LPC_MS);
  bool ret;
  float spectreAttenue[NB_CANAUX],
        *r = new float[ordre_lpc + 1],
	     *a = new float[ordre_lpc + 1],
	     *sf= new float[ordre_lpc + 1],
        *xr = new float[windowSize],
        gain,
        *coef = new float[ordre_lpc + 1], // coefficients du denominateur de la fct de trandfert
        *fpos = new float[ordre_lpc + 1], // nombre, puis frequences des formants
        *fbw = new float[ordre_lpc + 1];  // nombre, puis largeur de bande des formants
  double *realfft = new double[2 * FFTORDER_LPC],
         *imagfft = new double[2 * FFTORDER_LPC];
  int nbform;

  deb = _i(debut);

  if (nbSpectres==1) {
    fin = debut + windowSize;
    nb = 1;
  }
  else {
    fin = min(fin_, debut + (NB_LPC-1) * nbEchtDeplac + windowSize - 1);	/* pour ne pas deborder */
//    nb = (int)min(PICS_NB_SPECTRES_MAX,(fin - debut + 1- windowSize) / nbEchtDeplac+1);
    nb = (int)min(PICS_NB_SPECTRES_MAX,(fin - debut + 1) / nbEchtDeplac+1);
  }
  while ((debut + (nb-1) * nbEchtDeplac + windowSize) >= ct.f_e) nb--;
  if (nb<=0) {
    nbSpectres=0;
    return TRUE;
  }
  allocbuf(nb, NB_CANAUX);

  spectreAttenuation(attenuation, spectreAttenue);

  initham_((float *) NULL, windowSize);

  les_raclpc->resize(nb);
//  if (nb!=1) Wact->showpourcent("");
//  GetAsyncKeyState(VK_ESCAPE);
//  for (i = 0; (i<nb)&&((ret=((GetAsyncKeyState(VK_ESCAPE)&1)==0))==TRUE); i++) {
  for (i = 0; i < nb; i++) {
//    if (nb!=1) Wact->pourcent(i,nb);

    (*les_raclpc)[i].echt=i * nbEchtDeplac + debut + windowSize / 2;
    preac_f_(&parole[0], deb + i * nbEchtDeplac, xr, windowSize);
    ham_ff_(xr, xr, (float *) NULL, windowSize);

    autolpc (xr, r, a, sf, &gain, windowSize, ordre_lpc);

    realfft[0] = 1.0;
    imagfft[0] = 0.0;
    coef[0] = 1.;
    for (int j = 1;  j <= ordre_lpc; j++) {
      realfft[j] = -a[j];
      imagfft[j] = 0;
      coef[j] = -a[j];
    }
    for (int j = ordre_lpc + 1; j < 256; j++) realfft[j] = imagfft[j] = 0.;

    fband(ordre_lpc, coef,(float) en_cours.frequence,largeur,0.0,(float) (en_cours.frequence/2.0), fpos, fbw);

    fft(256, realfft, imagfft);

    if (gain == 0.0) gain = EPSILON;
    gain = (float) log10((double)gain);

   for(int j = 0; j < NB_CANAUX; j++)
     (*t_suivi[i])[j] = 10 * (gain - log10(realfft[j] * realfft[j] + imagfft[j] * imagfft[j] + EPSILON)) - spectreAttenue[j];

    nbform = (int) fpos[0];
    (*les_raclpc)[i].resize(nbform);

    for (int j = 1 ; j <= nbform ; j++) {
      int canal = IROUND(fpos[j] / en_cours.frequence * 2. * NB_CANAUX);
      float energy = (*t_suivi[i])[canal];
      (*les_raclpc)[i][j-1].fr = fpos[j];
      (*les_raclpc)[i][j-1].lar = fbw[j];
      (*les_raclpc)[i][j-1].db = max(energy, 0.);
    }
  }
  delete [] r;
  delete [] a;
  delete [] sf;
  delete [] xr;
  delete [] coef;
  delete [] fpos;
  delete [] fbw;
  delete [] realfft;
  delete [] imagfft;

//  if (nb != 1) Wact->pourcent(1,1);
  les_raclpc->resize(i);
  nbSpectres = i;

  return ret;     
}

void freeRacLpc()
{
  freebuf();
}

/*
 * Affichage des racines de LPC.
 */
void affRacLpc(
	long nb, 	/* nombre de spectres calcules. */
	long nbEchtDeplac,	/* nombre d'echantillons de deplacements entre deux spectres */
	float largeur,	/* largeur maximale de bande */
	long , //debut,	/* echantillon de debut de suivi */
	d_planche *fenetre)	 /* fenetre d'affichage */
{
  int i;

  snoiwidth(1);
  for (i = 0; i < nb; i++) {
    int j;
    float dy;
    long e;
    for (j = 0 ; j < (*les_raclpc)[i].size() ; j++) {
     if (  ((*les_raclpc)[i][j].lar < largeur)
	      &&((*les_raclpc)[i][j].fr < 5000.) ) {
       float y = IROUND((*les_raclpc)[i][j].fr
						/ fMaxSpectro * NB_CANAUX);
		 double bw = min(max((*les_raclpc)[i][j].lar
	                    , 10.), 100.),
              energyLevel = (*les_raclpc)[i][j].db;
       y = min(NB_CANAUX, y);
		 if (energyLevel + 10. * log10(bw / 100.) < SILENCE_SPECTRO)
	  	 MgiHue(NOIR, fenetre);
		 else MgiHue(MAGENTA, fenetre);
		 e = (*les_raclpc)[i].echt;
		 snorbox(e-nbEchtDeplac/2, y - 0.5,
      		  e+nbEchtDeplac/2, y + 0.5, fenetre);
		 dy = freqToEcr((*les_raclpc)[i][j].lar);
		 snortl(e , y - dy / 2., e , y + dy / 2., fenetre);
     }
    }
  }
}
void logRacLpc()
{
  int i,j;
  char ch[1000], ch1[1000], ch2[1000], ch3[1000];
  long e;
  racine_lpc *p;
  int etiq, note;

  if (!les_raclpc) return;
  logs.open1();
  logs.wr1(ml(6310,">> dtection de racines de LPC"));
  sprintf(ch,ml(6304,"file : %s"),en_cours.nom);
  logs.wr1(ch);
  logs.wr1(ml(6311,"Temps\tTemps\tF0\tener\tPhonme\tMot\tFreq.\tBande\tAmpl."));
  logs.wr1(ml(6312,"\t\t\t\t\t\t\tLargeur"));
  logs.wr1(ml(6313,"[ms]\t[echt]\t[F0]\t[dB]\t[phon]\t[mot]\t[Hz]\t[Hz]\t[dB]"));
  initCalculEnergie();
  for (i=0;i<les_raclpc->size();i++) {
    int f0, datems;
    float ener;
    e = (*les_raclpc)[i].echt;
    etiq = etiqs.search(e);
	 note = notes.search(e);
    ener = enerBandeFminFmaxCentre(e);
    f0 = max(pm.getf0(e), pm.getReadf0(e));
    datems = enMilli(e);
    sprintf(ch1,"%ld\t%ld\t%d\t%4.0f\t%s\t%s",
		  datems, e,
		  f0,
        ener,
        etiq >= 0 ? etiqs.Text(etiq) : "",
        note >= 0 ? notes.Text(note) : "");
    sprintf(ch2, "");
    for (j = 0; j < (*les_raclpc)[i].size(); j++) {
	   p = &(*les_raclpc)[i][j];
      sprintf(ch3,"\t%u\t%u\t%d",
   		     (unsigned int)p->fr,
		        (unsigned int)p->lar,
		        (int)p->db);
      strcat(ch2, ch3);
	 }
    sprintf(ch,"%s %s ", ch1, ch2);
    logs.wr1(ch);
  }
  logs.close1();
  freeSpecFft();
}


