/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include "common.h"

#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>
                                                                                                             
#include "signal.h"                                                                                                      
#include "outils.h"
#include "spectro.h"
#include "piquef0.h"
#include "pitchm.h"
#include "hanning.h"
#include "files.h"
#include "pitchmk.h"
#include "audio.h"
#include "fmaxspec.h"
#include "fftlpc.h"
#include "fft_mayer.h"
#include "notetiq.h"
#include "revoicingParam.h"
#include "add_files/htmlhelp.h"

WPiqueF0 *wpiquef0;
bool yaUneFenetrePiqueF0 = false;
static POINTF0buf lesPointsF0(1, "Pitch values read from file");

static POINT posF0;

//bool yaUneCourbeDePitch = FAUX;
//bool yaF0Changed = FAUX;
bool yaF0Selected = false;

extern void InvertPointSel(HDC hdc, int x, int y);


// --------------------

/* --------------------------------------------------- */
/* Code added/adapted by Parham...(24 Aug - 2 Sep '98) */
// ...Implementation of Pitch-modification options
//    under main menu: Prosody--Pitch...
PolyPointsF0 *ppointF0Modif;	// Information pertaining to PolyPointsF0
                              // during modifications.
int indiceF0PtModif, F0PtModif;
int xF0PtModif, yF0PtModif;	// Coordonnees du point
                                //    deplacer.

// Creation of a new PolyPointsF0 (an entire contour).
PolyPointsF0::PolyPointsF0(
	 POINTF0buf &pointF0,
	 int nbPt)
{
  PiqF0Point *pts = new PiqF0Point[nbPt];

  if (pts) {
    int i;
    this->selec = FAUX;
    this->nbPt = nbPt;
    this->pts = pts;
    for (i = 0; i < nbPt; i++) {
      this->pts[i].date = pointF0[i].date;
      this->pts[i].freq = pointF0[i].freq;
    }
    this->reste = (PolyPointsF0 *) NULL;
  }
  else {
    affiche_erreur(ml(6401,"PolyPointsF0 : no space left for this PolyPointF0"));
    this->pts = (PiqF0Point *) NULL;
    this->nbPt = 0;
  }
}

// Desctruction of a PolyPointsF0 (an entire contour).
PolyPointsF0::~PolyPointsF0()
{
  if (this->nbPt > 0)  delete [] this->pts;
  this->nbPt = 0;
}

/*
 * Trouve le PolyPointF0 auquel appartient un point,
 * et en cas de succes le point pointe par la souris
 * si elle est suffisamment proche d'un point de la courbe.
 */
// en retour
//     p==0  pas de polypoint trouv
//     p!=0  polypoint trouv
//               p donne le polypoint
//               F0 donne le F0 (0, 1)
//               indice donne le point
//                   indice==-1   pas de point trouv

//
// Retourne le polyPoint auquel un point appartient.
//
static PolyPointsF0 *trouvePtF0(
   PiqF0Point UnPoint,	 // point que l'on cherche.
   bool OnlySelect, // FALSE tous, TRUE seulement les slectionns
	int *indice,
   int *F0)
{
  PolyPointsF0 *p, *bingo;
  float domyMax = fMaxSpectro;
  float correc = 0.1;
  float seuil =  5. * ((float) enMilli(ct.f_a - ct.d_a + 1))
                    / ((float)wspectro->wsona->largeur)
	       + domyMax
	         / ((float) wspectro->wsona->hauteur) * correc;
  for (p = f0contour[0], bingo = (PolyPointsF0 *) NULL;
       !bingo && p; p = p->reste)
  if (!OnlySelect || p->selec) { int i, imin;
    float distCourbe, distPoint; // On conserve la
               //  distance minmale a la courbe et la
	       //  distance minimale aux points de la courbe.

    for (i = 0, distCourbe = distPoint= INFINI_F / 2.; i < p->nbPt; i++) {
      int dist = abs(enMilli(UnPoint.date) - enMilli(p->pts[i].date));
      if (dist < distCourbe) {
        distCourbe = dist;
        if (dist < distPoint) {
          distPoint = dist;
          imin = i;
        }
      }
   }

    if (distCourbe < INFINI_F / 2.) {
      bingo = p;
      *indice = imin;
    }
    else*indice = -1;
  }
  return(bingo);
}

static PolyPointsF0 *trouveF0Pt(
	PiqF0Point UnPoint,	 // point que l'on cherche.
	int *indice, int *F0)
{
  PolyPointsF0 *bingo;

  bingo = trouvePtF0(UnPoint, FALSE, indice, F0);
  return(bingo);
}




//
// Retourne le polyPoint slectionn auquel un point appartient.
//
// trs similaire  trouvePt()
static PolyPointsF0 *trouveF0PtSel(
	PiqF0Point UnPoint,	 // point que l'on cherche.
	int *indice, int *F0)
{
  PolyPointsF0 *bingo;

  bingo = trouvePtF0(UnPoint, TRUE, indice, F0);
  return(bingo);
}

// Display of an entire PolyPointsF0,
// superimposed on the spectrogram.
static void affUnPPointF0(
	 PolyPointsF0 *pp,
    COLORREF couleur)
{
 if (   dans(pp->pts[0].date, ct.d_a, ct.f_a)
      || dans(pp->pts[pp->nbPt-1].date, ct.d_a, ct.f_a)
      || dans(ct.d_a, pp->pts[0].date, pp->pts[pp->nbPt-1].date)
      || dans(ct.f_a, pp->pts[0].date, pp->pts[pp->nbPt-1].date)) {
   MgiHue(couleur, wspectro->wsona);
   if (pp->selec) {
     int i, x, y, xold, yold;

     xold = wspectro->wsona->LXtoDX((float) pp->pts[0].date);
     yold = wspectro->wsona->LYtoDY(FREQ2ECRF0(pp->pts[0].freq));

     SetROP2(wspectro->wsona->hDCcomp,R2_NOT);
     InvertPointSel(wspectro->wsona->hDCcomp,xold,yold);

     for (i = 1; i < pp->nbPt; i++) {
       x = wspectro->wsona->LXtoDX((float) pp->pts[i].date);
       y = wspectro->wsona->LYtoDY(FREQ2ECRF0(pp->pts[i].freq));
       InvertPointSel(wspectro->wsona->hDCcomp,x,y);
       wspectro->wsona->Dline2(xold,yold,x,y);
       xold = x;
       yold = y;
     }
     wspectro->wsona->SetPenWidth(1);
     SetROP2(wspectro->wsona->hDCcomp,R2_COPYPEN);
   }
   else {
  	  int i;
     wspectro->wsona->SetPenWidth(3);
     pm.fenPitch = (int) (en_cours.frequence * (float) FENPITCH / 1000.);
     pm.depEcht = en_cours.frequence * (DEPPITCH/1000.);

     float xcote = wspectro->wsona->DXtoLX(3) - wspectro->wsona->DXtoLX(0);
     float ycote = wspectro->wsona->DYtoLY(0) - wspectro->wsona->DYtoLY(3);

     for (i=0; i < pp->nbPt; i++)
       if ((pp->pts[i].freq > 0) && dans(pp->pts[i].date, ct.d_a, ct.f_a)) {
         float x1,y1;
         x1 = pp->pts[i].date - xcote/2.;
         y1 = max(pp->pts[i].freq / pm.F0Maxscale() * NB_CANAUX  - ycote/2., 0.);
         snorbox(x1, y1, x1 + xcote, y1 + ycote, wspectro->wsona);
       }
   }
   InvalidateRect(wspectro->wsona->HWindow, NULL, TRUE);
  }
}

// Display an entire PolyPointsF0 contour,
// superimposed on the spectrogram.
static void affPPointF0Selec()
{
  int i;
  // Other contours are erased.
  wspectro->wsona->spectro_nettoyer(0,INFINI_I);
  affUnPPointF0(f0contour[0], BLEU);
  affUnPPointF0(f0contour[1], ROUGE);
}

// ----------------------------
/*XM extern XM*/ int event_x_g,event_x_d; // utiliss pour stocker les limites
                         // gauche et droite  ne pas dpasser

// Functions responding to SHIFT + left-mouse-button...
// ...i.e., redrawing an F0 contour.
void drawDownPiqF0(int event_x,int event_y)
{
  PiqF0Point p;
  int indice, F0;
  PolyPointsF0 *pp;

  if (yaUnSignal != ASIGNAL) return;
  event_x_g=event_x;
  lesPointsF0.newsize(1);

//  trouveDatF0(event_x,event_y, &p);
  float x,y;
  snoiunscalexy(event_x, event_y, &x, &y, wspectro->wsona);
  p.date = (long)x;
  p.freq = CAN2HZF0(y);
  pp = trouveF0PtSel(p, &indice, &F0);
  lesPointsF0[0].date = f0contour[0]->pts[indice].date;
  if (f0contour[0]->pts[indice].freq != 0.0)
    lesPointsF0[0].freq = p.freq;
  else
    lesPointsF0[0].freq = 0.0;

  // Testing......
/*  FILE *fpt;
  char fname[10];
  sprintf(fname,"test2.txt");
  fpt = fopen(fname,"w");
  fprintf(fpt," drawDownPiqF0 :    event_x= %d    event_y= %d\n",event_x,event_y);
  fprintf(fpt,"                         x = %f        y = %5.1f\n",x,y);
  fprintf(fpt,"                    p.date = %ld   p.freq = %5.1f\n",p.date,p.freq);
  fprintf(fpt,"                        pp = %d\n",pp);
  fprintf(fpt,"          pointsF0[0].date = %ld    (indice = %d)\n",pointsF0[0].date,indice);
  fprintf(fpt,"          pointsF0[0].freq = %f     (F0 = %d)\n",pointsF0[0].freq,F0);
  fprintf(fpt,"   f0contour[0]->pts[indice].date = %ld    ...freq = %f\n",f0contour[0]->pts[indice].date,f0contour[0]->pts[indice].freq);
  fclose(fpt);
 */
  wspectro->wsona->SetPenWidth(3);
}

//
// Record a new F0 contour while preserving the nature (voiced or unvoiced) of
// each point.
//
void drawMotionPiqF0(int event_x,int event_y)
{ int x1, y1, x2, y2;
  HDC hDC;
  LOGPEN MyPen;
  HPEN hMyPen,holdpen;

  if (yaUnSignal != ASIGNAL) return;
  if (event_x<=event_x_g) return;
  event_x_g=event_x;

  lesPointsF0.newsize(lesPointsF0.size() + 1);

//  trouveDatF0(event_x,event_y, &points[nbPoints]);
  float x,y;
  snoiunscalexy(event_x,event_y,&x,&y,wspectro->wsona);
//  pointsF0[nbPointsF0].date = (long)x;
//  pointsF0[nbPointsF0].freq = CAN2HZF0(y);
  PiqF0Point p;
  int indice, F0;
  PolyPointsF0 *pp;
  p.date = (long)x;
  p.freq = CAN2HZF0(y);
  pp = trouveF0PtSel(p, &indice, &F0);
  if (indice >= 0)
  	lesPointsF0[lesPointsF0.size() - 1].date = f0contour[0]->pts[indice].date;
  else  {
  	affiche_erreur("horreur indice inferieur a 0");
  }

  lesPointsF0[lesPointsF0.size() - 1].freq = p.freq;

  x1 = wspectro->wsona->LXtoDX((float) lesPointsF0[max(0,lesPointsF0.size() - 2)].date);
  y1 = wspectro->wsona->LYtoDY(FREQ2ECRF0(lesPointsF0[max(0,lesPointsF0.size() - 2)].freq));
  x2 = wspectro->wsona->LXtoDX((float) lesPointsF0[lesPointsF0.size() - 1].date);
  y2 = wspectro->wsona->LYtoDY(FREQ2ECRF0(lesPointsF0[lesPointsF0.size() - 1].freq));

  hDC=GetDC(wspectro->wsona->HWindow);
  GetObject(GetStockObject(BLACK_PEN),sizeof(LOGPEN),&MyPen);
  MyPen.lopnWidth.x=3;
  hMyPen=CreatePenIndirect(&MyPen);
  holdpen=(HPEN)SelectObject(hDC,hMyPen);
  MoveToEx(hDC,x1,y1, (POINT *) NULL);
  LineTo(hDC,x2,y2);
  SelectObject(hDC,holdpen);
  DeleteObject(hMyPen);
  ReleaseDC(wspectro->wsona->HWindow,hDC);
}


//
// Update the F0 contour from the knowledge of the new points
// entered by the user.
//
void drawUpPiqF0(int event_x,int event_y)
{
  if (yaUnSignal != ASIGNAL) return;
  event_x=max(event_x_g,event_x);
  posF0.x = event_x;
  posF0.y = event_y;

  int i, x_smpl, x_indx, x_closest;

  x_indx = 0;
  x_smpl = 0.5 * enEcht(FENPITCH);
  while (x_smpl < lesPointsF0[0].date) {
    x_indx++;
    x_smpl = (0.5 * enEcht(FENPITCH)) + (x_indx * enEcht(DEPPITCH));
  }

  do  {
    int dist, mindist, closest_indx;

    closest_indx = 0;
    mindist = 9999;
    for (i=0; i < lesPointsF0.size(); i++) {
      dist = abs(x_smpl - lesPointsF0[i].date);
      if (dist < mindist) {
        mindist = dist;
        closest_indx = i;
      }
    }

    // The F0 correction is made only if the old F0 value was not zero.
    // if (f0contour[0]->pts[x_indx].freq > 0. && lesPointsF0[closest_indx].freq > 0.)
      f0contour[0]->pts[x_indx].freq = lesPointsF0[closest_indx].freq;    

    x_indx++;
    x_smpl = (0.5 * enEcht(FENPITCH)) + (x_indx * enEcht(DEPPITCH));
  } while (x_smpl <= lesPointsF0[lesPointsF0.size()-1].date);

  lesPointsF0.newsize(0);
  affPPointF0Selec();
}
//------------------------------------------

// Functions responding to CTRL + left-mouse-button...
// i.e., modification of a single point along the F0 contour.
/*
 * Enfoncer la souris lorsque l'on veut deplacer un point.
 */
void drawDownPiqF0Pt(int event_x,int event_y)
{
  PiqF0Point p;
  int indice, F0;
  PolyPointsF0 *pp;

//  trouveDatF0(event_x,event_y, &p);
  float x,y;
  snoiunscalexy(event_x, event_y, &x, &y, wspectro->wsona);
  p.date = (long)x;
  p.freq = CAN2HZF0(y);

  pp = trouveF0PtSel(p, &indice, &F0);

  // Testing......
/*  FILE *fpt;
  char fname[10];
  sprintf(fname,"tdtd.txt");
  fpt = fopen(fname,"w");
  fprintf(fpt," drawDownPiqF0Pt :  event_x= %d    event_y= %d\n",event_x,event_y);
  fprintf(fpt,"                         x = %f        y = %5.1f\n",x,y);
  fprintf(fpt,"                    p.date = %ld   p.freq = %5.1f\n",p.date,p.freq);
  fprintf(fpt,"                        pp = %d\n",pp);
  fprintf(fpt,"                    indice = %d      F0 = %d\n",indice,F0);
  fclose(fpt);
 */
  // --------------

  //  if (!pp || f0contour[0]->pts[indice].freq == 0.0)    To allow only voiced points to be updated
  if (!pp)
  { ppointF0Modif = (PolyPointsF0 *) NULL;
  }
  else if (indice > -1)
  { /* c'est le deplacement d'un point d'un PolyPointF0 selectionne. */
    ppointF0Modif = pp;
    indiceF0PtModif = indice;
    F0PtModif = F0;
    xF0PtModif = wspectro->wsona->LXtoDX((float) ppointF0Modif->pts[indice].date);
    yF0PtModif = wspectro->wsona->LYtoDY(FREQ2ECRF0(ppointF0Modif->pts[indice].freq));
    // calcul des limites en x du dplacement acceptable
    event_x_g = 0;
    event_x_d = wspectro->wsona->largeur-1;
    if (indice>0)
      event_x_g = max(event_x_g,wspectro->wsona->LXtoDX(
	  max(ct.d_es
	     ,(float) ppointF0Modif->pts[indice-1].date)));
    if (indice<pp->nbPt-1)
      event_x_d=min(event_x_d, wspectro->wsona->LXtoDX(
	  min(ct.f_es
	     ,(float) ppointF0Modif->pts[indice+1].date)));
  }
}

/*
 * Deplacement d'un point.
 */
void drawMotionPiqF0Pt(int event_x,int event_y)
{ HDC hDC;
  LOGPEN MyPen;
  HPEN hMyPen,holdpen;

  if ( !ppointF0Modif) return;

  // Testing......
/*  FILE *fpt;
  char fname[10];
  sprintf(fname,"tdtd.txt");
  fpt = fopen(fname,"a");
  fprintf(fpt," drawMotionPiqF0Pt : \n");
  fclose(fpt);
 */
  // --------------

  if (  (event_x<=event_x_g)
      ||(event_x>=event_x_d) ) return;

  hDC=GetDC(wspectro->wsona->HWindow);
  GetObject(GetStockObject(BLACK_PEN),sizeof(LOGPEN),&MyPen);
  hMyPen=CreatePenIndirect(&MyPen);
  holdpen=(HPEN)SelectObject(hDC,hMyPen);

  SetROP2(hDC,R2_NOT);

  /* D'abord on efface l'ancien point et les segments qui lui correspondent. */

  InvertPointSel(hDC,xF0PtModif,yF0PtModif);
  if (indiceF0PtModif)
  { MoveToEx(hDC
           ,wspectro->wsona->LXtoDX(ppointF0Modif->pts[indiceF0PtModif - 1].date)
		,wspectro->wsona->LYtoDY(FREQ2ECRF0(ppointF0Modif->pts[indiceF0PtModif - 1].freq)),
		(POINT *) NULL );
    LineTo(hDC, xF0PtModif, yF0PtModif);
  }
  if (indiceF0PtModif < ppointF0Modif->nbPt - 1)
  { MoveToEx(hDC, xF0PtModif, yF0PtModif, (POINT *) NULL);
    LineTo(hDC
           ,wspectro->wsona->LXtoDX(ppointF0Modif->pts[indiceF0PtModif + 1].date)
	   ,wspectro->wsona->LYtoDY(FREQ2ECRF0(ppointF0Modif->pts[indiceF0PtModif + 1].freq)) );
  }

  /* Ensuite on affiche le nouveau point. */
  InvertPointSel(wspectro->wsona->hDCcomp, event_x,event_y);
  if (indiceF0PtModif)
  { MoveToEx(hDC
           ,wspectro->wsona->LXtoDX(ppointF0Modif->pts[indiceF0PtModif - 1].date)
		,wspectro->wsona->LYtoDY(FREQ2ECRF0(ppointF0Modif->pts[indiceF0PtModif - 1].freq)),
		(POINT *) NULL );
	 LineTo(hDC, event_x, event_y);
  }
  if (indiceF0PtModif < ppointF0Modif->nbPt - 1)
  { MoveToEx(hDC, event_x, event_y, (POINT *) NULL);
	 LineTo(hDC
	   ,wspectro->wsona->LXtoDX(ppointF0Modif->pts[indiceF0PtModif + 1].date)
	   ,wspectro->wsona->LYtoDY(FREQ2ECRF0(ppointF0Modif->pts[indiceF0PtModif + 1].freq)) );
  }

  SetROP2(hDC,R2_COPYPEN);

  SelectObject(hDC,holdpen);
  DeleteObject(hMyPen);
  ReleaseDC(wspectro->wsona->HWindow,hDC);

  xF0PtModif = event_x;
  yF0PtModif = event_y;
}

/*
 * Position finale du point deplace.
 */
void drawUpPiqF0Pt(int event_x, int event_y)
{
  if ( !ppointF0Modif) return;
  event_x=max(event_x_g+1,min(event_x_d-1,event_x));

  //  trouveDatFr(event_x,event_y,&ppointModif->pts[indicePtModif]);
  float x,y;
  snoiunscalexy(event_x, event_y, &x, &y, wspectro->wsona);
  // ppointF0Modif->pts[indiceF0PtModif].date = x;   changing date is vorbidden
  ppointF0Modif->pts[indiceF0PtModif].freq = CAN2HZF0(y);

  affPPointF0Selec();
}



static STRING nomFicPar = "\0", nomFicPitch;
static int nbLus;
#define NBPTF0 1000
// LGLIGNEPITCH MUST equal the length of the line which
// is written in the pitch file
#define LGLIGNEPITCH 18

//
// Read a pitch file to be used for FO contour editing.
//
static void readF0ContourFile(
  char *filename)
{
  int nb;
  char test[LGLIGNEPITCH+1];

  strcpy(nomFicPar, en_cours.nom);
  strcpy(nomFicPitch, filename);

  FILE *f = fopen(nomFicPitch, "r");

  fscanf(f,"%s", test);
  if (strncmp(".pit",test,4)){
    affiche_erreur("Ce fichier n'est pas un fichier de pitch");
    return;
  }

  // ...read all F0 values (Hz)
  //    and their instants (msec converted to samples)...
  nb = 0;
  POINTF0buf pointsLus(NBPTF0, "Pitch values read from file");

  while (!feof(f)) {
    int date, f0;
    fscanf(f,"%d %d\n", &date, &f0);
    if (nb >= pointsLus.size())
      pointsLus.newsize(pointsLus.size() + NBPTF0);
    pointsLus[nb].freq = f0;
    pointsLus[nb].date = enEcht(date);
    nb++;
  }
  fclose(f);
  f0contour[0] = new PolyPointsF0(pointsLus, nb);
  f0contour[1] = new PolyPointsF0(pointsLus, nb);
}

// Read an entire Pitch file already created by CMItereF0(),
// store the data in a PolyPointsF0 structure, and
// display the portion of the contour within the current window.
void WPiqueF0::CMReadPitchFile()
{
  int i;
  // Open and read the entire F0 file,
  // already created by CMItereF0().
  STRING filename;
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "*.*",0,"","");

  if (yaUnSignal != ASIGNAL) return;
  for (i = 0; i < 2; i++) {
  	if (f0contour[i]) delete f0contour[i];
   f0contour[i] = (PolyPointsF0 *) NULL;
  }

  strcpy(filename,"*.*");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (TFileOpenDialog(Parent,FileData,0).DoExecute() == IDOK) {
    strcpy(filename,FileData.FileName);
    if (is_readable(filename) == 1) {
      readF0ContourFile(filename);

      //yaUneCourbeDePitch = TRUE;
      active(MENUSeeF0Contour);
      active(MENUModifF0Contour);
      active(MENUResynth);
      active(MENUSaveF0Contour);
    }
    else affiche_erreur(ml(6930,"Fichier de pitch innaccessible"));

 	 // Display the F0 contour, superimposed on the
  	 // current portion of the spectrogram.
  	 CMSeeF0Contour();
	 yaF0Selected = false;
  }
}


//
//  Calculates the F0 of the current file and uses it as the F0 contour.
//
void WPiqueF0::CMCurrentPitch()
{
  STRING filename;
  int i;
  if (yaUnSignal != ASIGNAL) return;

  for (i = 0; i < 2; i++) {
  	if (f0contour[i]) delete f0contour[i];
   f0contour[i] = (PolyPointsF0 *) NULL;
  }

  // on ouvre le fichier de sortie temporaire
  strcat(strcpy(filename,tempdir),"currenF0.pit");

  if (droit_ecrire(filename)) {
    souris (XC_watch);
    itereF0(en_cours, filename, typeLoc);
    souris (XC_top_left_arrow);
  }
  if (is_readable(filename) == 1) {
    readF0ContourFile(filename);

    //yaUneCourbeDePitch = TRUE;
    active(MENUSeeF0Contour);
    active(MENUModifF0Contour);
    active(MENUResynth);
    active(MENUSaveF0Contour);
  }
  else affiche_erreur(ml(6930,"Fichier de pitch innaccessible"));

  // Display the F0 contour, superimposed on the
  // current portion of the spectrogram.
  CMSeeF0Contour();
  yaF0Selected = FAUX;
}

// Display an F0 contour already read and stored by CMReadPitchFile().
void WPiqueF0::CMSeeF0Contour()
{
  lastCommand = MENUSeeF0Contour;
  if (strcmp(nomFicPar, en_cours.nom)) { //test, si le fichier a change.
    affiche_erreur(ml(6931,"Le fichier de pitch ne correspond pas au fichier de parole"));
    return;
  }

  // Other contours are erased.
  wspectro->wsona->spectro_nettoyer(0,INFINI_I);

  yaF0Selected = false;
  f0contour[0]->selec = false;
  affUnPPointF0(f0contour[0], BLEU);
}


// Display an F0 contour to be edited.
void WPiqueF0::CMModifF0Contour()
{
  lastCommand = MENUModifF0Contour;
  if (strcmp(nomFicPar, en_cours.nom)) { //test, si le fichier a change.
    affiche_erreur(ml(6931,"Le fichier de pitch ne correspond pas au fichier de parole"));
    return;
  }

  // Other contours are erased.
  wspectro->wsona->spectro_nettoyer(0,INFINI_I);
  affUnPPointF0(f0contour[1], ROUGE);
  f0contour[0]->selec = true;
  affUnPPointF0(f0contour[0], BLEU);
  yaF0Selected = true;
}

// Save the pitch contour
void WPiqueF0::CMSaveF0Contour()
{
  STRING filename;
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "*.*",0,"","");
  if (yaUnSignal != ASIGNAL) return;
  if (!f0contour[0]) return;

  strcpy(filename,"*.*");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (TFileSaveDialog(Parent,FileData,0).DoExecute() == IDOK) {
   strcpy(filename,FileData.FileName);
   if (droit_ecrire_message(filename)) {
	  souris (XC_watch);

     int nbValPitch, numPitch, i;
     char mes[LGLIGNEPITCH+1];
     int fd = open(filename, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, S_IREAD|S_IWRITE);

     sprintf(mes,".pit            \r\n"); // pour reconnaitre le fichier
  	  write(fd, mes, LGLIGNEPITCH);

     for (i = 0; i < f0contour[0]->nbPt; i++) {
       int n;
       // WATCH ! The length of the string MUST equal LGLIGNEPITCH
       sprintf(mes,"%10d %5d\r\n",
               enMilli(f0contour[0]->pts[i].date),
               (int)f0contour[0]->pts[i].freq);
       n = write(fd, mes, LGLIGNEPITCH);
       if (n != LGLIGNEPITCH)
         affiche_erreur("erreur d'ecriture");
     }

     souris (XC_top_left_arrow);
   }
  }
}

/* --------------------------------------- */
// Code added by Parham, 7 - 17 Sept. 1998
// ...Implementation of PSOLA resynthesis after F0-contour modif.
static INTbuf Pitch_Mark(1,"marques dans le signal tempo");
static float *Space_mod;
static float *Som_Han;
static float betaPitch =200. ;
static int pnonvois_ms = 10 ;
struct ProgDynPoint mark = {0.,0,0,0,0,0.};

static INTbuf New_Pitch_Mark(1,"marques dans le signal tempo");


static void RawtoWav(
	char* fichierraw,
	char* fichierwav,
	float frequence)
{
 typedef struct{
	unsigned short formatTag;
	unsigned short channels;
	unsigned long samplesPerSec;
	unsigned long avgBytesPerSec;
	unsigned short blockAlign;
	unsigned short bitsPerSample;
 } PCMWAVFMT;

/* Calcul de la taille du fichier en caracteres. */

	int fich = open(fichierraw, O_RDONLY | O_BINARY );
	int l;

	l = (fich>0) ?  lseek(fich, (long)0, SEEK_END) : 0 ;
	close(fich);

/* conversion brut ---> .wav */

	short x;
	int l1 = l + 38, l2 = l + 2;
	FILE *f = fopen(fichierraw, "rb"),
		  *g = fopen(fichierwav, "wb");

	PCMWAVFMT std = {1, 1, frequence, 2*frequence, 2, 16};

	if (!f || !g ) fprintf(stderr, "Unable to open files\n"), fflush(stderr);

/* En-tete du fichier */
	fprintf(g, "RIFF");
	fwrite((char *) &l1, sizeof(long), 1, g);
	fprintf(g, "WAVEfmt \020%c%c%c", 0, 0, 0);

	fwrite( &std, sizeof(PCMWAVFMT), 1, g);
	fprintf( g, "data");
	fwrite((char *) &l2, sizeof(long), 1, g);

	while (!feof(f)) {
			fread((char * ) &x, sizeof(short), 1, f);
			fwrite((char *)&x, sizeof(short), 1, g);
   }

	fclose(f);
	fclose(g);
}

// Get the NEW F0 value for a given sample. The sample number
// is expressed in relation to the beginning of the file.
unsigned int getnewf0(long echt)
{
  long echtR; // echt relatif au dbut du pitch calcul
  long n; // numro du f0

  if (!f0contour[0]->nbPt) return 0;
  echtR = echt - pm.fenPitch / 2;
  n = echtR / pm.depEcht;
  if (n < f0contour[0]->nbPt)
    return IROUND(f0contour[0]->pts[n].freq);
  else return 0;
}


extern notetiqs notes;
double getF0AttenuationFromAnnot(
  int sample,
  double attenuation) // in dB   -10 dB for an attenuation
{
  double att = exp(attenuation / 20. * log(10.));
  for (int i = 0; i < notes.nb - 1; i++) {
    // affichage de la limite
    if (dans(sample, notes.e[i]->limite, notes.e[i + 1]->limite)) {
      int left = notes.e[i]->limite,
          right = notes.e[i + 1]->limite,
          middle = (right + left) / 2.;
      if  (!strcmp(notes.e[i]->Text(),"v")) {
        if (sample < middle)
          return(1. - (sample - left) / ((double) (middle - left)) * (1. - att));
        else return(1. - (right - sample) / ((double) (right - middle)) * (1. - att));
      }
      else if (!strcmp(notes.e[i]->Text(),"\\")) {
        return(1. - (sample - left) / ((double) (right - left)) * (1. - att));
      }
    }
  }
  return(1.);
}


  /* il faut que l'etiquette apparaisse dans la fenetre d'affichage */
static double attenuationGain(
 double f,
 double fmin,
 double fmax,
 double dBAttenuation)
{
  double g	;
  if (f < fmin) g = 0.;
  else if (f < fmax) g = -dBAttenuation  * (1. - (cos((f - fmin) / (fmax - fmin) * _PI) + 1.) /2.);
  else g = -dBAttenuation;
  return(exp(log(10.) * g / 20.));
}

extern void addVoicingOLA(short *speechin, short *speechout, struct ctrl_temps ctloc, int debutEcht, int finEcht, double forcedF0,
								  double Fs, double voicingLevel, double F1, double bwF1, double F1level,
                          double higherFrequencyAttenuation, bool addFormant);
extern int getWindowSizeOLAaddVoicing();
short *HFForcedVoicedSignal = (short *) NULL;

void arb_PitchMod(          // "arbitrary Pitch-Modification"...
	INTbuf& Pitch_Mark,
   SAMPLEbuf& speech,
   struct ctrl_temps ctloc,
	float *Space_mod,
	float *Som_Han,
	float coef_tempo,
	float coef_pitch,
	long * der_marque_virt,
	long * der_marque_synt,
   long * der_marque,
   int lastPitchValue,
   int *lastSample,
   bool continued)
{
	int i,m,n,pitch,
		 ok, //pour l'arret de l'association des marques
		 reverse; // si pair (et nonvoise et egal au precedent) il y a inversement
	int nbm = Pitch_Mark.size();
	int nbms; //nbre de marques dans le signal modifie
	int mu_P,mu_PS;  // taille de la fenetre (depart, synt)
	int milieu, milieu_s ;   // milieu de la fenetre ( depart, synt)
   int interval;
	float * hanning;    // la fenetre de Hanning choisie
	float * hanning_s;    // la fenetre de Hanning finale
	int * space_fen; // morceau de signal de speech fenetr par Hanning

   HFForcedVoicedSignal = new short[ctloc.f_e - ctloc.d_e + 1];
   for (int ii = ctloc.d_e; ii <= ctloc.f_e; ii++)
     HFForcedVoicedSignal[ii  - ctloc.d_e] = speech[ii - ctloc.d_e];
   // all numerical parameters execpt the attenuation are useless.
   addVoicingOLA(HFForcedVoicedSignal, HFForcedVoicedSignal, ctloc, 0, ctloc.f_e - ctloc.d_e,
                 0., en_cours.frequence, revoicingLevel, 300., 200., 70., revoicingNoiseCorrection, false);

   /* marques virtuelles */

	INTbuf Pitch_Mark_Virt(1,"marques virtuelles");
	Pitch_Mark_Virt.newsize(nbm);
	for ( i=0 ; i< nbm ; i++ ) Pitch_Mark_Virt[i]=0;

	Pitch_Mark_Virt[0]=*der_marque_virt;
	for (i=1; i<nbm ; i++) {
     Pitch_Mark_Virt[i] = IROUND(coef_tempo * abs(Pitch_Mark[i]));
     if (Pitch_Mark[i] < 0)
				Pitch_Mark_Virt[i] = -Pitch_Mark_Virt[i] ;
   }

   /* calcul des marques pour le signal synthetis */

	INTbuf Pitch_Mark_Synt(1,"marques du signal synthetise");
	int *Mark_Asso;
	int *Mark_AssoForcedVoicing;
	int *Mark_AssoForcedVoicingNoise;
	if (coef_tempo >= 1 || coef_pitch < 1)
		nbms = ((int)2*(coef_tempo+1))*nbm ; 		// entier superieur a coef_tempo
   else nbms = (int) (2 * nbm / coef_tempo);

	Pitch_Mark_Synt.newsize(nbms);
	Mark_Asso = new int[nbms];
   Mark_AssoForcedVoicing = new int[nbms];
   Mark_AssoForcedVoicingNoise = new int[nbms];
	for (i=0; i < nbms; i++ ) Pitch_Mark_Synt[i]=0;
	for (i=0; i < nbms; i++ ) Mark_Asso[i] = Mark_AssoForcedVoicing[i] = Mark_AssoForcedVoicingNoise[i] = -1;

	Pitch_Mark_Synt[0]=*der_marque_synt;

	if (abs(Pitch_Mark_Synt[0]) == abs(Pitch_Mark_Virt[0])) Mark_Asso[0] = 0;
   else {
     m=0;
     while (abs(Pitch_Mark_Synt[0]) > abs(Pitch_Mark_Virt[m]))m++;
     if ( (abs(Pitch_Mark_Virt[m]) - abs(Pitch_Mark_Synt[0])) <
          (abs(Pitch_Mark_Synt[0]) - abs(Pitch_Mark_Virt[m-1])) )
          Mark_Asso[0] = m;
     else Mark_Asso[0] = m -1; // m!=0 car P_M_S[0] > (int)fabs(P_M_V[0]) par construction
   }
	if  (Pitch_Mark_Virt[Mark_Asso[0]] < 0)
			Pitch_Mark_Synt[0] = - abs(Pitch_Mark_Synt[0]);

  // --- TESTING....
/*  FILE *fpt;
  char fname[40];
  sprintf(fname,"rubb.txt");
  fpt = fopen(fname,"a");
  fclose(fpt);
 */
	i=0;
	ok=1;
	while ( ok )  {
     if (Pitch_Mark_Synt[i] < 0)
       pitch = enEcht(10);//a voir
     else {
       pitch = ( pm.getf0(Pitch_Mark[Mark_Asso[i]])?
                 pm.getf0(Pitch_Mark[Mark_Asso[i]])
                 :pm.getf0(Pitch_Mark[Mark_Asso[i]+1]) );
					// (Pour le cas de debut de voisement)
       if (!pitch) pitch = lastPitchValue;
       if (!pitch) {
         affiche_erreur("Scale_mod: pitch is given a default value");
         pitch = 100;      // 100 Hz
       }
       pitch = IROUND(en_cours.frequence / pitch);
     }

     // TESTING...
   /*  fpt = fopen(fname,"a");
     fprintf(fpt," i = %4d     pitch = %6d    Pitch_Mark_Synt.size() = %d\n",i,pitch,Pitch_Mark_Synt.size());
     fprintf(fpt," Pitch_Mark_Virt.size() = %d\n",Pitch_Mark_Virt.size());
     fprintf(fpt,"      Pitch_Mark.size() = %d\n",Pitch_Mark.size());
     fprintf(fpt,"       *der_marque_virt = %ld     (Pitch_Mark[0] = %d)\n",*der_marque_virt,Pitch_Mark[0]);
     fclose(fpt);
    */
     if (i+1 > Pitch_Mark_Synt.size()-1)
       affiche_erreur("erreur sur la taille du tableau de marques de synthese");

     interval = pitch;

     if (getnewf0(abs(Pitch_Mark[Mark_Asso[i]])) != 0)
       //interval *= (float)(pm.getf0(Pitch_Mark[Mark_Asso[i]])) / (float)(getnewf0(Pitch_Mark[Mark_Asso[i]]));
       interval = IROUND ( en_cours.frequence / getnewf0(abs(Pitch_Mark[Mark_Asso[i]])));
     Pitch_Mark_Synt[i+1]= abs(Pitch_Mark_Synt[i])
                              + interval;

     /* Association */

     if (abs(Pitch_Mark_Synt[i+1]) > abs(Pitch_Mark_Virt[nbm-1]))
       ok = 0;     // si on depasse la fin
     if (ok) {
       m=0;
       if (i+1 > Pitch_Mark_Synt.size()-1)
         affiche_erreur("erreur sur la taille du tableau de marques de synthese");

       // TESTING...
   /*    fpt = fopen(fname,"a");
       fprintf(fpt," ..... i = %4d     Mark_Asso[i] = %d\n",i,Mark_Asso[i]);
       fclose(fpt);
     */
       while (Pitch_Mark_Synt[i+1] > abs(Pitch_Mark_Virt[Mark_Asso[i]+m])) m++;
       if ( ( abs(Pitch_Mark_Virt[Mark_Asso[i]+m])- Pitch_Mark_Synt[i+1] ) <
				  ( Pitch_Mark_Synt[i+1] - abs(Pitch_Mark_Virt[Mark_Asso[i]+m-1])) )
				Mark_Asso[i+1] = Mark_Asso[i] + m;
       else Mark_Asso[i+1] = Mark_Asso[i] + m -1;


       if  (Pitch_Mark_Virt[Mark_Asso[i+1]] < 0) {
         Pitch_Mark_Synt[i+1] = - abs(Pitch_Mark_Synt[i+1]);
         if (getnewf0(abs(Pitch_Mark[Mark_Asso[i + 1]])) > 0) {
           int sample = abs(Pitch_Mark[Mark_Asso[i + 1]]);
           int newF0after = ((int) getnewf0(sample + enEcht(30.))) ? 1 : -1,
               newF0before = ((int) getnewf0(sample - enEcht(30.))) ? 1 : -1,
               F0After = ((int) pm.getf0(sample + enEcht(30.))) ? 1 : -1,
               F0Before = ((int) pm.getf0(sample - enEcht(30.))) ? 1 : -1;
           if ( newF0after * F0After < 0 ||  newF0before * F0Before < 0)  { // a voiced mark has to be found
             // we prevent local changes of F0 due to the sampling of the curve, actually discrepancies between F0 and the curve of F0)
             // and not real changes by testing 30 ms before or after.
             // because voicing is added at an unvoiced place.
             // the best solution is to find a mark just before, and if not possible just after

             int k, k0;
             for (k = Mark_Asso[i + 1] - 1; k > 0 && (Pitch_Mark[k] < 0 || !pm.getf0(abs(Pitch_Mark[k]))); k--);
             k0 = k;
             // to avoid the last period which could not be sufficiently intense
             if (k - 3 > 0 && Pitch_Mark[k - 3] > 0 && pm.getf0(abs(Pitch_Mark[k - 3]))) k = k - 3;

             if (!k) {// the first solution failed, try after
                for (int k = Mark_Asso[i] + 1; k < Pitch_Mark.size() && (Pitch_Mark[k] < 0 || !pm.getf0(abs(Pitch_Mark[k]))); k++);
                if (k == Pitch_Mark.size()) // fallback solution
                  k =  Mark_Asso[i + 1];
	          }
             Mark_AssoForcedVoicing[i+1] = k;
             // Mark_AssoForcedVoicingNoise[i+1] = k;
             int realF0 = pm.getf0(abs(Pitch_Mark[k]));
             if (!realF0)
             	affiche_erreur("Unvoiced pitch in forced addind voicing");
           }
         }
       }

      i++;
     }

   }       //(on s'arrete a i mais i+1 est ecrit (sans marque associee)!)

  /* Reconstruction avec la fenetre de Hanning */

	mu_P = 3 * enEcht(10);     // initialisation avec les nonvoiss.
	mu_PS = mu_P;
   hanning = new float [5000]; // taille a voir mais resiste a 50 Hz a 44 kHz
   hanning_s = new float [5000];
	space_fen = new int [5000];
   HANNING tableau_fen_han;


   // The synthesis begins with 1 to not resynthetize the mark synthetized in
   // the previous use of scalemod. (here, arb_PitchMod)
	i = (continued) ? 1 : 0;
	while (Mark_Asso[i] != -1 ) {
     if (Mark_AssoForcedVoicing[i] > 0) {
       int sample = Pitch_Mark[Mark_AssoForcedVoicing[i]];
       pitch = pm.getf0(sample);
       if (pitch)  pitch = IROUND(en_cours.frequence / pitch);
       else affiche_erreur("null pitch in adding voicing");
     }
     else if (Pitch_Mark[Mark_Asso[i]] < 0)
       pitch = enEcht(10);//a voir
     else {
        pitch = ( pm.getf0(Pitch_Mark[Mark_Asso[i]])?
							pm.getf0(Pitch_Mark[Mark_Asso[i]])
						  :pm.getf0(Pitch_Mark[Mark_Asso[i]+1]) );
        // (Pour le cas de debut de voisement)
        if (pitch)  pitch = IROUND(en_cours.frequence / pitch);
        /* else ok=0; */
     }

     mu_P = (int) 3 * pitch ;
     milieu = (mu_P - 1)/2;

     tableau_fen_han.fenetre(mu_P, hanning);


     mu_PS = mu_P;
     milieu_s = (mu_PS-1)/2;

     tableau_fen_han.fenetre(mu_PS, hanning_s);

     if (Mark_AssoForcedVoicing[i] > 0) {
       int N = IROUND(exp(log(2.) * ceil(log2(mu_P))));

       double *reelle, *imag, *module, *phase;
       reelle = new double[N];
       imag = new double[N];
       module = new double[N];
       phase = new double[N];
		 // adding voicing
       double attenuation = getF0AttenuationFromAnnot(abs(Pitch_Mark[Mark_Asso[i]]), revoicingTimeAttenuation);

       for (n = 0;  n < mu_P ; n++) {
     	   if  ( abs(Pitch_Mark[Mark_AssoForcedVoicing[i]])-ctloc.d_e + n - milieu < speech.size()
            && abs(Pitch_Mark[Mark_AssoForcedVoicing[i]])-ctloc.d_e + n - milieu >= 0)
    	 	   space_fen[n]= attenuation * hanning[n] *
						 	speech[(int)(fabs(Pitch_Mark[Mark_AssoForcedVoicing[i]]))-ctloc.d_e + n - milieu];

         else affiche_erreur("depassement dans space");
       }
       for (int j = 0; j < mu_P; j++) {
       	reelle[j] = space_fen[j];
         imag[j] = 0.;
       }
       for(int j = mu_P; j < N; j++)
         reelle[j] = imag[j] = 0.;
       fft(N, reelle, imag);
       for (int j = 0; j <= N/2; j++) {
         double gain = attenuationGain(j * en_cours.frequence / N, 600., 1200., 40.);
         reelle[j] =  reelle[j] * gain;
         if (j && j != N/2) reelle[N-j] = reelle[j];
         imag[j] = imag[j] * gain;
         if (j && j != N/2) imag[N-j] = -imag[j];
       }
       ifft(N, reelle, imag);
       for(int j = 0; j < mu_P; j++) space_fen[j] = reelle[j] / N;

       // adding the original or revoiced speech to set the frication noise
       for (n = 0;  n < mu_P ; n++) {
     	   if  ( abs(Pitch_Mark[Mark_Asso[i]])-ctloc.d_e + n - milieu < speech.size()
            && abs(Pitch_Mark[Mark_Asso[i]])-ctloc.d_e + n - milieu >= 0) {
     	 	   reelle[n]= hanning[n] *
						 	HFForcedVoicedSignal[(int) (fabs(Pitch_Mark[Mark_Asso[i]]))-ctloc.d_e + n - milieu];
     	 	   /* reelle[n] = hanning[n] *
						 	speech[(int)(fabs(Pitch_Mark[Mark_Asso[i]]))-ctloc.d_e + n - milieu];  */
            imag[n] = 0.;
         }
         else affiche_erreur("depassement dans space");
       }

       for(int j = 0; j < mu_P; j++) space_fen[j] += reelle[j];   // attention ne garder que cette ligne ou celle d'avant.
       delete [] reelle;
       delete [] imag;
       delete [] module;
       delete [] phase;
     }
     else for (n = 0; n < mu_P ; n++) {
     	 if  ( abs(Pitch_Mark[Mark_Asso[i]])-ctloc.d_e + n - milieu < speech.size()
            && abs(Pitch_Mark[Mark_Asso[i]])-ctloc.d_e + n - milieu >= 0)
     	 	space_fen[n] = hanning[n] * speech[(int)(fabs(Pitch_Mark[Mark_Asso[i]]))-ctloc.d_e + n - milieu];
       else affiche_erreur("depassement dans space pour forced voicing");

     }

     if ( (Pitch_Mark[Mark_Asso[i]] < 0 && i>0) &&
					(Mark_Asso[i-1] == Mark_Asso[i] && !(reverse % 2) ) )  {
       if ( Mark_Asso[i-1] == Mark_Asso[i] && Pitch_Mark[Mark_Asso[i-1]] < 0)
       for (n=0 ; n< mu_P ; n++) {
         int ind = abs(Pitch_Mark_Synt[i]) + n - milieu;
         if (ind >= 0) {
           Space_mod[ind] += space_fen[mu_P-1-n]; // inversion de la fenetre
           Som_Han[ind] += hanning_s [mu_P-1-n];
           *lastSample = ind;
         }
       }
       reverse++;
     }
     else {
       for (n=0 ; n< mu_P ; n++) {
         int ind = abs(Pitch_Mark_Synt[i]) + n - milieu;
         if (ind >= 0) {
           Space_mod[ind] += space_fen[n];
           Som_Han[ind] += hanning_s [n];
           *lastSample = ind;
         }
       }
       reverse=0;
     }

     i++;
   }
   *der_marque_synt = Pitch_Mark_Synt[i - 1];
   for (m = Pitch_Mark_Virt.size() - 1;
   	  m >=0 && abs(Pitch_Mark_Virt[m]) > abs(*der_marque_synt); m--);
   if (m >= 0) {
   	*der_marque_virt = Pitch_Mark_Virt[m];
      *der_marque = Pitch_Mark[m];
   }
   else affiche_erreur("erreur dans scalemod");

	delete [] hanning;
   delete [] hanning_s;
   delete [] space_fen;
   delete [] Mark_AssoForcedVoicing;
   delete [] Mark_AssoForcedVoicingNoise;
   delete [] Mark_Asso;
   delete [] HFForcedVoicedSignal;
}


// The main routine called by selecting menu-item:
//   Prosody--Pitch--Resynthesise Waveform...
void WPiqueF0::CMResynth()
{
  SAMPLEbuf speech(1, "speech signal for pitch-scale modifications");
  STRING resFile, resRawFile;
  int pnonvois;
  int premier_pic_Maxmin ;
  long der_marque_virt;
  long der_marque_synt;
  int lastPitchValue, lastSample;
  long der_marque;
  int i;
  int comp, debut;
  struct ctrl_temps ctloc, ct_;
  struct ProgDynPoint *lastImposedMark;
  bool continued = false;

  if (!f0contour[0]) return;
  lastPitchValue = lastSample = 0;
  lastImposedMark = (struct ProgDynPoint *) NULL;
  ctloc = ct;

  strcpy(resFile,tempdir);
  strcat(resFile,"modif.wav");
  strcpy(resRawFile,tempdir);
  strcat(resRawFile,"modif");

  pm.fenPitch = (int) (en_cours.frequence * (float) FENPITCH / 1000.);
  Space_mod = new float[(int)(ct.tf + pm.fenPitch)];
  Som_Han = new float[(int)(ct.tf + pm.fenPitch)];
  for (i=0; i < (int)(ct.tf); i++) {
    Space_mod[i] = 0;
    Som_Han[i] = 0.;
   }

  comp = 0;
  pnonvois=enEcht(pnonvois_ms);
  premier_pic_Maxmin = 0;
  debut = 1; // debut du signal pour traitement
  ctloc.d_e = 0;
  ctloc.f_e = ctloc.d_e + (int)min(((int)MAXECHTECR-1),(ctloc.tf-ctloc.d_e-1));
  ctloc.d_es = ctloc.d_e;
  ctloc.f_es = ctloc.f_e;
  ctloc.d_a = ctloc.d_es;
  ctloc.f_a = ctloc.f_es;
  souris(XC_watch);

  ct_.d_es = 0;


  while (ctloc.d_es + (int)(pm.fenPitch * 3) < ct.tf || ctloc.d_es == ct_.d_es) {
    struct ProgDynPoint *lastMark;
    INTbuf Pitch_MarkBis(1,"Copies des marques dans le signal tempo");
    int nbValPitch;
    pm_space.newsize(ctloc.f_e-ctloc.d_e+1);

    // Testing only.....
 /*    FILE *fpt;
    char fname[40];
    sprintf(fname,"diag.txt");
    fpt = fopen(fname,"w");
    fprintf(fpt,"%s\n",en_cours.nom);
    fprintf(fpt," ct.tf = %ld       der_marque = %ld\n",ct.tf,der_marque);
    fprintf(fpt," ctloc.d_e = %ld       ctloc.f_e = %ld\n",ctloc.d_e,ctloc.f_e);
    fclose(fpt);
  */
    // ----------------------------

   speech.newsize(ctloc.f_e - ctloc.d_e + 1);
   lireSignal(en_cours, en_cours.depl + ctloc.d_e*sizeof(SAMPLE),
                 speech, 0, ctloc.f_e - ctloc.d_e + 1);
    calculePitch(&nbValPitch, speech, ctloc, typeLoc);

    // calcul des marques
    souris(XC_watch);
    Pitch_Mark.newsize(ctloc.f_es - ctloc.d_es + 1);
    for (i = 0 ; i < ctloc.f_es - ctloc.d_es + 1; i++ ) Pitch_Mark[i]=0;


    lastMark = Pitch_Marqueur(Pitch_Mark, speech, ctloc, ctloc.d_e, ctloc.f_e, betaPitch,
                              pnonvois, lastImposedMark, &premier_pic_Maxmin);

    // The first mark must be the last mark found in the previous
    // pitch marking.
    if (continued && lastImposedMark) {
      //IL faut que les marques virtuelles et reelles soient dans un rapport 2
      // Nothing to do because the last mark is used to force the
      // pitch marking to begin with this mark.
      i=1;
      while (Pitch_Mark[i] != 0) i++;
      Pitch_Mark[0] = lastImposedMark->dat;
      // The last mark is discarded because it is not sufficiently reliable
      Pitch_Mark.newsize(i - 1);
    }
    else {
      i=0;
      while (Pitch_Mark[i] != 0) i++;
      // The last mark is discarded because it is not sufficiently reliable
      Pitch_Mark.newsize(i - 1);
    }

    // Tempo assumed same as original signal...
    optProso.timeFactor = 1.0;

    if (debut) {
      der_marque_virt = optProso.timeFactor * Pitch_Mark[0] ;
      der_marque_synt = der_marque_virt;
      debut = 0;
    }

    // calcul de dilatation ou compression
    arb_PitchMod(Pitch_Mark, speech, ctloc, Space_mod, Som_Han, optProso.timeFactor,
                optProso.f0Factor, &der_marque_virt, &der_marque_synt, &der_marque,
                lastPitchValue, &lastSample, continued);
    continued = true;

    if (Pitch_Mark[Pitch_Mark.size() - 1] > 0) {
      lastPitchValue = pm.getf0(Pitch_Mark[Pitch_Mark.size() - 1]);
      if (!lastPitchValue) {
        lastPitchValue = pm.getf0(Pitch_Mark[Pitch_Mark.size() - 1] - enEcht(4));
        if (!lastPitchValue)
          lastPitchValue = max(pm.getf0(Pitch_Mark[Pitch_Mark.size() - 1] + enEcht(4)),
                               pm.getf0(Pitch_Mark[Pitch_Mark.size() - 1] - enEcht(8)));
      }
      if (!lastPitchValue)
       affiche_erreur("CMPitchScaleMod: pitch nul sur marque voisee");
    }
    else lastPitchValue = 0;

    if (der_marque > 0) {
      lastImposedMark = &mark;
      lastImposedMark->dat = der_marque;
      premier_pic_Maxmin = (speech[der_marque - ctloc.d_e] > 0) ? 2 : 1;
    }
    else {
      lastImposedMark = (struct ProgDynPoint *) NULL;
      premier_pic_Maxmin = 0;
    }

    ct_ = ctloc;
    ctloc.d_es = abs(der_marque) - pm.fenPitch/2 ;
    ctloc.f_es = ctloc.d_es + (int)min(((int)MAXECHTECR-1),(ctloc.tf-ctloc.d_es-1));
    ctloc.d_a = ctloc.d_es;
    ctloc.f_a = ctloc.d_es + (int)MAXECHTECR -1;
    ctloc.d_e = ctloc.d_es;
    ctloc.f_e = ctloc.f_es;
    comp++;
   }

  if (ctloc.d_es == ct_.d_es)
    affiche_erreur(ml(9015,"Impossible de traiter tout le fichier"));

  souris(XC_top_left_arrow);
  ofstream sortie (resRawFile, ios::out | ios::binary);
  int Space_mod_int;
  for ( int e=0; e < lastSample ; e++) {
    Space_mod[e]=(Som_Han[e]?(int)(Space_mod[e]/Som_Han[e]):0);
    Space_mod_int = (int) Space_mod[e];
    sortie.write((char *) &Space_mod_int, sizeof(SAMPLE) ) ;
  }
  sortie.close();
  Pitch_Mark.newsize(0);
  delete [] Space_mod;
  delete [] Som_Han;

  RawtoWav(resRawFile, resFile, en_cours.frequence);
  unlink(resRawFile);
  wspectr->Show(SW_SHOWNORMAL);
  lecture_signal(resFile, 0);
  afficherSpectro();
  wspectr->UpdateWindow();

  syntheExists = true;
	{
    STRING message;
    STRING commande;
    CHARbuf buf(1,"");
    STARTUPINFO siStartInfo;
    PROCESS_INFORMATION piProcInfo;
    bool b;

    sprintf(message,"%s %s",ml(9012,"Le fichier synthtis est"),resFile);
    buf.cpy(message);
    ::PlaySound(resFile, 0, SND_ASYNC | SND_NOWAIT | SND_FILENAME);
   }
}

#ifdef BUFCONTROLS
PiqF0Point & POINTF0buf::operator[](long i)
{ if ((i<0)||(i>=ni))
  { char ch1[80],ch2[80];

    strcpy(ch1,"range error : ");
    strcat(ch1,name);
    sprintf(ch2,"POINTF0buf[i] : i=%ld out of range",i);
    MessageBox(GetActiveWindow() //PTWtop->HWindow
               ,ch2,ch1,MB_OK);
    i=ni;
    //((PiqF0Point huge *)p)[ni]=0;
  }
  return ((PiqF0Point huge *)p)[i];
}
#endif


DEFINE_RESPONSE_TABLE1(WPiqueF0,WSignal)
  EV_WM_SIZE,
  EV_COMMAND(MENUReadPitchFile,CMReadPitchFile),
  EV_COMMAND(MENUCurrentPitch,CMCurrentPitch),
  EV_COMMAND(MENUSeeF0Contour,CMSeeF0Contour),
  EV_COMMAND(MENUModifF0Contour,CMModifF0Contour),
  EV_COMMAND(MENUSaveF0Contour,CMSaveF0Contour),
  EV_COMMAND(MENUResynth,CMResynth),
  EV_COMMAND(MENURevoicingParam,CMRevoicingParam),
  EV_COMMAND(MENUProsodyHelp,CMHelpProsody),
  EV_COMMAND(MENUZoomout, CMzoomOut),
  EV_COMMAND(MENUZoomin, CMzoomIn),
  EV_COMMAND(MENUPlay, CMplay),
END_RESPONSE_TABLE;

WPiqueF0::WPiqueF0()
    : WSignal(PTWtop,ml(6943,"F0 contour modifications"))
{
  Attr.Style=WS_VISIBLE|WS_POPUPWINDOW|WS_CAPTION
		       |WS_THICKFRAME;
  Attr.X=60;
  Attr.Y=0;
  Attr.W=400;
  Attr.H=300;
  this->SetIcon(GetApplication(), IDI_ICON1);
  ::yaUneFenetrePiqueF0 = true;
}

WPiqueF0::~WPiqueF0()
{
  yaUneFenetrePiqueF0 = false;
  ::DrawMenuBar(PTWtop->HWindow);
  ::yaUneFenetrePiqueF0 = false;
  ::yaF0Selected = false;
}

void WPiqueF0::SetupWindow()
{
  WSignal::SetupWindow();
  this->AssignMenu("F0MODIF");
  syntheExists = false;
  lastCommand = 0;
}

void WPiqueF0::EvSize(uint sizeType, TSize& size) //RTMessage)
{
  Attr.H = maxi(100,Attr.H);
  Attr.W = maxi(100,Attr.W);
  WSignal::EvSize(sizeType, size);
  ::InvalidateRect(HWindow,NULL,TRUE);
}


bool WPiqueF0::CanClose()
{
  wspectro->wsona->spectro_nettoyer(0,INFINI_I);
  return true;
}

void Wtop::CMF0Modifications()
{
  if (yaUnSignal != ASIGNAL) return;

  desactive(PPMENUAllStandard);
  ::DrawMenuBar(PTWtop->HWindow);
  yaUneFenetrePiqueF0 = true;

  GetApplication()->MakeWindow(wpiquef0 = new WPiqueF0);
}

void WPiqueF0::CMplay()
{
  if (syntheExists) WSignal::CMplay();
}

void WPiqueF0::CMzoomIn()
{
  if (syntheExists) WSignal::CMzoomIn();
}

void WPiqueF0::CMzoomOut()
{
  if (syntheExists) WSignal::CMzoomOut();
}

void WPiqueF0::CMRevoicingParam()
{
  TWrevoicingParam(PTWtop,"WRevoicingParameters").Execute();
}

typedef HWND WINAPI (*HtmlHelpFunc)( HWND hwndCaller, LPCSTR pszFile,UINT uCommand, DWORD dwData );

const char pszHelpOCXCtrl [] = "hhctrl.ocx";
void WPiqueF0::CMHelpProsody()
{
  HINSTANCE hHelpOCX = ::LoadLibrary(pszHelpOCXCtrl);
  if (!hHelpOCX) {
    ::MessageBox(HWindow, "Cannot use the html help!", "Error", MB_OK);
   return;
  }

   HtmlHelpFunc HTMLHelp = (HtmlHelpFunc)::GetProcAddress(hHelpOCX, ATOM_HTMLHELP_API_ANSI);

  // Now call HTMLHelp functions:

  HTMLHelp(HWindow, helpfile, HH_DISPLAY_TOPIC, (DWORD) "mouseF0.htm");
}


