/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier pitchm.h
//
#ifndef _PITCHM_H
#define _PITCHM_H
extern FILE *ficpm;
extern bool noPitchMessage;
extern int typeLoc; // type du locuteur

//#define ORDRETF		1024
#define ORDRETFMAX	2048
#define SILENCE		(SILENCE_SPECTRO - 2.)	/* seuil au-dessous duquel on considere qu'il s'agit
				   de bruit de fond */
#define NBPTPREL	2000	/* nombre de points du spectre preleve 
				   (on suppose que la resolution de ce spectre est 1 Hz) */
#define DEPPITCH	8	/* deplacement en ms entre 2 estimations consecutives du pitch */
#define FENPITCH	32	/* duree (en ms)  de la fenetre de calcul du pitch */

#define FENENER		4	/* duree (en ms)  de la fenetre de calcul de l'energie de voisement */

/* Deux seuils, l'un pour definir le voisement, l'autre pour le bonus de non-voisement. */
#define MAXVOISEMENT	19
#define MAXSOURD	29
#define FREQPICVOIS	800.	/* frequence au-dessous de laquelle on recherche les pics qui permettent
				   de rechercher le voisement */
/*
 * Un point de programmation dynamique pour le lissage du pitch.
 */
struct pdynPoint {
  float dcum,		/* distance cumulee atteinte en ce point */
        dir;		/* direction avec le point precedent */
  int antDat,antTyp	/* date et type du point antecedent */
			//   qui est tabLisGlob[antDat][antTyp]
		/* revoir sans doute  supprimer
		   type du point (0 : sourd,
		   		  1 : le f0 avec la meilleure correlation,
				  2 : le f0 avec la seconde meilleure correlation) */
      ,ignore;		/* VRAI si le point est ignore a la suite du lissage */
  float bonus,		/* bonus de voisement ou de non voisement suivant le type du point */
        cvois,		/* coefficient de voisement */
        f0;		/* valeur de f0 (en Hz) */
};

/*
 * Un vecteur de programmation dynamique pour le lissage du pitch.
 */
typedef struct pdynPoint pdynVec[4];

/*
 * Les parametres de l'extracteur de pitch.
 */
struct paramPitch {
  float freqVois;	/* Frequence sup. de la bande de recherche de l'energie pour le voisement */
  float dfPrel;		/* Precision du spectre interpole */
  int f0min, f0max,	/* Domaine de frequence dans lequel on recherche F0 */
      f0minCorrel,	/* frequence minimale pour laquelle on calcule la correlation */
      fmax;		/* Frequence maximale du peigne */
  float k;		/* parametre d'amplitude n^(-1/k) des dents du peigne */
  int mem;		/* nombre de points a prendre en compte dans le lissage */
  float alpha,		/* ponderation de la distance */
 	beta;		/* ponderation de la courbure */
};
      
#define TABPITCHNULL {(int *) NULL, (int *) NULL, (int *) NULL}
//extern void aff_echelle();
//extern void eff_echelle();

extern FLOATbuf enerPeriod;  // Energie calculee sur le
                           // spectro bande large 
extern pdynVec *tabLisGlob; // tableau de lissage du pitch
	// tabLisGlob[i][j] contient les candidats au pitch
	//     i est l'indice temporel, et correspond
	//          l'chantillon pm.premEcht+i*pm.depEcht
	//     j va de 0  3 et donne les 4 candidats possibles
	//         0 est le candidat sourd
        //         1, 2, 3 sont les candidats voiss
extern SAMPLEbuf pm_space;
extern int pitchBrutMedian; // pitch brut median
class pitchm
{ public:
    // 1er chantillon : ct.d_es
    // spectres :
    //     calculs de ct.d_es  ct.f_es
    //         1er chantillon : ct.d_es
    //         deplacement : 256   revoir dsimpliciter
    //         nombre de spectres :
    //             (_i(ct.f_es)) - _i(ct.d_es)))/OFFSETFFT256 + 1
    //     ils sont rangs dans sps
    int premEchtAbs; // premier echantillon absolu (sans _i)
    int premEcht; //  premier et dernier echqntillons du calcul du pitch
    int derEcht;
    int nbPitch;  // nombre de valeurs du pitch
    int depEcht; // dplacement entre deux points de pitch
    int fenPitch; // fentre de calcul du pitch 
    int ordreTF;  // ordre FFT pour le pitch
    bool yaUneEchelle;
	 int nbGradReq;  // # of graduations necessary to display pitch
					         // One graduation per 100 Hz if possible
    spectres tabl_pitch;
    pitchm();
    void currentF0();

    void clear();
    double nb100HzPerGrad();
    double F0Maxscale();
    void F0Font();
    LOGFONT logfontF0;
	 void dispScale();
    unsigned int getf0(long echt);
	 unsigned int getReadf0(long echt);
};
extern pitchm pm;

// fichier pmenervo.cpp
extern void enerVois( // calcul de l'energie de voisement
	SAMPLEbuf& parole,
	long premEcht,	// premier
	long derEcht,   //   et dernier echantillon
	                // de recherche du pitch,
			//   (entre 0 et MAXECH) 
	FLOATbuf& ener, // energie au milieu de la
			// fenetre de calcul du pitch
	int pitchMedian);
// fichier pmlispit.cpp
extern void lisPitch(	// lissage du pitch 
	pdynVec huge *tabLis,
	int n,	// nombre de valeurs de pitch
	struct paramPitch& param);
extern void affMeilPitch( // affichage du pitch corrige
	INTarray& pitch,	// les valeurs du pitch
   int premEchtAbs,  // premier echantillon de calcul du pitch
	int n);		// nombre de valeurs de pitch 
extern void meilPitch(	// determination du pitch corrige
	INTarray& pitchCorrige, // tableau du pitch corrige
    	int n, 		// nombre de valeurs de pitch 
    	int mem);	// memoire du lissage 
extern void completePitch( // pour completer les valeurs
                // de pitch abandonnees par le lissage 
	INTarray& pitch,  // tableau des valeurs de pitch
    	int n);	// nombre de valeurs de pitch 
extern void calculePitch(
  int *nbValPitch,
  SAMPLEbuf& parole,
  struct ctrl_temps cte,
  int sexe);
extern void rectifiePitch(
	long premEcht, /* premier echantillon du calcul
	                 de pitch */
	INTarray& pitch,  /* tableau des valeurs de pitch */
    	int n,	      /* nombre de valeurs de pitch */
	struct paramPitch& param); /* parametres de
			     l'extraction du pitch */
// fichier pmmedian.cpp
extern int media_i( // Mediane d'un tableau de long
		   INTarray& t, int deb, int fin
                  ,int vMin, int vMax, int delta); 
// fichier pmmenu.cpp
void fprintf_tabLisGlob(FILE *ficm, int i);
void fprintf_pitchCorrige(FILE *ficm);
// fichier pmpeigne.cpp
extern void correlPeigne( // correlation entre le
                      //spectre preleve et le peigne 
	FLOATbuf& sPrel,	// spectre preleve
	FLOATbuf& correl,	// correlation
	int fmax,   // frequence maximale du spectre preleve
	        // (pour n'etudier qu'une partie du spectre) 
	int pitchm, // domaine de valeurs 
	int pitchM, //     de pitch a "peigner" 
	float& corTot); // correlation totale
extern float correlf0(	// idem mais pour une seule
                        // valeur de f0 
	FLOATbuf& sPrel,	// le spectre preleve 
	int fmax,	// frequence maximale du spectre preleve
	        // (pour n'etudier qu'une partie du spectre) 
	int f0);
extern void initPeigne(			/* initialisation des dents du peigne */
	int n,	  // nombre de dents du peigne 
	float k); // exposant des dents du peigne 
// fichier pmpics.cpp
extern void pics(  // construction du spectre preleve 
	FLOATbuf& s,	    // spectre de depart  
	FLOATbuf& spectrePrel, // spectre prlev
	int n,		    // nombre de points
	             // du spectre initial a considerer 
	int fmax,	    // frequence jusqu'a laquelle
	             //on somme l'energie des pics 
	float& enerPics, // energie des pics jusqu'a fmax
	int affich);	 // VRAI pour affichage de controle 
// fichier pmpitchm.cpp
extern void init_pdynPoint(pdynPoint huge *p);
extern void init_pdynVec(pdynVec huge *p);

extern void pitchBrutMartin( // calcul du pitch brut
	struct paramPitch& param); // parametres de l'extraction du pitch
extern void calUnPitch(	// calcul d'une valeur de pitch 
	long e,	 // le premier echantillon
	         // de la fenetre de calcul du pitch
	long fenPitch, // le nombre d'echantillons
	              // de la fenetre 
	int *f0,  // les trois candidats
                  //     le dernier ne sert que si les
                  //     deux premiers sont 
		  //	 ecartes de moins de 70 Hz 
	float *enerVois,  // les energies pour
		    // les 3 candidats dans la
		    // bande de voisement 
	float *coefVois,  // les coefficients
		    // de voisement pour les 3
		    // candidats 
	struct paramPitch& param); // les parametres du pitch
// fichier pmvoisem.cpp
extern void ener(    // energie dans une bande de frequence
   spectres &spectroTab,	 // spectra on which the energy is calculated
	FLOATbuf& res,  // tableau de l'energie calculee
   int premEcht,    // premier et dernier echantillon de la region.
   int derEcht,
	float fbas,  // frequences limites en Hz 
	float fhaut);
extern void histoVois(	// construction de l'histogramme de voisement
	FLOATbuf& s,  // le tableau dont on construit l'histogramme
	int debut,  // limites du sous-tableau
	int fin,    //     a balayer 
	INTbuf& histo); // nombre de classes de l'histogramme
extern int vsupHisto(	// seuillage d'un histogramme
	INTbuf& h, 	// histogramme
	int effectif,	// nombre d'individus total 
	float p);
extern void reglageBonus( //maxVoisement, medVoisement, maxEnerVois)
	float maxVoisement, // valeur maximale observee
	                    // du critere de voisement 
        float medVoisement, // voisement median 
	float maxEnerVois); // energie maximale dans
	                    // la bande de voisement 
extern float bonusVoise( // bonus de voisement
	float x,
      	float coefVois,
     	float en); // energie dans la bande de voisement 
extern float bonusSourd( // bonus de non voisement 
	float x,
	float coefVois,
      	float en); // energie dans la bande de voisement 
extern float coefVoisCorrel( // calcul du coefficient
                             // de voisement 
	FLOATbuf& correl,
	int f0);
extern void itereF0(struct infoFile &localEn_cours, char *filename, int typeLoc);
#endif
