/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier pmenervo.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include <malloc.h>

#include "spectro.h"
#include "pitchm.h"
#include "outils.h"
#include "spectre.h"
#include "signal.h"

long nEner; // nombre de valeurs d'energie calculees
#define FREVOIS 500.  // frequence au-dessous de
		  // laquelle on recherche le voisement
static FLOATbuf X(NB_CANAUX+1,"static X near enerVoisBrut()");

/*
 * Calcul de l'energie
 */
static float energieBandeF0( 
	FLOATbuf& spec_ 
        ,int chaut)
{ int c;
  float s;
  float *spec=(float *)&spec_[0];

  for (c = 0, s = 0.; c <= chaut; c++)
  { s += ( (spec[c] > SILENCE_SPECTRO)
           ? spec[c] - SILENCE_SPECTRO : 0. );
  }
    s /= (chaut + 1);
  return(s);
}


static void enerVoisBrut(
	long& deplac, // nombre d'echantillons de
		     //     deplacement
	long& nb,     // nombre de valeurs d'energies
		     //     calculees.
        FLOATbuf& enerVoise_)
{
  int i,	
      chaut,		/* dernier coef. du domaine de recherche du voisement */
      nbEcht = min((int) (en_cours.frequence / 1000. * FENENER), 256);
  long derEcht;

  chaut = IROUND(freqToEcr(FREVOIS));
  deplac = enEcht(FENENER/2);	
  nb = nEner = (pm.derEcht - pm.premEcht + 1 -  nbEcht) / deplac + 1;
  enerVoise_.newsize(nEner); 

  initSpecFft(nbEcht, 256);

  for (i = 0; i<nEner; i++) {
    specFft(pm_space, pm.premEcht + i * deplac, X, true, true);
    enerVoise_[i] = energieBandeF0(X, chaut);
    if (debug==DEBUG_PM) {
      fprintf(ficpm,"enerVoisBrut() res[%d]=%.3g\n",i,enerVoise_[i]);
    }
  }
  freeSpecFft();
}




/*
 * Calcul de l'energie ds la bande de voisement
 */
void enerVois( 
	SAMPLEbuf& , 
	long , 	// premier
	long ,  //   et dernier echantillon
	                // de recherche du pitch,
			//   (entre 0 et MAXECH) 
	FLOATbuf& ener, // energie au milieu de la
			// fenetre de calcul du pitch
	int pitchMedian)
{ int periodF0 = (int) (en_cours.frequence / pitchMedian),
      df = max(periodF0, pm.depEcht) / 2;
  long nb;
  long deplac;
  long e, i;
  FLOATbuf enerVoise(1,"enerVoise in enerVois()");

  // revoir nb ne sert pas

  enerVoisBrut( deplac, nb, enerVoise);
  for (e = pm.premEcht+pm.fenPitch/2, i = 0;
       i<pm.nbPitch;
       e += pm.depEcht, i++) {
    int j;
    int i0 = IROUND((e-df-pm.premEcht) / deplac),
        i1 = IROUND((e+df-pm.premEcht) / deplac);
    float en;

    i1=min(i1,(int)enerVoise.size()-1);
    for (j = i0, en = 0.; j <= i1; j++) 
      en += enerVoise[j];
    ener[i] = en / (max(0,i1-i0)+1);

    if (debug==DEBUG_PM) {
      fprintf(ficpm,"enerVois() res[%ld]=%.3g i1-i0=%d\n"
		   ,i,ener[i],i1-i0);
    }
  }
}


