/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier pmmenu.cpp
//

#include "common.h"
#pragma hdrstop

#include <math.h>
#include <malloc.h>
#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdio.h>
#include <dirent.h>

#include "format_f.h"
#include "spectre.h"
#include "filtre.h"
#include "spectro.h"
#include "fmaxspec.h"
#include "pitchm.h"
#include "fftlpc.h"
#include "signal.h"
#include "outils.h"
#include "files.h"

#include "piquef0.h"

/*
//////////////////////////////////////////////////
// description des tapes de calcul du pitch
// avec la succession des procdures
// et les principales variables ou tableaux calculs
//
////////////////////////////////
// l'enchanement des procdures
//
calcPitchMartin()
{ fft-normal()
  pitchBrutMartin() // calcule tabLisGlob
  { CalUnPitch()
    maxvoisement()
    medvoisement()
    PitchBrutMedian()
    maxEnerVois()
    reglageBonus()
    calcul des bonus
  }
  lisPitch()  // lisse tabLisGlob
  meilPitch()  // extrait PitchCorrige de tabLisGlob
  completePitch() // quelques modifications locales de pitchCorrige
                  // et mises  jour correspondantes de tabLisGlob
  rectifiePitch() // rectifie les attaques de voisement
  { dont calUnPitch() et lisPitch() locaux
      sur les attaques de voisement
    utilise alors une table locale tabLoc[5]
      de mme type que tabLisGlob
  }
  copie de pitchCorrige dans pitch
}

////////////////////////
// variables et tableaux
//
tabl_pitch[j]          spectres utilis pour ener() qui
                 calcule energie[j]
energie[j]       nergies des tabl_pitch[j]
tabLisGlob[i][k] les 4 candidats pitch  l'instant i
		   pour i idem pitchCorrig ci-dessous
		   pour k, de 0  3:
		     k=0 candidat sourd
                     k=1,2,3 candidats voiss
voise[i]         associ  tabLisGlob[i]
		 local  pitchBrutMartin
pitchCorrige[i]  les valeurs de pitch
		   premire valeur:
		     sur pm.premEcht avec i=0
		   dplacement: pm.depEcht=8 ms
		   nombre de pitch: pm.nbPitch
enerVoisBrut[m]    premire valeur:
		     sur pm.premEcht avec m=0
		   dplacement: 2 ms
		   dernire valeur: environ pm.derEcht
		     moins une valeur de fentre 
enerVois[i]   nergies de voisement
		   pour i idem pitchCorrig ci-dessus

//////////////////////////////////////////////////
// organisation des sources
//
nom fichier    appelle le(les) fichier(s)

pmmenu.cpp     pmpitchm.cpp pmlispit.cpp

pmlispit.cpp   pmpitchm.cpp (seulement calUnPitch())
	       pmvoisem.cpp (seulement BonusVoise())

pmpitchm.cpp   pmenervo.cpp pmmedian.cpp
	       pmpeigne.cpp pmpics.cpp
	       pmvoisem.cpp

pmpics.cpp     regres4.cpp regres6.cpp

*/

FILE *ficpm;

SAMPLEbuf pm_space(1,"pm_space in pitchm");

int typeLoc = -1;
bool noPitchMessage = false;

pitchm pm;
pitchm::pitchm()
{
  nbPitch=0;
  nbGradReq = 5;
  yaUneEchelle = false;
  F0Font();
}

void pitchm::clear()
{
	nbPitch=0;
}

static struct paramPitch param_locuteur;
int test_pitch_aff = FAUX;
static INTarray pitchCorrige(1,"pitchCorrige near calcPitchMartin()");

//
// Number of 100 Hz between two consecutive pitch graduations
//
double pitchm::nb100HzPerGrad()
{
   // # of graduations available
   int nbGrad = max(1,((int) fMaxSpectro) / 2000);

   if (nbGrad / nbGradReq < 1)
     return((double)(((int) (nbGradReq / nbGrad)) + 1));
   else if (nbGrad / nbGradReq >= 2)
     return((double) 1. / ((int) (nbGrad / nbGradReq)));
   else return(1.);
}

//
//  F0 value for NB_CANAUX (height of the spectro window)
//
double pitchm::F0Maxscale()
{
  // # of graduations available
  double nbGrad = max(1, fMaxSpectro / 2000.);

  return(nbGrad * nb100HzPerGrad() * 100.);
}

//
// Font to display pitch values for F0 axes
//
void pitchm::F0Font()
{
  int height = 12, width = 6;
  logfontF0.lfHeight=height;
  // logfontF0.lfWidth=width;
  logfontF0.lfEscapement=0;
  logfontF0.lfOrientation=0;
  logfontF0.lfWeight=400;
  logfontF0.lfItalic=false;
  logfontF0.lfUnderline=false;
  logfontF0.lfStrikeOut=false;
  logfontF0.lfCharSet=DEFAULT_CHARSET;
  logfontF0.lfOutPrecision=OUT_DEFAULT_PRECIS;
  logfontF0.lfClipPrecision=CLIP_DEFAULT_PRECIS;
  logfontF0.lfQuality=DEFAULT_QUALITY;
  logfontF0.lfPitchAndFamily=DEFAULT_PITCH | FF_DONTCARE;
  strcpy(logfontF0.lfFaceName,"arial");
}

//
//   Display the frequency scale for F0
//
void pitchm::dispScale()
{
  double F0Max = F0Maxscale();
  double nb100HzPerGrad_ = nb100HzPerGrad(),
         intervalle = 100.;
  int i, realNbGrad;

  switch (typeLoc) {
    case -1 : MgiHue (ROUGE, wspectro->wsona); break;
    case 0 : MgiHue (MAGENTA, wspectro->wsona); break;
    case 1 : MgiHue (CYAN, wspectro->wsona); break;
  }

  if (nb100HzPerGrad_ > 1. + EPSILON) {
    intervalle *= nb100HzPerGrad_;
    realNbGrad = nbGradReq / nb100HzPerGrad_;
  }
  else realNbGrad = nbGradReq ;

#ifdef UIY  // display on the bitmap
  for (i = 0; i <= nbGradReq; i++) {
    float ht = i * intervalle / F0Max * NB_CANAUX;
    snol((float) ct.d_a, ht, (float) ct.f_a, ht, wspectro->wsona);
  }
  ::InvalidateRect(wspectro->wsona->HWindow, NULL, TRUE);
#else
  wspectro->wvscale->affichage(0., F0Max, wspectro->wsona->incr / fMaxSpectro * F0Max, "F0 Hz");
#endif
#ifdef UIY
  // vieille version  
  HDC hDC=GetDC(wspectro->wsona->HWindow);
  RECT rect;
  HPEN hCPen;
  HFONT hfont;
  HGDIOBJ  hfontOld;

  hCPen=CreatePen(PS_SOLID, 1, (!typeLoc) ? MAGENTA : ((typeLoc > 0) ? CYAN : ROUGE));
  SelectObject(hDC,hCPen);
  hfont = CreateFontIndirect(&logfontF0);
  hfontOld = SelectObject(hDC, hfont);

  SetTextColor(hDC,(!typeLoc) ? MAGENTA : ((typeLoc > 0) ? CYAN : ROUGE));
  SetBkMode(hDC, TRANSPARENT);
  ::GetClientRect(wspectro->wsona->HWindow,&rect);

  for (i = 0; i <= realNbGrad; i++) {
    STRING s;
    float ht = i * intervalle / F0Max * NB_CANAUX;
    int yy = wspectro->wsona->LYtoDY(ht);
    sprintf(s, "%3d",(int)(i * intervalle));
    MoveToEx(hDC,0,yy, (POINT *) NULL);
    LineTo(hDC,rect.right+1,yy);
    TextOut(hDC, 5, yy - 14, s, strlen(s));
  }
  DeleteObject(hCPen);
  SelectObject(hDC, hfontOld);
  DeleteObject(hfont);
  ReleaseDC(wspectro->wsona->HWindow,hDC);
#endif
  yaUneEchelle = false;
}

void Wtop::CMPitchmEchelle()
{
  pm.yaUneEchelle = true;
  ::InvalidateRect(wspectro->wsona->HWindow, NULL, TRUE);
}


// Get the F0 value for a given sample. The sample
// is related to the beginning of the file.
unsigned int pitchm::getf0(long echt)
{
  long echtR; // echt relatif au dbut du pitch calcul
  long n; // numro du f0
  int nbpitch;

  nbpitch=min(nbPitch,(int)pitchCorrige.size());

  if (nbpitch==0) return 0;
  echtR = echt - fenPitch / 2 - premEchtAbs + depEcht / 2;
  n = echtR / depEcht;
  if (  (n>=0)&&(n<nbpitch) ) {
    if (pitchCorrige[n] == -1)
      affiche_erreur("pitch == -1");
    return max(pitchCorrige[n],0);    // max is used to prevent a -1 value (bug in pitch determination)
  }
  else return 0;
}

void fprintf_tabLisGlob(FILE *ficpm, int i)
{ int j;

  fprintf(ficpm,"tabLisGlob[%d]\n",i);
  for (j=0;j<4;j++)
  { pdynPoint *p=(pdynPoint *)&tabLisGlob[i][j];
    fprintf(ficpm,"... [%d] %.3g %.3g %d %d %d %.3g %.3g %.3g\n"
		,j
		,p->dcum,p->dir
		,p->antDat,p->antTyp,p->ignore
                ,p->bonus,p->cvois,p->f0);
  }
  fflush(ficpm);
}
void fprintf_pitchCorrige(FILE *ficpm)
{ int i,j;

  for (i=0;i<pm.nbPitch;i+=8)
  { fprintf(ficpm,"pitchCorrige[%d] =",i);
    for (j=i;((j-i)<8)&&(j<pm.nbPitch);j++)
      fprintf(ficpm," %d",pitchCorrige[j]);
    fprintf(ficpm,"\n");
  }
}
void log_pitch()
{ int i;
  STRING ch;
  long e;

  logs.open1();
  logs.wr1(">> pitch");
  sprintf(ch,ml(6810,"file : %s"),en_cours.nom);
  logs.wr1(ch);
  logs.wr1(ml(6811,"Temps\tTemps\tFreq."));
  logs.wr1(ml(6812,"[ms]\t[echt]\t[Hz]"));
  for (i=0;i<pm.nbPitch;i++)
  { e=pm.premEcht + i * pm.depEcht + pm.fenPitch/2.;
	 sprintf(ch,"%ld\t%ld\t%d",enMilli(e),e,pitchCorrige[i]);
	 logs.wr1(ch);
  }
  logs.close1();
}

//
// Suppression de la composante continue du signal.
//
//   soustrait la moyenne du signal
//   et contrle l'crtage ventuel
//

extern bool show_warnings;
       
static void supprimeCompContinue(
	SAMPLEbuf& res,
   SAMPLEbuf& donnees,
   int nbEcht)
{
  double s;
  int i, ok;
  SAMPLE sm;
  int minimum = -(1 << 15), maximum = (1 << 15) - 1;

  /* Calcul de la moyenne du signal. */
  for (i = 0, s = 0; i < nbEcht; i++) s += donnees[i];
  sm = (nbEcht) ? ((SAMPLE) (s / nbEcht)) : 0;

  for (i = 0, ok = VRAI; i < nbEcht; i++) {
    int x = donnees[i] - sm,
        ko = (x > maximum || x < minimum);
    ok = ok && !ko;
    res[i] = (!ko) ? x : ((x > maximum) ? maximum : minimum);
  }

  if (!ok && !noPitchMessage && show_warnings) affiche_erreur(ml(6801,"attention : composante continue gnante"));
}

/*
 * Calcul du pitch avec la methode du peigne frequentiel de Martin
 */
static void calcPitchMartin(
  SAMPLEbuf& parole,  // speech signal
  struct ctrl_temps cte)   // local speech indices in signal
{
  int nbSpectres;
  spectres tabl_pitch;

  if (debug==DEBUG_PM) {
    char ch[80];
    strcat(strcpy(ch,tempdir),"pm.txt");
    ficpm=fopen(ch,"wt");
  }

  pm_space.newsize(parole.size());
  pm.premEchtAbs = cte.d_e;
  pm.premEcht = 0;
  pm.derEcht = cte.f_e - cte.d_e;
  supprimeCompContinue(pm_space, parole, cte.f_e - cte.d_e + 1);
  pm.fenPitch = (int) (en_cours.frequence * (float) FENPITCH / 1000.);
  // supprimeCompContinue(pm_space, parole, cte.f_e - cte.d_e + 1);
  pm.ordreTF = (pm.fenPitch < 1024) ? 1024 : 2048;
  pm.depEcht = en_cours.frequence * (DEPPITCH/1000.);

  nbSpectres = (cte.f_e - cte.d_e + 1 - FENFFT256) / OFFSETFFT256 + 1;
  pm.tabl_pitch.resize(nbSpectres);
  pm.tabl_pitch.resize_sp(130);
  pm.tabl_pitch.depl = OFFSETFFT256;
  fft_normal(pm_space, pm.tabl_pitch, 0, nbSpectres, pm.tabl_pitch.depl, VRAI);

  //
  pm.nbPitch=(int)((cte.f_e - cte.d_e - pm.fenPitch + pm.depEcht -1 ) / pm.depEcht);
  pm.nbPitch=max(0,pm.nbPitch);

  // allocation du tableau de lissage du pitch
  // pm.nbPitch = max(pm.nbPitch, MAXECH / pm.depEcht);
  tabLisGlob = (pdynVec *) calloc(pm.nbPitch, sizeof(pdynVec));

  //
  pitchBrutMartin(param_locuteur);
  if (debug==DEBUG_PM)
  { int i;
    for (i=0;i<pm.nbPitch;i++) {
      fprintf_tabLisGlob(ficpm,i);
    }
  }

  lisPitch(tabLisGlob, pm.nbPitch, param_locuteur);
  if (debug==DEBUG_PM)
  { int i;
    for (i=0;i<pm.nbPitch;i++)
    { fprintf_tabLisGlob(ficpm,i);
    }
  }

  meilPitch(pitchCorrige, pm.nbPitch, param_locuteur.mem);
    // attention: la taille de pitchCorrige
    // peut avoir t modifie, et non pm.nbPitch
  if (debug==DEBUG_PM)
    fprintf_pitchCorrige(ficpm);

  completePitch(pitchCorrige, pm.nbPitch);
  if (debug==DEBUG_PM)
    fprintf_pitchCorrige(ficpm);

  rectifiePitch(pm.premEcht, pitchCorrige, pm.nbPitch, param_locuteur);
  if (debug==DEBUG_PM)
    fprintf_pitchCorrige(ficpm);

  // loguer le pitch
  log_pitch();

  pm_space.newsize(1);
  free(tabLisGlob);
  enerPeriod.newsize(1);
  if (debug==DEBUG_PM) fclose(ficpm);
}

void calculePitch(
  int *nbValPitch,
  SAMPLEbuf& parole,
  struct ctrl_temps cte,
  int sexe)	/* 0 locutrice, 1 locuteur, -1 n'importe */
{
  switch(sexe) {
  case 1 :
  	 	param_locuteur.freqVois    = 400.;
    	param_locuteur.f0min       = 65;
  		param_locuteur.f0max       = 400;
  		param_locuteur.f0minCorrel = 50;
  		param_locuteur.fmax        = 1500;
  		param_locuteur.k           = 1.7;
  		param_locuteur.mem         = 2;
  		param_locuteur.alpha       = 4.;
  		param_locuteur.beta        = 4.;
		break;
  case 0 :
    	param_locuteur.freqVois    = 600.;
  		param_locuteur.f0min       = 100;
  		param_locuteur.f0max       = 700;
  		param_locuteur.f0minCorrel = 50;
  		param_locuteur.fmax        = 2000;
  		param_locuteur.k           = 1.7;
  		param_locuteur.mem         = 2;
  		param_locuteur.alpha       = 4.;
  		param_locuteur.beta        = 4.;
      break;
  case -1 :
  		param_locuteur.freqVois    = 600.;
  		param_locuteur.f0min       = 65;
  		param_locuteur.f0max       = 800;
  		param_locuteur.f0minCorrel = 50;
  		param_locuteur.fmax        = 1900;
  		param_locuteur.k           = 1.7;
  		param_locuteur.mem         = 2;
  		param_locuteur.alpha       = 4.;
  		param_locuteur.beta        = 4.;
      break;
  }
  calcPitchMartin(parole, cte);
  *nbValPitch = pm.nbPitch;
}

// Calcul du pitch a l'ecran.
//
static void ex_pitch()
{
  int n;
  test_pitch_aff = VRAI;

  switch (typeLoc) {
    case -1 : MgiHue (ROUGE, wspectro->wsona); break;
    case 0 : MgiHue (MAGENTA, wspectro->wsona); break;
    case 1 : MgiHue (CYAN, wspectro->wsona); break;
  }

  calculePitch(&n, space, ct, typeLoc);
  affMeilPitch(pitchCorrige, ct.d_e, pm.nbPitch);
  active(PPMENUFormants); // Really only to activate MENUModF0Cont...
  Wact->wr("");
}


void Wtop::CMF0Screen()
{
  souris (XC_watch);
  ex_pitch();
  souris (XC_top_left_arrow);
}

// LGLIGNEPITCH MUST equal the length of the line which
// is written in the pitch file
#define LGLIGNEPITCH 18
void itereF0(
  struct infoFile &localEn_cours,
  char *ficF0,
  int sexe)
{
  int offset,
      nbValParEcran = (min(enEcht((long) (DUREE_FEN * 1000)), MAXECHTECR) - 1) / enEcht(DEPPITCH);
  struct ctrl_temps ctloc;
  SAMPLEbuf space_bis(1, "space_bis to iterate pitch calculations");
  int fd = open(ficF0, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, S_IREAD|S_IWRITE);

  if (fd<0) {
    affiche_erreur(ml(6900,"Ouverture du fichier de pitch impossible"));
    return;
  }

  ctloc.tf = localEn_cours.longueur / sizeof(SAMPLE);
  ctloc.d_e = 0;
  ctloc.f_e = min(ctloc.tf - 1, ctloc.d_e + min(enEcht((int) (DUREE_FEN * 1000)), MAXECHTECR) - 1);

  bool test = false;

  do {
    int nbValPitch, decal, numPitch, i;
    char mes[LGLIGNEPITCH+1];

    space_bis.newsize(ctloc.f_e - ctloc.d_e + 1);
    lireSignal(localEn_cours, localEn_cours.depl + sizeof(SAMPLE) * ctloc.d_e, space_bis, 0,
	       ctloc.f_e - ctloc.d_e + 1);
    calculePitch(&nbValPitch, space_bis, ctloc, sexe);

    /* et puis on sauvegarde le pitch... */
    decal = (ctloc.d_e == 0) ? 0 : 5;
    numPitch = IROUND((ctloc.d_e) / enEcht(DEPPITCH));
    offset = lseek(fd, (numPitch + ((test) ? 1 : 0)) * LGLIGNEPITCH, SEEK_SET);
    if (offset != (numPitch + ((test) ? 1 : 0)) * LGLIGNEPITCH) {
      affiche_erreur(ml(6901, "Erreur de dplacement dans le fichier de pitch"));
      close(fd);
      return;
    }

    if (!test) {     //(==0, on est en haut du fichier)
      sprintf(mes,".pit            \r\n"); // pour reconnaitre le fichier
      write(fd, mes, LGLIGNEPITCH);
      test = true;
    };

    for (i = 0; i < nbValPitch - decal; i++) {
      int n;
      // WATCH ! The length of the string MUST equal LGLIGNEPITCH
      sprintf(mes,"%10d %5d\r\n",
              enMilli(ctloc.d_e + i * enEcht(DEPPITCH) + pm.fenPitch / 2),
              pitchCorrige[i]);
      n = write(fd, mes, LGLIGNEPITCH);
      if (n != LGLIGNEPITCH)
        affiche_erreur("erreur d'ecriture");
    }


    /* et on prepare le prochain deplacement. */
    if (ctloc.f_e == ctloc.tf - 1) ctloc.f_e = ctloc.tf + 1;
    else if ((ctloc.tf - ctloc.f_e) < nbValParEcran / 3) {
      ctloc.f_e = ctloc.tf - 1;
      ctloc.d_e = max(0, ctloc.f_e - min(enEcht((int) (DUREE_FEN * 1000)), MAXECHTECR) - 1);
      // correction to be synchronized with previous windows of F0 calculation
      ctloc.d_e = ((((int)ctloc.d_e) / enEcht(DEPPITCH)) + 1) * enEcht(DEPPITCH);
    }
    else {
      ctloc.d_e = ctloc.d_e + (nbValPitch - 10) * enEcht(DEPPITCH);
      ctloc.f_e = min(ctloc.tf - 1, ctloc.d_e + min(enEcht((int) (DUREE_FEN * 1000)), MAXECHTECR) - 1);
    }
  }
  while (ctloc.f_e < ctloc.tf);
  close(fd);
}

// Calcul et sauvegarde du pitch sur tout le fichier de parole
void Wtop::CMItereF0()
{
  STRING filename;
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "*.*",0,"","");

  if (!yaUnSignal) return;


  strcpy(filename,"*.*");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (TFileSaveDialog(Parent,FileData,0).DoExecute() == IDOK) {
   strcpy(filename,FileData.FileName);
   if (droit_ecrire_message(filename)) {
	  souris (XC_watch);
     itereF0(en_cours, filename, typeLoc);
     souris (XC_top_left_arrow);
   }
  }
}

static infoFile files // domaine de recherche
	   = { "*.*","*.*","*.*",0,0,0 };
void Wtop::CMCorpusF0()
{
  DIR *repertoire;   /* Repertoire dans lequel on
                        cherche un fichier */
  int i, j;
  BOOL ret=TRUE;
  STRING ch;
  struct dirent *fichier;
  struct infoFile speechFile, *curfile, en_coursOld;
  long nbEcht = 0;  // nombre de secteurs du fichier resultat
  char proc[]="exploreCorpusEtiq()";

  en_coursOld = en_cours;
  if (getSpeechFormat(PTWtop, files)!=IDOK) return;
      /* on parcourt tous les fichiers du repertoire d'etiquetage */
  curfile = &speechFile;  

  // On verifie que le repertoire parole existe

  strcpy(ch,files.nom);
  splitmakedir(ch);
  if (!strlen(ch)) {
    affiche_erreur (ml(7306,"Repertoire du corpus parole inexistant"));
    return;
  }
  if (!strlen(ch) || (repertoire=opendir(ch)) == NULL) {
    affiche_erreur (ml(7306,"Repertoire du corpus parole inexistant"));
    return;
  }

  noPitchMessage = true;
  while ((fichier = readdir(repertoire)) != NULL) {
    char nom[_MAX_PATH];
    char ext[_MAX_EXT];
    STRING drive, dir, name;
    x_makepath(nom, files.nom, files.nom, fichier->d_name, fichier->d_name);
    _splitpath(nom, drive, dir, name, ext);
    _splitpath(files.nom,NULL,NULL,NULL,ch);
    if (strlen(name) && stricmp(ext, ch) == 0) {
      STRING s;
      strcat(strcat(strcpy(s, drive), dir), name);
      strcat(s, ".pit");
      strcpy(curfile->nom, nom);
      curfile->fmt_son = files.fmt_son;
      init_infoFile_read(curfile);
      en_cours.frequence = speechFile.frequence;
      souris (XC_watch);
      itereF0(speechFile, s, typeLoc);
      souris (XC_top_left_arrow);
    }
  }
  closedir(repertoire);
  en_cours = en_coursOld;
  noPitchMessage = false;
}

static STRING nomFicPar = "\0", nomFicPitch;
static INTbuf tabPitchLu(1, "tabPitchLu pitch values read from file"),
              datPitchLu(1, "instants of pitch values read from file");
static struct ctrl_temps ctPitchLu = {0,0,0,0,0,0,0,0};

// Affichage du pitch quand il existe (lu dans un fichier)
void Wtop::CMDispPitchValues()
{
  int echt;
  float xcote, ycote;

  // Display pitch values
  MgiHue (ROUGE, wspectro->wsona);
  // constants to use affMeilPitch correctly
  pm.fenPitch = (int) (en_cours.frequence * (float) FENPITCH / 1000.);
  pm.depEcht = en_cours.frequence * (DEPPITCH/1000.);

  xcote = wspectro->wsona->DXtoLX(3) - wspectro->wsona->DXtoLX(0);
  ycote = wspectro->wsona->DYtoLY(0) - wspectro->wsona->DYtoLY(3);

  for (echt = enEcht((((int) enMilli(ct.d_a)) / ((int)DEPPITCH)) * DEPPITCH) + pm.fenPitch / 2;
       echt < ct.f_a; echt += pm.depEcht) {
        	int f0 = pm.getReadf0(echt);
         if (f0 > 0) {
    		  float x1,y1;
    			x1 = echt - xcote/2.;
    			y1 = max(f0 / pm.F0Maxscale() * NB_CANAUX  - ycote/2., 0.);
    			snorbox(x1, y1, x1 + xcote, y1 + ycote, wspectro->wsona);
   		}
  }
  ::InvalidateRect(wspectro->wsona->HWindow, NULL, TRUE);
  active(MENUDispPitchValues);
}

// Read a set of FO values.   Returns true if reading is ok.
static bool readPitchValues(
  int echt) // Sample where reading starts
{
  FILE *f;
  int date, f0, nbLus;
  STRING test;

  if (!nomFicPitch || !strlen(nomFicPitch)) return false;
  f = fopen(nomFicPitch, "r");
  if (strcmp(nomFicPar, en_cours.nom)) {
    fclose(f);
    return false;
  }

  fscanf(f,"%s", test);
  if (strncmp(".pit",test,4)) {
    fclose(f);
    return false;
  }

  // reading the pitch values and instants of pitch values (in samples)
  do fscanf(f,"%d %d\n", &date, &f0);
  while (!feof(f) && enEcht(date) < echt);

  nbLus = 0;
  if (!feof(f)) {
    int status = 1;
    ctPitchLu.d_e = echt;
    ctPitchLu.f_e = echt + MAXECHTECR;

    tabPitchLu.newsize(1);
    datPitchLu.newsize(1);
    tabPitchLu[0] = f0;
    datPitchLu[0] = enEcht(date);
    nbLus = 1;
    while (!(feof(f) || status == EOF || !status) && enEcht(date) < ctPitchLu.f_e) {
      status = fscanf(f,"%d %d\n", &date, &f0);
      if (nbLus >= tabPitchLu.size()) {
        tabPitchLu.newsize(tabPitchLu.size()+100);
        datPitchLu.newsize(tabPitchLu.size()+100);
      }
      tabPitchLu[nbLus] = f0;
      datPitchLu[nbLus] = enEcht(date);
      nbLus++;
    }
  }
  else ctPitchLu.d_e = ctPitchLu.f_e = 0;

  tabPitchLu.newsize(nbLus);
  datPitchLu.newsize(nbLus);
  fclose(f);
  return((nbLus) ? true : false);
}

//
// Initializes F0 for the curent file
void pitchm::currentF0()
{
  // if the pitch is not available for this file compute it.
  if (!nomFicPitch || !strlen(nomFicPitch) || strcmp(nomFicPar, en_cours.nom)) {
    STRING ficpitch;
    sprintf(ficpitch, "%s\\temp\\current.pit", WinSnooriDirectory);
    if (droit_ecrire(ficpitch)) {
      souris (XC_watch);
      itereF0(en_cours, ficpitch, typeLoc);
      souris (XC_top_left_arrow);
    }
    if (is_readable(ficpitch) == 1) {
      strcpy(nomFicPar, en_cours.nom);
      strcpy(nomFicPitch, ficpitch);
    }
    ctPitchLu.d_e = 0;
    ctPitchLu.f_e = 0;
  }
}

static int lasti = 0, total = 0,
           diffDate = 100; // 100 time samples
// get a F0 value read in a pitch file
unsigned int pitchm::getReadf0(long echt)
{
  char test[LGLIGNEPITCH+1];
  int i, sens;

  if (!nomFicPitch || !strlen(nomFicPitch)) return 0;
  if (strcmp(nomFicPar, en_cours.nom)) return 0;

  // S'il y a eu un deplacement il faut relire les valeurs de pitch.
  if (!dans(echt, ctPitchLu.d_e, ctPitchLu.f_e)) {
    if (!readPitchValues(echt)) return 0;
    total++;
    lasti = 0;
    diffDate = 5 * (datPitchLu[1] - datPitchLu[0]);
  }

  i = lasti;
  if (i < tabPitchLu.size() - 1) sens = (abs(datPitchLu[i + 1] - echt) < abs(datPitchLu[i] - echt)) ? 1 : -1;
  else sens = -1;

  while (((sens > 0) ? i < tabPitchLu.size() - 1 : i) && abs(datPitchLu[i + sens] - echt) < abs(datPitchLu[i] - echt)) {
    i += sens;
  }
  lasti = i;
  if (abs(echt - datPitchLu[i]) < diffDate) return(tabPitchLu[i]);
  // if echt is beyond the end of file its F0 values has not been found and must set to zero.
  else return(0);
}




// Ouverture d'un fichier de pitch
//extern bool yaUneCourbeDePitch;
void Wtop::CMOpenPitchFile()
{
  STRING filename;
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "*.*",0,"","");

  if (!yaUnSignal) return;

  strcpy(filename,"*.*");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (TFileOpenDialog(Parent,FileData,0).DoExecute() == IDOK) {
    strcpy(filename,FileData.FileName);

  	 if (is_readable(filename) == 1) {
      strcpy(nomFicPar, en_cours.nom);
      strcpy(nomFicPitch, filename);
      Wtop::CMDispPitchValues();
      //yaUneCourbeDePitch = VRAI;
    }
    else affiche_erreur(ml(6930,"Fichier de pitch innaccessible"));
  }
}

// FLAGGG


/////////////////////////////////////////////
// fentre WchooseSpeaker
// Choix du type de locuteur
class TWchooseSpeaker : public TDialog
{   TRadioButton *PTany;
    TRadioButton *PTfemale;
    TRadioButton *PTmale;
  public:
    TWchooseSpeaker(PTWindowsObject AParent, LPSTR name);
    virtual void SetupWindow();
    virtual bool CanClose();
};

TWchooseSpeaker::TWchooseSpeaker(PTWindowsObject AParent, LPSTR name)
	:TDialog(AParent,name)
{
  // initialiser le contenu des contrles
  //   avec les variables correspondantes
  PTany=new TRadioButton(this,IDC_RADIOBUTTON7,0);
  PTfemale=new TRadioButton(this,IDC_RADIOBUTTON8,0);
  PTmale=new TRadioButton(this,IDC_RADIOBUTTON9,0);
}
void TWchooseSpeaker::SetupWindow()
{
  TDialog::SetupWindow();
  SetCaption(ml(6910,"type de locuteur"));
  switch (typeLoc) {
    case -1 :
    	PTany->SetCheck(BF_CHECKED);
      break;
    case 0 :
    	PTfemale->SetCheck(BF_CHECKED);
      break;
    case 1 :
    	PTmale->SetCheck(BF_CHECKED);
      break;
  }
  ::SetWindowText(GetItemHandle(IDC_GROUPBOX1),ml(6920,"choisir entre"));
  ::SetWindowText(GetItemHandle(IDCANCEL),ml(6422,"&Annuler"));
}

bool TWchooseSpeaker::CanClose()
{
  if (PTany->GetCheck()==BF_CHECKED) typeLoc = -1;
  if (PTfemale->GetCheck()==BF_CHECKED) typeLoc = 0;
  if (PTmale->GetCheck()==BF_CHECKED) typeLoc = 1;

  switch (typeLoc) {
     case -1 : case 0 :
     	 pm.nbGradReq = 5;
       break;
     case 1 : default :
       pm.nbGradReq = 4;
       break;
   }
  return true;
}

void Wtop::CMTypeOfSpeaker()
{
  PTWtop->GetApplication()->ExecDialog(
	   new TWchooseSpeaker(PTWtop,"WUpdateSpeaker"));
}



