/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier pmpeigne.cpp
//

#include "common.h"
#pragma hdrstop

#include <math.h>
#include <stdio.h>

#include "spectro.h"
#include "pitchm.h"

static FLOATbuf peigne(1 
		,"peigne near correlPeigne()");
		// les amplitudes des dents du peigne 
static FLOATbuf somHarm(1 
		,"somHarm near correlPeigne()");
		// tableau des sommes des harmoniques du peigne 

/*
 * Correlation entre le spectre preleve et le peigne.
 */
void correlPeigne(
	FLOATbuf& sPrel_,	// spectre preleve
	FLOATbuf& correl_,	// correlation 
	int fmax,   // frequence maximale du spectre preleve
	        // (pour n'etudier qu'une partie du spectre) 
	int pitchm, // domaine de valeurs 
	int pitchM, //     de pitch a "peigner" 
	float& corTot) // correlation totale 
{ int f0;
  float correlTot;
  float *sPrel=(float *)&sPrel_[0];
  float *correl=(float *)&correl_[0];

  for (f0 = pitchm, correlTot = 0.; f0 <= pitchM; f0++)
  { int i;
    for (i = 1, correl[f0] = 0.; i < fmax / f0; i++) 
      correl[f0] +=  peigne[i] * sPrel[i * f0];
    correlTot += correl[f0];
  }
  
  corTot = correlTot; 
}

/*
 * Calcul d'une correlation pour une valeur de f0 donnee.
//
//   la correlation pour f0 sur le spectre sPrel est la
//   somme des sPrel[i] pour i=1,2,... , c'est--dire
//   pour f0 et ses harmoniques
//     cette somme est divise (normalise) avant retour 
 */
float correlf0( 
	FLOATbuf& sPrel,	// le spectre preleve
	int fmax,	// frequence maximale du spectre preleve
	        // (pour n'etudier qu'une partie du spectre) 
	int f0)
{ int k, n;
  float e,ret;
  
  for (k = f0, e = 0., n = 0; k < fmax; k += f0) {
    e += sPrel[k];
    n++;
  }
  ret= (!n) ? 0. : e / max(somHarm[n],EPSILON);
  return ret;
}

/*
 * Initialisation du peigne.
 */
void initPeigne( 
	int n,	 // nombre de dents du peigne
	float k) // exposant des dents du peigne 
{ int i;

  peigne.newsize(n);
  somHarm.newsize(n);
  for (i = 0; i < n; 
       i++) somHarm[i] = 0.;
  for (i = 1; i < n; 
       i++)
  { peigne[i] = pow((double) i, (double)(-1. / k));
    somHarm[i] = somHarm[i - 1] + peigne[i];
  } 
  peigne[0] = 1.;	/* a tout hasard */

  if (debug==DEBUG_PM)
    for (i = 0; i < n; 
         i++)
      fprintf(ficpm,"peigne[%d]=%.3g somHarm[%d]=%.3g\n"
                     ,i,peigne[i],i,somHarm[i]);
}






