/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier pmpics.cpp
//

#include "common.h"
#pragma hdrstop

#include <math.h>
#include <stdio.h>
#include "snoori.h"

#include "spectro.h"
#include "pitchm.h"
#include "regres.h"
#include "signal.h"

/*
 * Construction d'un spectre preleve entre 0 et 2000 Hz ne contenant 
 * que les pics du spectre, suffisamment eleves:
 * > max du spectre entre 0 et NBPTPREL Hz - 30 dB
 * et > seuil du silence (33 dB).
 * Le spectre preleve est defini pour chaque Hz entre 0 et NBPTPREL Hz.
 */
void pics(
	FLOATbuf& s_,	    // spectre de depart
	FLOATbuf& spectrePrel_, // spectre prlev
	int n,		    // nombre de points
		     // du spectre initial a considerer
	int , //fmax,   // frequence jusqu'a laquelle
	             //on somme l'energie des pics 
	float& enerPics, // energie des pics jusqu'a fmax 
	int affich)	 // VRAI pour affichage de controle 
{ float df = en_cours.frequence / ((float)pm.ordreTF),
  	ener = 0.;
  int i;
  float *s=(float *)&s_[0];
  float *spectrePrel=(float *)&spectrePrel_[0];

  initRegres4();
  initRegres6();

  for (i = 0; i < NBPTPREL; i++)
    spectrePrel[i] = -INFINI_F/2.;
  for (i = 0; i < n-2; i++)
  { if (  (s[i] <= s[i+1])
	&&(s[i+1] >= s[i+2])
	&&(s[i+1] > SILENCE) )
    { int ig, id;  // indices des points encadrant
                   //   le maximum 
      
      id = ( (i<n-3) && (s[i+2]>s[i+3]) )
           ? i + 3 : i + 2;
      ig = ( (i>0) && (s[i]>s[i-1]) ) ? i-1 : i;

      // on regarde si en decalant d'un point vers
      //   la droite les 4 points sont
      //   mieux repartis autour du maximum 
      if (  (ig-id==4)
          &&(fabs(s[id]-s[ig+1])>fabs(s[ig-1]-s[id])) )
	ig = ig + 1;
      if (affich)
      { MgiHue(JAUNE,wspectro->wsona);
	snoiwidth(3);
	snortl(ig * df, dBMIN+5., id * df, dBMIN+5.
	       , wspectro->wsona);
      }
      // on prend en compte le maximum s'il
      //   correspond a au moins 4 points 
      if (id-ig>=3)
      {	float a, b, c;	// coefficients de la parabole 
	int maxBienPlace;

	regresPar4(&a, &b, &c, (float *)&s[ig]);
	// on verifie que le maximum de la parabole
	//   est bien place a proximite du pic
	//   du spectre. Si ce n'est pas le cas,
	//   on considere que ce pic n'est pas
	//   interessant. 
	maxBienPlace = fabs(i+1-(ig-b/2/a)) <= 1.;
	// Ca n'a pas marche avec 4 points. On essaie
	//   avec 6 pour sauver les beaux harmoniques.
	if (  !maxBienPlace
	    &&(ig>0)
	    &&(s[ig-1]<s[ig])
	    &&(s[id+1]>s[id]) )
	{ ig -= 1;
	  regresPar6(&a, &b, &c, (float *)&s[ig]);
	  maxBienPlace = fabs(i+1-(ig-b/2/a)) <= 1.;
	}
	if (maxBienPlace)
	{ int k, xHzmaxi;

	  xHzmaxi = IROUND((ig-b/2/a) * df);
	  if (xHzmaxi - 25 < FREQPICVOIS)
	  { float p = - b*b/4/a + c;
	    if (p > SILENCE) ener += (p - SILENCE);
	  } 
	  // On calcule le spectre preleve 25 Hz de
	  //   chaque cote du maximum. 
	  for (k = max(0,xHzmaxi-25);
	       k < min(NBPTPREL,xHzmaxi+25);
	       k++)
	  { float x = k / df - ig,
	    y = (a * x + b) * x + c;
	    spectrePrel[k] = (y > SILENCE)
	                     ? (y - SILENCE) : 0.; 
	    if (affich && (k>xHzmaxi-25) )
	    { snol(k-1., spectrePrel[k-1] + SILENCE - 5.
		  ,(float)k, spectrePrel[k] + SILENCE - 5.
		  , wspectro->wsona);
	    }
	  }
	}
	i = id;
      } 
    }
  }
  for (i = 0; i < NBPTPREL; i++)
    if (spectrePrel[i] <=  -INFINI_F/4.)
      spectrePrel[i] = 0.;

  enerPics = ener;
  return;
}
