/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier pmpitchm.cpp
//

#include "common.h"
#pragma hdrstop

#include <math.h>
#include <stdio.h>
#include <malloc.h>

#include "spectro.h"
#include "pitchm.h"
#include "signal.h"
#include "spectre.h"
#include "outils.h"

FLOATbuf enerPeriod(1,"enerPeriod in pmpitchm.cpp"); // Energie calculee
                          // sur le spectro bande large
int pitchBrutMedian; // pitch brut median
pdynVec *tabLisGlob; // tableau de lissage du pitch
       //=(pdynVec *) NULL; 
       // tabLisGlob[i][0]  candidat sourd pour le pitch
       // .[1] .[2] .[3]  candidats voiss


void init_pdynPoint(pdynPoint *p)
{     p->dcum  = 0.; //INFINI_F/10.;
      p->dir   = 0.; //INFINI_F/10.;
      p->antDat=0;
      p->antTyp=0; //-1;
      p->ignore=0;
      p->bonus =0; //-INFINI_F/10.;
      p->cvois =0.;
      p->f0    =0.;
}
void init_pdynVec(pdynVec *p)
{ int j;
  for (j=0;j<4;j++) init_pdynPoint(&(*p)[j]);
}

/*
 * Calcul du pitch avec l'algorithme a peigne propose par P. Martin.
 */
void pitchBrutMartin(
	struct paramPitch& param) // parametres de l'extraction du pitch
{
  long e;
  int i, j,
      nbSourd,			/* nombre de trames consecutivement manisfestement non voisees */
      pitchMoyen;
  float k = param.k,		/* parametre d'amplitude des dents du peigne */
	maxVoisement,		/* valeur maximale du critere de voisement */
  	medVoisement,		/* valeur mediane du critere de voisement */
  	maxEnerVois,		/* energie maximale dans la bande de voisement */
  	rapDep, decal;		/* pour considerer le bon indice dans le critere de voisement */
  FLOATbuf energie(1,"energie in pitchBrutMartin()");
	       // energie dans la bande de frequence
	       //   de voisement
  int pitchMartin[3]; // = TABPITCHNULL; // valeurs de F0 (3 valeurs candidates)

  if (debug==DEBUG_PM)
    fprintf(ficpm,"dbut pitchBrutMartin()\n");

  rapDep = ((float) pm.depEcht) / pm.tabl_pitch.depl;
  decal = ((float) pm.fenPitch - FENFFT256) / 2. / pm.tabl_pitch.depl;

  energie.newsize((pm.derEcht - pm.premEcht + 1 + 2 * pm.tabl_pitch.depl) / pm.tabl_pitch.depl);

  ener(pm.tabl_pitch, energie, pm.premEcht, pm.derEcht, 0., param.freqVois);

  enerPeriod.newsize(pm.nbPitch);

  initPeigne(NBPTPREL / param.f0minCorrel + 1, k);

  /* Initialisation du calcul de spectre */
  initSpecFft(pm.fenPitch, pm.ordreTF);

  for (e = pm.premEcht, i = 0, nbSourd = 0; e < pm.derEcht - pm.fenPitch; e+= pm.depEcht, i++) {
    float enerVois[3], coefVois[3],
	       energieFen = energie[IROUND(((float)i) * rapDep + decal)];
    int j;

    if (energieFen < 1.) nbSourd++;
    else nbSourd = 0;

    /* si plusieurs trames consecutives sont non voisees, on ne calcule pas le pitch. */
    if (nbSourd > 6)
    {
      pitchMartin[0] = pitchMartin[1] = pitchMartin[2] = -1;
    }
    else calUnPitch(e, pm.fenPitch, pitchMartin, enerVois, coefVois, param);

    // initialisation des points de programmation
    //   dynamique
    // luxueux peut-tre et redondant avec ce qui suit
    init_pdynVec(&tabLisGlob[i]);      

    // le point de programmation dynamique en
    //   considerant que le point est non voise 
    tabLisGlob[i][0].antDat = -1;

    for (j = 0; j < 3; j++) {
      // creation d'un point de programmation
      //   dynamique voise 
      tabLisGlob[i][j + 1].antDat = -1;
      tabLisGlob[i][j + 1].f0 = pitchMartin[j];
		if (tabLisGlob[i][j + 1].f0 > 1) {
        tabLisGlob[i][j + 1].bonus = enerVois[j];
        tabLisGlob[i][j + 1].cvois = coefVois[j];
      }
      else {
        tabLisGlob[i][j + 1].bonus = 0.;
	     tabLisGlob[i][j + 1].cvois = 0.;
      }
    }
  }

  pm.nbPitch = i; // utile si arrt au clavier
  freeSpecFft();

 { INTbuf histo(101,"histo in pitchBrutMartin()");
    FLOATbuf voise(1,"voise in pitchBrutMartin()");
	       // voisement brut (correlation + energie)

    voise.newsize(pm.nbPitch);
    for (i = 0; i < pm.nbPitch; i++)
      voise[i] = tabLisGlob[i][1].bonus;

    histoVois(voise, 0, pm.nbPitch - 1, histo);
    /* Recherche du voisement maximal (ou presque car on retient la valeur au-dessus de
       laquelle il y a x% des valeurs de voisement observees. */
    maxVoisement = vsupHisto(histo, pm.nbPitch, 0.10);
    medVoisement = vsupHisto(histo, pm.nbPitch, 0.6);
    if (debug==DEBUG_PM)
    { fprintf(ficpm,"maxVoisement=%.3g\n",maxVoisement);
      fprintf(ficpm,"medVoisement=%.3g\n",medVoisement);
    }
  }

  // On recupere les valeurs de F0 qui semblent
  //   correctes a premiere vue,
  //   c-a-d avec un bonus de voisement suffisamment fort.
  { INTarray f0Estim(1,"f0Estim in pitchBrutMartin()");
              // estimation grossiere du pitch

    f0Estim.newsize(pm.nbPitch);

    for (i = 0, j = 0, pitchMoyen = 0; i < pm.nbPitch; i++)
      if (tabLisGlob[i][1].bonus > 0.85 * maxVoisement)
      { f0Estim[j++] = tabLisGlob[i][1].f0;
        pitchMoyen +=  tabLisGlob[i][1].f0;
      }
    pitchBrutMedian = media_i(f0Estim, 0, j - 1
			   , param.f0min + 2
			   , param.f0max - 2, 2);
    if (debug==DEBUG_PM) 
    { char ch[80];
      for (i=0;i<j;i++)
      { sprintf(ch,"f0Estim %d : %d",i,f0Estim[i]);
        fprintf(ficpm,"%s\n",ch);
      }
      sprintf(ch,"Le pitch median est %d"
	      , pitchBrutMedian);
      fprintf(ficpm,"%s\n",ch);
      if (j)
      { sprintf(ch,"Le pitch moyen  est %d"
                     , (int)(pitchMoyen / j));
        fprintf(ficpm,"%s\n",ch);
      }
    }
  }

  { INTbuf histoEner(101,"histoEner in pitchBrutMartin()");
    enerVois(pm_space
	     ,0,0 
             , enerPeriod, pitchBrutMedian);
    histoVois(enerPeriod, 0, pm.nbPitch - 1, histoEner); 
    maxEnerVois = vsupHisto(histoEner //, 101
                           , pm.nbPitch, 0.10);
    if (debug==DEBUG_PM)
    { fprintf(ficpm,"maxEnerVois=%.3g\n",maxEnerVois);
    }
  }

  /* Reglage des fonctions de bonus et calcul des boni */
    reglageBonus(maxVoisement, medVoisement, maxEnerVois);

  for ( i = 0;
       (i<pm.nbPitch); 
       i++)
  {
    float en;
    if (debug==DEBUG_PM) fflush(ficpm);
    if (debug==DEBUG_PM)
      fprintf(ficpm,"calcul bonus  i=%d\n",i);


    if (   (enerPeriod[i]<0.5)
	|| (energie[IROUND(((float)i) * rapDep + decal)]
	    <0.5) ) en = 0.;
    else en = enerPeriod[i];

    tabLisGlob[i][0].bonus = min(min(bonusSourd(tabLisGlob[i][1].bonus, tabLisGlob[i][1].cvois, en),
				 bonusSourd(tabLisGlob[i][2].bonus, tabLisGlob[i][2].cvois, en)),
				 bonusSourd(tabLisGlob[i][3].bonus, tabLisGlob[i][3].cvois, en));
    tabLisGlob[i][1].bonus = bonusVoise(tabLisGlob[i][1].bonus, tabLisGlob[i][1].cvois, en);
    tabLisGlob[i][2].bonus = bonusVoise(tabLisGlob[i][2].bonus, tabLisGlob[i][2].cvois, en);
    tabLisGlob[i][3].bonus = bonusVoise(tabLisGlob[i][3].bonus, tabLisGlob[i][3].cvois, en);
  }

  if (debug==DEBUG_PM)
    fprintf(ficpm,"fin pitchBrutMartin()\n");
}


/*
 * Calcul d'une valeur de pitch
 */
void calUnPitch( 
	long e,	 // le premier echantillon
		 // de la fenetre de calcul du pitch
                 // en relatif dans pm_space
	long ,//fenPitch, // le nombre d'echantillons
	              // de la fenetre 
	int *f0,  // les trois candidats
	          //     le dernier ne sert que si les
		  //     deux premiers sont
		  //	 ecartes de moins de 70 Hz 
	float *enerVois,  // les energies pour
		    // les 3 candidats dans la
		    // bande de voisement 
	float *coefVois,  // les coefficients
		    // de voisement pour les 3
		    // candidats 
	struct paramPitch& param) // les parametres du pitch
{ FLOATbuf spectre2_4_(ORDRETFMAX/2+1
	       ,"spectre2_4 in calUnPitch()") // spectre
	,pics2_4_(NBPTPREL  // spectre preleve,
			  // ne contenant que les pics.
                          // 1 point de pics2_4_ par hertz
	       ,"pics2_4 in calUnPitch()")
	,correl2_4_(NBPTPREL     // tableau de correlation 
			  // 1 point de correl2_4_ par hertz
	       ,"correl2_4 in calUnPitch()");
  float *correl2_4=(float *)&correl2_4_[0];
  float	correl[3],			/* les correlations correspondant aux 3 candidats */
  	maxiF0Sup,			/* le maximum de correlation entre F0 et la frequence maximale */
  	enPics,				/* energie totale des pics du spectre preleve */
  	cTot;				/* somme de la correlation entre param.f0min et f0max Hz */
  int f0bis,
      jg, jd, j;	/* indices pour la recherche de F0 */


  if (debug==DEBUG_PM)
  { fprintf(ficpm,"calUnPitch[%ld]\n",e);
    fflush(ficpm);
  }
  // comme ce point ne sert a priori pas on l'initialise
  //   a des valeurs par defaut 
  f0[2]=-1;  
  correl[2] = 0.;

  for (j = 0; j < NBPTPREL; j++) correl2_4[j]=0.;
  specFft(pm_space,e, spectre2_4_, true, true);
  pics(spectre2_4_
      , pics2_4_, (int)((2000./en_cours.frequence)*pm.ordreTF)
      , param.fmax, enPics, FAUX);
  correlPeigne(pics2_4_, correl2_4_
      , param.fmax, param.f0minCorrel
      , param.f0max, cTot);

  // d'abord le 1er candidat
  //   le plus haut pic dans l'absolu
  // on recherche F0 en localisant la maximum de la
  //   correlation 
  correl[0] = max_fl(correl2_4, param.f0min, param.f0max, &f0[0]); //f01);

  // puis le 2me candidat
  //   le plus fort autre que le 1er candidat
  for (j = f0[0]-1; 
       (j>max(param.f0min,f0[0]-70))
       && (correl2_4[j]<=correl2_4[j+1]);
       j--);
  jg = max(param.f0min, min(j, f0[0]-8)); 
  for (j = f0[0]+1;
       (j<min(param.f0max,f0[0]+70)) 
       && (correl2_4[j]<=correl2_4[j-1]);
       j++);
  jd = min(param.f0max, max(j, f0[0]+8)); 
  correl[1]
        = max_fl(correl2_4
		    , param.f0min, jg
		    , &f0[1]); 
  maxiF0Sup = max_fl(correl2_4
		    , jd, param.f0max
		    , &f0bis);
  if (maxiF0Sup > correl[1])  
  { f0[1]=f0bis; 
    correl[1] = maxiF0Sup;
  }

  // puis le 3me candidat
  //   uniquement si les deux premiers sont
  //     proches l'un de l'autre (<70)
  // on recherche assez loin du 1er candidat 
  if (abs(f0[0] - f0[1]) < 70)
  {
    correl[2] = max_fl(correl2_4
		      , param.f0min, max(f0[0]-70,param.f0min)
		      , &f0[2]); 
    maxiF0Sup = max_fl(correl2_4
		      , min(f0[0]+70,param.f0max), param.f0max
		      , &f0bis);
    if (maxiF0Sup > correl[2])
    { f0[2]=f0bis;
      correl[2] = maxiF0Sup;
    }
  }

  /* Energie dans la bande qui sert a determiner le voisement */
  for (j = 0; j < 3; j++)
  { if (  (f0[j]>1) 
        &&(enPics>EPSILON) )
    { enerVois[j] = correlf0(pics2_4_, (int) param.freqVois, f0[j]);
      coefVois[j] = coefVoisCorrel(correl2_4_, f0[j]);
    }
    else
    { enerVois[j] = 0.;
      coefVois[j] = 0.;
    }
  }
  if (debug==DEBUG_PM)
  { fprintf(ficpm,"... f0[0..2] %d %d %d\n"
             ,f0[0],f0[1],f0[2]);
    fprintf(ficpm,"... enerVois[0..2] %.3g %.3g %.3g\n"
             ,enerVois[0],enerVois[1],enerVois[2]);
    fprintf(ficpm,"... coefVois[0..2] %.3g %.3g %.3g\n"
	     ,coefVois[0],coefVois[1],coefVois[2]);
    fflush(ficpm);
  }
}


