/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

extern int typeScale;

class PolyPointsKla : public PolyPoints {
  public:
  virtual void setDefaultValuesKla(int kpar, int focus) = 0;
  virtual void translateKla(int mj, int incr) = 0;
  virtual PolyPointsKla *subPPKla(int deb, int fin) = 0;
};
// end class PolyPointsKlatt

template <class TP, class POINT> class PolyPointsMD : public PolyPointsKla {
   public:
      TP pts;
      
      PolyPointsMD() {
         nbPt = pts.ni;
      }

      PolyPointsMD(TP &t) {
         pts.newsize(t.ni);
         for (int i = 0; i < t.ni; i++) pts[i] = t[i];
         nbPt = pts.ni;
      }

      PolyPointsMD(TabPoint<Point2P> &t, int focus) {
        int dim = DIMPNP(POINT);
        pts.newsize(t.ni);
        for (int i = 0; i < t.ni; i++) {
          pts[i].date = t[i].date;
          for (int j = 0; j < dim; j++)
            if (j == focus) pts[i].y[focus] = t[i].y[0];
            else pts[i].y[j] = -INFINI_F;
		  }
        nbPt = pts.ni;
       }

      ~PolyPointsMD() {
         nbPt = 0;
       }
       bool estSelec() { return selec; }
       bool avant(PolyPoints *s) { return debut() < s->debut(); }
       void selectionne(mode) { selec = mode;}
       void aff(int focus, YFCT yecran, int mode, int typeAff, COLORREF couleur, int ligne) {
       	pts.aff(focus, yecran, mode, typeAff, couleur, ligne);
       }
       int debut() { return pts[0].date;}    // date du debut du polypoints en millisecondes
       int fin() { return pts[pts.ni -1].date;}
       void setDefaultValues(int nfields, int att[], float val[]) {
         pts.setDefaultValues(nfields, att, val);
       }
       void distance(int date, float y, int focus, float correc, float seuil,
       					float *distPoint, float *distCourbe, int *indice) {
         pts.distance(date, y, focus, correc, seuil, distPoint, distCourbe, indice);
       }
       int date(int i) {return pts[i].date;}
       float y(int i, int focus) {return pts[i].y[focus];}
       int *pdate(int i) {return &pts[i].date;}
       float *py(int i, int focus) {return &(pts[i].y[focus]);};
		 void echantillonne(int depFor) {pts.echantillonnePP(depFor); nbPt = pts.size();}
       void removeFirstPoint() {pts.removeFirstPointPP(); nbPt = pts.size();}
       void removeLastPoint() {pts.removeLastPointPP(); nbPt = pts.size();}
       void translate(int focus, float incr, float vmin, float vmax, float vmul){
       	pts.translate(focus, incr, vmin, vmax, vmul);
       }
       void push(int date, float t[]) {
         pts.push(POINT(date, t));
         nbPt = pts.size();
       }
		 void subpp(int n) { pts.newsize(n); nbPt = pts.ni; };
       PolyPointsMD *subPPKla(int deb, int fin) {
         if (deb > fin || deb >= nbPt) return((PolyPointsMD *) NULL);
         PolyPointsMD<TP, POINT >  *npp = new PolyPointsMD<TP, POINT >;
         int ideb = max(0, deb), ifin = min(pts.ni - 1, fin);
         npp->pts.newsize(ifin - ideb + 1);
         for (int i = ideb; i <= ifin; i++) npp->pts[i - ideb] = pts[i];
         npp->nbPt = npp->pts.ni;
         return npp;
       }
       PolyPointsMD *subPP(int deb, int fin) {
         if (deb > fin || deb >= nbPt) return((PolyPointsMD *) NULL);
         PolyPointsMD<TP, POINT >  *npp = new PolyPointsMD<TP, POINT >;
         int ideb = max(0, deb), ifin = min(pts.ni - 1, fin);
         npp->pts.newsize(ifin - ideb + 1);
         for (int i = ideb; i <= ifin; i++) npp->pts[i - ideb] = pts[i];
         npp->nbPt = npp->pts.ni;
         return npp;
       }
       int indPP(int date) { int j;
         for (j = 0; j < pts.ni && date != pts[j].date; j++);
         return((j < pts.ni) ? j : -1);
       }
       void spline(int focus) { pts.spline(focus); }
       void translateKla(int mj, int incr){
         int focus = mineur(mj, typeScale);
  	      float vmin, vmax, vmul;
   	   switch (typeScale) {
   	     case SCALEFr:
    	       vmin = 50.; vmax = 7500; vmul = 10.;
    	       break;
           case SCALEBwC: case SCALEBwP:
    	  		 vmin = 10.; vmax = 400.; vmul = 2.;
      		 break;
           case SCALEDBC: case SCALEDBP:
             vmin = -120.; vmax = 120.; vmul = 1.;
             break;
           case SCALEF0:
             vmin = 0.; vmax = 800.; vmul = 2.;
             break;
          case SCALEDNb:
             vmin = 0.; vmax = 100.; vmul = 1.;
             break;
          }
          pts.translate(focus, (float) incr, vmin, vmax, vmul);
        }
        void setDefaultValuesKla(int kpar, int focus)
        {
          extern klatt_frame_t defaultFrame;
  			 int att[3], formant;
          float val[3];

          switch (kpar) {
    			case F0hz10_:
      		  if (focus == FREQUENCE) {
                formant = -1;  
        			 att[0] = AMPLITUDEF0P; val[0] = defaultFrame.AVpdb;
        			 att[1] = AMPLITUDE; val[1] = defaultFrame.AVdb;
        			 pts.setDefaultValues(2, att, val);
      		  }
        		  break;
    			case F1hz_: formant = 0;
              break;
    			case F2hz_: formant = 1;
        		  break;
    			case F3hz_: formant = 2;
              break;
    			case F4hz_: formant = 3;
              break;
    			case F5hz_: formant = 4;
              break;
    			case F6hz_: formant = 5;
        		  break;
            case F1Nhz_: formant = 6;
              break;
    			case F2Nhz_: formant = 7;
        		  break;
    			case F3Nhz_: formant = 8;
              break;

    			case FNZhz_:
       		  if (focus == FREQUENCE) {
         		 att[0] = LARGBANDECFNZ; val[0] = defaultFrame.BNZhz;
                pts.setDefaultValues(1, att, val);
              }
              break;
    		   case FNPhz_:
              if (focus == FREQUENCE) {
                att[0] = LARGBANDEC; val[0] = defaultFrame.BNPhz;
                att[1] = AMPLITUDE; val[1] = defaultFrame.ANP;
          		 pts.setDefaultValues(2, att, val);
              }
              break;
    			}

    			if (formant != -1 && focus == FREQUENCE) {
              if (kpar == F1Nhz_ || kpar == F2Nhz_ || kpar == F3Nhz_) {
      			 att[0] = LARGBANDEC; val[0] = 75.;
      			 att[1] = LARGBANDEP; val[1] = 75.;
              }
      		  else {
              	 att[0] = LARGBANDEC; val[0] = 50. + formant * formant * 10.;
      			 att[1] = LARGBANDEP; val[1] = 50. + formant * formant * 10.;
              }
      			att[2] = AMPLITUDE; val[2] = 60.;
      			pts.setDefaultValues(3, att, val);
    			}
        }
        private :
        bool selec;  // PolyPoints selectionne ? VRAI / FAUX
}; //end class PolyPointsMD


