/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier recher_a.cpp
//

#include "common.h"
#pragma hdrstop

#include <dirent.h>

#include "notetiq.h"
#include "cnotetiq.h"
#include "snoori.h"
#include "signal.h"
#include "files.h"
#include "outils.h"
#include "audio.h"
#include "format_f.h"

#define LG_PAUSE	1280     /* 1280 echantillons entre deux occurences de sequences */
#define FIRST_FILE	2	     /* pour sauter les deux premiers fichiers (. et ..) */
#define LG_TRANSI	640	       /* 640 echantillons de transition */

#define LENMAX_ANNOT 40
static char annot[LENMAX_ANNOT+2]=""; // texte  chercher

static STRING sortie_par; // fichier parole resultat
static infoFile files // domaine de recherche
	   = { "*.*","*.*","*.*",0,0,0 };
static infoFile *curfile; // en fait, pointe sur en_cours
static nbhitmax=100;
static nbhit;

static SAMPLEbuf pause(LG_PAUSE,"pause near Wtop::CMexploreCorpusEtiq()");                 /* pause entre deux sequences extraites  */

static struct notetiqs Ex; // etiquetage manuel du fichier d'extraction 

///////////////////////////////////////////
// remarques:
// -1- une rechercher exploreCorpusEtiq() utilise et casse
//     les en_cours, space, etiqs, et supprime le signal
//     en cours d'dition de la mme manire que la
//     commande "nouveau". On pourrait essayer de ne pas
//     les casser en utilisant des variables spcifiques,
//     ce qui marche sauf pour les points suivants:
//         - enMilli() utilise en_cours.frequence
 
/*
 * Cette fonction recherche et extrait une liste de sequences 
 * de phonemes dans un fichier.
 */

static void extraitAnnotFic( 
   char *path_note,   // nom du fichier d'etiquetage 
   long *nbEcht,	     // nombre d'echantillons du fichier resultat 
   infoFile& dstinfo_par)   // fichier resultat de signal temporel
{
  int j;
  char ch[200];
  bool isPar; // TRUE si le fichier de parole existe

  sprintf(ch,ml(7301,"file: %s"),path_note);
  logs.wr1(ch);
  x_makepath(ch,NULL,NULL,path_note,path_note);
  Wact->wr(ch);

  // lecture de l'etiquetage et lecture des
  // informations minimales pour la parole
  // pour initialiser la structure en_cours 
  if (isWAV(path_note))
  { strcpy(curfile->nom,path_note);
    init_infoFile_read(curfile);
    readnotetiqs(curfile->nom,notes
		,curfile->notedepl
		,curfile->notelongueur);
    isPar=TRUE;
  }
  else
  { char *p;
    p=files.nom;
    x_makepath(curfile->nom,p,p,path_note,p);
    p=files.nomAnnot;
    x_makepath(curfile->nomAnnot,p,p,path_note,p);
    curfile->fmt_son=files.fmt_son;
    curfile->fmt_word=files.fmt_word;
    isPar=init_infoFile_read(curfile);
    if (!isPar)
    { sprintf(ch,ml(7302,"le fichier de parole %s n'existe pas")
	     , curfile->nom);
      affiche_erreur(ch);
      logs.wr1(ml(7303,"... pas de fichier parole"));
    }
    lireAnnotations(&notes, *curfile); 
  }
  /* recherche et extraction du texte  chercher */
    for (j=0;(j<notes.nb-1)&&(nbhit<nbhitmax);j++)
    {
      if (!strcmp(annot, notes.e[j]->Text()))
      { // c'est un hit, la squence est trouve
	extrait1seq(notes,j,j
		   ,Ex
		   ,nbEcht,dstinfo_par,isPar
                   ,nbhit);
      }
    }
}

/*
 *  Lance la recherche des chaines phonetiques dans le corpus demande ...
 */

static bool exploreCorpusAnnot()
// en retour, TRUE si ok
{
  DIR *repertoire;	/* Repertoire dans lequel on
                           cherche un fichier */
  int i;
  bool ret=TRUE;
  STRING ch;
  struct dirent *fichier;

  infoFile dstinfo_sortie;
  long nbEcht = 0;  // nombre de secteurs du fichier resultat
  char proc[]="exploreCorpusAnnot()";

  nbhit=0;
  curfile=&en_cours;

  /* On verifie si le repertoire parole "site_par" existe */
  strcpy(ch,files.nom);
  splitmakedir(ch);
  if (!strlen(ch) ||(repertoire=opendir(ch)) == NULL)
  { affiche_erreur (ml(7306,"Repertoire du corpus parole inexistant"));
    return FALSE;
  }
  closedir(repertoire);

  // On verifie que le repertoire  d'etiquetage existe
  strcpy(ch,files.nomAnnot);
  splitmakedir(ch);
  if (!strlen(ch) || (repertoire=opendir(ch)) == NULL)
  { affiche_erreur (ml(7308,"Repertoire du corpus d'annotation inexistant"));
    return FALSE;
  }

  // on ouvre le fichier de sortie temporaire
  strcpy(sortie_par,"srchword.par"); 
  strcat(strcpy(dstinfo_sortie.nom,tempdir),sortie_par);
  dstinfo_sortie.nb_bits=NB_BITS;
  dstinfo_sortie.swap=SWAPINFILE;
  if (erboxmmioOpen(1,proc,dstinfo_sortie.nom
	,dstinfo_sortie.hmmio
	     =mmioOpen(dstinfo_sortie.nom,NULL
		   ,MMIO_WRITE|MMIO_CREATE))
	!=0) ret=FALSE;
  else
  { //cas normal
    // On verifie que l'utilisateur a bien entr
    //   un texte  chercher 
    if (ret&&(strlen(annot)==0))
    { affiche_erreur (ml(7310,"Aucun texte  chercher"));
      ret=FALSE;
    }

    /* LANCEMENT DE L'EXTRACTION DES CHAINES PHONETIQUES */

    Ex.clear();
    /* on commence par une pause */
    if (ret)
    { nbEcht += LG_PAUSE;
      for (i = 0; i < LG_PAUSE; i++) pause[i] = 0;
      copyBufToHFile(pause,0,dstinfo_sortie
		 , LG_PAUSE * sizeof(SAMPLE));
    }

    logs.open1();
    logs.wr1(ml(7315,"recherche d'annotation dans un corpus"));
    sprintf(ch,ml(7316,"note files:%s"),files.nomAnnot); logs.wr1(ch);
    sprintf(ch,ml(7313,"sound files:%s"),files.nom); logs.wr1(ch);
    sprintf(ch,ml(7317,"texte  chercher: %s"),annot); logs.wr1(ch);

    logs.wr1(ml(7340,"Dbut\tFin"));
    logs.wr1(ml(7341,"[ms]\t[ms]"));

    /* on parcourt tous les fichiers du repertoire d'etiquetage */
    while (  ret
           &&((fichier=readdir(repertoire)) != NULL)
           &&(nbhit!=nbhitmax) )
    if (fichier->d_name[0]!='.')
    { char nom[_MAX_PATH];
      char ext[_MAX_EXT];
		
      x_makepath(nom,files.nomAnnot
                  ,files.nomAnnot
		 ,fichier->d_name
		 ,fichier->d_name);
      _splitpath(nom,NULL,NULL,NULL,ext);
      _splitpath(files.nomAnnot,NULL,NULL,NULL,ch);
      if (stricmp(ext,ch)==0)
      { extraitAnnotFic(nom, &nbEcht
	      , dstinfo_sortie);
      }
    }
    if (erboxmmioClose(41,proc,dstinfo_sortie.nom
		    ,mmioClose(dstinfo_sortie.hmmio,0))
	!=0) ret=FALSE;
    logs.close1();
  }
  closedir(repertoire);

  if (ret&&(nbhit!=0))
  { // criture du fichier de sortie
    writeSoundFile wS;

    strcat(strcpy(curfile->nom,tempdir)
		 ,"srchword.wa$");
    Wact->wr3(ml(7318,"writing"),curfile->nom,"");
    wS.dstinfo=curfile;
    //
    wS.addfiledata(&dstinfo_sortie,0
          ,nsof(dstinfo_sortie.nom));
    for (i=0;i<Ex.nb;i++)
    { wS.addnote(Ex.e[i]->limite
		,Ex.e[i]->Text());
    }
    //
    wS.flush();
    lect_signal();
  }
  else reinitSignalAndScreen();
  remove(dstinfo_sortie.nom);

  Ex.clear();
  return ret;
}

////////////////////////////////////////////
// fentre exploration de corpus tiquet
//
// variables modifies par la fentre
//   annotation
//   suffEtiq
//   reperEtiq
//   //ficParRes
//   //ficAnnotRes
//////////////////////////////////////////
// le scrollbar
#define SCR_NMIN 1.
#define SCR_NMAX 10000.
// soit n la valeur donne par le scrollbar
//      et pos la position (de 0  100)
// la conversion de n  pos est logarithmique
//   selon la formule:
//       n=a*exp(k*pos)
//       avec a et k de telle sorte que l'on a:
//           pour pos==0, on a n==SCR_NMIN
//           pour pos==100, on a n==SCR_NMAX

class TWexplorecorpusannot : public TDialog
{   TStatic *PTSrchNote;
	 TGroupBox *PTDomainGroup;
    TGroupBox *PTNoteGroup;
    TGroupBox *PTSoundGroup;
    TStatic *PTNoteFile;
    TStatic *PTSoundFile;
    TStatic *PTNoteFormat;
    TStatic *PTSoundFormat;
    TButton *PTModifierDomain;
    TStatic *PTnbhitmax;
    TStatic *PTnbhitmaxtext;
    TEdit *PTtext;
    TScrollBar *PTSCRnbhitmax;
  public:
    TWexplorecorpusannot(PTWindowsObject AParent, LPSTR name);
    virtual void SetupWindow();
    virtual bool CanClose();
    void SetTexts();
    float scr_k() { return 0.01*log(SCR_NMAX/SCR_NMIN); }
    float scr_POStoN(int pos)
              { return max(1,(int)(SCR_NMIN*exp(scr_k()*pos))); }
    float scr_NtoPOS(int n)
              { return max(0,min(100
		       ,(int)(log(n/SCR_NMIN)/scr_k()))); }

	 // rponses aux vnements
	 /*
	 virtual void IDmodifierdomaine(RTMessage)
                     = [ID_FIRST+118];
    virtual void IDmodifiernbhitmax(RTMessage)
                     = [ID_FIRST+123];
	 */
	 void IDmodifierdomaine(WPARAM code);
	 void IDmodifiernbhitmax(WPARAM code);
	 DECLARE_RESPONSE_TABLE(TWexplorecorpusannot);
};
DEFINE_RESPONSE_TABLE1(TWexplorecorpusannot,TDialog)
  EV_CHILD_NOTIFY_ALL_CODES(118,IDmodifierdomaine),
  EV_CHILD_NOTIFY_ALL_CODES(123,IDmodifiernbhitmax),
END_RESPONSE_TABLE;

TWexplorecorpusannot::TWexplorecorpusannot(PTWindowsObject AParent, LPSTR name)
	:TDialog(AParent,name)
{ 
  // initialiser le contenu des contrles
  //   avec les variables correspondantes
  PTSrchNote=new TStatic(this,101,0);
  PTDomainGroup=new TGroupBox(this,111);
  PTNoteGroup=new TGroupBox(this,112);
  PTNoteFile=new TStatic(this,113,0); 
  PTNoteFormat=new TStatic(this,114,0);
  PTSoundGroup=new TGroupBox(this,115);
  PTSoundFile=new TStatic(this,116,0); 
  PTSoundFormat=new TStatic(this,117,0); 
  PTModifierDomain=new TButton(this,118);
  PTnbhitmax=new TStatic(this,122,0); 
  PTnbhitmaxtext=new TStatic(this,121,0); 
  PTtext= new TEdit(this,102,LENMAX_ANNOT);
  PTSCRnbhitmax=new TScrollBar(this,123);
}
void TWexplorecorpusannot::SetupWindow()
{ TDialog::SetupWindow();
  SetCaption(ml(7329,"Chercher un texte dans un rpertoire"));
  ::SetWindowText(GetItemHandle(IDCANCEL),ml(7330,"&Annuler"));
  PTtext->SetText(annot);
  PTSrchNote->SetText(ml(7325,"Texte  chercher:"));
  PTDomainGroup->SetCaption(ml(7320,"Domaine de recherche"));
  PTNoteGroup->SetCaption(ml(7326,"fichiers textes:"));
  PTSoundGroup->SetCaption(ml(7322,"fichiers son (si non .WAV):"));
  PTModifierDomain->SetCaption(ml(7323,"Modifier le domaine"));
  PTnbhitmaxtext->SetText(ml(7332,"textes max"));
  SetTexts();
  // initialisation du scrollbar
  PTSCRnbhitmax->SetPosition(scr_NtoPOS(nbhitmax));
}
bool TWexplorecorpusannot::CanClose()
{
  PTtext->GetText(annot,LENMAX_ANNOT);
  return TRUE;
}
void TWexplorecorpusannot::SetTexts()
{ STRING ch;

  strcpy(ch,ml(7327,"nom fichier: "));
  strcat(ch,files.nomAnnot);
  PTNoteFile->SetText(ch);
  //
  strcpy(ch,ml(7327,"nom fichier: "));
  strcat(ch,files.nom);
  PTSoundFile->SetText(ch);
  //
  init_fmt();
  sprintf(ch,ml(7328,"format: %s"),Fmt_par[files.fmt_son].nfmt);
  PTSoundFormat->SetText(ch);
  //
  sprintf(ch,ml(7328,"format: %s"),Fmt_text[files.fmt_word].nom);
  PTNoteFormat->SetText(ch);
  //
  sprintf(ch,"%d",nbhitmax);
  PTnbhitmax->SetText(ch);
}
void TWexplorecorpusannot::IDmodifierdomaine(WPARAM)
{ STRING ch;
  get_format(this,files);
  strcpy(ch,files.nom);
  x_makepath(files.nom,ch,ch,"*",ch);
  strcpy(ch,files.nomAnnot);
  x_makepath(files.nomAnnot,ch,ch,"*",ch);
  SetTexts();
}
void TWexplorecorpusannot::IDmodifiernbhitmax(WPARAM)
{ STRING ch;

  nbhitmax=scr_POStoN(PTSCRnbhitmax->GetPosition());
  sprintf(ch,"%d",nbhitmax);
  PTnbhitmax->SetText(ch);
}
void Wtop::CMexploreCorpusAnnot()
// fonction d'activation de la fentre dialogue
{
  if (UpdateCurrentFileBox() == IDABORT) return;
  if (PTWtop->GetModule()->ExecDialog(
	   new TWexplorecorpusannot(PTWtop
		 ,"Wexplorecorpusannote"))
     !=IDOK) return;
  souris(XC_watch);
  if (exploreCorpusAnnot()) logs.show1();
  souris(XC_top_left_arrow);
}


