/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier recher_c.cpp
//
                                    
#include "common.h"
#pragma hdrstop

#include <dirent.h>

#include "notetiq.h"
#include "cnotetiq.h"
#include "snoori.h"
#include "signal.h"
#include "files.h"
#include "outils.h"
#include "audio.h"
#include "menu.h"
#include "format_f.h"

#define NB_MAX_PHON	5                  /* longueur maximale d'une chaine de phonemes */
#define NB_MAX_SEQ	5	               /* nombre maximal de sequences a chercher */
#define LG_PAUSE	1280     /* 1280 echantillons entre deux occurences de sequences */
#define FIRST_FILE	2	     /* pour sauter les deux premiers fichiers (. et ..) */
#define LG_TRANSI	640	       /* 640 echantillons de transition */

typedef struct {
       int longueur;
       int contenu [NB_MAX_PHON];
} Sequence;

static char donnees [NB_MAX_SEQ][NB_MAX_PHON]; // tiquettes

static STRING sortie_par; // fichier parole resultat
static infoFile files // domaine de recherche
	   = { "*.*","*.*","*.*",0,0,0 };
static infoFile *curfile; // en fait, pointe sur en_cours
static nbhitmax=100;
static nbhit;

static SAMPLEbuf pause(LG_PAUSE,"pause near Wtop::CMexploreCorpusEtiq()");                 /* pause entre deux sequences extraites  */

static struct notetiqs Ex; // etiquetage manuel du fichier d'extraction 

static int l_liste;                   // nombre de sequences  
static Sequence list_seq[NB_MAX_SEQ]; // tableau de sequences 

///////////////////////////////////////////
// remarques:
// -1- une recherche exploreCorpusEtiq() utilise et casse
//     les en_cours, space, etiqs, et supprime le signal
//     en cours d'dition de la mme manire que la
//     commande "nouveau". On pourrait essayer de ne pas
//     les casser en utilisant des variables spcifiques,
//     ce qui marche sauf pour les points suivants:
//         - enMilli() utilise en_cours.frequence
 
//
// cette fonction compare la sequence de phonemes et l'etiquetage.
//

static BOOL comp_seq_phon ( 
    Sequence *seq,  // sequence a chercher
    int j)	    // indice dans l'etiquetage de la phrase
// en retour TRUE si trouv
//           alors len contient la longueur de la
//	                 squence trouve
// j peut tre en dehors de [0..etiqs.nb[
// alors les phonmes en dehors de cet espace
//     sont pris  ""
{ int i=0;

  for (i=0;  (i<seq->longueur)
	     &&comp_phon(
		      ((j+i>=0)&&(j+i<etiqs.nb))
		      ? etiqs.e[j+i]->Text() : ""
		     ,TextOfPhon(seq->contenu[i]) );
         i++);
  return (i==seq->longueur);
}

//////////////////////////////////////////////////////////
// extraction d'une et d'une seule squence
//     de phonmes trouve bonne dans un fichier
//     avec
//       - mise en logs
//       - mise de la parole dans le fichier
//             temporaire de parole
//       - mise de l'tiquetage dans Ex
//
// le fichier parole source est dcrit par en_cours
//
// cette procdure est utilise aussi par recher_a.cpp;
// elle n'a de raison d'tre que pour mettre en commun
// du code entre les recherches phonmes et textes
void extrait1seq(
    notetiqs& ne   // tiquetage source
   ,int j_deb      // 1er lment dans ne
   ,int j_fin      // dernier lment dans ne
   ,notetiqs& Ex   // tiquetage rsultat
   ,long *nbEcht	     // nombre d'echantillons du fichier resultat
   ,infoFile& dstinfo_par   // fichier resultat de signal temporel
   ,BOOL isPar  // TRUE si le fichier de parole existe
   ,int& nbhit)
{

  long nbEcht0; // nombre d'chantillon initial
	long nbEcht_p;	  // nombre d'echantillons
	                  // de parole de la sequence 
	long deb_g,deb_d,fin_g,fin_d;  // squence
		//  lire (en chantillons)
		// amorti croissant de deb_g  deb_d
		// tel que de deb_d  fin_g
                // amorti dcroissant de fin_g  fin_d
	int i,k;
	long il; 
	int j_len; // longueur du morceau  recopier
	char ch[80];
        // 

  j_len=j_fin-j_deb+1;
  sprintf(ch
			,"%ld\t%ld\t"
			,enMilli(ne.e[j_deb]->limite)
			,enMilli(ne.e[j_deb+j_len]->limite));
  for (k=0;k<j_len;k++)
	 sprintf(ch+strlen(ch),"%s,"
			,ne.e[j_deb+k]->Text());
  ch[strlen(ch)-1]=0;
  logs.wr1(ch);

	if (isPar)
	{ // extraction du morceau de parole

	  nbEcht0=*nbEcht;
	  // debut du morceau a recopier 
	  deb_d=ne.e[j_deb]->limite;
	  deb_g=deb_d-LG_TRANSI;
	  if (j_deb>0) deb_g=max(deb_g,ne.e[j_deb-1]->limite);
          deb_g=max(deb_g,0);

	  // fin du morceau a recopier 
	  fin_g=ne.e[j_fin+1]->limite;
	  fin_d=fin_g+LG_TRANSI;
	  if (j_fin<ne.nb-2)
	    fin_d=min(fin_d,ne.e[j_fin+2]->limite);
	  fin_d=min(fin_d,en_cours.longueur/sizeof(SAMPLE)-1);

	  // on extrait le signal temporel 
	  { long len=fin_d - deb_g + 1;
	    space.newsize(len);
	    nbEcht_p = lireSignal(en_cours,en_cours.depl + deb_g*sizeof(SAMPLE),space,0,len);
	    if (nbEcht_p!=len)
	    { char ch[80];
	      sprintf(ch,"unexpected (nbEcht_p=%ld)!=(len=%ld)"
	              ,nbEcht_p,len);
	      affiche_bug(ch);
            }
	  }

	  // croissance linaire du signal au dbut
	  //   du segment 
	  { long len_transi=deb_d-deb_g;
	    for (il=0;il<len_transi;il++)
	      space[il] *= ((float)il) / len_transi;
	  } 
	
	  // dcroissance linaire du signal en fin
	  //   de segment 
	  { long len_transi=fin_d-fin_g;
	    for (il=0;il<len_transi;il++)
	    { long ie=nbEcht_p-1-il;
	      space[ie] *= ((float)il) / len_transi;
	    }
          }
	  // copie du signal 
	  copyBufToHFile(space,0,dstinfo_par
	               , nbEcht_p * sizeof(SAMPLE));
	  *nbEcht += nbEcht_p;

	  // copie de la pause 
	  for (i = 0; i < LG_PAUSE; i++) pause[i] = 0;
	  copyBufToHFile(pause,0,dstinfo_par
	               , LG_PAUSE * sizeof(SAMPLE));
	  *nbEcht += LG_PAUSE;

	  // mise a jour de l'etiquetage 
	  for (k=0 ; k<j_len; k++)
	  { long limite,delta;
	    Ex.addS(limite=
	             ne.e[j_deb+k]->limite
		     +(delta=
		         - deb_g
			 + nbEcht0
		      ) );
	    Ex.addS(ne.e[j_deb+k+1]->limite
		     + delta );
	    Ex.e[Ex.search(limite)]->SetText(
			   ne.e[j_deb+k]->Text());
	  }
	}
	nbhit++;
	if (nbhit==nbhitmax)
	{ logs.wr1(ml(7304,"nombre de hits maximum atteint"));
	  logs.wr1(ml(7305,"la recherche est arrte"));
        }
}

/*
 * Cette fonction recherche et extrait une liste de sequences 
 * de phonemes dans un fichier.
 */

static void extraitEtiqFic( 
   char *path_man,   // nom du fichier d'etiquetage 
   long *nbEcht,	     // nombre d'echantillons du fichier resultat 
   infoFile& dstinfo_par)   // fichier resultat de signal temporel
{
  int i,j;
  char ch[200];
  BOOL isPar; // TRUE si le fichier de parole existe

  sprintf(ch,ml(7301,"file: %s"),path_man);
  logs.wr1(ch);
  x_makepath(ch,NULL,NULL,path_man,path_man);
  Wact->wr(ch);

  // lecture de l'etiquetage et lecture des
  // informations minimales pour la parole
  // pour initialiser la structure en_cours
  strcpy(curfile->nom_man,path_man);
  if (isWAV(path_man))
  { strcpy(curfile->nom,path_man);
    init_infoFile_read(curfile);
    readnotetiqs(curfile->nom,etiqs
		,curfile->etiqdepl
		,curfile->etiqlongueur);
    isPar=TRUE;
  }
  else
  { char *p;
    p=files.nom;
    x_makepath(curfile->nom,p,p,path_man,p);
    p=files.nom_man;
    x_makepath(curfile->nom_man,p,p,path_man,p);
    curfile->fmt_son=files.fmt_son;
    curfile->fmt_phon=files.fmt_phon;
    isPar=init_infoFile_read(curfile);
    if (!isPar)
    { sprintf(ch,ml(7302,"le fichier de parole %s n'existe pas")
	     , curfile->nom);
      affiche_erreur(ch);
      logs.wr1(ml(7303,"... pas de fichier parole"));
    }
    lireman(&etiqs, *curfile); 
  }
  /* recherche et extraction des chaines de phonemes */
  for (i = 0; i < l_liste; i++)
  { // on se deplace dans l'etiquette en cherchant
    //     la sequence de phonemes
    for (j=-list_seq[i].longueur+1;
	 (j<etiqs.nb)&&(nbhit<nbhitmax);
	 j++)
    { 
      if (comp_seq_phon (&list_seq[i],j))
      { // c'est un hit, la squence est trouve
        int j_deb,j_fin; // dbut et fin du morceau  recopier

	j_deb=max(j,0);
	j_fin=min(j+list_seq[i].longueur-1,etiqs.nb-2);
	//
	extrait1seq(etiqs,j_deb,j_fin
		   ,Ex
		   ,nbEcht,dstinfo_par,isPar
		   ,nbhit);

      }
    }
  }
}

/*
 *  Lance la recherche des chaines phonetiques dans le corpus demande ...
 */

static BOOL exploreCorpusEtiq()
// en retour, TRUE si ok
{
  DIR *repertoire;   /* Repertoire dans lequel on
                        cherche un fichier */
  int i, j; 
  BOOL ret=TRUE;
  STRING ch;
  struct dirent *fichier;

  infoFile dstinfo_sortie;
  long nbEcht = 0;  // nombre de secteurs du fichier resultat
  char proc[]="exploreCorpusEtiq()";

  nbhit=0;
  curfile=&en_cours;

  // On verifie que le repertoire parole existe 

  strcpy(ch,files.nom);
  i = strlen(ch);    // ATTENTION a supprimer
  splitmakedir(ch);
  if (!strlen(ch)) {
    affiche_erreur (ml(7306,"Repertoire du corpus parole inexistant"));
    return FALSE;
  }
  if (!strlen(ch) || (repertoire=opendir(ch)) == NULL) {
    affiche_erreur (ml(7306,"Repertoire du corpus parole inexistant"));
    return FALSE;
  }
  closedir(repertoire);

  /* On verifie si le repertoire d'etiquetage "site_man" existe */
  strcpy(ch,files.nom_man);
  splitmakedir(ch);
  if (!strlen(ch) ||(repertoire=opendir(ch)) == NULL) {
    affiche_erreur (ml(7307,"Repertoire du corpus d'etiquetage inexistant"));
    return FALSE;
  }

  // on ouvre le fichier de sortie temporaire
  strcpy(sortie_par,"srchphon.par"); 
  strcat(strcpy(dstinfo_sortie.nom,tempdir),sortie_par);
  dstinfo_sortie.nb_bits=NB_BITS;
  dstinfo_sortie.swap=SWAPINFILE;
  if (erboxmmioOpen(1,proc,dstinfo_sortie.nom
	,dstinfo_sortie.hmmio
	     =mmioOpen(dstinfo_sortie.nom,NULL
		   ,MMIO_WRITE|MMIO_CREATE))
	!=0) ret=FALSE;
  else
  { // Recuperation de la liste des sequences phonetiques           
    // Tous les cases non completees par l'utilisateur
    // comprises entre deux phonemes ou classes phonetiques
    // sont ignores    
    for (i=0,l_liste=0;i<NB_MAX_SEQ;i++)
    { int j1; // nombre de phonmes rels de la squence
      char phon;
      for (j=0,j1=0;j<NB_MAX_PHON;j++)
      { phon=donnees[i][j];
        list_seq[l_liste].contenu[j1]=phon;
        if (*TextOfPhon(phon)!=0) j1++;
      }
      list_seq[l_liste].longueur=j1;
      if (j1!=0) l_liste++;
    }

    // On verifie que l'utilisateur a bien entr
    //   au moins une chaine phonetique 
    if (l_liste==0)
    { affiche_erreur (ml(7309,"Aucune sequence a chercher"));
      ret=FALSE;
    }

    /* LANCEMENT DE L'EXTRACTION DES CHAINES PHONETIQUES */

    Ex.clear();
    /* on commence par une pause */
    if (ret)
    { nbEcht += LG_PAUSE;
      for (i = 0; i < LG_PAUSE; i++) pause[i] = 0; 
      copyBufToHFile(pause,0,dstinfo_sortie
		 , LG_PAUSE * sizeof(SAMPLE));
    }

    logs.open1(); 
    logs.wr1(ml(7311,"recherche d'tiquette dans un corpus"));
    sprintf(ch,ml(7312,"label files:%s"),files.nom_man); logs.wr1(ch);
    sprintf(ch,ml(7313,"sound files:%s"),files.nom); logs.wr1(ch);
    logs.wr1(ml(7314,"squences d'tiquettes:"));

    for (i=0;ret&&(i<l_liste);i++)
    { logs.puts1("...  ");
      for (j=0;j<list_seq[i].longueur;j++)
      { if (j!=0) logs.puts1(",");
        logs.puts1(TextOfPhon(list_seq[i].contenu[j]));
      }
      logs.rclf1();
    }

    logs.wr1(ml(7340,"Dbut\tFin"));
    logs.wr1(ml(7341,"[ms]\t[ms]"));

    /* on parcourt tous les fichiers du repertoire d'etiquetage */
    while (  ret
           &&((fichier=readdir(repertoire)) != NULL)
           &&(nbhit!=nbhitmax) )
    if (fichier->d_name[0]!='.')
    { char nom[_MAX_PATH];
      char ext[_MAX_EXT];
		
      x_makepath(nom,files.nom_man
                  ,files.nom_man
		 ,fichier->d_name
		 ,fichier->d_name);
      _splitpath(nom,NULL,NULL,NULL,ext);
      _splitpath(files.nom_man,NULL,NULL,NULL,ch);
      if (stricmp(ext,ch)==0)
      { extraitEtiqFic(nom, &nbEcht
	      , dstinfo_sortie);
      }
    }
    if (erboxmmioClose(41,proc,dstinfo_sortie.nom
		    ,mmioClose(dstinfo_sortie.hmmio,0))
	!=0) ret=FALSE;
    logs.close1();
  }
  closedir(repertoire);

  if (ret&&(nbhit!=0))
  { // criture du fichier de sortie
    writeSoundFile wS;

    strcat(strcpy(curfile->nom,tempdir)
		       ,"srchphon.wa$");
    Wact->wr3(ml(7318,"writing"),curfile->nom,"");
    wS.dstinfo=curfile;
    //
    wS.addfiledata(&dstinfo_sortie,0
          ,nsof(dstinfo_sortie.nom));
    for (i=0;i<Ex.nb;i++)
    { wS.addetiq(Ex.e[i]->limite
		,Ex.e[i]->Text());
    }
    //
    wS.flush();
    lect_signal();
  }
  else reinitSignalAndScreen();
  remove(dstinfo_sortie.nom);

  Ex.clear();
  return ret;
}

////////////////////////////////////////////
// fentre exploration de corpus tiquet
//
// variables modifies par la fentre
//   annotation
//   suffEtiq
//   reperEtiq
//   //ficParRes
//   //ficAnnotRes
//////////////////////////////////////////
// le scrollbar
#define SCR_NMIN 1.
#define SCR_NMAX 10000.
// soit n la valeur donne par le scrollbar
//      et pos la position (de 0  100)
// la conversion de n  pos est logarithmique
//   selon la formule:
//       n=a*exp(k*pos)
//       avec a et k de telle sorte que l'on a:
//           pour pos==0, on a n==SCR_NMIN
//           pour pos==100, on a n==SCR_NMAX

class TWexplorecorpusetiq : public TDialog
{   TStatic *PTSrchSeq;
    TGroupBox *PTDomainGroup;
    TGroupBox *PTLabelGroup;
    TGroupBox *PTSoundGroup;
    TStatic *PTLabelFile;
    TStatic *PTSoundFile;
    TStatic *PTLabelFormat;
    TStatic *PTSoundFormat;
    TButton *PTModifierDomain;
    TStatic *PTnbhitmax;
    TStatic *PTnbhitmaxtext;
    TButton *PTphon[NB_MAX_SEQ][NB_MAX_PHON];
    TScrollBar *PTSCRnbhitmax;
  public:
    TWexplorecorpusetiq(PTWindowsObject AParent, LPSTR name);
    virtual void SetupWindow();
    virtual bool CanClose();
    void SetTexts();
    float scr_k() { return 0.01*log(SCR_NMAX/SCR_NMIN); }
    float scr_POStoN(int pos)
              { return max(1,(int)(SCR_NMIN*exp(scr_k()*pos))); }
    float scr_NtoPOS(int n)
              { return max(0,min(100
		       ,(int)(log(n/SCR_NMIN)/scr_k()))); }
	 // rponses aux vnements
	 /*
	 virtual void IDmodifierdomaine(RTMessage)
							= [ID_FIRST+118];
	 virtual void IDmodifiernbhitmax(RTMessage)
							= [ID_FIRST+123];
 	 virtual void WMCommand(RTMessage)
						 = [WM_FIRST+WM_COMMAND];
	 */
	 void IDmodifierdomaine(WPARAM code);
	 void IDmodifiernbhitmax(WPARAM code);
	 LRESULT EvCommand(uint id,HWND,uint);
	 DECLARE_RESPONSE_TABLE(TWexplorecorpusetiq);
};
DEFINE_RESPONSE_TABLE1(TWexplorecorpusetiq,TDialog)
  EV_CHILD_NOTIFY_ALL_CODES(118,IDmodifierdomaine),
  EV_CHILD_NOTIFY_ALL_CODES(123,IDmodifiernbhitmax),
END_RESPONSE_TABLE;

TWexplorecorpusetiq::TWexplorecorpusetiq(PTWindowsObject AParent, LPSTR name)
	:TDialog(AParent,name)
{ int i,j;
  // initialiser le contenu des contrles
  //   avec les variables correspondantes
  PTSrchSeq=new TStatic(this,101,0);
  PTDomainGroup=new TGroupBox(this,111);
  PTLabelGroup=new TGroupBox(this,112);
  PTLabelFile=new TStatic(this,113,0); 
  PTLabelFormat=new TStatic(this,114,0); 
  PTSoundGroup=new TGroupBox(this,115);
  PTSoundFile=new TStatic(this,116,0); 
  PTSoundFormat=new TStatic(this,117,0); 
  PTModifierDomain=new TButton(this,118);
  PTnbhitmax=new TStatic(this,122,0); 
  PTnbhitmaxtext=new TStatic(this,121,0); 
  for (i=0;i<NB_MAX_SEQ;i++)
    for (j=0;j<NB_MAX_PHON;j++)
      PTphon[i][j]= new TButton(this,(i+1)*10+j+1,0);
  PTSCRnbhitmax=new TScrollBar(this,123);
}
void TWexplorecorpusetiq::SetupWindow()
{
  int i,j;
  char phon;
  static ind=0;
  TDialog::SetupWindow();
  SetCaption(ml(7331,"Chercher des phonmes dans un rpertoire"));
  ::SetWindowText(GetItemHandle(IDCANCEL),ml(7330,"&Annuler"));
  PTSrchSeq->SetText(ml(7319,"Squences de phonmes  chercher:"));
  PTDomainGroup->SetCaption(ml(7320,"Domaine de recherche"));
  PTLabelGroup->SetCaption(ml(7321,"fichiers phonmes:"));
  PTSoundGroup->SetCaption(ml(7322,"fichiers son (si non .WAV):"));
  PTModifierDomain->SetCaption(ml(7323,"Modifier le domaine"));
  PTnbhitmaxtext->SetText(ml(7324,"squ. max"));
  SetTexts();
  // initialisation des phonmes
  for (i=0;i<NB_MAX_SEQ;i++)
    for (j=0;j<NB_MAX_PHON;j++)
    { if (ind==0) donnees[i][j]=0;
      phon=donnees[i][j]; 
      PTphon[i][j]->SetWindowFont(queryFont(strlen(PhoneticFont) ? PhoneticFont : "arial",
                         7, 14, 0), false);
      PTphon[i][j]->SetCaption( (phon==0) ? "" : TextOfPhon(phon));
    }
  ind=1;
  // initialisation du scrollbar
  PTSCRnbhitmax->SetPosition(scr_NtoPOS(nbhitmax));
}
LRESULT TWexplorecorpusetiq::EvCommand(uint id,HWND hWndCtl,uint notifyCode) //RTMessage Msg)
{
  char phon;
  int i=id,j;
  //TDialog::WMCommand(Msg);
  if (  (i>=10)&&(i<=60) )
  { j=(i%10)-1;
	 i=(i/10)-1;
	 if ((phon=GetPhonemeOrClassByMenu())!=-128)
	 { donnees[i][j]=phon;
		PTphon[i][j]->SetCaption(TextOfPhon(phon));
	 }
  }
  else TWindow::EvCommand(id,hWndCtl,notifyCode);
  return 0;
}
bool TWexplorecorpusetiq::CanClose()
{
  return TRUE;
}
void TWexplorecorpusetiq::SetTexts()
{ STRING ch;

  strcpy(ch,ml(7327,"nom fichier: "));
  strcat(ch,files.nom_man);
  PTLabelFile->SetText(ch);
  //
  strcpy(ch,ml(7327,"nom fichier: "));
  strcat(ch,files.nom);
  PTSoundFile->SetText(ch);
  //
  init_fmt();
  sprintf(ch,ml(7328,"format: %s"),Fmt_par[files.fmt_son].nfmt);
  PTSoundFormat->SetText(ch);
  //
  sprintf(ch,ml(7328,"format: %s"),Fmt_phon[files.fmt_phon].nom);
  PTLabelFormat->SetText(ch);
  //
  sprintf(ch,"%d",nbhitmax);
  PTnbhitmax->SetText(ch);
}
void TWexplorecorpusetiq::IDmodifierdomaine(WPARAM)
{ STRING ch;
  get_format(this,files);
  strcpy(ch,files.nom);
  x_makepath(files.nom,ch,ch,"*",ch);
  strcpy(ch,files.nom_man);
  x_makepath(files.nom_man,ch,ch,"*",ch);
  SetTexts();
}
void TWexplorecorpusetiq::IDmodifiernbhitmax(WPARAM)
{ STRING ch;

  nbhitmax=scr_POStoN(PTSCRnbhitmax->GetPosition());
  sprintf(ch,"%d",nbhitmax);
  PTnbhitmax->SetText(ch);
}
void Wtop::CMexploreCorpusEtiq()
// fonction d'activation de la fentre dialogue
{
  if (UpdateCurrentFileBox() == IDABORT) return;
  if (PTWtop->GetModule()->ExecDialog(
	   new TWexplorecorpusetiq(PTWtop
		 ,"Wexplorecorpusetiq"))
     !=IDOK) return;
  souris(XC_watch);
  if (exploreCorpusEtiq()) logs.show1();
  souris(XC_top_left_arrow);
}

