/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier removeDC.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>

#include "snoori.h"
#include "signal.h"
#include "files.h"

union SAMPLEbytes {
    char c[2];
    short x;
};

int getDCfromFile(char *filename, int headerSizeToSkip, bool swap)
{
  FILE *f = fopen(filename, "rb");
  double s;
  int nbSample;
  SAMPLEbytes sample;

  fseek(f, headerSizeToSkip, SEEK_SET);
  for (nbSample = 0, s = 0.0; fread(&sample, sizeof(short), 1, f); nbSample++) {
    if (swap) {
      char t = sample.c[0];
      sample.c[0] = sample.c[1];
      sample.c[1] = t;
    }
    s += sample.x;
  }
  fclose(f);
  return ((int) (s / nbSample));
}

void removeDC(char *filename, int headerSizeToSkip, bool swap, int dc, char *fileout)
{
  FILE *f = fopen(filename, "rb");
  FILE *g= fopen(fileout, "wb");

  SAMPLEbytes sample;

  char *header = new char [headerSizeToSkip];

  fread(header, sizeof(char) * headerSizeToSkip, 1, f);
  fwrite(header, sizeof(char) * headerSizeToSkip, 1, g);

  while (fread(&sample, sizeof(short), 1, f)) {
    if (swap) {
      char t = sample.c[0];
      sample.c[0] = sample.c[1];
      sample.c[1] = t;
    }
    sample.x -= dc;
    if (swap) {
      char t = sample.c[0];
      sample.c[0] = sample.c[1];
      sample.c[1] = t;
    }
    fwrite(&sample, sizeof(short), 1, g);
  }

  fclose(f);
  fclose(g);
}

static STRING defaultSaveName = "";
static STRING defaultExt = "";

/*
 * Remove the direct current component (composante continue in french)
 */
void Wtop::CMremoveDC()
{
  char filename[_MAX_PATH];
  char ext[_MAX_EXT], drive[_MAX_DRIVE], name[_MAX_FNAME], dir[_MAX_DIR];
  int dc;
  if (!yaUnSignal) return;

  _splitpath(en_cours.nom, NULL, NULL, NULL, ext);
  if (strcmp(defaultSaveName, ""))
  _makepath(filename,NULL,NULL,defaultSaveName, defaultExt);
  else _makepath(filename,NULL,NULL,"*",ext);

  if (get_nom_fichier_save(PTWtop,filename)!=IDOK) return;
  if (strcmp(en_cours.nom,filename)==0) {
    ::MessageBox(HWindow
        ,ml(1017,"filename must be different from current filename")
	,ml(1012,"sauvegarde non effectue")
		  ,MB_OK);
    return;
  }
  affiche_warning("Does not process raw or wav files with non acoustic chunks !!\n (non acoustic data are processed as they were acoustic data)");
  souris(XC_watch);
  dc = getDCfromFile(en_cours.nom, en_cours.depl, en_cours.swap);
  removeDC(en_cours.nom, en_cours.depl, en_cours.swap, dc, filename);
  souris(XC_top_left_arrow);
}