/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier resampling.cpp
//  	(driv de audio.cpp)

#include "common.h"
#pragma hdrstop

//#include <dir.h>
//#include <sys\stat.h>
#include <string.h>
#include <errno.h>
#include "files.h"  //droit_ecrire_messsage
#include "editer.h" // infoEdit.  et FILTRER
#include "signal.h"
#include "filtre.h"
#include "notetiq.h"
#include "outils.h" // enEch
#include "resampling.h"

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
//
// Changement de frquence
//   [8000, 11025, 16000, 22050, 44100] <=> Idem
//   (l'erreur commmise (du  la simplification de la fraction)
//          est au maximum 0.25%)
//
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

class Resampling
{ public:
    // buffer initial
    short *pin;
    long nin;
    unsigned int freqin;
    // buffer rsultat
    short *pout;
    long nout;
    unsigned int freqout;

    int din;  // dout/din est la fraction simplifie de freqin/freqout
    int dout; //
    //
    profiltre filt;

    Resampling();
    ~Resampling();
    void bufferin(short *p, long np, unsigned int freq);  // pour rcuprer le signal d'entre
    void bufferout(unsigned int freq);
    float exec();
    int pf(long i); // buffer intermdiaire filtr (virtuel)
};

Resampling::Resampling()
{ nout=-1;
}

Resampling::~Resampling()
{ nout=-1;
}
void Resampling::bufferin(short *p, long np
             ,unsigned int freq)
{
  pin=p;
  nin=np;
  freqin=freq;
}

void Resampling::bufferout(unsigned int freq)
{ freqout=freq;
}


int Resampling::pf(long i)
{
  int j;
  long k;
  float sum;
  float *p;

  // l'appelant garantit i pour tre sur que p[k]
  //   est bien dans le buffer,
  //      (i>=0)
  //    &&( ((i+filt.ordre-1)/din)<nin )

  sum = 0.0;
  k=(i+din-1)/din;
  p=(float *)&(*filt.f)[0];

  for (j = (int)(k*din-i);
       j < filt.ordre;
       j+=din,k++)
       // accs direct  pin[] et *filt.f
       //   pour aller plus vite
       // les contrles adquats ont t
       //   faits auparavant
    sum += pin[k] * p[j];
  return (int)max(-32000
                  ,min(32000
		       ,(long)(sum*din)));
}


/************************************************/
/*    			FONCTION "PRINCIPALE"				*/
/************************************************/

float Resampling::exec()
{ float v;

  v=freqin/(float)freqout;        // que l'on va transformer en dout/din, fraction simplifie
  /*if      (v<0.55) { din=2; dout=1; }
  else if (v<0.63) { din=5; dout=3; }
  else if (v<0.7)  { din=3; dout=2; }
  else if (v<0.86) { din=4; dout=3; }
  else if (v<1.2)  { pout=pin; nout=nin; return; }
  else if (v<1.41) { din=3; dout=4; }
  else if (v<1.58) { din=2; dout=3; }
  else if (v<1.82) { din=3; dout=5; }
  else             { din=1; dout=2; }  */

  if      (v<0.21) { din=11; dout=2; }
  else if (v<0.31) { din=4; dout=1; }
  else if (v<0.43) { din=11; dout=4; }
  else if (v<0.59) { din=2; dout=1; }
  else if (v<0.71) { din=16; dout=11; }
  else if (v<0.85) { din=11; dout=8; }
  else if (v<1.19) { pout=pin; nout=nin; return(0.); }
  else if (v<1.41) { din=8; dout=11; }
  else if (v<1.72) { din=11; dout=16; }
  else if (v<2.38) { din=1; dout=2; }
  else if (v<3.37) { din=4; dout=11; }
  else if (v<4.75) { din=1; dout=4; }
  else if (v<6)    { din=2; dout=11; }
  else             { pout=pin; nout=nin; return (0.); } // plutot un message d'erreur..

  float erreur_pc = fabs(v-(dout/(float)din))/v;

  /*   Preparation du filtre   */

  strcpy(filt.type,"lp");
  filt.fc=0.5/(max(din,dout)); // le dnominateur provient de freq{in,out}/(freqin*din);
  										 // 0.5 = se placer dans la 1re moiti du spectre.
  filt.att=72.;
  filt.ordre=900; //  partir d'un ratio donner par Lou Scheffer 1991.
                  // Il travaille avec des rapport lvs 147/160 avec un filtre d'ordre 9000 (correspond  900 pour nous)
                  // un filtre 21500 (2150) trs bon mais lourd.
                  // nouveau din => 1250 correspond  12500
  nout=(nin*din-filt.ordre)/dout-2;  // -2 ?
  pout= new short[nout];

  desfir(&filt);   // Stockage du filtre dans filt.f de la classe profiltre

  { long i,n;
    int j,k,m;
    k=(int)min(1000,nout/20);

    for ( i=0; i<nout ; i+=k )
    {
      m=(int)min(k,nout-i);
      for (j=0,n=i;j<m;j++,n++)
			pout[n]=pf(n*dout);     // la fonction pf donne l'chantillon filtr
    }

    nout=max(0,i-k+m);
  }

  return (erreur_pc*100);
}

extern void RawtoWav( char* fichierraw, char* fichierwav, 	float frequence) ;

float resampling(           // renvoie l'erreur en % commise sur la F0
   short *pentree,      // signal d'entre
   long npentree,       // taille d'entree
	int freqentree,      // freq   d'entree
   short **psortie,      // .....  de sortie
   long *npsortie,
   int freqsortie,
   float *fraction_simple  // fraction de rechantillonnage simplifie
   )
{
      Resampling changefreq;
   	float erreur_pourcent;
      long i;

   	changefreq.bufferin(pentree,npentree,freqentree);  //initialisation
		changefreq.bufferout(freqsortie);
		erreur_pourcent = changefreq.exec();              // rchantillonnage
   	(*npsortie)=changefreq.nout;                         // stockage
      *psortie = (short*)realloc(*psortie,(*npsortie)*sizeof(short)); // jusqu'alors de taille = 1
      for (i=0 ; i<(*npsortie) ; i++ )
      	(*psortie)[i]=(changefreq.pout)[i];

      *fraction_simple = (float)changefreq.dout / (float)changefreq.din  ;  // fraction de rechantillonnage simplifie

      return (erreur_pourcent);
}

extern void Alignement();
bool dialogue_resampling();
static int frequence_sortie;
static STRING fichier_sortie;

void Wtop::CMResampling()
{
  int freqentree = en_cours.frequence;
  float fraction_simple=1.;  // fraction de rechantillonnage simplifie
  float erreur_pourcent;
    if (!yaUnSignal) return;
  if (dialogue_resampling()& (en_cours.frequence != frequence_sortie)){
    STRING s;
    souris(XC_watch);
    resample(en_cours.nom, fichier_sortie, ct.tf, (int) en_cours.frequence, frequence_sortie);
    souris(XC_top_left_arrow);
    if (strstr(fichier_sortie, ".wav"))
      sprintf(s, "in the wav file %s", fichier_sortie);
    else  sprintf(s, "in the raw file %s", fichier_sortie);
    ::MessageBox(PTWtop->HWindow, s, ml(9210,"Resampling completed."),
     					MB_OK | MB_TASKMODAL );
  }
}

void resample(
 char *fileIn,
 char *fileOut,
 int nbSampleIn,
 int sampleRateIn,
 int sampleRateOut)
{
  HANDLE hFile, hFileMapping;
  PBYTE pbFile;
  short *pentree;   // futur signal d'entre
  short *psortie;   //              de sortie
  long npentree = 1, npsortie =1, i;
  float erreur_pourcent, fraction_simple;

  npentree = nbSampleIn;
  hFile = CreateFile(en_cours.nom, GENERIC_READ, FILE_SHARE_READ, NULL,
                 				 OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
  if (hFile == INVALID_HANDLE_VALUE) return;
  hFileMapping = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);
  CloseHandle(hFile);
  if (hFileMapping == NULL) return;
  pbFile = (PBYTE) MapViewOfFile(hFileMapping, FILE_MAP_READ, 0, 0, 0);

  pentree = new short[npentree];
  psortie = new short[1]; // pour une premire allocation; la seconde se trouve dans resampling

  for (i=0 ; i<npentree ; i++ ) pentree[i] = getSample(pbFile, i);

  UnmapViewOfFile(pbFile);
  CloseHandle(hFileMapping);

  erreur_pourcent = resampling(pentree, npentree, sampleRateIn, &psortie, &npsortie, sampleRateOut, &fraction_simple);

  if (erreur_pourcent != -1) {  // pour l'instant cas -1 impossible
    /*   Gestion du fichier de sortie et des messages d'avertissement		*/
    STRING resRawFile, resFile ;
    STRING message_erreur;
#ifdef UIY
    sprintf(message_erreur," %s %s %s ", "L'erreur commise dans la simplification du rapport de reechantillonnage entraine une modification sur la F0 de ",
        		My_ftoa(erreur_pourcent), "%");
    affiche_erreur(message_erreur);
#endif
    if (!strstr(fichier_sortie, ".wav"))
      strcpy(resRawFile, fichier_sortie);
    else {
      strcpy(resRawFile,"");
      strncat(resRawFile, fichier_sortie, strlen(fichier_sortie)-4 );
    }

    strcpy(resFile, resRawFile);
    strcat(resFile, ".wav");

    ofstream sortie (resRawFile, ios::out | ios::binary);

    for ( int e=0; e < npsortie ; e++) {    // ct.tf  a t modifi dans resampling()
      sortie.write((char *) &psortie[e], sizeof(SAMPLE) ) ;
    }
    sortie.close();
    RawtoWav(resRawFile, resFile, sampleRateOut);
    unlink(resRawFile);
  }
  delete psortie; //attention psortie est cre dans resampling()
}

// fentre Wresampling
// Choix des paramtres pour le reechantillonnage

class TWresampling : public TDialog
{
    TStatic *PTFreq;
    TRadioButton *PTRes8000;
    TRadioButton *PTRes11025;
    TRadioButton *PTRes16000;
    TRadioButton *PTRes22050;
    TRadioButton *PTRes44100;
    TEdit *PTTargetFile;

  public:
    TWresampling(PTWindowsObject AParent, LPSTR name);
    virtual void SetupWindow();
    virtual bool CanClose();
    void IDSelectTargetFile(WPARAM code);
    DECLARE_RESPONSE_TABLE(TWresampling);
};

DEFINE_RESPONSE_TABLE1(TWresampling,TDialog)
  EV_CHILD_NOTIFY_ALL_CODES(IDC_BUTTONBrowse,IDSelectTargetFile),
END_RESPONSE_TABLE;

TWresampling::TWresampling(PTWindowsObject AParent, LPSTR name)
	:TDialog(AParent,name)
{
  // initialiser le contenu des contrles
  //   avec les variables correspondantes

  PTFreq = new TEdit(this, IDC_STATICFreqEn_cours,  78);
  PTRes8000=new TRadioButton(this,IDC_RADIO8000,0);
  PTRes11025=new TRadioButton(this,IDC_RADIO11025,0);
  PTRes16000=new TRadioButton(this,IDC_RADIO16000,0);
  PTRes22050=new TRadioButton(this,IDC_RADIO22050,0);
  PTRes44100=new TRadioButton(this,IDC_RADIO44100,0);
  PTTargetFile= new TEdit(this, IDC_EDITTargetFile, 78);
}

void TWresampling::SetupWindow()
{
  TDialog::SetupWindow();
  SetCaption("Frequency");    // voir message

  PTFreq->SetText(My_itoa(en_cours.frequence));
  switch ((int)en_cours.frequence) {
    case 8000 :
    	PTRes8000->SetCheck(BF_CHECKED);
      break;
    case 11025 :
    	PTRes11025->SetCheck(BF_CHECKED);
      break;
    case 16000 :
    	PTRes16000->SetCheck(BF_CHECKED);
      break;
    case 22050 :
    	PTRes22050->SetCheck(BF_CHECKED);
      break;
    case 44100 :
    	PTRes44100->SetCheck(BF_CHECKED);
      break;
  }

  STRING fichier;
  strcpy(fichier, tempdir);
  strcat(fichier, "modif.wav");

  PTTargetFile->SetText(fichier);

  ::SetWindowText(GetItemHandle(IDC_GROUPBOX1),"Frequency");
  ::SetWindowText(GetItemHandle(IDC_GROUPBOX7),"Resampling frequency");
  ::SetWindowText(GetItemHandle(IDCANCEL),ml(6422,"&Annuler"));
}

void TWresampling::IDSelectTargetFile(WPARAM)
{ STRING ch;
  STRING filename;
  strcpy(filename,"*.*");

  x_makepath(ch,NULL,NULL,filename,"*.*");
  if (SelectFile(this,ch)==IDOK)
    PTTargetFile->SetText(ch);

}


bool TWresampling::CanClose()
{
	if (PTRes8000->GetCheck()==BF_CHECKED)  frequence_sortie = 8000;
   if (PTRes11025->GetCheck()==BF_CHECKED)  frequence_sortie = 11025;
   if (PTRes16000->GetCheck()==BF_CHECKED)  frequence_sortie = 16000;
   if (PTRes22050->GetCheck()==BF_CHECKED)  frequence_sortie = 22050;
   if (PTRes44100->GetCheck()==BF_CHECKED)  frequence_sortie = 44100;

   PTTargetFile->GetText(fichier_sortie, 80);

   return true;
}

bool dialogue_resampling()
{
	if (( (new TWresampling(PTWtop,"Wresampling"))->Execute())==IDCANCEL ) return false;
   return true;
}



