/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// sauvanno.cpp
//

#include "common.h"
#pragma hdrstop

#include <string.h>
#include <stdio.h>
#include <time.h>

#include "signal.h"
#include "notetiq.h"
#include "outils.h"
#include "files.h"
#include "format_f.h"

/*
 * Sauvegarde des annotations.
 */
static void sauveAnnotCrin(
     notetiqs *annot,
     infoFile& infofile)  // fichier  lire
{
  FILE *pp = fopen(infofile.nomAnnot, "wt");
  int i;

  if (!pp)
  { affiche_erreur(ml(7701,"Erreur inatt..."));
    return;
  }

  fprintf(pp, "%d L %d %s\n", max(0,annot->nb-1)
		   , en_cours.nb_bits
		   , (en_cours.swap==SWAPINFILE)
		     ? "R" : "N");

  for (i = 0; i < annot->nb-1 && annot->e[i+1]->limite < ct.tf; i++)
    fprintf(pp, "%ld %ld %s\n"
		   , annot->e[i]->limite/NBECHTTRAME
		   , annot->e[i+1]->limite/NBECHTTRAME
                   , annot->e[i]->Text());

  fclose(pp);
}


/*
 * Sauvegarde d'un fichier d'annotations au format NIST.
 */
static void sauveAnnotNist(
     notetiqs *annot,
     infoFile& infofile)  // fichier  lire
{
  FILE *pp = fopen(infofile.nomAnnot,"wt");
  int i;

  if (!pp)
  { affiche_erreur(ml(7701,"Erreur inatt..."));
    return;
  }

  for (i = 0; i < annot->nb-1 && annot->e[i+1]->limite < ct.tf; i++)
    fprintf(pp,"%ld %ld %s\n"
		   , annot->e[i]->limite
		   , annot->e[i+1]->limite
                   , annot->e[i]->Text());
  fclose(pp);
}

/*
 * Sauvegarde d'un fichier d'annotations au format Sam.
 */
static void sauveAnnotSam( 
     notetiqs *annot,
     infoFile& infofile,  // fichier  lire
	char *motCle)
{
  FILE *pp = fopen(infofile.nomAnnot,"wt");
  time_t heure; 
  int i;

  if (!pp)
  { affiche_erreur(ml(7701,"Erreur inatt..."));
    return;
  }

  fprintf(pp, "FIL: label\n");
  fprintf(pp, "TYP: orthographic\n");
  fprintf(pp, "SYS: Snoori\n");
  time(&heure);
  fprintf(pp, "DAT:  %s", ctime(&heure));
  fprintf(pp, "LBD:\n");
  for (i = 0; i < annot->nb-1 && annot->e[i+1]->limite < ct.tf; i++)
    fprintf(pp,"%s: %9ld, %9ld, , , , %s\n", motCle
	  , annot->e[i]->limite
	  , annot->e[i+1]->limite
	  , annot->e[i]->Text());
  fprintf(pp, "ELF:\n");
  fclose(pp);
}

/*
 * Sauvegarde d'un fichier d'annotations.
 */
void sauveAnnotations( 
     notetiqs *annot,
     infoFile& infofile)  // fichier  lire
{ if (!droit_ecrire_message(infofile.nomAnnot)) return;

  switch (infofile.fmt_word)
  { case FMTTEXT_SAM :
      sauveAnnotSam(annot, infofile, "LBR");
      break;
    case FMTTEXT_NOISEX :
      sauveAnnotSam(annot, infofile, "UTS");
      break;
    case FMTTEXT_NIST :
      sauveAnnotNist(annot, infofile);
      break;
    case FMTTEXT_VECSYS :
    default :
      sauveAnnotCrin(annot, infofile);
      break;
  }
}

