/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// sauvman.cpp
//

#include "common.h"
#pragma hdrstop

#include <string.h>
#include <stdio.h>
#include <time.h>

#include "signal.h"
#include "notetiq.h"
#include "outils.h"
#include "files.h"
#include "format_f.h"

/*
 * Sauvegarde d'un fichier d'etiquettes phonetiques au format d'Alignement.
 */
static void sauvmanAlign(
     notetiqs *et,
     infoFile& infofile)  // fichier  lire
{
  FILE *pp = fopen(infofile.nom_man,"wt");
  int i;

  if (!pp)
  { affiche_erreur(ml(7701,"Sauvegarde impossible"));
    return;
  }

  fprintf(pp,".ali\n");
  fprintf(pp,"freq_echt %d\n",en_cours.frequence);
  for (i = 0; i < et->nb-1 && et->e[i+1]->limite < ct.tf; i++)
  { fprintf(pp,"\t%10d %10d \t%s\n"
	    ,enMilli(et->e[i]->limite)
	    ,enMilli(et->e[i+1]->limite)
	    ,et->e[i]->Text());
  }
  fprintf(pp,"\t%10d          0 \tFIN\n",enMilli(et->e[i]->limite));
  fprintf(pp,"\n%s\n", en_cours.nom);

  fclose(pp);
}


/*
 * Sauvegarde d'un fichier d'etiquettes phonetiques au format CRIN.
 */
static void sauvmanCrin( 
     notetiqs *et,
     infoFile& infofile)  // fichier  lire
{
  FILE *pp = fopen(infofile.nom_man,"wt");
  int i;

  if (!pp)
  { affiche_erreur(ml(7701,"Sauvegarde impossible"));
    return;
  }

  fprintf(pp,"%s\n",(en_cours.swap == SWAPINFILE)
                    ? "swap" : "normal");
  fprintf(pp,"%8.2lf\n",en_cours.frequence);
  for (i = 0; i < et->nb-1 && et->e[i+1]->limite < ct.tf; i++)
  { fprintf(pp,"{%2s},%5ld,%5ld,%5ld, (%8ld,%8ld)\n"
	    ,et->e[i]->Text()
	    ,enMilli(et->e[i]->limite)
	    ,enMilli(et->e[i+1]->limite)
	    ,enMilli( ( et->e[i]->limite
		       +et->e[i+1]->limite) / 2)
	    ,et->e[i]->limite*sizeof(SAMPLE)
	    ,et->e[i+1]->limite*sizeof(SAMPLE));
  }

  fclose(pp);
}

/*
 * Sauvegarde d'un fichier d'etiquettes phonetiques au format NIST.
 */
static void sauvmanNist( 
     notetiqs *et,
     infoFile& infofile)  // fichier  lire
{
  FILE *pp = fopen(infofile.nom_man,"wt");
  int i;

  if (!pp)
  { affiche_erreur(ml(7701,"Sauvegarde impossible"));
    return;
  }

  for (i = 0; i < et->nb-1 && et->e[i+1]->limite < ct.tf; i++)
    fprintf(pp,"%ld %ld %s\n"
	  , et->e[i]->limite, et->e[i+1]->limite
	  , et->e[i]->Text());
  fclose(pp);
}

/*
 * Sauvegarde d'un fichier d'etiquettes phonetiques au format Sam.
 */
static void sauvmanSam( 
     notetiqs *et,
     infoFile& infofile)  // fichier  lire
{
  FILE *pp = fopen(infofile.nom_man,"wt");
  time_t heure; 
  int i;

  if (!pp)
  { affiche_erreur(ml(7701,"Sauvegarde impossible"));
    return;
  }

  fprintf(pp, "FIL: label\n");
  fprintf(pp, "TYP: phonemic\n");
  fprintf(pp, "SYS: Snoori\n");
  time(&heure);
  fprintf(pp, "DAT:  %s", ctime(&heure));
  fprintf(pp, "LBD:\n");
  for (i = 0; i < et->nb-1 && et->e[i+1]->limite < ct.tf; i++)
    fprintf(pp,"LBB: %9ld, %9ld, %9ld, %s\n"
	  , et->e[i]->limite
	  , (et->e[i]->limite+et->e[i+1]->limite)/2
	  , et->e[i+1]->limite
	  , et->e[i]->Text());
  fprintf(pp, "ELF:\n");
  fclose(pp);
}

//**************************************
//* Sauvegarde de l'etiquetage manuel   
//**************************************

void sauvman( 
     notetiqs *et,
     infoFile& infofile)  // fichier  lire
{ if (!droit_ecrire(infofile.nom_man)) return;

  switch (infofile.fmt_phon)
  { case FMTPHON_SAM :
      sauvmanSam(et, infofile);
      break;
    case FMTPHON_NIST :
      sauvmanNist(et, infofile);
      break;
    case FMTPHON_ALIGN :
      sauvmanAlign(et, infofile);
      break;
    case FMTPHON_CRIN :
    default :
      sauvmanCrin(et, infofile);
      break;
  }
}

