/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier seleclp.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include "snoori.h"
#include "fftlpc.h"
#include "spectre.h"
#include "wcoupe.h"
#include "fft_mayer.h"

static FLOATbuf fftData_(2048,"static fftData near lpcSelective()");	/* tableau de transfert pour les FFT */
static float *fftData;

/*
 *  SELECTE_LPC retourne les coefficients de LPC dans le 
 *              domaine frequentiel [wa..wb] Hz.
 */
void seleclp(
	int np,       /* nombre de points d'entree */
	int ordp,     /* l'ordre de LPCSELEC */
	float wa,     /* frontiere d'analyse */
	float wb,
	float fech,   /* frequence d'echantionnage */
	float *sig,   /* signal d'entree */
	float *err,   /* erreur de LPC */
	float *coefp) /* les coefficients */
{
  int l, l2, w1, w2, p, nfft;
  float x, xw2, arc;
  double xr[1024], xi[1024],
         *realfft, *imagfft;

  if (np >= 256) nfft = 1024; else nfft = 512;
  realfft = new double[nfft];
  imagfft = new double[nfft];
  for (int i = 0; i < nfft; i++) xi[i] = 0.0;
  for (int i = 0; i < np; i++) xr[i] = sig[i];
  for (int i = np; i < nfft; i++) xr[i] = 0.0;
  for (int i = 0; i < nfft; i++) {
    realfft[i] = xr[i];
    imagfft[i] = xi[i];
  }

  fft(nfft, realfft, imagfft);

  w1 = (int) (nfft / fech * wa);
  w2 = (int) (nfft / fech * wb) - 1;
  for (int i = w1; i <= w2; i++)
    xr[i - w1] = realfft[i] * realfft[i] + imagfft[i] * imagfft[i];

  l = w2 - w1;
  l2 = l * 2;

  xw2 = xr[w2];
  for (int i = 0; i <= ordp; i++) {
      xi[i] = xr[0] + xw2;
      xw2 = - xw2;
      arc = 3.14159 * i / l;
      x = 0;
      for (int j = 1; j < l; j++) x = x + xr[j] * cos(arc * j);
      xi[i] = (xi[i] + x * 2) / l2;
  }

  *err = xi[0];
  if (*err == 0) for (int i = 1; i <= ordp; i++) coefp[i] = 0.0;
  else {
    x = xi[1] / *err;
    coefp[1] = x;
    *err = (1 - x * x) * *err;
    for (int i = 2; i <= ordp; i++) {
       p = i - 1;
       x = xi[i];
       for (int j = 1; j <= p; j++) x = x - xi[i - j] * coefp[j];
       x = x / *err;
       coefp[i] = x;
       for (int j = 1; j <= p; j++) xr[j] = coefp[j] - x * coefp[i - j];
       for (int j = 1; j <= p; j++) coefp[j] = xr[j];
       *err = (1 - coefp[i] * coefp[i]) * *err;
    }
    for (int i = 1; i <= ordp; i++) coefp[i] = - coefp[i];
  }
  coefp[0] = 1.0;
  delete [] realfft;
  delete [] imagfft;
}

int ordre_lpcsel_msTOecht(float ordre_ms
			,unsigned int fmin
			,unsigned int fmax)
// renvoie toujours un ordre pair
{ int i;
  i=     max(ORDRE_LPC_MIN
             ,min(ORDRE_LPC_MAX,
	 IROUND(2+ordre_ms
		  *2*(fmax-fmin+10.0)
		  /1000.0)
	 ));
  return i+i%2;
}
