/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

/////////////////////////////////////////////
// fentre WSettingsKlatt
// Choix des parametres de controle du synthetiseur de Klatt

#include "common.h"
#include "signal.h"
#include "polypoint.h"
#include "guiklat.h"

extern int depFor;
extern Wpiquefor *wpiquefor;

void Wpiquefor::initCtrlCommandKlatt()
{
  ctrlCommandKlatt.samplingRate = (yaUnSignal != ASIGNAL) ? 16000 : en_cours.frequence;
  ctrlCommandKlatt.branch = PARALLEL;
  ctrlCommandKlatt.frame = depFor;
  ctrlCommandKlatt.flutter = 0;
  ctrlCommandKlatt.output = COMPLETE;
  ctrlCommandKlatt.byteArrang = HIGHBYTE;
  ctrlCommandKlatt.source = KLATTSOURCE;
}


class TWSettingsKlatt : public TDialog
{
    TRadioButton *PTParallel;
    TRadioButton *PTCascadParall;
    TRadioButton *PTvoicing, *PTaspiration, *PTfrication, *PTcascout,
    				  *PTparout, *PTbypass, *PTallexcit, *PTcomplete;
    TRadioButton *PThighbyte, *PTlowbyte, *PT8000, *PT11025, *PT16000, *PT22050;
    TRadioButton *PTKlattSource, *PTLFsource;
    TScrollBar *PTflutter, *PTframe;
    TStatic *PTTextFlut, *PTTextFrame;
  public:
    TWSettingsKlatt(PTWindowsObject AParent, LPSTR name);
    virtual void SetupWindow();
    virtual bool CanClose();
    void IDscrollbar(WPARAM);
    void IDscrollFlutter(WPARAM);
    void IDscrollFrame(WPARAM);
	 DECLARE_RESPONSE_TABLE(TWSettingsKlatt);
};

DEFINE_RESPONSE_TABLE1(TWSettingsKlatt,TDialog)
 EV_CHILD_NOTIFY_ALL_CODES(IDC_SCROLLframe,IDscrollFrame),
 EV_CHILD_NOTIFY_ALL_CODES(IDC_SCROLLflutter,IDscrollFlutter),
END_RESPONSE_TABLE;

TWSettingsKlatt::TWSettingsKlatt(PTWindowsObject AParent, LPSTR name)
	:TDialog(AParent,name)
{
  // initialiser le contenu des contrles
  //   avec les variables correspondantes
  PTParallel = new TRadioButton(this, IDC_RADIOparallel, 0);
  PTCascadParall = new TRadioButton(this, IDC_RADIOcascadPara,0);
  PTflutter = new TScrollBar(this, IDC_SCROLLflutter, 0);
  PTTextFlut = new TStatic(this, IDC_STATICvalFlut);
  PTframe = new TScrollBar(this, IDC_SCROLLframe, 0);
  PTTextFrame = new TStatic(this, IDC_STATICvalFrame);
  PTvoicing = new TRadioButton(this, IDC_RADIOvoicing, 0);
  PTaspiration = new TRadioButton(this, IDC_RADIOaspiration, 0);
  PTfrication = new TRadioButton(this, IDC_RADIOfrication, 0);
  PTcascout = new TRadioButton(this, IDC_RADIOcascout, 0);
  PTparout = new TRadioButton(this, IDC_RADIOparaout, 0);
  PTbypass = new TRadioButton(this, IDC_RADIObypass, 0);
  PTallexcit = new TRadioButton(this, IDC_RADIOallexcit, 0);
  PTcomplete = new TRadioButton(this, IDC_RADIOcomplete, 0);
  PThighbyte = new TRadioButton(this, IDC_RADIOhighbyte, 0);
  PTlowbyte  = new TRadioButton(this, IDC_RADIOlowbyte, 0);
  PT8000 = new TRadioButton(this, IDC_RADIO8000, 0);
  PT11025 = new TRadioButton(this, IDC_RADIO11025, 0);
  PT16000 = new TRadioButton(this, IDC_RADIO16000, 0);
  PT22050 = new TRadioButton(this, IDC_RADIO22050, 0);
  PTKlattSource = new TRadioButton(this, IDC_RADIOKlattSource, 0);
  PTLFsource  = new TRadioButton(this, IDC_RADIOLFsource, 0);
}

void TWSettingsKlatt::SetupWindow()
{
  STRING s;
  TDialog::SetupWindow();
  SetCaption("Options of the Klatt synthesizer command");

  PTflutter->SetRange(0, 100, true);
  PTflutter->SetPosition(wpiquefor->ctrlCommandKlatt.flutter);
  sprintf(s, "%d %", wpiquefor->ctrlCommandKlatt.flutter);
  PTTextFlut->SetText(s);

  PTframe->SetRange(4, 32, true);
  PTframe->SetPosition(wpiquefor->ctrlCommandKlatt.frame);
  sprintf(s, "%d ms", wpiquefor->ctrlCommandKlatt.frame);
  PTTextFrame->SetText(s);

  if (wpiquefor->ctrlCommandKlatt.branch == PARALLEL) PTParallel->SetCheck(BF_CHECKED);
  else PTCascadParall->SetCheck(BF_CHECKED);

  if (wpiquefor->ctrlCommandKlatt.byteArrang == LOWBYTE)
    PTlowbyte->SetCheck(BF_CHECKED);
  if (wpiquefor->ctrlCommandKlatt.byteArrang == HIGHBYTE)
    PThighbyte->SetCheck(BF_CHECKED);

  if (wpiquefor->ctrlCommandKlatt.source == KLATTSOURCE)
    PTKlattSource->SetCheck(BF_CHECKED);
  if (wpiquefor->ctrlCommandKlatt.source == LFSOURCE)
    PTLFsource->SetCheck(BF_CHECKED);

  if (yaUnSignal == ASHADOW) switch (wpiquefor->ctrlCommandKlatt.samplingRate) {
    case 8000: PT8000->SetCheck(BF_CHECKED);
      break;
    case 11025: PT11025->SetCheck(BF_CHECKED);
      break;
    case 16000: PT16000->SetCheck(BF_CHECKED);
      break;
    case 22050: PT22050->SetCheck(BF_CHECKED);
      break;
  }
  else {   // If there is a real signal the sampling frequency of the copy
           // MUST be that of the original signal. 
    PT8000->EnableWindow(false);
    PT11025->EnableWindow(false);
    PT16000->EnableWindow(false);
    PT22050->EnableWindow(false);
  }

  switch (wpiquefor->ctrlCommandKlatt.output) {
  	case VOICING:
   	PTvoicing->SetCheck(BF_CHECKED);
      break;
	case SASPIRATION:
   	PTaspiration->SetCheck(BF_CHECKED);
      break;
	case FRICATION:
   	PTfrication->SetCheck(BF_CHECKED);
      break;
	case CASCADEOUT:
   	PTcascout->SetCheck(BF_CHECKED);
      break;
	case PARALLOUT:
   	PTparout->SetCheck(BF_CHECKED);
      break;
	case BYPASS:
   	PTbypass->SetCheck(BF_CHECKED);
      break;
	case ALLEXCIT:
   	PTallexcit->SetCheck(BF_CHECKED);
      break;
	case COMPLETE:
   	PTcomplete->SetCheck(BF_CHECKED);
      break;
  }
//  ::SetWindowText(GetItemHandle(IDC_GROUPBOX1), "Branche du synthtiseur");
//  ::SetWindowText(GetItemHandle(IDCANCEL), "&Annuler");
}

bool TWSettingsKlatt::CanClose()
{
  if (PTParallel->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.branch = PARALLEL;
  else if (PTCascadParall->GetCheck() ==BF_CHECKED) wpiquefor->ctrlCommandKlatt.branch = CASCPARALL;

  if (PTlowbyte->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.byteArrang = LOWBYTE;
  else if (PThighbyte->GetCheck() ==BF_CHECKED) wpiquefor->ctrlCommandKlatt.byteArrang = HIGHBYTE;

  if (PTKlattSource->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.source = KLATTSOURCE;
  else if (PTLFsource->GetCheck() ==BF_CHECKED) wpiquefor->ctrlCommandKlatt.source = LFSOURCE;

  if (PTvoicing->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.output = VOICING;
  else if (PTaspiration->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.output = SASPIRATION;
  else if (PTfrication->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.output = FRICATION;
  else if (PTcascout->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.output = CASCADEOUT;
  else if (PTparout->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.output = PARALLOUT;
  else if (PTbypass->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.output = BYPASS;
  else if (PTallexcit->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.output = ALLEXCIT;
  else if (PTcomplete->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.output = COMPLETE;

  if (PT8000->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.samplingRate = 8000;
  else if (PT11025->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.samplingRate = 11025;
  else if (PT16000->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.samplingRate = 16000;
  else if (PT22050->GetCheck() == BF_CHECKED) wpiquefor->ctrlCommandKlatt.samplingRate = 22050;

  delete PT8000;
  delete PT11025;
  delete PT16000;
  delete PT22050;
  delete PTCascadParall;
  delete PTParallel;
  delete PTvoicing;
  delete PTaspiration;
  delete PTfrication;
  delete PTcascout;
  delete PTparout;
  delete PTbypass;
  delete PTallexcit;
  delete PTcomplete;
  delete PTframe;
  delete PTTextFrame;
  delete PTflutter;
  delete PTlowbyte;
  delete PThighbyte;
  delete PTTextFlut;
  delete PTKlattSource;
  delete PTLFsource;
  return true;
}

void TWSettingsKlatt::IDscrollFlutter(WPARAM)
{
  int pos;
  STRING s;
  pos = PTflutter->GetPosition();
  wpiquefor->ctrlCommandKlatt.flutter = pos;
  sprintf(s, "%d %", wpiquefor->ctrlCommandKlatt.flutter);
  PTTextFlut->SetText(s);
}

void TWSettingsKlatt::IDscrollFrame(WPARAM)
{
  int pos;
  STRING s;
  pos = PTframe->GetPosition();
  wpiquefor->ctrlCommandKlatt.frame = pos;
  sprintf(s, "%d ms", wpiquefor->ctrlCommandKlatt.frame);
  PTTextFrame->SetText(s);
}

void Wpiquefor::CMSettingsKlatt()
{
  PTWtop->GetApplication()->ExecDialog(
	   new TWSettingsKlatt(PTWtop,"WSettingsKlatt"));
}


