/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier specfft.cpp
//

#include "common.h"
#pragma hdrstop

#include <math.h>
#include "fftlpc.h"
#include "spectre.h"

static FLOATbuf h(1,"static h near specFft()");
static FLOATbuf x_(1,"x near specFft()");
static float *x=(float *)&x_[0];
static int fen_signal,fen_fft;
static float dBCorrection; // Pour se ramener aux
                      // niveaux standards de Snoori. 


////////////////////////////////////////////////////
// Calcul du module d'un spectre avec appel d'une
// transformee de Fourier sur 256 points
//
// impratif, mais non contrl
//     256 <= fen_fft <= 4096
//     fen_signal<=fen_fft
//   et aussi
//     z et spec doivent tre sufisamment grand
//
// la bonne utilisation des fonctions
//     d'abord initSpecFft()  obligatoire 1 fois
//                            avant chaque utilisation
//     puis specFft() 1 ou plusieurs fois
//     enfin freeSpecFft() facultatif, mais recommand
//
// N.B.:
// - diffrences par rapport  la verions SUN de SNORRI
//       - pas de blackman possible (toujours hamming)
//       - fentres signal et fft donnes uniquement
//             sur l'init
//
void specFft(	
	SAMPLEbuf& z,	// signal temporel brut 
	long z_debut, 
	FLOATbuf& spec_, // spectre calcule	
	bool avecPreacc, // true if preemphasis
   bool withLog)    // true if with the decibel (log)
{
  int i;
  float *spec=(float *)&spec_[0];

  if (avecPreacc) {
   preac_f(z, z_debut, x_, fen_signal);
    ham_ff(x_, x_, &h, fen_signal);
  }		
  else 
    ham_sf(z, z_debut, x_, &h, fen_signal);
  for (i = fen_signal; i < fen_fft + 2; i++)  x[i] = 0.;
  switch (fen_fft) {
    case 256: fftr256(x_, 1); break;
    case 512: fftr512(x_, 1); break;
    case 1024: fftr1024(x_, 1); break;
    case 2048: fftr2048(x_, 1); break;
    case 4096: fftr4096(x_, 1); break;
    default: affiche_bug(
	 "specFft() : taille fenetre fft non prvue");
  } 

  for (i = 0; i <= fen_fft / 2; i++) {
    float xx;
    xx = x[i+i] *  x[i+i] + x[i+i+1] * x[i+i+1];
    if (xx < EPSILON)  xx = EPSILON;
    spec[i] = (withLog) ? (10. * log10(xx) + dBCorrection) : xx;
  }
}

//
// Initialisation de la fenetre de Hamming 
//
void initSpecFft(int fen_signal_
	,int fen_fft_)
{
  fen_signal=fen_signal_;
  fen_fft=fen_fft_;
  h.newsize(fen_signal+1);
  x_.newsize(fen_fft+2);
  initham(&h, fen_signal);
  x=(float *)&x_[0];
  /* Si les harmoniques etaient limitees a un point,
     l'amplitude ne dependrait que du nombre de points
     de la fenetre de hamming car l'amplitude du lobe
     principal est egale (en module) au nombre de
     points de la fenetre. Pour normaliser il faut
     donc soustraire 20. * log10((double) fen_signal);
     Bien sur, la convolution porte sur plus de points
     car les hamoniques de parole ne sont pas reduites
     a un point, les autres termes sont donc empiriques.
     En Gros, plus le nombre de points de la fenetre
     de hamming est grand moins le nombre de points du
     spectre de parole sont pris en compte, donc il faut
     compenser en renforcant l'amplitude du spectre final.
  */
  switch (fen_fft) {
    case 16:
      dBCorrection = 20. * log10(8. / ((double) fen_signal));
      break;
    case 32:
      dBCorrection = 20. * log10(16. / ((double) fen_signal));
      break;
    case 64:
      dBCorrection = 20. * log10(32. / ((double) fen_signal));
      break;
     case 128:
      dBCorrection = 20. * log10(64. / ((double) fen_signal));
      break;
     case 256:
      dBCorrection = 20. * log10(128. / ((double) fen_signal));
      break;
    case 512:
      dBCorrection = 20. * log10(256. / ((double) fen_signal));
      break;
     case 1024:
      dBCorrection = 20. * log10(512. / ((double) fen_signal));
      break;
     case 2048:
      dBCorrection = 20. * log10(1024. / ((double) fen_signal));
      break;
     case 4096:
      dBCorrection = 20. * log10(2048. / ((double) fen_signal));
      break;
    default: affiche_bug(
	 "initSpecFft() : taille fenetre fft non prvue");
  }
}
//
// libration des ressources
//
void freeSpecFft()
{ h.newsize(1);
  x_.newsize(1);
}


