/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier spectreLPCCEP.cpp
//

#include "common.h"
#pragma hdrstop

#include <math.h>
#include "fftlpc.h"
#include "fft_mayer.h"

static void LPCtoCC(float a[], int P, double gain, double c[], int M);

void spectreLPCCEP(
	float spectre[],
	SAMPLE *parole,
	long parole_debut,
        int fen_signal,
        int fftOrder,
	int preaccentuation,
	int LPCorder,
	int cepstrumOrder)
{
  int nbform;
  float *r = new float[LPCorder + 1],
	*a = new float[LPCorder + 1],
	*sf= new float[LPCorder + 1],
        *xr = new float[fen_signal],
        gain;
  double *real = new double[fftOrder],
    *cepstrum = new double[cepstrumOrder];
  

  initham_((float *) NULL, fen_signal);
  if (preaccentuation) {
    preac_f_(parole, parole_debut, xr, fen_signal);
    ham_ff_(xr, xr, NULL, fen_signal);
  }
  else ham_sf_(parole, parole_debut, xr, NULL, fen_signal);

  autolpc (xr, r, a, sf, &gain, fen_signal , LPCorder);

  LPCtoCC(a, LPCorder, (double) gain, cepstrum, cepstrumOrder);

  cepstrum[0] *= 10.;
  for (int i = 1; i < cepstrumOrder; i++) cepstrum[i] *= 5.;

  real[0] = cepstrum[0];
  for (int n = 1; n < cepstrumOrder; n++) real[n] = real[fftOrder - n] = cepstrum[n];
  for (int n = cepstrumOrder; n < fftOrder / 2; n++) real[n] = real[fftOrder - n] = 0.;
  real[fftOrder / 2] = 0.;

  realfft(fftOrder, real);
  for (int i = 0; i < fftOrder / 2; i++) spectre[i] = real[i];

  delete [] r;
  delete [] a;
  delete [] sf;
  delete [] xr;
  delete [] cepstrum;
  delete [] real;
 }


/*
 * http://www.owlnet.rice.edu/~elec532/PROJECTS98/speech/cepstrum/cepstrum.html
 */
static void LPCtoCC(
  float a[], // linear prediction coefficients
  int P, // LPC order
  double gain, // LPC gain
  double c[], // LPC cepstra
  int M) // number of cepstral coefficients
{
  int m, i;

  c[0] = log10(gain);
 
  for (m = 1; m <= P; m++) {
    double s;
    int k;
    for (k = 1, s = 0.; k <= m - 1; k++) s+= k * a[m - k] * c[k];
    c[m] = a[m] + 1. / m  * s;
  }
  for (m = P + 1; m < M; m++) {
    double s;
    int k;
    for (k = m - P, s = 0.; k <= m - 1; k++) s += k * a[m - k] * c[k];
    c[m] =  1. / m  * s;
  }
}
