/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier spectro.h
//
#include "criticalbands.h"
#ifndef _SPECTRO_H
#define _SPECTRO_H
/* types de spectrogrammes affichables */
#define FFT_L 					1
#define FFT_E 					2
#define FFT_REASS 			3
#define LPC 					4
#define CEP 					5
#define FFT_CRITICALBANDS 	6


class spectres
{ public:
    FLOATbuf **p; // table des spectres
    int ls; // longueur d'un spectre
    int ns; // nombre de spectres //calculs
    long echt; // chantillon du premier spectre
               //   en _i() donc relatif  ct.d_e
    int mode;
    bool isPreac; // TRUE si praccentuation
    float silence_spectro;
    float max_spectro;
    double samplingRate; // used to compute the frequency warping with the bark scale
    int nbBands;
    bool completionStrategy; // the stategy for choosing filters ASR like or spectro like (in the second case
    								  // the frequencies of filters are chosen linearly between 0 and Fs/2)
    FrequencyScale frequencyScale;    // Perceptive frequency scale (Bark, Mel or Mely)
    bool withCepstralSmoothing; // true if cepstral smoothing before applying critical bands
    bool nonLinearDisplay;  // true if the frequency display scale is the perceptual one (mel or bark)
    long depl; // delta chantillons entre spectres
      	     //   significatif seulement quand les
	     //   spectres sont calculs avec
	     //   fft_normal() ou lpc_normal()
    int shiftPixel; // delta echantillons entre spectres
	     //   exprim en pixel
	     //   (pour certains modes seulement)
    bool withTrueEnvelope; // true if the trueEnvelope (halle algorithm) is appliedinstead of the simple cepstral smoothing.
    int cMaxSpectro; // correspond  fMaxSpectro (frquence maximale du spectrogramme)
    int fen_signal;
    int fen_fft;
    int fen_fft1;
    // mode et frequence sont  lire dans en_cours

    spectres();
    ~spectres();
    void clear();
    int size() { return ns; }
    int size_sp() { return ls; }
    void resize(int nbSpectres);
    void resize_sp(int n);
    FLOATbuf & operator[](int i) { return *p[i]; }
};

// variables dans le fichier spectrog.cpp
extern spectres sps; // les spectres calculs
extern bool autospectro; // recalcul et affichage du
			// spectrogramme chaque fois
		        // que ncessaire

// dans le fichier spectrof.cpp
extern void spectroFin(
	SAMPLEbuf& cnv,	// signal temporel
	long premEcht,   // debut
	long derEcht,	//   et fin de calcul du zoom
	int mode, 
	struct ctrl_temps ctl,
	SpectroDisplay display, // display d'affichage
	int efface);	    // si VRAI on efface la fenetre
// dans le fichier spectrog.cpp
extern int fft_normal(
     SAMPLEbuf& cnv,	 /* signal temporel */
     spectres& res,	/* spectres calcules */
     long cnv_debut,
     int nb_fft,	 /* nombre de fft a calculer */
     long over,		 /* deplacement dans le signal
                            temporel pour chaque calcul */
     int avecPreacc);	 /* VRAI si avec preaccentuation */

extern void spectro_affichefft(
      float *X	// coefficient de la fft 
     ,int abscisse	// abscisse de la fft a afficher
     ,int shiftPixel
     ,d_planche *fenetre);
extern void saveOneSpectrumInFile(
      float *X, // coefficient du spectre
      int height, // number of pixels for each spectrum (height of the image)
      FILE *ficImage);

extern int sp_nbSpectres(long& echtDeb,long echtFin);
extern void spectrogram(
   SAMPLEbuf *sig,
   SpectroDisplay display,
	long	premEcht,	// echantillon de debut
	                        //    de calcul fft	
	long	derEcht,	// echantillon de fin
	                        //    de calcul de fft
   struct ctrl_temps ctl,
	int 	clear);		// option de nettoyage avant
	                        //   de visualiser
				// les spectros				
extern void spectro_SetMode(int mode);
extern void spectro_GetParams(int mode);
extern long spectro_GetDepl();
extern long spectro_GetFenSignal();
extern int _p(long echt); // rend un numro de spectre
// dans le fichier spectrop.cpp
    struct paramsp
    { bool isOpen; // TRUE si fenetre paramtre ouverte
      PTWindowsObject PTW; // param window
      int num; // numro , sert uniquement 
	       // l'affichage des fentres paramtres
      int menu;
      bool isPreac; // TRUE si praccentuation
      float silence_spectro;
      float max_spectro;
      float depl;
      int shiftPixel;
      float fen_signal;
    };
    struct paramspFFT : paramsp // transformee de Fourier
    { int fen_fft;
    };
    struct paramspCriticalBands : paramspFFT // Critical bands
    { double samplingRate; // used to compute the frequency warping with the bark scale
      bool completionStrategy;  // true if like ASR (Automatic Speech Recognition)
      FrequencyScale frequencyScale;    // Perceptive frequency scale
      bool withCepstralSmoothing;
      bool nonLinearDisplay;
      int nbBands;
    };
    struct paramspLisCep : paramsp // lissage cepstral
    { int fen_fft1; // fft du cepstre
      float debut_liftrage,fin_liftrage;
      int fen_fft2; // fft finale
      bool withTrueEnvelope; // true if the trueEnvelope (halle algorithm) is appliedinstead of the simple cepstral smoothing.
    };
    struct paramspLPC : paramsp // LPC
    { float ordre_lpc;
      int fen_fft;
    };


extern paramspFFT    psp_FFTE, psp_REASS, psp_FFTL;
extern paramspCriticalBands  psp_CRITICALBANDS;
extern paramspLisCep psp_LisCep;
extern paramspLPC    psp_LPC;

extern void initpsp();
#endif
