/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier spectrof.cpp                                             
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include "signal.h"
#include "spectro.h"
#include "outils.h"
#include "pics.h"
#include "spectre.h"
#include "fftlpc.h"
#include "criticalbands.h"

static FLOATbuf X(2048+1,"X near fft_zoom_no()");
static float Y[2048+1];

static int lastMode4Spectro = -1;

static void AfficheSpectro(
     SAMPLEbuf& cnv,	/* signal temporel */
     int premX,
     int derX,
     int premEcht,  /* first and last sample of the spectrogram to be saved */
     int derEcht,
     int mode,
     struct ctrl_temps ctl,
     SpectroDisplay display)
{
  int i,nb;
  long echt;
  FLOATbuf *PX=&X;
  float **criticalBands;
  int *centerFrequencies;
  racines1_lpc res_lpc;
  int trueMode;

  trueMode = (display.type == IMAGEFILE && lastMode4Spectro == FFT_CRITICALBANDS && sps.nonLinearDisplay) ?  FFT_CRITICALBANDS : mode;
  if (trueMode != mode) spectro_GetParams(trueMode);
  
  switch (trueMode) {
    case FFT_CRITICALBANDS :
      criticalBands = buildCriticalBandFilters(en_cours.frequence, sps.nbBands, sps.fen_fft, &centerFrequencies,
                                               sps.completionStrategy, sps.frequencyScale, ConstantMelWidth, sps.nonLinearDisplay);
    case FFT_E :
    case FFT_L :
       initSpecFft(sps.fen_signal, sps.fen_fft);
      break;
  }
  if (display.type == WINDOW) {
    nb=derX-premX+1;
    for (i = 0; i < nb; i += sps.shiftPixel) {
      echt = display.ws->DXtoLX(premX + i);
      if (   (echt >= ctl.d_e + sps.fen_signal / 2 - 3)
    	    && (echt < ctl.f_e - sps.fen_signal / 2 - 3) ) {
		  if (trueMode == CEP) {
          if (sps.withTrueEnvelope)
             halle(&X[0], &cnv[0], echt - ctl.d_e - sps.fen_signal / 2 + 1, sps.fen_fft1, sps.fen_signal,
                   enEchtf(psp_LisCep.debut_liftrage), enEchtf(psp_LisCep.fin_liftrage), 0., 6, sps.isPreac);
          else cepstre(&PX, &cnv[0], echt - ctl.d_e - sps.fen_signal / 2 + 1, 1, 0, sps.fen_signal,
          	        sps.fen_fft1, sps.fen_fft,
                    enEchtf(psp_LisCep.debut_liftrage), enEchtf(psp_LisCep.fin_liftrage), 1, sps.isPreac, false);
        }
        else if (trueMode == LPC)
      	  spectreLPC(&X[0], res_lpc, &cnv[0], echt - ctl.d_e - sps.fen_signal / 2 + 1,
                      sps.fen_signal, psp_LPC.fen_fft, VRAI,
                      ordre_lpc_msTOecht(psp_LPC.ordre_lpc), 1.);
		  else if (trueMode == FFT_CRITICALBANDS && sps.withCepstralSmoothing)
          cepstre(&PX, &cnv[0], echt - ctl.d_e - sps.fen_signal / 2 + 1, 1, 0, sps.fen_signal,
                  sps.fen_fft, sps.fen_fft,
                  enEchtf(psp_LisCep.debut_liftrage), enEchtf(psp_LisCep.fin_liftrage), 1, sps.isPreac, false);
        else specFft(cnv, echt - ctl.d_e - sps.fen_signal / 2 + 1, X, sps.isPreac, true);
        if (trueMode == FFT_CRITICALBANDS) {
          applyCriticalBandsff(&X[0], Y, sps.fen_fft, sps.cMaxSpectro, criticalBands, centerFrequencies, sps.nbBands);
 			 if ((sps.nbBands < sps.fen_fft/ 2) || sps.completionStrategy)
    			spectralCompletion(Y, sps.fen_fft, sps.cMaxSpectro, centerFrequencies, sps.nbBands, sps.completionStrategy);
          spectro_affichefft(Y, premX + i - (sps.shiftPixel - 1) / 2, sps.shiftPixel, display.ws);
        }
	     else spectro_affichefft(&X[0], premX + i - (sps.shiftPixel - 1) / 2, sps.shiftPixel, display.ws);
      }
    }
    lastMode4Spectro = mode;
    display.ws->spectro_consolider(premX, premX+i*sps.shiftPixel-1);
  }
  else {   /* IMAGEFILE */
    int nb = display.sf.width;
    FILE *f = fopen(display.sf.filename, "wb");
    if (!f) affiche_erreur("Cannot open image file");
    else {
      float shift = (min(ctl.f_e, derEcht) - premEcht + 1. - sps.fen_signal + 1.) / nb;
      STRING s;
      sprintf(s, "P5 %d %d 255 ", display.sf.height, nb);
      fwrite(s,  sizeof(char), strlen(s), f);

      for (i = 0; i < nb; i++) {
        echt = (int) (premEcht + i * shift + sps.fen_signal / 2);
		   if (trueMode == CEP)
            cepstre(&PX, &cnv[0], echt - ctl.d_e - sps.fen_signal / 2 + 1, 1, 0, sps.fen_signal,
          	        sps.fen_fft1, sps.fen_fft,
                    enEchtf(psp_LisCep.debut_liftrage), enEchtf(psp_LisCep.fin_liftrage), 1, VRAI, false);
         else if (trueMode == LPC)
      	  spectreLPC(&X[0], res_lpc, &cnv[0], echt - ctl.d_e - sps.fen_signal / 2 + 1,
                      sps.fen_signal, psp_LPC.fen_fft, VRAI,
                      ordre_lpc_msTOecht(psp_LPC.ordre_lpc), 1.);
		   else specFft(cnv, echt - ctl.d_e - sps.fen_signal / 2 + 1, X, sps.isPreac, true);
         if (trueMode == FFT_CRITICALBANDS) {
           applyCriticalBandsff(&X[0], Y, sps.fen_fft, sps.cMaxSpectro, criticalBands, centerFrequencies, sps.nbBands);
 			  if ((sps.nbBands < sps.fen_fft/ 2) || sps.completionStrategy)
    			 spectralCompletion(Y, sps.fen_fft, sps.cMaxSpectro, centerFrequencies, sps.nbBands, sps.completionStrategy);
           saveOneSpectrumInFile(Y, display.sf.height, f);
         }
	      else saveOneSpectrumInFile(&X[0], display.sf.height, f);
       }
       fclose(f);
    }
  }
  if (trueMode == FFT_CRITICALBANDS) {
    freeCriticalBandFilters(criticalBands, sps.nbBands);
    delete [] centerFrequencies;
  }
  freeSpecFft();
}

extern void fftAfficheeReassigned (
    SAMPLEbuf &cnv,	    /* signal temporel */
    struct ctrl_temps ctl,
    int premX,
    int premEcht,	    /* echantillon de debut de spectrogramme */
    float shift,	    /* deplacement */
    int nb,		    /* nombre de fft a calculer */
    int shiftPixel,	    /* nombre de pixels de deplacement entre 2 spectres */
    int nbPtFen,  	    /* nombre d'echantillons de la fenetre de TF. */
    int ordreFFT,
    SpectroDisplay display);


static void AfficheSpectroReass(
     SAMPLEbuf& cnv,	/* signal temporel */
     int premX,
     int premEcht,
     int derX,
     int derEcht,
     int mode,
     struct ctrl_temps ctl,
     SpectroDisplay display)
{
  // ATTENTION il faudra changer les paramtres pour une autre frquences que 16 kHz
  if (display.type == WINDOW)
    fftAfficheeReassigned (cnv, ctl, premX, display.ws->DXtoLX(premX), display.ws->DXpixtoLXre(1),
  								 derX - premX + 1, 1, sps.fen_signal, sps.fen_fft, display);
  else  {
    float  shift = (min(ctl.f_e, derEcht) - premEcht + 1. - sps.fen_signal + 1.) / display.sf.width;
    fftAfficheeReassigned (cnv, ctl, 0, premEcht + sps.fen_signal / 2, shift,
  								 display.sf.width, 1, sps.fen_signal, sps.fen_fft, display);
  }
}


/////////////////////////////////////////////////////

void spectroFin(
	SAMPLEbuf& cnv,	// signal temporel
	long premEcht,   // debut
	long derEcht,	//   et fin de calcul du zoom
	int mode, 
	struct ctrl_temps ctl,
	SpectroDisplay display, // display d'affichage
	int efface)	    // si VRAI on efface la fenetre
{
  int premX,derX;

  if (display.type == WINDOW) {
    Wdonnees_spectro *fenetre = display.ws;
    premX = fenetre->LXtoDX(premEcht);
    while (  premX < fenetre->largeur
          && fenetre->DXtoLX(premX) < ctl.d_e + sps.fen_signal/2+3)
           premX++;
    derX = fenetre->LXtoDX(derEcht);
    while (  derX >=0
    	      && fenetre->DXtoLX(derX) > ctl.f_e - sps.fen_signal/2-3)
           derX--;

    MgiHue(CoulFondSpec(), fenetre);
    if (efface) fenetre->clear_fenetre();

    if (premX > derX) {
      InvalidateRect(fenetre->HWindow,NULL,TRUE);
      return;
    }

    if (!efface) snoibox(premX, 0, derX, fenetre->hauteur-1, fenetre);
  }

  if (mode == FFT_REASS)
    AfficheSpectroReass(cnv, premX, premEcht, derX, derEcht, mode, ctl, display);
  else AfficheSpectro(cnv, premX, derX, premEcht, derEcht, mode, ctl, display);

  if (display.type == WINDOW) InvalidateRect(display.ws->HWindow, NULL, TRUE);
}


