/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier spectrog.cpp
//

#include "common.h"
#pragma hdrstop                                     

#include <math.h>                                                                           
#include <stdio.h>
#include <malloc.h>
#include "axes.h"
#include "signal.h"
#include "notetiq.h"
#include "fftlpc.h"
#include "spectre.h"
#include "spectro.h"
#include "lissage.h"
#include "outils.h"
#include "fmaxspec.h"
#include "files.h"
#include "fft_mayer.h"
#include "optionsavespectro.h"
#include "criticalbands.h"

spectres sps;

bool autospectro=true;

spectres::spectres()
{
  ns=0;
  p = (FLOATbuf **) NULL;
  initpsp();
}
spectres::~spectres()
{
  clear();
  if (p) free(p);
  p = (FLOATbuf **) NULL;
}

void spectres::clear()
{ int i;
  for (i=0;i<ns;i++) delete p[i];
  ns=0;
  if (p) free(p);
  p = (FLOATbuf **) NULL;
}

void spectres::resize(int nb)
{ int i;
  if (nb==0) { clear(); return; }
  if (nb>ns)
  { p=(FLOATbuf **)realloc(p,((long)nb)*sizeof(FLOATbuf *));
    for (i=ns;i<nb;i++)
      p[i]=new FLOATbuf(ls,"spectre");
  } 
  if (nb<ns)
  { for (i=ns-1;i>=nb;i--)
      delete p[i];
    p=(FLOATbuf **)realloc(p,((long)nb)*sizeof(FLOATbuf *));
  }
  ns=nb;
}
void spectres::resize_sp(int n)
{ int i;
  ls=n;
  for (i=0;i<ns;i++) p[i]->newsize(ls);
}

//
// Calcul des fft sur un morceau de signal.
//
int fft_normal(
     SAMPLEbuf& cnv,	 /* signal temporel */
     spectres &res,
     long cnv_debut,
     int nb_fft,	 /* nombre de fft a calculer */
     long deplacement,		 /* deplacement dans le signal
                            temporel pour chaque calcul */
     int avecPreacc)	 /* VRAI si avec preaccentuation */
// attention; les tailles des fentres signal et fft
//   sont fixes ( 256), et doivent tre les mmes
//   chez l'appelant. Le define correspondant s'appelle
//   TAILLE_FENETRE. Ceci est  revoir.
{ int j;

  res.resize(nb_fft);
  res.resize_sp(130);
  res.echt=cnv_debut;
  res.depl=deplacement;
  initSpecFft(256,256); 
  for (j = 0; j<nb_fft;j++) {
    specFft(cnv, cnv_debut + j * deplacement, res[j], avecPreacc, true);
  }
  freeSpecFft();
  sps.resize(j);
  return j;
}

//
// Spectrogramme calcule par LPC
//
static int lpc_normal(
     SAMPLEbuf& cnv,		/* signal temporel */
     long cnv_debut,
     int nb_lpc,		/* nombre de lpc a calculer */
     long deplacement,		/* deplacement dans le signal temporel */
     int fen_signal, //taille_fenetre, /* taille de la fenetre de calcul */
     int fen_fft,
     int ordre_lpc)		/* ordre de la lpc */
{
  float *r = new float[ordre_lpc + 1],
        *a = new float[ordre_lpc + 1],
        *sf = new float[ordre_lpc + 1],
        *sig = new float[fen_signal];

  double *realfft = new double[fen_fft],
         *imagfft = new double[fen_fft];


  sps.resize(nb_lpc);
  sps.resize_sp(fen_fft / 2 + 2);
  sps.echt=cnv_debut;
  sps.depl=deplacement;
  initham_((float *)NULL, fen_signal);

  for (int j = 0; j < nb_lpc; j++) {
    float e;
    float *pres;

    preac_f_(&cnv[0], cnv_debut+j*deplacement, sig, fen_signal);
    ham_ff_(sig, sig, (FLOATbuf *) NULL, fen_signal);
    autolpc (&sig[0], r, a, sf, &e, fen_signal , ordre_lpc);

    realfft[0] = 1.0;
    imagfft[0] = 0.0;
    for (int i=1;  i <= ordre_lpc; i++) {
      realfft[i] = -a[i];
      imagfft[i] = 0;
    }
    for (int i = ordre_lpc + 1; i < fen_fft; i++) realfft[i] = imagfft[i] = 0.;

    fft(fen_fft, realfft, imagfft);

    if (fabs(e) < EPSILON) e = EPSILON;
    e = (float) log10((double)e);

    pres=(float *)&(sps[j])[0];
    for(int i = 0; i <= fen_fft / 2; i++)
       pres[i] = 10*(e-log10(realfft[i] * realfft[i] + imagfft[i] * imagfft[i] + EPSILON) );
  }
  sps.resize(nb_lpc);
  delete [] r;
  delete [] a;
  delete [] sf;
  delete [] sig;
  delete [] realfft;
  delete [] imagfft;

  return nb_lpc;
}


// Cette fontion trace les axes pour les spectros
// 1 trait tout les 1000 Hz de 0 a la demi-frequence d'echantillonnage
//
/*
static void spectro_axes_(
     float fret,		// frequence d'echantillonage	
     long premEchtAff,	        // premier echantillon(abs)	
     long derEchtAff,		// dernier echantillon(abs)	
     d_planche *fenetre)        // fenetre d'affichage          
{
  int i;
  float ht;
  
  MgiHue(NOIR, fenetre);
  for (i = 0; i <= fret / 2 ; i += 1000)
  { ht = i * 2.0 * NB_CANAUX / fret;
    snol( (float) premEchtAff , ht
	 ,(float) derEchtAff  , ht, fenetre );
  } 
}
*/

//
// Affichage d'une FFT.
//
void spectro_affichefft(
      float *X // coefficient de la fft
     ,int abscisse	// abscisse de la fft a afficher
     ,int shiftPixel
     ,d_planche *fenetre)
{ int i,n;
  float e; // energie du canal a afficher
  float s; // somme des e
  int col; // couleur du canal a afficher
  RECT rect;
  float echelle;
  float kcol;

  if (shiftPixel==0) return;
  rect.left=abscisse;
  rect.right=abscisse+shiftPixel;
  rect.bottom=fenetre->LYtoDY(0.);

  echelle = NB_CANAUX / ((float) sps.cMaxSpectro);
  kcol=(NB_COUL_SPEC-2)
       /(sps.max_spectro-sps.silence_spectro);
  for (i = 0, n = 0, s = 0; i < min(sps.cMaxSpectro, sps.fen_fft / 2) ; i++)
  { s+=X[i]; n++;
    rect.top=fenetre->LYtoDY((i+1)*echelle);
    if (rect.top!=rect.bottom)
    { e = s/n;
      n=0; s=0;
      col = (e - sps.silence_spectro) * kcol;
      col = max(0, col);
      affGris(fenetre, &rect, min(col, NB_COUL_SPEC-1));
      rect.bottom=rect.top;
    }
  }
}

//
//  Sauvegarde d'un spectre dans un fichier
//
void saveOneSpectrumInFile(
      float *X, // coefficient du spectre
      int height, // number of pixels for each spectrum (height of the image)
      FILE *ficImage)
{
  int i, n, bottom, top;
  float e; // energie du canal a afficher
  float s; // somme des e
  int col; // couleur du canal a afficher

  float echelle;
  float kcol;
  unsigned char *imageSpectrum = new unsigned char[height];
  echelle = height / ((float) sps.cMaxSpectro);
  bottom = 0;

  kcol = 255 / (sps.max_spectro - sps.silence_spectro);
  for (i = 0, n = 0, s = 0; i < min(sps.cMaxSpectro, sps.fen_fft / 2) ; i++) {
    s += X[i]; n++;
    top = (i + 1) * echelle;
    if (top != bottom) {
       e = s/n;
       n=0; s=0;
       col = max(0, min((e-sps.silence_spectro) * kcol, 255));
       for (int ii = bottom; ii < top; ii++) imageSpectrum[ii] = 255 - col;
       bottom = top;
    }
  }
  fwrite(imageSpectrum, sizeof(unsigned char), height, ficImage);

  delete [] imageSpectrum;
}

static void prepare_fenetre(
     int clear,
     long x_min,
     long x_max,
     d_planche *fenetre)	
{
  if (clear == ON_NETTOIE)
  { fenetre->clear_fenetre();
  }
  else
  { MgiHue(CoulFondSpec(), fenetre);
    fenetre->Dfillrect(fenetre->LXtoDX(x_min)
		      ,fenetre->LYtoDY(0.)
		      ,fenetre->LXtoDX(x_max)-1
		      ,fenetre->LYtoDY(NB_CANAUX) );
  }
}



int sp_nbSpectres(long& echtDeb,long echtFin)
// renvoie le nombre de spectres 532000
// maximum)  calculer,
// et normalise echtDeb  un multiple
// de deplacement
{ int nbSpectres;

  echtDeb=(echtDeb/sps.depl)*sps.depl;
  while (echtDeb<sps.fen_signal/2) echtDeb+=sps.depl;
  nbSpectres = (int)( min(32000,
			  (echtFin - echtDeb + sps.depl/2)
		          /sps.depl +1) );
  while ( echtDeb+(nbSpectres-1)*sps.depl+sps.fen_signal/2
	  >=_i(ct.f_e) ) nbSpectres--;
  return nbSpectres;
}

void spectro_GetParams(int mode)
// prpare dans sps les paramtres de calcul des spectres
//   ceux communs  tous
//   + fen_fft lorsqu'il existe
{
  paramsp *p;

  sps.mode = mode;
  sps.shiftPixel = 0;
  switch (mode) // pointeur sur paramtres
  { case FFT_L : p=&psp_FFTL;   break;
    case FFT_E : p=&psp_FFTE;   break;
    case FFT_CRITICALBANDS : p=&psp_CRITICALBANDS; break;
    case FFT_REASS : p=&psp_REASS; break;
    case CEP :   p=&psp_LisCep; break;
    case LPC :   p=&psp_LPC;    break;
 }
  switch (mode) // fen_fft
  { case FFT_L : sps.fen_fft=psp_FFTL.fen_fft; break;
    case FFT_E : sps.fen_fft=psp_FFTE.fen_fft; break;
    case FFT_CRITICALBANDS : sps.fen_fft=psp_CRITICALBANDS.fen_fft; break;
    case FFT_REASS : sps.fen_fft=psp_REASS.fen_fft; break;
    case LPC : sps.fen_fft = psp_LPC.fen_fft; break;
    default:  sps.fen_fft=256; break;
  }
  switch (mode) {// fen_signal
    case FFT_CRITICALBANDS :
      sps.nbBands = psp_CRITICALBANDS.nbBands;
      sps.samplingRate = psp_CRITICALBANDS.samplingRate;
      sps.completionStrategy =  psp_CRITICALBANDS.completionStrategy;
      sps.frequencyScale =  psp_CRITICALBANDS.frequencyScale;
      sps.withCepstralSmoothing = psp_CRITICALBANDS.withCepstralSmoothing;
      sps.nonLinearDisplay = psp_CRITICALBANDS.nonLinearDisplay;
    case FFT_L :
    case FFT_E :
    case FFT_REASS :
      // il faut au moins 1 echantillon pour calculer une transformee de Fourier...
      sps.fen_signal=max(1,min(sps.fen_fft
                         ,(int)enEcht(p->fen_signal)));
      break;
    case CEP :
      sps.fen_signal = min(512,(int)enEcht(p->fen_signal));
      sps.withTrueEnvelope = psp_LisCep.withTrueEnvelope;
      sps.fen_fft1 = 1 << (1 + min((int)(log((float)sps.fen_signal) / log(2.)),10));
      sps.fen_fft = psp_LisCep.fen_fft2;
      break;
    case LPC :
      sps.fen_signal=(int)enEcht(p->fen_signal);
      break;
  }
  // paramtres communs
  sps.depl= max(1, enEcht(p->depl));

  // On verifie quelques valeurs pour eviter les erreurs dues
  // a l'utilisateur.
  sps.fen_signal = max(1, sps.fen_signal);
  // For all analyses but LPC, the FFT window must be longer than
  // the number of samples (fen_signal).
  if (mode != LPC && mode != CEP && sps.fen_fft < sps.fen_signal)
  	 affiche_erreur("Error in the determination of fen_fft in spectro_GetParams");
  sps.shiftPixel = max(1, p->shiftPixel);
  sps.isPreac=p->isPreac;
  sps.silence_spectro=p->silence_spectro;
  sps.max_spectro=p->max_spectro;
  sps.cMaxSpectro = fMaxSpectro / en_cours.frequence * sps.fen_fft;
}


//
// cette procedure calcule tout ou partie du spectrogramme
//								
void spectrogram(
   SAMPLEbuf *sig,
   SpectroDisplay display,
	long premEcht,	// echantillon de debut
	                        //    de calcul fft
	long derEcht,	// echantillon de fin
	                        //    de calcul de fft
   struct ctrl_temps ctl,
	int clear	// option de nettoyage avant
	                //   de visualiser les spectros				
	)           		
// parametre en sortie : les spectres dans sps
// parametres en entree : tous les autres

{
  long premEchtAff = ctl.d_a,
       derEchtAff = ctl.f_a;
  long echtDeb, echtFin;
  int nbSpectres;
  int mode;	// type de calcul

  mode=en_cours.mode;

  //   Wdonnees_spectro *fenetre,
  if (display.type ==  WINDOW) {
    display.ws->setLXcoor(premEchtAff,derEchtAff);
    display.ws->setLYcoor(0.,NB_CANAUX);
  }

  echtDeb = _i(premEcht);
  echtFin = _i(derEcht);

  // Chargement de la palette de niveau de gris
  switch (mode) {
    case FFT_E :
    case FFT_REASS :
      grisSpectro(1.7); break;
//    case CEP :
//      if (sps.withTrueEnvelope) grisSpectro(1.7);
//     break;
    default: grisSpectro(1.0); break;
  }

  // extraction des paramtres de calcul
  spectro_GetParams(mode);

  // calcul global des spectres si besoin
  switch (mode)  {
    case FFT_L:
    case FFT_E:
    case FFT_REASS:
    case FFT_CRITICALBANDS:
    case CEP:
    case LPC:
      spectroFin(*sig, premEcht, derEcht, mode, ctl, display, clear);
      break;
  }

  //spectro_axes_(fret, premEchtAff, derEchtAff, fenetre);
}
void spectro_SetMode(int mode)
{ en_cours.mode=mode;
  //
  { HMENU m=GetMenu(PTWtop->HWindow);
    WORD c=MF_CHECKED, u=MF_UNCHECKED;
    CheckMenuItem(m,MENUspFFTL,(mode==FFT_L)?c:u);
    CheckMenuItem(m,MENUspFFTE,(mode==FFT_E)?c:u);
    CheckMenuItem(m,MENUspREASS,(mode==FFT_REASS)?c:u);
    CheckMenuItem(m,MENUspLPC,(mode==LPC)?c:u);
    CheckMenuItem(m,MENUspCEP,(mode==CEP)?c:u);
    CheckMenuItem(m,MENUspCRITICALBANDS,(mode==FFT_CRITICALBANDS)?c:u);
  }
}
long spectro_GetDepl()
{ if (sps.shiftPixel==0) return sps.depl;
  else return max(1, wspectro->wsona->DXtoLX(sps.shiftPixel)
	            - wspectro->wsona->DXtoLX(0) );
}

long spectro_GetFenSignal()
{
  if (!sps.fen_signal) return 1;
  else return sps.fen_signal;
}

void Wdonnees_spectro::spectro_consolider(int xinf, int xsup)
{ // copie du spectro dans wspectro->bis
  xinf=max(xinf,0);
  xsup=min(xsup, largeur-1);
  if (xsup<xinf) return;
  BitBlt(bis->hDCcomp,xinf,0,xsup-xinf+1,hauteur,hDCcomp,xinf,0,SRCCOPY);
  //
  ::InvalidateRect(HWindow,NULL,TRUE);
}

void Wdonnees_spectro::spectro_nettoyer(int xinf, int xsup)
{
  // copie du spectro->bis dans wspectro
  xinf=max(xinf,0);
  xsup=min(xsup,largeur-1);
  if (xsup<xinf) return;
  BitBlt(hDCcomp,xinf,0,xsup-xinf+1,hauteur,bis->hDCcomp,xinf,0,SRCCOPY);
  //
  ::InvalidateRect(HWindow,NULL,TRUE);
}

static void execCMspectro(int type, SpectroDisplay display)
{
  long debut,fin;
  int clear, mute;

  int oldMode = en_cours.mode;
  spectro_SetMode(type);
  if (!yaUnSignal) return;
  souris(XC_watch);
  if ( (ct.debRegion >= ct.finRegion)
       || !dans(ct.debRegion, ct.d_e, ct.f_e)
       || !dans(ct.finRegion, ct.d_e, ct.f_e) ) {
   // pas de rgion dlimite, on fait tout l'cran
    debut=ct.d_es;
    fin=ct.f_es;
    clear=ON_NETTOIE;
  }
  else  {
    debut=ct.debRegion;
    fin=ct.finRegion;
    clear=ON_LAISSE;
  }

  if (display.type == WINDOW) {
    if (en_cours.mode == FFT_CRITICALBANDS && psp_CRITICALBANDS.nonLinearDisplay) {
      float highestFreq;
      if (psp_CRITICALBANDS.frequencyScale) { // Mel scale
        highestFreq = hertz2mel(en_cours.frequence / 2.);
        wspectro->wsona->incr = max(1000, niceVertIncr(0., highestFreq, 12, &mute, wspectro->wsona));
      }
      else { // Bark scale
        highestFreq = hertz2bark(en_cours.frequence / 2.);
        wspectro->wsona->incr = max(10, niceVertIncr(0., highestFreq, 12, &mute, wspectro->wsona));
      }
      wspectro->wvscale->affichage(0., highestFreq, wspectro->wsona->incr, (psp_CRITICALBANDS.frequencyScale) ? "Mel" : "Bark");
    }
    else {
      wspectro->wsona->incr = max(1000, niceVertIncr(0., fMaxSpectro, 12, &mute, wspectro->wsona));
      wspectro->wvscale->affichage(0., fMaxSpectro, wspectro->wsona->incr, "Hz");
    }
  }
  spectrogram(wspectro->wsona->sig, display, debut, fin, ct, clear);

  souris(XC_top_left_arrow);
}

/////////////////////////////////////////
// commandes du menu principal
//
void Wtop::CMspectro_fft_large()
{
  SpectroDisplay display;
  display.type = WINDOW;
  display.ws = wspectro->wsona;
  execCMspectro(FFT_L, display);
}

void Wtop::CMspectro_lpc()
{
  SpectroDisplay display;
  display.type = WINDOW;
  display.ws = wspectro->wsona;
  execCMspectro(LPC, display);
}

void Wtop::CMspectro_cepstre()
{
  SpectroDisplay display;
  display.type = WINDOW;
  display.ws = wspectro->wsona;
  execCMspectro(CEP, display);
}

void Wtop::CMspectro_fft_etroit()
{
  SpectroDisplay display;
  display.type = WINDOW;
  display.ws = wspectro->wsona;
  execCMspectro(FFT_E, display);
}

void Wtop::CMspectro_criticalBands()
{
  SpectroDisplay display;
  display.type = WINDOW;
  display.ws = wspectro->wsona;
  execCMspectro(FFT_CRITICALBANDS, display);
}

void Wtop::CMspectro_reass()
{
  SpectroDisplay display;
  display.type = WINDOW;
  display.ws = wspectro->wsona;
#ifdef RESIZE_REASS
 RECT rect;
  int deviation;

  ::GetClientRect(wspectro->wsona->HWindow,&rect);
  if (rect.bottom - rect.top < 512) PTWtop->forcedResizeSpectro(559);
  ::GetClientRect(wspectro->wsona->HWindow,&rect);
  deviation = 512 - (rect.bottom - rect.top);
  if (deviation) PTWtop->forcedResizeSpectro(512 + deviation);
#endif
  execCMspectro(FFT_REASS, display);
}


void Wtop::CMsave_spectrogramme()
{
  STRING filename;
  TOpenSaveDialog::TData FileData
	 ( OFN_SHOWHELP | OFN_OVERWRITEPROMPT,
	  "*.*",0,"","");

  if (!yaUnSignal) return;


  strcpy(filename,"*.*");
  x_makepath(FileData.FileName,NULL,NULL,filename,filename);

  if (TFileSaveDialog(Parent,FileData,0).DoExecute() == IDOK) {
   strcpy(filename,FileData.FileName);
   if (droit_ecrire_message(filename)) {
     SpectroDisplay display;
	  souris (XC_watch);

     display.type = IMAGEFILE;
     strcpy(display.sf.filename, filename);
     display.sf.width = currentWidthSpectro;
     display.sf.height = currentHeightSpectro;

     execCMspectro(en_cours.mode, display);
     souris (XC_top_left_arrow);
   }
  }
}

void Wtop::CMoptionSaveSpectro()
{
  TWoptionSaveSpectro(PTWtop,"WoptionSaveSpectro").Execute();
}
