/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier spectrop.cpp
//

// paramtres des 6 spectrogrammes de base

#include "common.h"
#pragma hdrstop

#include <stdio.h>

#include "spectro.h"
#include "param.h"
#include "fftlpc.h"
#include "signal.h"
#include "outils.h"
                                                                                         
DEFINE_RESPONSE_TABLE1(TWparasp,TFrameWindow)
  EV_CHILD_NOTIFY_ALL_CODES(93,IDdefauts),
  EV_CHILD_NOTIFY_ALL_CODES(94,IDdetails),
END_RESPONSE_TABLE;

// paramtres des coupes spectrales
paramspFFT    psp_FFTE, psp_REASS, psp_FFTL;
paramspCriticalBands psp_CRITICALBANDS;
paramspLisCep psp_LisCep;
paramspLPC    psp_LPC;

void psp_defauts(int n)
// initialise ou rinitialise les paramtres
//   spcifiques de
//       toutes les coupes possibles, si n <0
//       la coupe n, si n>=0
{ if ((n<0)||(n==FFT_E))
  { psp_FFTE.isPreac=TRUE;
    psp_FFTE.silence_spectro=SILENCE_SPECTRO;
    psp_FFTE.max_spectro=MAX_SPECTRO;
    psp_FFTE.depl=0.;
    psp_FFTE.shiftPixel=2;
    psp_FFTE.fen_signal=32;
    psp_FFTE.fen_fft=1024;
    if (psp_FFTE.isOpen)
      InvalidateRect(psp_FFTE.PTW->HWindow,NULL,TRUE);
  }
 if ((n<0)||(n==FFT_REASS))
  { psp_REASS.isPreac=TRUE;
    psp_REASS.silence_spectro=SILENCE_SPECTRO;
    psp_REASS.max_spectro=MAX_SPECTRO;
    psp_REASS.depl=0.;
    psp_REASS.shiftPixel=2;
    psp_REASS.fen_signal=32;
    psp_REASS.fen_fft=1024;
    if (psp_REASS.isOpen)
      InvalidateRect(psp_REASS.PTW->HWindow,NULL,TRUE);
  }
  if ((n<0)||(n==FFT_L))
  { psp_FFTL.isPreac=TRUE;
    psp_FFTL.silence_spectro=SILENCE_SPECTRO;
    psp_FFTL.max_spectro=MAX_SPECTRO;
    psp_FFTL.depl=0.;
    psp_FFTL.shiftPixel=1;
    psp_FFTL.fen_signal=4;
    psp_FFTL.fen_fft=256;
    if (psp_FFTL.isOpen)
      InvalidateRect(psp_FFTL.PTW->HWindow,NULL,TRUE);
  }
  if ((n<0)||(n==FFT_CRITICALBANDS))
  { psp_CRITICALBANDS.isPreac=TRUE;
    psp_CRITICALBANDS.silence_spectro=SILENCE_SPECTRO;
    psp_CRITICALBANDS.max_spectro=MAX_SPECTRO;
    psp_CRITICALBANDS.depl=0.;
    psp_CRITICALBANDS.shiftPixel=1;
    psp_CRITICALBANDS.fen_signal=32;
    psp_CRITICALBANDS.fen_fft=1024;
    psp_CRITICALBANDS.nbBands=psp_CRITICALBANDS.fen_fft/2;
    psp_CRITICALBANDS.completionStrategy = false; // spectro like
    psp_CRITICALBANDS.frequencyScale = false; // Bark scale
    psp_CRITICALBANDS.withCepstralSmoothing = false;
    psp_CRITICALBANDS.nonLinearDisplay = false;

    if (psp_CRITICALBANDS.isOpen)
      InvalidateRect(psp_CRITICALBANDS.PTW->HWindow,NULL,TRUE);
  }
  if ((n<0)||(n==CEP))
  { psp_LisCep.isPreac=TRUE;
    psp_LisCep.silence_spectro=SILENCE_SPECTRO;
    psp_LisCep.max_spectro=MAX_SPECTRO;
    psp_LisCep.depl=0.;
    psp_LisCep.shiftPixel=3;
    psp_LisCep.fen_signal=32;
    psp_LisCep.debut_liftrage=1000*NB_CEP_STD1_SEC;
    psp_LisCep.fin_liftrage=1000*NB_CEP_STD2_SEC;
    psp_LisCep.fen_fft2=512;
    psp_LisCep.withTrueEnvelope = false;
    if (psp_LisCep.isOpen)
      InvalidateRect(psp_LisCep.PTW->HWindow,NULL,TRUE);
  }
  if ((n<0)||(n==LPC))
  { psp_LPC.isPreac=TRUE;
    psp_LPC.silence_spectro=SILENCE_SPECTRO;
    psp_LPC.max_spectro=MAX_SPECTRO;
    psp_LPC.depl=8;
    psp_LPC.shiftPixel=0;
    psp_LPC.fen_signal=16;
    psp_LPC.ordre_lpc=ORDRE_LPC_STD_MS;
    psp_LPC.fen_fft=256;
    if (psp_LPC.isOpen)
      InvalidateRect(psp_LPC.PTW->HWindow,NULL,TRUE);
  }
}

void TWparasp::psp_details(int n)
{ CHARbuf buf(1,"");
  char ch[80];
  BOOL ind;
  float vf;

  if (!yaUnSignal)
    buf.cpy(ml(8530,"pas de signal"));
  else
  { buf.cpy(ml(8531,"frquence (Hz)="));
    sprintf(ch,"%u",(unsigned int)en_cours.frequence);
    buf.cat(ch);
    // fenetre signal
    ind=FALSE;
    switch (n)
    { case FFT_E : vf=psp_FFTE.fen_signal; ind=TRUE; break;
      case FFT_L : vf=psp_FFTL.fen_signal; ind=TRUE; break;
      case FFT_REASS : vf=psp_REASS.fen_signal; ind=TRUE; break;
      case FFT_CRITICALBANDS : vf=psp_CRITICALBANDS.fen_signal; ind=TRUE; break;
      case CEP : vf=psp_LisCep.fen_signal; ind=TRUE; break;
      case LPC : vf=psp_LPC.fen_signal; ind=TRUE; break;
    }
    if (ind)
    { sprintf(ch,ml(8532,"\nfenetre_signal (echt)=%.2f")
                ,enEchtf(vf));
      buf.cat(ch);
    }
    //
    if (n==CEP)
    { sprintf(ch,ml(8533,"\ndbut du liftrage (echt)=%.2f")
                ,enEchtf(psp_LisCep.debut_liftrage));
      buf.cat(ch);
      sprintf(ch,ml(8534,"\nfin du liftrage (echt)=%.2f")
                ,enEchtf(psp_LisCep.fin_liftrage));
      buf.cat(ch);
    }
    //
    if (n==LPC)
    { sprintf(ch,ml(8535,"\nordre_lpc (echt)=%d")
                ,ordre_lpc_msTOecht(psp_LPC.ordre_lpc));
      buf.cat(ch);
    }
    // dplacement
    ind=FALSE;
    switch (n)
    { case FFT_E: vf=psp_FFTE.shiftPixel
                     *( wspectro->wsona->DXtoLX(1)
		       -wspectro->wsona->DXtoLX(0)) ;
		  ind=TRUE; break;
      case FFT_L: vf=psp_FFTL.shiftPixel
                     *( wspectro->wsona->DXtoLX(1)
		       -wspectro->wsona->DXtoLX(0)) ;
		  ind=TRUE; break;
      case FFT_CRITICALBANDS: vf=psp_CRITICALBANDS.shiftPixel
                     *( wspectro->wsona->DXtoLX(1)
		       -wspectro->wsona->DXtoLX(0)) ;
		  ind=TRUE; break;
      case FFT_REASS: vf=psp_REASS.shiftPixel
                     *( wspectro->wsona->DXtoLX(1)
		       -wspectro->wsona->DXtoLX(0)) ;
		  ind=TRUE; break;
      case CEP:   vf=psp_LisCep.shiftPixel
                     *( wspectro->wsona->DXtoLX(1)
		       -wspectro->wsona->DXtoLX(0)) ;
		  ind=TRUE; break;
      case LPC:   vf=enEchtf(psp_LPC.depl);
                  ind=TRUE; break;
    }
    if (ind)
    { sprintf(ch,ml(8536,"\nshift (echt)=%.2f"),vf);
      buf.cat(ch);
    }
  }
  ::MessageBox(HWindow
	     ,(char *)&buf[0],ml(8512,"details")
		  ,MB_OK | MB_TASKMODAL );
}

//
void initpsp()
// initialise les paramtres des coupes spectrales
{ //
  psp_FFTE.isOpen=FALSE;
  psp_FFTE.num=FFT_E;
  psp_FFTE.menu=MENUspOptFFTE;
  //
  psp_FFTL.isOpen=FALSE;
  psp_FFTL.num=FFT_L;
  psp_FFTL.menu=MENUspOptFFTL;
  //
  psp_REASS.isOpen=FALSE;
  psp_REASS.num=FFT_REASS;
  psp_REASS.menu=MENUspOptREASS;
  //
  psp_CRITICALBANDS.isOpen=FALSE;
  psp_CRITICALBANDS.num=FFT_L;
  psp_CRITICALBANDS.menu=MENUspOptCRITICALBANDS;
   //
  psp_LisCep.isOpen=FALSE;
  psp_LisCep.num=CEP;
  psp_LisCep.menu=MENUspOptCEP;
  //
  psp_LPC.isOpen=FALSE;
  psp_LPC.num=LPC;
  psp_LPC.menu=MENUspOptLPC;
  //
  psp_defauts(-1);
}

/////////////////////////////////////////////////////
// fentres paramtres
//

TWparasp::TWparasp(PTWindowsObject AParent,char *titre
		  ,paramsp *p)
	:TFrameWindow(AParent,titre)
{ char ch[80];

  ticket=GetTicket();
  wsprintf(ch,"%s %d.",titre
	     ,ticket); 
  SetCaption(ch);
  DisableAutoCreate();

  param_=p;
  param_->isOpen=TRUE;
  param_->PTW=(PTWindowsObject)this;

  Attr.Style = WS_VISIBLE | WS_POPUPWINDOW | WS_CAPTION;
  Attr.X = 250+20*param_->num;
  Attr.Y = 20+20*param_->num;
  Attr.W = 350;
  Attr.H = 250;
  this->SetIcon(GetApplication(), IDI_ICON1);

  desactive(param_->menu);
  ::DrawMenuBar(PTWtop->HWindow);

  xx=2;yy=2;ww=Attr.W-6;hh=HNUM;
  // praccentuation
  new TWchoix(this,0,xx,yy,ww
	      ,&param_->isPreac
	      ,ml(8504,"Praccentu")
	      ,choix_YESNO 
	      ,ticket);                yy+=HNUM;
}
void TWparasp::FinirCreate(bool tout)
{
  int w2;
  // initialiser le contenu des contrles
  //   avec les variables correspondantes
  // niveaux d'affichage du spectro
  if (tout) {
    new TWfloat(this,0,xx,yy,ww
	      ,&param_->silence_spectro
	      ,ml(8501,"silence spectro (dB)")
	      ,10.,90.,10.,90.
	      ,ticket); yy+=HNUM;
    new TWfloat(this,0,xx,yy,ww
	      ,&param_->max_spectro
	      ,ml(8502,"maximum spectro (dB)")
	      ,50.,120.,50.,120.
	      ,ticket); yy+=HNUM;
  }
  yy+=HNUM;
  w2=Attr.W/2;
  new TButton(this,93,ml(8511,"Dfauts")
              ,   5,yy,w2-10,24,FALSE);
  new TButton(this,94,ml(8512,"Dtails")
	      ,w2+3,yy,w2-10,24,FALSE);
  yy+=HNUM;
  //
  Attr.H=yy+30;
}
bool TWparasp::CanClose()
{ param_->isOpen=FALSE;
  active(param_->menu);
  ::DrawMenuBar(PTWtop->HWindow);
  return TRUE;
}
void TWparasp::myTStatic(char *texte)
{ TStatic *PTS;
  PTS=new TStatic(this,-1,texte
		  ,xx,yy+2,ww,HNUM-2,0);
  yy+=HNUM;
  PTS->Attr.Style&=~SS_LEFT;
  PTS->Attr.Style|=SS_CENTER;
}
void TWparasp::IDdefauts(WPARAM)
{ psp_defauts(param_->num);
}
void TWparasp::IDdetails(WPARAM)
{ psp_details(param_->num);
}

//////////////////////////////////////////////////
// fentre paramtres FFT
//
// variables modifies par la fentre
//   (selon le paramtre d'appel p)
//   p->fen_fft
//   p->fen_signal
//   p->niv              
//   p->isPreac

static choix choix_fen_fftFFT[] = { {16,"16"},{32,"32"}
			  ,{64,"64"},{128,"128"}
			  ,{256,"256"},{512,"512"}
			  ,{1024,"1024"}
			  ,{0,""} };
class TWparaspFFT : public TWparasp
{   paramspFFT *param;

  public:
    TWparaspFFT(PTWindowsObject AParent,char *titre
		  ,float fen_signal_min,float fen_signal_max
		  ,paramspFFT *p);
	 // rponses aux vnements
	 /*
	 virtual void controle_fen_fft(RTMessage)=[WM_USER+2];
	 virtual void controle_fen_signal(RTMessage)=[WM_USER+3];
	 */
	 LRESULT controle_fen_fft(WPARAM,LPARAM);
	 LRESULT controle_fen_signal(WPARAM,LPARAM);
	 DECLARE_RESPONSE_TABLE(TWparaspFFT);
};
DEFINE_RESPONSE_TABLE1(TWparaspFFT,TWparasp)
  EV_MESSAGE(WM_USER+2,controle_fen_fft),
  EV_MESSAGE(WM_USER+3,controle_fen_signal),
END_RESPONSE_TABLE;

TWparaspFFT::TWparaspFFT(PTWindowsObject AParent,char *titre
		  ,float fen_signal_min,float fen_signal_max
		  ,paramspFFT *p)
	:TWparasp(AParent,titre,p)
{ param=p;
  //
  // initialiser le contenu des contrles
  //   avec les variables correspondantes
  //
  // fenetre signal
  new TWfloat(this,3,xx,yy,ww
	      ,&param->fen_signal
	      ,ml(8505,"fenetre_signal (ms)")
	      ,fen_signal_min,fen_signal_max
	      ,fen_signal_min,fen_signal_max
	      ,ticket);                 yy+=HNUM;
  // fen_fft
  new TWint(this,2,xx,yy,ww
	      ,&param->fen_fft
	      ,ml(8506,"fenetre_fft=")
	      ,256,4096,256,4096
	      ,ticket);                yy+=HNUM;
  // dplacement
  new TWint(this,0,xx,yy,ww
	          ,&param->shiftPixel
		  ,ml(8507,"dplacement (pixel)")
	          ,1,20,1,20
		  ,ticket); yy+=HNUM;
  //
  FinirCreate(true);
}
LRESULT TWparaspFFT::controle_fen_signal(WPARAM,LPARAM)
{ return 0; // revoir complter si besoin
}
LRESULT TWparaspFFT::controle_fen_fft(WPARAM,LPARAM)
{
  float log2ordre = log((float) param->fen_fft) / log(2.);
  if (floor(log2ordre) != log2ordre) param->fen_fft = (int) (floor(log2ordre) + 1);
  else param->fen_fft = (int) log2ordre;
  param->fen_fft = 1 << param->fen_fft;
  param->fen_fft = (param->fen_fft > 4096) ? 4096 : param->fen_fft;
  if (param->fen_fft < enEchtf(param->fen_signal)) {
    while (2 * param->fen_fft<enEchtf(param->fen_signal) && param->fen_fft < 4096)
		param->fen_fft *= 2;
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
   return 0; // revoir complter si besoin
}


//////////////////////////////////////////////////
// fentre paramtres FFT Critical bands
//
// variables modifies par la fentre
//   (selon le paramtre d'appel p)
//   p->fen_fft
//   p->fen_signal
//   p->nbBands
//   p->niv
//   p->isPreac

class TWparaspCriticalBands: public TWparasp
{   paramspCriticalBands *param;

  public:
    TWparaspCriticalBands(PTWindowsObject AParent,char *titre
		  ,float fen_signal_min,float fen_signal_max
		  ,paramspCriticalBands *p);
	 // rponses aux vnements
	 /*
	 virtual void controle_fen_fft(RTMessage)=[WM_USER+2];
	 virtual void controle_fen_signal(RTMessage)=[WM_USER+3];
	 */
	 LRESULT controle_fen_fft(WPARAM,LPARAM);
	 LRESULT controle_fen_signal(WPARAM,LPARAM);
	 LRESULT controle_nbBands(WPARAM,LPARAM);
	 DECLARE_RESPONSE_TABLE(TWparaspCriticalBands);
};
DEFINE_RESPONSE_TABLE1(TWparaspCriticalBands,TWparasp)
  EV_MESSAGE(WM_USER+2,controle_fen_fft),
  EV_MESSAGE(WM_USER+3,controle_fen_signal),
  EV_MESSAGE(WM_USER+4,controle_nbBands),
END_RESPONSE_TABLE;


static choix choix_type[] = { {0," spectro like"}
                             ,{1," ASR like"}
			  ,{0,""} };
static choix choix_echelle[] = { {0," Bark (PLP like)"}
                             ,{1," Mel (MFCC like)"}
                             ,{2," Mely (improved MFCC like)"}
			  ,{0,""} };
static choix choix_cepstre[] = { {0," Without prior smoothing"}
                             ,{1," With cepstral smoothing"}
			  ,{0,""} };

static choix choix_display[] = { {0," Linear"}
                             ,{1," Perceptual"}
			  ,{0,""} };

TWparaspCriticalBands::TWparaspCriticalBands(PTWindowsObject AParent,char *titre
		  ,float fen_signal_min,float fen_signal_max
		  ,paramspCriticalBands *p)
	:TWparasp(AParent,titre,p)
{ param=p;
  //
  // initialiser le contenu des contrles
  //   avec les variables correspondantes
  //
  // fenetre signal
  int truc = 0;
  new TWchoix(this,5,xx,yy,ww
	      ,&param->completionStrategy
	      ,ml(8553,"type:")
	      ,choix_type
	      ,0);                		yy+=HNUM;
  new TWchoixInt(this,6,xx,yy,ww
	      ,(int *)&param->frequencyScale 
	      ,ml(8554,"type:")
	      ,choix_echelle
	      ,0);                		yy+=HNUM;
/*  new TWchoix(this,6,xx,yy,ww
	      ,&param->frequencyScale
	      ,ml(8554,"type:")
	      ,choix_echelle
	      ,0);                		yy+=HNUM;  */
  new TWchoix(this,7,xx,yy,ww
	      ,&param->withCepstralSmoothing
	      ,ml(8555,"type:")
	      ,choix_cepstre
	      ,0);                		yy+=HNUM;
  new TWchoix(this,8,xx,yy,ww
	      ,&param->nonLinearDisplay
	      ,ml(8556,"type:")
	      ,choix_display
	      ,0);                		yy+=HNUM;
  new TWfloat(this,3,xx,yy,ww
	      ,&param->fen_signal
	      ,ml(8505,"fenetre_signal (ms)")
	      ,fen_signal_min,fen_signal_max
	      ,fen_signal_min,fen_signal_max
	      ,ticket);                 yy+=HNUM;
  // fen_fft
  new TWint(this,2,xx,yy,ww
	      ,&param->fen_fft
	      ,ml(8506,"fenetre_fft=")
	      ,256,4096,256,4096
	      ,ticket);                yy+=HNUM;
  // nbBands
  new TWint(this,4,xx,yy,ww
	      ,&param->nbBands
	      ,ml(8550,"number of critical bands=")
	      ,1,1024,1,1024
	      ,ticket);                yy+=HNUM;
  // dplacement
  new TWint(this,0,xx,yy,ww
	          ,&param->shiftPixel
		  ,ml(8507,"dplacement (pixel)")
	          ,1,20,1,20
		  ,ticket); yy+=HNUM;
  //
  FinirCreate(true);
}

LRESULT TWparaspCriticalBands::controle_fen_signal(WPARAM,LPARAM)
{ return 0; // revoir complter si besoin
}
LRESULT TWparaspCriticalBands::controle_fen_fft(WPARAM,LPARAM)
{
  float log2ordre = log((float) param->fen_fft) / log(2.);
  if (floor(log2ordre) != log2ordre) param->fen_fft = (int) (floor(log2ordre) + 1);
  else param->fen_fft = (int) log2ordre;
  param->fen_fft = 1 << param->fen_fft;
  param->fen_fft = (param->fen_fft > 4096) ? 4096 : param->fen_fft;
  if (param->fen_fft < enEchtf(param->fen_signal)) {
    while (2 * param->fen_fft<enEchtf(param->fen_signal) && param->fen_fft < 4096)
		param->fen_fft *= 2;
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
   return 0; // revoir complter si besoin
}

LRESULT TWparaspCriticalBands::controle_nbBands(WPARAM,LPARAM)
{
  param->nbBands = min(param->fen_fft / 2, param->nbBands);
  ::InvalidateRect(HWindow,NULL,TRUE);
  
  return 0; // revoir complter si besoin
}


//////////////////////////////////////////////////
// fentre paramtres lissage cepstral
//
// variables modifies par la fentre
//   (selon le paramtre d'appel p)
//   p->fen_signal
//   p->fen_fft1
//   p->liftrage[0]
//   p->liftrage[1]
//   p->fen_fft2
//   p->niv
//   p->isPreac

class TWparaspLisCep : public TWparasp
{   paramspLisCep *param;

  public:
	 TWparaspLisCep(PTWindowsObject AParent,char *titre
		  ,paramspLisCep *p);
	 // rponses aux vnements
	 /*
	 virtual void controle_fen_fft1(RTMessage)=[WM_USER+2];
	 virtual void controle_fen_signal(RTMessage)=[WM_USER+3];
	 virtual void controle_debut_liftrage(RTMessage)=[WM_USER+4];
	 virtual void controle_fin_liftrage(RTMessage)=[WM_USER+5];
	 */
	 LRESULT controle_fen_fft2(WPARAM,LPARAM);
	 LRESULT controle_fen_signal(WPARAM,LPARAM);
	 LRESULT controle_debut_liftrage(WPARAM,LPARAM);
	 LRESULT controle_fin_liftrage(WPARAM,LPARAM);
	 DECLARE_RESPONSE_TABLE(TWparaspLisCep);
};
DEFINE_RESPONSE_TABLE1(TWparaspLisCep,TWparasp)
  EV_MESSAGE(WM_USER+2,controle_fen_fft2),
  EV_MESSAGE(WM_USER+3,controle_fen_signal),
  EV_MESSAGE(WM_USER+4,controle_debut_liftrage),
  EV_MESSAGE(WM_USER+5,controle_fin_liftrage),
END_RESPONSE_TABLE;

static choix choix_algo[] = { {0," simple cepstral smoothing"}
                             ,{1," true envelope smoothing"}
                             ,{0,""} };

TWparaspLisCep::TWparaspLisCep(PTWindowsObject AParent
						,char *titre
		  ,paramspLisCep *p)
	:TWparasp(AParent,titre,p)
{
  param=p;
  // initialiser le contenu des contrles
  //   avec les variables correspondantes
  // fenetre signal
  new TWchoix(this,6,xx,yy,ww
	      ,&param->withTrueEnvelope
	      ,ml(8557,"type:")
	      ,choix_algo
	      ,0);                		yy+=HNUM;
  new TWfloat(this,3,xx,yy,ww
			,&param->fen_signal
			,ml(8505,"fenetre_signal (ms)")
			,0.5,64.,0.5,64.
			,ticket);                 yy+=HNUM;
  // ordre fft pour le cepstre
  new TWtext(this,-1,xx,yy,ww
			,"(*)"
			,ml(8514,"1re fenetre_fft")
			,0
			,ticket);                 yy+=HNUM;
  new TStatic(this,-1
,ml(8515,"(*) la plus petite puissance de deux \
suprieure  fenetre_signal")
		  ,xx,yy+2,ww,2*HNUM-2,0);
  yy+=2*HNUM;
  // liftrage
  new TWfloat(this,4,xx,yy,ww
			,&param->debut_liftrage
			,ml(8516,"dbut liftrage (sec)")
			,0.,5.,0.,5.
			,ticket);                 yy+=HNUM;
  new TWfloat(this,5,xx,yy,ww
			,&param->fin_liftrage
			,ml(8517,"fin liftrage (sec)")
			,0.,10.,0.,10.
			,ticket);                 yy+=HNUM;
  // ordre fft finale
  new TWint(this,2,xx,yy,ww
			,&param->fen_fft2
			,ml(8518,"2me fenetre_fft")
				  ,256,2048,256,2048
			,ticket);                 yy+=HNUM;
  // dplacement
  new TWint(this,0,xx,yy,ww
			,&param->shiftPixel
			,ml(8519,"dplacement (pixel)")
			,1,20,1,20
			,ticket); yy+=HNUM;
  //
  FinirCreate(true);
}
LRESULT TWparaspLisCep::controle_fen_signal(WPARAM,LPARAM)
{ return 0; // revoir complter si besoin
}
LRESULT TWparaspLisCep::controle_fen_fft2(WPARAM,LPARAM)
{
  float log2ordre = log((float) param->fen_fft2) / log(2.);
  if (floor(log2ordre) != log2ordre) param->fen_fft2 = (int) (floor(log2ordre) + 1);
  else param->fen_fft2 = (int) log2ordre;
  param->fen_fft2 = 1 << param->fen_fft2;
  param->fen_fft2 = (param->fen_fft2 > 2048) ? 2048 : param->fen_fft2;
  ::InvalidateRect(HWindow,NULL,TRUE);
  return 0; // revoir complter si besoin
}
LRESULT TWparaspLisCep::controle_debut_liftrage(WPARAM,LPARAM)
{ param->debut_liftrage=max(0
								,min(param->debut_liftrage
				 ,param->fin_liftrage));
  ::InvalidateRect(HWindow,NULL,TRUE);
  return 0;
}
LRESULT TWparaspLisCep::controle_fin_liftrage(WPARAM,LPARAM)
{ param->fin_liftrage=   max(param->debut_liftrage
									 ,param->fin_liftrage);
  ::InvalidateRect(HWindow,NULL,TRUE);
  return 0;
}

//////////////////////////////////////////////////
// fentre paramtres LPC
//
// variables modifies par la fentre
//   (selon le paramtre d'appel p)
//   p->fen_signal
//   p->ordre_lpc
//   p->rayon
//   p->niv
//   p->isPreac

class TWparaspLPC : public TWparasp
{   paramspLPC *param;

  public:
	 TWparaspLPC(PTWindowsObject AParent,char *titre
		  ,paramspLPC *p);
	 // rponses aux vnements
	 /*
	 virtual void controle_ordre_lpc(RTMessage)=[WM_USER+2];
	 virtual void controle_fen_signal(RTMessage)=[WM_USER+3];
	 */
	 LRESULT controle_ordre_lpc(WPARAM,LPARAM);
	 LRESULT controle_fen_signal(WPARAM,LPARAM);
	 LRESULT controle_fen_fft(WPARAM,LPARAM);
	 DECLARE_RESPONSE_TABLE(TWparaspLPC);
};
DEFINE_RESPONSE_TABLE1(TWparaspLPC,TWparasp)
  EV_MESSAGE(WM_USER+2,controle_ordre_lpc),
  EV_MESSAGE(WM_USER+3,controle_fen_signal),
  EV_MESSAGE(WM_USER+4,controle_fen_fft),
END_RESPONSE_TABLE;

TWparaspLPC::TWparaspLPC(PTWindowsObject AParent,char *titre
		  ,paramspLPC *p)
	:TWparasp(AParent,titre,p)
{
  param=p;
  // initialiser le contenu des contrles
  //   avec les variables correspondantes
  // fenetre signal
  new TWfloat(this,3,xx,yy,ww
			,&param->fen_signal
			,ml(8505,"fenetre_signal (ms)")
			,0.5,64.,0.5,64.
			,ticket);                 yy+=HNUM;
  // ordre_lpc
  new TWfloat(this,2,xx,yy,ww
			,&param->ordre_lpc
			,ml(8520,"ordre_lpc")
			,ORDRE_LPC_MIN_MS,ORDRE_LPC_MAX_MS
			,ORDRE_LPC_MIN_MS,ORDRE_LPC_MAX_MS
			,ticket);                 yy+=HNUM;

  new TWint(this,4,xx,yy,ww
			,&param->fen_fft
			,ml(8521,"fenetre_fft")
	      ,256,2048,256,2048
			,ticket);                 yy+=HNUM;
  // dplacement
  new TWfloat(this,0,xx,yy,ww
			,&param->depl
			,ml(8522,"dplacement (ms)")
			,0.5,32.,0.5,32.0
			,ticket); yy+=HNUM;
  FinirCreate(true);
}
LRESULT TWparaspLPC::controle_fen_signal(WPARAM,LPARAM)
{
  if (param->ordre_lpc > param->fen_signal/2) {
    param->fen_signal=2*param->ordre_lpc;
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  return 0;
}
LRESULT TWparaspLPC::controle_ordre_lpc(WPARAM,LPARAM)
{
  if (param->ordre_lpc>param->fen_signal/2) {
    param->ordre_lpc=param->fen_signal/2;
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
  return 0;
}
LRESULT TWparaspLPC::controle_fen_fft(WPARAM,LPARAM)
{
  float log2ordre = log((float) param->fen_fft) / log(2.);
  if (floor(log2ordre) != log2ordre) param->fen_fft = (int) (floor(log2ordre) + 1);
  else param->fen_fft = (int) log2ordre;
  param->fen_fft = 1 << param->fen_fft;
  param->fen_fft = (param->fen_fft > 2048) ? 2048 : param->fen_fft;
  if (param->fen_fft < param->ordre_lpc) {
    while (2 * param->fen_fft < param->ordre_lpc && param->fen_fft < 2048)
		param->fen_fft *= 2;
	 ::InvalidateRect(HWindow,NULL,TRUE);
  }
 return 0;
}


void Wtop::CMspectro_opt_fft_large()
{ if (psp_FFTL.isOpen) return;
  GetModule()->MakeWindow(
		new TWparaspFFT(this
			,ml(8523,"paramtres FFT large")
			,0.1,16.,&psp_FFTL));
}
void Wtop::CMspectro_opt_fft_etroit()
{ if (psp_FFTE.isOpen) return;
  GetModule()->MakeWindow(
		new TWparaspFFT(this
			,ml(8524,"paramtres FFT troite")
			,1.,128.,&psp_FFTE));
}
void Wtop::CMspectro_opt_fft_reass()
{ if (psp_REASS.isOpen) return;
  GetModule()->MakeWindow(
		new TWparaspFFT(this
			,ml(8529,"paramtres FFT rassigne")
			,1.,128.,&psp_REASS));
}
void Wtop::CMspectro_opt_criticalBands()
{ if (psp_CRITICALBANDS.isOpen) return;
  GetModule()->MakeWindow(
		new TWparaspCriticalBands(this
			,ml(8552,"paramtres FFT bandes critiques")
			,1.,128.,&psp_CRITICALBANDS));
}
void Wtop::CMspectro_opt_lpc()
{ if (psp_LPC.isOpen) return;
  GetModule()->MakeWindow(
		new TWparaspLPC(this
			,ml(8525,"paramtres LPC")
			,&psp_LPC));
}
void Wtop::CMspectro_opt_cepstre()
{ if (psp_LisCep.isOpen) return;
  GetModule()->MakeWindow(
		new TWparaspLisCep(this
			,ml(8526,"paramtres LisCep")
			,&psp_LisCep));
}


void Wtop::CMspectro_opt_defauts()
{ psp_defauts(-1);
}

