/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include "common.h"
#pragma hdrstop

#include <math.h>
#include <stdio.h>
#include "signal.h"
#include "fftlpc.h"
#include "spectre.h"
#include "spectro.h"
#include "outils.h"
#include "fmaxspec.h"

#ifdef UIY
static void affFFTZoom (float X [], int abscisse, d_planche *fenetre, int nbPixLarg);
static void initAffichage ();
static void videTampon (int coul, d_planche *fenetre);
static void videTampons (d_planche *fenetre);
static void initBufferEcran ();

#endif

float **reassignedSpectro = (float **) NULL;
#define NMAX	4096

static FLOATbuf h(NMAX, "hamming"), dh(NMAX, "hamming driv"), th(NMAX, "hamming x temps");	/* la fenetre de Hamming */
static float speech[NMAX + 2], hx[NMAX+2], dhx[NMAX+2], thx[NMAX+2], stft2[NMAX+2];

extern void realfft(int n, double *real);
extern void realifft(int n, double *real);

static void realfftN(float data[], int sens, int N)
{
  double *x;
  x = new double[N];
  int i;
  for (i = 0; i  < N; i++)
    x[i] = data[i];
  if (sens == 1) realfft(N, x);
  else realifft(N, x);
  data[0] = x[0]; data[1] = 0.;
  for (i = 1; i < N / 2; i++) {
    data[i << 1] = x[i];
    data[(i << 1) + 1] = -x[N - i];  // or x[N - i] without minus  ----- ATTENTION - BE CAREFUL check the sign of the imaginary part of the Fourier transform
   }
  delete [] x;
}


void fftAfficheeReassigned (
    SAMPLEbuf &cnv,	    /* signal temporel */
    struct ctrl_temps ctl,
    int premX,
    int premEcht,	    /* echantillon de debut de spectrogramme */
    float shift,	    /* deplacement */
    int nb,
    int shiftPixel,	    /* nombre de pixels de deplacement entre 2 spectres */
    int nbPtFen,  	    /* nombre d'echantillons de la fenetre de TF. */
    int ordreFFT,
    SpectroDisplay display)
{
    int i, decal, echt_cnv, plage, nbLostPoints = 0;
    int nbptDisplayed;
    float echelle, kcol;
    float globalEnergy, lostEnergy, averageColor, maxColor;

    reassignedSpectro = (float **) calloc(nb, sizeof(float *));
    for (i = 0; i < nb; i++) reassignedSpectro[i] = (float *) calloc(ordreFFT / 2, sizeof(float));
    /* On decale le calcul de la transforme de Fourier pour ne pas introduire de biais dans le calcul de dt */

    decal = nbPtFen / 2;

    initham(&h, nbPtFen);
    initHamDeriv (&dh, nbPtFen);    // ATTENTION il faut vrifier s'il faut multiplier par Fe
    initHamTimeMul (&th, nbPtFen);    // ATTENTION il faut vrifier s'il faut diviser par Fe

    // initBufferEcran ();
    // initAffichage ();
    // initSpecFft(nbPtFen, VRAI, ordreFFT, ordreFFT / 2); // pour vrifier
    // The spectrogram is first computed exhaustively
    for (i = 0, globalEnergy = 0.0, lostEnergy = 0.0; i < nb; i++) if (IROUND(premEcht + i * shift - sps.fen_signal / 2 + 1) >= 0) {
      int j;
      echt_cnv = IROUND(premEcht + i * shift - sps.fen_signal / 2 + 1 - ctl.d_e);

      for (j = 0; j < ordreFFT; j++) speech[j] = hx[j] = thx[j] = dhx[j] = 0.;
      preaccentuation(cnv, echt_cnv, speech, nbPtFen);

      applyWindow(speech, hx + decal, &h, nbPtFen);
      realfftN(hx, 1, ordreFFT);
      applyWindow(speech, dhx + decal, &dh, nbPtFen);
      realfftN(dhx, 1, ordreFFT);
      applyWindow(speech, thx + decal, &th, nbPtFen);
      realfftN(thx, 1, ordreFFT);

      for (j = 0; j < ordreFFT / 2; j++) {
        if (j >= 0 && j < ordreFFT / 2) { // ce test permet de faire des tests de dbugage
          int jj, ii;
          double dt, dtt, dw, stft2 = hx[j+j] * hx[j+j] + hx[j+j+1] * hx[j+j+1];

          if (stft2 > EPSILON) {
	         dt = - (thx[j + j] * hx[j + j] + thx[j + j + 1] * hx[j + j + 1]) / stft2;
            dw = (hx[j + j] * dhx[j + j + 1] - hx[j + j + 1] * dhx[j + j]) / stft2;
	         dtt = (dt + decal) / shift;       // decal vient du zero padding
	         dw = - dw / (2. * M_PI) * ordreFFT;

            ii = i - dtt;  // attention il faudra remettre ce terme
	         jj = (int) (j + dw) ;
            //ii = i;   // these two lines give a normal spectrogram
	         //jj = j;

            globalEnergy += (hx[j + j] * hx[j + j] + hx[j + j + 1] * hx[j + j + 1]);
//	         if (fabs(dw) < 15 && fabs(dt) < 2 * ordreFFT && ii >= 0 && ii < nb && jj >= 0 && jj < ordreFFT / 2)
	         if (ii >= 0 && ii < nb && jj >= 0 && jj < ordreFFT / 2)
               reassignedSpectro[ii][jj] += (hx[j + j] * hx[j + j] + hx[j + j + 1] * hx[j + j + 1]);
            else {
              lostEnergy += (hx[j + j] * hx[j + j] + hx[j + j + 1] * hx[j + j + 1]);
              nbLostPoints++;
            }
          }
       }
     }
   }

   for (i = 0; i < nb; i++) {
     for (int j = 0; j < ordreFFT / 2; j++) {
        reassignedSpectro[i][j] = 10. * log10(reassignedSpectro[i][j] + EPSILON);
     }
   }


   if (display.type == WINDOW) {
     // and then displayed
     echelle = NB_CANAUX / ((float) sps.cMaxSpectro);
     // echelle = NB_CANAUX / ((float) ordreFFT / 2.);
     kcol = (NB_COUL_SPEC - 2) /( sps.max_spectro - sps.silence_spectro);
     plage = display.ws->LYtoDY(NB_CANAUX) - display.ws->LYtoDY(0.);
     for (i = 0, nbptDisplayed = 0, averageColor = 0., maxColor = 0.; i < nb; i++) {
       int j, n, col, /* couleur du canal a afficher */
          abscisse = premX + i * shiftPixel;
       RECT rect;
       float s, e; /* energie du canal a afficher */

       rect.left = abscisse;
       rect.right = abscisse + shiftPixel;
       rect.bottom = display.ws->LYtoDY(0.);

       for (int k = 0, n = 0, s = 0.; k < min(sps.cMaxSpectro, sps.fen_fft / 2); k++) {
         if (plage < min(sps.cMaxSpectro, sps.fen_fft / 2)) s = max(s, reassignedSpectro[i][k]);
         else s += reassignedSpectro[i][k]; n++;
         rect.top = display.ws->LYtoDY((k + 1) * echelle);
         if (rect.top != rect.bottom) {
           // If there are less pixels than the number of fft bins the reassignment effect is
           // graphically cancelled by the average used to choose a color. In this case the
           // average is not used.
           if (plage < min(sps.cMaxSpectro, sps.fen_fft / 2)) e = s;
           else e = s / n;
           n = 0; s = 0;
           col = (e - sps.silence_spectro) * kcol;
           col = max(0, col);
           averageColor += col; nbptDisplayed ++;
           maxColor = max(col, maxColor);
           affGris(display.ws, &rect, min(col, NB_COUL_SPEC - 1));
           rect.bottom=rect.top;
         }
       }
     }
   }
   else {
     // and then displayed
     int height = display.sf.height;
     echelle = height / ((float) sps.cMaxSpectro);
     FILE *f = fopen(display.sf.filename, "wb");
     if (!f) affiche_erreur("Cannot open image file");
     else {
       STRING s;
       unsigned char *imageSpectrum = new unsigned char[height];
       sprintf(s, "P5 %d %d 255 ", display.sf.height, nb);
       fwrite(s,  sizeof(char), strlen(s), f);

       kcol = 255 /( sps.max_spectro - sps.silence_spectro);
       for (i = 0; i < nb; i++) {
         int j, n, col, /* couleur du canal a afficher */
            abscisse = premX + i * shiftPixel;
         int bottom, top;
         float s, e; /* energie du canal a afficher */
         bottom = 0;
         for (int k = 0, n = 0, s = 0.; k < min(sps.cMaxSpectro, sps.fen_fft / 2); k++) {
           if (height < min(sps.cMaxSpectro, sps.fen_fft / 2)) s = max(s, reassignedSpectro[i][k]);
           else s += reassignedSpectro[i][k]; n++;
           top = (k + 1) * echelle;
           if (top != bottom) {
             // If there are less pixels than the number of fft bins the reassignment effect is
             // graphically cancelled by the average used to choose a color. In this case the
             // average is not used.
             if (height < min(sps.cMaxSpectro, sps.fen_fft / 2)) e = s;
             else e = s / n;
             n = 0; s = 0;
             col = max(0, min((e - sps.silence_spectro) * kcol, 255));
             for (int ii = bottom; ii < top; ii++) imageSpectrum[ii] = 255 - col;
             bottom = top;
           }
         }
         fwrite(imageSpectrum, sizeof(unsigned char), height, f);
       }
       fclose(f);
       delete [] imageSpectrum;
     }
   }

   {
   	char s[500];
      sprintf(s, "%d points perdus, globalEnergy %g, lostEnergy %g, averageCol %g, maxColor %g",
      		  nbLostPoints, globalEnergy, lostEnergy, averageColor / (nbptDisplayed + 1), maxColor);
      // affiche_erreur(s);
   }

    free(reassignedSpectro);
}


