/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef _TABPOINTCPP
#define _TABPOINTCPP

#include "outils.h"
#include "div.h" //pour la fonction ml()

#define NTEMP 100
#define NEW_SIZETMP(P,N) {P = (T1 *) realloc((void *) P, (N + NTEMP) * sizeof(T1)); N+= NTEMP;}

// Computes the distance from the TabPoints to the point defined by (date,y)
// with respect to the focus selected
template <class T1> void TabPoint<T1>::distance(
	int date,
   float y,
   int focus,
   float correc,
   float seuil,
   float *distPoint,
   float *distCourbe,
   int *indice)
{
  int i;
  if (ni == 1) {
    float P0P[2];
    P0P[0]  = date - p[0].date;
    P0P[1] = (y - p[0].y[focus]) * correc;
    *distPoint = *distCourbe = sqrt(P0P[0] * P0P[0] + P0P[1] * P0P[1]);
    *indice = 0;
  }
  else for (i = 0, *distCourbe = *distPoint = INFINI_F / 2;  i < ni - 1; i++) {
    float P0P1[2], P0P[2], PP1[2], dcos, dsin, dP0P1, dPP1, dP0P;
    P0P1[0] = p[i + 1].date - p[i].date;
    P0P1[1] = (p[i + 1].y[focus] - p[i].y[focus]) * correc;
    P0P[0]  = date - p[i].date;
    P0P[1] = (y - p[i].y[focus]) * correc;
    PP1[0]  = date - p[i + 1].date;
    PP1[1] = (y - p[i + 1].y[focus]) * correc;
    dP0P1 = sqrt(P0P1[0] * P0P1[0] + P0P1[1] * P0P1[1]);
    if (dP0P1 < 0.1) {
      float dist = sqrt(P0P[0] * P0P[0] + P0P[1] * P0P[1]);
      if (dist < *distCourbe) {
	     *distCourbe = dist;
	   if (dist < *distPoint) {
	     *distPoint = dist;
	     *indice = i;
        }
      }
    }
    else {
      dcos = (P0P1[0] * P0P[0] + P0P1[1] * P0P[1]) / dP0P1;
      dsin = (P0P[0] * P0P1[1] - P0P[1] * P0P1[0]) / dP0P1;
      dPP1 = sqrt(PP1[0] * PP1[0] + PP1[1] * PP1[1]);
      dP0P = sqrt(P0P[0] * P0P[0] + P0P[1] * P0P[1]);
      if (dcos > -seuil && dcos < dP0P1 + seuil && fabs(dsin) < *distCourbe) {
	/* Le point est dans la bande de P0P1 et la distance a la courbe est meilleure. */
	*distCourbe = fabs(dsin);
	if (dP0P < *distPoint) {
	  *distPoint = dP0P;
	  *indice = i;
	}
	if (dPP1 < *distPoint) {
	  *distPoint = dPP1;
	  *indice = i + 1;
	}
      }	  
    }
  }
}

// Set the parameters to default values if they are undifined (-INFINI_F)
template <class T1> void TabPoint<T1>::setDefaultValues(
  int nfields, int att[], float val[])
{
  for (int i = 0; i < ni; i++)
    for (int j = 0; j < nfields && att[j] != -1; j++) {
       if (p[i].y[att[j]] < -INFINI_I/2.)
          p[i].y[att[j]] = val[j];
    }
}


template <class T1> void TabPoint<T1>::removeLastPointPP()
{
  if (ni == 1) return; // Don't remove polypoints with only one point.
  T1 *points = (T1 *) malloc((ni - 1) * sizeof(T1));
  for(int i = 0; i < ni - 1; i++) points[i] = p[i];
  free((char *) p);
  p = points;
  ni =  ni - 1;
}

template <class T1> void TabPoint<T1>::removeFirstPointPP()
{
  if (ni == 1) return; // Don't remove polypoints with only one point.
  T1 *points = (T1 *) malloc((ni - 1) * sizeof(T1));
  for(int i = 0; i < ni - 1; i++) points[i] = p[i + 1];
  free((char *) p);
  p = points;
  ni =  ni - 1;
}

 /*
 * Echantillonnage regulier des polypoints.
 */

/* XM PolyPoints *echantillonneRegul(
  PolyPoints *pp,
  int param)  XM*/	// parametre de Klatt (entre 1 et 40)

template <class T1> void TabPoint<T1>::echantillonnePP(
         int depFor)
{
  int pas = depFor,
    date,
    dim,
    i,	/* indice du point courant dans le polypoint a echantillonner */
    j,  /* indice de la facette. */
    nbi;	/* indice dans le PP echantillonne regulierement. */
  T1 *points = (T1 *) malloc(NTEMP * sizeof(T1));

  int ntemp = NTEMP;

  if (!ni) return;
  dim = (sizeof(T1)- 2 * sizeof(int))/sizeof(float)+1;

  for (date = (((int)p[0].date) / pas) * pas, i = 0, nbi = 0; date <= (int)p[ni - 1].date; date += pas)
    if (date >=  (int)p[0].date) { /* on tronque legerement le pp au debut. */
      if (date == (int)p[ni - 1].date) { /* cas de la fin. */
        if (nbi >= ntemp) NEW_SIZETMP(points, ntemp);
	points[nbi].date = date;
	for (j = 1; j < dim; j++)
	  points[nbi].y[j-1] = (int)p[ni - 1].y[j-1];
	nbi++;
      }
      else if ((int)p[i].date <= date && date <= (int)p[i + 1].date) {
        if (nbi >= ntemp) NEW_SIZETMP(points, ntemp);
	points[nbi].date = date;
	for (j = 1; j < dim; j++) {
	  float x = p[i].y[j-1]
	    + (date - p[i].date) * (p[i+1].y[j-1] - p[i].y[j-1]) /
	    (p[i+1].date - p[i].date);
	  points[nbi].y[j-1] = LROUND(x);
	}
	nbi++;
      }
      else {
	for(;date >= (int)p[i+1].date;i++);
        if (nbi >= ntemp) NEW_SIZETMP(points, ntemp);
	points[nbi].date = date;

	for (j = 1; j < dim; j++) {
	  float x = p[i].y[j-1]
	    + (date - p[i].date) * (p[i+1].y[j-1] - p[i].y[j-1]) /
	    (p[i+1].date - p[i].date);
	  points[nbi].y[j-1] = LROUND(x);
	}
	nbi++;
      }
    }
  free((char *) p);
  p = points;
  ni = nbi;
}

template <class T1> void TabPoint<T1>::echantillonnePPEcht(
  int depEcht)
{
  int pas = depEcht,
    dateEcht,
    dim,
    i,	/* indice du point courant dans le polypoint a echantillonner */
    j,  /* indice de la facette. */
    nbi;	/* indice dans le PP echantillonne regulierement. */
  T1 *points = (T1 *) malloc(NTEMP * sizeof(T1));

  int ntemp = NTEMP;

  if (!ni) return;
  dim = (sizeof(T1)- 2 * sizeof(int))/sizeof(float)+1;

  for (dateEcht = (((int)p[0].dateEcht) / pas) * pas, i = 0, nbi = 0; dateEcht <= (int)p[ni - 1].dateEcht; dateEcht += pas)
    if (dateEcht >=  (int)p[0].dateEcht) { /* on tronque legerement le pp au debut. */
      if (dateEcht == (int)p[ni - 1].dateEcht) { /* cas de la fin. */
        if (nbi >= ntemp) NEW_SIZETMP(points, ntemp);
     	  points[nbi].dateEcht = dateEcht;
        points[nbi].date = ::enMilli(dateEcht);
	     for (j = 1; j < dim; j++)
	       points[nbi].y[j-1] = (int)p[ni - 1].y[j-1];
          nbi++;
        }
        else if ((int)p[i].dateEcht <= dateEcht && dateEcht <= (int)p[i + 1].dateEcht) {
          if (nbi >= ntemp) NEW_SIZETMP(points, ntemp);
	       points[nbi].dateEcht = dateEcht;
	       points[nbi].date = ::enMilli(dateEcht);
	       for (j = 1; j < dim; j++) {
	         float x = p[i].y[j-1]
	    					 + (dateEcht - p[i].dateEcht) * (p[i+1].y[j-1] - p[i].y[j-1]) /
	   					    (p[i+1].dateEcht - p[i].dateEcht);
	  			points[nbi].y[j-1] = LROUND(x);
          }
			 nbi++;
        }
        else {
			 for(;dateEcht >= (int)p[i+1].dateEcht;i++);
          if (nbi >= ntemp) NEW_SIZETMP(points, ntemp);
	       points[nbi].dateEcht = dateEcht;
	       points[nbi].date = ::enMilli(dateEcht);

	       for (j = 1; j < dim; j++) {
	         float x = p[i].y[j-1]
	    					+ (dateEcht - p[i].dateEcht) * (p[i+1].y[j-1] - p[i].y[j-1]) /
	    					   (p[i+1].dateEcht - p[i].dateEcht);
	  			points[nbi].y[j-1] = LROUND(x);
          }
			 nbi++;
        }
    }
  free((char *) p);
  p = points;
  ni = nbi;
}

/*
 * Lissage d'un PolyPoints par une spline regularisante.
 */

template<class T1> void TabPoint<T1>::spline(int focus){
  int n, i, nRes, status;
  double *y, *t, *yRes, *tRes, *alpha;

  if (ni < 3) return;

  // 1 sous-intervalle pour 80ms (duree moyenne d'une voyelle)
  n = max(1, (p[ni - 1].date - p[0].date) / 80);

  nRes = ni;

  alpha = ALLOUERM(double, n + 3 + 1);
  y = ALLOUERM(double, ni);
  t = ALLOUERM(double, ni);
  yRes = ALLOUERM(double, nRes);
  tRes = ALLOUERM(double, nRes);
  for (i = 0; i < ni; i++) {
      t[i] = (double) p[i].date;
      y[i] = (double) p[i].y[focus];
  }

  status = bsreg(ni, t, y, 3, t[0], t[ni - 1], n,
	 0.00001, nRes, /*FAUX,*/ alpha, tRes, yRes);

  if (!status) {
    for (i = 0; i < ni; i++)
      p[i].y[focus]= yRes[i];
 }
  else affiche_erreur(ml(6412,"lissage impossible"));

  free(alpha);
  free(y); free(t);
  free(yRes); free(tRes);
}

template <class T1> void TabPoint<T1>::trouveDate(int event_x, int event_y, int indice)
{
  float x, y;
  snoiunscalexy(event_x, event_y, &x, &y, wspectro->wsona);
  TabDessin[indice].date = x;
  TabDessin[indice].y[0] = y;
}

////////////////////////////////////////
// trac d'un polypoint
//
template <class T1> void TabPoint<T1>::drawDown(int event_x, int event_y, int width )
{
  if (!yaUnSignal) return;
  event_x_g = event_x;
  TabDessin = (T1 *) malloc(sizeof(T1));
  trouveDate(event_x, event_y, 0);
  nid = 1;
  this->width = width;
}

template <class T1> void TabPoint<T1>::drawDown(int event_x,int event_y )
{
  if (!yaUnSignal) return;
  event_x_g = event_x;
  TabDessin = (T1 *) malloc(sizeof(T1));
  trouveDate(event_x, event_y, 0);
  nid = 1;
  wspectro->wsona->SetPenWidth(3);
}

template <class T1> void TabPoint<T1>::push(T1 t)
{
  int s = size();
  newsize(s+1);
  p[s] = t;
}

template <class T1> void TabPoint<T1>::drawMotion(int event_x,int event_y)
{
  int x1, y1, x2, y2;
  HDC hDC;
  float x, y;

  if (!yaUnSignal) return;
  if (event_x <= event_x_g) return;
  event_x_g = event_x;

  // Check wether the time is a strictly growing function.
  snoiunscalexy(event_x, event_y, &x, &y, wspectro->wsona);
  if (::enMilli((int)x) <= ::enMilli(TabDessin[nid - 1].date)) return;

  TabDessin = (T1 *) realloc((void *)TabDessin, (++nid) * sizeof(T1));
  trouveDate(event_x, event_y, nid - 1);

  x1 = wspectro->wsona->LXtoDX((float) TabDessin[nid - 2].date);
  y1 = wspectro->wsona->LYtoDY((float) TabDessin[nid - 2].y[0]);
  x2 = wspectro->wsona->LXtoDX((float) TabDessin[nid - 1].date);
  y2 = wspectro->wsona->LYtoDY((float) TabDessin[nid - 1].y[0]);
  hDC=GetDC(wspectro->wsona->HWindow);

  if (width <= 3) {
    LOGPEN MyPen;
    HPEN hMyPen,holdpen;
    GetObject(GetStockObject(BLACK_PEN),sizeof(LOGPEN),&MyPen);
    MyPen.lopnWidth.x = width;
    hMyPen=CreatePenIndirect(&MyPen);
    holdpen=(HPEN)SelectObject(hDC,hMyPen);
    MoveToEx(hDC,x1,y1, (POINT *) NULL);
    LineTo(hDC,x2,y2);
    SelectObject(hDC,holdpen);
    DeleteObject(hMyPen);
  }
  else {
    HBRUSH holdBrush, hBrush;
    LOGBRUSH brush;
    POINT pt[4];
    brush.lbStyle = BS_SOLID;
    brush.lbColor = NOIR;
    brush.lbHatch = 0;
    hBrush = CreateBrushIndirect(&brush);
    holdBrush = (HBRUSH) SelectObject(hDC, hBrush);
    pt[0].x = x1;
    pt[0].y = y1 - width / 2;
    pt[1].x = x2;
    pt[1].y = y2 - width / 2;
    pt[2].x = x2;
    pt[2].y = y2 + width / 2;
    pt[3].x = x1;
    pt[3].y = y1 + width / 2;
    SetPolyFillMode(hDC, WINDING);
    Polygon(hDC, pt, 4);
    SelectObject(hDC,holdBrush);
    DeleteObject(hBrush);
  }
  ReleaseDC(wspectro->wsona->HWindow,hDC);
}

template <class T1> void TabPoint<T1>::drawUp(int, int )
{
  if (!yaUnSignal) return;
  wspectro->wsona->SetPenWidth(1);

  if (nid > 1) {
    ni = nid;
    this->newsize(ni);
    for (int i = 0; i < nid; i++) p[i] = TabDessin[i];
    free((void *) TabDessin);
    nid = 0;
    TabDessin = (T1 *) NULL;
  }
}

template <class T1> void TabPoint<T1>::enMilli()
{
	for(int i = 0; i < ni; i++) p[i].date = ::enMilli(p[i].date);
}

template <class T1> void TabPoint<T1>::enEcht()
{                                            
	for(int i = 0; i < ni; i++) p[i].dateEcht = ::enEcht(p[i].date);
}


template <class T1> void TabPoint<T1>::aff(
  int focus,			/* facette a afficher. */
  YFCT yecran,			/* fonction de conversion pour l'affichage. */
  int mode,
  int typeAff,			// ligne ou carrs
  COLORREF couleur,
  int ligne)
{
  int i, x, y, xold, yold,
      d_aMs = ::enMilli(ct.d_a),
      f_aMs = ::enMilli(ct.f_a);

  if (!ni) return;

  if (focus >= (sizeof(T1)-sizeof(int))/sizeof(float)+1) {
     affiche_erreur(ml(9200,"Attempt to address an extra ordinate"));
     return;
  }

  /* Ensuite on affiche tous les points */
  xold = wspectro->wsona->LXtoDX((float) ::enEcht(p[0].date));
  yold = wspectro->wsona->LYtoDY(yecran(p[0].y[focus]));

  if (mode) {
    SetROP2(wspectro->wsona->hDCcomp,R2_NOT);
    wspectro->wsona->SetPenWidth(1);
    if (dans(p[0].date, d_aMs, f_aMs))
      InvertPointSel(wspectro->wsona->hDCcomp, xold,yold);
  }
  else {
    MgiHue(couleur, wspectro->wsona);
    if (ni == 1)
      PointSel(xold,yold);
    if (typeAff && !ligne)
      wspectro->wsona->SetPenWidth(3);
    else if (ligne) wspectro->wsona->SetPenWidth(1);
    else if (dans(p[0].date, d_aMs, f_aMs)) PointSel(xold,yold);
  }

  for (i = 1; i < ni; i++) {
    x = wspectro->wsona->LXtoDX((float) ::enEcht(p[i].date));
    y = wspectro->wsona->LYtoDY(yecran(p[i].y[focus]));
    if ((mode || !typeAff) && dans(p[i].date, d_aMs, f_aMs))
      if (mode) InvertPointSel(wspectro->wsona->hDCcomp, x,y); else PointSel(x,y);
    if (  (mode || typeAff)
	     && (dans(p[i].date, d_aMs, f_aMs) || dans(p[i-1].date, d_aMs, f_aMs))) {
      if (abs(x - xold) > 0) {
		  if (!dans(p[i-1].date, d_aMs, f_aMs)) {
	       int xx = (p[i-1].date < d_aMs) ? 0
	                 : wspectro->wsona->largeur - 1;
	 		 yold = yold + (y - yold) / (x - xold) * (xx - xold);
          xold = xx;
	     }
	     else if (!dans(p[i].date, d_aMs, f_aMs)) {
	       int xx = (p[i].date < d_aMs) ? 0
	    				 : wspectro->wsona->largeur - 1;
	       y = yold + (y - yold) / (x - xold) * (xx - xold);
	       x = xx;
        }
      }
      if (typeAff || ligne) wspectro->wsona->Dline2(xold,yold,x,y);
    }
    xold = x;
    yold = y;
  }
  wspectro->wsona->SetPenWidth(1);
  SetROP2(wspectro->wsona->hDCcomp,R2_COPYPEN);
}

template <class T1> void TabPoint<T1>::translate(
 int focus, float incr, float vmin, float vmax, float vmul)
{
  	for (int i = 0; i < ni; i++) {
    	float x = p[i].y[focus] + (incr * vmul);
    	p[i].y[focus] = max(vmin, min(x, vmax));
  	}
}

#endif

