/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

#include "common.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include <strstrea.h>
#include <fstream.h>
#include "fft_mayer.h"
#include "signal.h"
#include "filterOLA.h"

#define NN 512  // ordre de la TF
#define MM 128   // shift (Ra in the Laroche paper)
#define EPS 1e-6

extern void RawtoWav(char* fichierraw,char* fichierwav,float frequence);

static double  calphase(double aa, double  bb);
static double module(double aa, double bb);

/* The Laroche vocoder is described in
 * "Improved Phase vocoder Time-scale Modification of Audio"
 * Jean Laroche and Mark Dolson
 * IEE Transactions on Speech and audio porcessing, Vol7, No 3, May 1999
 */
void vocoderLaroche(
  float beta,
  bool withPhaseLocking)
{
  HANDLE hFile, hFileMapping;
  PBYTE pbFile;
  int i;
  double C_Continue = 0.;
  double spectreR[2][2 * NN], spectreI[2][2 * NN],
         spectreSynthR[2 * NN], spectreSynthI[2 * NN], *poidsFen;
  double phase0[2 * NN], phase[2][2 * NN];
  double hamNN[NN], ham2NN[2 * NN];
  float *Space_modf;
  STRING resFile, resRawFile;

  strcpy(resFile,tempdir);
  strcat(resFile,"modif.wav");
  strcpy(resRawFile,tempdir);
  strcat(resRawFile,"modif");
  ofstream sortie (resRawFile, ios::out | ios::binary);

  Space_modf = (float *) calloc(((int)ct.tf * beta) + 2* NN, sizeof(float));
  poidsFen = (double *) calloc(((int)ct.tf * beta) + 2 * NN, sizeof(double));

  hFile = CreateFile(en_cours.nom, GENERIC_READ, FILE_SHARE_READ, NULL,
                 				 OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
  if (hFile == INVALID_HANDLE_VALUE) return;

  hFileMapping = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);
  CloseHandle(hFile);

  if (hFileMapping == NULL) return;

  pbFile = (PBYTE) MapViewOfFile(hFileMapping, FILE_MAP_READ, 0, 0, 0);

  // initialization of the hamming windows
  for(int j = 0; j < NN; j++) hamNN[j] = 0.54 - 0.46 * cos((2. * _PI * j)/(NN - 1));
  for(int j = 0; j < 2 * NN; j++) ham2NN[j] = 0.54 - 0.46 * cos((2. * _PI * j)/(2 * NN - 1));

  for(i = 0, C_Continue = 0.; i < ct.tf; i++) {
   C_Continue += ((double) getSample(pbFile, i));
   //Space_mod[i] = getSample(pbFile, i);   //pour essayer une simple copie
  }

  C_Continue /= ct.tf;

  for(i = 0; i * MM < ct.tf - 2 * NN; i++) {
    double re[2*NN], im[2*NN], mod[2*NN];
    PicsSpectre pics;
    int ii = i % 2, iii = 1 - ii;

    // Computes all the Fourier transforms of the file.
    // The signal is zero padded.
    for(int j = 0; j < 2 * NN; j++) {
      int isample = i * MM + j;
      re[j] = (getSample(pbFile, isample) - C_Continue) * ham2NN[j];
      im[j] = 0.;
    }
#ifdef UIY
    for(int j = NN; j < 2*NN; j++) re[j] = 0.;
    for(int j = 0; j < 2*NN; j++) im[j] = 0.;
#endif
    fft(2*NN, re, im);
    for(int j = 0; j < 2*NN; j++) {
      spectreR[ii][j] = re[j];
      spectreI[ii][j] = im[j];
      mod[j] = 10. * log10(SQR(re[j]) + SQR(im[j]));
    }

    if (withPhaseLocking) {
      // Locates spectral peaks for the first half of the spectrum
      extractPeaks(mod, NN, 3., pics);

      for (int l = 0, ll = 0; l < pics.size(); l++) {
        double theta, ZR, ZI;
        int j = pics[l].xi;
        // phase unwrapping
        if (i) {
          double phase1, phase2, diffphase, diffphaseprincipale, freqinstantanee;
          phase1 = calphase(spectreR[iii][j], spectreI[iii][j])+MM*j*2.*_PI/(2.*NN);
          phase[ii][j] = phase2 = calphase(spectreR[ii][j], spectreI[ii][j]);

          diffphase = phase2-phase1;
          diffphaseprincipale = calphase(cos(diffphase), sin(diffphase));
          freqinstantanee = j * 2.* _PI / (2. * NN) + diffphaseprincipale / MM;
          phase[ii][j] = phase[iii][j]+ MM * beta * freqinstantanee;
        }
        else phase[ii][j] = beta * calphase(spectreR[ii][j], spectreI[ii][j]);

        // Calculate the rotation angle theta
        theta = phase[ii][j] - calphase(spectreR[ii][j], spectreI[ii][j]);
        ZR = cos(theta);
        ZI = sin(theta);

        // initialization of the first and last phase coefficient
        if (!l) phase[ii][0] = phase[ii][NN] = 0.;

        // Apply rotation theta to all channels around peak
        for (; ll < ((l < pics.size() - 1) ? (pics[l].xi + pics[l + 1].xi) / 2 : NN); ll++) if (ll) {
          // Computes the phase for the neigbouring points (important for the next frame)
          if (ll != j) phase[ii][ll] = calphase(spectreR[ii][j], spectreI[ii][j]) + theta;
          spectreSynthR[ll] = spectreR[ii][ll] * ZR - spectreI[ii][ll] * ZI;
          spectreSynthI[ll] = spectreR[ii][ll] * ZI + spectreI[ii][ll] * ZR;
          // The same for the symetric part of the spectrum
          spectreSynthR[2 * NN - ll] = spectreSynthR[ll];
          spectreSynthI[2 * NN - ll] = -spectreSynthI[ll];
        }
      }
    }
    else {
      // Computation of the phase for the first frame
      if (!i) {
        for(int j = 1; j < NN; j++)
         phase[ii][j] = phase0[j] = beta * calphase(spectreR[ii][j], spectreI[ii][j]);
        for(int j = 2*NN-1; j > NN; j--) phase[ii][j] = phase0[j] = -phase0[2 * NN - j];
        phase0[0] = phase[ii][0] = phase0[NN] = phase[ii][NN] = 0.;
      }
      // phase unwrapping
      if (i) {
        for(int j = 1; j < NN; j++) {
          double phase1, phase2, diffphase, diffphaseprincipale, freqinstantanee;
          phase1 = calphase(spectreR[iii][j], spectreI[iii][j])+MM*j*2.*_PI/(2.*NN);
          phase[ii][j] = phase2 = calphase(spectreR[ii][j], spectreI[ii][j]);

          diffphase = phase2-phase1;
	       diffphaseprincipale = calphase(cos(diffphase), sin(diffphase));
	       freqinstantanee = j*2.*_PI/(2.*NN)+diffphaseprincipale/MM;
	       phase[ii][j] = phase[iii][j]+ MM * beta * freqinstantanee;
        }
        // copy for the antisymetric part
        for(int j = 2 * NN - 1; j > NN; j--)
	       phase[ii][j] = -phase[ii][2 * NN - j];
        // initialization of the first and last phase coefficient
        phase[ii][0] = phase[ii][NN] = 0.;
      }

      // Computes the synthesis frame
      for(int j = 0; j < 2 * NN; j++) {
 	     spectreSynthR[j] = module(spectreR[ii][j], spectreI[ii][j]) * cos(phase[ii][j]);
	     spectreSynthI[j] = module(spectreR[ii][j], spectreI[ii][j]) * sin(phase[ii][j]);
	   }
    }

    spectreSynthR[NN] = re[NN];
    spectreSynthI[NN] = 0.;
    spectreSynthR[0] = re[0];
    spectreSynthI[0] = 0.;

    // and now the OLA stage
    for(int j = 0; j < 2 * NN; j++) {
      re[j] = spectreSynthR[j] / (2. * NN);
      im[j] = spectreSynthI[j] / (2. * NN);
    }
    ifft(2 * NN, re, im);
    for(int j = 0; j < 2 * NN; j++) {
      Space_modf[(int)(i * MM * beta + j)] += re[j]; // * ham2NN[j];
      poidsFen[(int)(i * MM * beta + j)] += ham2NN[j];
    }

  }

  for(i = 0; i < ((int)(ct.tf * beta)) && fabs(poidsFen[i]) > 0.079; i++)
    Space_modf[i] = Space_modf[i] / poidsFen[i];
  // slight damping at the extremities to prevent clicks
  for(int j = 0; j < 2* MM; j++) Space_modf[j] *= ((float) j) / (2. * MM);
  for(int j = 2 * MM - 1; j >= 0; j--) Space_modf[i - j] *= ((float) j) / (2. * MM);

  for (int j = 0; j < i; j++) {
    SAMPLE x = (SAMPLE) Space_modf[j];
    sortie.write((char *) &x, sizeof(SAMPLE)) ;
  }
  sortie.close();

  UnmapViewOfFile(pbFile);
  CloseHandle(hFileMapping);

  free((void *) Space_modf);

  free((void *) poidsFen);

  RawtoWav(resRawFile, resFile, en_cours.frequence);
  unlink(resRawFile);
  {
    STRING message, commande;
     CHARbuf buf(1,"");
     STARTUPINFO siStartInfo;
     PROCESS_INFORMATION piProcInfo;
     bool b;

     sprintf(message,"%s %s",ml(9012,"Le fichier synthtis est"),resFile);
     buf.cpy(message);
     sprintf(commande, "%s %s", SndRecorder, resFile);
     memset(&siStartInfo, 0, sizeof(STARTUPINFO));
     siStartInfo.cb = sizeof(STARTUPINFO);
     b = CreateProcess(NULL, commande, 0, 0, false, DETACHED_PROCESS, 0, 0, &siStartInfo, &piProcInfo);
     ::PlaySound(resFile, 0, SND_ASYNC | SND_NOWAIT | SND_FILENAME);
     ::MessageBox(PTWtop->HWindow,(char *)&buf[0],ml(9013,"Modification temps-F0 termines"),
     					MB_OK | MB_TASKMODAL );
  }
}

static double  calphase(double aa, double  bb)
{
  float phi, phi1;
  if ((fabs( aa)<EPS) && (fabs(bb)<EPS)) return (0.);
  phi1=atan(bb/aa);
  if ((aa>=0)&&(bb>=0)) phi=phi1;
  if ((aa<=0)&&(bb>=0)) phi=phi1+_PI;
  if ((aa>=0)&&(bb<0)) phi=phi1;
  if ((aa<=0)&&(bb<=0)) phi=phi1-_PI;

  return(phi);
}

static double module(double aa, double bb) {return(sqrt(aa*aa+bb*bb));}
