/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier wcoupe.h
//

#ifndef _WCOUPE_H
#define _WCOUPE_H

#include "criticalBands.h"
enum  STYLEAXES {SIMPLE = 0, NORMAL = 1};
extern void drawDownCoupe(int event_x);
extern void drawUpCoupe(int event_x);
extern void drawMotionCoupe(int event_x);

    struct paramCoupe
    { bool isOpen; // TRUE si fenetre paramtre ouverte
      PTWindowsObject PTW; // param window
		int num; // numro (de 0  5), sert uniquement 
			 // l'affichage des boutons et des
			 // fentres paramtres
		COLORREF color; // couleur de la courbe
		bool isAffichee; // TRUE si on affiche la courbe
		bool isPreac; // TRUE si praccentuation
		float niv;
		float fen_signal; // taille de la fenetre en ms
	 };
	 struct paramCoupeFFT : paramCoupe // transformee de Fourier
	 { int fen_fft;        // donne en log2
      bool moduleIsDisplayed;
	 };
    struct paramCoupeCriticalBands : paramCoupeFFT
    { int nbBands;
      double samplingRate;
      int analysis;    /* 0 Nothing, 1 PLP, 2 MEL cepstrum */
      int  PLPorder;
      bool intensityConversion;
      bool completionStrategy;  // true if ASR like
      FrequencyScale frequencyScale;    // perceptive frequency scale
      MelStrategy melWidthStrategy;  // To define how the width of mel filter is set
      bool withCepstralSmoothing; // true if linear cepstral smoothing is applied before critical band summation
      bool withDctIdctLiftering;   // true if the dct-liftering-idct is applied
      int nbMelLiftering;
      int nbFreqSamplesDisplayed;  // Number of frequency samples displayed to enable the display of more points than critical bands
    };
	 struct paramCoupeLisCep : paramCoupe // lissage cepstral
	 { bool beforeLastSpectrum;
      int fen_fft1; // premire fft du cepstre    donne en log2
      int fen_fft2; // deuxime fft du cepstre    donne en log2
      // cette deuxime FFT permet ventuellement de ne prendre qu'une partie du spectre pour le calcul du cepstre
		float debut_liftrage,fin_liftrage;
		int fen_fft3; // fft finale pour revenir  un spectre       donne en log2
	 };
    struct paramCoupeTrueEnv : paramCoupe // true envelope
    { int fen_fft;
      float debut_liftrage,fin_liftrage;
      float alpha;
      int nbIter;
    };
	 struct paramCoupeLPC : paramCoupe // LPC
	 { int fen_fft;
      float ordre_lpc;
		float rayon;
	 };
	 struct paramCoupeLPCCEP : paramCoupe // LPC
	 { int fen_fft;
      float ordre_lpc;
		float rayon;
	 };
	 struct paramCoupeLPCsel : paramCoupe // LPC slective
	 { float ordre_lpc;
		int fInf, fHaute;
	 };

extern paramCoupeFFT    pFFT1,pFFT2;
extern paramCoupeLisCep pLisCep1,pLisCep2;
extern paramCoupeTrueEnv pTrueEnv;
extern paramCoupeLPC    pLPC;
extern paramCoupeLPC    pLPCCEP;
extern paramCoupeLPCsel pLPCsel;
extern paramCoupeCriticalBands pCriticalbands;

extern int yaUneFenetreCoupe; // !=0 si fenetre coupe ouverte
extern long numEchtCoupeSpectrale;

class Wcoupe : public TFrameWindow
{
  public:
	 int ticket;
	 paramCoupeFFT    *pFFT1,*pFFT2;
    paramCoupeCriticalBands *pCriticalBands;
	 paramCoupeLisCep *pLisCep1,*pLisCep2;
	 paramCoupeLPC    *pLPC;
	 paramCoupeLPCCEP *pLPCCEP;
	 paramCoupeLPCsel *pLPCsel;
    paramCoupeTrueEnv *pTrueEnv;
    STYLEAXES styleAxes;
    int iconified;
	 Wcoupe();
	 virtual bool CanClose();
	 void PaintButton(HDC hDC,paramCoupe& p);
	 void defauts(int n);

	 // rponses aux vnements
	 /*
	 virtual void WMSize(RTMessage)=[WM_FIRST+WM_SIZE];
	 virtual void paraFFT1(RTMessage)=[ID_FIRST+101];
	 virtual void paraFFT2(RTMessage)=[ID_FIRST+102];
	 virtual void paraLisCep1(RTMessage)=[ID_FIRST+103];
	 virtual void paraLisCep2(RTMessage)=[ID_FIRST+104];
	 virtual void paraLPC(RTMessage)=[ID_FIRST+105];
	 virtual void paraLPCsel(RTMessage)=[ID_FIRST+106];
	 virtual void CMrecalculer(RTMessage)=[ID_FIRST+109];
	 virtual void CMdefauts(RTMessage)=[ID_FIRST+108];
	 */
	 void EvSize(uint sizeType, TSize &size);
	 void IDparaFFT1(WPARAM code);
	 void IDparaFFT2(WPARAM code);
	 void IDparaLisCep1(WPARAM code);
	 void IDparaLisCep2(WPARAM code);
	 void IDparaLPC(WPARAM code);
	 void IDparaLPCCEP(WPARAM code);
	 void IDparaLPCsel(WPARAM code);
	 void IDparaTrueEnv(WPARAM code);
	 void IDparaCriticalBands(WPARAM code);
	 void IDpara(WPARAM code);
	 void IDrecalculer(WPARAM code);
	 void IDdefauts(WPARAM code);
	 void IDaxes(WPARAM code);
    void IDCopyToClipboard(WPARAM code);
	 DECLARE_RESPONSE_TABLE(Wcoupe);
};

extern Wcoupe *PTWcoupe;

class Wdonnees_coupe : public d_planche
{
  public:
    struct ctrl_temps *lastct;
    SAMPLEbuf *lastSig;
	 Wdonnees_coupe(PTWindowsObject Aparent);
	 // rponses aux vnements
	 /*
	 virtual void WMLButtonDown(RTMessage)
			= [WM_FIRST+WM_LBUTTONDOWN];
	 virtual void WMLButtonUp(RTMessage)
			= [WM_FIRST+WM_LBUTTONUP];
	 virtual void WMMouseMove(RTMessage)
			= [WM_FIRST+WM_MOUSEMOVE];
	 */
    int charHeight;
    int charWidth;
    void EvSize(uint sizeType, TSize &size);
	 void EvLButtonDown(uint Modkeys, TPoint& point);
	 void EvLButtonUp(uint Modkeys, TPoint& point);
	 void EvMouseMove(uint Modkeys, TPoint& point);
	 DECLARE_RESPONSE_TABLE(Wdonnees_coupe);
};

extern Wdonnees_coupe *wcoupe;

extern void coupePositionCurseur(long numecht);   // revoir int ou long

// dans le fichier affuneco.cpp
extern void affUneCoupe( d_planche *fenetre,
		  SAMPLEbuf& cnv,
		  long echt,
        struct ctrl_temps *ctloc, STYLEAXES styleAxes);

class TWparaCoupe : public TFrameWindow
{   paramCoupe *param_;

  public:
	 int ticket,xx,yy,ww,hh;
	 TWparaCoupe(PTWindowsObject AParent,char *titre
		 ,paramCoupe *p);
	 void myTStatic(char *texte);
	 void FinirCreate();
	 virtual bool CanClose();
	 // rponses aux vnements
	 /*
	 virtual void IDdefaults(RTMessage)=[ID_FIRST+93];
	 virtual void IDdetails(RTMessage)=[ID_FIRST+94];
	 */
	 void IDdefaults(WPARAM code);
	 void IDdetails(WPARAM code);
	 DECLARE_RESPONSE_TABLE(TWparaCoupe);
};

#endif
