/* This file is part of WinSnoori.

 WinSnoori is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3 of the License.

WinSnoori is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with WinSnoori.  If not, see <http://www.gnu.org/licenses/>. */

// fichier spectrap.cpp
//

#include "common.h"
#pragma hdrstop

#include <stdio.h>

#include "spectro.h"
#include "notetiq.h"
#include "signal.h"

// les fonctions de ce fichier permettent de calculer
// un cran spectro en rutilisant des parties
// (des "bouts") de l'cran spectro prcdent
//
//  l'utilisation, on dfinit d'abord les bouts du
// spectro par la fonction ajtBoutSpectro() qui dfinit
// un bout  la fois. Puis enfin, on appelle une fois
// la fonction SpectroRapide().
//
// les "bouts" sont conservs dans une liste, vide au
// dpart, alimente par les ajtBoutSpectro() successifs,
// exploite par SpectroRapide(), et vide par la fin de
// SpectroRapide()
//
// lorsqu'on appelle SpectroRapide(), la couverture des
// "bouts" prpars par ajtBoutSpectro() doit tre
// totale, c'est--dire qu'il ne doit pas y avoir de
// trous (qui se verraient  l'cran par des zones
// non affiches
//
// chaque bout est dfini par:
//     sa destination : portion de l'cran qu'il
//         recouvre
//     sa source :
//         soit portion de l'cran  conserver
//             en place ou  dplacer
//         soit portion  recalculer

struct boutSpectro {
  long d_, f_,	/* echantillons extremes du  bout de
                   spectrogramme a deplacer *
		   -1 si le bout de spectrogramme n'existe pas */
       d, f;	/* echantillons extremes du bout de
                   spectrogramme apres deplacement */
  struct boutSpectro *reste;
};


// Les differents bouts d'un spectrogramme 
// autrement dit, la 'liste' des bouts
static struct boutSpectro *bouts
	      = (struct boutSpectro *) NULL;

/*
 * Creation d'un bout de spectrogramme 
 */
static struct boutSpectro *nvBout(
                 long d_, long f_, long d, long f)
{
  struct boutSpectro *p = ALLOUER(struct boutSpectro);
  p->d_ = d_;
  p->f_ = f_;
  p->d = d;
  p->f = f;
  p->reste = (struct boutSpectro *) NULL;
  return(p);
}

//Ajoute_queue(ajtqBoutSpectro, struct boutSpectro)
static struct boutSpectro *ajtqBoutSpectro(
       struct boutSpectro *l,
       struct boutSpectro *y)
{ struct boutSpectro *p = l;
  if (!p) return(y);
  if (!y) return(l);
  while (p->reste) p=p->reste;
  p->reste=y;
  return(l);
}
//Detruit_liste(detruitBouts, struct boutSpectro,)
static struct boutSpectro *detruitBouts(
       struct boutSpectro *x)
{ struct boutSpectro *p=x;
  if (!p) return((struct boutSpectro *) NULL);
  p->reste = detruitBouts(p->reste);
  ;
  free((char *) p);
  return((struct boutSpectro *) NULL);
}


/*
 * Affichage rapide du spectrogramme en deplacant les bouts
 * de spectrogramme
 * deja calcules. 
 * Attention 1 : le spectrogramme obtenu n'est pas absolument correct
 *   car apres les manipulations de l'editeur les fenetres
 *   des TF du nouveau spectrogramme
 *   ne correspondent pas aux anciennes fenetres en ce qui
 *   concerne leur date.
 * Attention 2 : les tableaux de reperage de Snoori
 *   (tabl pour le spectro grossier,
 *   et les energies dans la bande de voisement) ne sont pas
 *   mis a jour ; il faut
 *   donc recalculer un spectro a la sortie de l'editeur.
 * Attention 3 : Les bouts de spectrogramme doivent etre
 *   donnes dans l'ordre.
 */ 
void spectroRapide(d_planche *) // revoir paramtre non utilis
{ d_planche *fenetre = wspectro->wsona;
  d_plancheTmp fenetreTmp(fenetre)
              ,fenetreTmpBis(fenetre) ;
  struct boutSpectro *p;
  
  //
  // On commence par les bouts  garder
  // on sauve d'abord tout l'cran
  BitBlt(fenetreTmp.hDCcomp
	     ,0,0
	     ,fenetre->largeur
	     ,fenetre->hauteur
	 ,fenetre->hDCcomp
	     ,0,0
	 ,SRCCOPY);
  // ... idem pour la fenetre Bis
  BitBlt(fenetreTmpBis.hDCcomp
	     ,0,0
	     ,wspectro->wsona->bis->largeur
	     ,wspectro->wsona->bis->hauteur
	 ,wspectro->wsona->bis->hDCcomp
	     ,0,0
	 ,SRCCOPY);

  // on efface la fentre
  fenetre->clear_fenetre();
  // .. et la fenetre Bis
  wspectro->wsona->bis->clear_fenetre();

  // on va chercher les bouts  garder
  for (p = bouts; p; p = p->reste) if (p->d_ != -1)
  { int xsrc,xdst,width;
    xsrc=snoscalex(fenetre, (float) p->d_);
    width= snoscalex(fenetre, (float) p->f_ + 1.) -xsrc +1;
    xdst= snoscalex(fenetre,(float) p->d);
    BitBlt(fenetre->hDCcomp      // fenetre normale
	     ,xdst, 0
	     ,width
	     ,fenetre->hauteur
   	   ,fenetreTmp.hDCcomp
	     ,xsrc, 0
	   ,SRCCOPY );
    BitBlt(wspectro->wsona->bis->hDCcomp  // fenetre Bis
	     ,xdst, 0
	     ,width
	     ,fenetre->hauteur
   	   ,fenetreTmpBis.hDCcomp
	     ,xsrc, 0
	   ,SRCCOPY );
  }
  // facultatif, on raffiche la fentre tout de suite
  CopyArea_Dbw(fenetre
	   ,0,0  
	   ,fenetre->largeur
	   ,fenetre->hauteur
	   ,0,0
	   ,GXcopy);

  //
  // On termine par les morceaux qu'il faut recalculer
  if (autospectro) {
    long decal;
    SpectroDisplay display;
    display.type = WINDOW;
    display.ws = wspectro->wsona;

    spectro_GetParams(en_cours.mode);
    decal = spectro_GetFenSignal();
    for (p = bouts; p; p = p->reste)
      if (p->d_ == -1)
        spectrogram(wspectro->wsona->sig, display, p->d - decal, p->f + decal, ct, ON_LAISSE );
  }

  bouts = detruitBouts(bouts);
}

/*
 * Adjonction en queue d'un bout de spectro
 */
void ajtBoutSpectro(long d_, long f_, long d, long f)
// d_ f_ : dbut et fin du bout origine
// d  f  : dbut et fin du bout destination
// si (d_==-1)&&(f_==-1) le bout destination est recalcul
{
  if (d_ == -1 && f_ == -1) {
    if (f < ct.d_a || d > ct.f_a) return;
    if (d < ct.d_a) d = ct.d_a;
    if (f > ct.f_a) f = ct.f_a;
  }
  else {
    if (f_ < ct.d_a || f < ct.d_a || d_ > ct.f_a || d > ct.f_a) return;
    if (d_ < ct.d_a) {
      d += (ct.d_a - d_);
      d_ = ct.d_a;
    }
    if (d < ct.d_a) {
      d_ += (ct.d_a - d);
      d = ct.d_a;
    }
    if (f_ > ct.f_a) {
      f -= (f_ - ct.f_a);
      f_ = ct.f_a;
    }
    if (f > ct.f_a) {
      f_ -= (f - ct.f_a);
      f = ct.f_a;
    }
  }
  
  bouts = ajtqBoutSpectro(bouts, nvBout(d_, f_, d, f));
}

